@@ -1,7 +1,7 @@
 // -------------------------------------------------------------------------
 // AAI
 //
-// A skirmish AI for the TA Spring engine.
+// A skirmish AI for the Spring engine.
 // Copyright Alexander Seizinger
 //
 // Released under GPL license: see LICENSE.html for more information.
@@ -464,7 +464,8 @@ void AAIMap::ReadMapCacheFile()
 
 void AAIMap::ReadContinentFile()
 {
-	static const size_t buffer_sizeMax = 500;
+	// this size equals the one used in "AIAICallback::GetValue(AIVAL_LOCATE_FILE_..."
+	static const size_t buffer_sizeMax = 2048;
 	char buffer[buffer_sizeMax];
 	STRCPY(buffer, MAIN_PATH);
 	STRCAT(buffer, MAP_CACHE_PATH);
@@ -588,7 +589,8 @@ void AAIMap::ReadContinentFile()
 
 std::string AAIMap::LocateMapLearnFile(const bool forWriting) const {
 
-	const size_t buffer_sizeMax = 512;
+	// this size equals the one used in "AIAICallback::GetValue(AIVAL_LOCATE_FILE_..."
+	const size_t buffer_sizeMax = 2048;
 	char buffer[buffer_sizeMax];
 
 	STRCPY(buffer, MAIN_PATH);
@@ -618,7 +620,8 @@ std::string AAIMap::LocateMapLearnFile(const bool forWriting) const {
 
 std::string AAIMap::LocateMapCacheFile(const bool forWriting) const {
 
-	const size_t buffer_sizeMax = 512;
+	// this size equals the one used in "AIAICallback::GetValue(AIVAL_LOCATE_FILE_..."
+	const size_t buffer_sizeMax = 2048;
 	char buffer[buffer_sizeMax];
 
 	STRCPY(buffer, MAIN_PATH);
@@ -644,7 +647,8 @@ void AAIMap::ReadMapLearnFile(bool auto_set)
 {
 	const std::string mapLearn_filename = LocateMapLearnFile(false);
 
-	const size_t buffer_sizeMax = 512;
+	// this size equals the one used in "AIAICallback::GetValue(AIVAL_LOCATE_FILE_..."
+	const size_t buffer_sizeMax = 2048;
 	char buffer[buffer_sizeMax];
 
 	// open learning files
@@ -2003,8 +2007,8 @@ void AAIMap::SearchMetalSpots()
 	int MaxMetal = 0;
 	int TempMetal = 0;
 	int SpotsFound = 0;
-	int coordx, coordy;
-	float AverageMetal;
+	int coordx = 0, coordy = 0;
+//	float AverageMetal;
 
 	AAIMetalSpot temp;
 	float3 pos;
@@ -2021,7 +2025,7 @@ void AAIMap::SearchMetalSpots()
 	unsigned char DoubleRadius = cb->GetExtractorRadius() / 8.0;
 	int SquareRadius = (cb->GetExtractorRadius() / 16.0) * (cb->GetExtractorRadius() / 16.0); //used to speed up loops so no recalculation needed
 	int DoubleSquareRadius = (cb->GetExtractorRadius() / 8.0) * (cb->GetExtractorRadius() / 8.0); // same as above
-	int CellsInRadius = PI * XtractorRadius * XtractorRadius; //yadda yadda
+//	int CellsInRadius = PI * XtractorRadius * XtractorRadius; //yadda yadda
 	unsigned char* MexArrayA = new unsigned char [TotalCells];
 	unsigned char* MexArrayB = new unsigned char [TotalCells];
 	int* TempAverage = new int [TotalCells];
@@ -2039,7 +2043,7 @@ void AAIMap::SearchMetalSpots()
 		TotalMetal += MexArrayA[i];		// Count the total metal so you can work out an average of the whole map
 	}
 
-	AverageMetal = ((float)TotalMetal) / ((float)TotalCells);  //do the average
+//	AverageMetal = ((float)TotalMetal) / ((float)TotalCells);  //do the average
 
 	// Now work out how much metal each spot can make by adding up the metal from nearby spots
 	for (int y = 0; y != MetalMapHeight; y++)
@@ -2481,7 +2485,7 @@ void AAIMap::AddDefence(float3 *pos, int defence)
 	for(int y = yStart; y < yEnd; ++y)
 	{
 		// determine x-range
-		xRange = (int) floor( fastmath::apxsqrt2( (float) ( range * range - (y - yPos) * (y - yPos) ) ) + 0.5f );
+		xRange = (int) floor( fastmath::apxsqrt2( (float) ( std::max(1, range * range - (y - yPos) * (y - yPos)) ) ) + 0.5f );
 
 		xStart = xPos - xRange;
 		xEnd = xPos + xRange;
@@ -2540,7 +2544,8 @@ void AAIMap::AddDefence(float3 *pos, int defence)
 		}
 	}
 
-	static const size_t filename_sizeMax = 500;
+	// this size equals the one used in "AIAICallback::GetValue(AIVAL_LOCATE_FILE_..."
+	static const size_t filename_sizeMax = 2048;
 	char filename[filename_sizeMax];
 	STRCPY(filename, "AAIDefMap.txt");
 	ai->cb->GetValue(AIVAL_LOCATE_FILE_W, filename);
@@ -2627,7 +2632,7 @@ void AAIMap::RemoveDefence(float3 *pos, int defence)
 	for(int y = yStart; y < yEnd; ++y)
 	{
 		// determine x-range
-		xRange = (int) floor( fastmath::apxsqrt2( (float) ( range * range - (y - yPos) * (y - yPos) ) ) + 0.5f );
+		xRange = (int) floor( fastmath::apxsqrt2( (float) ( std::max(1, range * range - (y - yPos) * (y - yPos)) ) ) + 0.5f );
 
 		xStart = xPos - xRange;
 		xEnd = xPos + xRange;
@@ -2684,7 +2689,8 @@ float AAIMap::GetDefenceBuildsite(float3 *best_pos, const UnitDef *def, int xSta
 
 	float range =  bt->units_static[def->id].range / 8.0;
 
-	static const size_t filename_sizeMax = 500;
+	// this size equals the one used in "AIAICallback::GetValue(AIVAL_LOCATE_FILE_..."
+	static const size_t filename_sizeMax = 2048;
 	char filename[filename_sizeMax];
 	STRCPY(filename, "AAIDebug.txt");
 	ai->cb->GetValue(AIVAL_LOCATE_FILE_W, filename);
