@@ -31,19 +31,13 @@
 
 #include "Util.h"
 
-#include "ExternalAI/Interface/aidefines.h" // for SKIRMISH_AI_PROPERTY_DATA_DIR, AI_INTERFACES_DATA_DIR
+#include "System/maindefines.h"
+#include "System/SafeCStrings.h"
 #if defined USING_STREFLOP
 #include "lib/streflop/streflopC.h" // for streflop_init_Simple()
 #else
 #include <assert.h>
 #endif
-#if defined BUILDING_SKIRMISH_AI
-// for SKIRMISH_AI_PROPERTY_DATA_DIR
-#include "ExternalAI/Interface/SSkirmishAILibrary.h"
-#elif defined BUILDING_AI_INTERFACE
-// for AI_INTERFACE_PROPERTY_DATA_DIR
-#include "ExternalAI/Interface/SAIInterfaceLibrary.h"
-#endif
 
 
 char* util_allocStr(unsigned int length) {
@@ -58,7 +52,7 @@ char* util_allocStrCpy(const char* toCopy) {
 
 	const size_t copy_sizeMax = strlen(toCopy) + 1;
 	char* copy = (char*) calloc(copy_sizeMax, sizeof(char));
-	STRCPYS(copy, copy_sizeMax, toCopy);
+	STRCPY_T(copy, copy_sizeMax, toCopy);
 	return copy;
 }
 
