@@ -1,7 +1,7 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#ifndef _AIINTERFACELIBRARYINFO_H
-#define _AIINTERFACELIBRARYINFO_H
+#ifndef AI_INTERFACE_LIBRARY_INFO_H
+#define AI_INTERFACE_LIBRARY_INFO_H
 
 #include <vector>
 #include <map>
@@ -33,6 +33,7 @@ public:
 	virtual const std::string& GetName() const;
 	virtual const std::string& GetDescription() const;
 	virtual const std::string& GetURL() const;
+	virtual bool IsLookupSupported() const;
 	virtual const std::string& GetInfo(const std::string& key) const;
 
 	virtual void SetDataDir(const std::string& dataDir);
@@ -55,4 +56,4 @@ private:
 	std::map<std::string, std::string> key_description;
 };
 
-#endif // _AIINTERFACELIBRARYINFO_H
+#endif // AI_INTERFACE_LIBRARY_INFO_H
