@@ -5,25 +5,30 @@
 #include "ExternalAI/SkirmishAIWrapper.h"
 #include "ExternalAI/SkirmishAIData.h"
 #include "ExternalAI/SkirmishAIHandler.h"
+#include "ExternalAI/SSkirmishAICallbackImpl.h"
+#include "ExternalAI/IAILibraryManager.h"
 #include "ExternalAI/Interface/AISCommands.h"
 #include "Sim/Misc/GlobalSynced.h"
 #include "Game/GameHelper.h"
 #include "Game/GameSetup.h"
+#include "Game/GlobalUnsynced.h"
 #include "Game/Player.h"
 #include "Game/PlayerHandler.h"
 #include "Sim/Units/Unit.h"
+#include "Sim/Units/UnitHandler.h"
 #include "Sim/Misc/Team.h"
 #include "Sim/Misc/TeamHandler.h"
 #include "Sim/Weapons/WeaponDef.h"
-#include "NetProtocol.h"
-#include "GlobalUnsynced.h"
-#include "ConfigHandler.h"
-#include "LogOutput.h"
-#include "Util.h"
-#include "TimeProfiler.h"
+#include "System/NetProtocol.h"
+#include "System/Config/ConfigHandler.h"
+#include "System/Log/ILog.h"
+#include "System/Util.h"
+#include "System/TimeProfiler.h"
 
-#include "creg/STL_Map.h"
+#include "System/creg/STL_Map.h"
 
+CONFIG(int, CatchAIExceptions).defaultValue(1);
+CONFIG(bool, AI_UnpauseAfterInit).defaultValue(true);
 
 CR_BIND_DERIVED(CEngineOutHandler, CObject, )
 
@@ -53,7 +58,7 @@ bool CEngineOutHandler::IsCatchExceptions() {
 	static bool isCatchExceptions;
 
 	if (!init) {
-		isCatchExceptions = (configHandler->Get("CatchAIExceptions", 1) != 0);
+		isCatchExceptions = (configHandler->GetInt("CatchAIExceptions") != 0);
 		init = true;
 	}
 
@@ -64,9 +69,9 @@ void CEngineOutHandler::HandleAIException(const char* description) {
 
 	if (CEngineOutHandler::IsCatchExceptions()) {
 		if (description) {
-			logOutput.Print("AI Exception: \'%s\'", description);
+			LOG_L(L_ERROR, "AI Exception: \'%s\'", description);
 		} else {
-			logOutput.Print("AI Exception");
+			LOG_L(L_ERROR, "AI Exception");
 		}
 //		exit(-1);
 	}
@@ -95,10 +100,13 @@ CEngineOutHandler* CEngineOutHandler::GetInstance() {
 void CEngineOutHandler::Destroy() {
 
 	if (singleton != NULL) {
+		singleton->PreDestroy();
+
 		CEngineOutHandler* tmp = singleton;
 		singleton = NULL;
 		delete tmp;
-		tmp = NULL;
+
+		IAILibraryManager::Destroy();
 	}
 }
 
@@ -116,18 +124,17 @@ CEngineOutHandler::~CEngineOutHandler() {
 
 
 // This macro should be insterted at the start of each method sending AI events
-#define AI_EVT_MTH()                           \
-	if (id_skirmishAI.size() == 0) { return; } \
-	SCOPED_TIMER("AI Total");
+#define AI_EVT_MTH()                               \
+		if (id_skirmishAI.empty()) { return; } \
+		SCOPED_TIMER("AI Total");
 
 
-#define DO_FOR_SKIRMISH_AIS(FUNC)                    \
-	id_ai_t::iterator ai = id_skirmishAI.begin();    \
-                                                     \
-	for (; ai != id_skirmishAI.end(); ++ai) {        \
-			try {                                    \
-				ai->second->FUNC;                    \
-			} CATCH_AI_EXCEPTION;                    \
+#define DO_FOR_SKIRMISH_AIS(FUNC)                          \
+		for (id_ai_t::iterator ai = id_skirmishAI.begin(); \
+				ai != id_skirmishAI.end(); ++ai) {         \
+			try {                                          \
+				ai->second->FUNC;                          \
+			} CATCH_AI_EXCEPTION;                          \
 		}
 
 
@@ -164,19 +171,18 @@ void CEngineOutHandler::Update() {
 
 // Do only if the unit is not allied, in which case we know
 // everything about it anyway, and do not need to be informed
-#define DO_FOR_ALLIED_SKIRMISH_AIS(FUNC, ALLY_TEAM_ID, UNIT_ALLY_TEAM_ID)           \
-	if (!teamHandler->Ally(ALLY_TEAM_ID, UNIT_ALLY_TEAM_ID)) {                      \
-		id_ai_t::iterator ai = id_skirmishAI.begin();                               \
-                                                                                    \
-		for (; ai != id_skirmishAI.end(); ++ai) {                                   \
-			const int aiAllyTeam = teamHandler->AllyTeam(ai->second->GetTeamId());  \
-			if (teamHandler->Ally(aiAllyTeam, ALLY_TEAM_ID)) {                      \
-				try {                                                               \
-					ai->second->FUNC;                                               \
-				} CATCH_AI_EXCEPTION;                                               \
-			}                                                                       \
-		}                                                                           \
-	}
+#define DO_FOR_ALLIED_SKIRMISH_AIS(FUNC, ALLY_TEAM_ID, UNIT_ALLY_TEAM_ID)				\
+		if (!teamHandler->Ally(ALLY_TEAM_ID, UNIT_ALLY_TEAM_ID)) {						\
+			for (id_ai_t::iterator ai = id_skirmishAI.begin();							\
+					ai != id_skirmishAI.end(); ++ai) {									\
+				const int aiAllyTeam = teamHandler->AllyTeam(ai->second->GetTeamId());	\
+				if (teamHandler->Ally(aiAllyTeam, ALLY_TEAM_ID)) {						\
+					try {																\
+						ai->second->FUNC;												\
+					} CATCH_AI_EXCEPTION;												\
+				}																		\
+			}																			\
+		}
 
 
 void CEngineOutHandler::UnitEnteredLos(const CUnit& unit, int allyTeamId) {
@@ -217,35 +223,33 @@ void CEngineOutHandler::UnitLeftRadar(const CUnit& unit, int allyTeamId) {
 
 
 
-#define DO_FOR_TEAM_SKIRMISH_AIS(FUNC, TEAM_ID)                         \
-	if (team_skirmishAIs.find(TEAM_ID) != team_skirmishAIs.end()) {     \
-		ids_t::iterator ai = team_skirmishAIs[TEAM_ID].begin();         \
-                                                                        \
-		for (; ai != team_skirmishAIs[TEAM_ID].end(); ++ai) {           \
-			try {                                                       \
-				id_skirmishAI[*ai]->FUNC;                               \
-			} CATCH_AI_EXCEPTION;                                       \
-		}                                                               \
-	}
+#define DO_FOR_TEAM_SKIRMISH_AIS(FUNC, TEAM_ID)								\
+		if (team_skirmishAIs.find(TEAM_ID) != team_skirmishAIs.end()) {		\
+			for (ids_t::iterator ai = team_skirmishAIs[TEAM_ID].begin();	\
+					ai != team_skirmishAIs[TEAM_ID].end(); ++ai) {			\
+				try {														\
+					id_skirmishAI[*ai]->FUNC;								\
+				} CATCH_AI_EXCEPTION;										\
+			}																\
+		}
 
 
 
 // Send to all teams which the unit is not allied to,
-// and which have cheat-events enabled, or the unit in
-// sensor range.
-#define DO_FOR_ENEMY_SKIRMISH_AIS(FUNC, ALLY_TEAM_ID, UNIT)                                       \
-	id_ai_t::iterator ai = id_skirmishAI.begin();                                                 \
-	for (; ai != id_skirmishAI.end(); ++ai) {                                                     \
-		const CSkirmishAIWrapper* saw = ai->second;                                               \
-		const int aiAllyTeam = teamHandler->AllyTeam(saw->GetTeamId());                           \
-                                                                                                  \
-		if (teamHandler->Ally(aiAllyTeam, ALLY_TEAM_ID)) { continue; }                            \
-		if ((saw->IsCheatEventsEnabled() || IsUnitInLosOrRadarOfAllyTeam(UNIT, aiAllyTeam))) {    \
-			try {                                                                                 \
-				ai->second->FUNC;                                                                 \
-			} CATCH_AI_EXCEPTION;                                                                 \
-		}                                                                                         \
-	}
+// and which have cheat-events enabled, or the unit in sensor range.
+#define DO_FOR_ENEMY_SKIRMISH_AIS(FUNC, ALLY_TEAM_ID, UNIT)							\
+		for (id_ai_t::iterator ai = id_skirmishAI.begin();							\
+				ai != id_skirmishAI.end(); ++ai) {									\
+			const CSkirmishAIWrapper* saw = ai->second;								\
+			const int aiAllyTeam = teamHandler->AllyTeam(saw->GetTeamId());	\
+			if (!teamHandler->Ally(aiAllyTeam, ALLY_TEAM_ID) &&						\
+					(saw->IsCheatEventsEnabled() ||							\
+					IsUnitInLosOrRadarOfAllyTeam(UNIT, aiAllyTeam))) {				\
+				try {																\
+					ai->second->FUNC;												\
+				} CATCH_AI_EXCEPTION;												\
+			}																		\
+		}
 
 
 
@@ -427,23 +431,23 @@ void CEngineOutHandler::UnitDamaged(const CUnit& damaged, const CUnit* attacker,
 	// inform attacker units team (not allies)
 	if (attacker) {
 		const int at = attacker ? attacker->team : -1;
-
-		if (teamHandler->Ally(attacker->allyteam, damaged.allyteam)) { return; }
-		if (team_skirmishAIs.find(at) == team_skirmishAIs.end()) { return; }
-
-		// direction from the attacker's view
-		const float3 attackDir =
-			(attacker->pos -
-			helper->GetUnitErrorPos(&damaged, attacker->allyteam)).ANormalize();
-		const bool damagedInLosOrRadar = IsUnitInLosOrRadarOfAllyTeam(damaged, attacker->allyteam);
-
-		for (ids_t::iterator ai = team_skirmishAIs[at].begin(); ai != team_skirmishAIs[at].end(); ++ai) {
-			CSkirmishAIWrapper* saw = id_skirmishAI[*ai];
-
-			if (damagedInLosOrRadar || saw->IsCheatEventsEnabled()) {
-				try {
-					saw->EnemyDamaged(damagedUnitId, attackerUnitId, damage, attackDir, weaponDefId, paralyzer);
-				} CATCH_AI_EXCEPTION;
+		if (!teamHandler->Ally(attacker->allyteam, damaged.allyteam)
+				&& (team_skirmishAIs.find(at) != team_skirmishAIs.end())) {
+			// direction from the attacker's view
+			const float3 attackDir = (attacker->pos
+						- helper->GetUnitErrorPos(&damaged, attacker->allyteam))
+						.ANormalize();
+			const bool damagedInLosOrRadar = IsUnitInLosOrRadarOfAllyTeam(damaged, attacker->allyteam);
+			for (ids_t::iterator ai = team_skirmishAIs[at].begin(); ai != team_skirmishAIs[at].end(); ++ai)
+			{
+				CSkirmishAIWrapper* saw = id_skirmishAI[*ai];
+				if (damagedInLosOrRadar || saw->IsCheatEventsEnabled())
+				{
+					try {
+						saw->EnemyDamaged(damagedUnitId, attackerUnitId, damage,
+								attackDir, weaponDefId, paralyzer);
+					} CATCH_AI_EXCEPTION;
+				}
 			}
 		}
 	}
@@ -485,15 +489,63 @@ void CEngineOutHandler::CommandFinished(const CUnit& unit, const Command& comman
 
 	const int teamId           = unit.team;
 	const int unitId           = unit.id;
-	const int aiCommandTopicId = extractAICommandTopic(&command);
+	const int aiCommandTopicId = extractAICommandTopic(&command, uh->MaxUnits());
 
-	DO_FOR_TEAM_SKIRMISH_AIS(CommandFinished(unitId, aiCommandTopicId), teamId);
+	DO_FOR_TEAM_SKIRMISH_AIS(CommandFinished(unitId, command.aiCommandId, aiCommandTopicId), teamId);
 }
 
-void CEngineOutHandler::GotChatMsg(const char* msg, int fromPlayerId) {
+void CEngineOutHandler::SendChatMessage(const char* msg, int fromPlayerId) {
 	AI_EVT_MTH();
 
-	DO_FOR_SKIRMISH_AIS(GotChatMsg(msg, fromPlayerId))
+	DO_FOR_SKIRMISH_AIS(SendChatMessage(msg, fromPlayerId))
+}
+
+bool CEngineOutHandler::SendLuaMessages(int aiTeam, const char* inData, std::vector<const char*>& outData) {
+	SCOPED_TIMER("AI Total");
+
+	if (id_skirmishAI.empty()) {
+		return false;
+	}
+
+	id_ai_t::iterator it;
+	unsigned int n = 0;
+
+	if (aiTeam != -1) {
+		// get the AI's for team <aiTeam>
+		const team_ais_t::iterator aiTeamIter = team_skirmishAIs.find(aiTeam);
+
+		if (aiTeamIter == team_skirmishAIs.end()) {
+			return false;
+		}
+
+		// get the vector of ID's for the AI's
+		ids_t& aiIDs = aiTeamIter->second;
+		ids_t::iterator aiIDsIter;
+
+		outData.resize(aiIDs.size(), "");
+
+		// send only to AI's in team <aiTeam>
+		for (aiIDsIter = aiIDs.begin(); aiIDsIter != aiIDs.end(); ++aiIDsIter) {
+			const size_t aiID = aiIDs[*aiIDsIter];
+
+			CSkirmishAIWrapper* wrapperAI = id_skirmishAI[aiID];
+			wrapperAI->SendLuaMessage(inData, &outData[n++]);
+		}
+	} else {
+		outData.resize(id_skirmishAI.size(), "");
+
+		// broadcast to all AI's across all teams
+		//
+		// since neither AI ID's nor AI teams are
+		// necessarily consecutive, store responses
+		// in calling order
+		for (it = id_skirmishAI.begin(); it != id_skirmishAI.end(); ++it) {
+			CSkirmishAIWrapper* wrapperAI = it->second;
+			wrapperAI->SendLuaMessage(inData, &outData[n++]);
+		}
+	}
+
+	return true;
 }
 
 
@@ -501,6 +553,23 @@ void CEngineOutHandler::GotChatMsg(const char* msg, int fromPlayerId) {
 void CEngineOutHandler::CreateSkirmishAI(const size_t skirmishAIId) {
 	SCOPED_TIMER("AI Total");
 
+	//const bool unpauseAfterAIInit = configHandler->GetBool("AI_UnpauseAfterInit");
+
+	// Pause the game for letting the AI initialize,
+	// as this can take quite some time.
+	/*bool weDoPause = !gs->paused;
+	if (weDoPause) {
+		const std::string& myPlayerName = playerHandler->Player(gu->myPlayerNum)->name;
+		LOG(
+				"Player %s (auto)-paused the game for letting Skirmish AI"
+				" %s initialize for controlling team %i.%s",
+				myPlayerName.c_str(), key.GetShortName().c_str(), teamId,
+				(unpauseAfterAIInit ?
+				 " The game is auto-unpaused as soon as the AI is ready." :
+				 ""));
+		net->Send(CBaseNetProtocol::Get().SendPause(gu->myPlayerNum, true));
+	}*/
+
 	const SkirmishAIData* aiData = skirmishAIHandler.GetSkirmishAI(skirmishAIId);
 
 	if (aiData->status != SKIRMAISTATE_RELOADING) {
@@ -512,7 +581,6 @@ void CEngineOutHandler::CreateSkirmishAI(const size_t skirmishAIId) {
 		net->Send(CBaseNetProtocol::Get().SendAIStateChanged(gu->myPlayerNum, skirmishAIId, SKIRMAISTATE_ALIVE));
 	} else {
 		CSkirmishAIWrapper* aiWrapper = NULL;
-
 		try {
 			CSkirmishAIWrapper* aiWrapper_tmp = new CSkirmishAIWrapper(skirmishAIId);
 
@@ -525,7 +593,6 @@ void CEngineOutHandler::CreateSkirmishAI(const size_t skirmishAIId) {
 		} CATCH_AI_EXCEPTION;
 
 		const bool isDieing = skirmishAIHandler.IsLocalSkirmishAIDieing(skirmishAIId);
-
 		if (!isDieing) {
 			if (gs->frameNum > 0) {
 				// We will only get here if the AI is created mid-game.
@@ -534,7 +601,7 @@ void CEngineOutHandler::CreateSkirmishAI(const size_t skirmishAIId) {
 			// Send a UnitCreated event for each unit of the team.
 			// This will only do something if the AI is created mid-game.
 			const CTeam* team = teamHandler->Team(aiWrapper->GetTeamId());
-			CUnitSet::iterator u, uNext;
+			CUnitSet::const_iterator u, uNext;
 			for (u = team->units.begin(); u != team->units.end(); ) {
 				uNext = u; ++uNext;
 				try {
@@ -557,9 +624,9 @@ void CEngineOutHandler::SetSkirmishAIDieing(const size_t skirmishAIId) {
 	} CATCH_AI_EXCEPTION;
 }
 
-static void internal_aiErase(std::vector<size_t>& ais, const size_t skirmishAIId) {
+static void internal_aiErase(std::vector<unsigned char>& ais, const unsigned char skirmishAIId) {
 
-	for (std::vector<size_t>::iterator ai = ais.begin(); ai != ais.end(); ++ai) {
+	for (std::vector<unsigned char>::iterator ai = ais.begin(); ai != ais.end(); ++ai) {
 		if (*ai == skirmishAIId) {
 			ais.erase(ai);
 			return;
