@@ -1,37 +1,40 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 #include <SDL_keysym.h>
 #include <boost/cstdint.hpp>
 
 #include "OverheadController.h"
 
-#include "ConfigHandler.h"
+#include "System/Config/ConfigHandler.h"
 #include "Game/Camera.h"
 #include "Game/CameraHandler.h"
 #include "Game/UI/MouseHandler.h"
-#include "LogOutput.h"
 #include "Map/Ground.h"
-#include "GlobalUnsynced.h"
 #include "Rendering/GlobalRendering.h"
-#include "myMath.h"
+#include "System/Log/ILog.h"
+#include "System/myMath.h"
+#include "System/Input/KeyInput.h"
 
-extern boost::uint8_t *keys;
+CONFIG(float, MiddleClickScrollSpeed).defaultValue(0.01f);
+CONFIG(int, OverheadScrollSpeed).defaultValue(10);
+CONFIG(float, OverheadTiltSpeed).defaultValue(1.0f);
+CONFIG(bool, OverheadEnabled).defaultValue(true);
+CONFIG(float, OverheadFOV).defaultValue(45.0f);
 
 COverheadController::COverheadController()
-	: flipped(false),
-	zscale(0.5f),
-	height(500),
-	oldAltHeight(500),
-	changeAltHeight(true),
-	maxHeight(10000)
+	: flipped(false)
+	, zscale(0.5f)
+	, height(500)
+	, oldAltHeight(500)
+	, changeAltHeight(true)
+	, maxHeight(10000)
 {
-	middleClickScrollSpeed = configHandler->Get("MiddleClickScrollSpeed", 0.01f);
-	scrollSpeed = configHandler->Get("OverheadScrollSpeed",10)*0.1f;
-	tiltSpeed = configHandler->Get("OverheadTiltSpeed",1.0f);
-	enabled = !!configHandler->Get("OverheadEnabled",1);
-	fov = configHandler->Get("OverheadFOV", 45.0f);
+	middleClickScrollSpeed = configHandler->GetFloat("MiddleClickScrollSpeed");
+	scrollSpeed = configHandler->GetInt("OverheadScrollSpeed")*0.1f;
+	tiltSpeed = configHandler->GetFloat("OverheadTiltSpeed");
+	enabled = configHandler->GetBool("OverheadEnabled");
+	fov = configHandler->GetFloat("OverheadFOV");
 }
 
 void COverheadController::KeyMove(float3 move)
@@ -41,9 +44,9 @@ void COverheadController::KeyMove(float3 move)
 		move.y = -move.y;
 	}
 	move *= sqrt(move.z) * 200;
-	float pixelsize = camera->GetTanHalfFov() * 2/globalRendering->viewSizeY * height * 2;
-	pos.x += move.x * pixelsize * 2 * scrollSpeed;
-	pos.z -= move.y * pixelsize * 2 * scrollSpeed;
+
+	pos.x += move.x * pixelSize * 2.0f * scrollSpeed;
+	pos.z -= move.y * pixelSize * 2.0f * scrollSpeed;
 }
 
 void COverheadController::MouseMove(float3 move)
@@ -53,9 +56,9 @@ void COverheadController::MouseMove(float3 move)
 		move.y = -move.y;
 	}
 	move *= 100 * middleClickScrollSpeed;
-	float pixelsize = camera->GetTanHalfFov() * 2/globalRendering->viewSizeY * height * 2;
-	pos.x += move.x * pixelsize * (1+keys[SDLK_LSHIFT]*3) * scrollSpeed;
-	pos.z += move.y * pixelsize * (1+keys[SDLK_LSHIFT]*3) * scrollSpeed;
+
+	pos.x += move.x * pixelSize * (1 + keyInput->GetKeyState(SDLK_LSHIFT) * 3) * scrollSpeed;
+	pos.z += move.y * pixelSize * (1 + keyInput->GetKeyState(SDLK_LSHIFT) * 3) * scrollSpeed;
 }
 
 void COverheadController::ScreenEdgeMove(float3 move)
@@ -66,24 +69,24 @@ void COverheadController::ScreenEdgeMove(float3 move)
 void COverheadController::MouseWheelMove(float move)
 {
 	// tilt the camera if LCTRL is pressed
-	if (keys[SDLK_LCTRL]) {
-		zscale *= (1.0f + (0.01f * move * tiltSpeed * (keys[SDLK_LSHIFT] ? 3.0f : 1.0f)));
+	if (keyInput->IsKeyPressed(SDLK_LCTRL)) {
+		zscale *= (1.0f + (0.01f * move * tiltSpeed * (keyInput->IsKeyPressed(SDLK_LSHIFT) ? 3.0f : 1.0f)));
 		zscale = Clamp(zscale, 0.05f, 10.0f);
 	} else { // holding down LALT uses 'instant-zoom' from here to the end of the function
 		// ZOOM IN to mouse cursor instead of mid screen
 		if (move < 0) {
 			float3 cpos = pos - dir * height;
-			float dif = -height * move * 0.007f * (keys[SDLK_LSHIFT] ? 3:1);
+			float dif = -height * move * 0.007f * (keyInput->IsKeyPressed(SDLK_LSHIFT) ? 3:1);
 			if ((height - dif) < 60.0f) {
 				dif = height - 60.0f;
 			}
-			if (keys[SDLK_LALT]) { // instant-zoom: zoom in to standard view
+			if (keyInput->IsKeyPressed(SDLK_LALT)) { // instant-zoom: zoom in to standard view
 				dif = (height - oldAltHeight) / mouse->dir.y * dir.y;
 			}
 			float3 wantedPos = cpos + mouse->dir * dif;
-			float newHeight = ground->LineGroundCol(wantedPos, wantedPos + dir * 15000);
+			float newHeight = ground->LineGroundCol(wantedPos, wantedPos + dir * 15000, false);
 			if (newHeight < 0) {
-				newHeight = height* (1.0f + move * 0.007f * (keys[SDLK_LSHIFT] ? 3:1));
+				newHeight = height* (1.0f + move * 0.007f * (keyInput->IsKeyPressed(SDLK_LSHIFT) ? 3:1));
 			}
 			if ((wantedPos.y + (dir.y * newHeight)) < 0) {
 				newHeight = -wantedPos.y / dir.y;
@@ -94,7 +97,7 @@ void COverheadController::MouseWheelMove(float move)
 			}
 		// ZOOM OUT from mid screen
 		} else {
-			if (keys[SDLK_LALT]) { // instant-zoom: zoom out to the max
+			if (keyInput->IsKeyPressed(SDLK_LALT)) { // instant-zoom: zoom out to the max
 				if(height < maxHeight*0.5f && changeAltHeight){
 					oldAltHeight = height;
 					changeAltHeight = false;
@@ -103,11 +106,11 @@ void COverheadController::MouseWheelMove(float move)
 				pos.x  = gs->mapx * 4;
 				pos.z  = gs->mapy * 4.8f; // somewhat longer toward bottom
 			} else {
-				height *= 1 + move * 0.007f * (keys[SDLK_LSHIFT] ? 3:1);
+				height *= 1 + move * 0.007f * (keyInput->IsKeyPressed(SDLK_LSHIFT) ? 3:1);
 			}
 		}
 		// instant-zoom: turn on the smooth transition and reset the camera tilt
-		if (keys[SDLK_LALT]) {
+		if (keyInput->IsKeyPressed(SDLK_LALT)) {
 			zscale = 0.5f;
 			camHandler->CameraTransition(1.0f);
 		} else {
@@ -118,6 +121,7 @@ void COverheadController::MouseWheelMove(float move)
 
 void COverheadController::Update()
 {
+	pixelSize = (camera->GetTanHalfFov() * 2.0f) / globalRendering->viewSizeY * height * 2.0f;
 }
 
 float3 COverheadController::GetPos()
@@ -128,7 +132,7 @@ float3 COverheadController::GetPos()
 	pos.z = Clamp(pos.z, 0.01f, gs->mapy * SQUARE_SIZE - 0.01f);
 	height = Clamp(height, 60.0f, maxHeight);
 
-	pos.y = ground->GetHeight(pos.x,pos.z);
+	pos.y = ground->GetHeightAboveWater(pos.x, pos.z, false);
 	dir = float3(0.0f, -1.0f, flipped ? zscale : -zscale).ANormalize();
 
 	float3 cpos = pos - dir * height;
@@ -147,8 +151,9 @@ float3 COverheadController::SwitchFrom() const
 
 void COverheadController::SwitchTo(bool showText)
 {
-	if(showText)
-		logOutput.Print("Switching to Overhead (TA) style camera");
+	if (showText) {
+		LOG("Switching to Overhead (TA) style camera");
+	}
 }
 
 void COverheadController::GetState(StateMap& sm) const
