@@ -1,7 +1,7 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#ifndef __GAME_H__
-#define __GAME_H__
+#ifndef _GAME_H
+#define _GAME_H
 
 #include <time.h>
 #include <string>
@@ -9,31 +9,35 @@
 #include <set>
 
 #include "GameController.h"
-#include "creg/creg_cond.h"
+#include "System/creg/creg_cond.h"
 
-#include "lib/gml/gml.h"
-
-class CBaseWater;
+class IWater;
 class CConsoleHistory;
-class CWordCompletion;
 class CKeySet;
 class CInfoConsole;
 class LuaParser;
 class LuaInputReceiver;
 class ILoadSaveHandler;
 class Action;
+class ISyncedActionExecutor;
+class IUnsyncedActionExecutor;
 class ChatMessage;
 class SkirmishAIData;
+class CWorldDrawer;
 
-const int MAX_CONSECUTIVE_SIMFRAMES = 15;
 
 class CGame : public CGameController
 {
 private:
 	CR_DECLARE(CGame);	// Do not use CGame pointer in CR_MEMBER()!!!
 
+public:
+	void LoadGame(const std::string& mapName);
+	void SetupRenderingParams();
+
+private:
 	void LoadDefs();
-	void LoadSimulation(const std::string&);
+	void LoadSimulation(const std::string& mapName);
 	void LoadRendering();
 	void LoadInterface();
 	void LoadLua();
@@ -41,25 +45,33 @@ private:
 	void PostLoad();
 
 public:
-	CGame(std::string mapname, std::string modName, ILoadSaveHandler *saveFile);
+	CGame(const std::string& mapName, const std::string& modName, ILoadSaveHandler* saveFile);
+	virtual ~CGame();
 
 	bool Draw();
 	bool DrawMT();
 
-	static void DrawMTcb(void *c) {((CGame *)c)->DrawMT();}
+	static void DrawMTcb(void* c) { ((CGame *)c)->DrawMT(); }
 	bool Update();
 	/// Called when a key is released by the user
 	int KeyReleased(unsigned short k);
 	/// Called when the key is pressed by the user (can be called several times due to key repeat)
-	int KeyPressed(unsigned short k,bool isRepeat);
+	int KeyPressed(unsigned short k, bool isRepeat);
 	void ResizeEvent();
-	virtual ~CGame();
 
-	bool ActionPressed(const Action&, const CKeySet& ks, bool isRepeat);
-	bool ActionReleased(const Action&);
-	
+
+	bool ProcessCommandText(unsigned int key, const std::string& command);
+	bool ProcessKeyPressAction(unsigned int key, const Action& action);
+
+	bool ActionPressed(unsigned int key, const Action& action, bool isRepeat);
+	bool ActionReleased(const Action& action);
+
 	bool HasLag() const;
 
+	/// show GameEnd-window, calculate mouse movement etc.
+	void GameEnd(const std::vector<unsigned char>& winningAllyTeams);
+
+
 	enum GameDrawMode {
 		gameNotDrawing     = 0,
 		gameNormalDraw     = 1,
@@ -104,35 +116,33 @@ public:
 	bool showFPS;
 	bool showClock;
 	bool showSpeed;
-	bool showMTInfo;
+	int showMTInfo;
+	int mtInfoCtrl;
 	/// Prevents spectator msgs from being seen by players
 	bool noSpectatorChat;
+	volatile bool finishedLoading;
 
 	unsigned char gameID[16];
 
-	CInfoConsole *infoConsole;
-
-	void MakeMemDump(void);
-
+	CInfoConsole* infoConsole;
 	CConsoleHistory* consoleHistory;
-	CWordCompletion* wordCompletion;
 
 	void SetHotBinding(const std::string& action) { hotBinding = action; }
 
-private:
+public:
 	/// Save the game state to file.
 	void SaveGame(const std::string& filename, bool overwrite);
+	void DumpState(int newMinFrameNum, int newMaxFrameNum, int newFramePeriod);
+
 	/// Re-load the game.
 	void ReloadGame();
-	/// show GameEnd-window, calculate mouse movement etc.
-	void GameEnd();
 	/// Send a message to other players (allows prefixed messages with e.g. "a:...")
 	void SendNetChat(std::string message, int destination = -1);
 	/// Format and display a chat message received over network
 	void HandleChatMsg(const ChatMessage& msg);
-	
+
 	/// synced actions (received from server) go in here
-	void ActionReceived(const Action&, int playernum);
+	void ActionReceived(const Action& action, int playerID);
 
 	void DrawInputText();
 	void ParseInputTextGeometry(const std::string& geo);
@@ -142,11 +152,15 @@ private:
 
 	void ReColorTeams();
 
+public:
 	void ReloadCOB(const std::string& msg, int player);
+	void ReloadCEGs(const std::string& tag);
+
 	void StartSkip(int toFrame);
 	void DrawSkip(bool blackscreen = true);
 	void EndSkip();
 
+public:
 	std::string hotBinding;
 	float inputTextPosX;
 	float inputTextPosY;
@@ -169,14 +183,13 @@ public:
 		return playerTraffic;
 	}
 
-private:
+public:
 	void AddTraffic(int playerID, int packetCode, int length);
 	// <playerID, <packetCode, total bytes> >
 	std::map<int, PlayerTrafficInfo> playerTraffic;
 
 	void ClientReadNet();
 	void UpdateUI(bool cam);
-	bool DrawWorld();
 
 	void SimFrame();
 	void StartPlaying();
@@ -196,16 +209,23 @@ private:
 	float skipOldUserSpeed;
 	unsigned skipLastDraw;
 
+	/**
+	 * @see CGameServer#speedControl
+	 */
 	int speedControl;
-	int luaDrawTime;
+	int luaLockTime;
+	int luaExportSize;
 
 
 	/// for reloading the savefile
 	ILoadSaveHandler* saveFile;
+
+private:
+	CWorldDrawer* worldDrawer;
 };
 
 
 extern CGame* game;
 
 
-#endif // __GAME_H__
+#endif // _GAME_H
