@@ -1,21 +1,16 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "Rendering/GL/myGL.h"
-#include "mmgr.h"
+#include "GameHelper.h"
 
-#include "GlobalUnsynced.h"
 #include "Camera.h"
 #include "GameSetup.h"
-#include "GameHelper.h"
+#include "Game/GlobalUnsynced.h"
 #include "UI/LuaUI.h"
-#include "LogOutput.h"
+#include "Lua/LuaRules.h"
 #include "Map/Ground.h"
 #include "Map/MapDamage.h"
 #include "Map/ReadMap.h"
-#include "Rendering/Env/BaseWater.h"
 #include "Rendering/Models/3DModel.h"
-#include "Rendering/GroundDecalHandler.h"
 #include "Sim/Features/Feature.h"
 #include "Sim/Misc/CollisionHandler.h"
 #include "Sim/Misc/CollisionVolume.h"
@@ -28,17 +23,20 @@
 #include "Sim/Misc/RadarHandler.h"
 #include "Sim/Misc/ModInfo.h"
 #include "Sim/Projectiles/ExplosionGenerator.h"
+#include "Sim/Projectiles/ExplosionListener.h"
 #include "Sim/Projectiles/Projectile.h"
 #include "Sim/Units/CommandAI/MobileCAI.h"
 #include "Sim/Units/UnitTypes/Factory.h"
+#include "Sim/Units/BuildInfo.h"
 #include "Sim/Units/UnitDef.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitHandler.h"
 #include "Sim/Weapons/WeaponDefHandler.h"
 #include "Sim/Weapons/Weapon.h"
-#include "Sync/SyncTracer.h"
-#include "EventHandler.h"
-#include "myMath.h"
+#include "System/EventHandler.h"
+#include "System/mmgr.h"
+#include "System/myMath.h"
+#include "System/Sync/SyncTracer.h"
 
 //////////////////////////////////////////////////////////////////////
 // Construction/Destruction
@@ -49,30 +47,36 @@ CGameHelper* helper;
 
 CGameHelper::CGameHelper()
 {
-	stdExplosionGenerator = new CStdExplosionGenerator;
+	stdExplosionGenerator = new CStdExplosionGenerator();
 }
 
 CGameHelper::~CGameHelper()
 {
 	delete stdExplosionGenerator;
 
-	for(int a=0;a<128;++a){
-		std::list<WaitingDamage*>* wd=&waitingDamages[a];
-		while(!wd->empty()){
+	for (int a = 0; a < 128; ++a) {
+		std::list<WaitingDamage*>* wd = &waitingDamages[a];
+		while (!wd->empty()) {
 			delete wd->back();
 			wd->pop_back();
 		}
 	}
 }
 
+
+
 //////////////////////////////////////////////////////////////////////
 // Explosions/Damage
 //////////////////////////////////////////////////////////////////////
 
-void CGameHelper::DoExplosionDamage(CUnit* unit,
-	const float3& expPos, float expRad, float expSpeed,
-	bool ignoreOwner, CUnit* owner, float edgeEffectiveness,
-	const DamageArray& damages, int weaponId)
+void CGameHelper::DoExplosionDamage(
+	CUnit* unit,
+	CUnit* owner,
+	const float3& expPos,
+	float expRad,
+	float expSpeed,
+	bool ignoreOwner, float edgeEffectiveness,
+	const DamageArray& damages, int weaponDefID)
 {
 	if (ignoreOwner && (unit == owner)) {
 		return;
@@ -92,8 +96,8 @@ void CGameHelper::DoExplosionDamage(CUnit* unit,
 	float3 diffPos;
 
 	if (piece != NULL && unit->unitDef->usePieceCollisionVolumes && damageFrame == gs->frameNum) {
-		volume = piece->colvol;
-		basePos = piece->GetPos() + volume->GetOffsets();
+		volume = piece->GetCollisionVolume();
+		basePos = piece->GetAbsolutePos() + volume->GetOffsets();
 		basePos = unit->pos + 
 			unit->rightdir * basePos.x +
 			unit->updir    * basePos.y +
@@ -142,22 +146,26 @@ void CGameHelper::DoExplosionDamage(CUnit* unit,
 		expDist2 = std::min(expDist2, expRad);
 	}
 
-	float mod  = (expRad - expDist1) / (expRad - expDist1 * edgeEffectiveness);
-	float mod2 = (expRad - expDist2) / (expRad - expDist2 * edgeEffectiveness);
+	const float mod1 = std::max(0.01f, (expRad - expDist1) / (expRad - (expDist1 * edgeEffectiveness)));
+	const float mod2 = std::max(0.01f, (expRad - expDist2) / (expRad - (expDist2 * edgeEffectiveness)));
 
 	diffPos /= expDist;
 	diffPos.y += 0.12f;
 
-	if (mod < 0.01f)
-		mod = 0.01f;
-
+	// limit the impulse to prevent later FP overflow
+	// (several weapons have _default_ damage values in the order of 1e4,
+	// which make the simulation highly unstable because they can impart
+	// speeds of several thousand elmos/frame to units and throw them far
+	// outside the map)
 	const DamageArray damageDone = damages * mod2;
-	const float3 addedImpulse = diffPos * (damages.impulseFactor * mod * (damages[0] + damages.impulseBoost) * 3.2f);
+	const float rawImpulseStrength = damages.impulseFactor * mod1 * (damages.GetDefaultDamage() + damages.impulseBoost) * 3.2f;
+	const float modImpulseStrength = Clamp(rawImpulseStrength, -MAX_EXPLOSION_IMPULSE, MAX_EXPLOSION_IMPULSE);
+	const float3 addedImpulse = diffPos * modImpulseStrength;
 
 	if (expDist2 < (expSpeed * 4.0f)) { // damage directly
-		unit->DoDamage(damageDone, owner, addedImpulse, weaponId);
+		unit->DoDamage(damageDone, owner, addedImpulse, weaponDefID);
 	} else { // damage later
-		WaitingDamage* wd = new WaitingDamage((owner? owner->id: -1), unit->id, damageDone, addedImpulse, weaponId);
+		WaitingDamage* wd = new WaitingDamage((owner? owner->id: -1), unit->id, damageDone, addedImpulse, weaponDefID);
 		waitingDamages[(gs->frameNum + int(expDist2 / expSpeed) - 3) & 127].push_front(wd);
 	}
 }
@@ -172,7 +180,7 @@ void CGameHelper::DoExplosionDamage(CFeature* feature,
 
 		float expDist = std::max(dif.Length(), 0.1f);
 		float expMod = (expRad - expDist) / expRad;
-		float dmgScale = (damages[0] + damages.impulseBoost);
+		float dmgScale = (damages.GetDefaultDamage() + damages.impulseBoost);
 
 		// always do some damage with explosive stuff
 		// (DDM wreckage etc. is too big to normally
@@ -191,46 +199,45 @@ void CGameHelper::DoExplosionDamage(CFeature* feature,
 
 
 
-void CGameHelper::Explosion(
-	float3 expPos, const DamageArray& damages,
-	float expRad, float edgeEffectiveness,
-	float expSpeed, CUnit* owner,
-	bool damageGround, float gfxMod,
-	bool ignoreOwner, bool impactOnly,
-	CExplosionGenerator* explosionGraphics,
-	CUnit* hitUnit,
-	const float3& impactDir, int weaponId,
-	CFeature* hitFeature
-) {
-	if (luaUI) {
-		const WeaponDef* wd = weaponDefHandler->GetWeaponById(weaponId);
+void CGameHelper::Explosion(const ExplosionParams& params) {
+	const float3& pos = params.pos;
+	const float3& dir = params.dir;
+	const DamageArray& damages = params.damages;
 
-		if (wd != NULL && wd->cameraShake > 0.0f) {
-			luaUI->ShockFront(wd->cameraShake, expPos, expRad);
+	const WeaponDef* weaponDef = params.weaponDef;
+	const int weaponDefID = (weaponDef != NULL)? weaponDef->id: -1;
+
+	const float3 expPos = pos;
+
+	CUnit* owner = params.owner;
+	CUnit* hitUnit = params.hitUnit;
+	CFeature* hitFeature = params.hitFeature;
+
+	const float expRad = std::max(1.0f, params.areaOfEffect);
+	const float edgeEffectiveness = params.edgeEffectiveness;
+	const float expSpeed = params.explosionSpeed;
+	const float gfxMod = params.gfxMod;
+	const float realHeight = ground->GetHeightReal(expPos.x, expPos.z);
+	const float altitude = expPos.y - realHeight;
+
+	const bool impactOnly = params.impactOnly;
+	const bool ignoreOwner = params.ignoreOwner;
+	const bool damageGround = params.damageGround;
+	const bool noGfx = eventHandler.Explosion(weaponDefID, expPos, owner);
+
+	if (luaUI) {
+		if (weaponDef != NULL && weaponDef->cameraShake > 0.0f) {
+			luaUI->ShockFront(weaponDef->cameraShake, expPos, expRad);
 		}
 	}
 
-	const bool noGfx = eventHandler.Explosion(weaponId, expPos, owner);
-
-#ifdef TRACE_SYNC
-	tracefile << "Explosion: ";
-	tracefile << expPos.x << " " << damages[0] <<  " " << expRad << "\n";
-#endif
-
-	const float h2 = ground->GetHeight2(expPos.x, expPos.z);
-
-	expPos.y = std::max(expPos.y, h2);
-	expRad = std::max(expRad, 1.0f);
-
 	if (impactOnly) {
 		if (hitUnit) {
-			DoExplosionDamage(hitUnit, expPos, expRad, expSpeed, ignoreOwner, owner, edgeEffectiveness, damages, weaponId);
+			DoExplosionDamage(hitUnit, owner, expPos, expRad, expSpeed, ignoreOwner, edgeEffectiveness, damages, weaponDefID);
 		} else if (hitFeature) {
 			DoExplosionDamage(hitFeature, expPos, expRad, damages);
 		}
 	} else {
-		float height = std::max(expPos.y - h2, 0.0f);
-
 		{
 			// damage all units within the explosion radius
 			const vector<CUnit*>& units = qf->GetUnitsExact(expPos, expRad);
@@ -243,14 +250,14 @@ void CGameHelper::Explosion(
 					hitUnitDamaged = true;
 				}
 
-				DoExplosionDamage(unit, expPos, expRad, expSpeed, ignoreOwner, owner, edgeEffectiveness, damages, weaponId);
+				DoExplosionDamage(unit, owner, expPos, expRad, expSpeed, ignoreOwner, edgeEffectiveness, damages, weaponDefID);
 			}
 
 			// HACK: for a unit with an offset coldet volume, the explosion
 			// (from an impacting projectile) position might not correspond
 			// to its quadfield position so we need to damage it separately
 			if (hitUnit != NULL && !hitUnitDamaged) {
-				DoExplosionDamage(hitUnit, expPos, expRad, expSpeed, ignoreOwner, owner, edgeEffectiveness, damages, weaponId);
+				DoExplosionDamage(hitUnit, owner, expPos, expRad, expSpeed, ignoreOwner, edgeEffectiveness, damages, weaponDefID);
 			}
 		}
 
@@ -274,268 +281,36 @@ void CGameHelper::Explosion(
 			}
 		}
 
-		// deform the map
-		if (damageGround && !mapDamage->disabled && (expRad > height) && (damages.craterMult > 0.0f)) {
-			float damage = damages[0] * (1.0f - (height / expRad));
-			if (damage > (expRad * 10.0f)) {
-				damage = expRad * 10.0f; // limit the depth somewhat
+		// deform the map if the explosion was above-ground
+		// (but had large enough radius to touch the ground)
+		if (altitude >= -1.0f) {
+			if (damageGround && !mapDamage->disabled && (expRad > altitude) && (damages.craterMult > 0.0f)) {
+				// limit the depth somewhat
+				const float craterDepth = damages.GetDefaultDamage() * (1.0f - (altitude / expRad));
+				const float damageDepth = std::min(expRad * 10.0f, craterDepth);
+
+				mapDamage->Explosion(expPos, (damageDepth + damages.craterBoost) * damages.craterMult, expRad - altitude);
 			}
-			mapDamage->Explosion(expPos, (damage + damages.craterBoost) * damages.craterMult, expRad - height);
 		}
 	}
 
-	// use CStdExplosionGenerator by default
 	if (!noGfx) {
-		if (!explosionGraphics) {
-			explosionGraphics = stdExplosionGenerator;
+		// use CStdExplosionGenerator by default
+		IExplosionGenerator* explosionGenerator = stdExplosionGenerator;
+
+		if (weaponDef != NULL && weaponDef->explosionGenerator != NULL) {
+			explosionGenerator = weaponDef->explosionGenerator;
 		}
-		explosionGraphics->Explosion(expPos, damages[0], expRad, owner, gfxMod, hitUnit, impactDir);
+
+		explosionGenerator->Explosion(0, expPos, damages.GetDefaultDamage(), expRad, owner, gfxMod, hitUnit, dir);
 	}
 
-	groundDecals->AddExplosion(expPos, damages[0], expRad);
-	water->AddExplosion(expPos, damages[0], expRad);
+	CExplosionEvent explosionEvent(expPos, damages.GetDefaultDamage(), expRad, weaponDef);
+	FireExplosionEvent(explosionEvent);
 }
 
 
 
-//////////////////////////////////////////////////////////////////////
-// Raytracing
-//////////////////////////////////////////////////////////////////////
-
-// called by {CRifle, CBeamLaser, CLightningCannon}::Fire() and Skirmish AIs
-float CGameHelper::TraceRay(const float3& start, const float3& dir, float length, float /*power*/,
-			    const CUnit* owner, const CUnit*& hit, int collisionFlags,
-			    const CFeature** hitfeature)
-{
-	const bool ignoreAllies = !!(collisionFlags & COLLISION_NOFRIENDLY);
-	const bool ignoreFeatures = !!(collisionFlags & COLLISION_NOFEATURE);
-	const bool ignoreNeutrals = !!(collisionFlags & COLLISION_NONEUTRAL);
-
-	float lengthSq = length*length;
-
-	CollisionQuery cq;
-
-	GML_RECMUTEX_LOCK(quad); // TraceRay
-
-	vector<int> quads = qf->GetQuadsOnRay(start, dir, length);
-	vector<int>::iterator qi;
-	std::list<CUnit*>::const_iterator ui;
-
-	//! feature intersection
-	if (!ignoreFeatures) {
-		if (hitfeature)
-			*hitfeature = 0;
-
-		for (qi = quads.begin(); qi != quads.end(); ++qi) {
-			const CQuadField::Quad& quad = qf->GetQuad(*qi);
-
-			for (std::list<CFeature*>::const_iterator ui = quad.features.begin(); ui != quad.features.end(); ++ui) {
-				CFeature* f = *ui;
-
-				if (!f->blocking || !f->collisionVolume) {
-					// NOTE: why check the blocking property?
-					continue;
-				}
-
-				if (CCollisionHandler::Intersect(f, start, start + dir * length, &cq)) {
-					const float3& intPos = (cq.b0)? cq.p0: cq.p1;
-					const float lenSq = (intPos - start).SqLength();
-
-					//! we want the closest feature (intersection point) on the ray
-					if (lenSq < lengthSq) {
-						lengthSq = lenSq;
-						if(hitfeature)
-							*hitfeature = f;
-					}
-				}
-			}
-		}
-	}
-
-	//! unit intersection
-	hit = NULL;
-	for (qi = quads.begin(); qi != quads.end(); ++qi) {
-		const CQuadField::Quad& quad = qf->GetQuad(*qi);
-
-		for (std::list<CUnit*>::const_iterator ui = quad.units.begin(); ui != quad.units.end(); ++ui) {
-			const CUnit* u = *ui;
-
-			if (u == owner)
-				continue;
-			if (ignoreAllies && u->allyteam == owner->allyteam)
-				continue;
-			if (ignoreNeutrals && u->IsNeutral())
-				continue;
-
-			if (CCollisionHandler::Intersect(u, start, start + dir * length, &cq)) {
-				const float3& intPos = (cq.b0)? cq.p0: cq.p1;
-				const float lenSq = (intPos - start).SqLength();
-
-				//! we want the closest unit (intersection point) on the ray
-				if (lenSq < lengthSq) {
-					lengthSq = lenSq;
-					hit = u;
-				}
-			}
-		}
-	}
-
-	length = math::sqrt(lengthSq);
-
-	//! ground intersection
-	float groundLength = ground->LineGroundCol(start, start + dir * length);
-	if (length > groundLength && groundLength > 0) {
-		length = groundLength;
-	}
-
-	return length;
-}
-
-float CGameHelper::GuiTraceRay(const float3 &start, const float3 &dir, float length, const CUnit*& hit, bool useRadar, const CUnit* exclude)
-{
-	if (dir == ZeroVector) {
-		return -1.0f;
-	}
-
-	float returnLenSq = 1e9;
-	bool hover_factory = false;
-
-	hit = NULL;
-	CollisionQuery cq;
-
-	{
-		GML_RECMUTEX_LOCK(quad); //! GuiTraceRay
-
-		vector<int> quads = qf->GetQuadsOnRay(start, dir, length);
-		vector<int>::iterator qi;
-		std::list<CUnit*>::const_iterator ui;
-
-		//! unit intersection
-		for (qi = quads.begin(); qi != quads.end(); ++qi) {
-			const CQuadField::Quad& quad = qf->GetQuad(*qi);
-
-			for (ui = quad.units.begin(); ui != quad.units.end(); ++ui) {
-				CUnit* unit = *ui;
-				if (unit == exclude) {
-					continue;
-				}
-	
-				if ((unit->allyteam == gu->myAllyTeam) || gu->spectatingFullView ||
-					(unit->losStatus[gu->myAllyTeam] & (LOS_INLOS | LOS_CONTRADAR)) ||
-					(useRadar && radarhandler->InRadar(unit, gu->myAllyTeam)))
-				{
-		
-					CollisionVolume cv(unit->collisionVolume);
-	
-					if (unit->isIcon) {
-						//! for iconified units, just pretend the collision
-						//! volume is a sphere of radius <unit->IconRadius>
-						cv.Init(unit->iconRadius);
-					}
-	
-					if (CCollisionHandler::MouseHit(unit, start, start + dir * length, &cv, &cq)) {
-						//! get the distance to the ray-volume ingress point
-						const float lenSq = (cq.p0 - start).SqLength();
-						const bool isfactory = dynamic_cast<CFactory*>(unit);
-
-						if (lenSq < returnLenSq) {
-							if (!isfactory || !hit || hover_factory) {
-								hover_factory = isfactory;
-								returnLenSq = lenSq;
-								hit = unit;
-							}
-						} else if (!isfactory && hover_factory) {
-							//! give an unit in a factory a higher priority than the factory itself
-							hover_factory = isfactory;
-							returnLenSq = lenSq;
-							hit = unit;
-						}
-					}
-				}
-			}
-		}
-	}
-
-	float returnLen = math::sqrt(returnLenSq);
-
-	//! ground intersection
-	float groundLen = ground->LineGroundCol(start, start + dir * returnLen);
-	if (groundLen > 0.0f) {
-		returnLen  = (groundLen+200.0f < returnLen)? groundLen : returnLen;
-	}
-
-	return returnLen;
-}
-
-//! called by CPlayer (just for DirectControlUnit a.k.a. FPS mode)
-float CGameHelper::TraceRayTeam(const float3& start, const float3& dir, float length, CUnit*& hit, bool useRadar, CUnit* exclude, int allyteam)
-{
-	float groundLength = ground->LineGroundCol(start, start + dir * length);
-
-	if (length > groundLength && groundLength > 0) {
-		length = groundLength;
-	}
-
-	GML_RECMUTEX_LOCK(quad); // TraceRayTeam
-
-	vector<int> quads = qf->GetQuadsOnRay(start, dir, length);
-	hit = 0;
-
-	vector<int>::iterator qi;
-	for (qi = quads.begin(); qi != quads.end(); ++qi) {
-		const CQuadField::Quad& quad = qf->GetQuad(*qi);
-		std::list<CUnit*>::const_iterator ui;
-
-		// NOTE: switch this to custom volumes fully? (only
-		// used in FPS unit control mode, maybe unnecessary)
-		for (ui = quad.units.begin(); ui != quad.units.end(); ++ui) {
-			CUnit* u = *ui;
-
-			if (u == exclude)
-				continue;
-
-			const CollisionVolume* cv = u->collisionVolume;
-
-			if (teamHandler->Ally(u->allyteam, allyteam) || (u->losStatus[allyteam] & LOS_INLOS)) {
-				float3 dif = (u->midPos + cv->GetOffsets()) - start;
-				float closeLength = dif.dot(dir);
-
-				if (closeLength < 0)
-					continue;
-				if (closeLength > length)
-					continue;
-
-				float3 closeVect = dif - dir * closeLength;
-
-				if (closeVect.SqLength() < cv->GetBoundingRadiusSq()) {
-					length = closeLength;
-					hit = u;
-				}
-			} else if (useRadar && radarhandler->InRadar(u, allyteam)) {
-				float3 dif =
-					(u->midPos + cv->GetOffsets()) +
-					u->posErrorVector * radarhandler->radarErrorSize[allyteam] -
-					start;
-				float closeLength = dif.dot(dir);
-
-				if (closeLength < 0)
-					continue;
-				if (closeLength > length)
-					continue;
-
-				float3 closeVect = dif - dir * closeLength;
-
-				if (closeVect.SqLength() < cv->GetBoundingRadiusSq()) {
-					length = closeLength;
-					hit = u;
-				}
-			}
-		}
-	}
-
-	return length;
-}
-
 //////////////////////////////////////////////////////////////////////
 // Spatial unit queries
 //////////////////////////////////////////////////////////////////////
@@ -561,12 +336,11 @@ static inline void QueryUnits(TFilter filter, TQuery& query)
 {
 	GML_RECMUTEX_LOCK(qnum);
 
-	vector<int> quads = qf->GetQuads(query.pos, query.radius);
+	const vector<int> &quads = qf->GetQuads(query.pos, query.radius);
 
 	const int tempNum = gs->tempNum++;
-	vector<int>::iterator qi;
-
-	for (qi = quads.begin(); qi != quads.end(); ++qi) {
+	
+	for (vector<int>::const_iterator qi = quads.begin(); qi != quads.end(); ++qi) {
 		const CQuadField::Quad& quad = qf->GetQuad(*qi);
 		for (int t = 0; t < teamHandler->ActiveAllyTeams(); ++t) {
 			if (!filter.Team(t)) {
@@ -832,99 +606,129 @@ public:
 
 
 
-void CGameHelper::GenerateTargets(const CWeapon *weapon, CUnit* lastTarget,
-                                  std::map<float,CUnit*> &targets)
+void CGameHelper::GenerateWeaponTargets(const CWeapon* weapon, const CUnit* lastTargetUnit, std::multimap<float, CUnit*>& targets)
 {
 	GML_RECMUTEX_LOCK(qnum); // GenerateTargets
 
-	CUnit* attacker = weapon->owner;
-	float radius = weapon->range;
-	float3 pos = attacker->pos;
-	float heightMod = weapon->heightMod;
-	float aHeight = weapon->weaponPos.y;
+	const CUnit* attacker = weapon->owner;
+	const float radius    = weapon->range;
+	const float3& pos     = attacker->pos;
+	const float heightMod = weapon->heightMod;
+	const float aHeight   = weapon->weaponPos.y;
+
 	// how much damage the weapon deals over 1 second
-	float secDamage = weapon->weaponDef->damages[0] * weapon->salvoSize / weapon->reloadTime * 30;
-	bool paralyzer = !!weapon->weaponDef->damages.paralyzeDamageTime;
+	const float secDamage = weapon->weaponDef->damages.GetDefaultDamage() * weapon->salvoSize / weapon->reloadTime * GAME_SPEED;
+	const bool paralyzer  = !!weapon->weaponDef->damages.paralyzeDamageTime;
 
-	std::vector<int> quads = qf->GetQuads(pos, radius + (aHeight - std::max(0.f, readmap->minheight)) * heightMod);
+	const std::vector<int>& quads = qf->GetQuads(pos, radius + (aHeight - std::max(0.f, readmap->initMinHeight)) * heightMod);
 
-	int tempNum = gs->tempNum++;
-	std::vector<int>::iterator qi;
-	for (qi = quads.begin(); qi != quads.end(); ++qi) {
+	const int tempNum = gs->tempNum++;
+
+	typedef std::vector<int>::const_iterator VectorIt;
+	typedef std::list<CUnit*>::const_iterator ListIt;
+	
+	for (VectorIt qi = quads.begin(); qi != quads.end(); ++qi) {
 		for (int t = 0; t < teamHandler->ActiveAllyTeams(); ++t) {
 			if (teamHandler->Ally(attacker->allyteam, t)) {
 				continue;
 			}
-			std::list<CUnit*>::const_iterator ui;
+
 			const std::list<CUnit*>& allyTeamUnits = qf->GetQuad(*qi).teamUnits[t];
-			for (ui = allyTeamUnits.begin(); ui != allyTeamUnits.end(); ++ui) {
-				CUnit* unit = *ui;
-				if (unit->tempNum != tempNum && (unit->category & weapon->onlyTargetCategory)) {
-					unit->tempNum = tempNum;
-					if (unit->isUnderWater && !weapon->weaponDef->waterweapon) {
+
+			for (ListIt ui = allyTeamUnits.begin(); ui != allyTeamUnits.end(); ++ui) {
+				CUnit* targetUnit = *ui;
+				float targetPriority = 1.0f;
+
+				if (luaRules != NULL) {
+					const int targetAllowed = luaRules->AllowWeaponTarget(attacker->id, targetUnit->id, weapon->weaponNum, weapon->weaponDef->id, &targetPriority);
+
+					if (targetAllowed >= 0) {
+						if (targetAllowed > 0) {
+							targets.insert(std::pair<float, CUnit*>(targetPriority, targetUnit));
+						}
+
 						continue;
 					}
-					if (unit->isDead) {
+				}
+
+
+				if (targetUnit->tempNum != tempNum && (targetUnit->category & weapon->onlyTargetCategory)) {
+					targetUnit->tempNum = tempNum;
+
+					if (targetUnit->isUnderWater && !weapon->weaponDef->waterweapon) {
 						continue;
 					}
+					if (targetUnit->isDead) {
+						continue;
+					}
+
 					float3 targPos;
-					float value = 1.0f;
-					unsigned short unitLos = unit->losStatus[attacker->allyteam];
-					if (unitLos & LOS_INLOS) {
-						targPos = unit->midPos;
-					} else if (unitLos & LOS_INRADAR) {
-						const float radErr = radarhandler->radarErrorSize[attacker->allyteam];
-						targPos = unit->midPos + (unit->posErrorVector * radErr);
-						value *= 10.0f;
+					const unsigned short targetLOSState = targetUnit->losStatus[attacker->allyteam];
+
+					if (targetLOSState & LOS_INLOS) {
+						targPos = targetUnit->midPos;
+					} else if (targetLOSState & LOS_INRADAR) {
+						targPos = targetUnit->midPos + (targetUnit->posErrorVector * radarhandler->radarErrorSize[attacker->allyteam]);
+						targetPriority *= 10.0f;
 					} else {
 						continue;
 					}
+
 					const float modRange = radius + (aHeight - targPos.y) * heightMod;
-					if ((pos - targPos).SqLength2D() <= modRange * modRange){
-						float dist2d = (pos - targPos).Length2D();
-						value *= (dist2d * weapon->weaponDef->proximityPriority + modRange * 0.4f + 100.0f);
-						if (unitLos & LOS_INLOS) {
-							value *= (secDamage + unit->health);
-							if (unit == lastTarget) {
-								value *= weapon->avoidTarget ? 10.0f : 0.4f;
+
+					if ((pos - targPos).SqLength2D() <= modRange * modRange) {
+						const float dist2D = (pos - targPos).Length2D();
+						const float rangeMul = (dist2D * weapon->weaponDef->proximityPriority + modRange * 0.4f + 100.0f);
+						const float damageMul = weapon->weaponDef->damages[targetUnit->armorType] * targetUnit->curArmorMultiple;
+
+						targetPriority *= rangeMul;
+
+						if (targetLOSState & LOS_INLOS) {
+							targetPriority *= (secDamage + targetUnit->health);
+
+							if (targetUnit == lastTargetUnit) {
+								targetPriority *= weapon->avoidTarget ? 10.0f : 0.4f;
 							}
 
-							if (paralyzer && unit->paralyzeDamage > (modInfo.paralyzeOnMaxHealth? unit->maxHealth: unit->health)) {
-								value *= 4.0f;
+							if (paralyzer && targetUnit->paralyzeDamage > (modInfo.paralyzeOnMaxHealth? targetUnit->maxHealth: targetUnit->health)) {
+								targetPriority *= 4.0f;
 							}
 
 							if (weapon->hasTargetWeight) {
-								value *= weapon->TargetWeight(unit);
+								targetPriority *= weapon->TargetWeight(targetUnit);
 							}
 						} else {
-							value *= (secDamage + 10000.0f);
+							targetPriority *= (secDamage + 10000.0f);
 						}
-						if (unitLos & LOS_PREVLOS) {
-							value /= weapon->weaponDef->damages[unit->armorType]
-											 * unit->curArmorMultiple
-									 * unit->power * (0.7f + gs->randFloat() * 0.6f);
-							if (unit->category & weapon->badTargetCategory) {
-								value *= 100.0f;
+
+						if (targetLOSState & LOS_PREVLOS) {
+							targetPriority /= (damageMul * targetUnit->power * (0.7f + gs->randFloat() * 0.6f));
+
+							if (targetUnit->category & weapon->badTargetCategory) {
+								targetPriority *= 100.0f;
 							}
-							if (unit->crashing) {
-								value *= 1000.0f;
+							if (targetUnit->crashing) {
+								targetPriority *= 1000.0f;
 							}
 						}
-						targets.insert(std::pair<float, CUnit*>(value, unit));
+
+						targets.insert(std::pair<float, CUnit*>(targetPriority, targetUnit));
 					}
 				}
 			}
 		}
 	}
-/*
+
 #ifdef TRACE_SYNC
-	tracefile << "TargetList: " << attacker->id << " " << radius << " ";
-	std::map<float,CUnit*>::iterator ti;
-	for(ti=targets.begin();ti!=targets.end();++ti)
-		tracefile << (ti->first) <<  " " << (ti->second)->id <<  " ";
-	tracefile << "\n";
+	{
+		tracefile << "[GenerateWeaponTargets] attackerID, attackRadius: " << attacker->id << ", " << radius << " ";
+
+		for (std::multimap<float, CUnit*>::const_iterator ti = targets.begin(); ti != targets.end(); ++ti)
+			tracefile << "\tpriority: " << (ti->first) <<  ", targetID: " << (ti->second)->id <<  " ";
+
+		tracefile << "\n";
+	}
 #endif
-*/
 }
 
 CUnit* CGameHelper::GetClosestUnit(const float3 &pos, float searchRadius)
@@ -992,87 +796,11 @@ void CGameHelper::GetEnemyUnitsNoLosTest(const float3 &pos, float searchRadius,
 	QueryUnits(Filter::Enemy(searchAllyteam), q);
 }
 
+
 //////////////////////////////////////////////////////////////////////
 // Miscellaneous (i.e. not yet categorized)
 //////////////////////////////////////////////////////////////////////
 
-// called by {CFlameThrower, CLaserCannon, CEmgCannon, CBeamLaser, CLightningCannon}::TryTarget()
-bool CGameHelper::LineFeatureCol(const float3& start, const float3& dir, float length)
-{
-	GML_RECMUTEX_LOCK(quad); // GuiTraceRayFeature
-
-	std::vector<int> quads = qf->GetQuadsOnRay(start, dir, length);
-	std::vector<int>::iterator qi;
-
-	CollisionQuery cq;
-
-	for (qi = quads.begin(); qi != quads.end(); ++qi) {
-		const CQuadField::Quad& quad = qf->GetQuad(*qi);
-
-		for (std::list<CFeature*>::const_iterator ui = quad.features.begin(); ui != quad.features.end(); ++ui) {
-			CFeature* f = *ui;
-			CollisionVolume* cv = f->collisionVolume;
-
-			if (!f->blocking || !cv) {
-				// NOTE: why check the blocking property?
-				continue;
-			}
-
-			if (CCollisionHandler::Intersect(f, start, start + dir * length, &cq)) {
-				return true;
-			}
-		}
-	}
-	return false;
-}
-
-
-float CGameHelper::GuiTraceRayFeature(const float3& start, const float3& dir, float length, const CFeature*& feature)
-{
-	float nearHit = length;
-
-	GML_RECMUTEX_LOCK(quad); // GuiTraceRayFeature
-
-	std::vector<int> quads = qf->GetQuadsOnRay(start, dir, length);
-	std::vector<int>::iterator qi;
-
-	for (qi = quads.begin(); qi != quads.end(); ++qi) {
-		const CQuadField::Quad& quad = qf->GetQuad(*qi);
-		std::list<CFeature*>::const_iterator ui;
-
-		// NOTE: switch this to custom volumes fully?
-		// (not used for any LOF checks, maybe wasteful)
-		for (ui = quad.features.begin(); ui != quad.features.end(); ++ui) {
-			CFeature* f = *ui;
-
-			if (!gu->spectatingFullView && !f->IsInLosForAllyTeam(gu->myAllyTeam)) {
-				continue;
-			}
-			if (f->noSelect) {
-				continue;
-			}
-
-			CollisionVolume* cv = f->collisionVolume;
-			const float3& midPosOffset = cv? cv->GetOffsets(): ZeroVector;
-			const float3 dif = (f->midPos + midPosOffset) - start;
-			float closeLength = dif.dot(dir);
-
-			if (closeLength < 0)
-				continue;
-			if (closeLength > nearHit)
-				continue;
-
-			float3 closeVect = dif - dir * closeLength;
-			if (closeVect.SqLength() < f->sqRadius) {
-				nearHit = closeLength;
-				feature = f;
-			}
-		}
-	}
-
-	return nearHit;
-}
-
 float3 CGameHelper::GetUnitErrorPos(const CUnit* unit, int allyteam)
 {
 	float3 pos = unit->midPos;
@@ -1110,12 +838,7 @@ void CGameHelper::BuggerOff(float3 pos, float radius, bool spherical, bool force
 }
 
 
-float3 CGameHelper::Pos2BuildPos(const float3& pos, const UnitDef* ud)
-{
-	return Pos2BuildPos(BuildInfo(ud,pos,0));
-}
-
-float3 CGameHelper::Pos2BuildPos(const BuildInfo& buildInfo)
+float3 CGameHelper::Pos2BuildPos(const BuildInfo& buildInfo, bool synced)
 {
 	float3 pos;
 	if (buildInfo.GetXSize() & 2)
@@ -1128,7 +851,7 @@ float3 CGameHelper::Pos2BuildPos(const BuildInfo& buildInfo)
 	else
 		pos.z = floor((buildInfo.pos.z + 8) / (SQUARE_SIZE * 2)) * SQUARE_SIZE * 2;
 
-	pos.y = uh->GetBuildHeight(pos,buildInfo.def);
+	pos.y = uh->GetBuildHeight(pos, buildInfo.def, synced);
 	if (buildInfo.def->floater && pos.y < 0)
 		pos.y = -buildInfo.def->waterline;
 
@@ -1151,17 +874,17 @@ static const vector<SearchOffset>& GetSearchOffsetTable (int radius)
 	if (size > searchOffsets.size()) {
 		searchOffsets.resize (size);
 
-		for (int y=0;y<radius*2;y++)
-			for (int x=0;x<radius*2;x++)
+		for (int y = 0; y < radius*2; y++)
+			for (int x = 0; x < radius*2; x++)
 			{
-				SearchOffset& i = searchOffsets[y*radius*2+x];
+				SearchOffset& i = searchOffsets[y*radius*2 + x];
 
-				i.dx = x-radius;
-				i.dy = y-radius;
-				i.qdist = i.dx*i.dx+i.dy*i.dy;
+				i.dx = x - radius;
+				i.dy = y - radius;
+				i.qdist = i.dx*i.dx + i.dy*i.dy;
 			}
 
-		std::sort (searchOffsets.begin(), searchOffsets.end(), SearchOffsetComparator);
+		std::sort(searchOffsets.begin(), searchOffsets.end(), SearchOffsetComparator);
 	}
 
 	return searchOffsets;
@@ -1185,9 +908,9 @@ float3 CGameHelper::ClosestBuildSite(int team, const UnitDef* unitDef, float3 po
 		const float z = pos.z + ofs[so].dy * SQUARE_SIZE * 2;
 
 		BuildInfo bi(unitDef, float3(x, 0.0f, z), facing);
-		bi.pos = Pos2BuildPos(bi);
+		bi.pos = Pos2BuildPos(bi, false);
 
-		if (uh->TestUnitBuildSquare(bi, feature, allyTeam) && (!feature || feature->allyteam != allyTeam)) {
+		if (uh->TestUnitBuildSquare(bi, feature, allyTeam, false) && (!feature || feature->allyteam != allyTeam)) {
 			const int xs = (int) (x / SQUARE_SIZE);
 			const int zs = (int) (z / SQUARE_SIZE);
 			const int xsize = bi.GetXSize();
@@ -1240,7 +963,7 @@ float3 CGameHelper::ClosestBuildSite(int team, const UnitDef* unitDef, float3 po
 	return float3(-1.0f, 0.0f, 0.0f);
 }
 
-void CGameHelper::Update(void)
+void CGameHelper::Update()
 {
 	std::list<WaitingDamage*>* wd = &waitingDamages[gs->frameNum&127];
 	while (!wd->empty()) {
@@ -1251,173 +974,3 @@ void CGameHelper::Update(void)
 		delete w;
 	}
 }
-
-
-
-
-/** @return true if there is an allied unit within
-    the firing cone of <owner> (that might be hit) */
-bool CGameHelper::TestAllyCone(const float3& from, const float3& weaponDir, float length, float spread, int allyteam, CUnit* owner)
-{
-	int quads[1000];
-	int* endQuad = quads;
-	qf->GetQuadsOnRay(from, weaponDir, length, endQuad);
-
-	for (int* qi = quads; qi != endQuad; ++qi) {
-		const CQuadField::Quad& quad = qf->GetQuad(*qi);
-		for (std::list<CUnit*>::const_iterator ui = quad.teamUnits[allyteam].begin(); ui != quad.teamUnits[allyteam].end(); ++ui) {
-			CUnit* u = *ui;
-
-			if (u == owner)
-				continue;
-
-			if (TestConeHelper(from, weaponDir, length, spread, u))
-				return true;
-		}
-	}
-	return false;
-}
-
-/** same as TestAllyCone, but looks for neutral units */
-bool CGameHelper::TestNeutralCone(const float3& from, const float3& weaponDir, float length, float spread, CUnit* owner)
-{
-	int quads[1000];
-	int* endQuad = quads;
-	qf->GetQuadsOnRay(from, weaponDir, length, endQuad);
-
-	for (int* qi = quads; qi != endQuad; ++qi) {
-		const CQuadField::Quad& quad = qf->GetQuad(*qi);
-
-		for (std::list<CUnit*>::const_iterator ui = quad.units.begin(); ui != quad.units.end(); ++ui) {
-			CUnit* u = *ui;
-
-			if (u == owner)
-				continue;
-
-			if (u->IsNeutral()) {
-				if (TestConeHelper(from, weaponDir, length, spread, u))
-					return true;
-			}
-		}
-	}
-	return false;
-}
-
-
-/** helper for TestAllyCone and TestNeutralCone
-    @return true if the unit u is in the firing cone, false otherwise */
-bool CGameHelper::TestConeHelper(const float3& from, const float3& weaponDir, float length, float spread, const CUnit* u)
-{
-	// account for any offset, since we want to know if our shots might hit
-	float3 unitDir = (u->midPos + u->collisionVolume->GetOffsets()) - from;
-	// weaponDir defines the center of the cone
-	float closeLength = unitDir.dot(weaponDir);
-
-	if (closeLength <= 0)
-		return false;
-	if (closeLength > length)
-		closeLength = length;
-
-	float3 closeVect = unitDir - weaponDir * closeLength;
-	// NOTE: same caveat wrt. use of volumeBoundingRadius
-	// as for ::Explosion(), this will result in somewhat
-	// over-conservative tests for non-spherical volumes
-	float r = u->collisionVolume->GetBoundingRadius() + spread * closeLength + 1;
-
-	return (closeVect.SqLength() < r * r);
-}
-
-
-
-
-/** @return true if there is an allied unit within
-    the firing trajectory of <owner> (that might be hit) */
-bool CGameHelper::TestTrajectoryAllyCone(const float3& from, const float3& flatdir, float length, float linear, float quadratic, float spread, float baseSize, int allyteam, CUnit* owner)
-{
-	int quads[1000];
-	int* endQuad = quads;
-	qf->GetQuadsOnRay(from, flatdir, length, endQuad);
-
-	for (int* qi = quads; qi != endQuad; ++qi) {
-		const CQuadField::Quad& quad = qf->GetQuad(*qi);
-		for (std::list<CUnit*>::const_iterator ui = quad.teamUnits[allyteam].begin(); ui != quad.teamUnits[allyteam].end(); ++ui) {
-			CUnit* u = *ui;
-
-			if (u == owner)
-				continue;
-
-			if (TestTrajectoryConeHelper(from, flatdir, length, linear, quadratic, spread, baseSize, u))
-				return true;
-		}
-	}
-	return false;
-}
-
-/** same as TestTrajectoryAllyCone, but looks for neutral units */
-bool CGameHelper::TestTrajectoryNeutralCone(const float3& from, const float3& flatdir, float length, float linear, float quadratic, float spread, float baseSize, CUnit* owner)
-{
-	int quads[1000];
-	int* endQuad = quads;
-	qf->GetQuadsOnRay(from, flatdir, length, endQuad);
-
-	for (int* qi = quads; qi != endQuad; ++qi) {
-		const CQuadField::Quad& quad = qf->GetQuad(*qi);
-		for (std::list<CUnit*>::const_iterator ui = quad.units.begin(); ui != quad.units.end(); ++ui) {
-			CUnit* u = *ui;
-
-			if (u == owner)
-				continue;
-
-			if (u->IsNeutral()) {
-				if (TestTrajectoryConeHelper(from, flatdir, length, linear, quadratic, spread, baseSize, u))
-					return true;
-			}
-		}
-	}
-	return false;
-}
-
-
-/** helper for TestTrajectoryAllyCone and TestTrajectoryNeutralCone
-    @return true if the unit u is in the firing trajectory, false otherwise */
-bool CGameHelper::TestTrajectoryConeHelper(const float3& from, const float3& flatdir, float length, float linear, float quadratic, float spread, float baseSize, const CUnit* u)
-{
-	const CollisionVolume* cv = u->collisionVolume;
-	float3 dif = (u->midPos + cv->GetOffsets()) - from;
-	float3 flatdif(dif.x, 0, dif.z);
-	float closeFlatLength = flatdif.dot(flatdir);
-
-	if (closeFlatLength <= 0)
-		return false;
-	if (closeFlatLength > length)
-		closeFlatLength = length;
-
-	if (fabs(linear - quadratic * closeFlatLength) < 0.15f) {
-		// relatively flat region -> use approximation
-		dif.y -= (linear + quadratic * closeFlatLength) * closeFlatLength;
-
-		// NOTE: overly conservative for non-spherical volumes
-		float3 closeVect = dif - flatdir * closeFlatLength;
-		float r = cv->GetBoundingRadius() + spread * closeFlatLength + baseSize;
-		if (closeVect.SqLength() < r * r) {
-			return true;
-		}
-	} else {
-		float3 newfrom = from + flatdir * closeFlatLength;
-		newfrom.y += (linear + quadratic * closeFlatLength) * closeFlatLength;
-		float3 dir = flatdir;
-		dir.y = linear + quadratic * closeFlatLength;
-		dir.Normalize();
-
-		dif = (u->midPos + cv->GetOffsets()) - newfrom;
-		float closeLength = dif.dot(dir);
-
-		// NOTE: overly conservative for non-spherical volumes
-		float3 closeVect = dif - dir * closeLength;
-		float r = cv->GetBoundingRadius() + spread * closeFlatLength + baseSize;
-		if (closeVect.SqLength() < r * r) {
-			return true;
-		}
-	}
-	return false;
-}
