@@ -1,9 +1,8 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#ifndef __GAME_SERVER_H__
-#define __GAME_SERVER_H__
+#ifndef _GAME_SERVER_H
+#define _GAME_SERVER_H
 
-#include <boost/thread/recursive_mutex.hpp>
 #include <boost/thread/thread.hpp>
 #include <boost/scoped_ptr.hpp>
 #include <string>
@@ -15,9 +14,11 @@
 
 #include "GameData.h"
 #include "Sim/Misc/TeamBase.h"
-#include "UnsyncedRNG.h"
-#include "float3.h"
+#include "System/UnsyncedRNG.h"
+#include "System/float3.h"
 #include "System/myTime.h"
+#include "System/Platform/Synchro.h"
+
 
 namespace netcode
 {
@@ -40,15 +41,15 @@ class GameSkirmishAI;
  * this value is used as the sending player-number.
  */
 const unsigned SERVER_PLAYER = 255;
-const unsigned numCommands = 19;
+const unsigned numCommands = 20;
 extern const std::string commands[numCommands];
 
 class GameTeam : public TeamBase
 {
 public:
-	GameTeam() : active(false) {};
+	GameTeam() : active(false) {}
 	bool active;
-	GameTeam& operator=(const TeamBase& base) { TeamBase::operator=(base); return *this; };
+	GameTeam& operator=(const TeamBase& base) { TeamBase::operator=(base); return *this; }
 };
 
 /**
@@ -59,7 +60,7 @@ public:
  */
 class CGameServer
 {
-	friend class CCregLoadSaveHandler;     //For initialize server state after load
+	friend class CCregLoadSaveHandler; // For initializing server state after load
 public:
 	CGameServer(const std::string& hostIP, int hostPort, const GameData* const gameData, const CGameSetup* const setup);
 	~CGameServer();
@@ -72,7 +73,7 @@ public:
 	 * @brief Set frame after loading
 	 * WARNING! No checks are done, so be carefull
 	 */
-	void PostLoad(unsigned lastTick, int serverframenum);
+	void PostLoad(unsigned lastTick, int serverFrameNum);
 
 	void CreateNewFrame(bool fromServerThread, bool fixedFrameTime);
 
@@ -80,11 +81,17 @@ public:
 
 	void SetGamePausable(const bool arg);
 
-	bool HasDemo() const { return (demoReader != NULL); }
+	bool HasStarted() const { return gameHasStarted; }
+	bool HasGameID() const { return generatedGameID; }
 	/// Is the server still running?
 	bool HasFinished() const;
 
 	void UpdateSpeedControl(int speedCtrl);
+	static std::string SpeedControlToString(int speedCtrl);
+
+	#ifdef DEDICATED
+	const boost::scoped_ptr<CDemoRecorder>& GetDemoRecorder() const { return demoRecorder; }
+	#endif
 
 private:
 	/**
@@ -100,23 +107,23 @@ private:
 	 */
 	void KickPlayer(const int playerNum);
 
-	unsigned BindConnection(std::string name, const std::string& passwd, const std::string& version, bool isLocal, boost::shared_ptr<netcode::CConnection> link, bool reconnect = false);
+	unsigned BindConnection(std::string name, const std::string& passwd, const std::string& version, bool isLocal, boost::shared_ptr<netcode::CConnection> link, bool reconnect = false, int netloss = 0);
 
 	void CheckForGameStart(bool forced=false);
 	void StartGame();
 	void UpdateLoop();
 	void Update();
-	void ProcessPacket(const unsigned playernum, boost::shared_ptr<const netcode::RawPacket> packet);
+	void ProcessPacket(const unsigned playerNum, boost::shared_ptr<const netcode::RawPacket> packet);
 	void CheckSync();
 	void ServerReadNet();
 	void CheckForGameEnd();
 
-	/** @brief Generate a unique game identifier and sent it to all clients. */
+	/** @brief Generate a unique game identifier and send it to all clients. */
 	void GenerateAndSendGameID();
 	std::string GetPlayerNames(const std::vector<int>& indices) const;
 
 	/// read data from demo and send it to clients
-	void SendDemoData(const bool skipping=false);
+	bool SendDemoData(int targetFrameNum);
 
 	void Broadcast(boost::shared_ptr<const netcode::RawPacket> packet);
 
@@ -124,27 +131,30 @@ private:
 	 * @brief skip frames
 	 *
 	 * If you are watching a demo, this will push out all data until
-	 * targetframe to all clients
+	 * targetFrame to all clients
 	 */
-	void SkipTo(int targetframe);
+	void SkipTo(int targetFrameNum);
 
-	void Message(const std::string& message, bool broadcast=true);
-	void PrivateMessage(int playernum, const std::string& message);
+	void Message(const std::string& message, bool broadcast = true);
+	void PrivateMessage(int playerNum, const std::string& message);
 
-	void AddToPacketCache(boost::shared_ptr<const netcode::RawPacket> &pckt);
+	void AddToPacketCache(boost::shared_ptr<const netcode::RawPacket>& pckt);
 
-	float GetDemoTime();
+	bool AdjustPlayerNumber(netcode::RawPacket* buf, int pos, int val = -1);
+	void UpdatePlayerNumberMap();
+
+	float GetDemoTime() const;
 
 	/////////////////// game status variables ///////////////////
 
+	unsigned char playerNumberMap[256];
 	volatile bool quitServer;
-	int serverframenum;
+	int serverFrameNum;
 
 	spring_time serverStartTime;
 	spring_time readyTime;
 	spring_time gameStartTime;
 	spring_time gameEndTime;	///< Tick when game end was detected
-	bool sentGameOverMsg;
 	spring_time lastTick;
 	float timeLeft;
 	spring_time lastPlayerInfo;
@@ -158,18 +168,28 @@ private:
 	float internalSpeed;
 	bool cheating;
 
+	unsigned char ReserveNextAvailableSkirmishAIId();
+
+	std::map<unsigned char, GameSkirmishAI> ais;
+	std::list<unsigned char> usedSkirmishAIIds;
+	void FreeSkirmishAIId(const unsigned char skirmishAIId);
+
 	std::vector<GameParticipant> players;
-	size_t ReserveNextAvailableSkirmishAIId();
-	
-	std::map<size_t, GameSkirmishAI> ais;
-	std::list<size_t> usedSkirmishAIIds;
-	void FreeSkirmishAIId(const size_t skirmishAIId);
-	
 	std::vector<GameTeam> teams;
+	std::vector<unsigned char> winningAllyTeams;
 
 	float medianCpu;
 	int medianPing;
 	int curSpeedCtrl;
+
+	/**
+	 * throttles speed based on:
+	 * 0 : players (max cpu)
+	 * 1 : players (median cpu)
+	 * 2 : (same as 0)
+	 * -x: same as x, but ignores votes from players that may change
+	 *     the speed-control mode
+	 */
 	int speedControl;
 	/////////////////// game settings ///////////////////
 	boost::scoped_ptr<const CGameSetup> setup;
@@ -191,7 +211,7 @@ private:
 
 	/////////////////// sync stuff ///////////////////
 #ifdef SYNCCHECK
-	std::deque<int> outstandingSyncFrames;
+	std::set<int> outstandingSyncFrames;
 #endif
 	int syncErrorFrame;
 	int syncWarningFrame;
@@ -200,7 +220,7 @@ private:
 	void InternalSpeedChange(float newSpeed);
 	void UserSpeedChange(float newSpeed, int player);
 
-	void AddAdditionalUser( const std::string& name, const std::string& passwd );
+	void AddAdditionalUser( const std::string& name, const std::string& passwd, bool fromDemo = false );
 
 	bool hasLocalClient;
 	unsigned localClientNumber;
@@ -216,13 +236,18 @@ private:
 	boost::scoped_ptr<AutohostInterface> hostif;
 	UnsyncedRNG rng;
 	boost::thread* thread;
-	mutable boost::recursive_mutex gameServerMutex;
+
+	mutable Threading::RecursiveMutex gameServerMutex;
 
 	bool canReconnect;
-	bool gameHasStarted;
+	volatile bool gameHasStarted;
+	volatile bool generatedGameID;
+
+	spring_time lastBandwidthUpdate;
+	int linkMinPacketSize;
 };
 
 extern CGameServer* gameServer;
 
-#endif // __GAME_SERVER_H__
+#endif // _GAME_SERVER_H
 
