@@ -1,56 +1,60 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include <map>
-#include <SDL_keysym.h>
-
-#include "mmgr.h"
-
-#include "Sim/Misc/TeamHandler.h"
-#include "Sim/Misc/GlobalSynced.h"
 #include "SelectedUnits.h"
+
+#include "SelectedUnitsAI.h"
+#include "Camera.h"
+#include "GlobalUnsynced.h"
 #include "WaitCommandsAI.h"
-#include "Rendering/GL/myGL.h"
-#include "NetProtocol.h"
-#include "Net/PackPacket.h"
-#include "Sim/Units/Groups/GroupHandler.h"
-#include "Sim/Units/Groups/Group.h"
-#include "ExternalAI/EngineOutHandler.h"
+#include "Player.h"
+#include "PlayerHandler.h"
 #include "UI/CommandColors.h"
 #include "UI/GuiHandler.h"
 #include "UI/TooltipConsole.h"
-#include "LogOutput.h"
+#include "ExternalAI/EngineOutHandler.h"
+#include "ExternalAI/SkirmishAIHandler.h"
+#include "Rendering/CommandDrawer.h"
+#include "Rendering/LineDrawer.h"
+#include "Rendering/GL/myGL.h"
 #include "Rendering/GL/VertexArray.h"
-#include "SelectedUnitsAI.h"
+#include "Sim/Misc/TeamHandler.h"
+#include "Sim/Misc/GlobalSynced.h"
+#include "Sim/MoveTypes/MoveInfo.h"
 #include "Sim/Features/Feature.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitDef.h"
 #include "Sim/Units/UnitHandler.h"
 #include "Sim/Units/CommandAI/BuilderCAI.h"
 #include "Sim/Units/CommandAI/CommandAI.h"
-#include "Sim/Units/CommandAI/LineDrawer.h"
+#include "Sim/Units/Groups/GroupHandler.h"
+#include "Sim/Units/Groups/Group.h"
 #include "Sim/Units/UnitTypes/TransportUnit.h"
-#include "EventHandler.h"
-#include "ConfigHandler.h"
-#include "PlayerHandler.h"
-#include "Camera.h"
-#include "Sound/IEffectChannel.h"
-#include "Util.h"
+#include "System/Config/ConfigHandler.h"
+#include "System/EventHandler.h"
+#include "System/Log/ILog.h"
+#include "System/Util.h"
+#include "System/mmgr.h"
+#include "System/NetProtocol.h"
+#include "System/Net/PackPacket.h"
+#include "System/Input/KeyInput.h"
+#include "System/Sound/SoundChannels.h"
 
-extern boost::uint8_t *keys;
+#include <SDL_keysym.h>
+#include <map>
 
 
-//////////////////////////////////////////////////////////////////////
-// Construction/Destruction
-//////////////////////////////////////////////////////////////////////
+#define PLAY_SOUNDS 1
+
+CONFIG(bool, BuildIconsFirst).defaultValue(false);
+
 CSelectedUnits selectedUnits;
 
 
 CSelectedUnits::CSelectedUnits()
-: selectionChanged(false),
-	possibleCommandsChanged(true),
-	buildIconsFirst(false),
-	selectedGroup(-1)
+	: selectionChanged(false)
+	, possibleCommandsChanged(true)
+	, buildIconsFirst(false)
+	, selectedGroup(-1)
 {
 }
 
@@ -62,7 +66,7 @@ CSelectedUnits::~CSelectedUnits()
 
 void CSelectedUnits::Init(unsigned numPlayers)
 {
-	buildIconsFirst = !!configHandler->Get("BuildIconsFirst", 0);
+	buildIconsFirst = configHandler->GetBool("BuildIconsFirst");
 	netSelected.resize(numPlayers);
 }
 
@@ -85,9 +89,9 @@ CSelectedUnits::AvailableCommandsStruct CSelectedUnits::GetAvailableCommands()
 	int foundGroup2 = -2;
 	map<int, int> states;
 
-	for (CUnitSet::iterator ui = selectedUnits.begin(); ui != selectedUnits.end(); ++ui) {
-		vector<CommandDescription>* c = &(*ui)->commandAI->GetPossibleCommands();
-		vector<CommandDescription>::iterator ci;
+	for (CUnitSet::const_iterator ui = selectedUnits.begin(); ui != selectedUnits.end(); ++ui) {
+		const std::vector<CommandDescription>* c = &((*ui)->commandAI->GetPossibleCommands());
+		std::vector<CommandDescription>::const_iterator ci;
 		for (ci = c->begin(); ci != c->end(); ++ci) {
 			states[ci->id] = ci->disabled ? 2 : 1;
 		}
@@ -110,13 +114,13 @@ CSelectedUnits::AvailableCommandsStruct CSelectedUnits::GetAvailableCommands()
 		}
 	}
 
-	vector<CommandDescription> groupCommands;
+	std::vector<CommandDescription> groupCommands;
 
-	vector<CommandDescription> commands ;
-	// load the first set  (separating build and non-build commands)
-	for (CUnitSet::iterator ui = selectedUnits.begin(); ui != selectedUnits.end(); ++ui) {
-		vector<CommandDescription>* c = &(*ui)->commandAI->GetPossibleCommands();
-		vector<CommandDescription>::iterator ci;
+	std::vector<CommandDescription> commands ;
+	// load the first set (separating build and non-build commands)
+	for (CUnitSet::const_iterator ui = selectedUnits.begin(); ui != selectedUnits.end(); ++ui) {
+		const std::vector<CommandDescription>* c = &((*ui)->commandAI->GetPossibleCommands());
+		std::vector<CommandDescription>::const_iterator ci;
 		for (ci = c->begin(); ci != c->end(); ++ci) {
 			if (buildIconsFirst) {
 				if (ci->id >= 0) { continue; }
@@ -134,16 +138,16 @@ CSelectedUnits::AvailableCommandsStruct CSelectedUnits::GetAvailableCommands()
 	}
 
 	if (!buildIconsFirst && !gs->noHelperAIs) {
-		vector<CommandDescription>::iterator ci;
+		std::vector<CommandDescription>::const_iterator ci;
 		for(ci = groupCommands.begin(); ci != groupCommands.end(); ++ci) {
 			commands.push_back(*ci);
 		}
 	}
 
-	// load the second set  (all those that have not already been included)
-	for (CUnitSet::iterator ui = selectedUnits.begin(); ui != selectedUnits.end(); ++ui) {
-		vector<CommandDescription>* c = &(*ui)->commandAI->GetPossibleCommands();
-		vector<CommandDescription>::iterator ci;
+	// load the second set (all those that have not already been included)
+	for (CUnitSet::const_iterator ui = selectedUnits.begin(); ui != selectedUnits.end(); ++ui) {
+		std::vector<CommandDescription>* c = &(*ui)->commandAI->GetPossibleCommands();
+		std::vector<CommandDescription>::const_iterator ci;
 		for (ci = c->begin(); ci != c->end(); ++ci) {
 			if (buildIconsFirst) {
 				if (ci->id < 0)  { continue; }
@@ -160,7 +164,7 @@ CSelectedUnits::AvailableCommandsStruct CSelectedUnits::GetAvailableCommands()
 		}
 	}
 	if (buildIconsFirst && !gs->noHelperAIs) {
-		vector<CommandDescription>::iterator ci;
+		std::vector<CommandDescription>::const_iterator ci;
 		for (ci = groupCommands.begin(); ci != groupCommands.end(); ++ci) {
 			commands.push_back(*ci);
 		}
@@ -177,95 +181,85 @@ void CSelectedUnits::GiveCommand(Command c, bool fromUser)
 {
 	GML_RECMUTEX_LOCK(grpsel); // GiveCommand
 
-//	logOutput.Print("Command given %i",c.id);
+//	LOG_L(L_DEBUG, "Command given %i", c.id);
 	if ((gu->spectating && !gs->godMode) || selectedUnits.empty()) {
 		return;
 	}
 
-	if (fromUser) {		//add some statistics
+	const int& cmd_id = c.GetID();
+
+	if (fromUser) { // add some statistics
 		playerHandler->Player(gu->myPlayerNum)->currentStats.numCommands++;
-		if (selectedGroup!=-1) {
-			playerHandler->Player(gu->myPlayerNum)->currentStats.unitCommands+=grouphandlers[gu->myTeam]->groups[selectedGroup]->units.size();
+		if (selectedGroup != -1) {
+			playerHandler->Player(gu->myPlayerNum)->currentStats.unitCommands += grouphandlers[gu->myTeam]->groups[selectedGroup]->units.size();
 		} else {
-			playerHandler->Player(gu->myPlayerNum)->currentStats.unitCommands+=selectedUnits.size();
+			playerHandler->Player(gu->myPlayerNum)->currentStats.unitCommands += selectedUnits.size();
 		}
 	}
 
-	if (c.id == CMD_GROUPCLEAR) {
-		for(CUnitSet::iterator ui=selectedUnits.begin();ui!=selectedUnits.end();++ui){
-			if((*ui)->group){
+	if (cmd_id == CMD_GROUPCLEAR) {
+		for (CUnitSet::iterator ui = selectedUnits.begin(); ui != selectedUnits.end(); ++ui) {
+			if ((*ui)->group) {
 				(*ui)->SetGroup(0);
-				possibleCommandsChanged=true;
+				possibleCommandsChanged = true;
 			}
 		}
 		return;
 	}
-	else if (c.id == CMD_GROUPSELECT) {
+	else if (cmd_id == CMD_GROUPSELECT) {
 		SelectGroup((*selectedUnits.begin())->group->id);
 		return;
 	}
-	else if (c.id == CMD_GROUPADD) {
-		CGroup* group=0;
-		for(CUnitSet::iterator ui=selectedUnits.begin();ui!=selectedUnits.end();++ui){
-			if((*ui)->group){
-				group=(*ui)->group;
-				possibleCommandsChanged=true;
+	else if (cmd_id == CMD_GROUPADD) {
+		CGroup* group = NULL;
+		for (CUnitSet::iterator ui = selectedUnits.begin(); ui != selectedUnits.end(); ++ui) {
+			if ((*ui)->group) {
+				group = (*ui)->group;
+				possibleCommandsChanged = true;
 				break;
 			}
 		}
-		if(group){
-			for(CUnitSet::iterator ui=selectedUnits.begin();ui!=selectedUnits.end();++ui){
-				if(!(*ui)->group)
+		if (group) {
+			for (CUnitSet::iterator ui = selectedUnits.begin(); ui != selectedUnits.end(); ++ui) {
+				if (!(*ui)->group) {
 					(*ui)->SetGroup(group);
+				}
 			}
 			SelectGroup(group->id);
 		}
 		return;
 	}
-	else if (c.id == CMD_AISELECT) {
-		if (gs->noHelperAIs) {
-			logOutput.Print("LuaUI control is disabled");
-			return;
-		}
-		if(c.params[0]!=0){
-			CGroup* group=grouphandlers[gu->myTeam]->CreateNewGroup();
-
-			for(CUnitSet::iterator ui=selectedUnits.begin();ui!=selectedUnits.end();++ui){
-				(*ui)->SetGroup(group);
-			}
-			SelectGroup(group->id);
-		}
-		return;
-	}
-	else if (c.id == CMD_TIMEWAIT) {
+	else if (cmd_id == CMD_TIMEWAIT) {
 		waitCommandsAI.AddTimeWait(c);
 		return;
 	}
-	else if (c.id == CMD_DEATHWAIT) {
+	else if (cmd_id == CMD_DEATHWAIT) {
 		waitCommandsAI.AddDeathWait(c);
 		return;
 	}
-	else if (c.id == CMD_SQUADWAIT) {
+	else if (cmd_id == CMD_SQUADWAIT) {
 		waitCommandsAI.AddSquadWait(c);
 		return;
 	}
-	else if (c.id == CMD_GATHERWAIT) {
+	else if (cmd_id == CMD_GATHERWAIT) {
 		waitCommandsAI.AddGatherWait(c);
 		return;
 	}
 
 	SendCommand(c);
 
+	#if (PLAY_SOUNDS == 1)
 	if (!selectedUnits.empty()) {
-		CUnitSet::iterator ui = selectedUnits.begin();
+		CUnitSet::const_iterator ui = selectedUnits.begin();
 
-		int soundIdx = (*ui)->unitDef->sounds.ok.getRandomIdx();
+		const int soundIdx = (*ui)->unitDef->sounds.ok.getRandomIdx();
 		if (soundIdx >= 0) {
 			Channels::UnitReply.PlaySample(
 				(*ui)->unitDef->sounds.ok.getID(soundIdx), (*ui),
 				(*ui)->unitDef->sounds.ok.getVolume(soundIdx));
 		}
 	}
+	#endif
 }
 
 
@@ -273,7 +267,7 @@ void CSelectedUnits::AddUnit(CUnit* unit)
 {
 	// if unit is being transported by eg. Hulk or Atlas
 	// then we should not be able to select it
-	CTransportUnit *trans=unit->GetTransporter();
+	const CTransportUnit* trans = unit->GetTransporter();
 	if (trans != NULL && !trans->unitDef->isFirePlatform) {
 		return;
 	}
@@ -284,13 +278,14 @@ void CSelectedUnits::AddUnit(CUnit* unit)
 
 	GML_RECMUTEX_LOCK(sel); // AddUnit
 
-	selectedUnits.insert(unit);
-	AddDeathDependence(unit);
+	if (selectedUnits.insert(unit).second)
+		AddDeathDependence(unit, DEPENDENCE_SELECTED);
 	selectionChanged = true;
 	possibleCommandsChanged = true;
 
-	if (!(unit->group) || unit->group->id != selectedGroup)
+	if (!(unit->group) || unit->group->id != selectedGroup) {
 		selectedGroup = -1;
+	}
 
 	unit->commandAI->selected = true;
 }
@@ -300,12 +295,12 @@ void CSelectedUnits::RemoveUnit(CUnit* unit)
 {
 	GML_RECMUTEX_LOCK(sel); // RemoveUnit
 
-	selectedUnits.erase(unit);
-	DeleteDeathDependence(unit);
-	selectionChanged=true;
-	possibleCommandsChanged=true;
-	selectedGroup=-1;
-	unit->commandAI->selected=false;
+	if (selectedUnits.erase(unit))
+		DeleteDeathDependence(unit, DEPENDENCE_SELECTED);
+	selectionChanged = true;
+	possibleCommandsChanged = true;
+	selectedGroup = -1;
+	unit->commandAI->selected = false;
 }
 
 
@@ -314,15 +309,15 @@ void CSelectedUnits::ClearSelected()
 	GML_RECMUTEX_LOCK(sel); // ClearSelected
 
 	CUnitSet::iterator ui;
-	for(ui=selectedUnits.begin();ui!=selectedUnits.end();++ui){
-		(*ui)->commandAI->selected=false;
-		DeleteDeathDependence(*ui);
+	for (ui = selectedUnits.begin(); ui != selectedUnits.end(); ++ui) {
+		(*ui)->commandAI->selected = false;
+		DeleteDeathDependence(*ui, DEPENDENCE_SELECTED);
 	}
 
 	selectedUnits.clear();
-	selectionChanged=true;
-	possibleCommandsChanged=true;
-	selectedGroup=-1;
+	selectionChanged = true;
+	possibleCommandsChanged = true;
+	selectedGroup = -1;
 }
 
 
@@ -335,16 +330,16 @@ void CSelectedUnits::SelectGroup(int num)
 	CGroup* group=grouphandlers[gu->myTeam]->groups[num];
 
 	CUnitSet::iterator ui;
-	for(ui=group->units.begin();ui!=group->units.end();++ui){
+	for (ui = group->units.begin(); ui != group->units.end(); ++ui) {
 		if (!(*ui)->noSelect) {
-			(*ui)->commandAI->selected=true;
+			(*ui)->commandAI->selected = true;
 			selectedUnits.insert(*ui);
-			AddDeathDependence(*ui);
+			AddDeathDependence(*ui, DEPENDENCE_SELECTED);
 		}
 	}
 
-	selectionChanged=true;
-	possibleCommandsChanged=true;
+	selectionChanged = true;
+	possibleCommandsChanged = true;
 }
 
 
@@ -360,9 +355,7 @@ void CSelectedUnits::Draw()
 
 	GML_RECMUTEX_LOCK(grpsel); // Draw
 
-	if (cmdColors.unitBox[3] > 0.0f) {
-		glColor4fv(cmdColors.unitBox);
-
+	if (cmdColors.unitBox[3] > 0.05f) {
 		const CUnitSet* unitSet;
 		if (selectedGroup != -1) {
 			unitSet = &grouphandlers[gu->myTeam]->groups[selectedGroup]->units;
@@ -372,27 +365,58 @@ void CSelectedUnits::Draw()
 
 		CVertexArray* va = GetVertexArray();
 		va->Initialize();
-		va->EnlargeArrays(unitSet->size()*4, 0, VA_SIZE_0);
-		CUnitSet::const_iterator ui;
-		for (ui = unitSet->begin(); ui != unitSet->end(); ++ui) {
+		va->EnlargeArrays(unitSet->size() * 8, 0, VA_SIZE_C);
+
+		for (CUnitSet::const_iterator ui = unitSet->begin(); ui != unitSet->end(); ++ui) {
 			const CUnit* unit = *ui;
 			if (unit->isIcon) {
 				continue;
 			}
 
-			va->AddVertexQ0(unit->drawPos.x + unit->xsize * 4, unit->drawPos.y, unit->drawPos.z + unit->zsize * 4);
-			va->AddVertexQ0(unit->drawPos.x - unit->xsize * 4, unit->drawPos.y, unit->drawPos.z + unit->zsize * 4);
-			va->AddVertexQ0(unit->drawPos.x - unit->xsize * 4, unit->drawPos.y, unit->drawPos.z - unit->zsize * 4);
-			va->AddVertexQ0(unit->drawPos.x + unit->xsize * 4, unit->drawPos.y, unit->drawPos.z - unit->zsize * 4);
+			const int
+				uhxsize = (unit->xsize * SQUARE_SIZE) >> 1,
+				uhzsize = (unit->zsize * SQUARE_SIZE) >> 1,
+				mhxsize = (unit->mobility == NULL)? uhxsize: ((unit->mobility->xsize * SQUARE_SIZE) >> 1),
+				mhzsize = (unit->mobility == NULL)? uhzsize: ((unit->mobility->zsize * SQUARE_SIZE) >> 1);
+			const float3 verts[8] = {
+				// UnitDef footprint corners
+				float3(unit->drawPos.x + uhxsize, unit->drawPos.y, unit->drawPos.z + uhzsize),
+				float3(unit->drawPos.x - uhxsize, unit->drawPos.y, unit->drawPos.z + uhzsize),
+				float3(unit->drawPos.x - uhxsize, unit->drawPos.y, unit->drawPos.z - uhzsize),
+				float3(unit->drawPos.x + uhxsize, unit->drawPos.y, unit->drawPos.z - uhzsize),
+				// MoveDef footprint corners
+				float3(unit->drawPos.x + mhxsize, unit->drawPos.y, unit->drawPos.z + mhzsize),
+				float3(unit->drawPos.x - mhxsize, unit->drawPos.y, unit->drawPos.z + mhzsize),
+				float3(unit->drawPos.x - mhxsize, unit->drawPos.y, unit->drawPos.z - mhzsize),
+				float3(unit->drawPos.x + mhxsize, unit->drawPos.y, unit->drawPos.z - mhzsize),
+			};
+
+			const unsigned char colors[2][4] = {
+				{(0.0f + cmdColors.unitBox[0]) * 255, (0.0f + cmdColors.unitBox[1]) * 255, (0.0f + cmdColors.unitBox[2] * 255), cmdColors.unitBox[3] * 255},
+				{(1.0f - cmdColors.unitBox[0]) * 255, (1.0f - cmdColors.unitBox[1]) * 255, (1.0f - cmdColors.unitBox[2] * 255), cmdColors.unitBox[3] * 255},
+			};
+
+			va->AddVertexQC(verts[0], colors[0]);
+			va->AddVertexQC(verts[1], colors[0]);
+			va->AddVertexQC(verts[2], colors[0]);
+			va->AddVertexQC(verts[3], colors[0]);
+
+			if (globalRendering->drawdebug && (mhxsize != uhxsize || mhzsize != uhzsize)) {
+				va->AddVertexQC(verts[4], colors[1]);
+				va->AddVertexQC(verts[5], colors[1]);
+				va->AddVertexQC(verts[6], colors[1]);
+				va->AddVertexQC(verts[7], colors[1]);
+			}
 		}
-		va->DrawArray0(GL_QUADS);
+
+		va->DrawArrayC(GL_QUADS);
 	}
 
 	// highlight queued build sites if we are about to build something
 	// (or old-style, whenever the shift key is being held down)
 	if (cmdColors.buildBox[3] > 0.0f) {
 		if (!selectedUnits.empty() &&
-				((cmdColors.BuildBoxesOnShift() && keys[SDLK_LSHIFT]) ||
+				((cmdColors.BuildBoxesOnShift() && keyInput->IsKeyPressed(SDLK_LSHIFT)) ||
 				 ((guihandler->inCommand >= 0) &&
 					(guihandler->inCommand < int(guihandler->commands.size())) &&
 					(guihandler->commands[guihandler->inCommand].id < 0)))) {
@@ -409,14 +433,14 @@ void CSelectedUnits::Draw()
 						glColor4fv(cmdColors.buildBox);
 						myColor = true;
 					}
-					builder->DrawQuedBuildingSquares();
+					commandDrawer->DrawQuedBuildingSquares(builder);
 				}
 				else if (teamHandler->AlliedTeams(builder->owner->team, gu->myTeam)) {
 					if (myColor) {
 						glColor4fv(cmdColors.allyBuildBox);
 						myColor = false;
 					}
-					builder->DrawQuedBuildingSquares();
+					commandDrawer->DrawQuedBuildingSquares(builder);
 				}
 			}
 		}
@@ -436,41 +460,41 @@ void CSelectedUnits::DependentDied(CObject *o)
 	GML_RECMUTEX_LOCK(sel); // DependentDied - maybe superfluous, too late anyway
 
 	selectedUnits.erase((CUnit*)o);
-	selectionChanged=true;
-	possibleCommandsChanged=true;
+	selectionChanged = true;
+	possibleCommandsChanged = true;
 }
 
 
-void CSelectedUnits::NetSelect(vector<int>& s,int player)
+void CSelectedUnits::NetSelect(std::vector<int>& s, int playerId)
 {
-	assert(unsigned(player) < netSelected.size());
-	netSelected[player] = s;
+	assert(unsigned(playerId) < netSelected.size());
+	netSelected[playerId] = s;
 }
 
 
-void CSelectedUnits::NetOrder(Command &c, int playerID)
+void CSelectedUnits::NetOrder(Command& c, int playerId)
 {
-	assert(unsigned(playerID) < netSelected.size());
-	selectedUnitsAI.GiveCommandNet(c, playerID);
+	assert(unsigned(playerId) < netSelected.size());
+	selectedUnitsAI.GiveCommandNet(c, playerId);
 
-	if (netSelected[playerID].size() > 0) {
-		eoh->PlayerCommandGiven(netSelected[playerID], c, playerID);
+	if (netSelected[playerId].size() > 0) {
+		eoh->PlayerCommandGiven(netSelected[playerId], c, playerId);
 	}
 }
 
-void CSelectedUnits::ClearNetSelect(int player)
+void CSelectedUnits::ClearNetSelect(int playerId)
 {
-	netSelected[player].clear();
+	netSelected[playerId].clear();
 }
 
-void CSelectedUnits::AiOrder(int unitid, const Command &c, int playerID)
+void CSelectedUnits::AiOrder(int unitid, const Command &c, int playerId)
 {
 	CUnit* unit = uh->units[unitid];
 	if (unit == NULL) {
 		return;
 	}
 
-	const CPlayer* player = playerHandler->Player(playerID);
+	const CPlayer* player = playerHandler->Player(playerId);
 	if (player == NULL) {
 		return;
 	}
@@ -479,8 +503,8 @@ void CSelectedUnits::AiOrder(int unitid, const Command &c, int playerID)
 		// between time of giving valid orders on units which then change team
 		// due to e.g. LuaRules.
 
-		//logOutput.Print("Invalid order from player %i for (unit %i %s, team %i)",
-		//                playerID, unitid, unit->unitDefName.c_str(), unit->team);
+		//LOG_L(L_WARNING, "Invalid order from player %i for (unit %i %s, team %i)",
+		//		playerId, unitid, unit->unitDefName.c_str(), unit->team);
 		return;
 	}
 
@@ -540,7 +564,7 @@ static inline bool IsBetterLeader(const UnitDef* newDef, const UnitDef* oldDef)
 		}
 	}
 	else if (targetFeature) {
-		if (!targetFeature->createdFromUnit.empty()) {
+		if (targetFeature->udef != NULL) {
 			if ( newDef->canResurrect && !oldDef->canResurrect) { return true;  }
 			if (!newDef->canResurrect &&  oldDef->canResurrect) { return false; }
 		}
@@ -558,14 +582,13 @@ static inline bool IsBetterLeader(const UnitDef* newDef, const UnitDef* oldDef)
 // LuaUnsyncedRead::GetDefaultCommand --> CGuiHandler::GetDefaultCommand --> GetDefaultCmd
 int CSelectedUnits::GetDefaultCmd(const CUnit* unit, const CFeature* feature)
 {
-	GML_RECMUTEX_LOCK(sel); // GetDefaultCmd
-
-	// NOTE: the unitDef->aihint value is being ignored
 	int luaCmd;
 	if (eventHandler.DefaultCommand(unit, feature, luaCmd)) {
 		return luaCmd;
 	}
 
+	GML_RECMUTEX_LOCK(sel); // GetDefaultCmd
+
 	// return the default if there are no units selected
 	CUnitSet::const_iterator ui = selectedUnits.begin();
 	if (ui == selectedUnits.end()) {
@@ -628,6 +651,7 @@ void CSelectedUnits::DrawCommands()
 	glLineWidth(cmdColors.QueuedLineWidth());
 
 	GML_RECMUTEX_LOCK(unit); // DrawCommands
+	GML_RECMUTEX_LOCK(feat); // DrawCommands
 	GML_RECMUTEX_LOCK(grpsel); // DrawCommands
 	GML_STDMUTEX_LOCK(cai); // DrawCommands
 
@@ -635,16 +659,15 @@ void CSelectedUnits::DrawCommands()
 	if (selectedGroup != -1) {
 		CUnitSet& groupUnits = grouphandlers[gu->myTeam]->groups[selectedGroup]->units;
 		for(ui = groupUnits.begin(); ui != groupUnits.end(); ++ui) {
-			(*ui)->commandAI->DrawCommands();
+			commandDrawer->Draw((*ui)->commandAI);
 		}
 	} else {
 		for(ui = selectedUnits.begin(); ui != selectedUnits.end(); ++ui) {
-			(*ui)->commandAI->DrawCommands();
+			commandDrawer->Draw((*ui)->commandAI);
 		}
 	}
 
 	// draw the commands from AIs
-	grouphandlers[gu->myTeam]->DrawCommands();
 	waitCommandsAI.DrawCommands();
 
 	glLineWidth(1.0f);
@@ -658,27 +681,29 @@ void CSelectedUnits::DrawCommands()
 // LuaUnsyncedRead::GetCurrentTooltip --> CMouseHandler::GetCurrentTooltip
 // CMouseHandler::GetCurrentTooltip --> CMiniMap::GetToolTip --> GetTooltip
 // CMouseHandler::GetCurrentTooltip --> GetTooltip
-std::string CSelectedUnits::GetTooltip(void)
+std::string CSelectedUnits::GetTooltip()
 {
-	GML_RECMUTEX_LOCK(sel); // GetTooltip - called from TooltipConsole::Draw --> MouseHandler::GetCurrentTooltip --> GetTooltip
-
 	std::string s = "";
-	if (!selectedUnits.empty()) {
-		const CUnit* unit = (*selectedUnits.begin());
-		const CTeam* team = NULL;
+	{
+		GML_RECMUTEX_LOCK(sel); // GetTooltip - called from TooltipConsole::Draw --> MouseHandler::GetCurrentTooltip --> GetTooltip
+
+		if (!selectedUnits.empty()) {
+			const CUnit* unit = (*selectedUnits.begin());
+			const CTeam* team = NULL;
+
+			// show the player name instead of unit name if it has FBI tag showPlayerName
+			if (unit->unitDef->showPlayerName) {
+				team = teamHandler->Team(unit->team);
+				s = team->GetControllerName();
+			} else {
+				s = (*selectedUnits.begin())->tooltip;
+			}
 
-		// show the player name instead of unit name if it has FBI tag showPlayerName
-		if (unit->unitDef->showPlayerName) {
-			team = teamHandler->Team(unit->team);
-			s = team->GetControllerName();
-		} else {
-			s = (*selectedUnits.begin())->tooltip;
 		}
 
-	}
-
-	if (selectedUnits.empty()) {
-		return s;
+		if (selectedUnits.empty()) {
+			return s;
+		}
 	}
 
 	const string custom = eventHandler.WorldTooltip(NULL, NULL, NULL);
@@ -686,67 +711,73 @@ std::string CSelectedUnits::GetTooltip(void)
 		return custom;
 	}
 
-	char tmp[500];
-	int numFuel = 0;
-	float maxHealth = 0.0f, curHealth = 0.0f;
-	float maxFuel = 0.0f, curFuel = 0.0f;
-	float exp = 0.0f, cost = 0.0f, range = 0.0f;
-	float metalMake = 0.0f, metalUse = 0.0f, energyMake = 0.0f, energyUse = 0.0f;
+	{
+		GML_RECMUTEX_LOCK(sel); // GetTooltip
+
+		char tmp[500];
+		int numFuel = 0;
+		float maxHealth = 0.0f, curHealth = 0.0f;
+		float maxFuel = 0.0f, curFuel = 0.0f;
+		float exp = 0.0f, cost = 0.0f, range = 0.0f;
+		float metalMake = 0.0f, metalUse = 0.0f, energyMake = 0.0f, energyUse = 0.0f;
 
 #define NO_TEAM -32
 #define MULTI_TEAM -64
-	int ctrlTeam = NO_TEAM;
+		int ctrlTeam = NO_TEAM;
 
-	CUnitSet::iterator ui;
-	for (ui = selectedUnits.begin(); ui != selectedUnits.end(); ++ui) {
-		const CUnit* unit = *ui;
-		maxHealth  += unit->maxHealth;
-		curHealth  += unit->health;
-		exp        += unit->experience;
-		cost       += unit->metalCost + (unit->energyCost / 60.0f);
-		range      += unit->maxRange;
-		metalMake  += unit->metalMake;
-		metalUse   += unit->metalUse;
-		energyMake += unit->energyMake;
-		energyUse  += unit->energyUse;
-		maxFuel    += unit->unitDef->maxFuel;
-		curFuel    += unit->currentFuel;
-		if (unit->unitDef->maxFuel > 0) {
-			numFuel++;
+		CUnitSet::const_iterator ui;
+		for (ui = selectedUnits.begin(); ui != selectedUnits.end(); ++ui) {
+			const CUnit* unit = *ui;
+			maxHealth  += unit->maxHealth;
+			curHealth  += unit->health;
+			exp        += unit->experience;
+			cost       += unit->metalCost + (unit->energyCost / 60.0f);
+			range      += unit->maxRange;
+			metalMake  += unit->metalMake;
+			metalUse   += unit->metalUse;
+			energyMake += unit->energyMake;
+			energyUse  += unit->energyUse;
+			maxFuel    += unit->unitDef->maxFuel;
+			curFuel    += unit->currentFuel;
+			if (unit->unitDef->maxFuel > 0) {
+				numFuel++;
+			}
+			if (ctrlTeam == NO_TEAM) {
+				ctrlTeam = unit->team;
+			} else if (ctrlTeam != unit->team) {
+				ctrlTeam = MULTI_TEAM;
+			}
 		}
-		if(ctrlTeam == NO_TEAM)
-			ctrlTeam = unit->team;
-		else if(ctrlTeam != unit->team)
-			ctrlTeam = MULTI_TEAM;
+		if ((numFuel > 0) && (maxFuel > 0.0f)) {
+			curFuel = curFuel / numFuel;
+			maxFuel = maxFuel / numFuel;
+		}
+		const float num = selectedUnits.size();
+
+		s += CTooltipConsole::MakeUnitStatsString(
+			curHealth, maxHealth,
+			curFuel,   maxFuel,
+			(exp / num), cost, (range / num),
+			metalMake,  metalUse,
+			energyMake, energyUse);
+
+		if (gs->cheatEnabled && (num == 1)) {
+			const CUnit* unit = *selectedUnits.begin();
+			SNPRINTF(tmp, sizeof(tmp), "\xff\xc0\xc0\xff  [TechLevel %i]",
+				unit->unitDef->techLevel);
+			s += tmp;
+		}
+
+		std::string ctrlName = "";
+		if (ctrlTeam == MULTI_TEAM) {
+			ctrlName = "(Multiple teams)";
+		} else if (ctrlTeam != NO_TEAM) {
+			ctrlName = teamHandler->Team(ctrlTeam)->GetControllerName();
+		}
+		s += "\n\xff\xff\xff\xff" + ctrlName;
+
+		return s;
 	}
-	if ((numFuel > 0) && (maxFuel > 0.0f)) {
-		curFuel = curFuel / numFuel;
-		maxFuel = maxFuel / numFuel;
-	}
-	const float num = selectedUnits.size();
-
-	s += CTooltipConsole::MakeUnitStatsString(
-	       curHealth, maxHealth,
-	       curFuel,   maxFuel,
-	       (exp / num), cost, (range / num),
-	       metalMake,  metalUse,
-	       energyMake, energyUse);
-
-  if (gs->cheatEnabled && (num == 1)) {
-  	CUnit* unit = *selectedUnits.begin();
-    SNPRINTF(tmp, sizeof(tmp), "\xff\xc0\xc0\xff  [TechLevel %i]",
-             unit->unitDef->techLevel);
-    s += tmp;
-	}
-
-	std::string ctrlName = "";
-	if(ctrlTeam == MULTI_TEAM)
-		ctrlName = "(Multiple teams)";
-	else if(ctrlTeam != NO_TEAM)
-		ctrlName = teamHandler->Team(ctrlTeam)->GetControllerName();
-	s += "\n\xff\xff\xff\xff" + ctrlName;
-
-	return s;
 }
 
 
@@ -761,36 +792,34 @@ void CSelectedUnits::SetCommandPage(int page)
 }
 
 
-void CSelectedUnits::SendSelection(void)
+
+void CSelectedUnits::SendCommand(const Command& c)
 {
-	GML_RECMUTEX_LOCK(sel); // SendSelection
+	if (selectionChanged) {
+		// send new selection
+		GML_RECMUTEX_LOCK(sel); // SendSelection
 
-	// first, convert CUnit* to unit IDs.
-	std::vector<short> selectedUnitIDs(selectedUnits.size());
-	std::vector<short>::iterator i = selectedUnitIDs.begin();
-	CUnitSet::const_iterator ui = selectedUnits.begin();
-	for(; ui != selectedUnits.end(); ++i, ++ui) *i = (*ui)->id;
-	net->Send(CBaseNetProtocol::Get().SendSelect(gu->myPlayerNum, selectedUnitIDs));
-	selectionChanged=false;
-}
-
-
-void CSelectedUnits::SendCommand(Command& c)
-{
-	if (selectionChanged) {		//send new selection
-		SendSelection();
+		// first, convert CUnit* to unit IDs.
+		std::vector<short> selectedUnitIDs(selectedUnits.size());
+		std::vector<short>::iterator i = selectedUnitIDs.begin();
+		CUnitSet::const_iterator ui = selectedUnits.begin();
+		for(; ui != selectedUnits.end(); ++i, ++ui) {
+			*i = (*ui)->id;
+		}
+		net->Send(CBaseNetProtocol::Get().SendSelect(gu->myPlayerNum, selectedUnitIDs));
+		selectionChanged = false;
 	}
-	net->Send(CBaseNetProtocol::Get().SendCommand(gu->myPlayerNum, c.id, c.options, c.params));
+
+	net->Send(CBaseNetProtocol::Get().SendCommand(gu->myPlayerNum, c.GetID(), c.options, c.params));
 }
 
 
-void CSelectedUnits::SendCommandsToUnits(const vector<int>& unitIDs,
-                                         const vector<Command>& commands)
+void CSelectedUnits::SendCommandsToUnits(const std::vector<int>& unitIDs, const std::vector<Command>& commands)
 {
 	// NOTE: does not check for invalid unitIDs
 
 	if (gu->spectating && !gs->godMode) {
-		return; // don't waste bandwidth
+		return; // do not waste bandwidth
 	}
 
 	const unsigned unitIDCount  = unitIDs.size();
@@ -806,21 +835,22 @@ void CSelectedUnits::SendCommandsToUnits(const vector<int>& unitIDs,
 	}
 
 	unsigned msgLen = 0;
-	msgLen += (1 + 2 + 1); // msg type, msg size, player ID
+	msgLen += (1 + 2 + 1 + 1); // msg type, msg size, player ID, AI ID
 	msgLen += 2; // unitID count
 	msgLen += unitIDCount * 2;
 	msgLen += 2; // command count
 	msgLen += commandCount * (4 + 1 + 2); // id, options, params size
 	msgLen += totalParams * 4;
 	if (msgLen > 8192) {
-		logOutput.Print("Discarded oversized NETMSG_AICOMMANDS packet: %i\n",
-		                msgLen);
+		LOG_L(L_WARNING, "Discarded oversized NETMSG_AICOMMANDS packet: %i",
+				msgLen);
 		return; // drop the oversized packet
 	}
 	netcode::PackPacket* packet = new netcode::PackPacket(msgLen);
 	*packet << static_cast<unsigned char>(NETMSG_AICOMMANDS)
 	        << static_cast<unsigned short>(msgLen)
-	        << static_cast<unsigned char>(gu->myPlayerNum);
+	        << static_cast<unsigned char>(gu->myPlayerNum)
+	        << skirmishAIHandler.GetCurrentAIID();
 
 	*packet << static_cast<unsigned short>(unitIDCount);
 	for (std::vector<int>::const_iterator it = unitIDs.begin(); it != unitIDs.end(); ++it)
@@ -831,7 +861,7 @@ void CSelectedUnits::SendCommandsToUnits(const vector<int>& unitIDs,
 	*packet << static_cast<unsigned short>(commandCount);
 	for (unsigned i = 0; i < commandCount; ++i) {
 		const Command& cmd = commands[i];
-		*packet << static_cast<unsigned int>(cmd.id)
+		*packet << static_cast<unsigned int>(cmd.GetID())
 		        << cmd.options
 		        << static_cast<unsigned short>(cmd.params.size()) << cmd.params;
 	}
