@@ -1,33 +1,30 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-// Handling commands given to the currently  selected group of units.
-
-#include "StdAfx.h"
-
-#include "mmgr.h"
-
 #include "SelectedUnitsAI.h"
+
 #include "SelectedUnits.h"
-#include "LogOutput.h"
-#include "NetProtocol.h"
-#include "Sim/Misc/GlobalSynced.h"
 #include "GlobalUnsynced.h"
+#include "Player.h"
 #include "PlayerHandler.h"
 #include "WaitCommandsAI.h"
 #include "Map/Ground.h"
+#include "Sim/Misc/GlobalSynced.h"
 #include "Sim/Misc/QuadField.h"
 #include "Sim/Misc/TeamHandler.h"
 #include "Sim/MoveTypes/MoveType.h"
 #include "Sim/Units/UnitHandler.h"
 #include "Sim/Units/CommandAI/CommandAI.h"
+#include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitDef.h"
+#include "System/NetProtocol.h"
+#include "System/mmgr.h"
 
 const int CMDPARAM_MOVE_X = 0;
 const int CMDPARAM_MOVE_Y = 1;
 const int CMDPARAM_MOVE_Z = 2;
 
 
-//Global object
+// Global object
 CSelectedUnitsAI selectedUnitsAI;
 
 CSelectedUnitsAI::CSelectedUnitsAI()
@@ -57,10 +54,8 @@ inline void CSelectedUnitsAI::AddUnitSetMaxSpeedCommand(CUnit* unit,
 	// sets the wanted speed of this unit to its max speed
 	CCommandAI* cai = unit->commandAI;
 	if (cai->CanSetMaxSpeed()) {
-		Command c;
-		c.id = CMD_SET_WANTED_MAX_SPEED;
-		c.options = options;
-		c.params.push_back(unit->maxSpeed);
+		Command c(CMD_SET_WANTED_MAX_SPEED, options);
+		c.AddParam(unit->maxSpeed);
 		cai->GiveCommand(c, false);
 	}
 }
@@ -74,10 +69,8 @@ inline void CSelectedUnitsAI::AddGroupSetMaxSpeedCommand(CUnit* unit,
 	// is already in units per frame)
 	CCommandAI* cai = unit->commandAI;
 	if (cai->CanSetMaxSpeed()) {
-		Command c;
-		c.id = CMD_SET_WANTED_MAX_SPEED;
-		c.options = options;
-		c.params.push_back(minMaxSpeed);
+		Command c(CMD_SET_WANTED_MAX_SPEED, options);
+		c.AddParam(minMaxSpeed);
 		cai->GiveCommand(c, false);
 	}
 }
@@ -85,7 +78,7 @@ inline void CSelectedUnitsAI::AddGroupSetMaxSpeedCommand(CUnit* unit,
 
 static inline bool MayRequireSetMaxSpeedCommand(const Command &c)
 {
-	switch (c.id) {
+	switch (c.GetID()) {
 		// this is not a complete list
 		case CMD_STOP:
 		case CMD_WAIT:
@@ -103,28 +96,31 @@ static inline bool MayRequireSetMaxSpeedCommand(const Command &c)
 
 void CSelectedUnitsAI::GiveCommandNet(Command &c, int player)
 {
-	const vector<int>& netSelected = selectedUnits.netSelected[player];
-	vector<int>::const_iterator ui;
+	const std::vector<int>& netSelected = selectedUnits.netSelected[player];
+	std::vector<int>::const_iterator ui;
 
-	int nbrOfSelectedUnits = netSelected.size();
+	const int nbrOfSelectedUnits = netSelected.size();
+	const int& cmd_id = c.GetID();
 
 	if (nbrOfSelectedUnits < 1) {
 		// no units to command
 	}
-	else if ((c.id == CMD_ATTACK) &&
-			((c.params.size() == 6) ||
-			 ((c.params.size() == 4) && (c.params[3] > 0.001f)))) {
+	else if ((cmd_id == CMD_ATTACK) && (
+			(c.GetParamsCount() == 6) ||
+			((c.GetParamsCount() == 4) && (c.GetParam(3) > 0.001f))
+		))
+	{
 		SelectAttack(c, player);
 	}
 	else if (nbrOfSelectedUnits == 1) {
 		// a single unit selected
 		CUnit* unit = uh->units[*netSelected.begin()];
-		if(unit) {
-			unit->commandAI->GiveCommand(c, false);
+		if (unit) {
+			unit->commandAI->GiveCommand(c, true);
 			if (MayRequireSetMaxSpeedCommand(c)) {
 				AddUnitSetMaxSpeedCommand(unit, c.options);
 			}
-			if (c.id == CMD_WAIT) {
+			if (cmd_id == CMD_WAIT) {
 				if (player == gu->myPlayerNum) {
 					waitCommandsAI.AcknowledgeCommand(c);
 				}
@@ -148,7 +144,7 @@ void CSelectedUnitsAI::GiveCommandNet(Command &c, int player)
 	//   ALT+CTRL:  Group Locked       command  (maintain relative positions)
 	//
 
-	else if (((c.id == CMD_MOVE) || (c.id == CMD_FIGHT)) && (c.params.size() == 6)) {
+	else if (((cmd_id == CMD_MOVE) || (cmd_id == CMD_FIGHT)) && (c.GetParamsCount() == 6)) {
 		CalculateGroupData(player, !!(c.options & SHIFT_KEY));
 
 		MakeFrontMove(&c, player);
@@ -156,7 +152,7 @@ void CSelectedUnitsAI::GiveCommandNet(Command &c, int player)
 		const bool groupSpeed = !!(c.options & CONTROL_KEY);
 		for(ui = netSelected.begin(); ui != netSelected.end(); ++ui) {
 			CUnit* unit = uh->units[*ui];
-			if(unit){
+			if (unit) {
 				if (groupSpeed) {
 					AddGroupSetMaxSpeedCommand(unit, c.options);
 				} else {
@@ -165,11 +161,11 @@ void CSelectedUnitsAI::GiveCommandNet(Command &c, int player)
 			}
 		}
 	}
-	else if ((c.id == CMD_MOVE) && (c.options & ALT_KEY)) {
+	else if ((cmd_id == CMD_MOVE) && (c.options & ALT_KEY)) {
 		CalculateGroupData(player, !!(c.options & SHIFT_KEY));
 
 		// use the vector from the middle of group to new pos as forward dir
-		const float3 pos(c.params[0], c.params[1], c.params[2]);
+		const float3 pos(c.GetParam(0), c.GetParam(1), c.GetParam(3));
 		float3 frontdir = pos - centerCoor;
 		frontdir.y = 0.0f;
 		frontdir.ANormalize();
@@ -188,7 +184,7 @@ void CSelectedUnitsAI::GiveCommandNet(Command &c, int player)
 		const bool groupSpeed = !!(c.options & CONTROL_KEY);
 		for(ui = netSelected.begin(); ui != netSelected.end(); ++ui) {
 			CUnit* unit = uh->units[*ui];
-			if(unit){
+			if (unit) {
 				if (groupSpeed) {
 					AddGroupSetMaxSpeedCommand(unit, c.options);
 				} else {
@@ -198,7 +194,7 @@ void CSelectedUnitsAI::GiveCommandNet(Command &c, int player)
 		}
 	}
 	else if ((c.options & CONTROL_KEY) &&
-	         ((c.id == CMD_MOVE) || (c.id == CMD_PATROL) || (c.id == CMD_FIGHT))) {
+	         ((cmd_id == CMD_MOVE) || (cmd_id == CMD_PATROL) || (cmd_id == CMD_FIGHT))) {
 		CalculateGroupData(player, !!(c.options & SHIFT_KEY));
 
 		const bool groupSpeed = !(c.options & ALT_KEY);
@@ -239,7 +235,7 @@ void CSelectedUnitsAI::GiveCommandNet(Command &c, int player)
 				}
 			}
 		}
-		if (c.id == CMD_WAIT) {
+		if (cmd_id == CMD_WAIT) {
 			if (player == gu->myPlayerNum) {
 				waitCommandsAI.AcknowledgeCommand(c);
 			}
@@ -258,7 +254,7 @@ void CSelectedUnitsAI::CalculateGroupData(int player, bool queueing) {
 	sumLength = 0; ///
 	int mobileUnits = 0;
 	minMaxSpeed = 1e9f;
-	for(vector<int>::iterator ui = selectedUnits.netSelected[player].begin(); ui != selectedUnits.netSelected[player].end(); ++ui) {
+	for(std::vector<int>::iterator ui = selectedUnits.netSelected[player].begin(); ui != selectedUnits.netSelected[player].end(); ++ui) {
 		CUnit* unit=uh->units[*ui];
 		if(unit){
 			const UnitDef* ud = unit->unitDef;
@@ -314,7 +310,7 @@ void CSelectedUnitsAI::MakeFrontMove(Command* c,int player)
 	float3 nextPos(0.0f, 0.0f, 0.0f);//it's in "front" coordinates (rotated to real, moved by rightPos)
 
 	if(centerPos.distance(rightPos)<selectedUnits.netSelected[player].size()+33){	//Strange line! treat this as a standard move if the front isnt long enough
-		for(vector<int>::iterator ui = selectedUnits.netSelected[player].begin(); ui != selectedUnits.netSelected[player].end(); ++ui) {
+		for(std::vector<int>::iterator ui = selectedUnits.netSelected[player].begin(); ui != selectedUnits.netSelected[player].end(); ++ui) {
 			CUnit* unit=uh->units[*ui];
 			if(unit){
 				unit->commandAI->GiveCommand(*c, false);
@@ -358,22 +354,23 @@ void CSelectedUnitsAI::MakeFrontMove(Command* c,int player)
 			MoveToPos(oi->second, nextPos, sd, c, NULL, &newline);
 
 		if(oi == orderedUnits.end() || newline) {
-			std::vector<std::vector<int> *> sortUnitsVector;
-			for(std::multimap<float, std::vector<int> >::iterator ui = sortUnits.begin(); ui!=sortUnits.end(); ++ui)
+			std::vector<std::vector<int>*> sortUnitsVector;
+			for (std::multimap<float, std::vector<int> >::iterator ui = sortUnits.begin(); ui!=sortUnits.end(); ++ui) {
 				sortUnitsVector.push_back(&(*ui).second);
+			}
 
 			std::vector<int> randunits;
 			int sz = sortUnitsVector.size();
 			std::vector<int> sortPos(sz, 0);
-			for(std::vector<std::pair<int,Command> >::iterator fi = frontcmds.begin(); fi != frontcmds.end(); ++fi){
+			for (std::vector<std::pair<int,Command> >::iterator fi = frontcmds.begin(); fi != frontcmds.end(); ++fi) {
 				int bestpos = 0;
 				float bestval = 1.0f;
-				for(int i = 0; i < sz; ++i) {
-					int n = sortUnitsVector[i]->size();
-					int k = sortPos[i];
-					if(k < n) {
-						float val = (0.5f + k) / (float)n;
-						if(val < bestval) {
+				for (int i = 0; i < sz; ++i) {
+					const int n = sortUnitsVector[i]->size();
+					const int k = sortPos[i];
+					if (k < n) {
+						const float val = (0.5f + k) / (float)n;
+						if (val < bestval) {
 							bestval = val;
 							bestpos = i;
 						}
@@ -385,8 +382,8 @@ void CSelectedUnitsAI::MakeFrontMove(Command* c,int player)
 			}
 			sortUnits.clear();
 
-			int i=0;
-			for (std::vector<std::pair<int,Command> >::iterator fi = frontcmds.begin(); fi != frontcmds.end(); ++fi){
+			int i = 0;
+			for (std::vector<std::pair<int,Command> >::iterator fi = frontcmds.begin(); fi != frontcmds.end(); ++fi) {
 				uh->units[randunits[i++]]->commandAI->GiveCommand(fi->second, false);
 			}
 			frontcmds.clear();
@@ -398,54 +395,58 @@ void CSelectedUnitsAI::MakeFrontMove(Command* c,int player)
 void CSelectedUnitsAI::CreateUnitOrder(std::multimap<float,int>& out,int player)
 {
 	const vector<int>& netUnits = selectedUnits.netSelected[player];
-	for(vector<int>::const_iterator ui = netUnits.begin(); ui != netUnits.end(); ++ui){
-		CUnit* unit=uh->units[*ui];
-		if(unit){
+	for (vector<int>::const_iterator ui = netUnits.begin(); ui != netUnits.end(); ++ui) {
+		const CUnit* unit = uh->units[*ui];
+		if (unit) {
 			const UnitDef* ud = unit->unitDef;
-			float range=unit->maxRange;
-			if(range<1)
-				range=2000;		//give weaponless units a long range to make them go to the back
-			float value=(ud->metalCost*60+ud->energyCost)/unit->unitDef->health*range;
-			out.insert(std::pair<float,int>(value,*ui));
+			float range = unit->maxRange;
+			if (range < 1) {
+				// give weaponless units a long range to make them go to the back
+				range = 2000;
+			}
+			const float value = ((ud->metalCost * 60) + ud->energyCost) / unit->unitDef->health * range;
+			out.insert(std::pair<float, int>(value, *ui));
 		}
 	}
 }
 
 
-float3 CSelectedUnitsAI::MoveToPos(int unit, float3 nextCornerPos, float3 dir, Command* command, std::vector<std::pair<int,Command> > *frontcmds, bool *newline)
+float3 CSelectedUnitsAI::MoveToPos(int unit, float3 nextCornerPos, float3 dir, Command* command, std::vector<std::pair<int,Command> >* frontcmds, bool* newline)
 {
 	//int lineNum=posNum/numColumns;
 	//int colNum=posNum-lineNum*numColumns;
 	//float side=(0.25f+colNum*0.5f)*columnDist*(colNum&1 ? -1:1);
-	*newline=false;
-	if(nextCornerPos.x-addSpace>frontLength) {
-		nextCornerPos.x=0; nextCornerPos.z-=avgLength*2*8;
-		*newline=true;
+	*newline = false;
+	if ((nextCornerPos.x - addSpace) > frontLength) {
+		nextCornerPos.x = 0; nextCornerPos.z -= avgLength * 2 * 8;
+		*newline = true;
 	}
-	if(!frontcmds)
+	if (!frontcmds) {
 		return nextCornerPos;
-	int unitSize=16;
-	CUnit* u = uh->units[unit];
+	}
+	int unitSize = 16;
+	const CUnit* u = uh->units[unit];
 	if (u) {
 		const UnitDef* ud = u->unitDef;
-		unitSize = (int)((ud->xsize + ud->zsize)/2);
+		unitSize = (int)((ud->xsize + ud->zsize) / 2);
+	}
+	float3 retPos( nextCornerPos.x + unitSize * 8 * 2 + addSpace, 0, nextCornerPos.z);
+	float3 movePos(nextCornerPos.x + unitSize * 8     + addSpace, 0, nextCornerPos.z); // posit in coordinates of "front"
+	if (nextCornerPos.x == 0) {
+		movePos.x = unitSize * 8;
+		retPos.x -= addSpace;
 	}
-	float3 retPos(nextCornerPos.x+unitSize*8*2+addSpace, 0, nextCornerPos.z);
-	float3 movePos(nextCornerPos.x+unitSize*8+addSpace, 0, nextCornerPos.z); //posit in coordinates of "front"
-	if (nextCornerPos.x==0) { movePos.x=unitSize*8; retPos.x -= addSpace ;}
 	float3 pos;
 	pos.x = rightPos.x + (movePos.x * (dir.x / dir.y)) - (movePos.z * (dir.z/dir.y));
 	pos.z = rightPos.z + (movePos.x * (dir.z / dir.y)) + (movePos.z * (dir.x/dir.y));
-	pos.y = ground->GetHeight(pos.x, pos.z);
+	pos.y = ground->GetHeightAboveWater(pos.x, pos.z);
 
-	Command c;
-	c.id = command->id;
+	Command c(command->GetID(), command->options);
 	c.params.push_back(pos.x);
 	c.params.push_back(pos.y);
 	c.params.push_back(pos.z);
-	c.options = command->options;
 
-	frontcmds->push_back(std::pair<int,Command>(unit,c));
+	frontcmds->push_back(std::pair<int, Command>(unit, c));
 
 	return retPos;
 }
@@ -460,7 +461,7 @@ struct DistInfo {
 
 void CSelectedUnitsAI::SelectAttack(const Command& cmd, int player)
 {
-	vector<int> targets;
+	std::vector<int> targets;
 	const float3 pos0(cmd.params[0], cmd.params[1], cmd.params[2]);
 	if (cmd.params.size() == 4) {
 		SelectCircleUnits(pos0, cmd.params[3], targets, player);
@@ -473,22 +474,20 @@ void CSelectedUnitsAI::SelectAttack(const Command& cmd, int player)
 		return;
 	}
 
-	const vector<int>& selected = selectedUnits.netSelected[player];
+	const std::vector<int>& selected = selectedUnits.netSelected[player];
 	const int selectedCount = (int)selected.size();
 	if (selectedCount <= 0) {
 		return;
 	}
 
-	Command attackCmd;
-	attackCmd.id = CMD_ATTACK;
-	attackCmd.options = cmd.options;
+	Command attackCmd(CMD_ATTACK, cmd.options);
 	attackCmd.params.push_back(0.0f); // dummy
 
 	// delete the attack commands and bail for CONTROL_KEY
 	if (cmd.options & CONTROL_KEY) {
 		attackCmd.options |= SHIFT_KEY;
 		for (int s = 0; s < selectedCount; s++) {
-			CUnit* unit = uh->units[selected[s]];
+			const CUnit* unit = uh->units[selected[s]];
 			if (unit == NULL) {
 				continue;
 			}
@@ -506,7 +505,7 @@ void CSelectedUnitsAI::SelectAttack(const Command& cmd, int player)
 	const bool queueing = !!(cmd.options & SHIFT_KEY);
 
 	// get the group center
-	float3 midPos(0.0f, 0.0f, 0.0f);
+	float3 midPos(ZeroVector);
 	int realCount = 0;
 	for (int s = 0; s < selectedCount; s++) {
 		CUnit* unit = uh->units[selected[s]];
@@ -526,12 +525,12 @@ void CSelectedUnitsAI::SelectAttack(const Command& cmd, int player)
 	midPos /= (float)realCount;
 
 	// sort the targets
-	vector<DistInfo> distVec;
+	std::vector<DistInfo> distVec;
 	int t;
 	for (t = 0; t < targetsCount; t++) {
 		DistInfo di;
 		di.unitID = targets[t];
-		CUnit* unit = uh->units[di.unitID];
+		const CUnit* unit = uh->units[di.unitID];
 		const float3 unitPos = queueing ? LastQueuePosition(unit) : float3(unit->midPos);
 		di.dist = (unitPos - midPos).SqLength2D();
 		distVec.push_back(di);
@@ -563,11 +562,11 @@ void CSelectedUnitsAI::SelectAttack(const Command& cmd, int player)
 
 
 void CSelectedUnitsAI::SelectCircleUnits(const float3& pos, float radius,
-                                         vector<int>& units, int player)
+                                         std::vector<int>& units, int player)
 {
 	units.clear();
 
-	if ((player < 0) || (player >= playerHandler->ActivePlayers())) {
+	if (!playerHandler->IsValidPlayer(player)) {
 		return;
 	}
 	const CPlayer* p = playerHandler->Player(player);
@@ -576,7 +575,7 @@ void CSelectedUnitsAI::SelectCircleUnits(const float3& pos, float radius,
 	}
 	const int allyTeam = teamHandler->AllyTeam(p->team);
 
-	vector<CUnit*> tmpUnits = qf->GetUnits(pos, radius);
+	const vector<CUnit*> &tmpUnits = qf->GetUnits(pos, radius);
 
 	const float radiusSqr = (radius * radius);
 	const int count = (int)tmpUnits.size();
@@ -601,7 +600,7 @@ void CSelectedUnitsAI::SelectRectangleUnits(const float3& pos0,
 {
 	units.clear();
 
-	if ((player < 0) || (player >= playerHandler->ActivePlayers())) {
+	if (!playerHandler->IsValidPlayer(player)) {
 		return;
 	}
 	const CPlayer* p = playerHandler->Player(player);
@@ -613,11 +612,11 @@ void CSelectedUnitsAI::SelectRectangleUnits(const float3& pos0,
 	const float3 mins(std::min(pos0.x, pos1.x), 0.0f, std::min(pos0.z, pos1.z));
 	const float3 maxs(std::max(pos0.x, pos1.x), 0.0f, std::max(pos0.z, pos1.z));
 
-	vector<CUnit*> tmpUnits = qf->GetUnitsExact(mins, maxs);
+	const vector<CUnit*> &tmpUnits = qf->GetUnitsExact(mins, maxs);
 
 	const int count = (int)tmpUnits.size();
 	for (int i = 0; i < count; i++) {
-		CUnit* unit = tmpUnits[i];
+		const CUnit* unit = tmpUnits[i];
 		if ((unit == NULL) || (unit->allyteam == allyTeam) ||
 				!(unit->losStatus[allyTeam] & (LOS_INLOS | LOS_INRADAR))) {
 			continue;
@@ -627,7 +626,7 @@ void CSelectedUnitsAI::SelectRectangleUnits(const float3& pos0,
 }
 
 
-float3 CSelectedUnitsAI::LastQueuePosition(CUnit* unit)
+float3 CSelectedUnitsAI::LastQueuePosition(const CUnit* unit)
 {
 	const CCommandQueue& queue = unit->commandAI->commandQue;
 	CCommandQueue::const_reverse_iterator it;
