@@ -1,21 +1,23 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 #include "CommandColors.h"
+
+#include "System/mmgr.h"
+
+#include "Rendering/GL/myGL.h"
+#include "System/FileSystem/FileHandler.h"
+#include "System/FileSystem/SimpleParser.h"
+#include "System/Util.h"
+
 #include <cstdio>
 #include <cstring>
 #include <string>
 #include <vector>
 #include <map>
 
-#include "mmgr.h"
-
-#include "Rendering/GL/myGL.h"
-#include "FileSystem/FileHandler.h"
-#include "FileSystem/SimpleParser.h"
-#include "Util.h"
-
-using namespace std;
+using std::string;
+using std::vector;
+using std::map;
 
 /******************************************************************************/
 
@@ -210,7 +212,7 @@ bool CCommandColors::LoadConfig(const string& filename)
 			break;
 		}
 
-		vector<string> words = parser.Tokenize(line, 1);
+		const vector<string> &words = parser.Tokenize(line, 1);
 
 		const string command = StringToLower(words[0]);
 
