@@ -1,13 +1,11 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 #include <SDL_keysym.h>
 #include <SDL_mouse.h>
 
+#include "System/mmgr.h"
 #include "lib/gml/ThreadSafeContainers.h"
 
-#include "mmgr.h"
-
 #include "CommandColors.h"
 #include "CursorIcons.h"
 #include "GuiHandler.h"
@@ -20,8 +18,10 @@
 #include "Game/Camera.h"
 #include "Game/CameraHandler.h"
 #include "Game/GameHelper.h"
+#include "Game/GlobalUnsynced.h"
 #include "Game/Player.h"
 #include "Game/SelectedUnits.h"
+#include "Game/UI/LuaUI.h" // FIXME: for GML
 #include "Game/UI/UnitTracker.h"
 #include "Sim/Misc/TeamHandler.h"
 #include "Lua/LuaUnsyncedCtrl.h"
@@ -31,28 +31,51 @@
 #include "Map/MetalMap.h"
 #include "Map/ReadMap.h"
 #include "Rendering/IconHandler.h"
-#include "Rendering/ProjectileDrawer.hpp"
+#include "Rendering/LineDrawer.h"
+#include "Rendering/ProjectileDrawer.h"
 #include "Rendering/UnitDrawer.h"
+#include "Rendering/GL/myGL.h"
 #include "Rendering/GL/glExtra.h"
 #include "Rendering/GL/VertexArray.h"
 #include "Rendering/Textures/Bitmap.h"
 #include "Sim/Misc/LosHandler.h"
 #include "Sim/Misc/RadarHandler.h"
 #include "Sim/Units/CommandAI/CommandAI.h"
-#include "Sim/Units/CommandAI/LineDrawer.h"
 #include "Sim/Units/Groups/Group.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Weapons/WeaponDefHandler.h"
 #include "Sim/Weapons/Weapon.h"
-#include "System/ConfigHandler.h"
+#include "System/Config/ConfigHandler.h"
 #include "System/EventHandler.h"
 #include "System/Util.h"
 #include "System/TimeProfiler.h"
+#include "System/Input/KeyInput.h"
 #include "System/FileSystem/SimpleParser.h"
-#include "System/Sound/IEffectChannel.h"
+#include "System/Sound/SoundChannels.h"
 
 #include <boost/cstdint.hpp>
 
+#define PLAY_SOUNDS 1
+
+CONFIG(std::string, MiniMapGeometry).defaultValue("2 2 200 200");
+CONFIG(bool, MiniMapFullProxy).defaultValue(true);
+CONFIG(int, MiniMapButtonSize).defaultValue(16);
+
+CONFIG(float, MiniMapUnitSize)
+	.defaultValue(2.5f)
+	.minimumValue(0.0f);
+
+CONFIG(float, MiniMapUnitExp).defaultValue(0.25f);
+CONFIG(float, MiniMapCursorScale).defaultValue(-0.5f);
+CONFIG(bool, MiniMapIcons).defaultValue(true);
+
+CONFIG(int, MiniMapDrawCommands)
+	.defaultValue(1)
+	.minimumValue(0);
+
+CONFIG(bool, MiniMapDrawProjectiles).defaultValue(true);
+CONFIG(bool, SimpleMiniMapColors).defaultValue(false);
+
 //////////////////////////////////////////////////////////////////////
 // Construction/Destruction
 //////////////////////////////////////////////////////////////////////
@@ -60,9 +83,6 @@
 
 CMiniMap* minimap = NULL;
 
-extern boost::uint8_t* keys;
-
-
 CMiniMap::CMiniMap()
 : CInputReceiver(BACK),
   fullProxy(false),
@@ -88,23 +108,21 @@ CMiniMap::CMiniMap()
 		ypos = 0;
 	}
 	else {
-		const std::string geodef = "2 2 200 200";
-		const std::string geo = configHandler->GetString("MiniMapGeometry", geodef);
+		const std::string geo = configHandler->GetString("MiniMapGeometry");
 		ParseGeometry(geo);
 	}
 
-	fullProxy = !!configHandler->Get("MiniMapFullProxy", 1);
-	buttonSize = configHandler->Get("MiniMapButtonSize", 16);
+	fullProxy = configHandler->GetBool("MiniMapFullProxy");
+	buttonSize = configHandler->GetInt("MiniMapButtonSize");
 
-	unitBaseSize = configHandler->Get("MiniMapUnitSize", 2.5f);
-	unitBaseSize = std::max(0.0f, unitBaseSize);
-	unitExponent = configHandler->Get("MiniMapUnitExp", 0.25f);
+	unitBaseSize = configHandler->GetFloat("MiniMapUnitSize");
+	unitExponent = configHandler->GetFloat("MiniMapUnitExp");
 
-	cursorScale = configHandler->Get("MiniMapCursorScale", -0.5f);
-	useIcons = !!configHandler->Get("MiniMapIcons", 1);
-	drawCommands = std::max(0, configHandler->Get("MiniMapDrawCommands", 1));
-	drawProjectiles = !!configHandler->Get("MiniMapDrawProjectiles", 1);
-	simpleColors = !!configHandler->Get("SimpleMiniMapColors", 0);
+	cursorScale = configHandler->GetFloat("MiniMapCursorScale");
+	useIcons = configHandler->GetBool("MiniMapIcons");
+	drawCommands = configHandler->GetInt("MiniMapDrawCommands");
+	drawProjectiles = configHandler->GetBool("MiniMapDrawProjectiles");
+	simpleColors = configHandler->GetBool("SimpleMiniMapColors");
 
 	myColor[0]    = (unsigned char)(0.2f * 255);
 	myColor[1]    = (unsigned char)(0.9f * 255);
@@ -300,7 +318,7 @@ void CMiniMap::SetSlaveMode(bool newMode)
 
 void CMiniMap::ConfigCommand(const std::string& line)
 {
-	const vector<string> words = CSimpleParser::Tokenize(line, 1);
+	const vector<string> &words = CSimpleParser::Tokenize(line, 1);
 	if (words.empty()) {
 		return;
 	}
@@ -486,7 +504,7 @@ void CMiniMap::MoveView(int x, int y)
 {
 	const float3& pos = camera->pos;
 	const float3& dir = camera->forward;
-	float dist = ground->LineGroundCol(pos, pos + (dir * globalRendering->viewRange * 1.4f));
+	float dist = ground->LineGroundCol(pos, pos + (dir * globalRendering->viewRange * 1.4f), false);
 	float3 dif(0,0,0);
 	if (dist > 0) {
 		dif = dir * dist;
@@ -503,7 +521,7 @@ void CMiniMap::SelectUnits(int x, int y) const
 {
 	GML_RECMUTEX_LOCK(sel); // SelectUnits
 
-	if (!keys[SDLK_LSHIFT] && !keys[SDLK_LCTRL]) {
+	if (!keyInput->IsKeyPressed(SDLK_LSHIFT) && !keyInput->IsKeyPressed(SDLK_LCTRL)) {
 		selectedUnits.ClearSelected();
 	}
 
@@ -540,7 +558,7 @@ void CMiniMap::SelectUnits(int x, int y) const
 				if ((midPos.x > xmin) && (midPos.x < xmax) &&
 					(midPos.z > zmin) && (midPos.z < zmax)) {
 
-					if (keys[SDLK_LCTRL] && (selection.find(*ui) != selection.end())) {
+					if (keyInput->IsKeyPressed(SDLK_LCTRL) && (selection.find(*ui) != selection.end())) {
 						selectedUnits.RemoveUnit(*ui);
 					} else {
 						selectedUnits.AddUnit(*ui);
@@ -551,17 +569,19 @@ void CMiniMap::SelectUnits(int x, int y) const
 			}
 		}
 
+		#if (PLAY_SOUNDS == 1)
 		if (addedunits >= 2) {
 			Channels::UserInterface.PlaySample(mouse->soundMultiselID);
 		}
 		else if (addedunits == 1) {
-			int soundIdx = unit->unitDef->sounds.select.getRandomIdx();
+			const int soundIdx = unit->unitDef->sounds.select.getRandomIdx();
 			if (soundIdx >= 0) {
 				Channels::UnitReply.PlaySample(
 					unit->unitDef->sounds.select.getID(soundIdx), unit,
 					unit->unitDef->sounds.select.getVolume(soundIdx));
 			}
 		}
+		#endif
 	}
 	else {
 		// Single unit
@@ -580,14 +600,14 @@ void CMiniMap::SelectUnits(int x, int y) const
 
 		if (unit && ((unit->team == gu->myTeam) || gu->spectatingFullSelect)) {
 			if (bp.lastRelease < (gu->gameTime - mouse->doubleClickTime)) {
-				if (keys[SDLK_LCTRL] && (selection.find(unit) != selection.end())) {
+				if (keyInput->IsKeyPressed(SDLK_LCTRL) && (selection.find(unit) != selection.end())) {
 					selectedUnits.RemoveUnit(unit);
 				} else {
 					selectedUnits.AddUnit(unit);
 				}
 			} else {
 				//double click
-				if (unit->group && !keys[SDLK_LCTRL]) {
+				if (unit->group && !keyInput->IsKeyPressed(SDLK_LCTRL)) {
 					//select the current units group if it has one
 					selectedUnits.SelectGroup(unit->group->id);
 				} else {
@@ -603,7 +623,7 @@ void CMiniMap::SelectUnits(int x, int y) const
 					for (; team <= lastTeam; team++) {
 						CUnitSet& myUnits = teamHandler->Team(team)->units;
 						for (ui = myUnits.begin(); ui != myUnits.end(); ++ui) {
-							if ((*ui)->aihint == unit->aihint) {
+							if ((*ui)->unitDef->id == unit->unitDef->id) {
 								selectedUnits.AddUnit(*ui);
 							}
 						}
@@ -612,12 +632,14 @@ void CMiniMap::SelectUnits(int x, int y) const
 			}
 			bp.lastRelease = gu->gameTime;
 
-			int soundIdx = unit->unitDef->sounds.select.getRandomIdx();
+			#if (PLAY_SOUNDS == 1)
+			const int soundIdx = unit->unitDef->sounds.select.getRandomIdx();
 			if (soundIdx >= 0) {
 				Channels::UnitReply.PlaySample(
 					unit->unitDef->sounds.select.getID(soundIdx), unit,
 					unit->unitDef->sounds.select.getVolume(soundIdx));
 			}
+			#endif
 		}
 	}
 }
@@ -715,7 +737,7 @@ void CMiniMap::MouseMove(int x, int y, int dx, int dy, int button)
 		} else {
 			width = std::min(globalRendering->viewSizeX, width);
 		}
-		if (keys[SDLK_LSHIFT]) {
+		if (keyInput->IsKeyPressed(SDLK_LSHIFT)) {
 			width = (height * gs->mapx) / gs->mapy;
 		}
 		width = std::max(5, width);
@@ -756,7 +778,7 @@ void CMiniMap::MouseRelease(int x, int y, int button)
 
 	if (button == SDL_BUTTON_LEFT) {
 		if (showButtons && maximizeBox.Inside(x, y)) {
-			ToggleMaximized(!!keys[SDLK_LSHIFT]);
+			ToggleMaximized(!!keyInput->GetKeyState(SDLK_LSHIFT));
 			return;
 		}
 
@@ -786,7 +808,7 @@ CUnit* CMiniMap::GetSelectUnit(const float3& pos) const
 
 float3 CMiniMap::GetMapPosition(int x, int y) const
 {
-	const float mapHeight = readmap->maxheight + 1000.0f;
+	const float mapHeight = readmap->initMaxHeight + 1000.0f;
 	const float mapX = gs->mapx * SQUARE_SIZE;
 	const float mapY = gs->mapy * SQUARE_SIZE;
 	const float3 pos(mapX * float(x - xpos) / width, mapHeight,
@@ -804,7 +826,7 @@ void CMiniMap::ProxyMousePress(int x, int y, int button)
 			mapPos = unit->midPos;
 		} else {
 			mapPos = helper->GetUnitErrorPos(unit, gu->myAllyTeam);
-			mapPos.y = readmap->maxheight + 1000.0f;
+			mapPos.y = readmap->initMaxHeight + 1000.0f;
 		}
 	}
 
@@ -818,12 +840,6 @@ void CMiniMap::ProxyMousePress(int x, int y, int button)
 
 void CMiniMap::ProxyMouseRelease(int x, int y, int button)
 {
-	// is this really needed?
-//	CCamera *c = camera;
-//	camera = new CCamera(*c);
-
-//	const float3 tmpMouseDir = mouse->dir;
-
 	float3 mapPos = GetMapPosition(x, y);
 	const CUnit* unit = GetSelectUnit(mapPos);
 	if (unit) {
@@ -831,19 +847,14 @@ void CMiniMap::ProxyMouseRelease(int x, int y, int button)
 			mapPos = unit->midPos;
 		} else {
 			mapPos = helper->GetUnitErrorPos(unit, gu->myAllyTeam);
-			mapPos.y = readmap->maxheight + 1000.0f;
+			mapPos.y = readmap->initMaxHeight + 1000.0f;
 		}
 	}
 
 	float3 mousedir = float3(0.0f, -1.0f, 0.0f);
 	float3 campos = mapPos;
-//	float3 camfwd = mousedir; // not used?
 
 	guihandler->MouseRelease(x, y, -button, campos, mousedir);
-
-//	mouse->dir = tmpMouseDir;
-//	delete camera;
-//	camera = c;
 }
 
 
@@ -898,12 +909,13 @@ std::string CMiniMap::GetTooltip(int x, int y)
 		return buildTip;
 	}
 
-	GML_RECMUTEX_LOCK(sel); // GetToolTip - anti deadlock
-	GML_RECMUTEX_LOCK(quad); // GetToolTip - called from TooltipConsole::Draw --> MouseHandler::GetCurrentTooltip
+	{
+		GML_THRMUTEX_LOCK(unit, GML_DRAW); // GetTooltip
 
-	const CUnit* unit = GetSelectUnit(GetMapPosition(x, y));
-	if (unit) {
-		return CTooltipConsole::MakeUnitString(unit);
+		const CUnit* unit = GetSelectUnit(GetMapPosition(x, y));
+		if (unit) {
+			return CTooltipConsole::MakeUnitString(unit);
+		}
 	}
 
 	const string selTip = selectedUnits.GetTooltip();
@@ -984,7 +996,7 @@ void CMiniMap::Draw()
 
 void CMiniMap::DrawForReal(bool use_geo)
 {
-	SCOPED_TIMER("Draw minimap");
+	SCOPED_TIMER("MiniMap::DrawForReal");
 
 	//glEnable(GL_BLEND);
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -1098,7 +1110,7 @@ void CMiniMap::DrawForReal(bool use_geo)
 
 		const std::set<CUnit*>& units = unitDrawer->GetUnsortedUnits();
 
-		for (std::set<CUnit*>::const_iterator it = units.begin(); it != units.end(); it++) {
+		for (std::set<CUnit*>::const_iterator it = units.begin(); it != units.end(); ++it) {
 			DrawUnit(*it);
 		}
 
@@ -1174,51 +1186,37 @@ void CMiniMap::DrawForReal(bool use_geo)
 
 	glPopMatrix(); // revert to the 2d xform
 
-	// Draw the Camera Frustum
-	left.clear();
-	GetFrustumSide(cam2->bottom);
-	GetFrustumSide(cam2->top);
-	GetFrustumSide(cam2->rightside);
-	GetFrustumSide(cam2->leftside);
-
 	if (!minimap->maximized) {
-		// draw the camera lines
-		std::vector<fline>::iterator fli,fli2;
-		for(fli=left.begin();fli!=left.end();fli++){
-			for(fli2=left.begin();fli2!=left.end();fli2++){
-				if(fli==fli2)
-					continue;
-				if(fli->dir - fli2->dir == 0)
-					continue;
-				float colz = -(fli->base - fli2->base) / (fli->dir - fli2->dir);
-				if (fli2->left * (fli->dir - fli2->dir) > 0) {
-					if (colz > fli->minz && colz < 400096)
-						fli->minz = colz;
-				} else {
-					if (colz < fli->maxz && colz > -10000)
-						fli->maxz = colz;
-				}
-			}
-		}
+		// draw the camera frustum lines
+		cam2->GetFrustumSides(0.0f, 0.0f, 1.0f, true);
+		cam2->ClipFrustumLines(true, -10000.0f, 400096.0f);
+
+		const std::vector<CCamera::FrustumLine>& negSides = cam2->negFrustumSides;
+		const std::vector<CCamera::FrustumLine>& posSides = cam2->posFrustumSides;
+		std::vector<CCamera::FrustumLine>::const_iterator fli;
+
 		CVertexArray* va = GetVertexArray();
 		va->Initialize();
-		va->EnlargeArrays(left.size()*2, 0, VA_SIZE_2D0);
-		for(fli = left.begin(); fli != left.end(); fli++) {
-			if(fli->minz < fli->maxz) {
+		va->EnlargeArrays(negSides.size() * 2, 0, VA_SIZE_2D0);
+
+		for (fli = negSides.begin(); fli != negSides.end(); ++fli) {
+			if (fli->minz < fli->maxz) {
 				va->AddVertex2dQ0(fli->base + (fli->dir * fli->minz), fli->minz);
 				va->AddVertex2dQ0(fli->base + (fli->dir * fli->maxz), fli->maxz);
 			}
 		}
+
 		glLineWidth(2.5f);
-		glColor4f(0,0,0,0.5f);
+		glColor4f(0, 0, 0, 0.5f);
 		va->DrawArray2d0(GL_LINES);
 
 		glLineWidth(1.5f);
-		glColor4f(1,1,1,0.75f);
+		glColor4f(1, 1, 1, 0.75f);
 		va->DrawArray2d0(GL_LINES);
 		glLineWidth(1.0f);
 	}
 
+
 	// selection box
 	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
 	CMouseHandler::ButtonPressEvt& bp = mouse->buttons[SDL_BUTTON_LEFT];
@@ -1476,31 +1474,8 @@ void CMiniMap::DrawButtons()
 }
 
 
-void CMiniMap::GetFrustumSide(float3& side)
-{
-	fline temp;
-	static const float3 up(0,1,0);
 
-	float3 b=up.cross(side); // get vector for collision between frustum and horizontal plane
-
-	if(fabs(b.z) < 0.0001f)  // prevent div by zero
-		b.z = 0.00011f;
-
-	temp.dir = b.x / b.z;      // set direction to that
-	float3 c = b.cross(side);  // get vector from camera to collision line
-	float3 colpoint = cam2->pos - c * (cam2->pos.y/c.y); // a point on the collision line
-
-	temp.base = colpoint.x - colpoint.z * temp.dir; //get intersection between colpoint and z axis
-	temp.left = -1;
-	if(b.z>0)
-		temp.left = 1;
-	temp.maxz = gs->mapy * SQUARE_SIZE;
-	temp.minz = 0;
-	left.push_back(temp);
-}
-
-
-inline const CIconData* CMiniMap::GetUnitIcon(const CUnit* unit, float& scale) const
+inline const icon::CIconData* CMiniMap::GetUnitIcon(const CUnit* unit, float& scale) const
 {
 	scale = 1.0f;
 
@@ -1514,7 +1489,7 @@ inline const CIconData* CMiniMap::GetUnitIcon(const CUnit* unit, float& scale) c
 			|| ((losStatus & LOS_INRADAR)&&((losStatus & prevMask) == prevMask))
 			|| gu->spectatingFullView)
 		{
-			const CIconData* iconData = unit->unitDef->iconType.GetIconData();
+			const icon::CIconData* iconData = unit->unitDef->iconType.GetIconData();
 			if (iconData->GetRadiusAdjust()) {
 				scale *= (unit->radius / 30.0f);
 			}
@@ -1524,7 +1499,7 @@ inline const CIconData* CMiniMap::GetUnitIcon(const CUnit* unit, float& scale) c
 
 	//! show default icon (unknown unitdef)
 	if (losStatus & LOS_INRADAR) {
-		return iconHandler->GetDefaultIconData();
+		return icon::iconHandler->GetDefaultIconData();
 	}
 
 	return NULL;
@@ -1544,7 +1519,7 @@ void CMiniMap::DrawUnit(const CUnit* unit)
 
 	// includes the visibility check
 	float iconScale;
-	const CIconData* iconData = GetUnitIcon(unit, iconScale);
+	const icon::CIconData* iconData = GetUnitIcon(unit, iconScale);
 	if (iconData == NULL) {
 		return;
 	}
