@@ -1,22 +1,21 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include <cstring>
 
-#include "bitops.h"
+#include "System/bitops.h"
 #include "CommandColors.h"
-#include "FileSystem/FileHandler.h"
-#include "FileSystem/SimpleParser.h"
-#include "LogOutput.h"
-#include "Util.h"
-#include "GlobalUnsynced.h"
 #include "Rendering/GlobalRendering.h"
+#include "Rendering/GL/myGL.h"
+#include "Rendering/Textures/Bitmap.h"
 #include "MouseCursor.h"
 #include "HwMouseCursor.h"
-#include "myMath.h"
-#include "Rendering/Textures/Bitmap.h"
+#include "System/Log/ILog.h"
+#include "System/myMath.h"
+#include "System/Util.h"
+#include "System/FileSystem/FileHandler.h"
+#include "System/FileSystem/SimpleParser.h"
 
 using std::string;
 
@@ -101,7 +100,7 @@ bool CMouseCursor::BuildFromSpecFile(const string& name)
 		if (line.empty()) {
 			break;
 		}
-		const std::vector<std::string> words = parser.Tokenize(line, 2);
+		const std::vector<std::string> &words = parser.Tokenize(line, 2);
 		const std::string command = StringToLower(words[0]);
 
 		if ((command == "frame") && (words.size() >= 2)) {
@@ -136,16 +135,16 @@ bool CMouseCursor::BuildFromSpecFile(const string& name)
 				hwCursor->hotSpot = Center;
 			}
 			else {
-				logOutput.Print("%s: unknown hotspot  (%s)\n",
-							specFile.c_str(), words[1].c_str());
+				LOG_L(L_ERROR, "%s: unknown hotspot (%s)",
+						specFile.c_str(), words[1].c_str());
 			}
 		}
 		else if ((command == "lastframe") && (words.size() >= 1)) {
 			lastFrame = atoi(words[1].c_str());
 		}
 		else {
-			logOutput.Print("%s: unknown command  (%s)\n",
-			                specFile.c_str(), command.c_str());
+			LOG_L(L_ERROR, "%s: unknown command (%s)",
+					specFile.c_str(), command.c_str());
 		}
 	}
 
@@ -203,7 +202,7 @@ bool CMouseCursor::LoadCursorImage(const string& name, ImageData& image)
 
 	CBitmap b;
 	if (!b.Load(name)) {
-		logOutput.Print("CMouseCursor: Bad image file: %s", name.c_str());
+		LOG_L(L_ERROR, "CMouseCursor: Bad image file: %s", name.c_str());
 		return false;
 	}
 
