@@ -1,37 +1,31 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-
-#include "mmgr.h"
-
-#include <algorithm>
-#include <boost/cstdint.hpp>
-
 #include "MouseHandler.h"
-#include "Game/CameraHandler.h"
-#include "Game/Camera/CameraController.h"
-#include "Game/Camera.h"
+
 #include "CommandColors.h"
 #include "InputReceiver.h"
 #include "GuiHandler.h"
 #include "MiniMap.h"
 #include "MouseCursor.h"
-#include "System/Input/MouseInput.h"
 #include "TooltipConsole.h"
-#include "Sim/Units/Groups/Group.h"
+#include "Game/CameraHandler.h"
+#include "Game/Camera.h"
 #include "Game/Game.h"
-#include "Game/GameHelper.h"
-#include "Game/SelectedUnits.h"
+#include "Game/GlobalUnsynced.h"
+#include "Game/InMapDraw.h"
+#include "Game/Player.h"
 #include "Game/PlayerHandler.h"
+#include "Game/SelectedUnits.h"
+#include "Game/TraceRay.h"
+#include "Game/Camera/CameraController.h"
+#include "Game/UI/LuaUI.h" // FIXME: for GML
 #include "Game/UI/UnitTracker.h"
+#include "Lua/LuaInputReceiver.h"
 #include "Map/Ground.h"
 #include "Map/MapDamage.h"
-#include "Lua/LuaInputReceiver.h"
-#include "ConfigHandler.h"
 #include "Rendering/GlobalRendering.h"
 #include "Rendering/glFont.h"
 #include "Rendering/GL/myGL.h"
-#include "Rendering/InMapDraw.h"
 #include "Rendering/Textures/Bitmap.h"
 #include "Sim/Features/FeatureDef.h"
 #include "Sim/Features/Feature.h"
@@ -40,12 +34,20 @@
 #include "Sim/Units/UnitDef.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitHandler.h"
-#include "EventHandler.h"
-#include "Exceptions.h"
-#include "FastMath.h"
-#include "myMath.h"
-#include "Sound/ISound.h"
-#include "Sound/IEffectChannel.h"
+#include "Sim/Units/Groups/Group.h"
+#include "System/Config/ConfigHandler.h"
+#include "System/EventHandler.h"
+#include "System/Exceptions.h"
+#include "System/FastMath.h"
+#include "System/myMath.h"
+#include "System/mmgr.h"
+#include "System/Input/KeyInput.h"
+#include "System/Input/MouseInput.h"
+#include "System/Sound/ISound.h"
+#include "System/Sound/SoundChannels.h"
+
+#include <algorithm>
+#include <boost/cstdint.hpp>
 
 // can't be up there since those contain conflicting definitions
 #include <SDL_mouse.h>
@@ -53,13 +55,26 @@
 #include <SDL_keysym.h>
 
 
+#define PLAY_SOUNDS 1
+
+CONFIG(bool, HardwareCursor).defaultValue(false);
+CONFIG(bool, InvertMouse).defaultValue(false);
+CONFIG(float, DoubleClickTime).defaultValue(200.0f);
+
+CONFIG(float, ScrollWheelSpeed)
+	.defaultValue(25.0f)
+	.minimumValue(-255.f)
+	.maximumValue(255.f);
+
+CONFIG(float, CrossSize).defaultValue(12.0f);
+CONFIG(float, CrossAlpha).defaultValue(0.5f);
+CONFIG(float, CrossMoveScale).defaultValue(1.0f);
+CONFIG(float, MouseDragScrollThreshold).defaultValue(0.3f);
+
 //////////////////////////////////////////////////////////////////////
 // Construction/Destruction
 //////////////////////////////////////////////////////////////////////
 
-extern boost::uint8_t *keys;
-
-
 CMouseHandler* mouse = NULL;
 
 static CInputReceiver*& activeReceiver = CInputReceiver::GetActiveReceiverRef();
@@ -67,8 +82,8 @@ static CInputReceiver*& activeReceiver = CInputReceiver::GetActiveReceiverRef();
 
 CMouseHandler::CMouseHandler()
 	: hardwareCursor(false)
-	, lastx(300)
-	, lasty(200)
+	, lastx(-1)
+	, lasty(-1)
 	, hide(true)
 	, hwHide(true)
 	, locked(false)
@@ -79,10 +94,10 @@ CMouseHandler::CMouseHandler()
 	, crossSize(0.0f)
 	, crossAlpha(0.0f)
 	, crossMoveScale(0.0f)
+	, cursorScale(1.0f)
 	, activeButton(-1)
 	, dir(ZeroVector)
 	, soundMultiselID(0)
-	, cursorScale(1.0f)
 	, cursorText("")
 	, currentCursor(NULL)
 	, wasLocked(false)
@@ -97,26 +112,22 @@ CMouseHandler::CMouseHandler()
 
 	LoadCursors();
 
-	//! hide the cursor until we are ingame (hide it during loading screen etc.)
-	SDL_ShowCursor(SDL_DISABLE);
-
 #ifndef __APPLE__
-	hardwareCursor = !!configHandler->Get("HardwareCursor", 0);
+	hardwareCursor = configHandler->GetBool("HardwareCursor");
 #endif
 
 	soundMultiselID = sound->GetSoundId("MultiSelect", false);
 
-	invertMouse = !!configHandler->Get("InvertMouse",0);
-	doubleClickTime = configHandler->Get("DoubleClickTime", 200.0f) / 1000.0f;
+	invertMouse = configHandler->GetBool("InvertMouse");
+	doubleClickTime = configHandler->GetFloat("DoubleClickTime") / 1000.0f;
 
-	scrollWheelSpeed = configHandler->Get("ScrollWheelSpeed", 25.0f);
-	scrollWheelSpeed = Clamp(scrollWheelSpeed,-255.f,255.f);
+	scrollWheelSpeed = configHandler->GetFloat("ScrollWheelSpeed");
 
-	crossSize      = configHandler->Get("CrossSize", 12.0f);
-	crossAlpha     = configHandler->Get("CrossAlpha", 0.5f);
-	crossMoveScale = configHandler->Get("CrossMoveScale", 1.0f) * 0.005f;
+	crossSize      = configHandler->GetFloat("CrossSize");
+	crossAlpha     = configHandler->GetFloat("CrossAlpha");
+	crossMoveScale = configHandler->GetFloat("CrossMoveScale") * 0.005f;
 
-	dragScrollThreshold = configHandler->Get("MouseDragScrollThreshold", 0.3f);
+	dragScrollThreshold = configHandler->GetFloat("MouseDragScrollThreshold");
 
 	configHandler->NotifyOnChange(this);
 }
@@ -148,8 +159,8 @@ void CMouseHandler::LoadCursors()
 	AssignMouseCursor("Centroid",     "cursorcentroid",   mCenter,  false);
 	AssignMouseCursor("DeathWait",    "cursordwatch",     mCenter,  false);
 	AssignMouseCursor("DeathWait",    "cursorwait",       mCenter,  false); // backup
-	AssignMouseCursor("DGun",         "cursordgun",       mCenter,  false);
-	AssignMouseCursor("DGun",         "cursorattack",     mCenter,  false); // backup
+	AssignMouseCursor("ManualFire",   "cursordgun",       mCenter,  false); // FIXME
+	AssignMouseCursor("ManualFire",   "cursorattack",     mCenter,  false); // backup
 	AssignMouseCursor("Fight",        "cursorfight",      mCenter,  false);
 	AssignMouseCursor("Fight",        "cursorattack",     mCenter,  false); // backup
 	AssignMouseCursor("GatherWait",   "cursorgather",     mCenter,  false);
@@ -212,7 +223,7 @@ void CMouseHandler::MouseMove(int x, int y, int dx, int dy)
 		activeReceiver->MouseMove(x, y, dx, dy, activeButton);
 	}
 
-	if (inMapDrawer && inMapDrawer->keyPressed){
+	if (inMapDrawer && inMapDrawer->IsDrawMode()){
 		inMapDrawer->MouseMove(x, y, dx, dy, activeButton);
 	}
 
@@ -250,7 +261,7 @@ void CMouseHandler::MousePress(int x, int y, int button)
 	if (activeReceiver && activeReceiver->MousePress(x, y, button))
 		return;
 
-	if(inMapDrawer &&  inMapDrawer->keyPressed){
+	if(inMapDrawer && inMapDrawer->IsDrawMode()){
 		inMapDrawer->MousePress(x, y, button);
 		return;
 	}
@@ -303,8 +314,6 @@ void CMouseHandler::MousePress(int x, int y, int button)
 
 void CMouseHandler::MouseRelease(int x, int y, int button)
 {
-	GML_RECMUTEX_LOCK(sel); // MouseRelease
-
 	if (button > NUM_BUTTONS)
 		return;
 
@@ -313,7 +322,7 @@ void CMouseHandler::MouseRelease(int x, int y, int button)
 	dir = hide ? camera->forward: camera->CalcPixelDir(x, y);
 	buttons[button].pressed = false;
 
-	if (inMapDrawer && inMapDrawer->keyPressed){
+	if (inMapDrawer && inMapDrawer->IsDrawMode()){
 		inMapDrawer->MouseRelease(x, y, button);
 		return;
 	}
@@ -325,6 +334,8 @@ void CMouseHandler::MouseRelease(int x, int y, int button)
 		return;
 	}
 
+	GML_RECMUTEX_LOCK(sel); // MouseRelease
+
 	//! Switch camera mode on a middle click that wasn't a middle mouse drag scroll.
 	//! the latter is determined by the time the mouse was held down:
 	//! switch (dragScrollThreshold)
@@ -336,23 +347,29 @@ void CMouseHandler::MouseRelease(int x, int y, int button)
 		return;
 	}
 
-	if (gu->directControl) {
+	if (gu->fpsMode) {
 		return;
 	}
 
 	if ((button == SDL_BUTTON_LEFT) && !buttons[button].chorded) {
-		if (!keys[SDLK_LSHIFT] && !keys[SDLK_LCTRL]) {
+		if (!keyInput->IsKeyPressed(SDLK_LSHIFT) && !keyInput->IsKeyPressed(SDLK_LCTRL)) {
 			selectedUnits.ClearSelected();
 		}
 
 		if (buttons[SDL_BUTTON_LEFT].movement > 4) {
 			// select box
-			float dist=ground->LineGroundCol(buttons[SDL_BUTTON_LEFT].camPos,buttons[SDL_BUTTON_LEFT].camPos+buttons[SDL_BUTTON_LEFT].dir*globalRendering->viewRange*1.4f);
+			float dist = ground->LineGroundCol(
+				buttons[SDL_BUTTON_LEFT].camPos,
+				buttons[SDL_BUTTON_LEFT].camPos +
+				buttons[SDL_BUTTON_LEFT].dir * globalRendering->viewRange * 1.4f,
+				false
+			);
+
 			if(dist<0)
 				dist=globalRendering->viewRange*1.4f;
 			float3 pos1=buttons[SDL_BUTTON_LEFT].camPos+buttons[SDL_BUTTON_LEFT].dir*dist;
 
-			dist=ground->LineGroundCol(camera->pos,camera->pos+dir*globalRendering->viewRange*1.4f);
+			dist = ground->LineGroundCol(camera->pos, camera->pos + dir * globalRendering->viewRange * 1.4f, false);
 			if(dist<0)
 				dist=globalRendering->viewRange*1.4f;
 			float3 pos2=camera->pos+dir*dist;
@@ -417,8 +434,8 @@ void CMouseHandler::MouseRelease(int x, int y, int button)
 			for (; team <= lastTeam; team++) {
 				for(ui=teamHandler->Team(team)->units.begin();ui!=teamHandler->Team(team)->units.end();++ui){
 					float3 vec=(*ui)->midPos-camera->pos;
-					if(vec.dot(norm1)<0 && vec.dot(norm2)<0 && vec.dot(norm3)<0 && vec.dot(norm4)<0){
-						if (keys[SDLK_LCTRL] && selectedUnits.selectedUnits.find(*ui) != selectedUnits.selectedUnits.end()) {
+					if (vec.dot(norm1) < 0.0f && vec.dot(norm2) < 0.0f && vec.dot(norm3) < 0.0f && vec.dot(norm4) < 0.0f) {
+						if (keyInput->IsKeyPressed(SDLK_LCTRL) && selectedUnits.selectedUnits.find(*ui) != selectedUnits.selectedUnits.end()) {
 							selectedUnits.RemoveUnit(*ui);
 						} else {
 							selectedUnits.AddUnit(*ui);
@@ -428,30 +445,34 @@ void CMouseHandler::MouseRelease(int x, int y, int button)
 					}
 				}
 			}
+
+			#if (PLAY_SOUNDS == 1)
 			if (addedunits == 1) {
-				int soundIdx = unit->unitDef->sounds.select.getRandomIdx();
+				const int soundIdx = unit->unitDef->sounds.select.getRandomIdx();
 				if (soundIdx >= 0) {
 					Channels::UnitReply.PlaySample(
 						unit->unitDef->sounds.select.getID(soundIdx), unit,
 						unit->unitDef->sounds.select.getVolume(soundIdx));
 				}
 			}
-			else if(addedunits) //more than one unit selected
+			else if (addedunits) //more than one unit selected
 				Channels::UserInterface.PlaySample(soundMultiselID);
+			#endif
 		} else {
-			const CUnit* unit;
-			helper->GuiTraceRay(camera->pos,dir,globalRendering->viewRange*1.4f,unit,false);
+			CUnit* unit;
+			CFeature* feature;
+			TraceRay::GuiTraceRay(camera->pos, dir, globalRendering->viewRange*1.4f, false, NULL, unit, feature);
 			if (unit && ((unit->team == gu->myTeam) || gu->spectatingFullSelect)) {
 				if (buttons[button].lastRelease < (gu->gameTime - doubleClickTime)) {
 					CUnit* unitM = uh->units[unit->id];
-					if (keys[SDLK_LCTRL] && selectedUnits.selectedUnits.find((CUnit*)unit) != selectedUnits.selectedUnits.end()) {
+					if (keyInput->IsKeyPressed(SDLK_LCTRL) && selectedUnits.selectedUnits.find((CUnit*)unit) != selectedUnits.selectedUnits.end()) {
 						selectedUnits.RemoveUnit(unitM);
 					} else {
 						selectedUnits.AddUnit(unitM);
 					}
 				} else {
 					//double click
-					if (unit->group && !keys[SDLK_LCTRL]) {
+					if (unit->group && !keyInput->IsKeyPressed(SDLK_LCTRL)) {
 						//select the current unit's group if it has one
 						selectedUnits.SelectGroup(unit->group->id);
 					} else {
@@ -468,9 +489,10 @@ void CMouseHandler::MouseRelease(int x, int y, int button)
 							CUnitSet::iterator ui;
 							CUnitSet& teamUnits = teamHandler->Team(team)->units;
 							for (ui = teamUnits.begin(); ui != teamUnits.end(); ++ui) {
-								if (((*ui)->aihint == unit->aihint) &&
-										(camera->InView((*ui)->midPos) || keys[SDLK_LCTRL])) {
-									selectedUnits.AddUnit(*ui);
+								if ((*ui)->unitDef->id == unit->unitDef->id) {
+									if (camera->InView((*ui)->midPos) || keyInput->IsKeyPressed(SDLK_LCTRL)) {
+										selectedUnits.AddUnit(*ui);
+									}
 								}
 							}
 						}
@@ -478,12 +500,14 @@ void CMouseHandler::MouseRelease(int x, int y, int button)
 				}
 				buttons[button].lastRelease=gu->gameTime;
 
-				int soundIdx = unit->unitDef->sounds.select.getRandomIdx();
+				#if (PLAY_SOUNDS == 1)
+				const int soundIdx = unit->unitDef->sounds.select.getRandomIdx();
 				if (soundIdx >= 0) {
 					Channels::UnitReply.PlaySample(
 						unit->unitDef->sounds.select.getID(soundIdx), unit,
 						unit->unitDef->sounds.select.getVolume(soundIdx));
 				}
+				#endif
 			}
 		}
 	}
@@ -507,21 +531,25 @@ void CMouseHandler::DrawSelectionBox()
 		return;
 	}
 
-	if (gu->directControl) {
+	if (gu->fpsMode) {
 		return;
 	}
 	if (buttons[SDL_BUTTON_LEFT].pressed && !buttons[SDL_BUTTON_LEFT].chorded &&
 	   (buttons[SDL_BUTTON_LEFT].movement > 4) &&
-	   (!inMapDrawer || !inMapDrawer->keyPressed)) {
+	   (!inMapDrawer || !inMapDrawer->IsDrawMode())) {
+
+		float dist = ground->LineGroundCol(
+			buttons[SDL_BUTTON_LEFT].camPos,
+			buttons[SDL_BUTTON_LEFT].camPos +
+			buttons[SDL_BUTTON_LEFT].dir * globalRendering->viewRange * 1.4f,
+			false
+		);
 
-		float dist=ground->LineGroundCol(buttons[SDL_BUTTON_LEFT].camPos,
-		                                 buttons[SDL_BUTTON_LEFT].camPos
-		                                 + buttons[SDL_BUTTON_LEFT].dir*globalRendering->viewRange*1.4f);
 		if(dist<0)
 			dist=globalRendering->viewRange*1.4f;
 		float3 pos1=buttons[SDL_BUTTON_LEFT].camPos+buttons[SDL_BUTTON_LEFT].dir*dist;
 
-		dist=ground->LineGroundCol(camera->pos,camera->pos+dir*globalRendering->viewRange*1.4f);
+		dist = ground->LineGroundCol(camera->pos, camera->pos + dir * globalRendering->viewRange * 1.4f, false);
 		if(dist<0)
 			dist=globalRendering->viewRange*1.4f;
 		float3 pos2=camera->pos+dir*dist;
@@ -603,32 +631,24 @@ std::string CMouseHandler::GetCurrentTooltip(void)
 		return buildTip;
 	}
 
-	GML_RECMUTEX_LOCK(sel); // GetCurrentTooltip - anti deadlock
-	GML_RECMUTEX_LOCK(quad); // GetCurrentTooltip - called from ToolTipConsole::Draw --> MouseHandler::GetCurrentTooltip
-
 	const float range = (globalRendering->viewRange * 1.4f);
-	const CUnit* unit = NULL;
-	float udist = helper->GuiTraceRay(camera->pos, dir, range, unit, true);
-	const CFeature* feature = NULL;
-	float fdist = helper->GuiTraceRayFeature(camera->pos, dir, range, feature);
 
-	if ((udist > (range - 300.0f)) &&
-	    (fdist > (range - 300.0f)) && (unit == NULL)) {
-		return "";
-	}
+	CUnit* unit;
+	CFeature* feature;
+	float dist;
+	{
+		GML_THRMUTEX_LOCK(unit, GML_DRAW); // GetCurrentTooltip
+		GML_THRMUTEX_LOCK(feat, GML_DRAW); // GetCurrentTooltip
 
-	if (udist > fdist) {
-		unit = NULL;
-	} else {
-		feature = NULL;
-	}
+		dist = TraceRay::GuiTraceRay(camera->pos, dir, range, true, NULL, unit, feature);
 
-	if (unit) {
-		return CTooltipConsole::MakeUnitString(unit);
-	}
+		if (unit) {
+			return CTooltipConsole::MakeUnitString(unit);
+		}
 
-	if (feature) {
-		return CTooltipConsole::MakeFeatureString(feature);
+		if (feature) {
+			return CTooltipConsole::MakeFeatureString(feature);
+		}
 	}
 
 	const string selTip = selectedUnits.GetTooltip();
@@ -636,8 +656,8 @@ std::string CMouseHandler::GetCurrentTooltip(void)
 		return selTip;
 	}
 
-	if (udist < (range - 100.0f)) {
-		const float3 pos = camera->pos + (dir * udist);
+	if (dist <= range) {
+		const float3 pos = camera->pos + (dir * dist);
 		return CTooltipConsole::MakeGroundString(pos);
 	}
 
@@ -864,7 +884,7 @@ void CMouseHandler::DrawCursor()
 			glTranslatef(0.5f - globalRendering->pixelX * 0.5f, 0.5f - globalRendering->pixelY * 0.5f, 0.f);
 			glScalef(xscale, yscale, 1.f);
 
-			if (gu->directControl) {
+			if (gu->fpsMode) {
 				DrawFPSCursor();
 			} else {
 				DrawScrollCursor();
@@ -912,7 +932,7 @@ bool CMouseHandler::AssignMouseCursor(const std::string& cmdName,
 {
 	std::map<std::string, CMouseCursor*>::iterator cmdIt;
 	cmdIt = cursorCommandMap.find(cmdName);
-	const bool haveCmd  = (cmdIt  != cursorCommandMap.end());
+	const bool haveCmd = (cmdIt != cursorCommandMap.end());
 
 	std::map<std::string, CMouseCursor*>::iterator fileIt;
 	fileIt = cursorFileMap.find(fileName);
