@@ -1,11 +1,10 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 
 #include <set>
 #include <cctype>
 
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "LuaGaia.h"
 
@@ -24,10 +23,9 @@
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitDef.h"
 #include "Sim/Units/CommandAI/Command.h"
-#include "LogOutput.h"
-#include "FileSystem/FileHandler.h"
-#include "FileSystem/FileSystem.h"
-#include "Util.h"
+#include "System/FileSystem/FileHandler.h"
+#include "System/FileSystem/FileSystem.h"
+#include "System/Util.h"
 
 
 CLuaGaia* luaGaia = NULL;
@@ -47,7 +45,7 @@ void CLuaGaia::LoadHandler()
 
 	new CLuaGaia();
 
-	if (luaGaia->L == NULL) {
+	if (!luaGaia->IsValid()) {
 		delete luaGaia;
 	}
 }
@@ -67,18 +65,18 @@ CLuaGaia::CLuaGaia()
 {
 	luaGaia = this;
 
-	if (L == NULL) {
+	if (!IsValid()) {
 		return;
 	}
 
 	teamsLocked = true;
 
-	fullCtrl = true;
-	fullRead = true;
-	ctrlTeam = AllAccessTeam; //teamHandler->GaiaTeamID();
-	readTeam = AllAccessTeam;
-	readAllyTeam = AllAccessTeam;
-	selectTeam = teamHandler->GaiaTeamID();
+	SetFullCtrl(true, true);
+	SetFullRead(true, true);
+	SetCtrlTeam(AllAccessTeam, true); //teamHandler->GaiaTeamID();
+	SetReadTeam(AllAccessTeam, true);
+	SetReadAllyTeam(AllAccessTeam, true);
+	SetSelectTeam(teamHandler->GaiaTeamID(), true);
 
 	Init(LuaGaiaSyncedFilename, LuaGaiaUnsyncedFilename, SPRING_VFS_MAP);
 }
@@ -86,7 +84,7 @@ CLuaGaia::CLuaGaia()
 
 CLuaGaia::~CLuaGaia()
 {
-	if (L != NULL) {
+	if (L_Sim != NULL || L_Draw != NULL) {
 		Shutdown();
 		KillLua();
 	}
@@ -94,15 +92,15 @@ CLuaGaia::~CLuaGaia()
 }
 
 
-bool CLuaGaia::AddSyncedCode()
+bool CLuaGaia::AddSyncedCode(lua_State *L)
 {
 	return true;
 }
 
 
-bool CLuaGaia::AddUnsyncedCode()
+bool CLuaGaia::AddUnsyncedCode(lua_State *L)
 {
-	/*lua_pushstring(L, "UNSYNCED");
+	/*lua_pushliteral(L, "UNSYNCED");
 	lua_gettable(L, LUA_REGISTRYINDEX);*/
 
 	return true;
