@@ -1,11 +1,10 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 
 #include <set>
 #include <cctype>
 
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "LuaRules.h"
 
@@ -31,9 +30,9 @@
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitDef.h"
 #include "Sim/Units/UnitHandler.h"
-#include "Sim/Units/COB/CobInstance.h"
+#include "Sim/Units/Scripts/CobInstance.h"
 #include "Sim/Weapons/Weapon.h"
-#include "System/LogOutput.h"
+#include "System/Log/ILog.h"
 #include "System/FileSystem/FileHandler.h"
 #include "System/FileSystem/FileSystem.h"
 
@@ -58,7 +57,7 @@ void CLuaRules::LoadHandler()
 
 	new CLuaRules();
 
-	if (luaRules->L == NULL) {
+	if (!luaRules->IsValid()) {
 		delete luaRules;
 	}
 }
@@ -78,66 +77,66 @@ CLuaRules::CLuaRules()
 {
 	luaRules = this;
 
-	if (L == NULL) {
+	if (!IsValid()) {
 		return;
 	}
 
-	fullCtrl = true;
-	fullRead = true;
-	ctrlTeam = AllAccessTeam;
-	readTeam = AllAccessTeam;
-	readAllyTeam = AllAccessTeam;
-	selectTeam = AllAccessTeam;
+	SetFullCtrl(true, true);
+	SetFullRead(true, true);
+	SetCtrlTeam(AllAccessTeam, true);
+	SetReadTeam(AllAccessTeam, true);
+	SetReadAllyTeam(AllAccessTeam, true);
+	SetSelectTeam(AllAccessTeam, true);
 
 	Init(LuaRulesSyncedFilename, LuaRulesUnsyncedFilename, SPRING_VFS_MOD);
 
-	if (L == NULL) {
+	if (!IsValid()) {
 		return;
 	}
 
-	haveCommandFallback        = HasCallIn("CommandFallback");
-	haveAllowCommand           = HasCallIn("AllowCommand");
-	haveAllowUnitCreation      = HasCallIn("AllowUnitCreation");
-	haveAllowUnitTransfer      = HasCallIn("AllowUnitTransfer");
-	haveAllowUnitBuildStep     = HasCallIn("AllowUnitBuildStep");
-	haveAllowFeatureCreation   = HasCallIn("AllowFeatureCreation");
-	haveAllowFeatureBuildStep  = HasCallIn("AllowFeatureBuildStep");
-	haveAllowResourceLevel     = HasCallIn("AllowResourceLevel");
-	haveAllowResourceTransfer  = HasCallIn("AllowResourceTransfer");
-	haveAllowDirectUnitControl = HasCallIn("AllowDirectUnitControl");
-	haveAllowStartPosition     = HasCallIn("AllowStartPosition");
-	haveMoveCtrlNotify         = HasCallIn("MoveCtrlNotify");
-	haveTerraformComplete      = HasCallIn("TerraformComplete");
-	haveDrawUnit               = HasCallIn("DrawUnit");
-	haveDrawFeature            = HasCallIn("DrawFeature");
-	haveAICallIn               = HasCallIn("AICallIn");
-	haveUnitPreDamaged         = HasCallIn("UnitPreDamaged");
-	haveShieldPreDamaged       = HasCallIn("ShieldPreDamaged");
+	BEGIN_ITERATE_LUA_STATES();
 
-	SetupUnsyncedFunction("DrawUnit");
-	SetupUnsyncedFunction("DrawFeature");
-	SetupUnsyncedFunction("AICallIn");
+	haveCommandFallback        = HasCallIn(L, "CommandFallback");
+	haveAllowCommand           = HasCallIn(L, "AllowCommand");
+	haveAllowUnitCreation      = HasCallIn(L, "AllowUnitCreation");
+	haveAllowUnitTransfer      = HasCallIn(L, "AllowUnitTransfer");
+	haveAllowUnitBuildStep     = HasCallIn(L, "AllowUnitBuildStep");
+	haveAllowFeatureCreation   = HasCallIn(L, "AllowFeatureCreation");
+	haveAllowFeatureBuildStep  = HasCallIn(L, "AllowFeatureBuildStep");
+	haveAllowResourceLevel     = HasCallIn(L, "AllowResourceLevel");
+	haveAllowResourceTransfer  = HasCallIn(L, "AllowResourceTransfer");
+	haveAllowDirectUnitControl = HasCallIn(L, "AllowDirectUnitControl");
+	haveAllowStartPosition     = HasCallIn(L, "AllowStartPosition");
+
+	haveMoveCtrlNotify         = HasCallIn(L, "MoveCtrlNotify");
+	haveTerraformComplete      = HasCallIn(L, "TerraformComplete");
+	haveAllowWeaponTargetCheck = HasCallIn(L, "AllowWeaponTargetCheck");
+	haveAllowWeaponTarget      = HasCallIn(L, "AllowWeaponTarget");
+	haveUnitPreDamaged         = HasCallIn(L, "UnitPreDamaged");
+	haveShieldPreDamaged       = HasCallIn(L, "ShieldPreDamaged");
+
+	haveDrawUnit    = HasCallIn(L, "DrawUnit");
+	haveDrawFeature = HasCallIn(L, "DrawFeature");
+
+	SetupUnsyncedFunction(L, "DrawUnit");
+	SetupUnsyncedFunction(L, "DrawFeature");
+	SetupUnsyncedFunction(L, "RecvSkirmishAIMessage");
+
+	END_ITERATE_LUA_STATES();
 }
 
 
 CLuaRules::~CLuaRules()
 {
-	if (L != NULL) {
+	if (L_Sim != NULL || L_Draw != NULL) {
 		Shutdown();
 		KillLua();
 	}
 	luaRules = NULL;
-
-	// clear all lods
-	std::list<CUnit*>::iterator it;
-	for (it = uh->activeUnits.begin(); it != uh->activeUnits.end(); ++it) {
-		CUnit* unit = *it;
-		unit->SetLODCount(0);
-	}
 }
 
 
-bool CLuaRules::AddSyncedCode()
+bool CLuaRules::AddSyncedCode(lua_State *L)
 {
 	lua_getglobal(L, "Script");
 	LuaPushNamedCFunc(L, "PermitHelperAIs", PermitHelperAIs);
@@ -147,14 +146,14 @@ bool CLuaRules::AddSyncedCode()
 }
 
 
-bool CLuaRules::AddUnsyncedCode()
+bool CLuaRules::AddUnsyncedCode(lua_State *L)
 {
-	lua_pushstring(L, "UNSYNCED");
+	lua_pushliteral(L, "UNSYNCED");
 	lua_gettable(L, LUA_REGISTRYINDEX);
 
-	lua_pushstring(L, "Spring");
+	lua_pushliteral(L, "Spring");
 	lua_rawget(L, -2);
-	lua_pushstring(L, "UnitRendering");
+	lua_pushliteral(L, "UnitRendering");
 	lua_newtable(L);
 	LuaUnitRendering::PushEntries(L);
 	lua_rawset(L, -3);
@@ -172,50 +171,76 @@ bool CLuaRules::AddUnsyncedCode()
 // LuaRules Call-Ins
 //
 
-bool CLuaRules::SyncedUpdateCallIn(const string& name)
+bool CLuaRules::SyncedUpdateCallIn(lua_State *L, const string& name)
 {
-	if (name == "CommandFallback") {
-		haveCommandFallback        = HasCallIn("CommandFallback");
-	} else if (name == "AllowCommand") {
-		haveAllowCommand           = HasCallIn("AllowCommand");
-	} else if (name == "AllowUnitCreation") {
-		haveAllowUnitCreation      = HasCallIn("AllowUnitCreation");
-	} else if (name == "AllowUnitTransfer") {
-		haveAllowUnitTransfer      = HasCallIn("AllowUnitTransfer");
-	} else if (name == "AllowUnitBuildStep") {
-		haveAllowUnitBuildStep     = HasCallIn("AllowUnitBuildStep");
-	} else if (name == "AllowFeatureCreation") {
-		haveAllowFeatureCreation   = HasCallIn("AllowFeatureCreation");
-	} else if (name == "AllowFeatureBuildStep") {
-		haveAllowFeatureBuildStep  = HasCallIn("AllowFeatureBuildStep");
-	} else if (name == "AllowResourceLevel") {
-		haveAllowResourceLevel     = HasCallIn("AllowResourceLevel");
-	} else if (name == "AllowResourceTransfer") {
-		haveAllowResourceTransfer  = HasCallIn("AllowResourceTransfer");
-	} else if (name == "AllowDirectUnitControl") {
-		haveAllowDirectUnitControl = HasCallIn("AllowDirectUnitControl");
-	} else if (name == "AllowStartPosition") {
-		haveAllowStartPosition     = HasCallIn("AllowStartPosition");
-	} else if (name == "MoveCtrlNotify") {
-		haveMoveCtrlNotify         = HasCallIn("MoveCtrlNotify");
-	} else if (name == "TerraformComplete") {
-		haveTerraformComplete      = HasCallIn("TerraformComplete");
-	} else {
-		return CLuaHandleSynced::SyncedUpdateCallIn(name);
+	#define UPDATE_HAVE_CALLIN(callinName) \
+		have ## callinName = HasCallIn( L, #callinName );
+
+	     if (name == "CommandFallback"       ) { UPDATE_HAVE_CALLIN(CommandFallback); }
+	else if (name == "AllowCommand"          ) { UPDATE_HAVE_CALLIN(AllowCommand); }
+	else if (name == "AllowUnitCreation"     ) { UPDATE_HAVE_CALLIN(AllowUnitCreation); }
+	else if (name == "AllowUnitTransfer"     ) { UPDATE_HAVE_CALLIN(AllowUnitTransfer); }
+	else if (name == "AllowUnitBuildStep"    ) { UPDATE_HAVE_CALLIN(AllowUnitBuildStep); }
+	else if (name == "AllowFeatureCreation"  ) { UPDATE_HAVE_CALLIN(AllowFeatureCreation); }
+	else if (name == "AllowFeatureBuildStep" ) { UPDATE_HAVE_CALLIN(AllowFeatureBuildStep); }
+	else if (name == "AllowResourceLevel"    ) { UPDATE_HAVE_CALLIN(AllowResourceLevel); }
+	else if (name == "AllowResourceTransfer" ) { UPDATE_HAVE_CALLIN(AllowResourceTransfer); }
+	else if (name == "AllowDirectUnitControl") { UPDATE_HAVE_CALLIN(AllowDirectUnitControl); }
+	else if (name == "AllowStartPosition"    ) { UPDATE_HAVE_CALLIN(AllowStartPosition); }
+	else if (name == "MoveCtrlNotify"        ) { UPDATE_HAVE_CALLIN(MoveCtrlNotify); }
+	else if (name == "TerraformComplete"     ) { UPDATE_HAVE_CALLIN(TerraformComplete); }
+	else if (name == "UnitPreDamaged"        ) { UPDATE_HAVE_CALLIN(UnitPreDamaged); }
+	else if (name == "ShieldPreDamaged"      ) { UPDATE_HAVE_CALLIN(ShieldPreDamaged); }
+	else if (name == "AllowWeaponTargetCheck") { UPDATE_HAVE_CALLIN(AllowWeaponTargetCheck); }
+	else if (name == "AllowWeaponTarget"     ) { UPDATE_HAVE_CALLIN(AllowWeaponTarget); }
+	else {
+		return CLuaHandleSynced::SyncedUpdateCallIn(L, name);
 	}
+
+	#undef UPDATE_HAVE_CALLIN
 	return true;
 }
 
 
-bool CLuaRules::UnsyncedUpdateCallIn(const string& name)
+bool CLuaRules::UnsyncedUpdateCallIn(lua_State *L, const string& name)
 {
-	     if (name == "DrawUnit"   ) { haveDrawUnit    = HasCallIn("DrawUnit"   ); }
-	else if (name == "DrawFeature") { haveDrawFeature = HasCallIn("DrawFeature"); }
-	else if (name == "AICallIn"   ) { haveAICallIn    = HasCallIn("AICallIn"   ); }
+	     if (name == "DrawUnit"   ) { haveDrawUnit    = HasCallIn(L, "DrawUnit"   ); }
+	else if (name == "DrawFeature") { haveDrawFeature = HasCallIn(L, "DrawFeature"); }
 
-	return CLuaHandleSynced::UnsyncedUpdateCallIn(name);
+	return CLuaHandleSynced::UnsyncedUpdateCallIn(L, name);
 }
 
+/// pushes 7 items on the stack
+static void PushUnitAndCommand(lua_State* L, const CUnit* unit, const Command& cmd)
+{
+	// push the unit info
+	lua_pushnumber(L, unit->id);
+	lua_pushnumber(L, unit->unitDef->id);
+	lua_pushnumber(L, unit->team);
+
+	// push the command id
+	lua_pushnumber(L, cmd.GetID());
+
+	// push the params list
+	lua_newtable(L);
+	for (int p = 0; p < (int)cmd.params.size(); p++) {
+		lua_pushnumber(L, p + 1);
+		lua_pushnumber(L, cmd.params[p]);
+		lua_rawset(L, -3);
+	}
+
+	// push the options table
+	lua_newtable(L);
+	HSTR_PUSH_NUMBER(L, "coded", cmd.options);
+	HSTR_PUSH_BOOL(L, "alt",   !!(cmd.options & ALT_KEY));
+	HSTR_PUSH_BOOL(L, "ctrl",  !!(cmd.options & CONTROL_KEY));
+	HSTR_PUSH_BOOL(L, "shift", !!(cmd.options & SHIFT_KEY));
+	HSTR_PUSH_BOOL(L, "right", !!(cmd.options & RIGHT_MOUSE_KEY));
+	HSTR_PUSH_BOOL(L, "meta",  !!(cmd.options & META_KEY));
+
+	// push the command tag
+	lua_pushnumber(L, cmd.tag);
+}
 
 bool CLuaRules::CommandFallback(const CUnit* unit, const Command& cmd)
 {
@@ -230,34 +255,7 @@ bool CLuaRules::CommandFallback(const CUnit* unit, const Command& cmd)
 		return true; // the call is not defined
 	}
 
-	// push the unit info
-	lua_pushnumber(L, unit->id);
-	lua_pushnumber(L, unit->unitDef->id);
-	lua_pushnumber(L, unit->team);
-
-	// push the command id
-	lua_pushnumber(L, cmd.id);
-
-	// push the params list
-	lua_newtable(L);
-	for (int p = 0; p < (int)cmd.params.size(); p++) {
-		lua_pushnumber(L, p + 1);
-		lua_pushnumber(L, cmd.params[p]);
-		lua_rawset(L, -3);
-	}
-	HSTR_PUSH_NUMBER(L, "n", cmd.params.size());
-
-	// push the options table
-	lua_newtable(L);
-	HSTR_PUSH_NUMBER(L, "coded", cmd.options);
-	HSTR_PUSH_BOOL(L, "alt",   !!(cmd.options & ALT_KEY));
-	HSTR_PUSH_BOOL(L, "ctrl",  !!(cmd.options & CONTROL_KEY));
-	HSTR_PUSH_BOOL(L, "shift", !!(cmd.options & SHIFT_KEY));
-	HSTR_PUSH_BOOL(L, "right", !!(cmd.options & RIGHT_MOUSE_KEY));
-	HSTR_PUSH_BOOL(L, "meta",  !!(cmd.options & META_KEY));
-
-	// push the command tag
-	lua_pushnumber(L, cmd.tag);
+	PushUnitAndCommand(L, unit, cmd);
 
 	// call the function
 	if (!RunCallIn(cmdStr, 7, 1)) {
@@ -266,7 +264,7 @@ bool CLuaRules::CommandFallback(const CUnit* unit, const Command& cmd)
 
 	// get the results
 	if (!lua_isboolean(L, -1)) {
-		logOutput.Print("%s() bad return value\n", cmdStr.GetString().c_str());
+		LOG_L(L_WARNING, "%s() bad return value", cmdStr.GetString().c_str());
 		lua_pop(L, 1);
 		return true;
 	}
@@ -292,34 +290,7 @@ bool CLuaRules::AllowCommand(const CUnit* unit, const Command& cmd, bool fromSyn
 		return true; // the call is not defined
 	}
 
-	// push the unit info
-	lua_pushnumber(L, unit->id);
-	lua_pushnumber(L, unit->unitDef->id);
-	lua_pushnumber(L, unit->team);
-
-	// push the command id
-	lua_pushnumber(L, cmd.id);
-
-	// push the params list
-	lua_newtable(L);
-	for (int p = 0; p < (int)cmd.params.size(); p++) {
-		lua_pushnumber(L, p + 1);
-		lua_pushnumber(L, cmd.params[p]);
-		lua_rawset(L, -3);
-	}
-	HSTR_PUSH_NUMBER(L, "n", cmd.params.size());
-
-	// push the options table
-	lua_newtable(L);
-	HSTR_PUSH_NUMBER(L, "coded", cmd.options);
-	HSTR_PUSH_BOOL(L, "alt",   !!(cmd.options & ALT_KEY));
-	HSTR_PUSH_BOOL(L, "ctrl",  !!(cmd.options & CONTROL_KEY));
-	HSTR_PUSH_BOOL(L, "shift", !!(cmd.options & SHIFT_KEY));
-	HSTR_PUSH_BOOL(L, "right", !!(cmd.options & RIGHT_MOUSE_KEY));
-	HSTR_PUSH_BOOL(L, "meta",  !!(cmd.options & META_KEY));
-
-	// push the command tag
-	lua_pushnumber(L, cmd.tag);
+	PushUnitAndCommand(L, unit, cmd);
 
 	lua_pushboolean(L, fromSynced);
 
@@ -330,7 +301,7 @@ bool CLuaRules::AllowCommand(const CUnit* unit, const Command& cmd, bool fromSyn
 
 	// get the results
 	if (!lua_isboolean(L, -1)) {
-		logOutput.Print("%s() bad return value\n", cmdStr.GetString().c_str());
+		LOG_L(L_WARNING, "%s() bad return value", cmdStr.GetString().c_str());
 		lua_pop(L, 1);
 		return true;
 	}
@@ -370,7 +341,7 @@ bool CLuaRules::AllowUnitCreation(const UnitDef* unitDef,
 
 	// get the results
 	if (!lua_isboolean(L, -1)) {
-		logOutput.Print("%s() bad return value\n", cmdStr.GetString().c_str());
+		LOG_L(L_WARNING, "%s() bad return value", cmdStr.GetString().c_str());
 		lua_pop(L, 1);
 		return true;
 	}
@@ -407,7 +378,7 @@ bool CLuaRules::AllowUnitTransfer(const CUnit* unit, int newTeam, bool capture)
 
 	// get the results
 	if (!lua_isboolean(L, -1)) {
-		logOutput.Print("%s() bad return value\n", cmdStr.GetString().c_str());
+		LOG_L(L_WARNING, "%s() bad return value", cmdStr.GetString().c_str());
 		lua_pop(L, 1);
 		return true;
 	}
@@ -445,7 +416,7 @@ bool CLuaRules::AllowUnitBuildStep(const CUnit* builder,
 
 	// get the results
 	if (!lua_isboolean(L, -1)) {
-		logOutput.Print("%s() bad return value\n", cmdStr.GetString().c_str());
+		LOG_L(L_WARNING, "%s() bad return value", cmdStr.GetString().c_str());
 		lua_pop(L, 1);
 		return true;
 	}
@@ -483,7 +454,7 @@ bool CLuaRules::AllowFeatureCreation(const FeatureDef* featureDef,
 
 	// get the results
 	if (!lua_isboolean(L, -1)) {
-		logOutput.Print("%s() bad return value\n", cmdStr.GetString().c_str());
+		LOG_L(L_WARNING, "%s() bad return value", cmdStr.GetString().c_str());
 		lua_pop(L, 1);
 		return true;
 	}
@@ -521,7 +492,7 @@ bool CLuaRules::AllowFeatureBuildStep(const CUnit* builder,
 
 	// get the results
 	if (!lua_isboolean(L, -1)) {
-		logOutput.Print("%s() bad return value\n", cmdStr.GetString().c_str());
+		LOG_L(L_WARNING, "%s() bad return value", cmdStr.GetString().c_str());
 		lua_pop(L, 1);
 		return true;
 	}
@@ -546,7 +517,7 @@ bool CLuaRules::AllowResourceLevel(int teamID, const string& type, float level)
 	}
 
 	lua_pushnumber(L, teamID);
-	lua_pushstring(L, type.c_str());
+	lua_pushsstring(L, type);
 	lua_pushnumber(L, level);
 
 	// call the function
@@ -556,7 +527,7 @@ bool CLuaRules::AllowResourceLevel(int teamID, const string& type, float level)
 
 	// get the results
 	if (!lua_isboolean(L, -1)) {
-		logOutput.Print("%s() bad return value\n", cmdStr.GetString().c_str());
+		LOG_L(L_WARNING, "%s() bad return value", cmdStr.GetString().c_str());
 		lua_pop(L, 1);
 		return true;
 	}
@@ -583,7 +554,7 @@ bool CLuaRules::AllowResourceTransfer(int oldTeam, int newTeam,
 
 	lua_pushnumber(L, oldTeam);
 	lua_pushnumber(L, newTeam);
-	lua_pushstring(L, type.c_str());
+	lua_pushsstring(L, type);
 	lua_pushnumber(L, amount);
 
 	// call the function
@@ -593,7 +564,7 @@ bool CLuaRules::AllowResourceTransfer(int oldTeam, int newTeam,
 
 	// get the results
 	if (!lua_isboolean(L, -1)) {
-		logOutput.Print("%s() bad return value\n", cmdStr.GetString().c_str());
+		LOG_L(L_WARNING, "%s() bad return value", cmdStr.GetString().c_str());
 		lua_pop(L, 1);
 		return true;
 	}
@@ -629,7 +600,7 @@ bool CLuaRules::AllowDirectUnitControl(int playerID, const CUnit* unit)
 
 	// get the results
 	if (!lua_isboolean(L, -1)) {
-		logOutput.Print("%s() bad return value\n", cmdStr.GetString().c_str());
+		LOG_L(L_WARNING, "%s() bad return value", cmdStr.GetString().c_str());
 		lua_pop(L, 1);
 		return true;
 	}
@@ -666,7 +637,7 @@ bool CLuaRules::AllowStartPosition(int playerID, const float3& pos)
 
 	// get the results
 	if (!lua_isboolean(L, -1)) {
-		logOutput.Print("%s() bad return value\n", cmdStr.GetString().c_str());
+		LOG_L(L_WARNING, "%s() bad return value", cmdStr.GetString().c_str());
 		lua_pop(L, 1);
 		return true;
 	}
@@ -702,7 +673,7 @@ bool CLuaRules::MoveCtrlNotify(const CUnit* unit, int data)
 
 	// get the results
 	if (!lua_isboolean(L, -1)) {
-		logOutput.Print("%s() bad return value\n", cmdStr.GetString().c_str());
+		LOG_L(L_WARNING, "%s() bad return value", cmdStr.GetString().c_str());
 		lua_pop(L, 1);
 		return false;
 	}
@@ -744,7 +715,7 @@ bool CLuaRules::TerraformComplete(const CUnit* unit, const CUnit* build)
 
 	// get the results
 	if (!lua_isboolean(L, -1)) {
-		logOutput.Print("%s() bad return value\n", cmdStr.GetString().c_str());
+		LOG_L(L_WARNING, "%s() bad return value", cmdStr.GetString().c_str());
 		lua_pop(L, 1);
 		return false;
 	}
@@ -776,12 +747,12 @@ bool CLuaRules::UnitPreDamaged(const CUnit* unit, const CUnit* attacker,
 	LUA_CALL_IN_CHECK(L);
 	lua_checkstack(L, 11);
 
-	const int errfunc = SetupTraceback();
-
+	const int errfunc = SetupTraceback(L);
 	static const LuaHashString cmdStr("UnitPreDamaged");
+
 	if (!cmdStr.GetGlobalFunc(L)) {
 		// remove error handler
-		if (errfunc) lua_pop(L, 1);
+		if (errfunc) { lua_pop(L, 1); }
 		return false; // the call is not defined
 	}
 
@@ -791,7 +762,7 @@ bool CLuaRules::UnitPreDamaged(const CUnit* unit, const CUnit* attacker,
 	lua_pushnumber(L, unit->team);
 	lua_pushnumber(L, damage);
 	lua_pushboolean(L, paralyzer);
-	if (fullRead) {
+	if (GetFullRead(L)) {
 		lua_pushnumber(L, weaponID);
 		argCount += 1;
 		if (attacker != NULL) {
@@ -809,14 +780,18 @@ bool CLuaRules::UnitPreDamaged(const CUnit* unit, const CUnit* attacker,
 		*newDamage = lua_tonumber(L, -2);
 	} else if (!lua_isnumber(L, -2) || lua_isnil(L, -2)) {
 		// first value is obligatory, so may not be nil
-		logOutput.Print("%s(): 1st value returned should be a number (newDamage)\n", cmdStr.GetString().c_str());
+		LOG_L(L_WARNING,
+				"%s(): 1st value returned should be a number (newDamage)",
+				cmdStr.GetString().c_str());
 	}
 
 	if (impulseMult && lua_isnumber(L, -1)) {
 		*impulseMult = lua_tonumber(L, -1);
 	} else if (!lua_isnumber(L, -1) && !lua_isnil(L, -1)) {
 		// second value is optional, so nils are OK
-		logOutput.Print("%s(): 2nd value returned should be a number (impulseMult)\n", cmdStr.GetString().c_str());
+		LOG_L(L_WARNING,
+				"%s(): 2nd value returned should be a number (impulseMult)",
+				cmdStr.GetString().c_str());
 	}
 
 	lua_pop(L, 2);
@@ -836,15 +811,13 @@ bool CLuaRules::ShieldPreDamaged(
 	LUA_CALL_IN_CHECK(L);
 	lua_checkstack(L, 5 + 1);
 
-	const int errfunc(SetupTraceback());
+	const int errfunc(SetupTraceback(L));
 	static const LuaHashString cmdStr("ShieldPreDamaged");
 
 	bool ret = false;
 
 	if (!cmdStr.GetGlobalFunc(L)) {
-		if (errfunc) {
-			lua_pop(L, 1);
-		}
+		if (errfunc) { lua_pop(L, 1); }
 
 		// undefined call-in
 		return ret;
@@ -873,6 +846,82 @@ bool CLuaRules::ShieldPreDamaged(
 
 /******************************************************************************/
 
+int CLuaRules::AllowWeaponTargetCheck(unsigned int attackerID, unsigned int attackerWeaponNum, unsigned int attackerWeaponDefID)
+{
+	if (!haveAllowWeaponTargetCheck) {
+		return -1;
+	}
+
+	LUA_CALL_IN_CHECK(L);
+	lua_checkstack(L, 3 + 1);
+
+	const int errfunc(SetupTraceback(L));
+	static const LuaHashString cmdStr("AllowWeaponTargetCheck");
+
+	int ret = -1;
+
+	if (!cmdStr.GetGlobalFunc(L)) {
+		if (errfunc) { lua_pop(L, 1); }
+		return ret;
+	}
+
+	lua_pushnumber(L, attackerID);
+	lua_pushnumber(L, attackerWeaponNum);
+	lua_pushnumber(L, attackerWeaponDefID);
+
+	RunCallInTraceback(cmdStr, 3, 1, errfunc);
+
+	ret = (lua_isboolean(L, -1) && lua_toboolean(L, -1))? 1: 0;
+	lua_pop(L, -1);
+
+	return ret;
+}
+
+int CLuaRules::AllowWeaponTarget(
+	unsigned int attackerID,
+	unsigned int targetID,
+	unsigned int attackerWeaponNum,
+	unsigned int attackerWeaponDefID,
+	float* targetPriority)
+{
+	if (!haveAllowWeaponTarget) {
+		return -1;
+	}
+
+	LUA_CALL_IN_CHECK(L);
+	lua_checkstack(L, 4 + 2);
+
+	const int errfunc(SetupTraceback(L));
+	static const LuaHashString cmdStr("AllowWeaponTarget");
+
+	int ret = -1;
+
+	if (!cmdStr.GetGlobalFunc(L)) {
+		if (errfunc) { lua_pop(L, 1); }
+		return ret;
+	}
+
+	lua_pushnumber(L, attackerID);
+	lua_pushnumber(L, targetID);
+	lua_pushnumber(L, attackerWeaponNum);
+	lua_pushnumber(L, attackerWeaponDefID);
+
+	RunCallInTraceback(cmdStr, 4, 2, errfunc);
+
+	ret = (lua_isboolean(L, -2) && lua_toboolean(L, -2))? 1: 0;
+
+	if (targetPriority && lua_isnumber(L, -1)) {
+		*targetPriority = lua_tonumber(L, -1);
+	}
+
+	lua_pop(L, 2);
+	return ret;
+}
+
+/******************************************************************************/
+
+
+
 bool CLuaRules::DrawUnit(int unitID)
 {
 	if (!haveDrawUnit) {
@@ -901,7 +950,7 @@ bool CLuaRules::DrawUnit(int unitID)
 	}
 
 	if (!lua_isboolean(L, -1)) {
-		logOutput.Print("%s() bad return value\n", cmdStr.GetString().c_str());
+		LOG_L(L_WARNING, "%s() bad return value", cmdStr.GetString().c_str());
 		lua_pop(L, 1);
 		return false;
 	}
@@ -939,7 +988,7 @@ bool CLuaRules::DrawFeature(int featureID)
 	}
 
 	if (!lua_isboolean(L, -1)) {
-		logOutput.Print("%s() bad return value\n", cmdStr.GetString().c_str());
+		LOG_L(L_WARNING, "%s() bad return value", cmdStr.GetString().c_str());
 		lua_pop(L, 1);
 		return false;
 	}
@@ -951,49 +1000,6 @@ bool CLuaRules::DrawFeature(int featureID)
 
 
 
-const char* CLuaRules::AICallIn(const char* data, int inSize, int* outSize)
-{
-	if (!haveAICallIn) {
-		return NULL;
-	}
-
-	LUA_CALL_IN_CHECK(L);
-	lua_checkstack(L, 3);
-	static const LuaHashString cmdStr("AICallIn");
-	if (!cmdStr.GetRegistryFunc(L)) {
-		return NULL;
-	}
-
-	int argCount = 0;
-	if (data != NULL) {
-		if (inSize < 0) {
-			inSize = strlen(data);
-		}
-		lua_pushlstring(L, data, inSize);
-		argCount = 1;
-	}
-
-	if (!RunCallIn(cmdStr, argCount, 1)) {
-		return NULL;
-	}
-
-	if (!lua_isstring(L, -1)) {
-		lua_pop(L, 1);
-		return NULL;
-	}
-
-	size_t len;
-	const char* outData = lua_tolstring(L, -1, &len);
-	if (outSize != NULL) {
-		*outSize = len;
-	}
-
-	lua_pop(L, 1);
-
-	return outData;
-}
-
-
 /******************************************************************************/
 /******************************************************************************/
 
@@ -1018,8 +1024,8 @@ void CLuaRules::Cob2Lua(const LuaHashString& name, const CUnit* unit,
 {
 	static int callDepth = 0;
 	if (callDepth >= 16) {
-		logOutput.Print("CLuaRules::Cob2Lua() call overflow: %s\n",
-		                name.GetString().c_str());
+		LOG_L(L_WARNING, "CLuaRules::Cob2Lua() call overflow: %s",
+				name.GetString().c_str());
 		args[0] = 0; // failure
 		return;
 	}
@@ -1029,16 +1035,16 @@ void CLuaRules::Cob2Lua(const LuaHashString& name, const CUnit* unit,
 	const int top = lua_gettop(L);
 
 	if (!lua_checkstack(L, 1 + 3 + argsCount)) {
-		logOutput.Print("CLuaRules::Cob2Lua() lua_checkstack() error: %s\n",
-		                name.GetString().c_str());
+		LOG_L(L_WARNING, "CLuaRules::Cob2Lua() lua_checkstack() error: %s",
+				name.GetString().c_str());
 		args[0] = 0; // failure
 		lua_settop(L, top);
 		return;
 	}
 
 	if (!name.GetGlobalFunc(L)) {
-		logOutput.Print("CLuaRules::Cob2Lua() missing function: %s\n",
-		                name.GetString().c_str());
+		LOG_L(L_WARNING, "CLuaRules::Cob2Lua() missing function: %s",
+				name.GetString().c_str());
 		args[0] = 0; // failure
 		lua_settop(L, top);
 		return;
@@ -1113,11 +1119,8 @@ int CLuaRules::PermitHelperAIs(lua_State* L)
 		luaL_error(L, "Incorrect argument to PermitHelperAIs()");
 	}
 	gs->noHelperAIs = !lua_toboolean(L, 1);
-	if (gs->noHelperAIs) {
-		logOutput.Print("LuaRules has Disabled helper AIs");
-	} else {
-		logOutput.Print("LuaRules has Enabled helper AIs");
-	}
+	LOG("LuaRules has %s helper AIs",
+			(gs->noHelperAIs ? "disabled" : "enabled"));
 	return 0;
 }
 
