@@ -32,8 +32,8 @@ class CLuaRules : public CLuaHandleSynced
 		static void FreeHandler();
 
 	public: // call-ins
-		bool SyncedUpdateCallIn(const string& name);
-		bool UnsyncedUpdateCallIn(const string& name);
+		bool SyncedUpdateCallIn(lua_State *L, const string& name);
+		bool UnsyncedUpdateCallIn(lua_State *L, const string& name);
 
 		bool CommandFallback(const CUnit* unit, const Command& cmd);
 		bool AllowCommand(const CUnit* unit, const Command& cmd, bool fromSynced);
@@ -58,6 +58,17 @@ class CLuaRules : public CLuaHandleSynced
 		void Cob2Lua(const LuaHashString& funcName, const CUnit* unit,
 		             int& argsCount, int args[MAX_LUA_COB_ARGS]);
 
+		int AllowWeaponTargetCheck(
+			unsigned int attackerID,
+			unsigned int attackerWeaponNum,
+			unsigned int attackerWeaponDefID);
+		int AllowWeaponTarget(
+			unsigned int attackerID,
+			unsigned int targetID,
+			unsigned int attackerWeaponNum,
+			unsigned int attackerWeaponDefID,
+			float* targetPriority);
+
 		bool UnitPreDamaged(const CUnit* unit, const CUnit* attacker,
                              float damage, int weaponID, bool paralyzer,
                              float* newDamage, float* impulseMult);
@@ -67,15 +78,14 @@ class CLuaRules : public CLuaHandleSynced
 		// unsynced
 		bool DrawUnit(int unitID);
 		bool DrawFeature(int featureID);
-		const char* AICallIn(const char* data, int inSize, int* outSize);
 
 	private:
 		CLuaRules();
 		~CLuaRules();
 
 	protected:
-		bool AddSyncedCode();
-		bool AddUnsyncedCode();
+		bool AddSyncedCode(lua_State *L);
+		bool AddUnsyncedCode(lua_State *L);
 
 		int UnpackCobArg(lua_State* L);
 
@@ -96,11 +106,13 @@ class CLuaRules : public CLuaHandleSynced
 		bool haveAllowStartPosition;
 		bool haveMoveCtrlNotify;
 		bool haveTerraformComplete;
-		bool haveDrawUnit;
-		bool haveDrawFeature;
-		bool haveAICallIn;
 		bool haveUnitPreDamaged;
 		bool haveShieldPreDamaged;
+		bool haveAllowWeaponTargetCheck;
+		bool haveAllowWeaponTarget;
+
+		bool haveDrawUnit;
+		bool haveDrawFeature;
 
 	private:
 		static const int* currentCobArgs;
