@@ -1,13 +1,7 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 
-#include <string>
-#include <vector>
-using std::string;
-using std::vector;
-
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "LuaShaders.h"
 
@@ -16,11 +10,17 @@ using std::vector;
 #include "LuaHashString.h"
 #include "LuaHandle.h"
 #include "LuaOpenGL.h"
+#include "LuaOpenGLUtils.h"
 
 #include "Game/Camera.h"
 #include "Rendering/ShadowHandler.h"
-#include "LogOutput.h"
-#include "Util.h"
+#include "System/Log/ILog.h"
+#include "System/Util.h"
+
+#include <string>
+#include <vector>
+using std::string;
+using std::vector;
 
 
 int LuaShaders::activeShaderDepth = 0;
@@ -157,8 +157,8 @@ void LuaShaders::DeleteProgram(Program& p)
 
 int LuaShaders::GetShaderLog(lua_State* L)
 {
-	const LuaShaders& shaders = CLuaHandle::GetActiveShaders();
-	lua_pushstring(L, shaders.errorLog.c_str());
+	const LuaShaders& shaders = CLuaHandle::GetActiveShaders(L);
+	lua_pushsstring(L, shaders.errorLog);
 	return 1;
 }
 
@@ -320,7 +320,7 @@ static bool ParseUniformSetupTables(lua_State* L, int index, GLuint progName)
 /******************************************************************************/
 /******************************************************************************/
 
-static GLuint CompileObject(const vector<string>& sources, GLenum type,
+static GLuint CompileObject(lua_State *L, const vector<string>& sources, GLenum type,
                             bool& success)
 {
 	if (sources.empty()) {
@@ -330,7 +330,7 @@ static GLuint CompileObject(const vector<string>& sources, GLenum type,
 
 	GLuint obj = glCreateShader(type);
 	if (obj == 0) {
-		LuaShaders& shaders = CLuaHandle::GetActiveShaders();
+		LuaShaders& shaders = CLuaHandle::GetActiveShaders(L);
 		shaders.errorLog = "Could not create shader object";
 		return 0;
 	}
@@ -355,7 +355,7 @@ static GLuint CompileObject(const vector<string>& sources, GLenum type,
 		GLsizei logSize = sizeof(log);
 		glGetShaderInfoLog(obj, logSize, &logSize, log);
 		
-		LuaShaders& shaders = CLuaHandle::GetActiveShaders();
+		LuaShaders& shaders = CLuaHandle::GetActiveShaders(L);
 		shaders.errorLog = log;
 		if (shaders.errorLog.empty()) {
 			shaders.errorLog = "Empty error message:  code = "
@@ -398,7 +398,7 @@ static bool ParseSources(lua_State* L, int table,
 		}
 	}
 	else if (!lua_isnil(L, -1)) {
-		LuaShaders& shaders = CLuaHandle::GetActiveShaders();
+		LuaShaders& shaders = CLuaHandle::GetActiveShaders(L);
 		shaders.errorLog = "Invalid " + string(type) + " shader source";
 		lua_pop(L, 1);
 		return false;
@@ -409,10 +409,10 @@ static bool ParseSources(lua_State* L, int table,
 }
 
 
-static bool ApplyGeometryParameters(lua_State* L, int table, GLuint prog)
+static void ApplyGeometryParameters(lua_State* L, int table, GLuint prog)
 {
-	if (!glProgramParameteriEXT_NONGML) {
-		return true;
+	if (!IS_GL_FUNCTION_AVAILABLE(glProgramParameteriEXT)) {
+		return;
 	}
 
 	struct { const char* name; GLenum param; } parameters[] = {
@@ -430,8 +430,6 @@ static bool ApplyGeometryParameters(lua_State* L, int table, GLuint prog)
 		}
 		lua_pop(L, 1);
 	}
-
-	return true;
 }
 
 
@@ -455,16 +453,16 @@ int LuaShaders::CreateShader(lua_State* L)
 	}
 
 	bool success;
-	const GLuint vertObj = CompileObject(vertSrcs, GL_VERTEX_SHADER, success);
+	const GLuint vertObj = CompileObject(L, vertSrcs, GL_VERTEX_SHADER, success);
 	if (!success) {
 		return 0;
 	}
-	const GLuint geomObj = CompileObject(geomSrcs, GL_GEOMETRY_SHADER_EXT, success);
+	const GLuint geomObj = CompileObject(L, geomSrcs, GL_GEOMETRY_SHADER_EXT, success);
 	if (!success) {
 		glDeleteShader(vertObj);
 		return 0;
 	}
-	const GLuint fragObj = CompileObject(fragSrcs, GL_FRAGMENT_SHADER, success);
+	const GLuint fragObj = CompileObject(L, fragSrcs, GL_FRAGMENT_SHADER, success);
 	if (!success) {
 		glDeleteShader(vertObj);
 		glDeleteShader(geomObj);
@@ -492,7 +490,7 @@ int LuaShaders::CreateShader(lua_State* L)
 
 	glLinkProgram(prog);
 
-	LuaShaders& shaders = CLuaHandle::GetActiveShaders();
+	LuaShaders& shaders = CLuaHandle::GetActiveShaders(L);
 
 	GLint result;
 	glGetProgramiv(prog, GL_LINK_STATUS, &result);
@@ -523,7 +521,7 @@ int LuaShaders::DeleteShader(lua_State* L)
 		return 0;
 	}
 	const int progID = luaL_checkint(L, 1);
-	LuaShaders& shaders = CLuaHandle::GetActiveShaders();
+	LuaShaders& shaders = CLuaHandle::GetActiveShaders(L);
 	shaders.RemoveProgram(progID);
 	return 0;
 }
@@ -540,7 +538,7 @@ int LuaShaders::UseShader(lua_State* L)
 		return 1;
 	}
 		
-	LuaShaders& shaders = CLuaHandle::GetActiveShaders();
+	LuaShaders& shaders = CLuaHandle::GetActiveShaders(L);
 	const GLuint progName = shaders.GetProgramName(progID);
 	if (progName == 0) {
 		lua_pushboolean(L, false);
@@ -562,7 +560,7 @@ int LuaShaders::ActiveShader(lua_State* L)
 		progName = 0;
 	}
 	else {
-		LuaShaders& shaders = CLuaHandle::GetActiveShaders();
+		LuaShaders& shaders = CLuaHandle::GetActiveShaders(L);
 		progName = shaders.GetProgramName(progID);
 		if (progName == 0) {
 			return 0;
@@ -579,8 +577,8 @@ int LuaShaders::ActiveShader(lua_State* L)
 	glUseProgram(currentProgram);
 
 	if (error != 0) {
-		logOutput.Print("gl.ActiveShader: error(%i) = %s",
-		                error, lua_tostring(L, -1));
+		LOG_L(L_ERROR, "gl.ActiveShader: error(%i) = %s",
+				error, lua_tostring(L, -1));
 		lua_error(L);
 	}
 
@@ -630,7 +628,7 @@ static const char* UniformTypeString(GLenum type)
 
 int LuaShaders::GetActiveUniforms(lua_State* L)
 {
-	const LuaShaders& shaders = CLuaHandle::GetActiveShaders();
+	const LuaShaders& shaders = CLuaHandle::GetActiveShaders(L);
 	const GLuint progName = shaders.GetProgramName(L, 1);
 	if (progName == 0) {
 		return 0;
@@ -664,7 +662,7 @@ int LuaShaders::GetActiveUniforms(lua_State* L)
 
 int LuaShaders::GetUniformLocation(lua_State* L)
 {
-	const LuaShaders& shaders = CLuaHandle::GetActiveShaders();
+	const LuaShaders& shaders = CLuaHandle::GetActiveShaders(L);
 	const GLuint progName = shaders.GetProgramName(L, 1);
 	if (progName == 0) {
 		return 0;
@@ -766,16 +764,6 @@ int LuaShaders::UniformInt(lua_State* L)
 }
 
 
-static void UniformMatrix4dv(GLint location, const double dm[16])
-{
-	float fm[16];
-	for (int i = 0; i < 16; i++) {
-		fm[i] = (float)dm[i];
-	}
-	glUniformMatrix4fv(location, 1, GL_FALSE, fm);
-}
-
-
 int LuaShaders::UniformMatrix(lua_State* L)
 {
 	if (activeShaderDepth <= 0) {
@@ -790,22 +778,11 @@ int LuaShaders::UniformMatrix(lua_State* L)
 				luaL_error(L, "Incorrect arguments to gl.UniformMatrix()");
 			}
 			const string matName = lua_tostring(L, 2);
-			if (matName == "shadow") {
-				if (shadowHandler) {
-					glUniformMatrix4fv(location, 1, GL_FALSE,
-					                   shadowHandler->shadowMatrix.m);
-				}
-			}
-			else if (matName == "camera") {
-				UniformMatrix4dv(location, camera->GetViewMat());
-			}
-			else if (matName == "caminv") {
-				UniformMatrix4dv(location, camera->GetViewMatInv());
-			}
-			else if (matName == "camprj") {
-				UniformMatrix4dv(location, camera->GetProjMat());
-			}
-			else {
+			const CMatrix44f* mat = LuaOpenGLUtils::GetNamedMatrix(matName);
+
+			if (mat) {
+				glUniformMatrix4fv(location, 1, GL_FALSE, *mat);
+			} else {
 				luaL_error(L, "Incorrect arguments to gl.UniformMatrix()");
 			}
 			break;
@@ -848,7 +825,7 @@ int LuaShaders::SetShaderParameter(lua_State* L)
 		CheckDrawingEnabled(L, __FUNCTION__);
 	}
 
-	const LuaShaders& shaders = CLuaHandle::GetActiveShaders();
+	const LuaShaders& shaders = CLuaHandle::GetActiveShaders(L);
 	const GLuint progName = shaders.GetProgramName(L, 1);
 	if (progName == 0) {
 		return 0;
@@ -857,7 +834,7 @@ int LuaShaders::SetShaderParameter(lua_State* L)
 	const GLenum param = (GLenum)luaL_checkint(L, 2);
 	const GLint  value =  (GLint)luaL_checkint(L, 3);
 
-	if (glProgramParameteriEXT_NONGML) {
+	if (IS_GL_FUNCTION_AVAILABLE(glProgramParameteriEXT)) {
 		glProgramParameteriEXT(progName, param, value);
 	}
 
