@@ -1,7 +1,7 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#ifndef LUA_SYNCED_INFO_H
-#define LUA_SYNCED_INFO_H
+#ifndef LUA_SYNCED_READ_H
+#define LUA_SYNCED_READ_H
 
 struct lua_State;
 
@@ -84,13 +84,15 @@ class LuaSyncedRead {
 		static int GetUnitNearestEnemy(lua_State* L);
 
 		static int GetFeaturesInRectangle(lua_State* L);
+		static int GetFeaturesInSphere(lua_State* L);
+		static int GetFeaturesInCylinder(lua_State* L);
+		static int GetProjectilesInRectangle(lua_State* L);
 
 		static int ValidUnitID(lua_State* L);
 		static int GetUnitTooltip(lua_State* L);
 		static int GetUnitDefID(lua_State* L);
 		static int GetUnitTeam(lua_State* L);
 		static int GetUnitAllyTeam(lua_State* L);
-		static int GetUnitLineage(lua_State* L);
 		static int GetUnitNeutral(lua_State* L);
 		static int GetUnitHealth(lua_State* L);
 		static int GetUnitIsDead(lua_State* L);
@@ -164,6 +166,7 @@ class LuaSyncedRead {
 		static int GetFeatureResources(lua_State* L);
 		static int GetFeatureNoSelect(lua_State* L);
 		static int GetFeatureResurrect(lua_State* L);
+		static int GetFeatureCollisionVolumeData(lua_State* L);
 
 		static int GetProjectilePosition(lua_State* L);
 		static int GetProjectileVelocity(lua_State* L);
@@ -213,5 +216,4 @@ class LuaSyncedRead {
 		static int GetCOBGlobalVar(lua_State* L);
 };
 
-
-#endif /* LUA_SYNCED_INFO_H */
+#endif /* LUA_SYNCED_READ_H */
