@@ -1,6 +1,5 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 
 #include <set>
 #include <string>
@@ -9,7 +8,7 @@
 #include <map>
 #include <cctype>
 
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "LuaUnitDefs.h"
 
@@ -32,8 +31,7 @@
 #include "Sim/Misc/LosHandler.h"
 #include "Sim/Misc/QuadField.h"
 #include "Sim/Misc/Wind.h"
-#include "Sim/MoveTypes/AirMoveType.h"
-#include "Sim/MoveTypes/TAAirMoveType.h"
+#include "Sim/MoveTypes/MoveInfo.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitDef.h"
 #include "Sim/Units/UnitHandler.h"
@@ -44,16 +42,12 @@
 #include "Sim/Units/CommandAI/Command.h"
 #include "Sim/Units/CommandAI/CommandAI.h"
 #include "Sim/Units/CommandAI/FactoryCAI.h"
-#include "Sim/Units/CommandAI/LineDrawer.h"
 #include "Sim/Weapons/Weapon.h"
 #include "Sim/Weapons/WeaponDefHandler.h"
-#include "LogOutput.h"
-#include "FileSystem/FileHandler.h"
-#include "FileSystem/SimpleParser.h"
-#include "FileSystem/FileSystem.h"
-#include "Util.h"
-
-using namespace std;
+#include "System/FileSystem/FileHandler.h"
+#include "System/FileSystem/SimpleParser.h"
+#include "System/FileSystem/FileSystem.h"
+#include "System/Util.h"
 
 
 static ParamMap paramMap;
@@ -92,7 +86,7 @@ bool LuaUnitDefs::PushEntries(lua_State* L)
 
 	const map<string, int>& udMap = unitDefHandler->unitDefIDsByName;
 	map<string, int>::const_iterator udIt;
-	for (udIt = udMap.begin(); udIt != udMap.end(); udIt++) {
+	for (udIt = udMap.begin(); udIt != udMap.end(); ++udIt) {
 	  const UnitDef* ud = unitDefHandler->GetUnitDefByID(udIt->second);
 		if (ud == NULL) {
 	  	continue;
@@ -186,7 +180,7 @@ static int UnitDefIndex(lua_State* L)
 			return 1;
 		}
 		case STRING_TYPE: {
-			lua_pushstring(L, ((string*)p)->c_str());
+			lua_pushsstring(L, *((string*)p));
 			return 1;
 		}
 		case FUNCTION_TYPE: {
@@ -316,9 +310,9 @@ static int WeaponDefToID(lua_State* L, const void* data)
 static int SafeIconType(lua_State* L, const void* data)
 {
 	// the iconType is unsynced because LuaUI has SetUnitDefIcon()
-	if (CLuaHandle::GetActiveHandle()->GetUserMode()) {
-		const CIcon& iconType = *((const CIcon*)data);
-		lua_pushstring(L, iconType->GetName().c_str());
+	if (!CLuaHandle::GetSynced(L)) {
+		const icon::CIcon& iconType = *((const icon::CIcon*)data);
+		lua_pushsstring(L, iconType->GetName());
 		return 1;
 	}
 	return 0;
@@ -331,8 +325,8 @@ static int CustomParamsTable(lua_State* L, const void* data)
 	lua_newtable(L);
 	map<string, string>::const_iterator it;
 	for (it = params.begin(); it != params.end(); ++it) {
-		lua_pushstring(L, it->first.c_str());
-		lua_pushstring(L, it->second.c_str());
+		lua_pushsstring(L, it->first);
+		lua_pushsstring(L, it->second);
 		lua_rawset(L, -3);
 	}
 	return 1;
@@ -356,7 +350,6 @@ static int BuildOptions(lua_State* L, const void* data)
 			lua_rawset(L, -3);
 		}
 	}
-	HSTR_PUSH_NUMBER(L, "n", count);
 	return 1;
 }
 
@@ -366,7 +359,7 @@ static inline int BuildCategorySet(lua_State* L, const vector<string>& cats)
 	lua_newtable(L);
 	const int count = (int)cats.size();
 	for (int i = 0; i < count; i++) {
-		lua_pushstring(L, cats[i].c_str());
+		lua_pushsstring(L, cats[i]);
 		lua_pushboolean(L, true);
 		lua_rawset(L, -3);
 	}
@@ -377,7 +370,7 @@ static inline int BuildCategorySet(lua_State* L, const vector<string>& cats)
 static int CategorySetFromBits(lua_State* L, const void* data)
 {
 	const int bits = *((const int*)data);
-	const vector<string> cats =
+	const vector<string> &cats =
 		CCategoryHandler::Instance()->GetCategoryNames(bits);
 	return BuildCategorySet(L, cats);
 }
@@ -387,22 +380,22 @@ static int CategorySetFromString(lua_State* L, const void* data)
 {
 	const string& str = *((const string*)data);
 	const string lower = StringToLower(str);
-	const vector<string> cats = CSimpleParser::Tokenize(lower, 0);
+	const vector<string> &cats = CSimpleParser::Tokenize(lower, 0);
 	return BuildCategorySet(L, cats);
 }
 
 
 static int WeaponsTable(lua_State* L, const void* data)
 {
-	const vector<UnitDef::UnitDefWeapon>& weapons =
-		*((const vector<UnitDef::UnitDefWeapon>*)data);
+	const vector<UnitDefWeapon>& weapons =
+		*((const vector<UnitDefWeapon>*)data);
 
 	const int weaponCount = (int)weapons.size();
 
 	lua_newtable(L);
 
 	for (int i = 0; i < weaponCount; i++) {
-		const UnitDef::UnitDefWeapon& udw = weapons[i];
+		const UnitDefWeapon& udw = weapons[i];
 		const WeaponDef* weapon = udw.def;
 		lua_pushnumber(L, i + 1);
 		lua_newtable(L); {
@@ -424,7 +417,6 @@ static int WeaponsTable(lua_State* L, const void* data)
 		}
 		lua_rawset(L, -3);
 	}
-	HSTR_PUSH_NUMBER(L, "n", weaponCount);
 
 	return 1;
 }
@@ -434,7 +426,7 @@ static void PushGuiSoundSet(lua_State* L, const string& name,
                             const GuiSoundSet& soundSet)
 {
 	const int soundCount = (int)soundSet.sounds.size();
-	lua_pushstring(L, name.c_str());
+	lua_pushsstring(L, name);
 	lua_newtable(L);
 	for (int i = 0; i < soundCount; i++) {
 		lua_pushnumber(L, i + 1);
@@ -442,12 +434,11 @@ static void PushGuiSoundSet(lua_State* L, const string& name,
 		const GuiSoundSet::Data& sound = soundSet.sounds[i];
 		HSTR_PUSH_STRING(L, "name",   sound.name);
 		HSTR_PUSH_NUMBER(L, "volume", sound.volume);
-		if (CLuaHandle::GetActiveHandle()->GetUserMode()) {
+		if (!CLuaHandle::GetSynced(L)) {
 			HSTR_PUSH_NUMBER(L, "id", sound.id);
 		}
 		lua_rawset(L, -3);
 	}
-	HSTR_PUSH_NUMBER(L, "n", soundCount);
 	lua_rawset(L, -3);
 }
 
@@ -526,7 +517,8 @@ static int MoveDataTable(lua_State* L, const void* data)
 		default: { HSTR_PUSH_STRING(L, "family", "error"); break; }
 	}
 
-	HSTR_PUSH_NUMBER(L, "size",          md->size);
+	HSTR_PUSH_NUMBER(L, "xsize",         md->xsize);
+	HSTR_PUSH_NUMBER(L, "zsize",         md->zsize);
 	HSTR_PUSH_NUMBER(L, "depth",         md->depth);
 	HSTR_PUSH_NUMBER(L, "maxSlope",      md->maxSlope);
 	HSTR_PUSH_NUMBER(L, "slopeMod",      md->slopeMod);
@@ -557,28 +549,28 @@ static int TotalEnergyOut(lua_State* L, const void* data)
 }
 
 
-#define TYPE_STRING_FUNC(name)                          \
-	static int Is ## name(lua_State* L, const void* data) \
-	{                                                     \
-		const string& type = *((const string*)data);        \
-		lua_pushboolean(L, type == #name);                  \
-		return 1;                                           \
+
+#define TYPE_FUNC(FuncName, LuaType)                    \
+	static int FuncName(lua_State* L, const void* data) \
+	{                                                   \
+		const UnitDef* ud = (const UnitDef*) data;      \
+		lua_push ## LuaType(L, ud->FuncName());         \
+		return 1;                                       \
 	}
 
-TYPE_STRING_FUNC(Bomber);
-TYPE_STRING_FUNC(Builder);
-TYPE_STRING_FUNC(Building);
-TYPE_STRING_FUNC(Factory);
-TYPE_STRING_FUNC(Fighter);
-TYPE_STRING_FUNC(Transport);
-TYPE_STRING_FUNC(GroundUnit);
-TYPE_STRING_FUNC(MetalExtractor);
+TYPE_FUNC(GetTypeString, string);
+TYPE_FUNC(IsBuildingUnit, boolean);
+TYPE_FUNC(IsFactoryUnit, boolean);
+TYPE_FUNC(IsFighterUnit, boolean);
+TYPE_FUNC(IsBomberUnit, boolean);
+TYPE_FUNC(IsGroundUnit, boolean);
+
 
 
 #define TYPE_MODEL_FUNC(name, param)                  \
 	static int name(lua_State* L, const void* data)   \
 	{                                                 \
-		const UnitDef* ud = ((const UnitDef*) data);  \
+		const UnitDef* ud = (const UnitDef*) data;    \
 		const S3DModel* model = ud->LoadModel();      \
 		lua_pushnumber(L, model->param);              \
 		return 1;                                     \
@@ -636,14 +628,12 @@ ADD_BOOL("canAttackWater",  canAttackWater); // CUSTOM
 	ADD_FUNCTION("stockpileWeaponDef", ud.stockpileWeaponDef, WeaponDefToID);
 	ADD_FUNCTION("iconType",           ud.iconType,           SafeIconType);
 
-	ADD_FUNCTION("isBomber",         ud.type, IsBomber);
-	ADD_FUNCTION("isBuilder",        ud.type, IsBuilder);
-	ADD_FUNCTION("isBuilding",       ud.type, IsBuilding);
-	ADD_FUNCTION("isFactory",        ud.type, IsFactory);
-	ADD_FUNCTION("isFighter",        ud.type, IsFighter);
-	ADD_FUNCTION("isTransport",      ud.type, IsTransport);
-	ADD_FUNCTION("isGroundUnit",     ud.type, IsGroundUnit);
-	ADD_FUNCTION("isMetalExtractor", ud.type, IsMetalExtractor);
+	ADD_FUNCTION("type",         ud, GetTypeString); // NOTE: deprecated, remove after 0.83.*
+	ADD_FUNCTION("isBuilding",   ud, IsBuildingUnit);
+	ADD_FUNCTION("isFactory",    ud, IsFactoryUnit);
+	ADD_FUNCTION("isFighter",    ud, IsFighterUnit);
+	ADD_FUNCTION("isBomber",     ud, IsBomberUnit);
+	ADD_FUNCTION("isGroundUnit", ud, IsGroundUnit);
 
 	ADD_FUNCTION("height",  ud, ModelHeight);
 	ADD_FUNCTION("radius",  ud, ModelRadius);
@@ -658,26 +648,19 @@ ADD_BOOL("canAttackWater",  canAttackWater); // CUSTOM
 	ADD_FUNCTION("maxz",    ud, ModelMaxz);
 
 	ADD_INT("id", ud.id);
+	ADD_INT("cobID", ud.cobID);
 
 	ADD_STRING("name",      ud.name);
 	ADD_STRING("humanName", ud.humanName);
-	ADD_STRING("filename",  ud.filename);
 
 	ADD_STRING("tooltip", ud.tooltip);
 
-	ADD_STRING("type", ud.type);
-
-	ADD_STRING("gaia", ud.gaia);
-
 	ADD_STRING("wreckName", ud.wreckName);
 	ADD_STRING("deathExplosion", ud.deathExplosion);
 	ADD_STRING("selfDExplosion", ud.selfDExplosion);
 
 	ADD_STRING("buildpicname", ud.buildPicName);
 
-	ADD_INT("aihint", ud.aihint);
-	ADD_INT("cobID",  ud.cobID);
-
 	ADD_INT("techLevel",   ud.techLevel);
 	ADD_INT("maxThisUnit", ud.maxThisUnit);
 
@@ -712,10 +695,9 @@ ADD_BOOL("canAttackWater",  canAttackWater); // CUSTOM
 	ADD_FLOAT("speed",    ud.speed);
 	ADD_FLOAT("turnRate", ud.turnRate);
 	ADD_BOOL("turnInPlace", ud.turnInPlace);
-	ADD_FLOAT("turnInPlaceDistance", ud.turnInPlaceDistance);
 	ADD_FLOAT("turnInPlaceSpeedLimit", ud.turnInPlaceSpeedLimit);
 
-	ADD_BOOL("upright",   ud.upright);
+	ADD_BOOL("upright", ud.upright);
 	ADD_BOOL("collide", ud.collide);
 
 	ADD_FLOAT("losHeight",     ud.losHeight);
@@ -735,7 +717,7 @@ ADD_BOOL("canAttackWater",  canAttackWater); // CUSTOM
 
 	ADD_FLOAT("mass", ud.mass);
 
-	ADD_FLOAT("maxSlope",      ud.maxSlope);
+	ADD_FLOAT("maxSlope",      ud.maxHeightDif); // NOTE: deprecated, remove after 0.83.*
 	ADD_FLOAT("maxHeightDif",  ud.maxHeightDif);
 	ADD_FLOAT("minWaterDepth", ud.minWaterDepth);
 	ADD_FLOAT("waterline",     ud.waterline);
@@ -762,7 +744,7 @@ ADD_BOOL("canAttackWater",  canAttackWater); // CUSTOM
 	ADD_FLOAT("buildDistance",  ud.buildDistance);
 	ADD_FLOAT("buildSpeed",     ud.buildSpeed);
 	ADD_FLOAT("repairSpeed",    ud.repairSpeed);
-	ADD_FLOAT("maxRepairSpeed",    ud.repairSpeed);
+	ADD_FLOAT("maxRepairSpeed", ud.repairSpeed);
 	ADD_FLOAT("reclaimSpeed",   ud.reclaimSpeed);
 	ADD_FLOAT("resurrectSpeed", ud.resurrectSpeed);
 	ADD_FLOAT("captureSpeed",   ud.captureSpeed);
@@ -774,6 +756,7 @@ ADD_BOOL("canAttackWater",  canAttackWater); // CUSTOM
 	ADD_BOOL("canHover",          ud.canhover);
 	ADD_BOOL("floater",           ud.floater);
 	ADD_BOOL("builder",           ud.builder);
+	ADD_BOOL("isBuilder",         ud.builder); // NOTE: deprecated, remove after 0.83.*
 	ADD_BOOL("onOffable",         ud.onoffable);
 	ADD_BOOL("activateWhenBuilt", ud.activateWhenBuilt);
 
@@ -781,7 +764,8 @@ ADD_BOOL("canAttackWater",  canAttackWater); // CUSTOM
 	ADD_BOOL("capturable",  ud.capturable);
 	ADD_BOOL("repairable",  ud.repairable);
 
-	ADD_BOOL("canDGun",               ud.canDGun);
+	ADD_BOOL("canDGun",               ud.canManualFire); // NOTE: deprecated, remove after 0.83.*
+	ADD_BOOL("canManualFire",         ud.canManualFire);
 	ADD_BOOL("canCloak",              ud.canCloak);
 	ADD_BOOL("canRestore",            ud.canRestore);
 	ADD_BOOL("canRepair",             ud.canRepair);
@@ -797,7 +781,7 @@ ADD_BOOL("canAttackWater",  canAttackWater); // CUSTOM
 	ADD_BOOL("canCapture",            ud.canCapture);
 	ADD_BOOL("canResurrect",          ud.canResurrect);
 	ADD_BOOL("canLoopbackAttack",     ud.canLoopbackAttack);
-	ADD_BOOL("canCrash",              ud.canCrash);
+	ADD_BOOL("canCrash",              ud.canLoopbackAttack); // NOTE: deprecated, remove after 0.83.*
 	ADD_BOOL("canFireControl",        ud.canFireControl);
 	ADD_INT( "fireState",             ud.fireState);
 	ADD_INT( "moveState",             ud.moveState);
@@ -847,8 +831,6 @@ ADD_BOOL("canAttackWater",  canAttackWater); // CUSTOM
 	ADD_INT("xsize", ud.xsize);
 	ADD_INT("zsize", ud.zsize);
 
-	ADD_INT("buildangle", ud.buildangle);
-
 	// transport stuff
 	ADD_INT(  "transportCapacity",     ud.transportCapacity);
 	ADD_INT(  "transportSize",         ud.transportSize);
@@ -861,8 +843,8 @@ ADD_BOOL("canAttackWater",  canAttackWater); // CUSTOM
 	ADD_BOOL( "cantBeTransported",     ud.cantBeTransported);
 	ADD_BOOL( "transportByEnemy",      ud.transportByEnemy);
 	ADD_INT(  "transportUnloadMethod", ud.transportUnloadMethod);
-	ADD_FLOAT("fallSpeed",			       ud.fallSpeed);
-	ADD_FLOAT("unitFallSpeed",		     ud.unitFallSpeed);
+	ADD_FLOAT("fallSpeed",             ud.fallSpeed);
+	ADD_FLOAT("unitFallSpeed",         ud.unitFallSpeed);
 
 	ADD_BOOL( "startCloaked",     ud.startCloaked);
 	ADD_FLOAT("cloakCost",        ud.cloakCost);
@@ -906,8 +888,6 @@ ADD_BOOL("canAttackWater",  canAttackWater); // CUSTOM
 	ADD_INT(  "flareSalvoSize",   ud.flareSalvoSize);
 	ADD_INT(  "flareSalvoDelay",  ud.flareSalvoDelay);
 
-	ADD_BOOL("smoothAnim", ud.smoothAnim);
-
 	ADD_BOOL("levelGround", ud.levelGround);
 	ADD_BOOL("strafeToAttack", ud.strafeToAttack);
 
@@ -923,10 +903,6 @@ ADD_BOOL("canAttackWater",  canAttackWater); // CUSTOM
 	ADD_FLOAT("nanoColorG",   ud.nanoColor.y);
 	ADD_FLOAT("nanoColorB",   ud.nanoColor.z);
 
-//	vector<CExplosionGenerator*>  sfxExplGens;
-	ADD_STRING("pieceTrailCEGTag",   ud.pieceTrailCEGTag);
-	ADD_INT(   "pieceTrailCEGRange", ud.pieceTrailCEGRange);
-
 	ADD_STRING("scriptName", ud.scriptName);
 	ADD_STRING("scriptPath", ud.scriptPath);
 
