@@ -1,7 +1,6 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include <set>
 #include <string>
@@ -17,18 +16,16 @@
 #include "LuaDefs.h"
 #include "LuaHandle.h"
 #include "LuaUtils.h"
+#include "Game/TraceRay.h"
 #include "Sim/Misc/CategoryHandler.h"
 #include "Sim/Misc/DamageArrayHandler.h"
 #include "Sim/Projectiles/Projectile.h"
 #include "Sim/Weapons/Weapon.h"
 #include "Sim/Weapons/WeaponDefHandler.h"
-#include "FileSystem/SimpleParser.h"
-#include "LogOutput.h"
-#include "Util.h"
+#include "System/FileSystem/SimpleParser.h"
+#include "System/Util.h"
 #include "Sim/Misc/GlobalSynced.h"
 
-using namespace std;
-
 
 static ParamMap paramMap;
 
@@ -67,7 +64,7 @@ bool LuaWeaponDefs::PushEntries(lua_State* L)
 
 	const map<string, int>& weaponMap = weaponDefHandler->weaponID;
 	map<string, int>::const_iterator wit;
-	for (wit = weaponMap.begin(); wit != weaponMap.end(); wit++) {
+	for (wit = weaponMap.begin(); wit != weaponMap.end(); ++wit) {
 		const WeaponDef* wd = &weaponDefHandler->weaponDefs[wit->second];
 		if (wd == NULL) {
 	  	continue;
@@ -152,7 +149,7 @@ static int WeaponDefIndex(lua_State* L)
 			return 1;
 		}
 		case STRING_TYPE: {
-			lua_pushstring(L, ((string*)p)->c_str());
+			lua_pushsstring(L, *((string*)p));
 			return 1;
 		}
 		case FUNCTION_TYPE: {
@@ -315,34 +312,50 @@ static int VisualsTable(lua_State* L, const void* data)
 }
 
 
-static int NoFeatureCollide(lua_State* L, const void* data)
+
+static int NoEnemyCollide(lua_State* L, const void* data)
 {
 	const int bits = *((const int*) data);
-	lua_pushboolean(L, (bits & COLLISION_NOFEATURE));
+	lua_pushboolean(L, (bits & Collision::NOENEMIES));
 	return 1;
 }
 
 static int NoFriendlyCollide(lua_State* L, const void* data)
 {
 	const int bits = *((const int*) data);
-	lua_pushboolean(L, (bits & COLLISION_NOFRIENDLY));
+	lua_pushboolean(L, (bits & Collision::NOFRIENDLIES));
+	return 1;
+}
+
+static int NoFeatureCollide(lua_State* L, const void* data)
+{
+	const int bits = *((const int*) data);
+	lua_pushboolean(L, (bits & Collision::NOFEATURES));
 	return 1;
 }
 
 static int NoNeutralCollide(lua_State* L, const void* data)
 {
 	const int bits = *((const int*) data);
-	lua_pushboolean(L, (bits & COLLISION_NONEUTRAL));
+	lua_pushboolean(L, (bits & Collision::NONEUTRALS));
 	return 1;
 }
 
+static int NoGroundCollide(lua_State* L, const void* data)
+{
+	const int bits = *((const int*) data);
+	lua_pushboolean(L, (bits & Collision::NOGROUND));
+	return 1;
+}
+
+
 
 static inline int BuildCategorySet(lua_State* L, const vector<string>& cats)
 {
 	lua_newtable(L);
 	const int count = (int)cats.size();
 	for (int i = 0; i < count; i++) {
-		lua_pushstring(L, cats[i].c_str());
+		lua_pushsstring(L, cats[i]);
 		lua_pushboolean(L, true);
 		lua_rawset(L, -3);
 	}
@@ -353,7 +366,7 @@ static inline int BuildCategorySet(lua_State* L, const vector<string>& cats)
 static int CategorySetFromBits(lua_State* L, const void* data)
 {
 	const int bits = *((const int*)data);
-	const vector<string> cats =
+	const vector<string> &cats =
 		CCategoryHandler::Instance()->GetCategoryNames(bits);
 	return BuildCategorySet(L, cats);
 }
@@ -363,7 +376,7 @@ static int CategorySetFromBits(lua_State* L, const void* data)
 {
 	const string& str = *((const string*)data);
 	const string lower = StringToLower(str);
-	const vector<string> cats = CSimpleParser::Tokenize(lower, 0);
+	const vector<string> &cats = CSimpleParser::Tokenize(lower, 0);
 	return BuildCategorySet(L, cats);
 }*/
 
@@ -374,8 +387,8 @@ static int CustomParamsTable(lua_State* L, const void* data)
 	lua_newtable(L);
 	map<string, string>::const_iterator it;
 	for (it = params.begin(); it != params.end(); ++it) {
-		lua_pushstring(L, it->first.c_str());
-		lua_pushstring(L, it->second.c_str());
+		lua_pushsstring(L, it->first);
+		lua_pushsstring(L, it->second);
 		lua_rawset(L, -3);
 	}
 	return 1;
@@ -393,12 +406,19 @@ static int GuiSoundSetTable(lua_State* L, const void* data)
 		const GuiSoundSet::Data& sound = soundSet.sounds[i];
 		HSTR_PUSH_STRING(L, "name",   sound.name);
 		HSTR_PUSH_NUMBER(L, "volume", sound.volume);
-		if (CLuaHandle::GetActiveHandle()->GetUserMode()) {
+		if (!CLuaHandle::GetSynced(L)) {
 			HSTR_PUSH_NUMBER(L, "id", sound.id);
 		}
 		lua_rawset(L, -3);
 	}
-	HSTR_PUSH_NUMBER(L, "n", soundCount);
+	return 1;
+}
+
+
+static int DeprecatedMaxVelocity(lua_State* L, const void* data)
+{
+	const float projspeed = *((const float*) data);
+	lua_pushnumber(L, projspeed * GAME_SPEED);
 	return 1;
 }
 
@@ -421,17 +441,20 @@ static bool InitParamMap()
 	ADD_FUNCTION("fireSound", wd.firesound, GuiSoundSetTable);
 
 	ADD_FUNCTION("customParams",         wd.customParams,   CustomParamsTable);
-	ADD_FUNCTION("noFeatureCollide",     wd.collisionFlags, NoFeatureCollide);
+	ADD_FUNCTION("noEnemyCollide",       wd.collisionFlags, NoEnemyCollide);
 	ADD_FUNCTION("noFriendlyCollide",    wd.collisionFlags, NoFriendlyCollide);
+	ADD_FUNCTION("noFeatureCollide",     wd.collisionFlags, NoFeatureCollide);
 	ADD_FUNCTION("noNeutralCollide",     wd.collisionFlags, NoNeutralCollide);
+	ADD_FUNCTION("noGroundCollide",      wd.collisionFlags, NoGroundCollide);
 	ADD_FUNCTION("onlyTargetCategories", wd.onlyTargetCategory, CategorySetFromBits);
 
+	ADD_FUNCTION("maxVelocity", wd.projectilespeed, DeprecatedMaxVelocity); // NOTE: deprecated (needs to be linked via _FUNCTION, else it would be writeable)
+
 	ADD_INT("id", wd.id);
 
 	ADD_INT("tdfId", wd.tdfId);
 
 	ADD_STRING("name",        wd.name);
-	ADD_STRING("filename",    wd.filename);
 	ADD_STRING("description", wd.description);
 	ADD_STRING("cegTag",      wd.cegTag);
 
@@ -477,13 +500,11 @@ static bool InitParamMap()
 
 	ADD_FLOAT("metalCost",  wd.metalcost);
 	ADD_FLOAT("energyCost", wd.energycost);
-	ADD_FLOAT("supplyCost", wd.supplycost);
 
 	ADD_BOOL("turret", wd.turret);
 	ADD_BOOL("onlyForward", wd.onlyForward);
 	ADD_BOOL("waterWeapon", wd.waterweapon);
 	ADD_BOOL("tracks", wd.tracks);
-	ADD_BOOL("dropped", wd.dropped);
 	ADD_BOOL("paralyzer", wd.paralyzer);
 
 	ADD_BOOL("noAutoTarget",   wd.noAutoTarget);
@@ -510,7 +531,6 @@ static bool InitParamMap()
 	ADD_FLOAT("startvelocity", wd.startvelocity);
 	ADD_FLOAT("weaponAcceleration", wd.weaponacceleration);
 	ADD_FLOAT("turnRate", wd.turnrate);
-	ADD_FLOAT("maxVelocity", wd.maxvelocity);
 
 	ADD_FLOAT("projectilespeed", wd.projectilespeed);
 	ADD_FLOAT("explosionSpeed", wd.explosionSpeed);
@@ -558,8 +578,6 @@ static bool InitParamMap()
 	ADD_FLOAT("heightBoostFactor",  wd.heightBoostFactor);
 	ADD_FLOAT("proximityPriority",  wd.proximityPriority);
 
-//	CExplosionGenerator *explosionGenerator;
-
 	ADD_BOOL("sweepFire", wd.sweepFire);
 
 	ADD_BOOL("canAttackGround", wd.canAttackGround);
