@@ -1,35 +1,37 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "BaseGroundDrawer.h"
 
 #include "Game/Camera.h"
+#include "Game/GlobalUnsynced.h"
 #include "Game/SelectedUnits.h"
 #include "Game/UI/GuiHandler.h"
 #include "Ground.h"
 #include "HeightLinePalette.h"
+#include "MetalMap.h"
 #include "ReadMap.h"
 #include "MapInfo.h"
+#include "Rendering/IPathDrawer.h"
+#include "Rendering/Env/ITreeDrawer.h"
 #include "Sim/Misc/LosHandler.h"
 #include "Sim/Misc/RadarHandler.h"
-#include "Sim/MoveTypes/MoveInfo.h"
-#include "Sim/MoveTypes/MoveMath/MoveMath.h"
-#include "Sim/Path/PathManager.h"
-#include "Sim/Units/UnitDef.h"
-#include "Sim/Units/UnitDefHandler.h"
-#include "Sim/Units/UnitHandler.h"
-#include "ConfigHandler.h"
-#include "FastMath.h"
-#include "myMath.h"
-#include "Rendering/Env/BaseTreeDrawer.h"
+#include "System/Config/ConfigHandler.h"
+#include "System/FastMath.h"
+#include "System/myMath.h"
+
+CONFIG(float, GroundLODScaleReflection).defaultValue(1.0f);
+CONFIG(float, GroundLODScaleRefraction).defaultValue(1.0f);
+CONFIG(float, GroundLODScaleUnitReflection).defaultValue(1.0f);
+CONFIG(bool, HighResLos).defaultValue(false);
+CONFIG(int, ExtraTextureUpdateRate).defaultValue(45);
 
 CBaseGroundDrawer::CBaseGroundDrawer(void)
 {
-	LODScaleReflection = configHandler->Get("GroundLODScaleReflection", 1.0f);
-	LODScaleRefraction = configHandler->Get("GroundLODScaleRefraction", 1.0f);
-	LODScaleUnitReflection = configHandler->Get("GroundLODScaleUnitReflection", 1.0f);
+	LODScaleReflection = configHandler->GetFloat("GroundLODScaleReflection");
+	LODScaleRefraction = configHandler->GetFloat("GroundLODScaleRefraction");
+	LODScaleUnitReflection = configHandler->GetFloat("GroundLODScaleUnitReflection");
 
 	infoTexAlpha = 0.25f;
 	infoTex = 0;
@@ -38,10 +40,18 @@ CBaseGroundDrawer::CBaseGroundDrawer(void)
 	drawLineOfSight = false;
 	drawRadarAndJammer = true;
 	wireframe = false;
+	advShading = false;
+	highResInfoTex = false;
+	updateTextureState = 0;
 
-	extraTex = 0;
-	extraTexPal = 0;
-	extractDepthMap = 0;
+	extraTex = NULL;
+	extraTexPal = NULL;
+	extractDepthMap = NULL;
+
+#ifdef USE_GML
+	multiThreadDrawGroundShadow = false;
+	multiThreadDrawGround = false;
+#endif
 
 	extraTexPBO.Bind();
 	extraTexPBO.Resize(gs->pwr2mapx * gs->pwr2mapy * 4);
@@ -49,8 +59,8 @@ CBaseGroundDrawer::CBaseGroundDrawer(void)
 
 	highResInfoTexWanted = false;
 
-	highResLosTex = !!configHandler->Get("HighResLos", 0);
-	extraTextureUpdateRate = std::max(4, configHandler->Get("ExtraTextureUpdateRate", 45) - 1);
+	highResLosTex = configHandler->GetBool("HighResLos");
+	extraTextureUpdateRate = std::max(4, configHandler->GetInt("ExtraTextureUpdateRate") - 1);
 
 	jamColor[0] = (int)(losColorScale * 0.25f);
 	jamColor[1] = (int)(losColorScale * 0.0f);
@@ -69,6 +79,7 @@ CBaseGroundDrawer::CBaseGroundDrawer(void)
 	alwaysColor[2] = (int)(losColorScale * 0.25f);
 
 	heightLinePal = new CHeightLinePalette();
+	groundTextures = NULL;
 }
 
 
@@ -130,18 +141,20 @@ void CBaseGroundDrawer::DrawTrees(bool drawReflection) const
 
 
 
-//todo: this part of extra textures is a mess really ...
+// XXX this part of extra textures is a mess really ...
 void CBaseGroundDrawer::DisableExtraTexture()
 {
-	if(drawLineOfSight){
+	if (drawLineOfSight) {
 		SetDrawMode(drawLos);
 	} else {
 		SetDrawMode(drawNormal);
 	}
-	extraTex=0;
-	highResInfoTexWanted=false;
-	updateTextureState=0;
-	while(!UpdateExtraTexture());
+
+	extraTex = 0;
+	highResInfoTexWanted = false;
+	updateTextureState = 0;
+
+	while (!UpdateExtraTexture());
 }
 
 
@@ -150,86 +163,111 @@ void CBaseGroundDrawer::SetHeightTexture()
 	if (drawMode == drawHeight)
 		DisableExtraTexture();
 	else {
-		SetDrawMode (drawHeight);
-		highResInfoTexWanted=true;
-		extraTex=0;
-		updateTextureState=0;
-		while(!UpdateExtraTexture());
+		SetDrawMode(drawHeight);
+
+		highResInfoTexWanted = true;
+		extraTex = 0;
+		updateTextureState = 0;
+
+		while (!UpdateExtraTexture());
 	}
 }
 
 
-void CBaseGroundDrawer::SetMetalTexture(unsigned char* tex,float* extractMap,unsigned char* pal,bool highRes)
+
+void CBaseGroundDrawer::SetMetalTexture(const CMetalMap* map)
 {
 	if (drawMode == drawMetal)
 		DisableExtraTexture();
 	else {
-		SetDrawMode (drawMetal);
+		SetDrawMode(drawMetal);
 
-		highResInfoTexWanted=false;
-		extraTex=tex;
-		extraTexPal=pal;
-		extractDepthMap=extractMap;
-		updateTextureState=0;
-		while(!UpdateExtraTexture());
+		highResInfoTexWanted = false;
+		extraTex = &map->metalMap[0];
+		extraTexPal = map->metalPal;
+		extractDepthMap = &map->extractionMap[0];
+		updateTextureState = 0;
+
+		while (!UpdateExtraTexture());
 	}
 }
 
 
-void CBaseGroundDrawer::SetPathMapTexture()
+void CBaseGroundDrawer::TogglePathTraversabilityTexture()
 {
-	if (drawMode == drawPath) {
+	if (drawMode == drawPathTraversability) {
 		DisableExtraTexture();
 	} else {
-		SetDrawMode(drawPath);
+		SetDrawMode(drawPathTraversability);
+
 		extraTex = 0;
 		highResInfoTexWanted = false;
 		updateTextureState = 0;
-		while(!UpdateExtraTexture());
+
+		while (!UpdateExtraTexture());
 	}
 }
 
+void CBaseGroundDrawer::TogglePathHeatTexture()
+{
+	if (drawMode == drawPathHeat) {
+		DisableExtraTexture();
+	} else {
+		SetDrawMode(drawPathHeat);
+
+		extraTex = 0;
+		highResInfoTexWanted = false;
+		updateTextureState = 0;
+
+		while (!UpdateExtraTexture());
+	}
+}
+
+void CBaseGroundDrawer::TogglePathCostTexture()
+{
+	if (drawMode == drawPathCost) {
+		DisableExtraTexture();
+	} else {
+		SetDrawMode(drawPathCost);
+		extraTex = 0;
+		highResInfoTexWanted = false;
+		updateTextureState = 0;
+
+		while (!UpdateExtraTexture());
+	}
+}
+
+
 
 void CBaseGroundDrawer::ToggleLosTexture()
 {
-	if (drawMode==drawLos) {
-		drawLineOfSight=false;
+	if (drawMode == drawLos) {
+		drawLineOfSight = false;
 		DisableExtraTexture();
 	} else {
-		drawLineOfSight=true;
+		drawLineOfSight = true;
+
 		SetDrawMode(drawLos);
-		extraTex=0;
-		highResInfoTexWanted=highResLosTex;
-		updateTextureState=0;
-		while(!UpdateExtraTexture());
+		extraTex = 0;
+		highResInfoTexWanted = highResLosTex;
+		updateTextureState = 0;
+
+		while (!UpdateExtraTexture());
 	}
 }
 
 
 void CBaseGroundDrawer::ToggleRadarAndJammer()
 {
-	drawRadarAndJammer=!drawRadarAndJammer;
-	if (drawMode==drawLos){
-		updateTextureState=0;
-		while(!UpdateExtraTexture());
-	}
-}
-
-
-void CBaseGroundDrawer::ToggleHeatMapTexture()
-{
-	if (drawMode == drawHeat) {
-		DisableExtraTexture();
-	} else {
-		SetDrawMode(drawHeat);
-		extraTex = 0;
-		highResInfoTexWanted = false;
+	drawRadarAndJammer = !drawRadarAndJammer;
+	if (drawMode == drawLos) {
 		updateTextureState = 0;
-		while(!UpdateExtraTexture());
+		while (!UpdateExtraTexture());
 	}
 }
 
 
+
 static inline int InterpolateLos(const unsigned short* p, int xsize, int ysize,
                                  int mip, int factor, int x, int y)
 {
@@ -268,8 +306,8 @@ bool CBaseGroundDrawer::UpdateExtraTexture()
 		return true;
 	}
 
-	const unsigned short* myLos         = &loshandler->losMap[gu->myAllyTeam].front();
-	const unsigned short* myAirLos      = &loshandler->airLosMap[gu->myAllyTeam].front();
+	const unsigned short* myLos         = &loshandler->losMaps[gu->myAllyTeam].front();
+	const unsigned short* myAirLos      = &loshandler->airLosMaps[gu->myAllyTeam].front();
 	const unsigned short* myRadar       = &radarhandler->radarMaps[gu->myAllyTeam].front();
 	const unsigned short* myJammer      = &radarhandler->jammerMaps[gu->myAllyTeam].front();
 #ifdef SONAR_JAMMER_MAPS
@@ -278,7 +316,7 @@ bool CBaseGroundDrawer::UpdateExtraTexture()
 #endif
 
 	if (updateTextureState < extraTextureUpdateRate) {
-		const int pwr2mapx_half = gs->pwr2mapx / 2;
+		const int pwr2mapx_half = gs->pwr2mapx >> 1;
 
 		int starty;
 		int endy;
@@ -286,6 +324,7 @@ bool CBaseGroundDrawer::UpdateExtraTexture()
 		GLbyte* infoTexMem;
 
 		extraTexPBO.Bind();
+
 		if (highResInfoTexWanted) {
 			starty = updateTextureState * gs->mapy / extraTextureUpdateRate;
 			endy = (updateTextureState + 1) * gs->mapy / extraTextureUpdateRate;
@@ -300,100 +339,19 @@ bool CBaseGroundDrawer::UpdateExtraTexture()
 			infoTexMem = (GLbyte*)extraTexPBO.MapBuffer(offset, (endy - starty) * pwr2mapx_half * 4);
 		}
 
-#define COLOR_R 2
-#define COLOR_G 1
-#define COLOR_B 0
-#define COLOR_A 3
+		switch (drawMode) {
+			case drawPathTraversability:
+			case drawPathHeat:
+			case drawPathCost: {
+				pathDrawer->UpdateExtraTexture(drawMode, starty, endy, offset, reinterpret_cast<unsigned char*>(infoTexMem));
+			} break;
 
-		switch(drawMode) {
-			case drawPath: {
-				if (guihandler->inCommand > 0 &&
-				    static_cast<size_t>(guihandler->inCommand) < guihandler->commands.size() &&
-				    guihandler->commands[guihandler->inCommand].type == CMDTYPE_ICON_BUILDING)
-				{
-					// use the current build order
-					for (int y = starty; y < endy; ++y) {
-						const int y_pwr2mapx_half = y*pwr2mapx_half;
-						const int y_16            = y*16;
-						for (int x = 0; x < gs->hmapx; ++x) {
-							float m;
-							if (!loshandler->InLos(float3(x*16+8, 0, y_16+8), gu->myAllyTeam)) {
-								m = 0.25f;
-							} else {
-								const UnitDef* unitdef = unitDefHandler->GetUnitDefByID(-guihandler->commands[guihandler->inCommand].id);
-								CFeature* f;
-
-								GML_RECMUTEX_LOCK(quad); // UpdateExtraTexture - testunitbuildsquare accesses features in the quadfield
-
-								if (uh->TestUnitBuildSquare(BuildInfo(unitdef, float3(x*16+8, 0, y_16+8), guihandler->buildFacing), f, gu->myAllyTeam)) {
-									if (f) {
-										m = 0.5f;
-									} else {
-										m = 1.0f;
-									}
-								} else {
-									m = 0.0f;
-								}
-							}
-							m = (int) (m*255.0f);
-							const int a = (y_pwr2mapx_half + x) * 4 - offset;
-							infoTexMem[a + COLOR_R] = 255 - m;
-							infoTexMem[a + COLOR_G] = m;
-							infoTexMem[a + COLOR_B] = 0;
-							infoTexMem[a + COLOR_A] = 255;
-						}
-					}
-				} else {
-					const MoveData* md = NULL;
-					{
-						GML_RECMUTEX_LOCK(sel); // UpdateExtraTexture
-						if (!selectedUnits.selectedUnits.empty()) {
-							md = (*selectedUnits.selectedUnits.begin())->unitDef->movedata;
-						}
-					}
-
-					if (md != NULL) {
-						// use the first selected unit, if it has the ability to move
-						const bool showBlockedMap = (gs->cheatEnabled || gu->spectating);
-						for (int y = starty; y < endy; ++y) {
-							const int y_pwr2mapx_half = y*pwr2mapx_half;
-							const int y_2             = y*2;
-							for (int x = 0; x < gs->hmapx; ++x) {
-								float m = md->moveMath->SpeedMod(*md, x*2, y_2);
-								if (showBlockedMap && (md->moveMath->IsBlocked2(*md, x*2+1, y_2+1) & (CMoveMath::BLOCK_STRUCTURE | CMoveMath::BLOCK_TERRAIN))) {
-									m = 0.0f;
-								}
-								m = std::min(1.0f, fastmath::apxsqrt(m));
-								m = (int) (m*255.0f);
-								const int a = (y_pwr2mapx_half + x) * 4 - offset;
-								infoTexMem[a + COLOR_R] = 255 - m;
-								infoTexMem[a + COLOR_G] = m;
-								infoTexMem[a + COLOR_B] = 0;
-								infoTexMem[a + COLOR_A] = 255;
-							}
-						}
-					} else {
-						// we have nothing to show
-						// -> draw a dark red overlay
-						for (int y = starty; y < endy; ++y) {
-							const int y_pwr2mapx_half = y*pwr2mapx_half;
-							for (int x = 0; x < gs->hmapx; ++x) {
-								const int a = (y_pwr2mapx_half + x) * 4 - offset;
-								infoTexMem[a + COLOR_R] = 100;
-								infoTexMem[a + COLOR_G] = 0;
-								infoTexMem[a + COLOR_B] = 0;
-								infoTexMem[a + COLOR_A] = 255;
-							}
-						}
-					}
-				}
-				break;
-			}
 			case drawMetal: {
 				for (int y = starty; y < endy; ++y) {
 					const int y_pwr2mapx_half = y*pwr2mapx_half;
 					const int y_2 = y*2;
 					const int y_hmapx = y * gs->hmapx;
+
 					for (int x = 0; x < gs->hmapx; ++x) {
 						const int a   = (y_pwr2mapx_half + x) * 4 - offset;
 						const int alx = ((x*2) >> loshandler->airMipLevel);
@@ -411,29 +369,26 @@ bool CBaseGroundDrawer::UpdateExtraTexture()
 				}
 				break;
 			}
-			case drawHeat: {
-				for (int y = starty; y < endy; ++y) {
-					const int y_pwr2hmapx = y * pwr2mapx_half;
-					for (int x = 0; x  < gs->hmapx; ++x) {
-						const int i = (y_pwr2hmapx + x) * 4 - offset;
-						infoTexMem[i + COLOR_R] = (unsigned char)Clamp(8*pathManager->GetHeatOnSquare(x<<1, y<<1), 32, 255);
-						infoTexMem[i + COLOR_G] = 32;
-						infoTexMem[i + COLOR_B] = 32;
-						infoTexMem[i + COLOR_A] = 255;
-					}
-				}
-				break;
-			}
+
 			case drawHeight: {
 				extraTexPal = heightLinePal->GetData();
+
+				//NOTE: the resolution of our PBO/ExtraTexture is gs->pwr2mapx * gs->pwr2mapy (we don't use it fully)
+				//      while the corner heightmap has (gs->mapx + 1) * (gs->mapy + 1).
+				//      So for the case POT(gs->mapx) == gs->mapx we would get a buffer overrun in our PBO
+				//      when iterating (gs->mapx + 1) * (gs->mapy + 1). So we just skip +1, it may give
+				//      semi incorrect results, but it is the easiest solution.
+				const float* heightMap = readmap->GetCornerHeightMapUnsynced();
+
 				for (int y = starty; y < endy; ++y) {
 					const int y_pwr2mapx = y * gs->pwr2mapx;
-					const int y_mapx     = y * gs->mapx;
-					for (int x = 0; x  < gs->mapx; ++x) {
-						const float height = readmap->centerheightmap[y_mapx + x];
+					const int y_mapx     = y * gs->mapxp1;
+
+					for (int x = 0; x < gs->mapx; ++x) {
+						const float height = heightMap[y_mapx + x];
 						const unsigned int value = (((unsigned int)(height * 8.0f)) % 255) * 3;
 						const int i = (y_pwr2mapx + x) * 4 - offset;
-						infoTexMem[i + COLOR_R] = 64 + (extraTexPal[value]     >> 1);
+						infoTexMem[i + COLOR_R] = 64 + (extraTexPal[value    ] >> 1);
 						infoTexMem[i + COLOR_G] = 64 + (extraTexPal[value + 1] >> 1);
 						infoTexMem[i + COLOR_B] = 64 + (extraTexPal[value + 2] >> 1);
 						infoTexMem[i + COLOR_A] = 255;
@@ -451,9 +406,11 @@ bool CBaseGroundDrawer::UpdateExtraTexture()
 				const int airSizeY = loshandler->airSizeY;
 				const int losMipLevel = loshandler->losMipLevel + lowRes;
 				const int airMipLevel = loshandler->airMipLevel + lowRes;
+
 				if (drawRadarAndJammer) {
 					const int rxsize = radarhandler->xsize;
 					const int rzsize = radarhandler->zsize;
+
 					for (int y = starty; y < endy; ++y) {
 						for (int x = 0; x < endx; ++x) {
 							int totalLos = 255;
@@ -463,7 +420,7 @@ bool CBaseGroundDrawer::UpdateExtraTexture()
 								totalLos = inLos + inAir;
 							}
 #ifdef SONAR_JAMMER_MAPS
-							const bool useRadar = (ground->GetHeight2(xPos, zPos) >= 0.0f);
+							const bool useRadar = (ground->GetHeightReal(xPos, zPos, false) >= 0.0f);
 							const unsigned short* radarMap  = useRadar ? myRadar  : mySonar;
 							const unsigned short* jammerMap = useRadar ? myJammer : mySonarJammer;
 #else
@@ -502,15 +459,11 @@ bool CBaseGroundDrawer::UpdateExtraTexture()
 				}
 				break;
 			}
+
 			case drawNormal:
 				break;
 		} // switch (drawMode)
 
-#undef COLOR_R
-#undef COLOR_G
-#undef COLOR_B
-#undef COLOR_A
-
 		extraTexPBO.UnmapBuffer();
 		/*
 		glBindTexture(GL_TEXTURE_2D, infoTex);
@@ -533,7 +486,7 @@ bool CBaseGroundDrawer::UpdateExtraTexture()
 			glGenTextures(1,&infoTex);
 			glBindTexture(GL_TEXTURE_2D, infoTex);
 
-			//todo: maybe use GL_RGB5 as internalformat?
+			// XXX maybe use GL_RGB5 as internalformat?
 			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
 			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
@@ -564,3 +517,29 @@ bool CBaseGroundDrawer::UpdateExtraTexture()
 	updateTextureState++;
 	return false;
 }
+
+
+void CBaseGroundDrawer::UpdateCamRestraints(CCamera* cam)
+{
+	// add restraints for camera sides
+	cam->GetFrustumSides(readmap->currMinHeight - 100.0f,  readmap->currMaxHeight + 30.0f,  SQUARE_SIZE);
+
+	// add restraint for maximum view distance (use flat z-dir as side)
+	// this is supposed to prevent (far) terrain from first being drawn
+	// and then immediately z-clipped away
+	const float3& camDir3D  = cam->forward;
+	      float3  camDir2D  = float3(camDir3D.x, 0.0f, camDir3D.z);
+	      float3  camOffset = ZeroVector;
+
+	// FIXME magic constants
+	static const float miny = 0.0f;
+	static const float maxy = 255.0f / 3.5f;
+
+	// prevent colinearity in top-down view
+	if (fabs(camDir3D.dot(UpVector)) < 0.95f) {
+		camDir2D  = camDir2D.SafeANormalize();
+		camOffset = camDir2D * globalRendering->viewRange * 1.05f;
+
+		cam->GetFrustumSide(camDir2D, camOffset, miny, maxy, SQUARE_SIZE, (camDir3D.y > 0.0f), false);
+	}
+}
