@@ -3,16 +3,36 @@
 #ifndef __BASE_GROUND_DRAWER_H__
 #define __BASE_GROUND_DRAWER_H__
 
-#include <vector>
+#include <map>
 #include "Rendering/GL/myGL.h"
+#include "Rendering/GL/LightHandler.h"
 #include "Rendering/GL/PBO.h"
-#include "float3.h"
+#include "System/float3.h"
 
+class CMetalMap;
 class CHeightLinePalette;
+class CBaseGroundTextures;
+class CCamera;
 
 class CBaseGroundDrawer
 {
 public:
+	enum {
+		COLOR_R = 2,
+		COLOR_G = 1,
+		COLOR_B = 0,
+		COLOR_A = 3,
+	};
+	enum BaseGroundDrawMode {
+		drawNormal,
+		drawLos,
+		drawMetal,
+		drawHeight,
+		drawPathTraversability,
+		drawPathHeat,
+		drawPathCost,
+	};
+
 	CBaseGroundDrawer(void);
 	virtual ~CBaseGroundDrawer(void);
 
@@ -24,42 +44,35 @@ public:
 	virtual void SetupRefrDrawPass(void) {}
 
 	virtual void Update() = 0;
+	virtual void UpdateSunDir() = 0;
 
 	virtual void IncreaseDetail() = 0;
 	virtual void DecreaseDetail() = 0;
 
-#ifdef USE_GML
-	int multiThreadDrawGround;
-	int multiThreadDrawGroundShadow;
-#endif
-
-	enum BaseGroundDrawMode {
-		drawNormal,
-		drawLos,
-		drawMetal,
-		drawHeight,
-		drawPath,
-		drawHeat
-	};
-
-protected:
 	virtual void SetDrawMode(BaseGroundDrawMode dm) { drawMode = dm; }
+	virtual GL::LightHandler* GetLightHandler() { return NULL; }
 
-public:
-	void DrawTrees(bool drawReflection=false) const;
+	void DrawTrees(bool drawReflection = false) const;
 
 	// Everything that deals with drawing extra textures on top
 	void DisableExtraTexture();
 	void SetHeightTexture();
-	void SetMetalTexture(unsigned char* tex,float* extractMap,unsigned char* pal,bool highRes);
-	void SetPathMapTexture();
+	void SetMetalTexture(const CMetalMap*);
+	void TogglePathTraversabilityTexture();
+	void TogglePathHeatTexture();
+	void TogglePathCostTexture();
 	void ToggleLosTexture();
 	void ToggleRadarAndJammer();
-	void ToggleHeatMapTexture();
 	bool UpdateExtraTexture();
-	bool DrawExtraTex() const { return drawMode!=drawNormal; };
+	bool DrawExtraTex() const { return drawMode != drawNormal; }
 
+	CBaseGroundTextures* GetGroundTextures() { return groundTextures; }
+
+	void UpdateCamRestraints(CCamera* camera);
+
+public:
 	bool wireframe;
+	bool advShading;
 
 	float LODScaleReflection;
 	float LODScaleRefraction;
@@ -79,7 +92,7 @@ public:
 
 	const unsigned char* extraTex;
 	const unsigned char* extraTexPal;
-	float* extractDepthMap;
+	const float* extractDepthMap;
 
 	float infoTexAlpha;
 
@@ -92,8 +105,13 @@ public:
 	bool highResLosTex;
 	int extraTextureUpdateRate;
 
+#ifdef USE_GML
+	bool multiThreadDrawGround;
+	bool multiThreadDrawGroundShadow;
+#endif
+
 	CHeightLinePalette* heightLinePal;
+	CBaseGroundTextures* groundTextures;
 };
 
 #endif // __BASE_GROUND_DRAWER__
-
