@@ -1,21 +1,20 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "AdvSky.h"
 
 #include "Game/Camera.h"
 #include "Map/MapInfo.h"
-#include "Map/ReadMap.h"
 #include "Rendering/GlobalRendering.h"
-#include "Rendering/Textures/Bitmap.h"
 #include "Rendering/GL/VertexArray.h"
-#include "LogOutput.h"
-#include "TimeProfiler.h"
-#include "ConfigHandler.h"
-#include "Matrix44f.h"
-#include "GlobalUnsynced.h"
+#include "Rendering/Textures/Bitmap.h"
+#include "Sim/Misc/GlobalSynced.h"
+#include "System/Config/ConfigHandler.h"
+#include "System/Exceptions.h"
+#include "System/TimeProfiler.h"
+#include "System/Matrix44f.h"
+#include "System/myMath.h"
 
 #define Y_PART 10.0
 #define X_PART 10.0
@@ -23,62 +22,82 @@
 #define CLOUD_DETAIL 6
 #define CLOUD_MASK (CLOUD_SIZE-1)
 
-//static unsigned int cdtex;
-
-using std::max;
-using std::min;
 
 CAdvSky::CAdvSky()
+	: skydir1(ZeroVector)
+	, skydir2(ZeroVector)
+	, cdtex(0)
+	, cloudFP(0)
+	, drawFlare(false)
+	, cloudTexMem(NULL)
+	, skyTex(0)
+	, skyDot3Tex(0)
+	, cloudDot3Tex(0)
+	, sunTex(0)
+	, sunFlareTex(0)
+	, skyTexUpdateIter(0)
+	, skyDomeList(0)
+	, sunFlareList(0)
+	, skyAngle(0.0f)
+	, domeheight(0.0f)
+	, domeWidth(0.0f)
+	, sunTexCoordX(0.0f)
+	, sunTexCoordY(0.0f)
+	, randMatrix(NULL)
+	, rawClouds(NULL)
+	, blendMatrix(NULL)
+	, cloudThickness(NULL)
+	, oldCoverBaseX(-5)
+	, oldCoverBaseY(0)
+	, updatecounter(0)
 {
-	PrintLoadMsg("Creating sky");
+	skytexpart = new unsigned char[512][4];
 
-	randMatrix=newmat3<int>(16,32,32);
-	rawClouds=newmat2<int>(CLOUD_SIZE,CLOUD_SIZE);
-	blendMatrix=newmat3<int>(CLOUD_DETAIL,32,32);
+	if (!(FBO::IsSupported() && GLEW_ARB_fragment_program && ProgramStringIsNative(GL_FRAGMENT_PROGRAM_ARB, "ARB/clouds.fp"))) {
+		throw content_error("ADVSKY: missing OpenGL features!");
+	}
 
-	updatecounter=0;
+	randMatrix = newmat3<int>(16, 32, 32);
+	rawClouds = newmat2<int>(CLOUD_SIZE, CLOUD_SIZE);
+	blendMatrix = newmat3<int>(CLOUD_DETAIL, 32, 32);
 
-	domeheight=cos(PI/16)*1.01f;
-	domeWidth=sin(2*PI/32)*400*1.7f;
+	memset(alphaTransform, 0, 1024);
+	memset(thicknessTransform, 0, 1024);
+	memset(covers, 0, 4 * 32 * sizeof(float));
 
-	sundir2=mapInfo->light.sunDir;
-	sundir2.y=0;
-	if(sundir2.Length()==0)
-		sundir2.x=1;
-	sundir2.ANormalize();
-	sundir1=sundir2.cross(UpVector);
+	domeheight = cos(PI / 16) * 1.01f;
+	domeWidth = sin(2 * PI / 32) * 400 * 1.7f;
 
-	modSunDir.y=mapInfo->light.sunDir.y;
-	modSunDir.x=0;
-	modSunDir.z=sqrt(mapInfo->light.sunDir.x*mapInfo->light.sunDir.x+mapInfo->light.sunDir.z*mapInfo->light.sunDir.z);
+	UpdateSkyDir();
+	InitSun();
+	UpdateSunDir();
 
-	sunTexCoordX=0.5f;
-	sunTexCoordY=GetTexCoordFromDir(modSunDir);
-
-	for(int a=0;a<CLOUD_DETAIL;a++)
-		cloudDown[a]=false;
-	for(int a=0;a<5;a++)
-		cloudDetailDown[a]=false;
+	for (int a = 0; a < CLOUD_DETAIL; a++)
+		cloudDown[a] = false;
+	for (int a = 0; a < 5; a++)
+		cloudDetailDown[a] = false;
 
 	cloudDensity = mapInfo->atmosphere.cloudDensity;
 	cloudColor = mapInfo->atmosphere.cloudColor;
 	skyColor = mapInfo->atmosphere.skyColor;
 	sunColor = mapInfo->atmosphere.sunColor;
 	fogStart = mapInfo->atmosphere.fogStart;
-	if (fogStart>0.99f) globalRendering->drawFog = false;
+	if (fogStart > 0.99f) globalRendering->drawFog = false;
 
-	dynamicSky=true;
+	dynamicSky = true;
 	CreateClouds();
-	dynamicSky=!!configHandler->Get("DynamicSky",0);
+	dynamicSky = configHandler->GetBool("DynamicSky");
 
-	InitSun();
-	oldCoverBaseX=-5;
+	cloudFP = LoadFragmentProgram("ARB/clouds.fp");
 
-	cloudFP=LoadFragmentProgram("ARB/clouds.fp");
+	CreateSkyDomeList();
+}
 
+void CAdvSky::CreateSkyDomeList()
+{
 	glGetError();
-	displist=glGenLists(1);
-	glNewList(displist, GL_COMPILE);
+	skyDomeList = glGenLists(1);
+	glNewList(skyDomeList, GL_COMPILE);
 	glDisable(GL_TEXTURE_2D);
 	glDisable(GL_LIGHTING);
 	glDisable(GL_ALPHA_TEST);
@@ -157,7 +176,7 @@ CAdvSky::~CAdvSky()
 	glDeleteTextures(1, &skyTex);
 	glDeleteTextures(1, &skyDot3Tex);
 	glDeleteTextures(1, &cloudDot3Tex);
-	glDeleteLists(displist,1);
+	glDeleteLists(skyDomeList, 1);
 
 	glDeleteTextures(1, &sunTex);
 	glDeleteTextures(1, &sunFlareTex);
@@ -171,6 +190,8 @@ CAdvSky::~CAdvSky()
 	delmat3<int>(randMatrix);
 	delmat2<int>(rawClouds);
 	delmat3<int>(blendMatrix);
+
+	delete[] skytexpart;
 }
 
 void CAdvSky::Draw()
@@ -180,10 +201,10 @@ void CAdvSky::Draw()
 
 	glPushMatrix();
 //	glTranslatef(camera->pos.x,camera->pos.y,camera->pos.z);
-	CMatrix44f m(camera->pos,sundir1,UpVector,sundir2);
+	CMatrix44f m(camera->pos,skydir1,UpVector,skydir2);
 	glMultMatrixf(m.m);
 
-	float3 modCamera=sundir1*camera->pos.x+sundir2*camera->pos.z;
+	float3 modCamera=skydir1*camera->pos.x+skydir2*camera->pos.z;
 
 	glMatrixMode(GL_TEXTURE);
 	  glActiveTextureARB(GL_TEXTURE2_ARB);
@@ -195,7 +216,7 @@ void CAdvSky::Draw()
 	  glActiveTextureARB(GL_TEXTURE0_ARB);
 	glMatrixMode(GL_MODELVIEW);
 
-	glCallList(displist);
+	glCallList(skyDomeList);
 
 	glMatrixMode(GL_TEXTURE);
 	  glActiveTextureARB(GL_TEXTURE2_ARB);
@@ -210,28 +231,19 @@ void CAdvSky::Draw()
 	glEnable(GL_DEPTH_TEST);
 	glDisable(GL_BLEND);
 
-	glFogfv(GL_FOG_COLOR,mapInfo->atmosphere.fogColor);
-	glFogf(GL_FOG_START,globalRendering->viewRange*fogStart);
-	glFogf(GL_FOG_END,globalRendering->viewRange);
-	glFogf(GL_FOG_DENSITY,1.0f);
-	glFogi(GL_FOG_MODE,GL_LINEAR);
-	if (globalRendering->drawFog) {
-		glEnable(GL_FOG);
-	} else {
-		glDisable(GL_FOG);
-	}
+	ISky::SetupFog();
 }
 
 float3 CAdvSky::GetCoord(int x, int y)
 {
-	float3 c;
-	float fy=((float)y/Y_PART)*2*PI;
-	float fx=((float)x/X_PART)*2*PI;
+	float fy = ((float)y/Y_PART) * 2 * PI;
+	float fx = ((float)x/X_PART) * 2 * PI;
 
-	c.x=sin(fy/32)*sin(fx)*400;
-	c.y=(cos(fy/32)-domeheight)*400;
-	c.z=sin(fy/32)*cos(fx)*400;
-	return c;
+	return float3(
+		fastmath::sin(fy/32) * fastmath::sin(fx),
+		fastmath::cos(fy/32) - domeheight,
+		fastmath::sin(fy/32) * fastmath::cos(fx)
+	) * 400;
 }
 
 void CAdvSky::CreateClouds()
@@ -243,23 +255,18 @@ void CAdvSky::CreateClouds()
 	glGenTextures(1, &skyDot3Tex);
 	glGenTextures(1, &cloudDot3Tex);
 
-	unsigned char (* skytex)[512][4]=new unsigned char[512][512][4]; // this is too big for the stack
+	unsigned char (* skytex)[512][4] = new unsigned char[512][512][4]; // this is too big for the stack
 
 	glGenTextures(1, &cdtex);
-//	CBitmap pic("bitmaps/clouddetail.bmp");
-	unsigned char mem[256*256];
-//	for(int a=0;a<256*256;++a){
-//		mem[a]=pic.mem[a*4];
-//	}
 	glBindTexture(GL_TEXTURE_2D, cdtex);
 	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR/*_MIPMAP_NEAREST*/);
-	glBuildMipmaps(GL_TEXTURE_2D,GL_LUMINANCE ,256, 256, GL_LUMINANCE, GL_UNSIGNED_BYTE, mem);
+	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, 256, 256, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
 
 	unsigned char randDetailMatrix[32*32];
 	glGenTextures(12, detailTextures);
-	for(int a=0;a<6;++a){
-		int size=min(32,256>>a);
+	for (int a=0; a<6; ++a) {
+		int size = std::min(32,256>>a);
 		CreateRandDetailMatrix(randDetailMatrix,size);
 		glBindTexture(GL_TEXTURE_2D, detailTextures[a]);
 		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
@@ -273,53 +280,31 @@ void CAdvSky::CreateClouds()
 		glTexImage2D(GL_TEXTURE_2D,0,GL_LUMINANCE, size, size,0,GL_LUMINANCE, GL_UNSIGNED_BYTE, randDetailMatrix);
 	}
 
-	for(int y=0;y<512;y++){
-		for(int x=0;x<512;x++){
-			float3 dir=GetDirFromTexCoord(x/512.0f,y/512.0f);
-			float sunDist=acos(dir.dot(modSunDir))*70;
-			float sunMod=12.0f/(12+sunDist);
-
-			float red=(skyColor.x+sunMod*sunColor.x);
-			float green=(skyColor.y+sunMod*sunColor.y);
-			float blue=(skyColor.z+sunMod*sunColor.z);
-			if(red>1)
-				red=1;
-			if(green>1)
-				green=1;
-			if(blue>1)
-				blue=1;
-			skytex[y][x][0]=(unsigned char)(red*255);
-			skytex[y][x][1]=(unsigned char)(green*255);
-			skytex[y][x][2]=(unsigned char)(blue*255);
-			skytex[y][x][3]=255;
+	for (int y=0; y<512 ;y++) {
+		for(int x=0; x<512; x++) {
+			UpdateTexPart(x, y, skytex[y]);
 		}
 	}
 
 	glBindTexture(GL_TEXTURE_2D, skyTex);
 	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
-	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
-	glBuildMipmaps(GL_TEXTURE_2D,GL_RGBA8 ,512, 512, GL_RGBA, GL_UNSIGNED_BYTE, skytex[0][0]);
+	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR/*_MIPMAP_NEAREST*/);
+//	glBuildMipmaps(GL_TEXTURE_2D,GL_RGBA8 ,512, 512, GL_RGBA, GL_UNSIGNED_BYTE, skytex[0][0]);
+	glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA8 ,512, 512, 0, GL_RGBA, GL_UNSIGNED_BYTE, skytex[0][0]);
 	delete [] skytex;
 
-	unsigned char (* skytex2)[256][4]=new unsigned char[256][256][4];
+	unsigned char (* skytex2)[256][4] = new unsigned char[256][256][4];
 	for(int y=0;y<256;y++){
 		for(int x=0;x<256;x++){
-			float3 dir=GetDirFromTexCoord(x/256.0f,y/256.0f);
-			float sunDist=acos(dir.dot(modSunDir))*50;
-			float sunMod=0.3f/sqrt(sunDist)+3.0f/(1+sunDist);
-			float green=min(1.0f,(0.55f+sunMod));
-			float blue=203-(40.0f/(3+sunDist));
-			skytex2[y][x][0]=(unsigned char)(255-y/2);				//sun on borders
-			skytex2[y][x][1]=(unsigned char)(green*255);			//sun light through
-			skytex2[y][x][2]=(unsigned char) blue;						//ambient
-			skytex2[y][x][3]=255;
+			UpdateTexPartDot3(x, y, skytex2[y]);
 		}
 	}
 
 	glBindTexture(GL_TEXTURE_2D, skyDot3Tex);
 	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
-	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
-	glBuildMipmaps(GL_TEXTURE_2D,GL_RGBA8 ,256, 256, GL_RGBA, GL_UNSIGNED_BYTE, skytex2[0][0]);
+	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR/*_MIPMAP_NEAREST*/);
+//	glBuildMipmaps(GL_TEXTURE_2D,GL_RGBA8 ,256, 256, GL_RGBA, GL_UNSIGNED_BYTE, skytex2[0][0]);
+	glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA8 ,256, 256, 0, GL_RGBA, GL_UNSIGNED_BYTE, skytex2[0][0]);
 	delete [] skytex2;
 
 	for(int a=0;a<CLOUD_DETAIL;a++){
@@ -327,12 +312,19 @@ void CAdvSky::CreateClouds()
 		CreateRandMatrix(randMatrix[a+8],1-a*0.03f);
 	}
 
-	char *scrap=new char[CLOUD_SIZE*CLOUD_SIZE*4];
 	glBindTexture(GL_TEXTURE_2D, cloudDot3Tex);
 	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
-	glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA8, CLOUD_SIZE, CLOUD_SIZE,0,GL_RGBA, GL_UNSIGNED_BYTE, scrap);
-  delete [] scrap;
+	glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA8, CLOUD_SIZE, CLOUD_SIZE,0,GL_RGBA, GL_UNSIGNED_BYTE, NULL);
+
+	fbo.reloadOnAltTab = true;
+	fbo.Bind();
+	fbo.AttachTexture(cdtex);
+	bool status = fbo.CheckStatus("ADVSKY");
+	FBO::Unbind();
+	if (!status) {
+		throw content_error("ADVSKY: FBO is unavailable");
+	}
 
 	CreateTransformVectors();
 	for(int i=0; i<CLOUD_DETAIL+7; ++i)
@@ -343,13 +335,13 @@ inline void CAdvSky::UpdatePart(int ast, int aed, int a3cstart, int a4cstart) {
 	int* rc = *rawClouds + ast;
 	unsigned char* ct = cloudTexMem + 4 * ast;
 
-	int	yam2 = ydif[(ast - 2) & CLOUD_MASK];
-	int	yam1 = ydif[(ast - 1) & CLOUD_MASK];
-	int	yaa  = ydif[(ast)     & CLOUD_MASK];
-	int	ap1 = (ast + 1) & CLOUD_MASK;
+	int yam2 = ydif[(ast - 2) & CLOUD_MASK];
+	int yam1 = ydif[(ast - 1) & CLOUD_MASK];
+	int yaa  = ydif[(ast)     & CLOUD_MASK];
+	int ap1  = (ast + 1) & CLOUD_MASK;
 
-	int	a3c = ast + a3cstart;
-	int	a4c = ast + a4cstart;
+	int a3c  = ast + a3cstart;
+	int a4c  = ast + a4cstart;
 
 	for (int a = ast; a < aed; ++rc, ++ct) {
 		int yap1 = ydif[ap1] += (int) cloudThickness[++a3c] - cloudThickness[++a] * 2 + cloudThickness[++a4c];
@@ -376,10 +368,10 @@ inline void CAdvSky::UpdatePart(int ast, int aed, int a3cstart, int a4cstart) {
 
 void CAdvSky::Update()
 {
-	if(!dynamicSky)
+	if (!dynamicSky)
 		return;
 
-	SCOPED_TIMER("Sky Update");
+	SCOPED_TIMER("AdvSky::Update");
 
 	CreateDetailTex();
 
@@ -390,10 +382,10 @@ void CAdvSky::Update()
 	switch(updatepart) { // smoothen out the workload across draw frames
 	case 0: {
 		for(int a=0; a<CLOUD_DETAIL; a++) {
-			float fade=(gs->frameNum/(70.0f*(2<<(CLOUD_DETAIL-1-a))));
-			fade-=floor(fade/2)*2;
+			float fade = gs->frameNum / (70.0f * (2<<(CLOUD_DETAIL-1-a)));
+			fade -= floor(fade/2)*2;
 			if(fade>1) {
-				fade=2-fade;
+				fade = 2 - fade;
 				if(!cloudDown[a]) {
 					cloudDown[a]=true;
 					CreateRandMatrix(randMatrix[a+8],1-a*0.03f);
@@ -452,7 +444,7 @@ void CAdvSky::Update()
 			int blend=bm[by&31][31&31], **prcy=prc, *pkernel=kernel;
 			for(int y2=0; y2<cs4a; ++y2, ++prcy, pkernel+=CLOUD_SIZE/4) {
 				int *prcx=(*prcy)+cmcs8a, *pkrn=pkernel;
-				for(int x2=cmcs8a; x2<std::min(CLOUD_SIZE, cs4a+cmcs8a); ++x2)
+				for(int x2=cmcs8a; x2 < std::min(CLOUD_SIZE, cs4a+cmcs8a); ++x2)
 					(*prcx++)+=blend*(*pkrn++); // prcx = rawClouds[y2+y][x2+cmcs8a], x2<CLOUD_SIZE
 				prcx-=CLOUD_SIZE;
 				for(int x2=std::max(CLOUD_SIZE,cmcs8a); x2<cs4a+cmcs8a; ++x2)
@@ -473,13 +465,19 @@ void CAdvSky::Update()
 		for(int a=0;a<CLOUD_SIZE*CLOUD_SIZE;a++)
 			cloudThickness[a]=alphaTransform[rawClouds[0][a]>>7];
 
-		cloudThickness[CLOUD_SIZE*CLOUD_SIZE]=cloudThickness[CLOUD_SIZE*CLOUD_SIZE-1];	//this one is read in one place, so to avoid reading uninitialized mem ...
+		// this one is read in one place, so to avoid reading uninitialized mem ...
+		cloudThickness[CLOUD_SIZE*CLOUD_SIZE] = cloudThickness[CLOUD_SIZE*CLOUD_SIZE-1];
 
-		//create the cloud shading
-		for(int a=0;a<CLOUD_SIZE;++a){
-			ydif[a]=(int)cloudThickness[(a+3*CLOUD_SIZE)] + cloudThickness[(a+2*CLOUD_SIZE)] + cloudThickness[(a+1*CLOUD_SIZE)] + 
-				cloudThickness[(a+0*CLOUD_SIZE)] - cloudThickness[(a+CLOUD_SIZE*(CLOUD_SIZE-1))] - 
-				cloudThickness[(a+CLOUD_SIZE*(CLOUD_SIZE-2))] - cloudThickness[(a+CLOUD_SIZE*(CLOUD_SIZE-3))];
+		// create the cloud shading
+		for (int a = 0; a < CLOUD_SIZE; ++a) {
+			ydif[a] =
+				(int)cloudThickness[(a+3*CLOUD_SIZE)]
+				+    cloudThickness[(a+2*CLOUD_SIZE)]
+				+    cloudThickness[(a+1*CLOUD_SIZE)]
+				+    cloudThickness[(a+0*CLOUD_SIZE)]
+				-    cloudThickness[(a+CLOUD_SIZE*(CLOUD_SIZE-1))]
+				-    cloudThickness[(a+CLOUD_SIZE*(CLOUD_SIZE-2))]
+				-    cloudThickness[(a+CLOUD_SIZE*(CLOUD_SIZE-3))];
 		}
 
 		ydif[0] += cloudThickness[0+CLOUD_SIZE*(CLOUD_SIZE-3)] - cloudThickness[0]*2 + cloudThickness[0+4*CLOUD_SIZE];
@@ -498,9 +496,9 @@ void CAdvSky::Update()
 		break;
 	}
 	case CLOUD_DETAIL+5: {
-		int modDensity=(int) ((1-cloudDensity)*256);
-		for(int a=0; a<CLOUD_SIZE*CLOUD_SIZE; ++a){
-			int f=(rawClouds[0][a]>>8)-modDensity;
+		const int modDensity = (int) ((1-cloudDensity)*256);
+		for (int a = 0; a < CLOUD_SIZE*CLOUD_SIZE; ++a) {
+			const int f = (rawClouds[0][a]>>8)-modDensity;
 			cloudTexMem[a*4+3]=std::max(0, std::min(255, f));
 		}
 		break;
@@ -600,29 +598,48 @@ void CAdvSky::DrawSun()
 	glBindTexture(GL_TEXTURE_2D, sunFlareTex);
 	glTexSubImage2D(GL_TEXTURE_2D,0,0,0,32,3,GL_LUMINANCE,GL_UNSIGNED_BYTE,buf);
 
-	glColor4f(0.4f*sunColor.x,0.4f*sunColor.y,0.4f*sunColor.z,0.0f);
+	const float si = skyLight->GetLightIntensity();
+	const float3 sc = sunColor * si;
+
+	glColor4f(sc.x, sc.y, sc.z, 0.0f);
 	glCallList(sunFlareList);
 
 	glEnable(GL_DEPTH_TEST);
 	glPopMatrix();
 }
 
-void CAdvSky::SetCloudShadow(int texunit)
-{
+void CAdvSky::UpdateSunFlare() {
+	if(sunFlareList)
+		glDeleteLists(sunFlareList, 1);
+	float3 ldir=modSunDir.cross(UpVector);
+	ldir.ANormalize();
+	float3 udir=modSunDir.cross(ldir);
+
+	sunFlareList=glGenLists(1);
+	glNewList(sunFlareList, GL_COMPILE);
+	glDisable(GL_FOG);
+	glBindTexture(GL_TEXTURE_2D, sunFlareTex);
+	glBlendFunc(GL_ONE_MINUS_DST_COLOR,GL_ONE);
+	glBegin(GL_TRIANGLE_STRIP);
+	for(int x=0;x<257;++x){
+		float dx = sin(x*2*PI/256.0f);
+		float dy = cos(x*2*PI/256.0f);
+
+		glTexCoord2f(x/256.0f,0.125f);
+		glVertexf3(modSunDir*5+ldir*dx*0.0014f+udir*dy*0.0014f);
+		glTexCoord2f(x/256.0f,0.875f);
+		glVertexf3(modSunDir*5+ldir*dx*4+udir*dy*4);
+	}
+	glEnd();
+	if (globalRendering->drawFog) glEnable(GL_FOG);
+	glEndList();
 }
 
-void CAdvSky::ResetCloudShadow(int texunit)
-{
-}
-
-void CAdvSky::DrawShafts()
-{
-}
 
 
 void CAdvSky::InitSun()
 {
-	unsigned char* mem=new unsigned char[128*128*4];
+	unsigned char* mem = new unsigned char[128*128*4];
 
 	for(int y=0;y<128;++y){
 		for(int x=0;x<128;++x){
@@ -660,32 +677,9 @@ void CAdvSky::InitSun()
 	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
 	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
 //	gluBuild2DMipmaps(GL_TEXTURE_2D,1 ,32, 2, GL_ALPHA, GL_UNSIGNED_BYTE, mem);
-	glTexImage2D(GL_TEXTURE_2D,0,GL_LUMINANCE ,32, 4,0, GL_LUMINANCE, GL_UNSIGNED_BYTE, mem);
+	glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE ,32, 4, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, mem);
 
 	delete [] mem;
-
-	float3 ldir=modSunDir.cross(UpVector);
-	ldir.ANormalize();
-	float3 udir=modSunDir.cross(ldir);
-
-	sunFlareList=glGenLists(1);
-	glNewList(sunFlareList, GL_COMPILE);
-		glDisable(GL_FOG);
-		glBindTexture(GL_TEXTURE_2D, sunFlareTex);
-		glBlendFunc(GL_ONE_MINUS_DST_COLOR,GL_ONE);
-		glBegin(GL_TRIANGLE_STRIP);
-		for(int x=0;x<257;++x){
-			float dx=sin(x*2*PI/256.0f);
-			float dy=cos(x*2*PI/256.0f);
-
-			glTexCoord2f(x/256.0f,0.125f);
-			glVertexf3(modSunDir*5+ldir*dx*0.0014f+udir*dy*0.0014f);
-			glTexCoord2f(x/256.0f,0.875f);
-			glVertexf3(modSunDir*5+ldir*dx*4+udir*dy*4);
-		}
-		glEnd();
-		if (globalRendering->drawFog) glEnable(GL_FOG);
-	glEndList();
 }
 
 void CAdvSky::CreateCover(int baseX, int baseY, float *buf)
@@ -732,8 +726,10 @@ void CAdvSky::CreateCover(int baseX, int baseY, float *buf)
 	}
 }
 
-void CAdvSky::CreateDetailTex(void)
+void CAdvSky::CreateDetailTex()
 {
+	fbo.Bind(); //! render to cdtex
+
 	glViewport(0,0,256,256);
 	glLoadIdentity();
 	glMatrixMode(GL_PROJECTION);
@@ -744,7 +740,7 @@ void CAdvSky::CreateDetailTex(void)
 	glDisable(GL_LIGHTING);
 	glEnable(GL_TEXTURE_2D);
 	glClearColor(0,0,0,0);
-	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+	glClear(GL_COLOR_BUFFER_BIT);
 	glEnable(GL_BLEND);
 	glBlendFunc(GL_SRC_ALPHA,GL_ONE);
 	glDisable(GL_DEPTH_TEST);
@@ -752,9 +748,9 @@ void CAdvSky::CreateDetailTex(void)
 	static unsigned char randDetailMatrix[32*32];
 
 	for(int a=0;a<5;++a){
-		float fade=(gs->frameNum/float(30<<a));
-		fade-=floor(fade/2)*2;
-		int size=min(32,256>>a);
+		float fade = gs->frameNum / float(30<<a);
+		fade -= floor(fade/2)*2;
+		int size = std::min(32,256>>a);
 
 		if(fade>1){
 			fade=2-fade;
@@ -773,10 +769,9 @@ void CAdvSky::CreateDetailTex(void)
 			}
 
 		}
-		float tSize=max(1,8>>a);
-		float c=pow(2.0f,a)*6/255.0f;
-//		logOutput.Print("%f",c);
-		CVertexArray* va=GetVertexArray();
+		float tSize = std::max(1,8>>a);
+		float c = pow(2.0f,a)*6/255.0f;
+		CVertexArray* va = GetVertexArray();
 		va->Initialize();
 		va->CheckInitSize(4*VA_SIZE_T);
 		va->AddVertexQT(ZeroVector,0,0);
@@ -784,7 +779,7 @@ void CAdvSky::CreateDetailTex(void)
 		va->AddVertexQT(float3(1,1,0),tSize,tSize);
 		va->AddVertexQT(UpVector,0,tSize);
 
-		float ifade=fade*fade*(3-2*fade);
+		float ifade = fade*fade*(3-2*fade);
 
 		glBindTexture(GL_TEXTURE_2D, detailTextures[a+6]);
 		glColor4f(c,c,c,1-ifade);
@@ -798,46 +793,140 @@ void CAdvSky::CreateDetailTex(void)
 	CBitmap bm(buf,256,256);
 	bm.Save("dc.bmp");
 */
-	glBindTexture(GL_TEXTURE_2D, cdtex);
-	glCopyTexSubImage2D(GL_TEXTURE_2D,0,0,0,0,0,256,256);
-//	SwapBuffers(hDC);
-//	SleepEx(500,true);
 
 	glViewport(globalRendering->viewPosX,0,globalRendering->viewSizeX,globalRendering->viewSizeY);
+	fbo.Unbind();
+
 	glEnable(GL_DEPTH_TEST);
 }
 
+void CAdvSky::UpdateSunDir() {
+	const float3& L = skyLight->GetLightDir();
+
+	sundir2 = L;
+	sundir2.y = 0.0f;
+
+	if (sundir2.SqLength() == 0.0f)
+		sundir2.x = 1.0f;
+
+	sundir2.ANormalize();
+	sundir1 = sundir2.cross(UpVector);
+
+	modSunDir.y = L.y;
+	modSunDir.x = 0.0f;
+	modSunDir.z = math::sqrt(L.x * L.x + L.z * L.z);
+
+	sunTexCoordX = 0.5f;
+	sunTexCoordY = GetTexCoordFromDir(modSunDir);
+
+	UpdateSunFlare();
+}
+
+void CAdvSky::UpdateSkyDir() {
+	skydir2 = mapInfo->atmosphere.skyDir;
+	skydir2.y = 0.0f;
+
+	if (skydir2.SqLength() == 0.0f)
+		skydir2.x = 1.0f;
+
+	skydir2.ANormalize();
+	skydir1 = skydir2.cross(UpVector);
+	skyAngle = GetRadFromXY(skydir2.x, skydir2.z) + PI / 2.0f;
+}
+
+void CAdvSky::UpdateSkyTexture() {
+	const int mod = skyTexUpdateIter % 3;
+
+	if (mod <= 1) {
+		const int y = (skyTexUpdateIter / 3) * 2 + mod;
+		for (int x = 0; x < 512; x++) {
+			UpdateTexPart(x, y, skytexpart);
+		}
+		glBindTexture(GL_TEXTURE_2D, skyTex);
+		glTexSubImage2D(GL_TEXTURE_2D, 0, 0, y, 512, 1, GL_RGBA, GL_UNSIGNED_BYTE, skytexpart[0]);
+	} else {
+		const int y = (skyTexUpdateIter / 3);
+		for (int x = 0; x < 256; x++) {
+			UpdateTexPartDot3(x, y, skytexpart);
+		}
+		glBindTexture(GL_TEXTURE_2D, skyDot3Tex);
+		glTexSubImage2D(GL_TEXTURE_2D, 0, 0, y, 256, 1, GL_RGBA, GL_UNSIGNED_BYTE, skytexpart[0]);
+	}
+
+	skyTexUpdateIter = (skyTexUpdateIter + 1) % (512 + 256);
+}
+
+
+
 float3 CAdvSky::GetDirFromTexCoord(float x, float y)
 {
 	float3 dir;
 
-	dir.x=(x-0.5f)*domeWidth;
-	dir.z=(y-0.5f)*domeWidth;
+	dir.x = (x - 0.5f) * domeWidth;
+	dir.z = (y - 0.5f) * domeWidth;
 
-	float hdist=sqrt(dir.x*dir.x+dir.z*dir.z);
-	float fy=asin(hdist/400);
-	dir.y=(cos(fy)-domeheight)*400;
+	const float hdist = math::sqrt(dir.x * dir.x + dir.z * dir.z);
+	const float ang = GetRadFromXY(dir.x, dir.z) + skyAngle;
+	const float fy = asin(hdist / 400);
+
+	dir.x = hdist * cos(ang);
+	dir.z = hdist * sin(ang);
+	dir.y = (fastmath::cos(fy) - domeheight) * 400;
 
 	dir.ANormalize();
-
 	return dir;
 }
 
-//should be improved
-//only take stuff in yz plane
-float CAdvSky::GetTexCoordFromDir(float3 dir)
+// should be improved
+// only take stuff in yz plane
+float CAdvSky::GetTexCoordFromDir(const float3& dir)
 {
-	float tp=0.5f;
-	float step=0.25f;
+	float tp = 0.5f;
+	float step = 0.25f;
 
-	for(int a=0;a<10;++a){
-		float tx=0.5f+tp;
-		float3 d=GetDirFromTexCoord(tx,0.5f);
-		if(d.y<dir.y)
-			tp-=step;
+	for (int a = 0; a < 10; ++a) {
+		float tx = 0.5f + tp;
+		const float3& d = GetDirFromTexCoord(tx, 0.5f);
+
+		if (d.y < dir.y)
+			tp -= step;
 		else
-			tp+=step;
-		step*=0.5f;
+			tp += step;
+
+		step *= 0.5f;
 	}
-	return 0.5f+tp;
+
+	return (0.5f + tp);
+}
+
+void CAdvSky::UpdateTexPartDot3(int x, int y, unsigned char (*texp)[4]) {
+	const float3& dir = GetDirFromTexCoord(x / 256.0f, (255.0f - y) / 256.0f);
+
+	const float sunInt = skyLight->GetLightIntensity();
+	const float sunDist = acos(dir.dot(skyLight->GetLightDir())) * 50;
+	const float sunMod = sunInt * (0.3f / math::sqrt(sunDist) + 3.0f / (1 + sunDist));
+
+	const float green = std::min(1.0f, (0.55f + sunMod));
+	const float blue  = 203 - sunInt * (40.0f / (3 + sunDist));
+
+	texp[x][0] = (unsigned char) (sunInt * (255 - std::min(255.0f, sunDist))); // sun on borders
+	texp[x][1] = (unsigned char) (green * 255); // sun light through
+	texp[x][2] = (unsigned char)  blue; // ambient
+	texp[x][3] = 255;
+}
+
+void CAdvSky::UpdateTexPart(int x, int y, unsigned char (*texp)[4]) {
+	const float3& dir = GetDirFromTexCoord(x / 512.0f, (511.0f - y) / 512.0f);
+
+	const float sunDist = acos(dir.dot(skyLight->GetLightDir())) * 70;
+	const float sunMod = skyLight->GetLightIntensity() * 12.0f / (12 + sunDist);
+
+	const float red   = std::min(skyColor.x + sunMod * sunColor.x, 1.0f);
+	const float green = std::min(skyColor.y + sunMod * sunColor.y, 1.0f);
+	const float blue  = std::min(skyColor.z + sunMod * sunColor.z, 1.0f);
+
+	texp[x][0] = (unsigned char)(red   * 255);
+	texp[x][1] = (unsigned char)(green * 255);
+	texp[x][2] = (unsigned char)(blue  * 255);
+	texp[x][3] = 255;
 }
