@@ -6,18 +6,19 @@
 #include "Map/Ground.h"
 #include "Map/ReadMap.h"
 #include "Map/MapInfo.h"
+#include "Rendering/GlobalRendering.h"
 #include "Rendering/UnitDrawer.h"
 #include "Rendering/GL/myGL.h"
-#include "Rendering/Env/BaseSky.h"
+#include "Rendering/Env/ISky.h"
 #include "Rendering/Env/CubeMapHandler.h"
-#include "System/ConfigHandler.h"
+#include "System/Config/ConfigHandler.h"
+
+CONFIG(int, CubeTexSizeSpecular).defaultValue(128);
+CONFIG(int, CubeTexSizeReflection).defaultValue(128);
 
 static char cameraMemBuf[sizeof(CCamera)];
 
-CubeMapHandler* CubeMapHandler::GetInstance() {
-	static CubeMapHandler cmh;
-	return &cmh;
-}
+CubeMapHandler* cubeMapHandler = NULL;
 
 CubeMapHandler::CubeMapHandler() {
 	envReflectionTexID = 0;
@@ -28,17 +29,19 @@ CubeMapHandler::CubeMapHandler() {
 	specTexSize = 0;
 
 	currReflectionFace = 0;
+	specularTexIter = 0;
 	mapSkyReflections = false;
+
+	specTexBuf = NULL;
 }
 
 bool CubeMapHandler::Init() {
-	specTexSize = configHandler->Get("CubeTexSizeSpecular", 128);
-	reflTexSize = configHandler->Get("CubeTexSizeReflection", 128);
+	specTexSize = configHandler->GetInt("CubeTexSizeSpecular");
+	reflTexSize = configHandler->GetInt("CubeTexSizeReflection");
+	specTexBuf = new unsigned char[specTexSize * 4];
 
 	mapSkyReflections = !(mapInfo->smf.skyReflectModTexName.empty());
 
-	const float specExp = configHandler->Get("CubeTexSpecularExponent", 100.0f);
-
 	{
 		glGenTextures(1, &specularTexID);
 		glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, specularTexID);
@@ -47,12 +50,12 @@ bool CubeMapHandler::Init() {
 		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
 		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 
-		CreateSpecularFace(GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB, specTexSize, float3( 1,  1,  1), float3( 0, 0, -2), float3(0, -2,  0), specExp);
-		CreateSpecularFace(GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB, specTexSize, float3(-1,  1, -1), float3( 0, 0,  2), float3(0, -2,  0), specExp);
-		CreateSpecularFace(GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB, specTexSize, float3(-1,  1, -1), float3( 2, 0,  0), float3(0,  0,  2), specExp);
-		CreateSpecularFace(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB, specTexSize, float3(-1, -1,  1), float3( 2, 0,  0), float3(0,  0, -2), specExp);
-		CreateSpecularFace(GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB, specTexSize, float3(-1,  1,  1), float3( 2, 0,  0), float3(0, -2,  0), specExp);
-		CreateSpecularFace(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB, specTexSize, float3( 1,  1, -1), float3(-2, 0,  0), float3(0, -2,  0), specExp);
+		CreateSpecularFace(GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB, specTexSize, float3( 1,  1,  1), float3( 0, 0, -2), float3(0, -2,  0));
+		CreateSpecularFace(GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB, specTexSize, float3(-1,  1, -1), float3( 0, 0,  2), float3(0, -2,  0));
+		CreateSpecularFace(GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB, specTexSize, float3(-1,  1, -1), float3( 2, 0,  0), float3(0,  0,  2));
+		CreateSpecularFace(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB, specTexSize, float3(-1, -1,  1), float3( 2, 0,  0), float3(0,  0, -2));
+		CreateSpecularFace(GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB, specTexSize, float3(-1,  1,  1), float3( 2, 0,  0), float3(0, -2,  0));
+		CreateSpecularFace(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB, specTexSize, float3( 1,  1, -1), float3(-2, 0,  0), float3(0, -2,  0));
 	}
 
 	{
@@ -115,6 +118,8 @@ void CubeMapHandler::Free() {
 		glDeleteTextures(1, &skyReflectionTexID);
 		skyReflectionTexID = 0;
 	}
+
+	delete [] specTexBuf;
 }
 
 
@@ -176,6 +181,7 @@ void CubeMapHandler::CreateReflectionFace(unsigned int glType, const float3& cam
 
 	glPushAttrib(GL_FOG_BIT | GL_DEPTH_BUFFER_BIT);
 	glViewport(0, 0, reflTexSize, reflTexSize);
+
 	if (skyOnly) {
 		glDepthMask(GL_FALSE);
 		glDisable(GL_DEPTH_TEST);
@@ -196,8 +202,8 @@ void CubeMapHandler::CreateReflectionFace(unsigned int glType, const float3& cam
 	if (camera->forward.y ==  1.0f) { camera->up = float3(0.0f, 0.0f,  1.0f); }
 	if (camera->forward.y == -1.0f) { camera->up = float3(0.0f, 0.0f, -1.0f); }
 
-	camera->pos.y = ground->GetHeight(camera->pos.x, camera->pos.z) + 50.0f;
-	camera->Update(false, false);
+	camera->pos.y = ground->GetHeightAboveWater(camera->pos.x, camera->pos.z, false) + 50.0f;
+	camera->Update(false);
 
 	sky->Draw();
 
@@ -214,34 +220,92 @@ void CubeMapHandler::CreateReflectionFace(unsigned int glType, const float3& cam
 
 	camera->~CCamera();
 	new (camera) CCamera(*(CCamera*) cameraMemBuf);
-	camera->Update(false);
+	camera->Update();
 }
 
 
-void CubeMapHandler::CreateSpecularFace(
-	unsigned int texType,
-	int size,
-	const float3& cdir,
-	const float3& xdif,
-	const float3& ydif,
-	float specExponent)
-{
-	unsigned char* buf = new unsigned char[size * size * 4];
+void CubeMapHandler::UpdateSpecularTexture() {
+	if (!unitDrawer->advShading) {
+		return;
+	}
 
-	for (int y = 0; y < size; ++y) {
-		for (int x = 0; x < size; ++x) {
-			const float3 dir = (cdir + (xdif * (x + 0.5f)) / size + (ydif * (y + 0.5f)) / size).Normalize();
-			const float dot = std::max(0.0f, dir.dot(mapInfo->light.sunDir));
-			const float spec = std::min(1.0f, pow(dot, specExponent) + pow(dot, 3.0f) * 0.25f);
+	glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, specularTexID);
 
-			buf[(y * size + x) * 4 + 0] = (unsigned char) (mapInfo->light.unitSpecularColor.x * spec * 255);
-			buf[(y * size + x) * 4 + 1] = (unsigned char) (mapInfo->light.unitSpecularColor.y * spec * 255);
-			buf[(y * size + x) * 4 + 2] = (unsigned char) (mapInfo->light.unitSpecularColor.z * spec * 255);
-			buf[(y * size + x) * 4 + 3] = 255;
+	int specularTexRow = specularTexIter / 3; //FIXME WTF
+
+	switch (specularTexIter % 3) {
+		case 0: {
+			UpdateSpecularFace(GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB, specTexSize, float3( 1,  1,  1), float3( 0, 0, -2), float3(0, -2,  0), specularTexRow, specTexBuf);
+			UpdateSpecularFace(GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB, specTexSize, float3(-1,  1, -1), float3( 0, 0,  2), float3(0, -2,  0), specularTexRow, specTexBuf);
+			break;
+		}
+		case 1: {
+			UpdateSpecularFace(GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB, specTexSize, float3(-1,  1, -1), float3( 2, 0,  0), float3(0,  0,  2), specularTexRow, specTexBuf);
+			UpdateSpecularFace(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB, specTexSize, float3(-1, -1,  1), float3( 2, 0,  0), float3(0,  0, -2), specularTexRow, specTexBuf);
+			break;
+		}
+		case 2: {
+			UpdateSpecularFace(GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB, specTexSize, float3(-1,  1,  1), float3( 2, 0,  0), float3(0, -2,  0), specularTexRow, specTexBuf);
+			UpdateSpecularFace(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB, specTexSize, float3( 1,  1, -1), float3(-2, 0,  0), float3(0, -2,  0), specularTexRow, specTexBuf);
+			break;
 		}
 	}
 
-	//! note: no mipmaps, cubemap linear filtering is broken
-	glTexImage2D(texType, 0, GL_RGBA8, size, size, 0, GL_RGBA,GL_UNSIGNED_BYTE, buf);
-	delete[] buf;
+	// update one face of one row per frame
+	++specularTexIter;
+	specularTexIter = specularTexIter % (specTexSize * 3);
 }
+
+void CubeMapHandler::CreateSpecularFacePart(
+	unsigned int texType,
+	unsigned int size,
+	const float3& cdir,
+	const float3& xdif,
+	const float3& ydif,
+	unsigned int y,
+	unsigned char* buf)
+{
+	// TODO move to a shader
+	for (int x = 0; x < size; ++x) {
+		const float3 dir = (cdir + (xdif * (x + 0.5f)) / size + (ydif * (y + 0.5f)) / size).Normalize();
+		const float dot  = std::max(0.0f, dir.dot(sky->GetLight()->GetLightDir()));
+		const float spec = std::min(1.0f, pow(dot, mapInfo->light.specularExponent) + pow(dot, 3.0f) * 0.25f);
+
+		buf[x * 4 + 0] = (mapInfo->light.unitSpecularColor.x * spec * 255);
+		buf[x * 4 + 1] = (mapInfo->light.unitSpecularColor.y * spec * 255);
+		buf[x * 4 + 2] = (mapInfo->light.unitSpecularColor.z * spec * 255);
+		buf[x * 4 + 3] = 255;
+	}
+}
+
+void CubeMapHandler::CreateSpecularFace(
+	unsigned int texType,
+	unsigned int size,
+	const float3& cdir,
+	const float3& xdif,
+	const float3& ydif)
+{
+	std::vector<unsigned char> buf(size * size * 4, 0);
+
+	for (int y = 0; y < size; ++y) {
+		CreateSpecularFacePart(texType, size, cdir, xdif, ydif, y, &buf[y * size * 4]);
+	}
+
+	//! note: no mipmaps, cubemap linear filtering is broken
+	glTexImage2D(texType, 0, GL_RGBA8, size, size, 0, GL_RGBA, GL_UNSIGNED_BYTE, &buf[0]);
+}
+
+void CubeMapHandler::UpdateSpecularFace(
+	unsigned int texType,
+	unsigned int size,
+	const float3& cdir,
+	const float3& xdif,
+	const float3& ydif,
+	unsigned int y,
+	unsigned char* buf)
+{
+	CreateSpecularFacePart(texType, size, cdir, xdif, ydif, y, buf);
+
+	glTexSubImage2D(texType, 0, 0, y, size, 1, GL_RGBA, GL_UNSIGNED_BYTE, buf);
+}
+
