@@ -1,8 +1,7 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 #include <algorithm>
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "GrassDrawer.h"
 #include "Game/Camera.h"
@@ -12,19 +11,22 @@
 #include "Map/ReadMap.h"
 #include "Rendering/GlobalRendering.h"
 #include "Rendering/ShadowHandler.h"
+#include "Rendering/Env/ISky.h"
 #include "Rendering/GL/myGL.h"
+#include "Rendering/GL/FBO.h"
 #include "Rendering/GL/VertexArray.h"
-#include "Rendering/Shaders/ShaderHandler.hpp"
-#include "Rendering/Shaders/Shader.hpp"
+#include "Rendering/Shaders/ShaderHandler.h"
+#include "Rendering/Shaders/Shader.h"
 #include "Rendering/Textures/Bitmap.h"
 #include "Sim/Misc/Wind.h"
 #include "System/myMath.h"
-#include "System/ConfigHandler.h"
+#include "System/Config/ConfigHandler.h"
 #include "System/Exceptions.h"
-#include "System/GlobalUnsynced.h"
 #include "System/Util.h"
 #include "System/FileSystem/FileHandler.h"
 
+CONFIG(int, GrassDetail).defaultValue(3);
+
 static const float turfSize        = 20.0f;            // single turf size
 static const float partTurfSize    = turfSize * 0.6f;  // single turf size
 static const int   grassSquareSize = 4;                // mapsquares per grass square
@@ -42,7 +44,7 @@ inline float fRand(float size)
 
 CGrassDrawer::CGrassDrawer()
 {
-	const int detail = configHandler->Get("GrassDetail", 3);
+	const int detail = configHandler->GetInt("GrassDetail");
 
 	if (detail == 0) {
 		grassOff = true;
@@ -57,7 +59,7 @@ CGrassDrawer::CGrassDrawer()
 	if (grassdata) {
 		if (grassbm.width != gs->mapx / grassSquareSize || grassbm.height != gs->mapy / grassSquareSize) {
 			char b[128];
-			SNPRINTF(b, sizeof(b), "grass-map has wrong size (%dx%d, should be %dx%d)\n", 
+			SNPRINTF(b, sizeof(b), "grass-map has wrong size (%dx%d, should be %dx%d)\n",
 				grassbm.width, grassbm.height, gs->mapx / 4, gs->mapy / 4);
 			throw std::runtime_error(b);
 		}
@@ -102,8 +104,9 @@ CGrassDrawer::CGrassDrawer()
 
 	{
 		CBitmap grassBladeTexBM;
-		if (!grassBladeTexBM.Load(mapInfo->smf.grassBladeTexName)) {
+		if (!grassBladeTexBM.Load(mapInfo->grass.grassBladeTexName)) {
 			//! map didn't define a grasstex, so generate one
+			grassBladeTexBM.channels = 4;
 			grassBladeTexBM.Alloc(256,64);
 
 			for (int a = 0; a < 16; ++a) {
@@ -147,6 +150,7 @@ CGrassDrawer::~CGrassDrawer(void)
 
 
 void CGrassDrawer::LoadGrassShaders() {
+	#define sh shaderHandler
 	grassShaders.resize(GRASS_PROGRAM_LAST, NULL);
 
 	static const std::string shaderNames[GRASS_PROGRAM_LAST] = {
@@ -179,17 +183,15 @@ void CGrassDrawer::LoadGrassShaders() {
 		"#define GRASS_ANIMATION\n":
 		"";
 
-	CShaderHandler* sh = shaderHandler;
-
 	if (!globalRendering->haveGLSL) {
 		grassShaders[GRASS_PROGRAM_DIST_BASIC] = sh->CreateProgramObject("[GrassDrawer]", shaderNames[GRASS_PROGRAM_DIST_BASIC] + "ARB", true);
 		grassShaders[GRASS_PROGRAM_DIST_BASIC]->AttachShaderObject(sh->CreateShaderObject("ARB/grassFarNS.vp", "", GL_VERTEX_PROGRAM_ARB));
 
-		// these remain stubs if !canUseShadows
+		// these remain stubs if !shadowsSupported
 		grassShaders[GRASS_PROGRAM_NEAR_SHADOW] = sh->CreateProgramObject("[GrassDrawer]", shaderNames[GRASS_PROGRAM_NEAR_SHADOW] + "ARB", true);
 		grassShaders[GRASS_PROGRAM_DIST_SHADOW] = sh->CreateProgramObject("[GrassDrawer]", shaderNames[GRASS_PROGRAM_DIST_SHADOW] + "ARB", true);
 
-		if (shadowHandler->canUseShadows) {
+		if (shadowHandler->shadowsSupported) {
 			grassShaders[GRASS_PROGRAM_NEAR_SHADOW]->AttachShaderObject(sh->CreateShaderObject("ARB/grass.vp", "", GL_VERTEX_PROGRAM_ARB));
 			grassShaders[GRASS_PROGRAM_DIST_SHADOW]->AttachShaderObject(sh->CreateShaderObject("ARB/grassFar.vp", "", GL_VERTEX_PROGRAM_ARB));
 		}
@@ -213,8 +215,11 @@ void CGrassDrawer::LoadGrassShaders() {
 			grassShaders[i]->SetUniform2f(0, 1.0f / (gs->pwr2mapx  * SQUARE_SIZE), 1.0f / (gs->pwr2mapy * SQUARE_SIZE));
 			grassShaders[i]->SetUniform2f(1, 1.0f / (gs->mapx      * SQUARE_SIZE), 1.0f / (gs->mapy     * SQUARE_SIZE));
 			grassShaders[i]->Disable();
+			grassShaders[i]->Validate();
 		}
 	}
+
+	#undef sh
 }
 
 
@@ -255,7 +260,7 @@ public:
 				for (int x2 = 0; x2 < grassBlockSize; ++x2) { //!loop over all squares in block
 					if (*gm) {
 						float3 squarePos((xgbsx + 0.5f) * gSSsq, 0.0f, (ygbsy + 0.5f) * gSSsq);
-							squarePos.y = ground->GetHeight2(squarePos.x, squarePos.z);
+							squarePos.y = ground->GetHeightReal(squarePos.x, squarePos.z, false);
 
 						/*if (!camera->InView(squarePos, gSSsq * 2.0f)) { //the QuadField visibility check should be enough
 							// double the radius of the check, grass on the left of
@@ -277,8 +282,8 @@ public:
 								const float dy = (ygbsy + fRand(1)) * gSSsq;
 								const float col = 0.62f;
 
-								float3 pos(dx, ground->GetHeight2(dx, dy), dy);
-									pos.y -= ground->GetSlope(dx, dy) * 10.0f + 0.03f;
+								float3 pos(dx, ground->GetHeightReal(dx, dy, false), dy);
+									pos.y -= ground->GetSlope(dx, dy, false) * 10.0f + 0.03f;
 
 								glColor3f(col, col, col);
 
@@ -343,7 +348,7 @@ public:
 
 			if (!grass->va) {
 				grass->va = new CVertexArray;;
-				grass->pos = float3((x + 0.5f) * bMSsq, ground->GetHeight2((x + 0.5f) * bMSsq, (y + 0.5f) * bMSsq), (y + 0.5f) * bMSsq);
+				grass->pos = float3((x + 0.5f) * bMSsq, ground->GetHeightReal((x + 0.5f) * bMSsq, (y + 0.5f) * bMSsq, false), (y + 0.5f) * bMSsq);
 
 				va = grass->va;
 				va->Initialize();
@@ -372,8 +377,8 @@ public:
 								const float dy = (ygbsy + fRand(1)) * gSSsq;
 								const float col = 1.0f;
 
-								float3 pos(dx, ground->GetHeight2(dx, dy) + 0.5f, dy);
-									pos.y -= (ground->GetSlope(dx, dy) * 10.0f + 0.03f);
+								float3 pos(dx, ground->GetHeightReal(dx, dy, false) + 0.5f, dy);
+									pos.y -= (ground->GetSlope(dx, dy, false) * 10.0f + 0.03f);
 
 								va->AddVertexQTN(pos, 0.0f,         0.0f, float3(-partTurfSize, -partTurfSize, col));
 								va->AddVertexQTN(pos, 1.0f / 16.0f, 0.0f, float3( partTurfSize, -partTurfSize, col));
@@ -417,7 +422,7 @@ void CGrassDrawer::Draw(void)
 	CBaseGroundDrawer* gd = readmap->GetGroundDrawer();
 	Shader::IProgramObject* grassShader = NULL;
 
-	if (shadowHandler->drawShadows && !gd->DrawExtraTex()) {
+	if (shadowHandler->shadowsLoaded && !gd->DrawExtraTex()) {
 		grassShader = grassShaders[GRASS_PROGRAM_NEAR_SHADOW];
 		grassShader->Enable();
 
@@ -449,8 +454,8 @@ void CGrassDrawer::Draw(void)
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE_ARB, GL_COMPARE_R_TO_TEXTURE);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FUNC_ARB, GL_LEQUAL);
 		glTexParameteri(GL_TEXTURE_2D, GL_DEPTH_TEXTURE_MODE_ARB, GL_ALPHA);
-		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FAIL_VALUE_ARB, 1.0f - mapInfo->light.groundShadowDensity);
-		
+		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FAIL_VALUE_ARB, 1.0f - sky->GetLight()->GetGroundShadowDensity());
+
 		static const float texConstant[] = {
 			mapInfo->light.groundAmbientColor.x * 1.24f,
 			mapInfo->light.groundAmbientColor.y * 1.24f,
@@ -485,9 +490,9 @@ void CGrassDrawer::Draw(void)
 
 		if (globalRendering->haveGLSL) {
 			grassShader->SetUniformMatrix4fv(6, false, &shadowHandler->shadowMatrix.m[0]);
-			grassShader->SetUniform4fv(7, const_cast<float*>(&(shadowHandler->GetShadowParams().x)));
+			grassShader->SetUniform4fv(7, &(shadowHandler->GetShadowParams().x));
 			grassShader->SetUniform1f(8, gs->frameNum);
-			grassShader->SetUniform3fv(9, const_cast<float*>(&(windSpeed.x)));
+			grassShader->SetUniform3fv(9, &(windSpeed.x));
 		} else {
 			glMatrixMode(GL_MATRIX0_ARB);
 			glLoadMatrixf(shadowHandler->shadowMatrix.m);
@@ -495,7 +500,7 @@ void CGrassDrawer::Draw(void)
 
 		glMatrixMode(GL_PROJECTION);
 		glPushMatrix();
-		glMultMatrixd(camera->GetViewMat());
+		glMultMatrixf(camera->GetViewMatrix());
 		glMatrixMode(GL_MODELVIEW);
 		glPushMatrix();
 		glLoadIdentity();
@@ -569,7 +574,7 @@ void CGrassDrawer::Draw(void)
 	glDepthMask(false);
 
 
-	if (shadowHandler->drawShadows && !gd->DrawExtraTex()) {
+	if (shadowHandler->shadowsLoaded && !gd->DrawExtraTex()) {
 		glActiveTextureARB(GL_TEXTURE3_ARB);
 		glBindTexture(GL_TEXTURE_2D, farTex);
 		glActiveTextureARB(GL_TEXTURE0_ARB);
@@ -585,9 +590,9 @@ void CGrassDrawer::Draw(void)
 
 		if (globalRendering->haveGLSL) {
 			grassShader->SetUniformMatrix4fv(6, false, &shadowHandler->shadowMatrix.m[0]);
-			grassShader->SetUniform4fv(7, const_cast<float*>(&(shadowHandler->GetShadowParams().x)));
+			grassShader->SetUniform4fv(7, &(shadowHandler->GetShadowParams().x));
 			grassShader->SetUniform1f(8, gs->frameNum);
-			grassShader->SetUniform3fv(9, const_cast<float*>(&(windSpeed.x)));
+			grassShader->SetUniform3fv(9, &(windSpeed.x));
 		}
 	} else {
 		grassShader = grassShaders[GRASS_PROGRAM_DIST_BASIC];
@@ -598,7 +603,7 @@ void CGrassDrawer::Draw(void)
 			grassShader->SetUniform2f(12, 1.0f / (gs->mapx     * SQUARE_SIZE), 1.0f / (gs->mapy     * SQUARE_SIZE));
 		} else {
 			grassShader->SetUniform1f(8, gs->frameNum);
-			grassShader->SetUniform3fv(9, const_cast<float*>(&(windSpeed.x)));
+			grassShader->SetUniform3fv(9, &(windSpeed.x));
 		}
 
 		glBindTexture(GL_TEXTURE_2D, farTex);
@@ -660,7 +665,7 @@ void CGrassDrawer::Draw(void)
 
 		if (grassMap[y * gs->mapx / grassSquareSize + x]) {
 			float3 squarePos((x + 0.5f) * gSSsq, 0.0f, (y + 0.5f) * gSSsq);
-				squarePos.y = ground->GetHeight2(squarePos.x, squarePos.z);
+				squarePos.y = ground->GetHeightReal(squarePos.x, squarePos.z, false);
 			const float3 billboardDirZ = (squarePos - camera->pos).ANormalize();
 			const float3 billboardDirX = (billboardDirZ.cross(UpVector)).ANormalize();
 			const float3 billboardDirY = billboardDirX.cross(billboardDirZ);
@@ -694,8 +699,8 @@ void CGrassDrawer::Draw(void)
 				const float dy = (y + fRand(1)) * gSSsq;
 				const float col = 1.0f;
 
-				float3 pos(dx, ground->GetHeight2(dx, dy) + 0.5f, dy);
-					pos.y -= (ground->GetSlope(dx, dy) * 10.0f + 0.03f);
+				float3 pos(dx, ground->GetHeightReal(dx, dy, false) + 0.5f, dy);
+					pos.y -= (ground->GetSlope(dx, dy, false) * 10.0f + 0.03f);
 
 				if (camera->InView(pos, turfSize * 0.7f)) {
 					va->AddVertexQTN(pos,         0.0f, 0.0f, float3(-partTurfSize, -partTurfSize, col));
@@ -719,7 +724,7 @@ void CGrassDrawer::Draw(void)
 
 	glDisable(GL_ALPHA_TEST);
 
-	if (shadowHandler->drawShadows && !gd->DrawExtraTex()) {
+	if (shadowHandler->shadowsLoaded && !gd->DrawExtraTex()) {
 		glActiveTextureARB(GL_TEXTURE0_ARB);
 		glTexEnvi(GL_TEXTURE_ENV, GL_RGB_SCALE_ARB, 1);
 		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
@@ -855,13 +860,18 @@ void CGrassDrawer::CreateGrassDispList(int listNum)
 
 void CGrassDrawer::CreateGrassBladeTex(unsigned char* buf)
 {
-	float3 col(0.59f+fRand(0.11f),0.81f+fRand(0.08f),0.57f+fRand(0.11f));
+	float3 col( mapInfo->grass.color + float3(fRand(0.11f),fRand(0.08f),fRand(0.11f)) );
+	col.x = Clamp(col.x, 0.f, 1.f);
+	col.y = Clamp(col.y, 0.f, 1.f);
+	col.z = Clamp(col.z, 0.f, 1.f);
+
 	for(int y=0;y<64;++y){
 		for(int x=0;x<16;++x){
-			buf[(y*256+x)*4+0]=(unsigned char) ((col.x*(0.4f+.6f*y/(64.0f)))*255);
-			buf[(y*256+x)*4+1]=(unsigned char) ((col.y*(0.4f+.6f*y/(64.0f)))*255);
-			buf[(y*256+x)*4+2]=(unsigned char) ((col.z*(0.4f+.6f*y/(64.0f)))*255);
-			buf[(y*256+x)*4+3]=1;
+			const float brightness = (0.4f + 0.6f * (y/64.0f)) * 255.f;
+			buf[(y*256+x)*4+0] = (unsigned char)(col.x * brightness);
+			buf[(y*256+x)*4+1] = (unsigned char)(col.y * brightness);
+			buf[(y*256+x)*4+2] = (unsigned char)(col.z * brightness);
+			buf[(y*256+x)*4+3] = 1;
 		}
 	}
 }
@@ -874,6 +884,12 @@ void CGrassDrawer::CreateFarTex(void)
 	memset(buf,0,64*sizeMod*1024*sizeMod*4);
 	memset(buf2,0,256*sizeMod*256*sizeMod*4);
 
+	FBO fbo;
+	fbo.Bind();
+	fbo.CreateRenderBuffer(GL_DEPTH_ATTACHMENT_EXT, GL_DEPTH_COMPONENT16, 256*sizeMod, 256*sizeMod);
+	fbo.CreateRenderBuffer(GL_COLOR_ATTACHMENT0_EXT, GL_RGBA8, 256*sizeMod, 256*sizeMod);
+	fbo.CheckStatus("GRASSDRAWER");
+
 	glPushMatrix();
 	glLoadIdentity();
 	glMatrixMode(GL_PROJECTION);
@@ -886,7 +902,7 @@ void CGrassDrawer::CreateFarTex(void)
 	glEnable(GL_TEXTURE_2D);
 	glDisable(GL_FOG);
 	glDisable(GL_BLEND);
-	glColor4f(1,1,1,1);			
+	glColor4f(1,1,1,1);
 	glViewport(0,0,256*sizeMod,256*sizeMod);
 
 	for(int a=0;a<16;++a){
@@ -902,7 +918,7 @@ void CGrassDrawer::CreateFarTex(void)
 		glOrtho(-partTurfSize, partTurfSize, -partTurfSize, partTurfSize, -turfSize, turfSize);
 
 		glCallList(grassDL);
-		
+
 		glReadPixels(0,0,256*sizeMod,256*sizeMod,GL_RGBA,GL_UNSIGNED_BYTE,buf2);
 
 //		CBitmap bm(buf2,512,512);
@@ -948,6 +964,8 @@ void CGrassDrawer::CreateFarTex(void)
 	glMatrixMode(GL_MODELVIEW);
 	glPopMatrix();
 
+	fbo.Unbind();
+
 	glGenTextures(1, &farTex);
 	glBindTexture(GL_TEXTURE_2D, farTex);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
