@@ -1,22 +1,24 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 #include "FeatureDrawer.h"
 
 #include "Game/Camera.h"
+#include "Game/GlobalUnsynced.h"
 #include "Lua/LuaRules.h"
 #include "Map/MapInfo.h"
 #include "Map/ReadMap.h"
 #include "Map/BaseGroundDrawer.h"
 #include "Rendering/GlobalRendering.h"
 #include "Rendering/FarTextureHandler.h"
-#include "Rendering/Env/BaseWater.h"
-#include "Rendering/Env/BaseTreeDrawer.h"
-#include "Rendering/GL/VertexArray.h"
+#include "Rendering/Env/ISky.h"
+#include "Rendering/Env/ITreeDrawer.h"
+#include "Rendering/Env/IWater.h"
+#include "Rendering/GL/glExtra.h"
 #include "Rendering/GL/myGL.h"
+#include "Rendering/GL/VertexArray.h"
 #include "Rendering/ShadowHandler.h"
-#include "Rendering/Shaders/Shader.hpp"
+#include "Rendering/Shaders/Shader.h"
 #include "Rendering/Textures/S3OTextureHandler.h"
 #include "Rendering/Textures/3DOTextureHandler.h"
 #include "Rendering/UnitDrawer.h"
@@ -24,14 +26,15 @@
 #include "Sim/Features/Feature.h"
 #include "Sim/Features/FeatureHandler.h"
 #include "Sim/Misc/GlobalSynced.h"
-#include "System/ConfigHandler.h"
+#include "System/Config/ConfigHandler.h"
 #include "System/EventHandler.h"
-#include "System/GlobalUnsynced.h"
 #include "System/myMath.h"
-
+#include "System/Util.h"
 
 #define DRAW_QUAD_SIZE 32
 
+CONFIG(bool, ShowRezBars).defaultValue(true);
+
 CFeatureDrawer* featureDrawer = NULL;
 
 /******************************************************************************/
@@ -54,9 +57,9 @@ CFeatureDrawer::CFeatureDrawer(): CEventClient("[CFeatureDrawer]", 313373, false
 	drawQuadsX = gs->mapx/DRAW_QUAD_SIZE;
 	drawQuadsY = gs->mapy/DRAW_QUAD_SIZE;
 	drawQuads.resize(drawQuadsX * drawQuadsY);
-
-	showRezBars = !!configHandler->Get("ShowRezBars", 1);
-
+#ifdef USE_GML
+	showRezBars = configHandler->GetBool("ShowRezBars");
+#endif
 	opaqueModelRenderers.resize(MODELTYPE_OTHER, NULL);
 	cloakedModelRenderers.resize(MODELTYPE_OTHER, NULL);
 
@@ -85,8 +88,10 @@ CFeatureDrawer::~CFeatureDrawer()
 void CFeatureDrawer::RenderFeatureCreated(const CFeature* feature)
 {
 	CFeature* f = const_cast<CFeature*>(feature);
+	texturehandlerS3O->UpdateDraw();
+
 #if defined(USE_GML) && GML_ENABLE_SIM
-	if(f->model && TEX_TYPE(f) < 0)
+	if(!gmlShareLists && f->model && TEX_TYPE(f) < 0)
 		TEX_TYPE(f) = texturehandlerS3O->LoadS3OTextureNow(f->model);
 #endif
 
@@ -129,10 +134,18 @@ void CFeatureDrawer::UpdateDrawQuad(CFeature* feature)
 {
 	const int oldDrawQuad = feature->drawQuad;
 	if (oldDrawQuad >= -1) {
-		const int newDrawQuad =
-			int(feature->pos.z / DRAW_QUAD_SIZE / SQUARE_SIZE) * drawQuadsX +
-			int(feature->pos.x / DRAW_QUAD_SIZE / SQUARE_SIZE);
+		int newDrawQuadX = feature->pos.x / DRAW_QUAD_SIZE / SQUARE_SIZE;
+		int newDrawQuadY = feature->pos.z / DRAW_QUAD_SIZE / SQUARE_SIZE;
+		newDrawQuadX = Clamp(newDrawQuadX, 0, drawQuadsX - 1);
+		newDrawQuadY = Clamp(newDrawQuadY, 0, drawQuadsY - 1);
+		const int newDrawQuad = newDrawQuadY * drawQuadsX + newDrawQuadX;
+
 		if (oldDrawQuad != newDrawQuad) {
+			//TODO check if out of map features get drawn, when the camera is outside of the map
+			//     (q: does DrawGround render the border quads in such cases?)
+			assert(oldDrawQuad < drawQuadsX * drawQuadsY);
+			assert(newDrawQuad < drawQuadsX * drawQuadsY);
+
 			if (oldDrawQuad >= 0)
 				drawQuads[oldDrawQuad].features.erase(feature);
 			drawQuads[newDrawQuad].features.insert(feature);
@@ -156,23 +169,20 @@ void CFeatureDrawer::Update()
 }
 
 
-void CFeatureDrawer::UpdateDrawPos(CFeature* f)
+inline void CFeatureDrawer::UpdateDrawPos(CFeature* f)
 {
 //#if defined(USE_GML) && GML_ENABLE_SIM
 //	f->drawPos = f->pos + (f->speed * ((float)globalRendering->lastFrameStart - (float)f->lastUnitUpdate) * globalRendering->weightedSpeedFactor);
 //#else
 	f->drawPos = f->pos + (f->speed * globalRendering->timeOffset);
 //#endif
-	f->drawMidPos = f->drawPos + (f->midPos - f->pos);
+	f->drawMidPos = f->drawPos + f->relMidPos;
 }
 
 
 void CFeatureDrawer::Draw()
 {
-	if(globalRendering->drawFog) {
-		glEnable(GL_FOG);
-		glFogfv(GL_FOG_COLOR, mapInfo->atmosphere.fogColor);
-	}
+	ISky::SetupFog();
 
 	GML_RECMUTEX_LOCK(feat); // Draw
 
@@ -215,33 +225,28 @@ void CFeatureDrawer::Draw()
 
 	glDisable(GL_TEXTURE_2D);
 	glDisable(GL_FOG);
-
+#ifdef USE_GML
 	DrawFeatureStats();
+#endif
 }
 
 void CFeatureDrawer::DrawOpaqueFeatures(int modelType)
 {
-	typedef std::set<CFeature*> FeatureSet;
-	typedef std::map<int, FeatureSet> FeatureRenderBin;
-
 	FeatureRenderBin& featureBin = opaqueModelRenderers[modelType]->GetFeatureBinMutable();
 
 	FeatureRenderBin::iterator featureBinIt;
 	FeatureSet::iterator featureSetIt;
 
 	for (featureBinIt = featureBin.begin(); featureBinIt != featureBin.end(); ++featureBinIt) {
-		if (modelType == MODELTYPE_S3O || modelType == MODELTYPE_OBJ) {
+		if (modelType != MODELTYPE_3DO) {
 			texturehandlerS3O->SetS3oTexture(featureBinIt->first);
 		}
 
 		FeatureSet& featureSet = featureBinIt->second;
 
 		for (featureSetIt = featureSet.begin(); featureSetIt != featureSet.end(); ) {
-			FeatureSet::iterator featureSetItNext(featureSetIt); ++featureSetItNext;
-
-			if (!DrawFeatureNow(*featureSetIt)) {
-				featureSet.erase(*featureSetIt);
-				featureSetIt = featureSetItNext;
+			if (!DrawFeatureNow(featureSetIt->first)) {
+				featureSetIt = set_erase(featureSet, featureSetIt);
 			} else {
 				++featureSetIt;
 			}
@@ -249,10 +254,11 @@ void CFeatureDrawer::DrawOpaqueFeatures(int modelType)
 	}
 }
 
+#ifdef USE_GML
 void CFeatureDrawer::DrawFeatureStats()
 {
 	if (!drawStat.empty()) {
-		if (!water->drawReflection) {
+		if (!water->IsDrawReflection()) {
 			for (std::vector<CFeature *>::iterator fi = drawStat.begin(); fi != drawStat.end(); ++fi) {
 				DrawFeatureStatBars(*fi);
 			}
@@ -269,7 +275,7 @@ void CFeatureDrawer::DrawFeatureStatBars(const CFeature* feature)
 
 	glPushMatrix();
 	glTranslatef(interPos.x, interPos.y, interPos.z);
-	glCallList(CCamera::billboardList);
+	glMultMatrixf(camera->GetBillBoardMatrix());
 
 	const float recl = feature->reclaimLeft;
 	const float rezp = feature->resurrectProgress;
@@ -296,16 +302,17 @@ void CFeatureDrawer::DrawFeatureStatBars(const CFeature* feature)
 
 	glPopMatrix();
 }
+#endif
 
-bool CFeatureDrawer::DrawFeatureNow(const CFeature* feature)
+bool CFeatureDrawer::DrawFeatureNow(const CFeature* feature, float alpha)
 {
-	if (!camera->InView(feature->pos, feature->radius * 4.0f)) { return false; }
+	if (!camera->InView(feature->pos, feature->drawRadius)) { return false; }
 	if (!feature->IsInLosForAllyTeam(gu->myAllyTeam) && !gu->spectatingFullView) { return false; }
 
 	glPushMatrix();
 	glMultMatrixf(feature->transMatrix.m);
 
-	unitDrawer->SetTeamColour(feature->team, feature->tempalpha);
+	unitDrawer->SetTeamColour(feature->team, alpha);
 
 	if (!(feature->luaDraw && luaRules && luaRules->DrawFeature(feature->id))) {
 		feature->model->DrawStatic();
@@ -339,10 +346,7 @@ void CFeatureDrawer::DrawFadeFeatures(bool noAdvShading)
 		glEnable(GL_ALPHA_TEST);
 		glAlphaFunc(GL_GREATER, 0.5f);
 
-		if (globalRendering->drawFog) {
-			glEnable(GL_FOG);
-			glFogfv(GL_FOG_COLOR, mapInfo->atmosphere.fogColor);
-		}
+		ISky::SetupFog();
 
 		{
 			GML_RECMUTEX_LOCK(feat); // DrawFadeFeatures
@@ -370,14 +374,10 @@ void CFeatureDrawer::DrawFadeFeatures(bool noAdvShading)
 
 void CFeatureDrawer::DrawFadeFeaturesHelper(int modelType) {
 	{
-		typedef std::set<CFeature*> FeatureSet;
-		typedef std::map<int, FeatureSet> FeatureRenderBin;
-		typedef std::map<int, FeatureSet>::iterator FeatureRenderBinIt;
-
 		FeatureRenderBin& featureBin = cloakedModelRenderers[modelType]->GetFeatureBinMutable();
 
 		for (FeatureRenderBinIt it = featureBin.begin(); it != featureBin.end(); ++it) {
-			if (modelType == MODELTYPE_S3O || modelType == MODELTYPE_OBJ) {
+			if (modelType != MODELTYPE_3DO) {
 				texturehandlerS3O->SetS3oTexture(it->first);
 			}
 
@@ -386,26 +386,23 @@ void CFeatureDrawer::DrawFadeFeaturesHelper(int modelType) {
 	}
 }
 
-void CFeatureDrawer::DrawFadeFeaturesSet(std::set<CFeature*>& fadeFeatures, int modelType)
+void CFeatureDrawer::DrawFadeFeaturesSet(FeatureSet& fadeFeatures, int modelType)
 {
-	for (std::set<CFeature*>::const_iterator fi = fadeFeatures.begin(); fi != fadeFeatures.end(); ) {
-		std::set<CFeature*>::const_iterator fiNext(fi); fiNext++;
+	for (FeatureSet::iterator fi = fadeFeatures.begin(); fi != fadeFeatures.end(); ) {
+		const float cols[] = {1.0f, 1.0f, 1.0f, fi->second};
 
-		const float cols[] = {1.0f, 1.0f, 1.0f, (*fi)->tempalpha};
-
-		if (modelType == MODELTYPE_S3O) {
+		if (modelType != MODELTYPE_3DO) {
 			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, cols);
 		}
 
 		// hack, sorting objects by distance would look better
-		glAlphaFunc(GL_GREATER, (*fi)->tempalpha / 2.0f);
+		glAlphaFunc(GL_GREATER, fi->second / 2.0f);
 		glColor4fv(cols);
 
-		if (!DrawFeatureNow(*fi)) {
-			fadeFeatures.erase(*fi);
-			fi = fiNext;
+		if (!DrawFeatureNow(fi->first, fi->second)) {
+			fi = set_erase(fadeFeatures, fi);
 		} else {
-			fi++;
+			++fi;
 		}
 	}
 }
@@ -415,7 +412,6 @@ void CFeatureDrawer::DrawFadeFeaturesSet(std::set<CFeature*>& fadeFeatures, int
 
 void CFeatureDrawer::DrawShadowPass()
 {
-	glDisable(GL_CULL_FACE); //FIXME: enable culling for s3o models
 	glPolygonOffset(1.0f, 1.0f);
 	glEnable(GL_POLYGON_OFFSET_FILL);
 
@@ -445,7 +441,14 @@ void CFeatureDrawer::DrawShadowPass()
 		// (we are just interested in the 255 alpha here)
 		glBindTexture(GL_TEXTURE_2D, 0);
 
-		for (int modelType = MODELTYPE_3DO; modelType < MODELTYPE_OTHER; modelType++) {
+		// 3DO's have clockwise-wound faces and
+		// (usually) holes, so disable backface
+		// culling for them
+		glDisable(GL_CULL_FACE);
+		DrawOpaqueFeatures(MODELTYPE_3DO);
+		glEnable(GL_CULL_FACE);
+
+		for (int modelType = MODELTYPE_S3O; modelType < MODELTYPE_OTHER; modelType++) {
 			DrawOpaqueFeatures(modelType);
 		}
 
@@ -456,7 +459,6 @@ void CFeatureDrawer::DrawShadowPass()
 	po->Disable();
 
 	glDisable(GL_POLYGON_OFFSET_FILL);
-	glEnable(GL_CULL_FACE);
 }
 
 
@@ -469,9 +471,11 @@ public:
 	float sqFadeDistBegin;
 	float sqFadeDistEnd;
 	bool farFeatures;
+#ifdef USE_GML
+	std::vector<CFeature*>* statFeatures;
+#endif
 
 	std::vector<CFeatureDrawer::DrawQuad>* drawQuads;
-	std::vector<CFeature*>* statFeatures;
 
 	void DrawQuad(int x, int y)
 	{
@@ -483,8 +487,7 @@ public:
 		for (std::set<CFeature*>::const_iterator fi = dq->features.begin(); fi != dq->features.end(); ++fi) {
 			CFeature* f = (*fi);
 
-			if (f->def->drawType != DRAWTYPE_MODEL)
-				continue;
+			assert(f->def->drawType == DRAWTYPE_MODEL);
 
 			if (gu->spectatingFullView || f->IsInLosForAllyTeam(gu->myAllyTeam)) {
 				if (drawReflection) {
@@ -498,7 +501,7 @@ public:
 							camera->pos * (f->midPos.y / dif) +
 							f->midPos * (-camera->pos.y / dif);
 					}
-					if (ground->GetApproximateHeight(zeroPos.x, zeroPos.z) > f->radius) {
+					if (ground->GetApproximateHeight(zeroPos.x, zeroPos.z, false) > f->drawRadius) {
 						continue;
 					}
 				}
@@ -509,9 +512,10 @@ public:
 
 				const float sqDist = (f->pos - camera->pos).SqLength();
 				const float farLength = f->sqRadius * unitDrawDist * unitDrawDist;
-
+#ifdef USE_GML
 				if (statFeatures && (f->reclaimLeft < 1.0f || f->resurrectProgress > 0.0f))
 					statFeatures->push_back(f);
+#endif
 
 				if (sqDist < farLength) {
 					float sqFadeDistE;
@@ -526,13 +530,12 @@ public:
 					}
 
 					if (sqDist < sqFadeDistB) {
-						f->tempalpha = 0.99f;
 						cloakedModelRenderers[MDL_TYPE(f)]->DelFeature(f);
 						opaqueModelRenderers[MDL_TYPE(f)]->AddFeature(f);
 					} else if (sqDist < sqFadeDistE) {
-						f->tempalpha = 1.0f - (sqDist - sqFadeDistB) / (sqFadeDistE - sqFadeDistB);
+						const float falpha = 1.0f - (sqDist - sqFadeDistB) / (sqFadeDistE - sqFadeDistB);
 						opaqueModelRenderers[MDL_TYPE(f)]->DelFeature(f);
-						cloakedModelRenderers[MDL_TYPE(f)]->AddFeature(f);
+						cloakedModelRenderers[MDL_TYPE(f)]->AddFeature(f, falpha);
 					}
 				} else {
 					if (farFeatures) {
@@ -558,18 +561,27 @@ void CFeatureDrawer::GetVisibleFeatures(int extraSize, bool drawFar)
 	CFeatureQuadDrawer drawer;
 	drawer.drawQuads = &drawQuads;
 	drawer.drawQuadsX = drawQuadsX;
-	drawer.drawReflection = water->drawReflection;
-	drawer.drawRefraction = water->drawRefraction;
+	drawer.drawReflection = water->IsDrawReflection();
+	drawer.drawRefraction = water->IsDrawRefraction();
 	drawer.unitDrawDist = unitDrawer->unitDrawDist;
 	drawer.sqFadeDistEnd = featureDist * featureDist;
 	drawer.sqFadeDistBegin = 0.75f * 0.75f * featureDist * featureDist;
 	drawer.farFeatures = drawFar;
+#ifdef USE_GML
 	drawer.statFeatures = showRezBars ? &drawStat : NULL;
+#endif
 
 	readmap->GridVisibility(camera, DRAW_QUAD_SIZE, featureDist, &drawer, extraSize);
 }
 
+void CFeatureDrawer::SwapFeatures() {
+	GML_RECMUTEX_LOCK(feat); // SwapFeatures
 
+	for(int i = 0; i < MODELTYPE_OTHER; ++i) {
+		opaqueModelRenderers[i]->SwapFeatures();
+		cloakedModelRenderers[i]->SwapFeatures();
+	}
+}
 
 void CFeatureDrawer::PostLoad()
 {
