@@ -1,27 +1,31 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 
-//#include <omp.h>
 #include <ostream>
 #include <fstream>
 #include <string.h>
 #include <IL/il.h>
 //#include <IL/ilu.h>
+#include <SDL_video.h>
 #include <boost/thread.hpp>
-#include "mmgr.h"
+#include "System/mmgr.h"
 
-#include "Rendering/GlobalRendering.h"
 #ifndef BITMAP_NO_OPENGL
-#include "Rendering/GL/myGL.h"
+	#include "Rendering/GL/myGL.h"
+	#include "System/TimeProfiler.h"
 #endif // !BITMAP_NO_OPENGL
-#include "FileSystem/FileHandler.h"
-#include "FileSystem/FileSystem.h"
-#include "GlobalUnsynced.h"
-#include "Bitmap.h"
-#include "bitops.h"
 
-boost::mutex devilMutex; // devil functions, whilst expensive, aren'T thread-save
+#include "Bitmap.h"
+#include "Rendering/GlobalRendering.h"
+#include "System/bitops.h"
+#include "System/Log/ILog.h"
+#include "System/OpenMP_cond.h"
+#include "System/FileSystem/DataDirsAccess.h"
+#include "System/FileSystem/FileQueryFlags.h"
+#include "System/FileSystem/FileHandler.h"
+
+
+boost::mutex devilMutex; // devil functions, whilst expensive, aren't thread-save
 
 static const float blurkernel[9] = {
 	1.0f/16.0f, 2.0f/16.0f, 1.0f/16.0f,
@@ -116,6 +120,17 @@ CBitmap& CBitmap::operator=(const CBitmap& bm)
 		delete[] mem;
 		mem = new unsigned char[size];
 		memcpy(mem, bm.mem, size);
+
+#ifndef BITMAP_NO_OPENGL
+		textype = bm.textype;
+		delete ddsimage;
+		if (bm.ddsimage == NULL) {
+			ddsimage = NULL;
+		} else {
+			ddsimage = new nv_dds::CDDSImage();
+			*ddsimage = *(bm.ddsimage);
+		}
+#endif // !BITMAP_NO_OPENGL
 	}
 
 	return *this;
@@ -127,7 +142,9 @@ void CBitmap::Alloc(int w, int h)
 	delete[] mem;
 	xsize = w;
 	ysize = h;
-	const int size = w*h*channels;
+
+	const int size = w * h * channels;
+
 	mem = new unsigned char[size];
 	memset(mem, 0, size);
 }
@@ -135,6 +152,10 @@ void CBitmap::Alloc(int w, int h)
 
 bool CBitmap::Load(std::string const& filename, unsigned char defaultAlpha)
 {
+#ifndef BITMAP_NO_OPENGL
+	ScopedTimer timer("Textures::CBitmap::Load");
+#endif
+
 	bool noAlpha = true;
 
 	delete[] mem;
@@ -145,13 +166,17 @@ bool CBitmap::Load(std::string const& filename, unsigned char defaultAlpha)
 #endif // !BITMAP_NO_OPENGL
 
 	if (filename.find(".dds") != std::string::npos) {
+		bool status = false;
+
+#ifndef BITMAP_NO_OPENGL
 		type = BitmapTypeDDS;
 		xsize = 0;
 		ysize = 0;
 		channels = 0;
-#ifndef BITMAP_NO_OPENGL
+
 		ddsimage = new nv_dds::CDDSImage();
-		bool status = ddsimage->load(filename);
+		status = ddsimage->load(filename);
+
 		if (status) {
 			xsize = ddsimage->get_width();
 			ysize = ddsimage->get_height();
@@ -171,17 +196,16 @@ bool CBitmap::Load(std::string const& filename, unsigned char defaultAlpha)
 					break;
 			}
 		}
-		return status;
-#else
-		return false;
 #endif // !BITMAP_NO_OPENGL
+		return status;
 	}
+
 	type = BitmapTypeStandardRGBA;
 	channels = 4;
 
 	CFileHandler file(filename);
 	if (file.FileExists() == false) {
-		Alloc(1,1);
+		Alloc(1, 1);
 		return false;
 	}
 
@@ -198,7 +222,7 @@ bool CBitmap::Load(std::string const& filename, unsigned char defaultAlpha)
 
 	const bool success = !!ilLoadL(IL_TYPE_UNKNOWN, buffer, file.FileSize());
 	ilDisable(IL_ORIGIN_SET);
-	delete [] buffer;
+	delete[] buffer;
 
 	if (success == false) {
 		xsize = 1;
@@ -234,7 +258,7 @@ bool CBitmap::Load(std::string const& filename, unsigned char defaultAlpha)
 }
 
 
-bool CBitmap::LoadGrayscale (const std::string& filename)
+bool CBitmap::LoadGrayscale(const std::string& filename)
 {
 	type = BitmapTypeStandardAlpha;
 	channels = 1;
@@ -268,16 +292,17 @@ bool CBitmap::LoadGrayscale (const std::string& filename)
 	ysize = ilGetInteger(IL_IMAGE_HEIGHT);
 
 	delete[] mem;
+	mem = NULL; // to prevent a dead-pointer in case of an out-of-memory exception on the next line
 	mem = new unsigned char[xsize * ysize];
 	memcpy(mem, ilGetData(), xsize * ysize);
-	
+
 	ilDeleteImages(1, &ImageName);
-	
+
 	return true;
 }
 
 
-bool CBitmap::Save(std::string const& filename, bool opaque)
+bool CBitmap::Save(std::string const& filename, bool opaque) const
 {
 	if (type == BitmapTypeDDS) {
 #ifndef BITMAP_NO_OPENGL
@@ -316,7 +341,7 @@ bool CBitmap::Save(std::string const& filename, bool opaque)
 
 	ilTexImage(xsize, ysize, 1, 4, IL_RGBA, IL_UNSIGNED_BYTE, buf);
 
-	const std::string fullpath = filesystem.LocateFile(filename, FileSystem::WRITE);
+	const std::string fullpath = dataDirsAccess.LocateFile(filename, FileQueryFlags::WRITE);
 	const bool success = ilSaveImage((char*)fullpath.c_str());
 
 	ilDeleteImages(1, &ImageName);
@@ -328,8 +353,10 @@ bool CBitmap::Save(std::string const& filename, bool opaque)
 
 
 #ifndef BITMAP_NO_OPENGL
-const GLuint CBitmap::CreateTexture(bool mipmaps)
+const unsigned int CBitmap::CreateTexture(bool mipmaps) const
 {
+	ScopedTimer timer("Textures::CBitmap::CreateTexture");
+
 	if (type == BitmapTypeDDS) {
 		return CreateDDSTexture();
 	}
@@ -347,7 +374,7 @@ const GLuint CBitmap::CreateTexture(bool mipmaps)
 		return bm.CreateTexture(mipmaps);
 	}
 
-	GLuint texture;
+	unsigned int texture;
 
 	glGenTextures(1, &texture);
 	glBindTexture(GL_TEXTURE_2D, texture);
@@ -374,7 +401,7 @@ const GLuint CBitmap::CreateTexture(bool mipmaps)
 }
 
 
-const GLuint CBitmap::CreateDDSTexture(GLuint texID)
+const unsigned int CBitmap::CreateDDSTexture(unsigned int texID) const
 {
 	glPushAttrib(GL_TEXTURE_BIT);
 
@@ -392,7 +419,7 @@ const GLuint CBitmap::CreateDDSTexture(GLuint texID)
 		glEnable(GL_TEXTURE_2D);
 		glBindTexture(GL_TEXTURE_2D, texID);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
-		if (!ddsimage->upload_texture2D()) {
+		if (!ddsimage->upload_texture2D(0, GL_TEXTURE_2D)) {
 			glDeleteTextures(1, &texID);
 			texID = 0;
 		}
@@ -458,36 +485,27 @@ void CBitmap::CreateAlpha(unsigned char red, unsigned char green, unsigned char
 			aCol[a] = cCol / 255.0f / numCounted;
 		}
 	}
-	for (int y=0; y < ysize; ++y) {
-		for (int x=0; x < xsize; ++x) {
-			const int index = (y*xsize + x) * 4;
-			if ((mem[index + 0] == red) &&
-				(mem[index + 1] == green) &&
-				(mem[index + 2] == blue))
-			{
-				mem[index + 0] = (unsigned char) (aCol.x * 255);
-				mem[index + 1] = (unsigned char) (aCol.y * 255);
-				mem[index + 2] = (unsigned char) (aCol.z * 255);
-				mem[index + 3] = 0;
-			}
-		}
-	}
+
+	SColor c(red, green, blue);
+	SColor a(aCol.x, aCol.y, aCol.z, 0.0f);
+	SetTransparent(c, a);
 }
 
 
-// Depreciated (Only used by GUI which will be replaced by CEGUI anyway)
-void CBitmap::SetTransparent(unsigned char red, unsigned char green, unsigned char blue)
+void CBitmap::SetTransparent(const SColor& c, const SColor& trans)
 {
-	for (unsigned int y = 0; y < xsize; y++) {
-		for (unsigned int x = 0; x < xsize; x++) {
-			const unsigned int index = (y*xsize + x) * 4;
-			if ((mem[index + 0] == red) &&
-				(mem[index + 1] == green) &&
-				(mem[index + 2] == blue))
-			{
-				// set transparent
-				mem[index + 3] = 0;
-			}
+	if (type != BitmapTypeStandardRGBA) {
+		return;
+	}
+
+	static const uint32_t RGB = 0x00FFFFFF;
+
+	uint32_t* mem_i = reinterpret_cast<uint32_t*>(mem);
+	for (unsigned int y = 0; y < ysize; ++y) {
+		for (unsigned int x = 0; x < xsize; ++x) {
+			if ((*mem_i & RGB) == (c.i & RGB))
+				*mem_i = trans.i;
+			mem_i++;
 		}
 	}
 }
@@ -496,10 +514,7 @@ void CBitmap::SetTransparent(unsigned char red, unsigned char green, unsigned ch
 void CBitmap::Renormalize(float3 newCol)
 {
 	float3 aCol;
-	//	float3 aSpread;
-
 	float3 colorDif;
-	//	float3 spreadMul;
 	for (int a=0; a < 3; ++a) {
 		int cCol = 0;
 		int numCounted = 0;
@@ -515,25 +530,12 @@ void CBitmap::Renormalize(float3 newCol)
 		aCol[a] = cCol / 255.0f / numCounted;
 		cCol /= xsize*ysize;
 		colorDif[a] = newCol[a] - aCol[a];
-
-/*		int spread=0;
-        for(int y=0;y<ysize;++y){
-        for(int x=0;x<xsize;++x){
-        if(mem[(y*xsize+x)*4+3]!=0){
-        int dif=mem[(y*xsize+x)*4+a]-cCol;
-        spread+=abs(dif);
-        }
-        }
-        }
-        aSpread.xyz[a]=spread/255.0f/numCounted;
-        spreadMul.xyz[a]=(float)(newSpread[a]/aSpread[a]);*/
 	}
 	for (int a=0; a < 3; ++a) {
 		for (int y=0; y < ysize; ++y) {
 			for (int x=0; x < xsize; ++x) {
 				const unsigned int index = (y*xsize + x) * 4;
 				float nc = float(mem[index + a]) / 255.0f + colorDif[a];
-				//float r=newCol.xyz[a]+(nc-newCol.xyz[a])*spreadMul.xyz[a];
 				mem[index + a] = (unsigned char) (std::min(255.f, std::max(0.0f, nc*255)));
 			}
 		}
@@ -541,7 +543,8 @@ void CBitmap::Renormalize(float3 newCol)
 }
 
 
-inline void kernelBlur(CBitmap* dst, const unsigned char* src, int x, int y, int channel, float weight) {
+inline static void kernelBlur(CBitmap* dst, const unsigned char* src, int x, int y, int channel, float weight)
+{
 	float fragment = 0.0f;
 
 	const int pos = (x + y * dst->xsize) * dst->channels + channel;
@@ -582,26 +585,23 @@ void CBitmap::Blur(int iterations, float weight)
 	dst->Alloc(xsize,ysize);
 
 	for (int i=0; i < iterations; ++i){
-		#pragma omp parallel private(y,x,i)
 		{
-			#pragma omp for
-			for (int y=0; y < ysize; ++y) {
-				for (int x=0; x < xsize; ++x) {
-					for (int i=0; i < channels; ++i) {
-						kernelBlur(dst, src->mem, x, y, i, weight);
+			int j,y,x;
+			#pragma omp parallel for private(j,x,y)
+			for (y=0; y < ysize; y++) {
+				for (x=0; x < xsize; x++) {
+					for (j=0; j < channels; j++) {
+						kernelBlur(dst, src->mem, x, y, j, weight);
 					}
 				}
 			}
 		}
-		CBitmap* buf = dst;
-		dst = src;
-		src = buf;
+		std::swap(src, dst);
 	}
 
 	if (dst == this) {
-		CBitmap* buf = dst;
-		dst = src;
-		src = buf;
+		// make sure we don't delete `this`
+		std::swap(src, dst);
 	}
 
 	delete dst;
@@ -609,7 +609,7 @@ void CBitmap::Blur(int iterations, float weight)
 
 
 // Unused
-CBitmap CBitmap::GetRegion(int startx, int starty, int width, int height)
+CBitmap CBitmap::GetRegion(int startx, int starty, int width, int height) const
 {
 	CBitmap bm;
 
@@ -632,48 +632,58 @@ CBitmap CBitmap::GetRegion(int startx, int starty, int width, int height)
 }
 
 
-CBitmap CBitmap::CreateMipmapLevel(void)
+void CBitmap::CopySubImage(const CBitmap& src, unsigned int xpos, unsigned int ypos)
 {
-	CBitmap bm;
-
-	delete[] bm.mem;
-	bm.xsize = xsize / 2;
-	bm.ysize = ysize / 2;
-	bm.mem = new unsigned char[bm.xsize*bm.ysize * 4];
-
-	for (int y=0; y < (ysize / 2); ++y) {
-		for (int x=0; x < (xsize / 2); ++x) {
-			float r=0, g=0, b=0, a=0;
-			for (int y2=0; y2 < 2; ++y2) {
-				for (int x2=0; x2 < 2; ++x2) {
-					const int index = ((y*2 + y2)*xsize + x*2 + x2) * 4;
-					r += mem[index + 0];
-					g += mem[index + 1];
-					b += mem[index + 2];
-					a += mem[index + 3];
-				}
-			}
-			const int index = (y*bm.xsize + x) * 4;
-			bm.mem[index]     = (unsigned char) (r / 4);
-			bm.mem[index + 1] = (unsigned char) (g / 4);
-			bm.mem[index + 2] = (unsigned char) (b / 4);
-			bm.mem[index + 3] = (unsigned char) (a / 4);
-		}
+	if (xpos + src.xsize >= xsize || ypos + src.ysize >= ysize) {
+		LOG_L(L_WARNING, "CBitmap::CopySubImage src image does not fit into dst");
+		return;
 	}
 
-	return bm;
+	if (src.type != BitmapTypeStandardRGBA || type != BitmapTypeStandardRGBA) {
+		return;
+	}
 
+	for (int y=0; y < src.ysize; ++y) {
+		const int pixelDst = (((ypos + y) * xsize) + xpos) * channels;
+		const int pixelSrc = ((y * src.xsize) + 0 ) * channels;
+
+		// copy the whole line
+		memcpy(mem + pixelDst, src.mem + pixelSrc, channels * src.xsize);
+	}
 }
 
 
-CBitmap CBitmap::CreateRescaled(int newx, int newy)
+SDL_Surface* CBitmap::CreateSDLSurface(bool newPixelData) const
 {
-	CBitmap bm;
+	SDL_Surface* surface = NULL;
 
-	delete[] bm.mem;
-	bm.xsize = newx;
-	bm.ysize = newy;
-	bm.mem   = new unsigned char[bm.xsize*bm.ysize * 4];
+	unsigned char* surfData = NULL;
+	if (newPixelData) {
+		// copy pixel data
+		surfData = new unsigned char[xsize * ysize * channels];
+		memcpy(surfData, mem, xsize * ysize * channels);
+	} else {
+		surfData = mem;
+	}
+
+	// This will only work with 24bit RGB and 32bit RGBA pictures
+	surface = SDL_CreateRGBSurfaceFrom(surfData, xsize, ysize, 8 * channels, xsize * channels, 0x000000FF, 0x0000FF00, 0x00FF0000, (channels == 4) ? 0xFF000000 : 0);
+	if ((surface == NULL) && newPixelData) {
+		// cleanup when we failed to the create surface
+		delete[] surfData;
+	}
+
+	return surface;
+}
+
+
+CBitmap CBitmap::CreateRescaled(int newx, int newy) const
+{
+	newx = std::max(1, newx);
+	newy = std::max(1, newy);
+
+	CBitmap bm;
+	bm.Alloc(newx, newy);
 
 	const float dx = (float) xsize / newx;
 	const float dy = (float) ysize / newy;
@@ -736,6 +746,18 @@ void CBitmap::InvertColors()
 }
 
 
+void CBitmap::InvertAlpha()
+{
+	if (type != BitmapTypeStandardRGBA) return; // Don't try to invert DDS
+	for (int y = 0; y < ysize; ++y) {
+		for (int x = 0; x < xsize; ++x) {
+			const int base = ((y * xsize) + x) * 4;
+			mem[base + 3] = 0xFF - mem[base + 3];
+		}
+	}
+}
+
+
 void CBitmap::GrayScale()
 {
 	if (type != BitmapTypeStandardRGBA) {
@@ -784,15 +806,16 @@ void CBitmap::Tint(const float tint[3])
 
 void CBitmap::ReverseYAxis()
 {
-	unsigned char tmpPixel[channels];
+    if (type != BitmapTypeStandardRGBA) return; // don't try to flip DDS
+	unsigned char* tmpLine = new unsigned char[channels * xsize];
 	for (int y=0; y < (ysize / 2); ++y) {
-		for (int x=0; x < xsize; ++x) {
-			const int pixelLow  = (((y            ) * xsize) + x) * channels;
-			const int pixelHigh = (((ysize - 1 - y) * xsize) + x) * channels;
-			// copy all channels at once
-			memcpy(tmpPixel,        mem + pixelHigh, channels);
-			memcpy(mem + pixelHigh, mem + pixelLow,  channels);
-			memcpy(mem + pixelLow,  tmpPixel,        channels);
-		}
+		const int pixelLow  = (((y            ) * xsize) + 0) * channels;
+		const int pixelHigh = (((ysize - 1 - y) * xsize) + 0) * channels;
+
+		// copy the whole line
+		memcpy(tmpLine,         mem + pixelHigh, channels * xsize);
+		memcpy(mem + pixelHigh, mem + pixelLow,  channels * xsize);
+		memcpy(mem + pixelLow,  tmpLine,         channels * xsize);
 	}
+	delete[] tmpLine;
 }
