@@ -1,15 +1,14 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "NamedTextures.h"
 
-#include "bitops.h"
-#include "Rendering/GlobalRendering.h"
 #include "Rendering/GL/myGL.h"
 #include "Bitmap.h"
-#include "System/GlobalUnsynced.h"
+#include "Rendering/GlobalRendering.h"
+#include "System/bitops.h"
+#include "System/TimeProfiler.h"
 #include "System/Vec2.h"
 
 
@@ -26,6 +25,8 @@ void CNamedTextures::Init()
 
 void CNamedTextures::Kill()
 {
+	GML_STDMUTEX_LOCK(ntex); // Kill
+
 	map<string, TexInfo>::iterator it;
 	for (it = texMap.begin(); it != texMap.end(); ++it) {
 		const GLuint texID = it->second.id;
@@ -42,6 +43,8 @@ bool CNamedTextures::Bind(const string& texName)
 		return false;
 	}
 
+	GML_STDMUTEX_LOCK(ntex); // Bind
+
 	map<string, TexInfo>::iterator it = texMap.find(texName);
 	if (it != texMap.end()) {
 		const GLuint texID = it->second.id;
@@ -76,8 +79,10 @@ bool CNamedTextures::Bind(const string& texName)
 }
 
 
-bool CNamedTextures::Load(const string& texName, GLuint texID)
+bool CNamedTextures::Load(const string& texName, unsigned int texID)
 {
+	ScopedTimer timer("Textures::NamedTextures");
+
 	//! strip off the qualifiers
 	string filename = texName;
 	bool border  = false;
@@ -242,11 +247,13 @@ bool CNamedTextures::Load(const string& texName, GLuint texID)
 
 void CNamedTextures::Update()
 {
+	GML_STDMUTEX_LOCK(ntex); // Update
+
 	if (texWaiting.empty()) {
 		return;
 	}
 	glPushAttrib(GL_TEXTURE_BIT);
-	for (std::vector<string>::iterator it = texWaiting.begin(); it != texWaiting.end(); it++) {
+	for (std::vector<string>::iterator it = texWaiting.begin(); it != texWaiting.end(); ++it) {
 		map<string, TexInfo>::iterator mit = texMap.find(*it);
 		if (mit != texMap.end()) {
 			Load(*it,mit->second.id);
@@ -262,6 +269,9 @@ bool CNamedTextures::Free(const string& texName)
 	if (texName.empty()) {
 		return false;
 	}
+
+	GML_STDMUTEX_LOCK(ntex); // Free
+
 	map<string, TexInfo>::iterator it = texMap.find(texName);
 	if (it != texMap.end()) {
 		const GLuint texID = it->second.id;
@@ -278,6 +288,9 @@ const CNamedTextures::TexInfo* CNamedTextures::GetInfo(const string& texName)
 	if (texName.empty()) {
 		return false;
 	}
+
+	GML_STDMUTEX_LOCK(ntex); // GetInfo
+
 	map<string, TexInfo>::const_iterator it = texMap.find(texName);
 	if (it != texMap.end()) {
 		return &it->second;
