@@ -7,8 +7,6 @@
 #include <vector>
 #include <string>
 
-#include "Rendering/GL/myGL.h"
-
 using std::map;
 using std::string;
 using std::vector;
@@ -18,8 +16,12 @@ class CNamedTextures {
 		static void Init();
 		static void Kill();
 
-		//! reload textures we couldn't load cause Bind() was called when compiling a DList
-		//! (else it would reupload the texturedata each call of the dlist, so we delay it and load them here)
+		/**
+		 * Reload textures we could not load because Bind() was called
+		 * when compiling a DList.
+		 * Otherwise, it would re-upload the texture-data on each call
+		 * of the DList, so we delay it and load them here.
+		 */
 		static void Update();
 
 		static bool Bind(const string& texName);
@@ -27,7 +29,7 @@ class CNamedTextures {
 
 		struct TexInfo {
 			TexInfo()
-			: id(0), type(-1), xsize(-1), ysize(-1), alpha(false) {}
+				: id(0), type(-1), xsize(-1), ysize(-1), alpha(false) {}
 			unsigned int id;
 			int type;
 			int xsize;
@@ -38,7 +40,7 @@ class CNamedTextures {
 		static const TexInfo* GetInfo(const string& texName);
 
 	private:
-		static bool Load(const string& texName, GLuint texID);
+		static bool Load(const string& texName, unsigned int texID);
 
 		static map<string, TexInfo> texMap;
 		static vector<string> texWaiting;
