@@ -1,6 +1,5 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 
 #include "glFont.h"
 #include <string>
@@ -17,21 +16,30 @@
 #include "Rendering/GlobalRendering.h"
 #include "Rendering/GL/VertexArray.h"
 #include "Rendering/Textures/Bitmap.h"
-#include "System/LogOutput.h"
+#include "System/Log/ILog.h"
 #include "System/myMath.h"
 #include "System/FileSystem/FileHandler.h"
 #include "System/FileSystem/FileSystem.h"
-#include "System/GlobalUnsynced.h"
 #include "System/Util.h"
 #include "System/Exceptions.h"
 #include "System/mmgr.h"
 #include "System/float4.h"
-#include "bitops.h"
+#include "System/bitops.h"
 
 #undef GetCharWidth // winapi.h
 
 using std::string;
 
+
+#define LOG_SECTION_FONT "Font"
+LOG_REGISTER_SECTION_GLOBAL(LOG_SECTION_FONT)
+
+// use the specific section for all LOG*() calls in this source file
+#ifdef LOG_SECTION_CURRENT
+	#undef LOG_SECTION_CURRENT
+#endif
+#define LOG_SECTION_CURRENT LOG_SECTION_FONT
+
 /*******************************************************************************/
 /*******************************************************************************/
 
@@ -319,7 +327,7 @@ void CFontTextureRenderer::ApproximateTextureWidth(int* width, int* height)
 	 */
 	unsigned int numPixelsAvg = (numPixels + numPixels2) / 2;
 
-	*width  = next_power_of_2(math::ceil(streflop::sqrtf( (float)numPixelsAvg )));
+	*width  = next_power_of_2(math::ceil(math::sqrtf( (float)numPixelsAvg )));
 	*height = next_power_of_2(math::ceil( (float)numPixelsAvg / (float)*width ));
 
 	if (*width > 2048)
@@ -410,11 +418,12 @@ CglFont::CglFont(const std::string& fontfile, int size, int _outlinewidth, float
 	autoOutlineColor(true),
 	setColor(false)
 {
+	va  = new CVertexArray();
+	va2 = new CVertexArray();
+
 	if (size<=0)
 		size = 14;
 
-	const float invSize = 1.0f / size;
-	const float normScale = invSize / 64.0f;
 
 	//! setup character range
 	charstart = 32;
@@ -422,6 +431,8 @@ CglFont::CglFont(const std::string& fontfile, int size, int _outlinewidth, float
 	chars     = (charend - charstart) + 1;
 
 #ifndef   HEADLESS
+	const float invSize = 1.0f / size;
+	const float normScale = invSize / 64.0f;
 	FT_Library library;
 	FT_Face face;
 
@@ -577,11 +588,11 @@ CglFont* CglFont::LoadFont(const std::string& fontFile, int size, int outlinewid
 	try {
 		CglFont* newFont = new CglFont(fontFile, size, outlinewidth, outlineweight);
 		return newFont;
-	} catch (texture_size_exception&) {
-		logOutput.Print("FONT-ERROR: Couldn't create GlyphAtlas! (try to reduce reduce font size/outlinewidth)");
+	} catch (const texture_size_exception& ex) {
+		LOG_L(L_ERROR, "Failed creating font: Could not create GlyphAtlas! (try to reduce the font size/outline-width)");
 		return NULL;
-	} catch (content_error& e) {
-		logOutput.Print(std::string(e.what()));
+	} catch (const content_error& ex) {
+		LOG_L(L_ERROR, "Failed creating font: %s", ex.what());
 		return NULL;
 	}
 }
@@ -591,8 +602,8 @@ CglFont::~CglFont()
 {
 	glDeleteTextures(1, &fontTexture);
 
-	stripTextColors.clear();
-	stripOutlineColors.clear();
+	delete va;
+	delete va2;
 }
 
 
@@ -1095,7 +1106,7 @@ void CglFont::AddEllipsis(std::list<line>& lines, std::list<word>& words, float
 
 	//! sometimes words aren't hyphenated for visual aspects
 	//! but if we put an ellipsis in there, it is better to show as many as possible characters of those words
-	std::list<word>::iterator nextwi(l->end); nextwi++;
+	std::list<word>::iterator nextwi(l->end); ++nextwi;
 	if (
 	    (!l->forceLineBreak) &&
 	    (nextwi != words.end()) &&
@@ -1125,7 +1136,7 @@ void CglFont::AddEllipsis(std::list<line>& lines, std::list<word>& words, float
 			space.numSpaces = 1;
 			space.width = spaceAdvance;
 			std::list<word>::iterator wi(l->end);
-			l->end++;
+			++l->end;
 			if (l->end == words.end()) {
 				space.pos = wi->pos + wi->text.length() + 1;
 			} else {
@@ -1141,7 +1152,7 @@ void CglFont::AddEllipsis(std::list<line>& lines, std::list<word>& words, float
 	ellipsis.text  = "\x85";
 	ellipsis.width = ellipsisAdvance;
 	std::list<word>::iterator wi(l->end);
-	l->end++;
+	++l->end;
 	if (l->end == words.end()) {
 		ellipsis.pos = wi->pos + wi->text.length() + 1;
 	} else {
@@ -1154,9 +1165,8 @@ void CglFont::AddEllipsis(std::list<line>& lines, std::list<word>& words, float
 
 void CglFont::WrapTextConsole(std::list<word>& words, float maxWidth, float maxHeight) const
 {
-	if (words.empty())
+	if (words.empty() || (lineHeight<=0.0f))
 		return;
-
 	const bool splitAllWords = false;
 	const unsigned int maxLines = (unsigned int)math::floor(std::max(0.0f, maxHeight / lineHeight ));
 
@@ -1186,7 +1196,7 @@ void CglFont::WrapTextConsole(std::list<word>& words, float maxWidth, float maxH
 				currLineValid = false;
 				currLine = &(lines.back());
 				currLine->start = wi;
-				currLine->start++;
+				++currLine->start;
 		} else {
 			currLine->width += wi->width;
 			currLine->end = wi;
@@ -1202,6 +1212,7 @@ void CglFont::WrapTextConsole(std::list<word>& words, float maxWidth, float maxH
 					//! last word W is larger than 0.5 * maxLineWidth, split it into
 					//! get 'L'eft and 'R'ight parts of the split (wL becomes Left, *wi becomes R)
 
+					bool restart = (currLine->start == wi);
 					//! turns *wi into R
 					word wL = SplitWord(*wi, freeWordSpace);
 
@@ -1217,7 +1228,9 @@ void CglFont::WrapTextConsole(std::list<word>& words, float maxWidth, float maxH
 
 					//! insert the L-part right before R
 					wi = words.insert(wi, wL);
-					wi++;
+					if(restart)
+						currLine->start = wi;
+					++wi;
 				}
 
 				//! insert the forced linebreak (either after W or before R)
@@ -1231,11 +1244,11 @@ void CglFont::WrapTextConsole(std::list<word>& words, float maxWidth, float maxH
 					currLineValid = false;
 					currLine = &(lines.back());
 					currLine->start = wi;
-					wi--; //! compensate the wi++ downwards
+					--wi; //! compensate the wi++ downwards
 			}
 		}
 
-		wi++;
+		++wi;
 
 		if (wi == words.end()) {
 			break;
@@ -1259,14 +1272,14 @@ void CglFont::WrapTextConsole(std::list<word>& words, float maxWidth, float maxH
 	if (addEllipsis)
 		AddEllipsis(lines, words, maxWidth);
 
-	wi = currLine->end; wi++;
+	wi = currLine->end; ++wi;
 	wi = words.erase(wi, words.end());
 }
 
 
 void CglFont::WrapTextKnuth(std::list<word>& words, float maxWidth, float maxHeight) const
 {
-	//todo: FINISH ME!!! (Knuths algorithm would try to share deadspace between lines, with the smallest sum of (deadspace of line)^2)
+	// TODO FINISH ME!!! (Knuths algorithm would try to share deadspace between lines, with the smallest sum of (deadspace of line)^2)
 }
 
 
@@ -1372,7 +1385,7 @@ void CglFont::RemergeColorCodes(std::list<word>* words, std::list<colorcode>& co
 	for (ci = colorcodes.begin(); ci != colorcodes.end(); ++ci) {
 		while(wi != words->end() && wi->pos <= ci->pos) {
 			wi2 = wi;
-			wi++;
+			++wi;
 		}
 
 		word wc;
@@ -1391,7 +1404,7 @@ void CglFont::RemergeColorCodes(std::list<word>* words, std::list<colorcode>& co
 
 		if (wi2->isSpace || wi2->isLineBreak) {
 			while(wi2 != words->end() && (wi2->isSpace || wi2->isLineBreak))
-				wi2++;
+				++wi2;
 
 			if (wi == words->end() && (wi2->pos + wi2->numSpaces) < ci->pos) {
 				return;
@@ -1417,14 +1430,14 @@ void CglFont::RemergeColorCodes(std::list<word>* words, std::list<colorcode>& co
 			}
 		}
 		wi = wi2;
-		wi++;
+		++wi;
 	}
 }
 
 
 int CglFont::WrapInPlace(std::string& text, float _fontSize, const float maxWidth, const float maxHeight) const
 {
-	//todo: make an option to insert '-' for word wrappings (and perhaps try to syllabificate)
+	// TODO make an option to insert '-' for word wrappings (and perhaps try to syllabificate)
 
 	if (_fontSize <= 0.0f)
 		_fontSize = fontSize;
@@ -1443,7 +1456,7 @@ int CglFont::WrapInPlace(std::string& text, float _fontSize, const float maxWidt
 	//! create the wrapped string
 	text = "";
 	unsigned int numlines = 0;
-	if (words.size() > 0) {
+	if (!words.empty()) {
 		numlines++;
 		for (std::list<word>::iterator wi = words.begin(); wi != words.end(); ++wi) {
 			if (wi->isSpace) {
@@ -1465,7 +1478,7 @@ int CglFont::WrapInPlace(std::string& text, float _fontSize, const float maxWidt
 
 std::list<std::string> CglFont::Wrap(const std::string& text, float _fontSize, const float maxWidth, const float maxHeight) const
 {
-	//todo: make an option to insert '-' for word wrappings (and perhaps try to syllabificate)
+	// TODO make an option to insert '-' for word wrappings (and perhaps try to syllabificate)
 
 	if (_fontSize <= 0.0f)
 		_fontSize = fontSize;
@@ -1484,7 +1497,7 @@ std::list<std::string> CglFont::Wrap(const std::string& text, float _fontSize, c
 	//! create the string lines of the wrapped text
 	std::list<word>::iterator lastColorCode = words.end();
 	std::list<std::string> strlines;
-	if (words.size() > 0) {
+	if (!words.empty()) {
 		strlines.push_back("");
 		std::string* sl = &strlines.back();
 		for (std::list<word>::iterator wi = words.begin(); wi != words.end(); ++wi) {
@@ -1523,9 +1536,9 @@ void CglFont::SetTextColor(const float4* color)
 	if (color == NULL) color = &white;
 
 	if (inBeginEnd && !(*color==textColor)) {
-		if ((va.stripArrayPos - va.stripArray) != (va.drawArrayPos - va.drawArray)) {
+		if ((va->stripArrayPos - va->stripArray) != (va->drawArrayPos - va->drawArray)) {
 			stripTextColors.push_back(*color);
-			va.EndStrip();
+			va->EndStrip();
 		} else {
 			float4& back = stripTextColors.back();
 			back = *color;
@@ -1541,9 +1554,9 @@ void CglFont::SetOutlineColor(const float4* color)
 	if (color == NULL) color = ChooseOutlineColor(textColor);
 
 	if (inBeginEnd && !(*color==outlineColor)) {
-		if ((va2.stripArrayPos - va2.stripArray) != (va2.drawArrayPos - va2.drawArray)) {
+		if ((va2->stripArrayPos - va2->stripArray) != (va2->drawArrayPos - va2->drawArray)) {
 			stripOutlineColors.push_back(*color);
-			va2.EndStrip();
+			va2->EndStrip();
 		} else {
 			float4& back = stripOutlineColors.back();
 			back = *color;
@@ -1561,18 +1574,18 @@ void CglFont::SetColors(const float4* _textColor, const float4* _outlineColor)
 
 	if (inBeginEnd) {
 		if (!(*_textColor==textColor)) {
-			if ((va.stripArrayPos - va.stripArray) != (va.drawArrayPos - va.drawArray)) {
+			if ((va->stripArrayPos - va->stripArray) != (va->drawArrayPos - va->drawArray)) {
 				stripTextColors.push_back(*_textColor);
-				va.EndStrip();
+				va->EndStrip();
 			} else {
 				float4& back = stripTextColors.back();
 				back = *_textColor;
 			}
 		}
 		if (!(*_outlineColor==outlineColor)) {
-			if ((va2.stripArrayPos - va2.stripArray) != (va2.drawArrayPos - va2.drawArray)) {
+			if ((va2->stripArrayPos - va2->stripArray) != (va2->drawArrayPos - va2->drawArray)) {
 				stripOutlineColors.push_back(*_outlineColor);
-				va2.EndStrip();
+				va2->EndStrip();
 			} else {
 				float4& back = stripOutlineColors.back();
 				back = *_outlineColor;
@@ -1604,7 +1617,7 @@ const float4* CglFont::ChooseOutlineColor(const float4& textColor)
 void CglFont::Begin(const bool immediate, const bool resetColors)
 {
 	if (inBeginEnd) {
-		logOutput.Print("FontError: called Begin() multiple times");
+		LOG_L(L_ERROR, "called Begin() multiple times");
 		return;
 	}
 
@@ -1617,8 +1630,8 @@ void CglFont::Begin(const bool immediate, const bool resetColors)
 
 	inBeginEnd = true;
 
-	va.Initialize();
-	va2.Initialize();
+	va->Initialize();
+	va2->Initialize();
 	stripTextColors.clear();
 	stripOutlineColors.clear();
 	stripTextColors.push_back(textColor);
@@ -1629,12 +1642,12 @@ void CglFont::Begin(const bool immediate, const bool resetColors)
 void CglFont::End()
 {
 	if (!inBeginEnd) {
-		logOutput.Print("FontError: called End() without Begin()");
+		LOG_L(L_ERROR, "called End() without Begin()");
 		return;
 	}
 	inBeginEnd = false;
 
-	if (va.drawIndex()==0) {
+	if (va->drawIndex() == 0) {
 		return;
 	}
 
@@ -1646,22 +1659,22 @@ void CglFont::End()
 	glEnable(GL_BLEND);
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
-	if (va2.drawIndex() > 0) {
+	if (va2->drawIndex() > 0) {
 		if (stripOutlineColors.size() > 1) {
 			ColorMap::iterator sci = stripOutlineColors.begin();
-			va2.DrawArray2dT(GL_QUADS,TextStripCallback,&sci);
+			va2->DrawArray2dT(GL_QUADS,TextStripCallback,&sci);
 		} else {
 			glColor4fv(outlineColor);
-			va2.DrawArray2dT(GL_QUADS);
+			va2->DrawArray2dT(GL_QUADS);
 		}
 	}
 
 	if (stripTextColors.size() > 1) {
 		ColorMap::iterator sci = stripTextColors.begin();
-		va.DrawArray2dT(GL_QUADS,TextStripCallback,&sci);
+		va->DrawArray2dT(GL_QUADS,TextStripCallback,&sci);
 	} else {
 		if (setColor) glColor4fv(textColor);
-		va.DrawArray2dT(GL_QUADS);
+		va->DrawArray2dT(GL_QUADS);
 	}
 
 	glPopAttrib();
@@ -1686,7 +1699,7 @@ void CglFont::RenderString(float x, float y, const float& scaleX, const float& s
 	const float lineHeight_ = scaleY * lineHeight;
 	unsigned int length = (unsigned int)str.length();
 
-	va.EnlargeArrays(length * 16 * sizeof(float), 0);
+	va->EnlargeArrays(length * 16 * sizeof(float), 0);
 
 	int skippedLines;
 	bool endOfString, colorChanged;
@@ -1721,10 +1734,10 @@ void CglFont::RenderString(float x, float y, const float& scaleX, const float& s
 
 		g = &glyphs[*c];
 
-		va.AddVertex2dQT(x+scaleX*g->x0, y+scaleY*g->y1, g->u0, g->v1);
-		va.AddVertex2dQT(x+scaleX*g->x0, y+scaleY*g->y0, g->u0, g->v0);
-		va.AddVertex2dQT(x+scaleX*g->x1, y+scaleY*g->y0, g->u1, g->v0);
-		va.AddVertex2dQT(x+scaleX*g->x1, y+scaleY*g->y1, g->u1, g->v1);
+		va->AddVertex2dQT(x+scaleX*g->x0, y+scaleY*g->y1, g->u0, g->v1);
+		va->AddVertex2dQT(x+scaleX*g->x0, y+scaleY*g->y0, g->u0, g->v0);
+		va->AddVertex2dQT(x+scaleX*g->x1, y+scaleY*g->y0, g->u1, g->v0);
+		va->AddVertex2dQT(x+scaleX*g->x1, y+scaleY*g->y1, g->u1, g->v1);
 	} while(true);
 }
 
@@ -1738,8 +1751,8 @@ void CglFont::RenderStringShadow(float x, float y, const float& scaleX, const fl
 	const float lineHeight_ = scaleY * lineHeight;
 	unsigned int length = (unsigned int)str.length();
 
-	va.EnlargeArrays(length * 16 * sizeof(float), 0);
-	va2.EnlargeArrays(length * 16 * sizeof(float), 0);
+	va->EnlargeArrays(length * 16 * sizeof(float), 0);
+	va2->EnlargeArrays(length * 16 * sizeof(float), 0);
 
 	int skippedLines;
 	bool endOfString, colorChanged;
@@ -1778,16 +1791,16 @@ void CglFont::RenderStringShadow(float x, float y, const float& scaleX, const fl
 		const float dx1 = x + scaleX * g->x1, dy1 = y + scaleY * g->y1;
 
 		//! draw shadow
-		va2.AddVertex2dQT(dx0+shiftX-ssX, dy1-shiftY-ssY, g->us0, g->vs1);
-		va2.AddVertex2dQT(dx0+shiftX-ssX, dy0-shiftY+ssY, g->us0, g->vs0);
-		va2.AddVertex2dQT(dx1+shiftX+ssX, dy0-shiftY+ssY, g->us1, g->vs0);
-		va2.AddVertex2dQT(dx1+shiftX+ssX, dy1-shiftY-ssY, g->us1, g->vs1);
+		va2->AddVertex2dQT(dx0+shiftX-ssX, dy1-shiftY-ssY, g->us0, g->vs1);
+		va2->AddVertex2dQT(dx0+shiftX-ssX, dy0-shiftY+ssY, g->us0, g->vs0);
+		va2->AddVertex2dQT(dx1+shiftX+ssX, dy0-shiftY+ssY, g->us1, g->vs0);
+		va2->AddVertex2dQT(dx1+shiftX+ssX, dy1-shiftY-ssY, g->us1, g->vs1);
 
 		//! draw the actual character
-		va.AddVertex2dQT(dx0, dy1, g->u0, g->v1);
-		va.AddVertex2dQT(dx0, dy0, g->u0, g->v0);
-		va.AddVertex2dQT(dx1, dy0, g->u1, g->v0);
-		va.AddVertex2dQT(dx1, dy1, g->u1, g->v1);
+		va->AddVertex2dQT(dx0, dy1, g->u0, g->v1);
+		va->AddVertex2dQT(dx0, dy0, g->u0, g->v0);
+		va->AddVertex2dQT(dx1, dy0, g->u1, g->v0);
+		va->AddVertex2dQT(dx1, dy1, g->u1, g->v1);
 	} while(true);
 }
 
@@ -1800,8 +1813,8 @@ void CglFont::RenderStringOutlined(float x, float y, const float& scaleX, const
 	const float lineHeight_ = scaleY * lineHeight;
 	unsigned int length = (unsigned int)str.length();
 
-	va.EnlargeArrays(length * 16 * sizeof(float), 0);
-	va2.EnlargeArrays(length * 16 * sizeof(float), 0);
+	va->EnlargeArrays(length * 16 * sizeof(float), 0);
+	va2->EnlargeArrays(length * 16 * sizeof(float), 0);
 
 	int skippedLines;
 	bool endOfString, colorChanged;
@@ -1840,34 +1853,34 @@ void CglFont::RenderStringOutlined(float x, float y, const float& scaleX, const
 		const float dx1 = x + scaleX * g->x1, dy1 = y + scaleY * g->y1;
 
 		//! draw outline
-		va2.AddVertex2dQT(dx0-shiftX, dy1-shiftY, g->us0, g->vs1);
-		va2.AddVertex2dQT(dx0-shiftX, dy0+shiftY, g->us0, g->vs0);
-		va2.AddVertex2dQT(dx1+shiftX, dy0+shiftY, g->us1, g->vs0);
-		va2.AddVertex2dQT(dx1+shiftX, dy1-shiftY, g->us1, g->vs1);
+		va2->AddVertex2dQT(dx0-shiftX, dy1-shiftY, g->us0, g->vs1);
+		va2->AddVertex2dQT(dx0-shiftX, dy0+shiftY, g->us0, g->vs0);
+		va2->AddVertex2dQT(dx1+shiftX, dy0+shiftY, g->us1, g->vs0);
+		va2->AddVertex2dQT(dx1+shiftX, dy1-shiftY, g->us1, g->vs1);
 
 		//! draw the actual character
-		va.AddVertex2dQT(dx0, dy1, g->u0, g->v1);
-		va.AddVertex2dQT(dx0, dy0, g->u0, g->v0);
-		va.AddVertex2dQT(dx1, dy0, g->u1, g->v0);
-		va.AddVertex2dQT(dx1, dy1, g->u1, g->v1);
+		va->AddVertex2dQT(dx0, dy1, g->u0, g->v1);
+		va->AddVertex2dQT(dx0, dy0, g->u0, g->v0);
+		va->AddVertex2dQT(dx1, dy0, g->u1, g->v0);
+		va->AddVertex2dQT(dx1, dy1, g->u1, g->v1);
 	} while(true);
 }
 
 
-void CglFont::glWorldPrint(const float3 p, const float size, const std::string& str)
+void CglFont::glWorldPrint(const float3& p, const float size, const std::string& str)
 {
 
 	glPushMatrix();
 		glTranslatef(p.x, p.y, p.z);
-		glCallList(CCamera::billboardList);
-		Begin(false,false);
+		glMultMatrixf(camera->GetBillBoardMatrix());
+		Begin(false, false);
 			glPrint(0.0f, 0.0f, size, FONT_DESCENDER | FONT_CENTER | FONT_OUTLINE, str);
 		End();
 	glPopMatrix();
 }
 
 
-void CglFont::glPrint(GLfloat x, GLfloat y, float s, const int& options, const std::string& text)
+void CglFont::glPrint(float x, float y, float s, const int& options, const std::string& text)
 {
 	//! s := scale or absolute size?
 	if (options & FONT_SCALE) {
@@ -1916,7 +1929,7 @@ void CglFont::glPrint(GLfloat x, GLfloat y, float s, const int& options, const s
 		y = (int)y;
 	}
 
-	//! backup text & outline colors (also ::ColorResetIndicator will reset to those)
+	// backup text & outline colors (also ::ColorResetIndicator will reset to those)
 	baseTextColor = textColor;
 	baseOutlineColor = outlineColor;
 
@@ -1946,7 +1959,7 @@ void CglFont::glPrint(GLfloat x, GLfloat y, float s, const int& options, const s
 	SetColors(&baseTextColor,&baseOutlineColor);
 }
 
-void CglFont::glPrintTable(GLfloat x, GLfloat y, float s, const int& options, const std::string& text) {
+void CglFont::glPrintTable(float x, float y, float s, const int& options, const std::string& text) {
 	int col = 0;
 	int row = 0;
 	std::vector<std::string> coltext;
@@ -1975,7 +1988,7 @@ void CglFont::glPrintTable(GLfloat x, GLfloat y, float s, const int& options, co
 				--pos;
 				break;
 
-			//! column separator is `\t`==`horizontal tab`
+			// column separator is `\t`==`horizontal tab`
 			case '\x09':
 				++col;
 				if(col >= coltext.size()) {
@@ -2083,14 +2096,14 @@ void CglFont::glPrintTable(GLfloat x, GLfloat y, float s, const int& options, co
 		VSNPRINTF(out, sizeof(out), fmt, ap);  \
 		va_end(ap);
 
-void CglFont::glFormat(GLfloat x, GLfloat y, float s, const int& options, const char* fmt, ...)
+void CglFont::glFormat(float x, float y, float s, const int& options, const char* fmt, ...)
 {
 	FORMAT_STRING(fmt,fmt,text);
 	glPrint(x, y, s, options, string(text));
 }
 
 
-void CglFont::glFormat(GLfloat x, GLfloat y, float s, const int& options, const string& fmt, ...)
+void CglFont::glFormat(float x, float y, float s, const int& options, const string& fmt, ...)
 {
 	FORMAT_STRING(fmt,fmt.c_str(),text);
 	glPrint(x, y, s, options, string(text));
