@@ -6,11 +6,14 @@
 #include <string>
 #include <map>
 
-#include "float3.h"
+#include "System/float3.h"
+
+enum {
+	DRAWTYPE_MODEL = 0,
+	DRAWTYPE_TREE  = 1, // >= different types of trees
+	DRAWTYPE_NONE = -1,
+};
 
-#define DRAWTYPE_MODEL 0
-#define DRAWTYPE_TREE 1 // >= different types of trees
-#define DRAWTYPE_NONE -1
 
 
 struct S3DModel;
@@ -20,21 +23,14 @@ struct FeatureDef
 {
 	CR_DECLARE_STRUCT(FeatureDef);
 
-	FeatureDef()
-		: collisionVolume(NULL)
-		, id(-1)
-		, metal(0), energy(0), maxHealth(0), reclaimTime(0), mass(0),
-		upright(false), drawType(0), model(NULL),
-		resurrectable(false), smokeTime(0), destructable(false), reclaimable(true), autoreclaim(true), blocking(false),
-		burnable(false), floating(false), noSelect(false), geoThermal(false),
-		xsize(0), zsize(0) {}
+	FeatureDef();
+	~FeatureDef();
 
 	S3DModel* LoadModel() const;
 	CollisionVolume* collisionVolume;
 
 	std::string myName;
 	std::string description;
-	std::string filename;
 
 	int id;
 
@@ -42,17 +38,15 @@ struct FeatureDef
 	float energy;
 	float maxHealth;
 	float reclaimTime;
-
 	/// used to see if the object can be overrun
 	float mass;
 
-	bool upright;
 	int drawType;
-	S3DModel* model;
+	mutable S3DModel* model;
 	std::string modelname;
 
 	/// -1 := only if it is the 1st wreckage of the unitdef (default), 0 := no it isn't, 1 := yes it is
-	int  resurrectable;
+	int resurrectable;
 
 	int smokeTime;
 
@@ -63,15 +57,15 @@ struct FeatureDef
 	bool burnable;
 	bool floating;
 	bool noSelect;
-
 	bool geoThermal;
+	bool upright;
 
 	/// name of feature that this turn into when killed (not reclaimed)
 	std::string deathFeature;
 
-	/// each size is 8 units
+	/// each size is SQUARE_SIZE units
 	int xsize;
-	/// each size is 8 units
+	/// each size is SQUARE_SIZE units
 	int zsize;
 
 	std::map<std::string, std::string> customParams;
