@@ -1,12 +1,18 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 #include "CollisionVolume.h"
-#include "System/LogOutput.h"
+#include "System/Log/ILog.h"
 #include "System/mmgr.h"
 
 
-static CLogSubsystem LOG_COLVOL("CollisionVolume");
+#define LOG_SECTION_COLVOL "CollisionVolume"
+LOG_REGISTER_SECTION_GLOBAL(LOG_SECTION_COLVOL)
+
+// use the specific section for all LOG*() calls in this source file
+#ifdef LOG_SECTION_CURRENT
+	#undef LOG_SECTION_CURRENT
+#endif
+#define LOG_SECTION_CURRENT LOG_SECTION_COLVOL
 
 
 CR_BIND(CollisionVolume, );
@@ -80,39 +86,57 @@ CollisionVolume::CollisionVolume(const CollisionVolume* v, float defaultRadius)
 	}
 }
 
-CollisionVolume::CollisionVolume(const std::string& volTypeStr, const float3& scales, const float3& offsets, int hitTestType)
+CollisionVolume::CollisionVolume(const std::string& volTypeString, const float3& scales, const float3& offsets, int hitTestType)
 {
 	int volType = COLVOL_TYPE_FOOTPRINT;
 	int volAxis = COLVOL_AXIS_Z;
 
-	if (!volTypeStr.empty()) {
-		const std::string volTypeStrLC(StringToLower(volTypeStr));
+	if (!volTypeString.empty()) {
+		const std::string& volTypeStr = StringToLower(volTypeString);
+		const std::string& volTypePrefix = volTypeStr.substr(0, 3);
 
-		if (volTypeStrLC.find("ell") != std::string::npos) {
+		if (volTypePrefix == "ell") {
 			volType = COLVOL_TYPE_ELLIPSOID;
-		} else if (volTypeStrLC.find("cyl") != std::string::npos) {
+		} else if (volTypePrefix == "cyl") {
 			volType = COLVOL_TYPE_CYLINDER;
 
-			if (volTypeStrLC.size() == 4) {
-				switch (volTypeStrLC[3]) {
-					case 'x': { volAxis = COLVOL_AXIS_X; } break;
-					case 'y': { volAxis = COLVOL_AXIS_Y; } break;
-					case 'z': { volAxis = COLVOL_AXIS_Z; } break;
-					default: {} break;
-				}
+			switch (volTypeStr[volTypeStr.size() - 1]) {
+				case 'x': { volAxis = COLVOL_AXIS_X; } break;
+				case 'y': { volAxis = COLVOL_AXIS_Y; } break;
+				case 'z': { volAxis = COLVOL_AXIS_Z; } break;
+				default: {} break;
 			}
-		} else if (volTypeStrLC.find("box") != std::string::npos) {
+		} else if (volTypePrefix == "box") {
 			volType = COLVOL_TYPE_BOX;
 		}
 	}
 
+	const char* typeStr = NULL;
 	switch (volType) {
-		case COLVOL_TYPE_ELLIPSOID: { logOutput.Print(LOG_COLVOL, "New ellipsoid"); } break;
-		case COLVOL_TYPE_CYLINDER:  { logOutput.Print(LOG_COLVOL, "New cylinder");  } break;
-		case COLVOL_TYPE_BOX:       { logOutput.Print(LOG_COLVOL, "New box");       } break;
-		case COLVOL_TYPE_FOOTPRINT: { logOutput.Print(LOG_COLVOL, "New footprint"); } break;
+		case COLVOL_TYPE_ELLIPSOID: {
+			typeStr = "ellipsoid";
+		} break;
+		case COLVOL_TYPE_CYLINDER: {
+			typeStr = "cylinder";
+		} break;
+		case COLVOL_TYPE_BOX: {
+			typeStr = "cuboid";
+		} break;
+		case COLVOL_TYPE_FOOTPRINT: {
+			typeStr = "footprint";
+		} break;
 		default: {} break;
 	}
+	if (typeStr != NULL) {
+		LOG_L(L_DEBUG,
+				"%s (scale: <%.2f, %.2f, %.2f>, "
+				"offsets: <%.2f, %.2f, %.2f>, "
+				"test-type: %d, axis: %d)",
+				typeStr,
+				scales.x, scales.y, scales.z,
+				offsets.x, offsets.y, offsets.z,
+				hitTestType, volAxis);
+	}
 
 	Init(scales, offsets, volType, hitTestType, volAxis);
 }
@@ -121,9 +145,8 @@ CollisionVolume::CollisionVolume(const std::string& volTypeStr, const float3& sc
 
 void CollisionVolume::Init(float r)
 {
-	// called iif a unit or feature does not define a custom volume;
-	// <s> is the object's default RADIUS (not its diameter), so we
-	// need to double it to get the full-length scales
+	// <r> is the object's default RADIUS (not its diameter),
+	// so we need to double it to get the full-length scales
 	Init(float3(r * 2.0f, r * 2.0f, r * 2.0f), ZeroVector, volumeType, testType, primaryAxis);
 }
 
@@ -143,20 +166,18 @@ void CollisionVolume::Init(const float3& scales, const float3& offsets, int vTyp
 	// or zero; if the resulting vector is <1, 1, 1>,
 	// then the unit / feature loaders will override
 	// the (clone) scales with the model's radius
-	SetAxisScales(std::max(1.0f, scales.x), std::max(1.0f, scales.y), std::max(1.0f, scales.z));
+	const float3 adjScales(std::max(1.0f, scales.x), std::max(1.0f, scales.y), std::max(1.0f, scales.z));
 
 	if (volumeType == COLVOL_TYPE_ELLIPSOID) {
-		// if all axes (or half-axes) are equal in scale,
-		// volume is a sphere (a special-case ellipsoid)
-		if ((streflop::fabsf(axisHScales.x - axisHScales.y) < EPS) &&
-		    (streflop::fabsf(axisHScales.y - axisHScales.z) < EPS)) {
-
-			logOutput.Print(LOG_COLVOL, "auto-converting spherical COLVOL_TYPE_ELLIPSOID to COLVOL_TYPE_SPHERE");
+		// if all axes are equal in scale, volume is a sphere (a special-case ellipsoid)
+		if ((math::fabsf(adjScales.x - adjScales.y) < EPS) &&
+		    (math::fabsf(adjScales.y - adjScales.z) < EPS))
+		{
+			LOG_L(L_DEBUG, "auto-converting spherical COLVOL_TYPE_ELLIPSOID to COLVOL_TYPE_SPHERE");
 			volumeType = COLVOL_TYPE_SPHERE;
 		}
 	}
 
-
 	// secondaryAxes[0] = (primaryAxis + 1) % COLVOL_NUM_AXES;
 	// secondaryAxes[1] = (primaryAxis + 2) % COLVOL_NUM_AXES;
 
@@ -175,9 +196,10 @@ void CollisionVolume::Init(const float3& scales, const float3& offsets, int vTyp
 		} break;
 	}
 
-	SetBoundingRadius();
+	SetAxisScales(adjScales.x,adjScales.y,adjScales.z);
 }
 
+
 void CollisionVolume::SetBoundingRadius() {
 	// set the radius of the minimum bounding sphere
 	// that encompasses this custom collision volume
@@ -212,7 +234,7 @@ void CollisionVolume::SetBoundingRadius() {
 	}
 }
 
-void CollisionVolume::SetAxisScales(float xs, float ys, float zs) {
+void CollisionVolume::SetAxisScales(const float& xs, const float& ys, const float& zs) {
 	axisScales.x = xs;
 	axisScales.y = ys;
 	axisScales.z = zs;
@@ -231,9 +253,11 @@ void CollisionVolume::SetAxisScales(float xs, float ys, float zs) {
 
 	// scale was unspecified
 	defaultScale = (xs == 1.0f && ys == 1.0f && zs == 1.0f);
+
+	SetBoundingRadius();
 }
 
-void CollisionVolume::RescaleAxes(float xs, float ys, float zs) {
+void CollisionVolume::RescaleAxes(const float& xs, const float& ys, const float& zs) {
 	axisScales.x *= xs; axisHScales.x *= xs;
 	axisScales.y *= ys; axisHScales.y *= ys;
 	axisScales.z *= zs; axisHScales.z *= zs;
