@@ -1,6 +1,5 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 #include "GlobalSynced.h"
 
 #include <assert.h>
@@ -14,6 +13,7 @@
 #include "Sim/Misc/TeamHandler.h"
 #include "Sim/Misc/GlobalConstants.h"
 #include "System/Util.h"
+#include "System/Log/DefaultFormatter.h"
 
 
 /**
@@ -59,6 +59,10 @@ CGlobalSynced::CGlobalSynced()
 {
 	mapx  = 512;
 	mapy  = 512;
+	mapxm1 = mapx - 1;
+	mapym1 = mapy - 1;
+	mapxp1 = mapx + 1;
+	mapyp1 = mapy + 1;
 	mapSquares = mapx * mapy;
 	hmapx = mapx>>1;
 	hmapy = mapy>>1;
@@ -78,6 +82,8 @@ CGlobalSynced::CGlobalSynced()
 	tempNum = 2;
 	useLuaGaia = true;
 
+	log_formatter_setFrameNumReference(&frameNum);
+
 	teamHandler = new CTeamHandler();
 }
 
@@ -85,12 +91,14 @@ CGlobalSynced::CGlobalSynced()
 CGlobalSynced::~CGlobalSynced()
 {
 	SafeDelete(teamHandler);
+
+	log_formatter_setFrameNumReference(NULL);
 }
 
 
 void CGlobalSynced::LoadFromSetup(const CGameSetup* setup)
 {
-	noHelperAIs = !!setup->noHelperAIs;
+	noHelperAIs = setup->noHelperAIs;
 	useLuaGaia  = setup->useLuaGaia;
 
 	skirmishAIHandler.LoadFromSetup(*setup);
