@@ -1,16 +1,16 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 #include <assert.h>
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "GroundBlockingObjectMap.h"
 
 #include "GlobalSynced.h"
 #include "GlobalConstants.h"
 #include "Sim/Objects/SolidObject.h"
-#include "Sim/Path/PathManager.h"
-#include "creg/STL_Map.h"
+#include "Sim/Path/IPathManager.h"
+#include "System/creg/STL_Map.h"
+#include "lib/gml/gmlmut.h"
 
 CGroundBlockingObjectMap* groundBlockingObjectMap;
 
@@ -33,6 +33,8 @@ inline static const int GetObjectID(CSolidObject* obj)
 
 void CGroundBlockingObjectMap::AddGroundBlockingObject(CSolidObject* object)
 {
+	GML_STDMUTEX_LOCK(block); // AddGroundBlockingObject
+
 	const int objID = GetObjectID(object);
 
 	object->isMarkedOnBlockingMap = true;
@@ -43,16 +45,15 @@ void CGroundBlockingObjectMap::AddGroundBlockingObject(CSolidObject* object)
 		object->mapPos.y &= 0xfffffe;
 	}
 
-	const int bx = object->mapPos.x;
-	const int bz = object->mapPos.y;
-	const int minXSqr = bx;
-	const int minZSqr = bz;
-	const int maxXSqr = bx + object->xsize;
-	const int maxZSqr = bz + object->zsize;
+	const int bx = object->mapPos.x, sx = object->xsize;
+	const int bz = object->mapPos.y, sz = object->zsize;
+	const int minXSqr = bx, maxXSqr = bx + sx;
+	const int minZSqr = bz, maxZSqr = bz + sz;
 
 	for (int zSqr = minZSqr; zSqr < maxZSqr; zSqr++) {
 		for (int xSqr = minXSqr; xSqr < maxXSqr; xSqr++) {
 			const int idx = xSqr + zSqr * gs->mapx;
+
 			BlockingMapCell& cell = groundBlockingMap[idx];
 			BlockingMapCellIt it = cell.find(objID);
 
@@ -63,7 +64,7 @@ void CGroundBlockingObjectMap::AddGroundBlockingObject(CSolidObject* object)
 	}
 
 	// FIXME: needs dependency injection (observer pattern?)
-	if (!object->mobility && pathManager) {
+	if (object->mobility == NULL && pathManager) {
 		pathManager->TerrainChange(minXSqr, minZSqr, maxXSqr, maxZSqr);
 	}
 }
@@ -71,6 +72,8 @@ void CGroundBlockingObjectMap::AddGroundBlockingObject(CSolidObject* object)
 
 void CGroundBlockingObjectMap::AddGroundBlockingObject(CSolidObject* object, const unsigned char* yardMap, unsigned char mask)
 {
+	GML_STDMUTEX_LOCK(block); // AddGroundBlockingObject
+
 	const int objID = GetObjectID(object);
 
 	object->isMarkedOnBlockingMap = true;
@@ -81,17 +84,18 @@ void CGroundBlockingObjectMap::AddGroundBlockingObject(CSolidObject* object, con
 		object->mapPos.y &= 0xfffffe;
 	}
 
-	const int bx = object->mapPos.x;
-	const int bz = object->mapPos.y;
-	const int minXSqr = bx;
-	const int minZSqr = bz;
-	const int maxXSqr = bx + object->xsize;
-	const int maxZSqr = bz + object->zsize;
+	const int bx = object->mapPos.x, sx = object->xsize;
+	const int bz = object->mapPos.y, sz = object->zsize;
+	const int minXSqr = bx, maxXSqr = bx + sx;
+	const int minZSqr = bz, maxZSqr = bz + sz;
 
 	for (int z = 0; minZSqr + z < maxZSqr; z++) {
 		for (int x = 0; minXSqr + x < maxXSqr; x++) {
+			// unit yardmaps always contain sx=UnitDef::xsize * sz=UnitDef::zsize
+			// cells (the unit->mobility footprint can have different dimensions)
 			const int idx = minXSqr + x + (minZSqr + z) * gs->mapx;
-			const int off = x + z * object->xsize;
+			const int off = x + z * sx;
+
 			BlockingMapCell& cell = groundBlockingMap[idx];
 			BlockingMapCellIt it = cell.find(objID);
 
@@ -102,7 +106,7 @@ void CGroundBlockingObjectMap::AddGroundBlockingObject(CSolidObject* object, con
 	}
 
 	// FIXME: needs dependency injection (observer pattern?)
-	if (!object->mobility && pathManager) {
+	if (object->mobility == NULL && pathManager) {
 		pathManager->TerrainChange(minXSqr, minZSqr, maxXSqr, maxZSqr);
 	}
 }
@@ -110,14 +114,17 @@ void CGroundBlockingObjectMap::AddGroundBlockingObject(CSolidObject* object, con
 
 void CGroundBlockingObjectMap::RemoveGroundBlockingObject(CSolidObject* object)
 {
+	GML_STDMUTEX_LOCK(block); // RemoveGroundBlockingObject
+
 	const int objID = GetObjectID(object);
 
-	object->isMarkedOnBlockingMap = false;
 	const int bx = object->mapPos.x;
 	const int bz = object->mapPos.y;
 	const int sx = object->xsize;
 	const int sz = object->zsize;
 
+	object->isMarkedOnBlockingMap = false;
+
 	for (int z = bz; z < bz + sz; ++z) {
 		for (int x = bx; x < bx + sx; ++x) {
 			int idx = x + z * gs->mapx;
@@ -131,7 +138,7 @@ void CGroundBlockingObjectMap::RemoveGroundBlockingObject(CSolidObject* object)
 	}
 
 	// FIXME: needs dependency injection (observer pattern?)
-	if (!object->mobility) {
+	if (object->mobility == NULL && pathManager) {
 		pathManager->TerrainChange(bx, bz, bx + sx, bz + sz);
 	}
 }
@@ -156,6 +163,8 @@ void CGroundBlockingObjectMap::MoveGroundBlockingObject(CSolidObject* object, fl
   * pointer to the top-most / bottom-most blocking object is returned.
   */
 CSolidObject* CGroundBlockingObjectMap::GroundBlockedUnsafe(int mapSquare, bool topMost) {
+	GML_STDMUTEX_LOCK(block); // GroundBlockedUnsafe
+
 	if (groundBlockingMap[mapSquare].empty()) {
 		return NULL;
 	}
@@ -164,9 +173,9 @@ CSolidObject* CGroundBlockingObjectMap::GroundBlockedUnsafe(int mapSquare, bool
 	BlockingMapCellIt it = cell.begin();
 	CSolidObject* p = it->second;
 	CSolidObject* q = it->second;
-	it++;
+	++it;
 
-	for (; it != cell.end(); it++) {
+	for (; it != cell.end(); ++it) {
 		CSolidObject* obj = it->second;
 		if (obj->pos.y > p->pos.y) { p = obj; }
 		if (obj->pos.y < q->pos.y) { q = obj; }
@@ -214,11 +223,14 @@ void CGroundBlockingObjectMap::CloseBlockingYard(CSolidObject* yard, const unsig
 
 bool CGroundBlockingObjectMap::CanCloseYard(CSolidObject* yard)
 {
+	GML_STDMUTEX_LOCK(block); // CanCloseYard
+
 	const int objID = GetObjectID(yard);
 
 	for (int z = yard->mapPos.y; z < yard->mapPos.y + yard->zsize; ++z) {
 		for (int x = yard->mapPos.x; x < yard->mapPos.x + yard->xsize; ++x) {
 			const int idx = z * gs->mapx + x;
+
 			BlockingMapCell& cell = groundBlockingMap[idx];
 			BlockingMapCellIt it = cell.find(objID);
 
