@@ -1,22 +1,22 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "ModInfo.h"
 
 #include "Game/GameSetup.h"
+#include "Lua/LuaConfig.h"
 #include "Lua/LuaParser.h"
 #include "Lua/LuaSyncedRead.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitTypes/Builder.h"
 #include "Rendering/GlobalRendering.h"
-#include "System/GlobalUnsynced.h"
-#include "System/LogOutput.h"
-#include "System/ConfigHandler.h"
+#include "System/Log/ILog.h"
+#include "System/Config/ConfigHandler.h"
 #include "System/FileSystem/ArchiveScanner.h"
 #include "System/Exceptions.h"
 
+CONFIG(bool, TeamNanoSpray).defaultValue(true);
 
 CModInfo modInfo;
 
@@ -24,15 +24,14 @@ CModInfo modInfo;
 void CModInfo::Init(const char* modArchive)
 {
 	filename = modArchive;
-
 	humanName = archiveScanner->NameFromArchive(modArchive);
 
 	const CArchiveScanner::ArchiveData md = archiveScanner->GetArchiveData(humanName);
 
-	shortName   = md.shortName;
-	version     = md.version;
-	mutator     = md.mutator;
-	description = md.description;
+	shortName   = md.GetShortName();
+	version     = md.GetVersion();
+	mutator     = md.GetMutator();
+	description = md.GetDescription();
 
 	// initialize the parser
 	LuaParser parser("gamedata/modrules.lua",
@@ -43,21 +42,22 @@ void CModInfo::Init(const char* modArchive)
 	parser.EndTable();
 	parser.Execute();
 	if (!parser.IsValid()) {
-		logOutput.Print("Error loading modrules, using defaults");
-		logOutput.Print(parser.GetErrorLog());
+		LOG_L(L_ERROR, "Failed loading mod-rules, using defaults; error: %s",
+				parser.GetErrorLog().c_str());
 	}
 	const LuaTable root = parser.GetRoot();
 
 	// determine if bombers are allowed to leave map boundaries
 	const LuaTable movementTbl = root.SubTable("movement");
 	allowAirPlanesToLeaveMap = movementTbl.GetBool("allowAirPlanesToLeaveMap", true);
+	allowPushingEnemyUnits = movementTbl.GetBool("allowPushingEnemyUnits", false);
 
 	// determine whether the modder allows the user to use team coloured nanospray
 	const LuaTable nanosprayTbl = root.SubTable("nanospray");
 	allowTeamColors = nanosprayTbl.GetBool("allow_team_colors", true);
 	if (allowTeamColors) {
 		// Load the users preference for team coloured nanospray
-		globalRendering->teamNanospray = !!configHandler->Get("TeamNanoSpray", 1);
+		globalRendering->teamNanospray = configHandler->GetBool("TeamNanoSpray");
 	}
 
 	// constructions
@@ -127,7 +127,7 @@ void CModInfo::Init(const char* modArchive)
 	requireSonarUnderWater = sensors.GetBool("requireSonarUnderWater", true);
 	/// LoS
 	const LuaTable los = sensors.SubTable("los");
-	// losMipLevel is used as index to readmap->mipHeightmap,
+	// losMipLevel is used as index to readmap->mipHeightmaps,
 	// so the max value is CReadMap::numHeightMipMaps - 1
 	losMipLevel = los.GetInt("losMipLevel", 1);
 	losMul = los.GetFloat("losMul", 1.0f);
@@ -143,4 +143,7 @@ void CModInfo::Init(const char* modArchive)
 		                    "The minimum value is 0. The maximum value is 30.");
 	}
 	airLosMul = los.GetFloat("airLosMul", 1.0f);
+
+	const LuaTable system = root.SubTable("system");
+	luaThreadingModel = system.GetInt("luaThreadingModel", MT_LUA_SINGLE_BATCH);
 }
