@@ -5,7 +5,7 @@
 #ifndef TEAMHANDLER_H
 #define TEAMHANDLER_H
 
-#include "creg/creg_cond.h"
+#include "System/creg/creg_cond.h"
 #include "Team.h"
 #include "AllyTeam.h"
 
@@ -97,11 +97,17 @@ public:
 	bool IsValidTeam(int id) const {
 		return ((id >= 0) && (id < ActiveTeams()));
 	}
-
 	bool IsActiveTeam(int id) const {
 		return IsValidTeam(id);
 	}
 
+	bool IsValidAllyTeam(int id) const {
+		return ((id >= 0) && (id < ActiveAllyTeams()));
+	}
+	bool IsActiveAllyTeam(int id) const {
+		return IsValidAllyTeam(id);
+	}
+
 	void GameFrame(int frameNum);
 
 private:
