@@ -1,7 +1,7 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#ifndef AAIRMOVETYPE_H_
-#define AAIRMOVETYPE_H_
+#ifndef A_AIR_MOVE_TYPE_H_
+#define A_AIR_MOVE_TYPE_H_
 
 #include "MoveType.h"
 #include "Sim/Misc/AirBaseHandler.h"
@@ -9,14 +9,14 @@
 /**
  * Supposed to be an abstract class.
  * Do not create an instance of this class.
- * Use either CTAAirMoveType or CAirMoveType instead.
+ * Use either CHoverAirMoveType or CStrafeAirMoveType instead.
  */
 class AAirMoveType : public AMoveType
 {
 	CR_DECLARE(AAirMoveType);
 public:
 
-	enum AircraftState{
+	enum AircraftState {
 		AIRCRAFT_LANDED,
 		AIRCRAFT_FLYING,
 		AIRCRAFT_LANDING,
@@ -29,10 +29,19 @@ public:
 	AAirMoveType(CUnit* unit);
 	~AAirMoveType();
 
+	virtual bool Update();
+	virtual bool IsFighter() const = 0;
+	virtual void Takeoff() = 0;
+	virtual void SetState(AircraftState state) = 0;
+
+	void ReservePad(CAirBaseHandler::LandingPad* lp);
+	void DependentDied(CObject* o);
+
 	bool UseSmoothMesh() const;
+
+
 	/// goalpos to resume flying to after landing
 	float3 oldGoalPos;
-	float3 oldpos;
 	float3 reservedLandingPos;
 
 	float wantedHeight;
@@ -41,20 +50,19 @@ public:
 	bool collide;
 	/// controls use of smoothGround for determining altitude
 	bool useSmoothMesh;
-	/// unit found to be dangerously close to our path
-	CUnit* lastColWarning;
-	/// 1=generally forward of us, 2=directly in path
-	int lastColWarningType;
-
 	bool autoLand;
 
-	virtual bool IsFighter() = 0;
-	virtual void Takeoff() = 0;
-	void ReservePad(CAirBaseHandler::LandingPad* lp);
-	void DependentDied(CObject* o);
-
 protected:
-	virtual void SetState(AircraftState state) = 0;
+	void CheckForCollision();
+	bool MoveToRepairPad();
+	void UpdateFuel();
+
+	/// unit found to be dangerously close to our path
+	CUnit* lastColWarning;
+
+	/// 1=generally forward of us, 2=directly in path
+	int lastColWarningType;
+	int lastFuelUpdateFrame;
 };
 
-#endif // AAIRMOVETYPE_H_
+#endif // A_AIR_MOVE_TYPE_H_
