@@ -3,10 +3,10 @@
 #ifndef MOVETYPE_H
 #define MOVETYPE_H
 
-#include "creg/creg_cond.h"
-#include "Object.h"
+#include "System/creg/creg_cond.h"
 #include "Sim/Misc/AirBaseHandler.h"
-#include "float3.h"
+#include "System/Object.h"
+#include "System/float3.h"
 
 class CUnit;
 
@@ -16,32 +16,36 @@ class AMoveType : public CObject
 
 public:
 	AMoveType(CUnit* owner);
-	virtual ~AMoveType(void);
+	virtual ~AMoveType() {}
 
 	virtual void StartMoving(float3 pos, float goalRadius) = 0;
 	virtual void StartMoving(float3 pos, float goalRadius, float speed) = 0;
 	virtual void KeepPointingTo(float3 pos, float distance, bool aggressive) = 0;
 	virtual void KeepPointingTo(CUnit* unit, float distance, bool aggressive);
 	virtual void StopMoving() = 0;
-	virtual void ImpulseAdded(void);
+	virtual void ImpulseAdded(const float3&) {}
 	virtual void ReservePad(CAirBaseHandler::LandingPad* lp);
 
 	virtual void SetGoal(const float3& pos);
 	virtual void SetMaxSpeed(float speed);
 	virtual void SetWantedMaxSpeed(float speed);
-	virtual void LeaveTransport(void);
+	virtual void LeaveTransport() {}
 
-	virtual void Update() = 0;
+	virtual bool Update() = 0;
 	virtual void SlowUpdate();
 
-	int forceTurn;
-	int forceTurnTo;
+	virtual bool IsSkidding() const { return false; }
+	virtual bool IsFlying() const { return false; }
+	virtual bool IsReversing() const { return false; }
+
+	bool WantsRepair() const;
+	bool WantsRefuel() const;
 
 	CUnit* owner;
 
 	float3 goalPos;
-	float3 oldPos;             // owner position at last Update(); only used by GMT
-	float3 oldSlowUpdatePos;   // owner position at last SlowUpdate(); only used by GMT
+	float3 oldPos;             // owner position at last Update()
+	float3 oldSlowUpdatePos;   // owner position at last SlowUpdate()
 
 	float maxSpeed;
 	float maxWantedSpeed;
