@@ -3,10 +3,12 @@
 #ifndef WORLD_OBJECT_H
 #define WORLD_OBJECT_H
 
-#include "Object.h"
-#include "float3.h"
+#include "System/Object.h"
+#include "System/float3.h"
 
 struct S3DModel;
+
+
 class CWorldObject: public CObject
 {
 public:
@@ -15,9 +17,9 @@ public:
 	CWorldObject()
 		: id(0)
 		, pos(ZeroVector)
-		, radius(0)
-		, sqRadius(0)
-		, drawRadius(0)
+		, radius(0.0f)
+		, sqRadius(0.0f)
+		, drawRadius(0.0f)
 		, useAirLos(false)
 		, alwaysVisible(false)
 		, model(NULL)
@@ -25,17 +27,20 @@ public:
 	CWorldObject(const float3& pos)
 		: id(0)
 		, pos(pos)
-		, radius(0)
-		, sqRadius(0)
-		, drawRadius(0)
+		, radius(0.0f)
+		, sqRadius(0.0f)
+		, drawRadius(0.0f)
 		, useAirLos(false)
 		, alwaysVisible(false)
 		, model(NULL)
 	{}
 
-	void SetRadius(float r);
 	virtual ~CWorldObject();
 
+	void SetRadius(float radius);
+
+
+public:
 	int id;
 
 	float3 pos;
@@ -43,7 +48,8 @@ public:
 	float sqRadius;
 
 	float drawRadius;   ///< used to see if in los
-	bool useAirLos;     ///< if true, object's visibility is checked against airLosMap[allyteam]
+	/// if true, the object's visibility is checked against airLosMap[allyteam]
+	bool useAirLos;
 	bool alwaysVisible;
 
 	S3DModel* model;
