@@ -1,17 +1,17 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "Game/Camera.h"
 #include "Game/GameHelper.h"
+#include "Game/GlobalUnsynced.h"
 #include "Map/Ground.h"
 #include "Rendering/GlobalRendering.h"
 #include "Rendering/GL/myGL.h"
 #include "Rendering/GL/VertexArray.h"
 #include "Rendering/Textures/TextureAtlas.h"
 #include "Rendering/Colors.h"
-#include "Rendering/ProjectileDrawer.hpp"
+#include "Rendering/ProjectileDrawer.h"
 #include "Rendering/UnitDrawer.h"
 #include "Rendering/Models/IModelParser.h"
 #include "Rendering/Models/3DOParser.h"
@@ -21,7 +21,6 @@
 #include "Sim/Projectiles/ProjectileHandler.h"
 #include "Sim/Projectiles/Unsynced/SmokeTrailProjectile.h"
 #include "Sim/Units/Unit.h"
-#include "System/GlobalUnsynced.h"
 #include "System/Matrix44f.h"
 #include "System/myMath.h"
 #include "System/Sync/SyncTracer.h"
@@ -33,15 +32,16 @@ CR_BIND_DERIVED(CPieceProjectile, CProjectile, (float3(0, 0, 0), float3(0, 0, 0)
 
 CR_REG_METADATA(CPieceProjectile,(
 	CR_SETFLAG(CF_Synced),
-	CR_SERIALIZER(creg_Serialize), // numCallback, oldInfos
+	CR_SERIALIZER(creg_Serialize), // oldInfos
 	CR_MEMBER(flags),
 	CR_MEMBER(dispList),
+	CR_MEMBER(cegID),
 	// NOTE: what about this?
 	// CR_MEMBER(omp),
 	CR_MEMBER(spinVec),
 	CR_MEMBER(spinSpeed),
 	CR_MEMBER(spinAngle),
-	CR_MEMBER(oldSmoke),
+	CR_MEMBER(oldSmokePos),
 	CR_MEMBER(oldSmokeDir),
 	CR_MEMBER(alphaThreshold),
 	// CR_MEMBER(target),
@@ -49,13 +49,11 @@ CR_REG_METADATA(CPieceProjectile,(
 	CR_MEMBER(curCallback),
 	CR_MEMBER(age),
 	CR_MEMBER(colorTeam),
-	CR_MEMBER(ceg),
 	CR_RESERVED(36)
 	));
 
 void CPieceProjectile::creg_Serialize(creg::ISerializer& s)
 {
-	s.Serialize(numCallback, sizeof(int));
 	for (int i = 0; i < 8; i++) {
 		s.Serialize(oldInfos[i], sizeof(CPieceProjectile::OldInfo));
 	}
@@ -68,7 +66,7 @@ CPieceProjectile::CPieceProjectile(const float3& pos, const float3& speed, Local
 	omp(NULL),
 	spinAngle(0.0f),
 	alphaThreshold(0.1f),
-	oldSmoke(pos),
+	oldSmokePos(pos),
 	drawTrail(true),
 	curCallback(0),
 	age(0)
@@ -76,21 +74,12 @@ CPieceProjectile::CPieceProjectile(const float3& pos, const float3& speed, Local
 	checkCol = false;
 
 	if (owner) {
-		// choose a (synced) random tag-postfix string k from the
-		// range given in UnitDef and stick it onto pieceTrailCEGTag
-		// (assumes all possible "tag + k" CEG identifiers are valid)
-		// if this piece does not override the FBI and wants a trail
 		if ((flags & PF_NoCEGTrail) == 0) {
-			const int size = owner->unitDef->pieceTrailCEGTag.size();
-			const int range = owner->unitDef->pieceTrailCEGRange;
-			const int num = gs->randInt() % range;
-			const char* tag = owner->unitDef->pieceTrailCEGTag.c_str();
+			const std::vector<std::string>& pieceCEGs = owner->unitDef->pieceCEGTags;
+			const std::string& cegTag = !pieceCEGs.empty()? pieceCEGs[gs->randInt() % pieceCEGs.size()]: "";
 
-			if (size > 0) {
-				char cegTag[1024];
-				SNPRINTF(cegTag, sizeof(cegTag) - 1, "%s%d", tag, num);
-				cegTag[1023] = 0;
-				ceg.Load(explGenHandler, cegTag);
+			if (!cegTag.empty()) {
+				cegID = gCEG->Load(explGenHandler, cegTag.c_str());
 			} else {
 				flags |= PF_NoCEGTrail;
 			}
@@ -118,11 +107,9 @@ CPieceProjectile::CPieceProjectile(const float3& pos, const float3& speed, Local
 		useAirLos = true;
 	}
 
-	numCallback = new int;
-	*numCallback = 0;
 	oldSmokeDir = speed;
 	oldSmokeDir.Normalize();
-	float3 camDir = (pos-camera->pos).Normalize();
+	const float3 camDir = (pos - camera->pos).Normalize();
 
 	if (camera->pos.distance(pos) + (1 - fabs(camDir.dot(oldSmokeDir))) * 3000 < 200) {
 		drawTrail = false;
@@ -154,11 +141,15 @@ CPieceProjectile::CPieceProjectile(const float3& pos, const float3& speed, Local
 	ph->AddProjectile(this);
 }
 
-
-CPieceProjectile::~CPieceProjectile(void)
+void CPieceProjectile::Detach()
 {
-	delete numCallback;
+	// SYNCED
+	CProjectile::Detach();
+}
 
+CPieceProjectile::~CPieceProjectile()
+{
+	// UNSYNCED
 	if (curCallback)
 		curCallback->drawCallbacker = 0;
 
@@ -170,38 +161,87 @@ CPieceProjectile::~CPieceProjectile(void)
 
 void CPieceProjectile::Collision()
 {
-	if (speed.SqLength() > Square(gs->randFloat() * 5 + 1) && pos.y > radius + 2) {
-		float3 norm = ground->GetNormal(pos.x, pos.z);
-		float ns = speed.dot(norm);
-		speed -= norm * ns * 1.6f;
-		pos += norm * 0.1f;
-	} else {
-		if (flags & PF_Explode) {
-			helper->Explosion(pos, DamageArray(50), 5, 0, 10, owner(), false, 1.0f, false, false, 0, 0, ZeroVector, -1);
-		}
-		if (flags & PF_Smoke) {
-			if (flags & PF_NoCEGTrail) {
-				float3 dir = speed;
-				dir.Normalize();
+	const float3& norm = ground->GetNormal(pos.x, pos.z);
+	const float ns = speed.dot(norm);
 
-				CSmokeTrailProjectile* tp =
-					new CSmokeTrailProjectile(pos, oldSmoke, dir, oldSmokeDir, owner(),
-					false, true, 7, Smoke_Time, 0.5f, drawTrail, 0, projectileDrawer->smoketrailtex);
-				tp->creationTime += (8 - ((age) & 7));
-			}
-		}
+	speed -= (norm * ns * 1.6f);
+	pos += (norm * 0.1f);
 
-		CProjectile::Collision();
-		oldSmoke = pos;
+	if (flags & PF_Explode) {
+		const DamageArray damageArray(50.0f);
+		const CGameHelper::ExplosionParams params = {
+			pos,
+			ZeroVector,
+			damageArray,
+			NULL,              // weaponDef
+			owner(),
+			NULL,              // hitUnit
+			NULL,              // hitFeature
+			5.0f,              // areaOfEffect
+			0.0f,              // edgeEffectiveness
+			10.0f,             // explosionSpeed
+			1.0f,              // gfxMod
+			false,             // impactOnly
+			false,             // ignoreOwner
+			true               // damageGround
+		};
+
+		helper->Explosion(params);
 	}
+	if (flags & PF_Smoke) {
+		if (flags & PF_NoCEGTrail) {
+			float3 dir = speed;
+			dir.Normalize();
+
+			CSmokeTrailProjectile* tp = new CSmokeTrailProjectile(
+				pos, oldSmokePos,
+				dir, oldSmokeDir,
+				owner(),
+				false,
+				true,
+				7,
+				Smoke_Time,
+				0.5f,
+				drawTrail,
+				0,
+				projectileDrawer->smoketrailtex);
+			tp->creationTime += (8 - ((age) & 7));
+		}
+	}
+
+	// keep flying with some small probability
+	if (gs->randFloat() < 0.666f) {
+		CProjectile::Collision();
+	}
+
+	oldSmokePos = pos;
 }
 
 void CPieceProjectile::Collision(CUnit* unit)
 {
-	if (unit == owner())
+	if (unit == owner()) {
 		return;
+	}
 	if (flags & PF_Explode) {
-		helper->Explosion(pos, DamageArray(50), 5, 0, 10, owner(), false, 1.0f, false, false, 0, unit, ZeroVector, -1);
+		const DamageArray damageArray(50.0f);
+		const CGameHelper::ExplosionParams params = {
+			pos,
+			ZeroVector,
+			damageArray,
+			NULL,                                            // weaponDef
+			owner(),
+			unit,                                            // hitUnit
+			NULL,                                            // hitFeature
+			5.0f,                                            // areaOfEffect
+			0.0f,                                            // edgeEffectiveness
+			10.0f,                                           // explosionSpeed
+			1.0f,                                            // gfxMod
+			false,                                           // impactOnly
+			false,                                           // ignoreOwner
+			true                                             // damageGround
+		};
+
+		helper->Explosion(params);
 	}
 	if (flags & PF_Smoke) {
 		if (flags & PF_NoCEGTrail) {
@@ -209,21 +249,22 @@ void CPieceProjectile::Collision(CUnit* unit)
 			dir.Normalize();
 
 			CSmokeTrailProjectile* tp =
-				new CSmokeTrailProjectile(pos, oldSmoke, dir, oldSmokeDir, owner(),
+				new CSmokeTrailProjectile(pos, oldSmokePos, dir, oldSmokeDir, owner(),
 				false, true, 7, Smoke_Time, 0.5f, drawTrail, 0, projectileDrawer->smoketrailtex);
 			tp->creationTime += (8 - ((age) & 7));
 		}
 	}
 
 	CProjectile::Collision(unit);
-	oldSmoke = pos;
+	oldSmokePos = pos;
 }
 
 
-bool CPieceProjectile::HasVertices(void)
+bool CPieceProjectile::HasVertices() const
 {
-	if (omp == NULL)
+	if (omp == NULL) {
 		return false;
+	}
 
 	return (omp->GetVertexCount() > 0);
 }
@@ -277,15 +318,16 @@ void CPieceProjectile::Update()
 			float3 dir = speed;
 			dir.Normalize();
 
-			if (curCallback)
+			if (curCallback) {
 				curCallback->drawCallbacker = 0;
+			}
 
 			curCallback =
-				new CSmokeTrailProjectile(pos, oldSmoke, dir, oldSmokeDir, owner(),
+				new CSmokeTrailProjectile(pos, oldSmokePos, dir, oldSmokeDir, owner(),
 				age == 8, false, 14, Smoke_Time, 0.5f, drawTrail, this, projectileDrawer->smoketrailtex);
 			useAirLos = curCallback->useAirLos;
 
-			oldSmoke = pos;
+			oldSmokePos = pos;
 			oldSmokeDir = dir;
 
 			if (!drawTrail) {
@@ -297,11 +339,12 @@ void CPieceProjectile::Update()
 		}
 	} else {
 		// TODO: pass a more sensible ttl to the CEG (age-related?)
-		ceg.Explosion(pos, 100, 0.0f, 0x0, 0.0f, 0x0, speed);
+		gCEG->Explosion(cegID, pos, 100, 0.0f, 0x0, 0.0f, 0x0, speed);
 	}
 
-	if (age > 10)
+	if (age > 10) {
 		checkCol = true;
+	}
 }
 
 
@@ -323,14 +366,10 @@ void CPieceProjectile::Draw()
 			va->EnlargeArrays(4+4*numParts,0,VA_SIZE_TC);
 			if (drawTrail) {
 				// draw the trail as a single quad if camera close enough
-				float3 dif(drawPos - camera->pos);
-				dif.Normalize();
-				float3 dir1(dif.cross(dir));
-				dir1.Normalize();
-				float3 dif2(oldSmoke - camera->pos);
-				dif2.Normalize();
-				float3 dir2(dif2.cross(oldSmokeDir));
-				dir2.Normalize();
+				const float3 dif  = (drawPos - camera->pos).Normalize();
+				const float3 dir1 = (dif.cross(dir)).Normalize();
+				const float3 dif2 = (oldSmokePos - camera->pos).Normalize();
+				const float3 dir2 = (dif2.cross(oldSmokeDir)).Normalize();
 
 				float a1 = ((1 - 0.0f / (Smoke_Time)) * 255) * (0.7f + fabs(dif.dot(dir)));
 				float alpha = std::min(255.0f, std::max(0.f, a1));
@@ -360,13 +399,13 @@ void CPieceProjectile::Draw()
 
 				va->AddVertexQTC(drawPos - dir1 * size, txs, projectileDrawer->smoketrailtex->ystart, col);
 				va->AddVertexQTC(drawPos + dir1 * size, txs, projectileDrawer->smoketrailtex->yend,   col);
-				va->AddVertexQTC(oldSmoke + dir2 * size2, projectileDrawer->smoketrailtex->xend, projectileDrawer->smoketrailtex->yend,   col2);
-				va->AddVertexQTC(oldSmoke - dir2 * size2, projectileDrawer->smoketrailtex->xend, projectileDrawer->smoketrailtex->ystart, col2);
+				va->AddVertexQTC(oldSmokePos + dir2 * size2, projectileDrawer->smoketrailtex->xend, projectileDrawer->smoketrailtex->yend,   col2);
+				va->AddVertexQTC(oldSmokePos - dir2 * size2, projectileDrawer->smoketrailtex->xend, projectileDrawer->smoketrailtex->ystart, col2);
 			} else {
 				// draw the trail as particles
-				const float dist = pos.distance(oldSmoke);
+				const float dist = pos.distance(oldSmokePos);
 				const float3 dirpos1 = pos - dir * dist * 0.33f;
-				const float3 dirpos2 = oldSmoke + oldSmokeDir * dist * 0.33f;
+				const float3 dirpos2 = oldSmokePos + oldSmokeDir * dist * 0.33f;
 
 				for (int a = 0; a < numParts; ++a) { //! CAUTION: loop count must match EnlargeArrays above
 					float alpha = 255.0f;
@@ -376,7 +415,7 @@ void CPieceProjectile::Draw()
 					col[3] = (unsigned char) (alpha);
 
 					const float size = 1.0f + ((a) * (1.0f / Smoke_Time)) * 14.0f;
-					const float3 pos1 = CalcBeizer(float(a) / (numParts), pos, dirpos1, dirpos2, oldSmoke);
+					const float3 pos1 = CalcBeizer(float(a) / (numParts), pos, dirpos1, dirpos2, oldSmokePos);
 
 					#define st projectileDrawer->smoketex[0]
 					va->AddVertexQTC(pos1 + ( camera->up+camera->right) * size, st->xstart, st->ystart, col);
@@ -390,9 +429,6 @@ void CPieceProjectile::Draw()
 	}
 
 	DrawCallback();
-	if (curCallback == 0) {
-		DrawCallback();
-	}
 }
 
 void CPieceProjectile::DrawOnMinimap(CVertexArray& lines, CVertexArray& points)
@@ -400,19 +436,13 @@ void CPieceProjectile::DrawOnMinimap(CVertexArray& lines, CVertexArray& points)
 	points.AddVertexQC(pos, color4::red);
 }
 
-void CPieceProjectile::DrawCallback(void)
+void CPieceProjectile::DrawCallback()
 {
-	if (*numCallback != globalRendering->drawFrame) {
-		*numCallback = globalRendering->drawFrame;
-		return;
-	}
-	*numCallback = 0;
-
 	inArray = true;
 	unsigned char col[4];
 
 	if (flags & PF_Fire) {
-		va->EnlargeArrays(8*4,0,VA_SIZE_TC);
+		va->EnlargeArrays(8 * 4, 0, VA_SIZE_TC);
 		for (int age = 0; age < 8; ++age) {
 			float modage = age;
 			float3 interPos = oldInfos[age]->pos;
