@@ -3,25 +3,21 @@
 #ifndef PROJECTILE_H
 #define PROJECTILE_H
 
-#include "Rendering/GL/myGL.h"
+#include "lib/gml/gml.h" // for GML_ENABLE_SIM
 
 #ifdef _MSC_VER
 #pragma warning(disable:4291)
 #endif
 
-class CProjectileHandler;
-class CBuilding;
-
 #include "ExplosionGenerator.h"
 #include "Sim/Units/UnitHandler.h"
+#include "System/float3.h"
+#include "System/Vec2.h"
 
 class CUnit;
 class CFeature;
 class CVertexArray;
 
-#define COLLISION_NOFRIENDLY	1
-#define COLLISION_NOFEATURE		2
-#define COLLISION_NONEUTRAL		4
 
 class CProjectile: public CExpGenSpawnable
 {
@@ -31,49 +27,71 @@ class CProjectile: public CExpGenSpawnable
 	CProjectile();
 
 public:
-	static bool inArray;
-	static CVertexArray* va;
-	static int DrawArray();
-
-	virtual void Draw();
-	virtual void DrawOnMinimap(CVertexArray& lines, CVertexArray& points);
 	CProjectile(const float3& pos, const float3& speed, CUnit* owner, bool isSynced, bool isWeapon, bool isPiece);
+	virtual ~CProjectile();
+	virtual void Detach();
+
 	virtual void Collision();
 	virtual void Collision(CUnit* unit);
 	virtual void Collision(CFeature* feature);
-	virtual ~CProjectile();
 	virtual void Update();
 	virtual void Init(const float3& pos, CUnit* owner);
 
+	virtual void Draw() {}
+	virtual void DrawOnMinimap(CVertexArray& lines, CVertexArray& points);
+	virtual void DrawCallback() {}
+
+	inline CUnit* owner() const {
+		return
+#if defined(USE_GML) && GML_ENABLE_SIM
+		*(CUnit * volatile *)&
+#endif
+		uh->units[ownerId]; // Note: this death dependency optimization using "ownerId" is logically flawed, since ids are being reused it could return a unit that is not the original owner
+	}
+
+
+	void SetQuadFieldCellCoors(const int2& cell) { quadFieldCellCoors = cell; }
+	int2 GetQuadFieldCellCoors() const { return quadFieldCellCoors; }
+
+	void SetQuadFieldCellIter(const std::list<CProjectile*>::iterator& it) { quadFieldCellIter = it; }
+	const std::list<CProjectile*>::iterator& GetQuadFieldCellIter() { return quadFieldCellIter; }
+
+	unsigned int GetProjectileType() const { return projectileType; }
+	unsigned int GetCollisionFlags() const { return collisionFlags; }
+
+
+	static bool inArray;
+	static CVertexArray* va;
+	static int DrawArray();
+
 	bool synced; ///< is this projectile part of the simulation?
 	bool weapon; ///< is this a weapon projectile? (true implies synced true)
 	bool piece;  ///< is this a piece projectile? (true implies synced true)
 
 	bool luaMoveCtrl;
 	bool checkCol;
+	bool ignoreWater;
 	bool deleteMe;
 	bool castShadow;
-	unsigned int collisionFlags;
 
-	float3 drawPos;
 #if defined(USE_GML) && GML_ENABLE_SIM
 	unsigned lastProjUpdate;
 #endif
 
-	CUnit* owner() const { return uh->units[ownerId]; }
-	int GetProjectileType() const { return projectileType; }
-
 	float3 dir;
 	float3 speed;
+	float3 drawPos;
+
 	float mygravity;
-
-	virtual void DrawCallback(void) {}
-
-	float tempdist; // temp distance used for sorting when rendering
+	float tempdist; ///< temp distance used for sorting when rendering
 	
 protected:
-	int ownerId;
-	int projectileType;
+	unsigned int ownerId;
+	unsigned int projectileType;
+	unsigned int collisionFlags;
+
+	int2 quadFieldCellCoors;
+	std::list<CProjectile*>::iterator quadFieldCellIter;
 };
 
 #endif /* PROJECTILE_H */
