@@ -1,7 +1,7 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#ifndef PROJECTILEHANDLER_H
-#define PROJECTILEHANDLER_H
+#ifndef PROJECTILE_HANDLER_H
+#define PROJECTILE_HANDLER_H
 
 #include <list>
 #include <set>
@@ -9,8 +9,8 @@
 #include <stack>
 #include "lib/gml/ThreadSafeContainers.h"
 
-#include "MemPool.h"
-#include "float3.h"
+#include "System/MemPool.h"
+#include "System/float3.h"
 
 #define UNSYNCED_PROJ_NOEVENT 1 // bypass id and event handling for unsynced projectiles (faster)
 
@@ -18,17 +18,22 @@ class CProjectile;
 class CUnit;
 class CFeature;
 class CGroundFlash;
+struct UnitDef;
 struct FlyingPiece;
 struct S3DOPrimitive;
 struct S3DOPiece;
 struct SS3OVertex;
+
 struct piececmp {
 	bool operator() (const FlyingPiece* fp1, const FlyingPiece* fp2) const;
 };
+struct projdetach {
+	static void Detach(CProjectile* p);
+};
 
 typedef std::pair<CProjectile*, int> ProjectileMapPair;
 typedef std::map<int, ProjectileMapPair> ProjectileMap;
-typedef ThreadListSim<std::list<CProjectile*>, std::set<CProjectile*>, CProjectile*> ProjectileContainer;
+typedef ThreadListSim<std::list<CProjectile*>, std::set<CProjectile*>, CProjectile*, projdetach> ProjectileContainer;
 typedef ThreadListSimRender<std::list<CGroundFlash*>, std::set<CGroundFlash*>, CGroundFlash*> GroundFlashContainer;
 #if defined(USE_GML) && GML_ENABLE_SIM
 typedef ThreadListSimRender<std::set<FlyingPiece*>, std::set<FlyingPiece*, piececmp>, FlyingPiece*> FlyingPieceContainer;
@@ -40,8 +45,9 @@ typedef ThreadListSimRender<std::set<FlyingPiece*, piececmp>, void, FlyingPiece*
 
 class CProjectileHandler
 {
-public:
 	CR_DECLARE(CProjectileHandler);
+
+public:
 	CProjectileHandler();
 	virtual ~CProjectileHandler();
 	void Serialize(creg::ISerializer* s);
@@ -73,24 +79,30 @@ public:
 
 	void Update();
 	void UpdateProjectileContainer(ProjectileContainer&, bool);
+	void UpdateParticleSaturation() {
+		particleSaturation     = (maxParticles     > 0)? (currentParticles     / float(maxParticles    )): 1.0f;
+		nanoParticleSaturation = (maxNanoParticles > 0)? (currentNanoParticles / float(maxNanoParticles)): 1.0f;
+	}
 	
 	void AddProjectile(CProjectile* p);
 	void AddGroundFlash(CGroundFlash* flash);
 	void AddFlyingPiece(int team, float3 pos, float3 speed, const S3DOPiece* object, const S3DOPrimitive* piece);
 	void AddFlyingPiece(int textureType, int team, float3 pos, float3 speed, SS3OVertex* verts);
+	void AddNanoParticle(const float3&, const float3&, const UnitDef*, int team, bool highPriority);
+	void AddNanoParticle(const float3&, const float3&, const UnitDef*, int team, float radius, bool inverse, bool highPriority);
 
-	ProjectileContainer syncedProjectiles;    //! contains only projectiles that can change simulation state
-	ProjectileContainer unsyncedProjectiles;  //! contains only projectiles that cannot change simulation state
-	FlyingPieceContainer flyingPieces3DO;     //! unsynced
-	FlyingPieceContainer flyingPiecesS3O;     //! unsynced
-	GroundFlashContainer groundFlashes;       //! unsynced
+	ProjectileContainer syncedProjectiles;    // contains only projectiles that can change simulation state
+	ProjectileContainer unsyncedProjectiles;  // contains only projectiles that cannot change simulation state
+	FlyingPieceContainer flyingPieces3DO;     // unsynced
+	FlyingPieceContainer flyingPiecesS3O;     // unsynced
+	GroundFlashContainer groundFlashes;       // unsynced
 
 	int maxUsedSyncedID;
 	int maxUsedUnsyncedID;
-	std::list<int> freeSyncedIDs;             //! available synced (weapon, piece) projectile ID's
-	std::list<int> freeUnsyncedIDs;           //! available unsynced projectile ID's
-	ProjectileMap syncedProjectileIDs;        //! ID ==> <projectile, allyteam> map for living synced projectiles
-	ProjectileMap unsyncedProjectileIDs;      //! ID ==> <projectile, allyteam> map for living unsynced projectiles
+	std::list<int> freeSyncedIDs;             // available synced (weapon, piece) projectile ID's
+	std::list<int> freeUnsyncedIDs;           // available unsynced projectile ID's
+	ProjectileMap syncedProjectileIDs;        // ID ==> <projectile, allyteam> map for living synced projectiles
+	ProjectileMap unsyncedProjectileIDs;      // ID ==> <projectile, allyteam> map for living unsynced projectiles
 
 	int maxParticles;              // different effects should start to cut down on unnececary(unsynced) particles when this number is reached
 	int maxNanoParticles;
@@ -99,10 +111,10 @@ public:
 	float particleSaturation;      // currentParticles / maxParticles ratio
 	float nanoParticleSaturation;
 
-	int numPerlinProjectiles; //! unsynced
+	int numPerlinProjectiles;      // unsynced
 };
 
 
 extern CProjectileHandler* ph;
 
-#endif /* PROJECTILEHANDLER_H */
+#endif /* PROJECTILE_HANDLER_H */
