@@ -1,15 +1,14 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "ExploSpikeProjectile.h"
 #include "Game/Camera.h"
+#include "Game/GlobalUnsynced.h"
 #include "Rendering/GlobalRendering.h"
-#include "Rendering/ProjectileDrawer.hpp"
+#include "Rendering/ProjectileDrawer.h"
 #include "Rendering/GL/VertexArray.h"
 #include "Rendering/Textures/TextureAtlas.h"
-#include "System/GlobalUnsynced.h"
 
 CR_BIND_DERIVED(CExploSpikeProjectile, CProjectile, );
 
@@ -74,21 +73,19 @@ void CExploSpikeProjectile::Update()
 
 void CExploSpikeProjectile::Draw()
 {
-	inArray=true;
+	inArray = true;
 
-	float3 dif(pos-camera->pos2);
-	dif.ANormalize();
-	float3 dir2(dif.cross(dir));
-	dir2.ANormalize();
+	const float3 dif = (pos - camera->pos).ANormalize();
+	const float3 dir2 = (dif.cross(dir)).ANormalize();
 
 	unsigned char col[4];
-	float a=std::max(0.f,alpha-alphaDecay*globalRendering->timeOffset)*255;
-	col[0]=(unsigned char)(a*color.x);
-	col[1]=(unsigned char)(a*color.y);
-	col[2]=(unsigned char)(a*color.z);
-	col[3]=1;
+	const float a = std::max(0.0f, alpha-alphaDecay * globalRendering->timeOffset) * 255;
+	col[0] = (unsigned char)(a * color.x);
+	col[1] = (unsigned char)(a * color.y);
+	col[2] = (unsigned char)(a * color.z);
+	col[3] = 1;
 
-	const float3 l = dir * length + lengthGrowth * globalRendering->timeOffset;
+	const float3 l = (dir * length) + (lengthGrowth * globalRendering->timeOffset);
 	const float3 w = dir2 * width;
 
 	#define let projectileDrawer->laserendtex
