@@ -1,22 +1,20 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "SmokeTrailProjectile.h"
 
 #include "Game/Camera.h"
 #include "Map/Ground.h"
 #include "Rendering/GlobalRendering.h"
-#include "Rendering/ProjectileDrawer.hpp"
+#include "Rendering/ProjectileDrawer.h"
 #include "Rendering/GL/myGL.h"
 #include "Rendering/GL/VertexArray.h"
 #include "Rendering/Textures/TextureAtlas.h"
 #include "Sim/Misc/Wind.h"
 #include "System/myMath.h"
-#include "System/GlobalUnsynced.h"
 
-CR_BIND_DERIVED(CSmokeTrailProjectile, CProjectile, (ZeroVector, ZeroVector, ZeroVector, ZeroVector, NULL, 0, 0, 0, 0, 0, 0, NULL, NULL));
+CR_BIND_DERIVED(CSmokeTrailProjectile, CProjectile, (ZeroVector, ZeroVector, ZeroVector, ZeroVector, NULL, false, false, 0.0f, 0, 0.0f, false, NULL, NULL));
 
 CR_REG_METADATA(CSmokeTrailProjectile,(
 	CR_MEMBER(pos1),
@@ -53,7 +51,7 @@ CSmokeTrailProjectile::CSmokeTrailProjectile(
 	bool firstSegment,
 	bool lastSegment,
 	float size,
-	float time,
+	int time,
 	float color,
 	bool drawTrail,
 	CProjectile* drawCallback,
@@ -64,7 +62,7 @@ CSmokeTrailProjectile::CSmokeTrailProjectile(
 	pos2(pos2),
 	orgSize(size),
 	creationTime(gs->frameNum),
-	lifeTime((int)time),
+	lifeTime(time),
 	color(color),
 	dir1(dir1),
 	dir2(dir2),
@@ -78,9 +76,9 @@ CSmokeTrailProjectile::CSmokeTrailProjectile(
 	checkCol = false;
 	castShadow = true;
 
-	//if no custom texture is defined, use the default texture
-	//Note that this will crash anyway (no idea why) so never have a null texture!
-	if (texture == 0) {
+	// if no custom texture is defined, use the default texture
+	// Note that this will crash anyway (no idea why) so never have a null texture!
+	if (texture == NULL) {
 		texture = projectileDrawer->smoketrailtex;
 	}
 
@@ -92,7 +90,7 @@ CSmokeTrailProjectile::CSmokeTrailProjectile(
 		const float dist = pos1.distance(pos2);
 		dirpos1 = pos1 - dir1 * dist * 0.33f;
 		dirpos2 = pos2 + dir2 * dist * 0.33f;
-		float3 mp = (pos1 + pos2) / 2;
+//		float3 mp = (pos1 + pos2) / 2;
 		midpos = CalcBeizer(0.5f, pos1, dirpos1, dirpos2, pos2);
 		middir = (dir1 + dir2).ANormalize();
 		drawSegmented = true;
@@ -113,24 +111,20 @@ void CSmokeTrailProjectile::Draw()
 {
 	inArray = true;
 	float age = gs->frameNum + globalRendering->timeOffset - creationTime;
-	va->EnlargeArrays(8*4, 0, VA_SIZE_TC);
+	va->EnlargeArrays(8 * 4, 0, VA_SIZE_TC);
 
-	if(drawTrail){
-		float3 dif(pos1-camera->pos2);
-		dif.ANormalize();
-		float3 odir1(dif.cross(dir1));
-		odir1.ANormalize();
-		float3 dif2(pos2-camera->pos2);
-		dif2.ANormalize();
-		float3 odir2(dif2.cross(dir2));
-		odir2.ANormalize();
+	if (drawTrail) {
+		const float3 dif1  = (pos1 - camera->pos).ANormalize();
+		const float3 dif2  = (pos2 - camera->pos).ANormalize();
+		const float3 odir1 = (dif1.cross(dir1)).ANormalize();
+		const float3 odir2 = (dif2.cross(dir2)).ANormalize();
 
 		unsigned char col[4];
-		float a1=(1 - (float) age / lifeTime) * 255;
+		float a1 = (1 - (float) age / lifeTime) * 255;
 		if (lastSegment) {
 			a1 = 0;
 		}
-		a1 *= 0.7f + fabs(dif.dot(dir1));
+		a1 *= 0.7f + fabs(dif1.dot(dir1));
 		float alpha = std::min(255.f, std::max(0.f, a1));
 		col[0] = (unsigned char) (color * alpha);
 		col[1] = (unsigned char) (color * alpha);
@@ -153,50 +147,48 @@ void CSmokeTrailProjectile::Draw()
 		float size2 = 1 + ((age + 8) * (1.0f / lifeTime)) * orgSize;
 
 		if (drawSegmented) {
-			float3 dif3(midpos-camera->pos2);
-			dif3.ANormalize();
-			float3 odir3(dif3.cross(middir));
-			odir3.ANormalize();
+			const float3 dif3 = (midpos - camera->pos).ANormalize();
+			const float3 odir3 = (dif3.cross(middir)).ANormalize();
 			float size3 = 0.2f + ((age + 4) * (1.0f / lifeTime)) * orgSize;
 
 			unsigned char col3[4];
 			float a2 = (1 - (float)(age + 4) / lifeTime) * 255;
 			a2 *= 0.7f + fabs(dif3.dot(middir));
-			alpha = std::min(255.f, std::max(0.f, a2));
+			alpha = std::min(255.0f, std::max(0.0f, a2));
 			col3[0] = (unsigned char) (color * alpha);
 			col3[1] = (unsigned char) (color * alpha);
 			col3[2] = (unsigned char) (color * alpha);
 			col3[3] = (unsigned char) alpha;
 
-			const float midtexx = texture->xstart + (texture->xend - texture->xstart)*0.5f;
+			const float midtexx = texture->xstart + ((texture->xend - texture->xstart) * 0.5f);
 
-			va->AddVertexQTC(pos1 -   odir1 * size,  texture->xstart, texture->ystart, col);
-			va->AddVertexQTC(pos1 +   odir1 * size,  texture->xstart, texture->yend,   col);
-			va->AddVertexQTC(midpos + odir3 * size3, midtexx,         texture->yend,   col3);
-			va->AddVertexQTC(midpos - odir3 * size3, midtexx,         texture->ystart, col3);
+			va->AddVertexQTC(pos1   - (odir1 * size),  texture->xstart, texture->ystart, col);
+			va->AddVertexQTC(pos1   + (odir1 * size),  texture->xstart, texture->yend,   col);
+			va->AddVertexQTC(midpos + (odir3 * size3), midtexx,         texture->yend,   col3);
+			va->AddVertexQTC(midpos - (odir3 * size3), midtexx,         texture->ystart, col3);
 
-			va->AddVertexQTC(midpos - odir3 * size3, midtexx,         texture->ystart, col3);
-			va->AddVertexQTC(midpos + odir3 * size3, midtexx,         texture->yend,   col3);
-			va->AddVertexQTC(pos2 + odir2   * size2, texture->xend,   texture->yend,   col2);
-			va->AddVertexQTC(pos2 - odir2   * size2, texture->xend,   texture->ystart, col2);
+			va->AddVertexQTC(midpos - (odir3 * size3), midtexx,         texture->ystart, col3);
+			va->AddVertexQTC(midpos + (odir3 * size3), midtexx,         texture->yend,   col3);
+			va->AddVertexQTC(pos2   + (odir2 * size2), texture->xend,   texture->yend,   col2);
+			va->AddVertexQTC(pos2   - (odir2 * size2), texture->xend,   texture->ystart, col2);
 		} else {
-			va->AddVertexQTC(pos1 - odir1 * size,    texture->xstart, texture->ystart, col);
-			va->AddVertexQTC(pos1 + odir1 * size,    texture->xstart, texture->yend,   col);
-			va->AddVertexQTC(pos2 + odir2 * size2,   texture->xend,   texture->yend,   col2);
-			va->AddVertexQTC(pos2 - odir2 * size2,   texture->xend,   texture->ystart, col2);
+			va->AddVertexQTC(pos1 - (odir1 * size),    texture->xstart, texture->ystart, col);
+			va->AddVertexQTC(pos1 + (odir1 * size),    texture->xstart, texture->yend,   col);
+			va->AddVertexQTC(pos2 + (odir2 * size2),   texture->xend,   texture->yend,   col2);
+			va->AddVertexQTC(pos2 - (odir2 * size2),   texture->xend,   texture->ystart, col2);
 		}
 	} else {	//draw as particles
 		unsigned char col[4];
 		for (int a = 0; a < 8; ++a) {
-			float a1 = 1 - (float)(age + a) / lifeTime;
-			float alpha = std::min(255.f, std::max(0.f, a1 * 255));
+			const float a1 = 1 - (float)(age + a) / lifeTime;
+			const float alpha = std::min(255.0f, std::max(0.0f, a1 * 255));
 			col[0] = (unsigned char) (color * alpha);
 			col[1] = (unsigned char) (color * alpha);
 			col[2] = (unsigned char) (color * alpha);
 			col[3] = (unsigned char) alpha;
 			float size = ((0.2f + (age + a) * (1.0f / lifeTime)) * orgSize) * 1.2f;
 
-			const float3 pos = CalcBeizer(a / 8.0f, pos1, dirpos1, dirpos2, pos2);
+//			const float3 pos = CalcBeizer(a / 8.0f, pos1, dirpos1, dirpos2, pos2);
 
 			#define st projectileDrawer->smoketex[0]
 			va->AddVertexQTC(pos1 + ( camera->up + camera->right) * size, st->xstart, st->ystart, col);
