@@ -1,7 +1,7 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#ifndef __FIRE_BALL_PROJECTILE_H__
-#define __FIRE_BALL_PROJECTILE_H__
+#ifndef _FIRE_BALL_PROJECTILE_H
+#define _FIRE_BALL_PROJECTILE_H
 
 #include "WeaponProjectile.h"
 #include <deque>
@@ -12,19 +12,20 @@
 #define SPARK_QUEUE std::deque<Spark>
 #endif
 
-class CFireBallProjectile :
-	public CWeaponProjectile
+class CFireBallProjectile : public CWeaponProjectile
 {
 	CR_DECLARE(CFireBallProjectile);
 	CR_DECLARE_SUB(Spark);
 public:
 	CFireBallProjectile(const float3& pos,const float3& speed, CUnit* owner,
 			CUnit *target, const float3 &targetPos, const WeaponDef* weaponDef);
-	~CFireBallProjectile(void);
+	~CFireBallProjectile();
 
 	void Draw();
 	void Update();
 
+	void Collision();
+
 	struct Spark {
 		CR_DECLARE_STRUCT(Spark);
 		float3 pos;
@@ -33,12 +34,10 @@ public:
 		int ttl;
 	};
 
-	void Collision();
-
 private:
 	SPARK_QUEUE sparks;
 
 	void EmitSpark();
 };
 
-#endif // __FIRE_BALL_PROJECTILE_H__
+#endif // _FIRE_BALL_PROJECTILE_H
