@@ -1,7 +1,6 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "Game/GameHelper.h"
 #include "Sim/Projectiles/ProjectileHandler.h"
@@ -16,11 +15,9 @@
 #include "Rendering/Models/IModelParser.h"
 #include "Map/Ground.h"
 #include "System/Matrix44f.h"
-#include "System/GlobalUnsynced.h"
-#include "System/Sound/IEffectChannel.h"
-#include "System/LogOutput.h"
+#include "System/Sound/SoundChannels.h"
 #ifdef TRACE_SYNC
-	#include "Sync/SyncTracer.h"
+	#include "System/Sync/SyncTracer.h"
 #endif
 
 
@@ -39,8 +36,8 @@ CR_REG_METADATA(CWeaponProjectile,(
 	CR_MEMBER(ttl),
 	CR_MEMBER(bounces),
 	CR_MEMBER(keepBouncing),
-	CR_MEMBER(ceg),
 	CR_MEMBER(cegTag),
+	CR_MEMBER(cegID),
 	CR_MEMBER(interceptTarget),
 	CR_MEMBER(id),
 	CR_RESERVED(15),
@@ -58,7 +55,7 @@ CWeaponProjectile::CWeaponProjectile(): CProjectile()
 	interceptTarget = 0;
 	bounces = 0;
 	keepBouncing = true;
-	cegTag = "";
+	cegID = -1U;
 }
 
 CWeaponProjectile::CWeaponProjectile(const float3& pos, const float3& speed,
@@ -72,6 +69,7 @@ CWeaponProjectile::CWeaponProjectile(const float3& pos, const float3& speed,
 	target(target),
 	targetPos(targetPos),
 	cegTag(weaponDef? weaponDef->cegTag: std::string("")),
+	cegID(-1U),
 	colorTeam(0),
 	startpos(pos),
 	ttl(ttl),
@@ -86,21 +84,23 @@ CWeaponProjectile::CWeaponProjectile(const float3& pos, const float3& speed,
 	}
 
 	if (target) {
-		AddDeathDependence(target);
+		AddDeathDependence(target, DEPENDENCE_WEAPONTARGET);
 	}
 
 	if (interceptTarget) {
 		interceptTarget->targeted = true;
-		AddDeathDependence(interceptTarget);
+		AddDeathDependence(interceptTarget, DEPENDENCE_INTERCEPTTARGET);
 	}
-	if (weaponDef) {
-		if(weaponDef->interceptedByShieldType) {
+
+	if (weaponDef != NULL) {
+		if (weaponDef->interceptedByShieldType) {
 			interceptHandler.AddShieldInterceptableProjectile(this);
 		}
 
 		alwaysVisible = weaponDef->visuals.alwaysVisible;
+		ignoreWater = weaponDef->waterweapon;
 
-		model = LoadModel(weaponDef);
+		model = weaponDef->LoadModel();
 
 		collisionFlags = weaponDef->collisionFlags;
 	}
@@ -108,105 +108,64 @@ CWeaponProjectile::CWeaponProjectile(const float3& pos, const float3& speed,
 	ph->AddProjectile(this);
 }
 
-CWeaponProjectile::~CWeaponProjectile(void)
-{
-}
+
 
 void CWeaponProjectile::Collision()
 {
-	if (/* !weaponDef->noExplode || gs->frameNum & 1 */ true) {
-		// don't do damage only on odd-numbered frames
-		// for noExplode projectiles (it breaks coldet)
-		float3 impactDir = speed;
-		impactDir.Normalize();
-
-		// Dynamic Damage
-		DamageArray dynDamages;
-		if (weaponDef->dynDamageExp > 0)
-			dynDamages = weaponDefHandler->DynamicDamages(weaponDef->damages,
-					startpos, pos, (weaponDef->dynDamageRange > 0)
-							? weaponDef->dynDamageRange
-							: weaponDef->range,
-					weaponDef->dynDamageExp, weaponDef->dynDamageMin,
-					weaponDef->dynDamageInverted);
-
-		helper->Explosion(
-			pos,
-			(weaponDef->dynDamageExp > 0)? dynDamages: weaponDef->damages,
-			weaponDef->areaOfEffect,
-			weaponDef->edgeEffectiveness,
-			weaponDef->explosionSpeed,
-			owner(),
-			true,
-			weaponDef->noExplode? 0.3f: 1.0f,
-			weaponDef->noExplode || weaponDef->noSelfDamage,
-			weaponDef->impactOnly,
-			weaponDef->explosionGenerator,
-			0,
-			impactDir,
-			weaponDef->id
-		);
-	}
-
-	if (weaponDef->soundhit.getID(0) > 0) {
-		Channels::Battle.PlaySample(weaponDef->soundhit.getID(0), this, weaponDef->soundhit.getVolume(0));
-	}
-
-	if (!weaponDef->noExplode){
-		CProjectile::Collision();
-	} else {
-		if (TraveledRange()) {
-			CProjectile::Collision();
-		}
-	}
-
+	Collision((CFeature*)NULL);
 }
 
 void CWeaponProjectile::Collision(CFeature* feature)
 {
-	if(gs->randFloat() < weaponDef->fireStarter)
+	if (feature && (gs->randFloat() < weaponDef->fireStarter)) {
 		feature->StartFire();
+	}
 
 	if (/* !weaponDef->noExplode || gs->frameNum & 1 */ true) {
 		// don't do damage only on odd-numbered frames
 		// for noExplode projectiles (it breaks coldet)
 		float3 impactDir = speed;
-		impactDir.Normalize();
+		impactDir.SafeNormalize();
 
 		// Dynamic Damage
-		DamageArray dynDamages;
-		if (weaponDef->dynDamageExp > 0)
-			dynDamages = weaponDefHandler->DynamicDamages(weaponDef->damages,
-					startpos, pos, (weaponDef->dynDamageRange > 0)
-							? weaponDef->dynDamageRange
-							: weaponDef->range,
-					weaponDef->dynDamageExp, weaponDef->dynDamageMin,
-					weaponDef->dynDamageInverted);
+		DamageArray damageArray;
+		if (weaponDef->dynDamageExp > 0) {
+			damageArray = weaponDefHandler->DynamicDamages(
+				weaponDef->damages,
+				startpos, pos,
+				(weaponDef->dynDamageRange > 0)
+					? weaponDef->dynDamageRange
+					: weaponDef->range,
+				weaponDef->dynDamageExp, weaponDef->dynDamageMin,
+				weaponDef->dynDamageInverted);
+		} else {
+			damageArray = weaponDef->damages;
+		}
 
-		helper->Explosion(
+		CGameHelper::ExplosionParams params = {
 			pos,
-			(weaponDef->dynDamageExp > 0)? dynDamages: weaponDef->damages,
+			impactDir,
+			damageArray,
+			weaponDef,
+			owner(),
+			NULL,                                             // hitUnit
+			feature,
 			weaponDef->areaOfEffect,
 			weaponDef->edgeEffectiveness,
 			weaponDef->explosionSpeed,
-			owner(),
-			true,
-			weaponDef->noExplode? 0.3f: 1.0f,
-			weaponDef->noExplode || weaponDef->noSelfDamage,
+			weaponDef->noExplode? 0.3f: 1.0f,                 // gfxMod
 			weaponDef->impactOnly,
-			weaponDef->explosionGenerator,
-			0,
-			impactDir,
-			weaponDef->id,
-			feature
-		);
+			weaponDef->noExplode || weaponDef->noSelfDamage,  // ignoreOwner
+			true                                              // damgeGround
+		};
+		helper->Explosion(params);
 	}
 
 	if (weaponDef->soundhit.getID(0) > 0) {
 		Channels::Battle.PlaySample(weaponDef->soundhit.getID(0), this, weaponDef->soundhit.getVolume(0));
 	}
 
-	if (!weaponDef->noExplode){
+	if (!weaponDef->noExplode) {
 		CProjectile::Collision();
 	} else {
 		if (TraveledRange()) {
@@ -221,38 +180,40 @@ void CWeaponProjectile::Collision(CUnit* unit)
 		// don't do damage only on odd-numbered frames
 		// for noExplode projectiles (it breaks coldet)
 		float3 impactDir = speed;
-		impactDir.Normalize();
+		impactDir.SafeNormalize();
 
 		// Dynamic Damage
-		DamageArray damages;
+		DamageArray damageArray;
 		if (weaponDef->dynDamageExp > 0) {
-			damages = weaponDefHandler->DynamicDamages(weaponDef->damages,
-					startpos, pos,
-					weaponDef->dynDamageRange > 0?
-						weaponDef->dynDamageRange:
-						weaponDef->range,
-					weaponDef->dynDamageExp, weaponDef->dynDamageMin,
-					weaponDef->dynDamageInverted);
+			damageArray = weaponDefHandler->DynamicDamages(
+				weaponDef->damages,
+				startpos, pos,
+				weaponDef->dynDamageRange > 0?
+					weaponDef->dynDamageRange:
+					weaponDef->range,
+				weaponDef->dynDamageExp, weaponDef->dynDamageMin,
+				weaponDef->dynDamageInverted);
 		} else {
-			damages = weaponDef->damages;
+			damageArray = weaponDef->damages;
 		}
 
-		helper->Explosion(
+		CGameHelper::ExplosionParams params = {
 			pos,
-			damages,
+			impactDir,
+			damageArray,
+			weaponDef,
+			owner(),
+			unit,
+			NULL,                                            // hitFeature
 			weaponDef->areaOfEffect,
 			weaponDef->edgeEffectiveness,
 			weaponDef->explosionSpeed,
-			owner(),
-			true,
-			weaponDef->noExplode? 0.3f: 1.0f,
-			weaponDef->noExplode || weaponDef->noSelfDamage,
+			weaponDef->noExplode? 0.3f: 1.0f,                 // gfxMod
 			weaponDef->impactOnly,
-			weaponDef->explosionGenerator,
-			unit,
-			impactDir,
-			weaponDef->id
-		);
+			weaponDef->noExplode || weaponDef->noSelfDamage,  // ignoreOwner
+			true                                              // damageGround
+		};
+		helper->Explosion(params);
 	}
 
 	if (weaponDef->soundhit.getID(0) > 0) {
@@ -260,9 +221,9 @@ void CWeaponProjectile::Collision(CUnit* unit)
 				weaponDef->soundhit.getVolume(0));
 	}
 
-	if (!weaponDef->noExplode)
+	if (!weaponDef->noExplode) {
 		CProjectile::Collision(unit);
-	else {
+	} else {
 		if (TraveledRange())
 			CProjectile::Collision();
 	}
@@ -286,9 +247,9 @@ void CWeaponProjectile::UpdateGroundBounce()
 		float wh = 0;
 
 		if (!weaponDef->waterBounce) {
-			wh = ground->GetHeight2(pos.x, pos.z);
+			wh = ground->GetHeightReal(pos.x, pos.z);
 		} else if (weaponDef->groundBounce) {
-			wh = ground->GetHeight(pos.x, pos.z);
+			wh = ground->GetHeightAboveWater(pos.x, pos.z);
 		}
 
 		if (pos.y < wh) {
@@ -299,17 +260,18 @@ void CWeaponProjectile::UpdateGroundBounce()
 				//Collision();
 				keepBouncing = false;
 			} else {
-				float3 tempPos = pos;
 				const float3& normal = ground->GetNormal(pos.x, pos.z);
+				const float dot = speed.dot(normal);
+
 				pos -= speed;
-				float dot = speed.dot(normal);
-				speed -= (speed + normal*fabs(dot))*(1 - weaponDef->bounceSlip);
-				speed += (normal*(fabs(dot)))*(1 + weaponDef->bounceRebound);
+				speed -= (speed + normal * fabs(dot)) * (1 - weaponDef->bounceSlip);
+				speed += (normal * (fabs(dot))) * (1 + weaponDef->bounceRebound);
 				pos += speed;
 
 				if (weaponDef->bounceExplosionGenerator) {
-					weaponDef->bounceExplosionGenerator->Explosion(pos,
-							speed.Length(), 1, owner(), 1, NULL, normal);
+					weaponDef->bounceExplosionGenerator->Explosion(
+						-1U, pos, speed.Length(), 1, owner(), 1, NULL, normal
+					);
 				}
 			}
 		}
@@ -332,11 +294,13 @@ void CWeaponProjectile::DrawOnMinimap(CVertexArray& lines, CVertexArray& points)
 
 void CWeaponProjectile::DependentDied(CObject* o)
 {
-	if(o==interceptTarget)
-		interceptTarget=0;
+	if (o == interceptTarget) {
+		interceptTarget = NULL;
+	}
 
-	if(o==target)
-		target=0;
+	if (o == target) {
+		target = NULL;
+	}
 }
 
 void CWeaponProjectile::PostLoad()
@@ -346,18 +310,7 @@ void CWeaponProjectile::PostLoad()
 //	if(weaponDef->interceptedByShieldType)
 //		interceptHandler.AddShieldInterceptableProjectile(this);
 
-	if (!weaponDef->visuals.modelName.empty()) {
-		if (weaponDef->visuals.model == NULL) {
-			std::string modelname = string("objects3d/") + weaponDef->visuals.modelName;
-			if (modelname.find(".") == std::string::npos) {
-				modelname += ".3do";
-			}
-			const_cast<WeaponDef*>(weaponDef)->visuals.model = modelParser->Load3DModel(modelname);
-		}
-		if (weaponDef->visuals.model) {
-			model = weaponDef->visuals.model;
-		}
-	}
+	model = weaponDef->LoadModel();
 
 //	collisionFlags = weaponDef->collisionFlags;
 }
