@@ -1,32 +1,27 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
-#include <cassert>
+#include "System/mmgr.h"
 
 #include "AirCAI.h"
-#include "LineDrawer.h"
 #include "Game/GameHelper.h"
+#include "Game/GlobalUnsynced.h"
 #include "Game/SelectedUnits.h"
-#include "Game/UI/CommandColors.h"
 #include "Map/Ground.h"
-#include "Rendering/GL/glExtra.h"
-#include "Rendering/GL/myGL.h"
 #include "Sim/Misc/TeamHandler.h"
-#include "Sim/MoveTypes/AirMoveType.h"
+#include "Sim/MoveTypes/StrafeAirMoveType.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitDef.h"
 #include "Sim/Units/UnitHandler.h"
 #include "Sim/Units/Groups/Group.h"
 #include "Sim/Weapons/Weapon.h"
 #include "Sim/Weapons/WeaponDefHandler.h"
-#include "System/GlobalUnsynced.h"
 #include "System/myMath.h"
-#include "System/LogOutput.h"
+#include "System/Log/ILog.h"
 #include "System/Util.h"
 
+#include <cassert>
 
-CR_BIND_DERIVED(CAirCAI,CMobileCAI , );
+CR_BIND_DERIVED(CAirCAI, CMobileCAI, );
 
 CR_REG_METADATA(CAirCAI, (
 				CR_MEMBER(basePos),
@@ -58,74 +53,76 @@ CAirCAI::CAirCAI(CUnit* owner)
 	cancelDistance = 16000;
 	CommandDescription c;
 
-	if(owner->unitDef->canAttack){
-		c.id=CMD_AREA_ATTACK;
-		c.action="areaattack";
-		c.type=CMDTYPE_ICON_AREA;
-		c.name="Area attack";
-		c.mouseicon=c.name;
-		c.tooltip="Sets the aircraft to attack enemy units within a circle";
+	if (owner->unitDef->canAttack) {
+		c.id = CMD_AREA_ATTACK;
+		c.action = "areaattack";
+		c.type = CMDTYPE_ICON_AREA;
+		c.name = "Area attack";
+		c.mouseicon = c.name;
+		c.tooltip = "Sets the aircraft to attack enemy units within a circle";
 		possibleCommands.push_back(c);
 	}
 
-	if(owner->unitDef->canLoopbackAttack){
+	if (owner->unitDef->canLoopbackAttack) {
 		c.params.clear();
-		c.id=CMD_LOOPBACKATTACK;
-		c.action="loopbackattack";
-		c.type=CMDTYPE_ICON_MODE;
-		c.name="Loopback";
-		c.mouseicon=c.name;
+		c.id = CMD_LOOPBACKATTACK;
+		c.action = "loopbackattack";
+		c.type = CMDTYPE_ICON_MODE;
+		c.name = "Loopback";
+		c.mouseicon = c.name;
 		c.params.push_back("0");
 		c.params.push_back("Normal");
 		c.params.push_back("Loopback");
-		c.tooltip="Loopback attack: Sets if the aircraft should loopback after an attack instead of overflying target";
+		c.tooltip = "Loopback attack: Sets if the aircraft should loopback after an attack instead of overflying target";
 		possibleCommands.push_back(c);
 		nonQueingCommands.insert(CMD_LOOPBACKATTACK);
 	}
 
-	basePos=owner->pos;
-	goalPos=owner->pos;
+	basePos = owner->pos;
+	goalPos = owner->pos;
 }
 
-CAirCAI::~CAirCAI(void)
+CAirCAI::~CAirCAI()
 {
 }
 
-void CAirCAI::GiveCommandReal(const Command &c)
+void CAirCAI::GiveCommandReal(const Command& c)
 {
 	// take care not to allow aircraft to be ordered to move out of the map
-	if (c.id != CMD_MOVE && !AllowedCommand(c, true))
+	if ((c.GetID() != CMD_MOVE) && !AllowedCommand(c, true)) {
 		return;
-
-	else if (c.id == CMD_MOVE && c.params.size() >= 3 &&
-			(c.params[0] < 0.f || c.params[2] < 0.f
+	} else if (c.GetID() == CMD_MOVE && c.params.size() >= 3 &&
+			(c.params[0] < 0.0f || c.params[2] < 0.0f
 			 || c.params[0] > gs->mapx*SQUARE_SIZE
 			 || c.params[2] > gs->mapy*SQUARE_SIZE))
+	{
 		return;
+	}
 
-	if (c.id == CMD_SET_WANTED_MAX_SPEED) {
+	if (c.GetID() == CMD_SET_WANTED_MAX_SPEED) {
 	  return;
 	}
 
-	if (c.id == CMD_AUTOREPAIRLEVEL) {
+	if (c.GetID() == CMD_AUTOREPAIRLEVEL) {
 		if (c.params.empty()) {
 			return;
 		}
-		CAirMoveType* airMT;
+		CStrafeAirMoveType* airMT;
 		if (owner->usingScriptMoveType) {
-			airMT = (CAirMoveType*)owner->prevMoveType;
+			airMT = (CStrafeAirMoveType*)owner->prevMoveType;
 		} else {
-			airMT = (CAirMoveType*)owner->moveType;
+			airMT = (CStrafeAirMoveType*)owner->moveType;
 		}
-		switch((int)c.params[0]){
+		switch ((int)c.params[0]) {
 			case 0: { airMT->repairBelowHealth = 0.0f; break; }
 			case 1: { airMT->repairBelowHealth = 0.3f; break; }
 			case 2: { airMT->repairBelowHealth = 0.5f; break; }
 			case 3: { airMT->repairBelowHealth = 0.8f; break; }
 		}
-		for(vector<CommandDescription>::iterator cdi = possibleCommands.begin();
-				cdi != possibleCommands.end(); ++cdi){
-			if(cdi->id==CMD_AUTOREPAIRLEVEL){
+		for (vector<CommandDescription>::iterator cdi = possibleCommands.begin();
+				cdi != possibleCommands.end(); ++cdi)
+		{
+			if (cdi->id == CMD_AUTOREPAIRLEVEL) {
 				char t[10];
 				SNPRINTF(t,10,"%d", (int)c.params[0]);
 				cdi->params[0]=t;
@@ -136,23 +133,24 @@ void CAirCAI::GiveCommandReal(const Command &c)
 		return;
 	}
 
-	if (c.id == CMD_IDLEMODE) {
+	if (c.GetID() == CMD_IDLEMODE) {
 		if (c.params.empty()) {
 			return;
 		}
-		CAirMoveType* airMT;
+		CStrafeAirMoveType* airMT;
 		if (owner->usingScriptMoveType) {
-			airMT = (CAirMoveType*)owner->prevMoveType;
+			airMT = (CStrafeAirMoveType*)owner->prevMoveType;
 		} else {
-			airMT = (CAirMoveType*)owner->moveType;
+			airMT = (CStrafeAirMoveType*)owner->moveType;
 		}
-		switch((int)c.params[0]){
+		switch ((int)c.params[0]){
 			case 0: { airMT->autoLand = false; break; }
 			case 1: { airMT->autoLand = true;  break; }
 		}
-		for(vector<CommandDescription>::iterator cdi = possibleCommands.begin();
-				cdi != possibleCommands.end(); ++cdi){
-			if(cdi->id == CMD_IDLEMODE){
+		for (vector<CommandDescription>::iterator cdi = possibleCommands.begin();
+				cdi != possibleCommands.end(); ++cdi)
+		{
+			if (cdi->id == CMD_IDLEMODE){
 				char t[10];
 				SNPRINTF(t, 10, "%d", (int)c.params[0]);
 				cdi->params[0] = t;
@@ -163,23 +161,24 @@ void CAirCAI::GiveCommandReal(const Command &c)
 		return;
 	}
 
-	if (c.id == CMD_LOOPBACKATTACK) {
+	if (c.GetID() == CMD_LOOPBACKATTACK) {
 		if (c.params.empty()) {
 			return;
 		}
-		CAirMoveType* airMT;
+		CStrafeAirMoveType* airMT;
 		if (owner->usingScriptMoveType) {
-			airMT = (CAirMoveType*)owner->prevMoveType;
+			airMT = (CStrafeAirMoveType*)owner->prevMoveType;
 		} else {
-			airMT = (CAirMoveType*)owner->moveType;
+			airMT = (CStrafeAirMoveType*)owner->moveType;
 		}
-		switch((int)c.params[0]){
+		switch ((int)c.params[0]) {
 			case 0: { airMT->loopbackAttack = false; break; }
 			case 1: { airMT->loopbackAttack = true;  break; }
 		}
-		for(vector<CommandDescription>::iterator cdi = possibleCommands.begin();
-				cdi != possibleCommands.end(); ++cdi){
-			if(cdi->id == CMD_LOOPBACKATTACK){
+		for (vector<CommandDescription>::iterator cdi = possibleCommands.begin();
+				cdi != possibleCommands.end(); ++cdi)
+		{
+			if (cdi->id == CMD_LOOPBACKATTACK){
 				char t[10];
 				SNPRINTF(t, 10, "%d", (int)c.params[0]);
 				cdi->params[0] = t;
@@ -191,14 +190,15 @@ void CAirCAI::GiveCommandReal(const Command &c)
 	}
 
 	if (!(c.options & SHIFT_KEY)
-			&& nonQueingCommands.find(c.id) == nonQueingCommands.end()) {
-		activeCommand=0;
-		tempOrder=false;
+			&& nonQueingCommands.find(c.GetID()) == nonQueingCommands.end())
+	{
+		activeCommand = 0;
+		tempOrder = false;
 	}
 
-	if (c.id == CMD_AREA_ATTACK && c.params.size() < 4){
-		Command c2 = c;
-		c2.id = CMD_ATTACK;
+	if (c.GetID() == CMD_AREA_ATTACK && c.params.size() < 4) {
+		Command c2(CMD_ATTACK, c.options);
+		c2.params = c.params;
 		CCommandAI::GiveAllowedCommand(c2);
 		return;
 	}
@@ -208,55 +208,54 @@ void CAirCAI::GiveCommandReal(const Command &c)
 
 void CAirCAI::SlowUpdate()
 {
-	if(gs->paused) // Commands issued may invoke SlowUpdate when paused
+	if (gs->paused) { // Commands issued may invoke SlowUpdate when paused
 		return;
-	if (!commandQue.empty() && commandQue.front().timeOut < gs->frameNum) {
+	}
+	if (!commandQue.empty() && (commandQue.front().timeOut < gs->frameNum)) {
 		FinishCommand();
 		return;
 	}
 
 	if (owner->usingScriptMoveType) {
-		return; // avoid the invalid (CAirMoveType*) cast
+		return; // avoid the invalid (CStrafeAirMoveType*) cast
 	}
 
-	AAirMoveType* myPlane=(AAirMoveType*) owner->moveType;
+	AAirMoveType* myPlane = (AAirMoveType*) owner->moveType;
 
 	bool wantToRefuel = LandRepairIfNeeded();
-	if(!wantToRefuel && owner->unitDef->maxFuel > 0){
+	if (!wantToRefuel && owner->unitDef->maxFuel > 0) {
 		wantToRefuel = RefuelIfNeeded();
 	}
 
-	if(commandQue.empty()){
-		if(myPlane->aircraftState == AAirMoveType::AIRCRAFT_FLYING
-				&& !owner->unitDef->DontLand() && myPlane->autoLand) {
+	if (commandQue.empty()) {
+		if (myPlane->aircraftState == AAirMoveType::AIRCRAFT_FLYING
+				&& !owner->unitDef->DontLand() && myPlane->autoLand)
+		{
 			StopMove();
 //			myPlane->SetState(AAirMoveType::AIRCRAFT_LANDING);
 		}
 
-		if(owner->unitDef->canAttack && owner->fireState>=2
-				&& owner->moveState != 0 && owner->maxRange > 0){
-			if(myPlane->IsFighter()){
-				float testRad=1000 * owner->moveState;
-				CUnit* enemy=helper->GetClosestEnemyAircraft(
-					owner->pos + (owner->speed * 10), testRad, owner->allyteam);
-				if(IsValidTarget(enemy)) {
-					Command nc;
-					nc.id = CMD_ATTACK;
+		if (owner->unitDef->canAttack && owner->fireState >= FIRESTATE_FIREATWILL
+				&& owner->moveState != MOVESTATE_HOLDPOS && owner->maxRange > 0)
+		{
+			if (myPlane->IsFighter()) {
+				float testRad = 1000 * owner->moveState;
+				CUnit* enemy = helper->GetClosestEnemyAircraft(owner->pos 
+						+ (owner->speed * 10), testRad, owner->allyteam);
+				if (IsValidTarget(enemy)) {
+					Command nc(CMD_ATTACK);
 					nc.params.push_back(enemy->id);
-					nc.options = 0;
 					commandQue.push_front(nc);
 					inCommand = false;
 					return;
 				}
 			}
 			const float searchRadius = 500 * owner->moveState;
-			CUnit* enemy = helper->GetClosestValidTarget(
-				owner->pos + (owner->speed * 20), searchRadius, owner->allyteam, this);
+			CUnit* enemy = helper->GetClosestValidTarget(owner->pos
+					+ (owner->speed * 20), searchRadius, owner->allyteam, this);
 			if (enemy != NULL) {
-				Command nc;
-				nc.id = CMD_ATTACK;
+				Command nc(CMD_ATTACK);
 				nc.params.push_back(enemy->id);
-				nc.options = 0;
 				commandQue.push_front(nc);
 				inCommand = false;
 				return;
@@ -266,23 +265,26 @@ void CAirCAI::SlowUpdate()
 	}
 
 	Command& c = commandQue.front();
-
-	if (c.id == CMD_WAIT) {
+	const int& cmd_id = c.GetID();
+	
+	if (cmd_id == CMD_WAIT) {
 		if ((myPlane->aircraftState == AAirMoveType::AIRCRAFT_FLYING)
-		    && !owner->unitDef->DontLand() && myPlane->autoLand) {
+		    	&& !owner->unitDef->DontLand() && myPlane->autoLand)
+		{
 			StopMove();
 //			myPlane->SetState(AAirMoveType::AIRCRAFT_LANDING);
 		}
 		return;
 	}
 
-	if (c.id != CMD_STOP && c.id != CMD_AUTOREPAIRLEVEL
-			&& c.id != CMD_IDLEMODE && c.id != CMD_SET_WANTED_MAX_SPEED) {
+	if (cmd_id != CMD_STOP && cmd_id != CMD_AUTOREPAIRLEVEL
+			&& cmd_id != CMD_IDLEMODE && cmd_id != CMD_SET_WANTED_MAX_SPEED)
+	{
 		myPlane->Takeoff();
 	}
 
 	if (wantToRefuel) {
-		switch (c.id) {
+		switch (cmd_id) {
 			case CMD_AREA_ATTACK:
 			case CMD_ATTACK:
 			case CMD_FIGHT:
@@ -290,12 +292,12 @@ void CAirCAI::SlowUpdate()
 		}
 	}
 
-	switch(c.id){
-		case CMD_AREA_ATTACK:{
+	switch (cmd_id) {
+		case CMD_AREA_ATTACK: {
 			ExecuteAreaAttack(c);
 			return;
 		}
-		default:{
+		default: {
 			CMobileCAI::Execute();
 			return;
 		}
@@ -303,7 +305,7 @@ void CAirCAI::SlowUpdate()
 }
 
 /*
-void CAirCAI::ExecuteMove(Command &c){
+void CAirCAI::ExecuteMove(Command& c){
 	if(tempOrder){
 		tempOrder = false;
 		inCommand = true;
@@ -315,16 +317,16 @@ void CAirCAI::ExecuteMove(Command &c){
 	float3 pos(c.params[0], c.params[1], c.params[2]);
 	commandPos2 = pos;
 	myPlane->goalPos = pos;// This is not what we want move to do
-	if(owner->unitDef->canAttack && !(c.options & CONTROL_KEY)){
-		if(owner->fireState >= 2 && owner->moveState != 0 && owner->maxRange > 0){
+	if (owner->unitDef->canAttack && !(c.options & CONTROL_KEY)){
+		if (owner->fireState >= FIRESTATE_FIREATWILL && owner->moveState != MOVESTATE_HOLDPOS && owner->maxRange > 0) {
 			if(myPlane->isFighter){
 				float testRad = 500 * owner->moveState;
 				CUnit* enemy = helper->GetClosestEnemyAircraft(
 					owner->pos+owner->speed * 20, testRad, owner->allyteam);
 				if(enemy && ((enemy->unitDef->canfly && !enemy->crashing
 						&& myPlane->isFighter) || (!enemy->unitDef->canfly
-						&& (!myPlane->isFighter || owner->moveState == 2)))){
-					if(owner->moveState != 1
+						&& (!myPlane->isFighter || owner->moveState == MOVESTATE_ROAM)))) {
+					if(owner->moveState != MOVESTATE_MANEUVER
 							|| LinePointDist(commandPos1, commandPos2, enemy->pos) < 1000){
 						Command nc;
 						nc.id = CMD_ATTACK;
@@ -338,15 +340,15 @@ void CAirCAI::ExecuteMove(Command &c){
 					}
 				}
 			}
-			if((!myPlane->isFighter || owner->moveState == 2) && owner->maxRange > 0){
+			if((!myPlane->isFighter || owner->moveState == MOVESTATE_ROAM) && owner->maxRange > 0){
 				float testRad = 325 * owner->moveState;
 				CUnit* enemy = helper->GetClosestEnemyUnit(
 					owner->pos + owner->speed * 30, testRad, owner->allyteam);
 				if(enemy && (owner->hasUWWeapons || !enemy->isUnderWater)
 						&& ((enemy->unitDef->canfly && !enemy->crashing
 						&& myPlane->isFighter) || (!enemy->unitDef->canfly
-						&& (!myPlane->isFighter || owner->moveState==2)))){
-					if(owner->moveState!=1 || LinePointDist(
+						&& (!myPlane->isFighter || owner->moveState == MOVESTATE_ROAM)))){
+					if(owner->moveState != MOVESTATE_MANEUVER || LinePointDist(
 							commandPos1, commandPos2, enemy->pos) < 1000){
 						Command nc;
 						nc.id = CMD_ATTACK;
@@ -368,25 +370,26 @@ void CAirCAI::ExecuteMove(Command &c){
 	return;
 }*/
 
-void CAirCAI::ExecuteFight(Command &c)
+void CAirCAI::ExecuteFight(Command& c)
 {
 	assert((c.options & INTERNAL_ORDER) || owner->unitDef->canFight);
 	AAirMoveType* myPlane = (AAirMoveType*) owner->moveType;
-	if(tempOrder){
-		tempOrder=false;
-		inCommand=true;
+	if (tempOrder) {
+		tempOrder = false;
+		inCommand = true;
 	}
-	if(c.params.size()<3){		//this shouldnt happen but anyway ...
-		logOutput.Print("Error: got fight cmd with less than 3 params on %s in AirCAI",
-			owner->unitDef->humanName.c_str());
+	if (c.params.size() < 3) { // this shouldn't happen but anyway ...
+		LOG_L(L_ERROR,
+				"Received a Fight command with less than 3 params on %s in AirCAI",
+				owner->unitDef->humanName.c_str());
 		return;
 	}
-	if(c.params.size() >= 6){
-		if(!inCommand){
-			commandPos1 = float3(c.params[3],c.params[4],c.params[5]);
+	if (c.params.size() >= 6) {
+		if (!inCommand) {
+			commandPos1 = float3(c.params[3], c.params[4], c.params[5]);
 		}
 	} else {
-		// Some hackery to make sure the line (commandPos1,commandPos2) is NOT
+		// HACK to make sure the line (commandPos1,commandPos2) is NOT
 		// rotated (only shortened) if we reach this because the previous return
 		// fight command finished by the 'if((curPos-pos).SqLength2D()<(127*127)){'
 		// condition, but is actually updated correctly if you click somewhere
@@ -397,36 +400,35 @@ void CAirCAI::ExecuteFight(Command &c)
 		}
 	}
 	goalPos = float3(c.params[0], c.params[1], c.params[2]);
-	if(!inCommand){
+	if (!inCommand) {
 		inCommand = true;
-		commandPos2=goalPos;
+		commandPos2 = goalPos;
 	}
-	if(c.params.size() >= 6){
+	if (c.params.size() >= 6) {
 		goalPos = ClosestPointOnLine(commandPos1, commandPos2, owner->pos);
 	}
 
-	// CMD_FIGHT is pretty useless if !canAttack but we try to honour the modders wishes anyway...
-	if (owner->unitDef->canAttack && owner->fireState >= 2
-			&& owner->moveState != 0 && owner->maxRange > 0) {
+	// CMD_FIGHT is pretty useless if !canAttack, but we try to honour the modders wishes anyway...
+	if (owner->unitDef->canAttack && (owner->fireState >= FIRESTATE_FIREATWILL)
+			&& (owner->moveState != MOVESTATE_HOLDPOS) && (owner->maxRange > 0))
+	{
 		CUnit* enemy = NULL;
-		if(myPlane->IsFighter()) {
+		if (myPlane->IsFighter()) {
 			const float3 curPosOnLine = ClosestPointOnLine(
 					commandPos1, commandPos2, owner->pos + owner->speed*10);
 			const float searchRadius = 1000 * owner->moveState;
 			enemy = helper->GetClosestEnemyAircraft(curPosOnLine, searchRadius, owner->allyteam);
 		}
-		if(IsValidTarget(enemy) && (owner->moveState!=1
+		if (IsValidTarget(enemy) && (owner->moveState != MOVESTATE_MANEUVER
 				|| LinePointDist(commandPos1, commandPos2, enemy->pos) < 1000))
 		{
-			Command nc;
-			nc.id=CMD_ATTACK;
+			Command nc(CMD_ATTACK, c.options|INTERNAL_ORDER);
 			nc.params.push_back(enemy->id);
-			nc.options=c.options|INTERNAL_ORDER;
 			commandQue.push_front(nc);
-			tempOrder=true;
-			inCommand=false;
-			if(lastPC1!=gs->frameNum){	//avoid infinite loops
-				lastPC1=gs->frameNum;
+			tempOrder = true;
+			inCommand = false;
+			if (lastPC1 != gs->frameNum) { // avoid infinite loops
+				lastPC1 = gs->frameNum;
 				SlowUpdate();
 			}
 			return;
@@ -436,15 +438,13 @@ void CAirCAI::ExecuteFight(Command &c)
 			const float searchRadius = 500 * owner->moveState;
 			enemy = helper->GetClosestValidTarget(curPosOnLine, searchRadius, owner->allyteam, this);
 			if (enemy != NULL) {
-				Command nc;
-				nc.id = CMD_ATTACK;
+				Command nc(CMD_ATTACK, c.options|INTERNAL_ORDER);
 				nc.params.push_back(enemy->id);
-				nc.options = c.options | INTERNAL_ORDER;
 				PushOrUpdateReturnFight();
 				commandQue.push_front(nc);
 				tempOrder = true;
 				inCommand = false;
-				if(lastPC2 != gs->frameNum){	//avoid infinite loops
+				if (lastPC2 != gs->frameNum) { // avoid infinite loops
 					lastPC2 = gs->frameNum;
 					SlowUpdate();
 				}
@@ -454,12 +454,13 @@ void CAirCAI::ExecuteFight(Command &c)
 	}
 	myPlane->goalPos = goalPos;
 
-	CAirMoveType* airmt = dynamic_cast<CAirMoveType*>(myPlane);
+	CStrafeAirMoveType* airmt = dynamic_cast<CStrafeAirMoveType*>(myPlane);
 	const float radius = airmt ? std::max(airmt->turnRadius + 2*SQUARE_SIZE, 128.f) : 127.f;
 
 	// we're either circling or will get to the target in 8 frames
-	if((owner->pos - goalPos).SqLength2D() < (radius * radius)
-			|| (owner->pos + owner->speed*8 - goalPos).SqLength2D() < 127*127) {
+	if ((owner->pos - goalPos).SqLength2D() < (radius * radius)
+			|| (owner->pos + owner->speed*8 - goalPos).SqLength2D() < 127*127)
+	{
 		FinishCommand();
 	}
 	return;
@@ -470,7 +471,7 @@ void CAirCAI::ExecuteAttack(Command& c)
 	assert(owner->unitDef->canAttack);
 	targetAge++;
 
-	if (tempOrder && owner->moveState == 1) {
+	if (tempOrder && owner->moveState == MOVESTATE_MANEUVER) {
 		// limit how far away we fly
 		if (orderTarget && LinePointDist(commandPos1, commandPos2, orderTarget->pos) > 1500) {
 			owner->SetUserTarget(0);
@@ -509,11 +510,10 @@ void CAirCAI::ExecuteAttack(Command& c)
 			CUnit* targetUnit = uh->GetUnit(c.params[0]);
 
 			if (targetUnit != NULL && targetUnit != owner) {
-				orderTarget = targetUnit;
-				owner->AttackUnit(orderTarget, false);
-				AddDeathDependence(orderTarget);
+				SetOrderTarget(targetUnit);
+				owner->AttackUnit(targetUnit, false);
 				inCommand = true;
-				SetGoal(orderTarget->pos, owner->pos, cancelDistance);
+				SetGoal(targetUnit->pos, owner->pos, cancelDistance);
 			} else {
 				FinishCommand();
 				return;
@@ -528,7 +528,7 @@ void CAirCAI::ExecuteAttack(Command& c)
 	return;
 }
 
-void CAirCAI::ExecuteAreaAttack(Command &c)
+void CAirCAI::ExecuteAreaAttack(Command& c)
 {
 	assert(owner->unitDef->canAttack);
 	AAirMoveType* myPlane = (AAirMoveType*) owner->moveType;
@@ -546,8 +546,7 @@ void CAirCAI::ExecuteAreaAttack(Command &c)
 			inCommand = false;
 		if (orderTarget && orderTarget->pos.SqDistance2D(pos) > Square(radius)) {
 			inCommand = false;
-			DeleteDeathDependence(orderTarget);
-			orderTarget = 0;
+			SetOrderTarget(NULL);
 		}
 		if (owner->commandShotCount < 0) {
 			if ((c.params.size() == 4) && (commandQue.size() > 1)) {
@@ -557,7 +556,7 @@ void CAirCAI::ExecuteAreaAttack(Command &c)
 			else if (owner->userAttackGround) {
 				// reset the attack position after each run
 				float3 attackPos = pos + (gs->randVector() * radius);
-					attackPos.y = ground->GetHeight(attackPos.x, attackPos.z);
+				attackPos.y = ground->GetHeightAboveWater(attackPos.x, attackPos.z);
 
 				owner->AttackGround(attackPos, false);
 				owner->commandShotCount = 0;
@@ -574,15 +573,14 @@ void CAirCAI::ExecuteAreaAttack(Command &c)
 
 			if (enemyUnitIDs.empty()) {
 				float3 attackPos = pos + gs->randVector() * radius;
-				attackPos.y = ground->GetHeight(attackPos.x, attackPos.z);
+				attackPos.y = ground->GetHeightAboveWater(attackPos.x, attackPos.z);
 				owner->AttackGround(attackPos, false);
 			} else {
 				// note: the range of randFloat() is inclusive of 1.0f
 				const unsigned int idx(gs->randFloat() * (enemyUnitIDs.size() - 1));
-
-				orderTarget = uh->GetUnitUnsafe( enemyUnitIDs[idx] );
-				owner->AttackUnit(orderTarget, false);
-				AddDeathDependence(orderTarget);
+				CUnit* targetUnit = uh->GetUnitUnsafe(enemyUnitIDs[idx]);
+				SetOrderTarget(targetUnit);
+				owner->AttackUnit(targetUnit, false);
 			}
 		}
 	}
@@ -594,41 +592,40 @@ void CAirCAI::ExecuteGuard(Command& c)
 
 	const CUnit* guardee = uh->GetUnit(c.params[0]);
 
-	if (guardee != NULL && UpdateTargetLostTimer(guardee->id)) {
-		if (owner->unitDef->canAttack && guardee->lastAttack + 40 < gs->frameNum
-				&& owner->maxRange > 0 && IsValidTarget(guardee->lastAttacker))
-		{
-			Command nc;
-			nc.id = CMD_ATTACK;
-			nc.params.push_back(guardee->lastAttacker->id);
-			nc.options = c.options | INTERNAL_ORDER;
-			commandQue.push_front(nc);
-			SlowUpdate();
-			return;
-		} else {
-			Command c2;
-				c2.id = CMD_MOVE;
-				c2.options = c.options | INTERNAL_ORDER;
-				c2.timeOut = gs->frameNum + 60;
+	if (guardee == NULL) { FinishCommand(); return; }
+	if (UpdateTargetLostTimer(guardee->id) == 0) { FinishCommand(); return; }
+	if (guardee->outOfMapTime > (GAME_SPEED * 5)) { FinishCommand(); return; }
 
-			if (guardee->pos.IsInBounds()) {
-				c2.params.push_back(guardee->pos.x);
-				c2.params.push_back(guardee->pos.y);
-				c2.params.push_back(guardee->pos.z);
-			} else {
-				float3 clampedGuardeePos = guardee->pos;
-					clampedGuardeePos.CheckInBounds();
+	const bool pushAttackCommand =
+		(owner->maxRange > 0.0f) &&
+		owner->unitDef->canAttack &&
+		((guardee->lastAttack + 40) < gs->frameNum) &&
+		IsValidTarget(guardee->lastAttacker);
 
-				c2.params.push_back(clampedGuardeePos.x);
-				c2.params.push_back(clampedGuardeePos.y);
-				c2.params.push_back(clampedGuardeePos.z);
-			}
-
-			commandQue.push_front(c2);
-			return;
-		}
+	if (pushAttackCommand) {
+		Command nc(CMD_ATTACK, c.options|INTERNAL_ORDER);
+		nc.params.push_back(guardee->lastAttacker->id);
+		commandQue.push_front(nc);
+		SlowUpdate();
 	} else {
-		FinishCommand();
+		Command c2(CMD_MOVE, c.options|INTERNAL_ORDER);
+		c2.timeOut = gs->frameNum + 60;
+
+		if (guardee->pos.IsInBounds()) {
+			c2.params.push_back(guardee->pos.x);
+			c2.params.push_back(guardee->pos.y);
+			c2.params.push_back(guardee->pos.z);
+		} else {
+			float3 clampedGuardeePos = guardee->pos;
+
+			clampedGuardeePos.CheckInBounds();
+
+			c2.params.push_back(clampedGuardeePos.x);
+			c2.params.push_back(clampedGuardeePos.y);
+			c2.params.push_back(clampedGuardeePos.z);
+		}
+
+		commandQue.push_front(c2);
 	}
 }
 
@@ -650,101 +647,29 @@ int CAirCAI::GetDefaultCmd(const CUnit* pointed, const CFeature* feature)
 
 bool CAirCAI::IsValidTarget(const CUnit* enemy) const {
 	return CMobileCAI::IsValidTarget(enemy) && !enemy->crashing
-		&& (((CAirMoveType*)owner->moveType)->isFighter || !enemy->unitDef->canfly);
+		&& (((CStrafeAirMoveType*)owner->moveType)->isFighter || !enemy->unitDef->canfly);
 }
 
-void CAirCAI::DrawCommands(void)
+
+
+void CAirCAI::FinishCommand()
 {
-	lineDrawer.StartPath(owner->drawMidPos, cmdColors.start);
-
-	if (owner->selfDCountdown != 0) {
-		lineDrawer.DrawIconAtLastPos(CMD_SELFD);
-	}
-
-	CCommandQueue::iterator ci;
-	for (ci = commandQue.begin(); ci != commandQue.end(); ++ci) {
-		switch (ci->id) {
-			case CMD_MOVE: {
-				const float3 endPos(ci->params[0], ci->params[1], ci->params[2]);
-				lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.move);
-				break;
-			}
-			case CMD_FIGHT: {
-				const float3 endPos(ci->params[0], ci->params[1], ci->params[2]);
-				lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.fight);
-				break;
-			}
-			case CMD_PATROL: {
-				const float3 endPos(ci->params[0], ci->params[1], ci->params[2]);
-				lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.patrol);
-				break;
-			}
-			case CMD_ATTACK: {
-				if (ci->params.size() == 1) {
-					const CUnit* unit = uh->GetUnit(ci->params[0]);
-
-					if ((unit != NULL) && isTrackable(unit)) {
-						const float3 endPos = helper->GetUnitErrorPos(unit, owner->allyteam);
-						lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.attack);
-					}
-				} else {
-					const float3 endPos(ci->params[0], ci->params[1], ci->params[2]);
-					lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.attack);
-				}
-				break;
-			}
-			case CMD_AREA_ATTACK: {
-				const float3 endPos(ci->params[0], ci->params[1], ci->params[2]);
-
-				lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.attack);
-				lineDrawer.Break(endPos, cmdColors.attack);
-				glColor4fv(cmdColors.attack);
-				glSurfaceCircle(endPos, ci->params[3], 20);
-				lineDrawer.RestartWithColor(cmdColors.attack);
-				break;
-			}
-			case CMD_GUARD: {
-				const CUnit* unit = uh->GetUnit(ci->params[0]);
-
-				if ((unit != NULL) && isTrackable(unit)) {
-					const float3 endPos = helper->GetUnitErrorPos(unit, owner->allyteam);
-					lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.guard);
-				}
-				break;
-			}
-			case CMD_WAIT: {
-				DrawWaitIcon(*ci);
-				break;
-			}
-			case CMD_SELFD: {
-				lineDrawer.DrawIconAtLastPos(ci->id);
-				break;
-			}
-			default:
-				DrawDefaultCommand(*ci);
-				break;
-		}
-	}
-	lineDrawer.FinishPath();
-}
-
-void CAirCAI::FinishCommand(void)
-{
-	targetAge=0;
+	targetAge = 0;
 	CCommandAI::FinishCommand();
 }
 
-void CAirCAI::BuggerOff(float3 pos, float radius)
+void CAirCAI::BuggerOff(const float3& pos, float radius)
 {
 	AAirMoveType* myPlane = dynamic_cast<AAirMoveType*>(owner->moveType);
-	if(myPlane) {
+	if (myPlane) {
 		myPlane->Takeoff();
 	} else {
 		CMobileCAI::BuggerOff(pos, radius);
 	}
 }
 
-void CAirCAI::SetGoal(const float3 &pos, const float3 &curPos, float goalRadius){
+void CAirCAI::SetGoal(const float3& pos, const float3& curPos, float goalRadius)
+{
 	owner->moveType->SetGoal(pos);
 	CMobileCAI::SetGoal(pos, curPos, goalRadius);
 }
