@@ -1,21 +1,15 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "CommandAI.h"
 #include "FactoryCAI.h"
-#include "LineDrawer.h"
 #include "ExternalAI/EngineOutHandler.h"
 #include "ExternalAI/SkirmishAIHandler.h"
-#include "Sim/Units/Groups/Group.h"
 #include "Game/GameHelper.h"
+#include "Game/GlobalUnsynced.h"
 #include "Game/SelectedUnits.h"
 #include "Game/WaitCommandsAI.h"
-#include "Game/UI/CommandColors.h"
-#include "Game/UI/CursorIcons.h"
-#include "Rendering/GL/myGL.h"
-#include "Rendering/GL/glExtra.h"
 #include "Lua/LuaRules.h"
 #include "Map/Ground.h"
 #include "Map/MapDamage.h"
@@ -23,24 +17,24 @@
 #include "Sim/Features/FeatureHandler.h"
 #include "Sim/Misc/TeamHandler.h"
 #include "Sim/MoveTypes/MoveType.h"
+#include "Sim/Units/BuildInfo.h"
 #include "Sim/Units/UnitDef.h"
 #include "Sim/Units/UnitDefHandler.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitHandler.h"
+#include "Sim/Units/Groups/Group.h"
 #include "Sim/Units/UnitTypes/Factory.h"
 #include "Sim/Weapons/WeaponDefHandler.h"
 #include "Sim/Weapons/Weapon.h"
-#include "EventHandler.h"
-#include "LoadSave/LoadSaveInterface.h"
-#include "LogOutput.h"
-#include "myMath.h"
-#include "creg/STL_Set.h"
-#include "creg/STL_Deque.h"
+#include "System/EventHandler.h"
+#include "System/LoadSave/LoadSaveInterface.h"
+#include "System/myMath.h"
+#include "System/Util.h"
+#include "System/creg/STL_Set.h"
+#include "System/creg/STL_Deque.h"
 #include <assert.h>
-#include "GlobalUnsynced.h"
-#include "Util.h"
 
-const int TARGET_LOST_TIMER =120;	// in calls to SlowUpdate() (approx. once every second)
+static const int TARGET_LOST_TIMER = 120; // in calls to SlowUpdate() (approx. twice every second)
 
 CR_BIND(CCommandQueue, );
 CR_REG_METADATA(CCommandQueue, (
@@ -115,23 +109,23 @@ CCommandAI::CCommandAI(CUnit* owner):
 	c.tooltip = "Stop: Cancel the units current actions";
 	possibleCommands.push_back(c);
 
-	if (isAttackCapable()) {
+	if (IsAttackCapable()) {
 		c.id = CMD_ATTACK;
 		c.action = "attack";
 		c.type = CMDTYPE_ICON_UNIT_OR_MAP;
 		c.name = "Attack";
 		c.mouseicon = c.name;
-		c.tooltip = "Attack: Attacks an unit or a position on the ground";
+		c.tooltip = "Attack: Attacks a unit or a position on the ground";
 		possibleCommands.push_back(c);
 	}
 
-	if (owner->unitDef->canDGun) {
-		c.id = CMD_DGUN;
-		c.action = "dgun";
+	if (owner->unitDef->canManualFire) {
+		c.id = CMD_MANUALFIRE;
+		c.action = "manualfire";
 		c.type = CMDTYPE_ICON_MAP;
-		c.name = "DGun";
+		c.name = "ManualFire";
 		c.mouseicon = c.name;
-		c.tooltip = "DGun: Attacks using the units special weapon";
+		c.tooltip = "ManualFire: Attacks with manually-fired weapon";
 		possibleCommands.push_back(c);
 	}
 
@@ -210,7 +204,7 @@ CCommandAI::CCommandAI(CUnit* owner):
 		c.type = CMDTYPE_ICON_MODE;
 		c.name = "Fire state";
 		c.mouseicon = c.name;
-		c.params.push_back("2");
+		c.params.push_back(IntToString(FIRESTATE_FIREATWILL));
 		c.params.push_back("Hold fire");
 		c.params.push_back("Return fire");
 		c.params.push_back("Fire at will");
@@ -226,7 +220,7 @@ CCommandAI::CCommandAI(CUnit* owner):
 		c.type = CMDTYPE_ICON_MODE;
 		c.name = "Move state";
 		c.mouseicon = c.name;
-		c.params.push_back("1");
+		c.params.push_back(IntToString(MOVESTATE_MANEUVER));
 		c.params.push_back("Hold pos");
 		c.params.push_back("Maneuver");
 		c.params.push_back("Roam");
@@ -234,7 +228,7 @@ CCommandAI::CCommandAI(CUnit* owner):
 		possibleCommands.push_back(c);
 		nonQueingCommands.insert(CMD_MOVE_STATE);
 	} else {
-		owner->moveState = 0;
+		owner->moveState = MOVESTATE_HOLDPOS;
 	}
 
 	if (owner->unitDef->canRepeat) {
@@ -314,9 +308,24 @@ CCommandAI::CCommandAI(CUnit* owner):
 
 CCommandAI::~CCommandAI()
 {
-	if (orderTarget) {
-		DeleteDeathDependence(orderTarget);
-		orderTarget = 0;
+	SetOrderTarget(NULL);
+	ClearCommandDependencies();
+}
+
+void CCommandAI::ClearCommandDependencies() {
+	while (!commandDeathDependences.empty()) {
+		DeleteDeathDependence(*commandDeathDependences.begin(), DEPENDENCE_COMMANDQUE);
+	}
+}
+
+void CCommandAI::AddCommandDependency(const Command &c) {
+	int cpos;
+	if (c.IsObjectCommand(cpos)) {
+		int refid = c.params[cpos];
+		CObject* ref = (refid < uh->MaxUnits()) ? (CObject*)uh->GetUnit(refid) :
+						(CObject*)featureHandler->GetFeature(refid - uh->MaxUnits());
+		if (ref)
+			AddDeathDependence(ref, DEPENDENCE_COMMANDQUE);
 	}
 }
 
@@ -331,11 +340,12 @@ vector<CommandDescription>& CCommandAI::GetPossibleCommands()
 }
 
 
-bool CCommandAI::isAttackCapable() const
+bool CCommandAI::IsAttackCapable() const
 {
 	const UnitDef* ud = owner->unitDef;
-	return (ud->canAttack &&
-	        (!ud->weapons.empty() || ud->canKamikaze || (ud->type == "Factory")));
+	const bool b = (!ud->weapons.empty() || ud->canKamikaze || (ud->IsFactoryUnit()));
+
+	return (ud->canAttack && b);
 }
 
 
@@ -346,7 +356,7 @@ static inline const CUnit* GetCommandUnit(const Command& c, int idx) {
 	}
 
 	if (c.IsAreaCommand()) {
-		return false;
+		return NULL;
 	}
 
 	const CUnit* unit = uh->GetUnit(c.params[idx]);
@@ -369,8 +379,8 @@ static inline bool IsCommandInMap(const Command& c)
 
 bool CCommandAI::AllowedCommand(const Command& c, bool fromSynced)
 {
-	// check if the command is in the map first
-	switch (c.id) {
+	// TODO check if the command is in the map first, for more commands
+	switch (c.GetID()) {
 		case CMD_MOVE:
 		case CMD_ATTACK:
 		case CMD_AREA_ATTACK:
@@ -380,7 +390,7 @@ bool CCommandAI::AllowedCommand(const Command& c, bool fromSynced)
 		case CMD_PATROL:
 		case CMD_RESTORE:
 		case CMD_FIGHT:
-		case CMD_DGUN:
+		case CMD_MANUALFIRE:
 		case CMD_UNLOAD_UNIT:
 		case CMD_UNLOAD_UNITS: {
 			if (!IsCommandInMap(c)) { return false; }
@@ -388,7 +398,7 @@ bool CCommandAI::AllowedCommand(const Command& c, bool fromSynced)
 
 		default: {
 			// build commands
-			if (c.id < 0 && !IsCommandInMap(c)) { return false; }
+			if (c.GetID() < 0 && !IsCommandInMap(c)) { return false; }
 		} break;
 	}
 
@@ -397,14 +407,15 @@ bool CCommandAI::AllowedCommand(const Command& c, bool fromSynced)
 	// AI's may do as they like
 	const CSkirmishAIHandler::ids_t& saids = skirmishAIHandler.GetSkirmishAIsInTeam(owner->team);
 	const bool aiOrder = (!saids.empty());
-
-	switch (c.id) {
-		case CMD_DGUN:
-			if (!ud->canDGun)
+	const int& cmd_id = c.GetID();
+	
+	switch (cmd_id) {
+		case CMD_MANUALFIRE:
+			if (!ud->canManualFire)
 				return false;
 
 		case CMD_ATTACK: {
-			if (!isAttackCapable())
+			if (!IsAttackCapable())
 				return false;
 
 			if (c.params.size() == 1) {
@@ -419,7 +430,7 @@ bool CCommandAI::AllowedCommand(const Command& c, bool fromSynced)
 
 					// check if attack ground is really attack ground
 					if (!aiOrder && !fromSynced &&
-						fabs(cPos.y - ground->GetHeight2(cPos.x, cPos.z)) > SQUARE_SIZE) {
+						fabs(cPos.y - ground->GetHeightReal(cPos.x, cPos.z)) > SQUARE_SIZE) {
 						return false;
 					}
 				}
@@ -452,15 +463,16 @@ bool CCommandAI::AllowedCommand(const Command& c, bool fromSynced)
 			const CUnit* reclaimeeUnit = GetCommandUnit(c, 0);
 			const CFeature* reclaimeeFeature = NULL;
 
+			if (c.IsAreaCommand()) { return true; }
 			if (!ud->canReclaim) { return false; }
 			if (reclaimeeUnit && !reclaimeeUnit->unitDef->reclaimable) { return false; }
 			if (reclaimeeUnit && !reclaimeeUnit->AllowedReclaim(owner)) { return false; }
 			if (reclaimeeUnit && !reclaimeeUnit->pos.IsInBounds()) { return false; }
 
-			if (reclaimeeUnit == NULL && !c.IsAreaCommand()) {
+			if (reclaimeeUnit == NULL && !c.params.empty()) {
 				const unsigned int reclaimeeFeatureID(c.params[0]);
 
-				if (!c.params.empty() && reclaimeeFeatureID >= uh->MaxUnits()) {
+				if (reclaimeeFeatureID >= uh->MaxUnits()) {
 					reclaimeeFeature = featureHandler->GetFeature(reclaimeeFeatureID - uh->MaxUnits());
 
 					if (reclaimeeFeature && !reclaimeeFeature->def->reclaimable) {
@@ -490,35 +502,35 @@ bool CCommandAI::AllowedCommand(const Command& c, bool fromSynced)
 	}
 
 
-	if (c.id == CMD_FIRE_STATE
+	if (cmd_id == CMD_FIRE_STATE
 			&& (c.params.empty() || !CanChangeFireState()))
 	{
 		return false;
 	}
-	if (c.id == CMD_MOVE_STATE
+	if (cmd_id == CMD_MOVE_STATE
 			&& (c.params.empty() || (!ud->canmove && !ud->builder)))
 	{
 		return false;
 	}
-	if (c.id == CMD_REPEAT && (c.params.empty() || !ud->canRepeat || ((int)c.params[0] % 2) != (int)c.params[0]/* only 0 or 1 allowed */))
+	if (cmd_id == CMD_REPEAT && (c.params.empty() || !ud->canRepeat || ((int)c.params[0] % 2) != (int)c.params[0]/* only 0 or 1 allowed */))
 	{
 		return false;
 	}
-	if (c.id == CMD_TRAJECTORY
+	if (cmd_id == CMD_TRAJECTORY
 			&& (c.params.empty() || ud->highTrajectoryType < 2))
 	{
 		return false;
 	}
-	if (c.id == CMD_ONOFF
+	if (cmd_id == CMD_ONOFF
 			&& (c.params.empty() || !ud->onoffable || owner->beingBuilt || ((int)c.params[0] % 2) != (int)c.params[0]/* only 0 or 1 allowed */))
 	{
 		return false;
 	}
-	if (c.id == CMD_CLOAK && (c.params.empty() || !ud->canCloak || ((int)c.params[0] % 2) != (int)c.params[0]/* only 0 or 1 allowed */))
+	if (cmd_id == CMD_CLOAK && (c.params.empty() || !ud->canCloak || ((int)c.params[0] % 2) != (int)c.params[0]/* only 0 or 1 allowed */))
 	{
 		return false;
 	}
-	if (c.id == CMD_STOCKPILE && !stockpileWeapon)
+	if (cmd_id == CMD_STOCKPILE && !stockpileWeapon)
 	{
 		return false;
 	}
@@ -550,7 +562,7 @@ inline void CCommandAI::SetCommandDescParam0(const Command& c)
 {
 	vector<CommandDescription>::iterator cdi;
 	for (cdi = possibleCommands.begin(); cdi != possibleCommands.end(); ++cdi) {
-		if (cdi->id == c.id) {
+		if (cdi->id == c.GetID()) {
 			char t[10];
 			SNPRINTF(t, 10, "%d", (int)c.params[0]);
 			cdi->params[0] = t;
@@ -562,7 +574,7 @@ inline void CCommandAI::SetCommandDescParam0(const Command& c)
 
 bool CCommandAI::ExecuteStateCommand(const Command& c)
 {
-	switch (c.id) {
+	switch (c.GetID()) {
 		case CMD_FIRE_STATE: {
 			owner->fireState = (int)c.params[0];
 			SetCommandDescParam0(c);
@@ -647,7 +659,7 @@ void CCommandAI::GiveAllowedCommand(const Command& c, bool fromSynced)
 		return;
 	}
 
-	switch (c.id) {
+	switch (c.GetID()) {
 		case CMD_SELFD: {
 			if (owner->unitDef->canSelfD) {
 				if (!(c.options & SHIFT_KEY) || commandQue.empty()) {
@@ -657,12 +669,10 @@ void CCommandAI::GiveAllowedCommand(const Command& c, bool fromSynced)
 						owner->selfDCountdown = owner->unitDef->selfDCountdown*2+1;
 					}
 				}
-				else {
-				if (commandQue.back().id == CMD_SELFD) {
+				else if (commandQue.back().GetID() == CMD_SELFD) {
 					commandQue.pop_back();
-					} else {
-						commandQue.push_back(c);
-					}
+				} else {
+					commandQue.push_back(c);
 				}
 			}
 			return;
@@ -670,7 +680,7 @@ void CCommandAI::GiveAllowedCommand(const Command& c, bool fromSynced)
 		case CMD_SET_WANTED_MAX_SPEED: {
 			if (CanSetMaxSpeed() &&
 			    (commandQue.empty() ||
-			     (commandQue.back().id != CMD_SET_WANTED_MAX_SPEED))) {
+			     (commandQue.back().GetID() != CMD_SET_WANTED_MAX_SPEED))) {
 				// bail early, do not check for overlaps or queue cancelling
 				commandQue.push_back(c);
 				if (commandQue.size()==1 && !owner->beingBuilt) {
@@ -696,53 +706,48 @@ void CCommandAI::GiveAllowedCommand(const Command& c, bool fromSynced)
 	// flush the queue for immediate commands
 	if (!(c.options & SHIFT_KEY)) {
 		if (!commandQue.empty()) {
-			if ((commandQue.front().id == CMD_DGUN)      ||
-			    (commandQue.front().id == CMD_ATTACK)    ||
-			    (commandQue.front().id == CMD_AREA_ATTACK)) {
+			const int& cmd_id = commandQue.front().GetID();
+			if ((cmd_id == CMD_MANUALFIRE) ||
+			    (cmd_id == CMD_ATTACK)     ||
+			    (cmd_id == CMD_AREA_ATTACK)) {
 				owner->AttackUnit(0,true);
 			}
 			waitCommandsAI.ClearUnitQueue(owner, commandQue);
+			ClearCommandDependencies();
 			commandQue.clear();
 		}
 		inCommand=false;
-		if(orderTarget){
-			DeleteDeathDependence(orderTarget);
-			orderTarget=0;
-		}
+		SetOrderTarget(NULL);
 	}
 
-	if (c.id == CMD_PATROL) {
+	AddCommandDependency(c);
+
+	if (c.GetID() == CMD_PATROL) {
 		CCommandQueue::iterator ci = commandQue.begin();
-		for (; ci != commandQue.end() && ci->id != CMD_PATROL; ci++) {
+		for (; ci != commandQue.end() && ci->GetID() != CMD_PATROL; ++ci) {
 			// just increment
 		}
 		if (ci == commandQue.end()) {
 			if (commandQue.empty()) {
-				Command c2;
-				c2.id = CMD_PATROL;
+				Command c2(CMD_PATROL, c.options);
 				c2.params.push_back(owner->pos.x);
 				c2.params.push_back(owner->pos.y);
 				c2.params.push_back(owner->pos.z);
-				c2.options = c.options;
 				commandQue.push_back(c2);
 			}
 			else {
 				do {
-					ci--;
+					--ci;
 					if (ci->params.size() >= 3) {
-						Command c2;
-						c2.id = CMD_PATROL;
+						Command c2(CMD_PATROL, c.options);
 						c2.params = ci->params;
-						c2.options = c.options;
 						commandQue.push_back(c2);
 						break;
 					} else if (ci == commandQue.begin()) {
-						Command c2;
-						c2.id = CMD_PATROL;
+						Command c2(CMD_PATROL, c.options);
 						c2.params.push_back(owner->pos.x);
 						c2.params.push_back(owner->pos.y);
 						c2.params.push_back(owner->pos.z);
-						c2.options = c.options;
 						commandQue.push_back(c2);
 						break;
 					}
@@ -756,8 +761,7 @@ void CCommandAI::GiveAllowedCommand(const Command& c, bool fromSynced)
 	bool first;
 	if (CancelCommands(c, commandQue, first) > 0) {
 		if (first) {
-			Command stopCommand;
-			stopCommand.id = CMD_STOP;
+			Command stopCommand(CMD_STOP);
 			commandQue.push_front(stopCommand);
 			SlowUpdate();
 		}
@@ -769,11 +773,10 @@ void CCommandAI::GiveAllowedCommand(const Command& c, bool fromSynced)
 		return;
 	}
 
-	if (c.id == CMD_ATTACK) {
+	if (c.GetID() == CMD_ATTACK) {
 		// avoid weaponless units moving to 0 distance when given attack order
 		if (owner->weapons.empty() && (owner->unitDef->canKamikaze == false)) {
-			Command c2;
-			c2.id = CMD_STOP;
+			Command c2(CMD_STOP);
 			commandQue.push_back(c2);
 			return;
 		}
@@ -794,7 +797,7 @@ void CCommandAI::GiveWaitCommand(const Command& c)
 		return;
 	}
 	else if (c.options & SHIFT_KEY) {
-		if (commandQue.back().id == CMD_WAIT) {
+		if (commandQue.back().GetID() == CMD_WAIT) {
 			waitCommandsAI.RemoveWaitCommand(owner, commandQue.back());
 			commandQue.pop_back();
 		} else {
@@ -802,7 +805,7 @@ void CCommandAI::GiveWaitCommand(const Command& c)
 			return;
 		}
 	}
-	else if (commandQue.front().id == CMD_WAIT) {
+	else if (commandQue.front().GetID()== CMD_WAIT) {
 		waitCommandsAI.RemoveWaitCommand(owner, commandQue.front());
 		commandQue.pop_front();
 	}
@@ -838,9 +841,7 @@ void CCommandAI::ExecuteInsert(const Command& c, bool fromSynced)
 	}
 
 	// make the command
-	Command newCmd;
-	newCmd.id                  = (int)c.params[1];
-	newCmd.options   = (unsigned char)c.params[2];
+	Command newCmd((int)c.params[1], (unsigned char)c.params[2]);
 	for (int p = 3; p < (int)c.params.size(); p++) {
 		newCmd.params.push_back(c.params[p]);
 	}
@@ -858,8 +859,8 @@ void CCommandAI::ExecuteInsert(const Command& c, bool fromSynced)
 		if (c.options & CONTROL_KEY) {
 			// check the build order
 			const map<int, CFactoryCAI::BuildOption>& bOpts = facCAI->buildOptions;
-			if ((newCmd.id != CMD_STOP) && (newCmd.id != CMD_WAIT) &&
-			    ((newCmd.id >= 0) || (bOpts.find(newCmd.id) == bOpts.end()))) {
+			if ((newCmd.GetID() != CMD_STOP) && (newCmd.GetID() != CMD_WAIT) &&
+			    ((newCmd.GetID() >= 0) || (bOpts.find(newCmd.GetID()) == bOpts.end()))) {
 				return;
 			}
 			facBuildQueue = true;
@@ -905,7 +906,7 @@ void CCommandAI::ExecuteInsert(const Command& c, bool fromSynced)
 			return;
 		}
 		if ((c.options & RIGHT_MOUSE_KEY) && (insertIt != queue->end())) {
-			insertIt++; // insert after the tagged command
+			++insertIt; // insert after the tagged command
 		}
 	}
 
@@ -922,10 +923,10 @@ void CCommandAI::ExecuteInsert(const Command& c, bool fromSynced)
 		inCommand = false;
 		targetDied = false;
 		unimportantMove = false;
-		orderTarget = NULL;
+		SetOrderTarget(NULL);
 		const Command& cmd = queue->front();
 		eoh->CommandFinished(*owner, cmd);
-		eventHandler.UnitCmdDone(owner, cmd.id, cmd.tag);
+		eventHandler.UnitCmdDone(owner, cmd.GetID(), cmd.tag);
 	}
 
 	queue->insert(insertIt, newCmd);
@@ -933,28 +934,29 @@ void CCommandAI::ExecuteInsert(const Command& c, bool fromSynced)
 	if (!owner->stunned) {
 		SlowUpdate();
 	}
-
-	return;
 }
 
 
 void CCommandAI::ExecuteRemove(const Command& c)
 {
+	CCommandQueue* queue = &commandQue;
+	CFactoryCAI* facCAI = dynamic_cast<CFactoryCAI*>(this);
+
+	// if false, remove commands by tag
+	const bool removeByID = (c.options & ALT_KEY);
 	// disable repeating during the removals
 	const bool prevRepeat = repeatOrders;
-	repeatOrders = false;
-
-	CCommandQueue* queue = &commandQue;
 
+	// erase commands by a list of command types
+	bool active = false;
 	bool facBuildQueue = false;
-	CFactoryCAI* facCAI = dynamic_cast<CFactoryCAI*>(this);
 
 	if (facCAI) {
 		if (c.options & CONTROL_KEY) {
-			// check the build order
+			// keep using the build-order queue
 			facBuildQueue = true;
 		} else {
-			// use the new commands
+			// use the command-queue for new units
 			queue = &facCAI->newUnitCommands;
 		}
 	}
@@ -963,33 +965,39 @@ void CCommandAI::ExecuteRemove(const Command& c)
 		return;
 	}
 
-	// erase commands by a list of command types
-	bool active = false;
-	for (unsigned int p = 0; p < c.params.size(); p++) {
-		const int removeValue = (int)c.params[p]; // tag or id
+	repeatOrders = false;
 
-		if (facBuildQueue && (removeValue == 0) && !(c.options & ALT_KEY)) {
-			continue; // don't remove tag=0 commands from build queues, they
-			          // are used the same way that CMD_STOP is, to void orders
+	for (unsigned int p = 0; p < c.params.size(); p++) {
+		const int removeValue = c.params[p]; // tag or id
+
+		if (facBuildQueue && !removeByID && (removeValue == 0)) {
+			// don't remove commands with tag 0 from build queues, they
+			// are used the same way that CMD_STOP is (to void orders)
+			continue;
 		}
+
 		CCommandQueue::iterator ci;
+
 		do {
 			for (ci = queue->begin(); ci != queue->end(); ++ci) {
 				const Command& qc = *ci;
-				if (c.options & ALT_KEY) {
-					if (qc.id != removeValue)  { continue; }
+
+				if (removeByID) {
+					if (qc.GetID() != removeValue)  { continue; }
 				} else {
 					if (qc.tag != removeValue) { continue; }
 				}
 
-				if (qc.id == CMD_WAIT) {
+				if (qc.GetID() == CMD_WAIT) {
 					waitCommandsAI.RemoveWaitCommand(owner, qc);
 				}
 
 				if (facBuildQueue) {
-					facCAI->RemoveBuildCommand(ci);
-					ci = queue->begin();
-					break; // the removal may have corrupted the iterator
+					// if ci == queue->begin() and !queue->empty(), this pop_front()'s
+					// via CFAI::ExecuteStop; otherwise only modifies *ci (not <queue>)
+					if (facCAI->RemoveBuildCommand(ci)) {
+						ci = queue->begin(); break;
+					}
 				}
 
 				if (!facCAI && (ci == queue->begin())) {
@@ -1001,67 +1009,61 @@ void CCommandAI::ExecuteRemove(const Command& c)
 					}
 					active = true;
 				}
+
 				queue->erase(ci);
 				ci = queue->begin();
-				break; // the removal may have corrupted the iterator
+
+				// the removal may have corrupted the iterator
+				break;
 			}
 		}
 		while (ci != queue->end());
 	}
 
 	repeatOrders = prevRepeat;
-	return;
 }
 
 
-/**
-* @brief Determins if c will cancel a queued command
-* @return true if c will cancel a queued command
-**/
-bool CCommandAI::WillCancelQueued(Command &c)
+bool CCommandAI::WillCancelQueued(const Command& c)
 {
-	return (this->GetCancelQueued(c, commandQue) != this->commandQue.end());
+	return (GetCancelQueued(c, commandQue) != commandQue.end());
 }
 
 
-/**
-* @brief Finds the queued command that would be canceled by the Command c
-* @return An iterator located at the command, or commandQue.end() if no such queued command exsists
-**/
-CCommandQueue::iterator CCommandAI::GetCancelQueued(const Command &c,
-                                                          CCommandQueue& q)
+CCommandQueue::iterator CCommandAI::GetCancelQueued(const Command& c, CCommandQueue& q)
 {
 	CCommandQueue::iterator ci = q.end();
 
 	while (ci != q.begin()) {
+		--ci; //iterate from the end and dont check the current order
+		const Command& c2 = *ci;
+		const int& cmd_id = c.GetID();
+		const int& cmd2_id = c2.GetID();
 
-		ci--; //iterate from the end and dont check the current order
-		const Command& t = *ci;
-
-		if (((c.id == t.id) || ((c.id < 0) && (t.id < 0))
-				|| (t.id == CMD_FIGHT && c.id == CMD_ATTACK && t.params.size() == 1))
-				&& (t.params.size() == c.params.size())) {
+		if (((cmd_id == cmd2_id) || ((cmd_id < 0) && (cmd2_id < 0))
+				|| (cmd2_id == CMD_FIGHT && cmd_id == CMD_ATTACK && c2.params.size() == 1))
+				&& (c2.params.size() == c.params.size())) {
 			if (c.params.size() == 1) {
-			  // assume the param is a unit of feature id
-				if ((t.params[0] == c.params[0]) &&
-				    (t.id != CMD_SET_WANTED_MAX_SPEED)) {
+				// assume the param is a unit-ID or feature-ID
+				if ((c2.params[0] == c.params[0]) &&
+				    (cmd2_id != CMD_SET_WANTED_MAX_SPEED)) {
 					return ci;
 				}
 			}
 			else if (c.params.size() >= 3) {
-				if (c.id < 0) {
+				if (cmd_id < 0) {
 					BuildInfo bc(c);
-					BuildInfo bt(t);
-					if (bc.def && bt.def
-					    && fabs(bc.pos.x - bt.pos.x) * 2 <= std::max(bc.GetXSize(), bt.GetXSize()) * SQUARE_SIZE
-					    && fabs(bc.pos.z - bt.pos.z) * 2 <= std::max(bc.GetZSize(), bt.GetZSize()) * SQUARE_SIZE) {
+					BuildInfo bc2(c2);
+					if (bc.def && bc2.def
+					    && fabs(bc.pos.x - bc2.pos.x) * 2 <= std::max(bc.GetXSize(), bc2.GetXSize()) * SQUARE_SIZE
+					    && fabs(bc.pos.z - bc2.pos.z) * 2 <= std::max(bc.GetZSize(), bc2.GetZSize()) * SQUARE_SIZE) {
 						return ci;
 					}
 				} else {
 					// assume this means that the first 3 makes a position
 					float3 cp(c.params[0], c.params[1], c.params[2]);
-					float3 tp(t.params[0], t.params[1], t.params[2]);
-					if ((cp - tp).SqLength2D() < (17.0f * 17.0f)) {
+					float3 c2p(c2.params[0], c2.params[1], c2.params[2]);
+					if ((cp - c2p).SqLength2D() < (17.0f * 17.0f)) {
 						return ci;
 					}
 				}
@@ -1072,8 +1074,7 @@ CCommandQueue::iterator CCommandAI::GetCancelQueued(const Command &c,
 }
 
 
-int CCommandAI::CancelCommands(const Command &c, CCommandQueue& q,
-                               bool& first)
+int CCommandAI::CancelCommands(const Command &c, CCommandQueue& q, bool& first)
 {
 	first = false;
 	int cancelCount = 0;
@@ -1089,24 +1090,24 @@ int CCommandAI::CancelCommands(const Command &c, CCommandQueue& q,
 		CCommandQueue::iterator firstErase = ci;
 		CCommandQueue::iterator lastErase = ci;
 
-		ci++;
-		if ((ci != q.end()) && (ci->id == CMD_SET_WANTED_MAX_SPEED)) {
+		++ci;
+		if ((ci != q.end()) && (ci->GetID() == CMD_SET_WANTED_MAX_SPEED)) {
 			lastErase = ci;
 			cancelCount++;
-			ci++;
+			++ci;
 		}
 
-		if ((ci != q.end()) && (ci->id == CMD_WAIT)) {
-  		waitCommandsAI.RemoveWaitCommand(owner, *ci);
+		if ((ci != q.end()) && (ci->GetID() == CMD_WAIT)) {
+			waitCommandsAI.RemoveWaitCommand(owner, *ci);
 			lastErase = ci;
 			cancelCount++;
-			ci++;
+			++ci;
 		}
 
-		lastErase++; // STL: erase the range [first, last)
+		++lastErase; // STL: erase the range [first, last)
 		q.erase(firstErase, lastErase);
 
-		if (c.id >= 0) {
+		if (c.GetID() >= 0) {
 			return cancelCount; // only delete one non-build order
 		}
 	}
@@ -1115,18 +1116,13 @@ int CCommandAI::CancelCommands(const Command &c, CCommandQueue& q,
 }
 
 
-/**
-* @brief Returns commands that overlap c, but will not be canceled by c
-* @return a vector containing commands that overlap c
-*/
-std::vector<Command> CCommandAI::GetOverlapQueued(const Command &c)
+std::vector<Command> CCommandAI::GetOverlapQueued(const Command& c)
 {
 	return GetOverlapQueued(c, commandQue);
 }
 
 
-std::vector<Command> CCommandAI::GetOverlapQueued(const Command &c,
-                                                  CCommandQueue& q)
+std::vector<Command> CCommandAI::GetOverlapQueued(const Command& c, CCommandQueue& q)
 {
 	CCommandQueue::iterator ci = q.end();
 	std::vector<Command> v;
@@ -1134,10 +1130,10 @@ std::vector<Command> CCommandAI::GetOverlapQueued(const Command &c,
 
 	if (ci != q.begin()){
 		do {
-			ci--; //iterate from the end and dont check the current order
+			--ci; //iterate from the end and dont check the current order
 			const Command& t = *ci;
 
-			if (((t.id == c.id) || ((c.id < 0) && (t.id < 0))) &&
+			if (((t.GetID() == c.GetID()) || ((c.GetID() < 0) && (t.GetID() < 0))) &&
 			    (t.params.size() == c.params.size())){
 				if (c.params.size()==1) {
 					// assume the param is a unit or feature id
@@ -1186,10 +1182,8 @@ int CCommandAI::UpdateTargetLostTimer(int unitID)
 	return targetLostTimer;
 }
 
-/**
-* @brief Causes this CommandAI to execute the attack order c
-*/
-void CCommandAI::ExecuteAttack(Command &c)
+
+void CCommandAI::ExecuteAttack(Command& c)
 {
 	assert(owner->unitDef->canAttack);
 
@@ -1214,13 +1208,9 @@ void CCommandAI::ExecuteAttack(Command &c)
 			CUnit* targetUnit = uh->GetUnit(c.params[0]);
 
 			if (targetUnit != NULL && targetUnit != owner) {
-				owner->AttackUnit(targetUnit, c.id == CMD_DGUN);
+				owner->AttackUnit(targetUnit, c.GetID() == CMD_MANUALFIRE);
 
-				if (orderTarget)
-					DeleteDeathDependence(orderTarget);
-
-				orderTarget = targetUnit;
-				AddDeathDependence(orderTarget);
+				SetOrderTarget(targetUnit);
 				inCommand = true;
 			} else {
 				FinishCommand();
@@ -1228,15 +1218,13 @@ void CCommandAI::ExecuteAttack(Command &c)
 			}
 		} else {
 			float3 pos(c.params[0], c.params[1], c.params[2]);
-			owner->AttackGround(pos, c.id == CMD_DGUN);
+			owner->AttackGround(pos, c.GetID() == CMD_MANUALFIRE);
 			inCommand = true;
 		}
 	}
 }
 
-/**
-* @brief executes the stop command c
-*/
+
 void CCommandAI::ExecuteStop(Command &c)
 {
 	owner->AttackUnit(0,true);
@@ -1247,14 +1235,6 @@ void CCommandAI::ExecuteStop(Command &c)
 	FinishCommand();
 }
 
-/**
-* @brief executes the DGun command c
-*/
-void CCommandAI::ExecuteDGun(Command &c)
-{
-	ExecuteAttack(c);
-	return;
-}
 
 void CCommandAI::SlowUpdate()
 {
@@ -1266,7 +1246,7 @@ void CCommandAI::SlowUpdate()
 
 	Command& c = commandQue.front();
 
-	switch (c.id) {
+	switch (c.GetID()) {
 		case CMD_WAIT: {
 			return;
 		}
@@ -1287,8 +1267,8 @@ void CCommandAI::SlowUpdate()
 			ExecuteAttack(c);
 			return;
 		}
-		case CMD_DGUN: {
-			ExecuteDGun(c);
+		case CMD_MANUALFIRE: {
+			ExecuteAttack(c);
 			return;
 		}
 	}
@@ -1310,7 +1290,7 @@ int CCommandAI::GetDefaultCmd(const CUnit* pointed, const CFeature* feature)
 {
 	if (pointed) {
 		if (!teamHandler->Ally(gu->myAllyTeam, pointed->allyteam)) {
-			if (isAttackCapable()) {
+			if (IsAttackCapable()) {
 				return CMD_ATTACK;
 			}
 		}
@@ -1318,119 +1298,64 @@ int CCommandAI::GetDefaultCmd(const CUnit* pointed, const CFeature* feature)
 	return CMD_STOP;
 }
 
-void CCommandAI::DependentDied(CObject *o)
-{
-	if(o==orderTarget){
-		targetDied=true;
-		orderTarget=0;
+
+void CCommandAI::AddDeathDependence(CObject* o, DependenceType dep) {
+	if (dep == DEPENDENCE_COMMANDQUE) {
+		if (commandDeathDependences.insert(o).second) // prevent multiple dependencies for the same object
+			CObject::AddDeathDependence(o, dep);
+		return;
 	}
+	CObject::AddDeathDependence(o, dep);
 }
 
 
-bool CCommandAI::isTrackable(const CUnit* unit) const
-{
-	return ((unit->losStatus[owner->allyteam] & (LOS_INLOS | LOS_INRADAR)) != 0) ||
-	       (unit->unitDef->speed <= 0.0f);
+void CCommandAI::DeleteDeathDependence(CObject* o, DependenceType dep) {
+	if (dep == DEPENDENCE_COMMANDQUE) {
+		if (commandDeathDependences.erase(o))
+			CObject::DeleteDeathDependence(o, dep);
+		return;
+	}
+	CObject::DeleteDeathDependence(o,dep);
 }
 
 
-void CCommandAI::DrawWaitIcon(const Command& cmd) const
+void CCommandAI::DependentDied(CObject* o)
 {
-	waitCommandsAI.AddIcon(cmd, lineDrawer.GetLastPos());
-}
-
-
-void CCommandAI::DrawCommands(void)
-{
-	lineDrawer.StartPath(owner->drawMidPos, cmdColors.start);
-
-	if (owner->selfDCountdown != 0) {
-		lineDrawer.DrawIconAtLastPos(CMD_SELFD);
+	if (o == orderTarget) {
+		targetDied = true;
+		orderTarget = NULL;
 	}
 
-	CCommandQueue::iterator ci;
-	for (ci = commandQue.begin(); ci != commandQue.end(); ++ci) {
-		switch (ci->id) {
-			case CMD_ATTACK:
-			case CMD_DGUN: {
-				if (ci->params.size() == 1) {
-					const CUnit* unit = uh->GetUnit(ci->params[0]);
-
-					if ((unit != NULL) && isTrackable(unit)) {
-						const float3 endPos =
-							helper->GetUnitErrorPos(unit, owner->allyteam);
-						lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.attack);
-					}
-				} else {
-					const float3 endPos(ci->params[0],ci->params[1]+3,ci->params[2]);
-					lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.attack);
+	if (commandDeathDependences.erase(o) && o != owner) {
+		CFactoryCAI* facCAI = dynamic_cast<CFactoryCAI*>(this);
+		CCommandQueue& dq = facCAI ? facCAI->newUnitCommands : commandQue;
+		int lastTag;
+		int curTag = -1;
+		do {
+			lastTag = curTag;
+			for (CCommandQueue::iterator qit = dq.begin(); qit != dq.end(); ++qit) {
+				Command &c = *qit;
+				int cpos;
+				if (c.IsObjectCommand(cpos) && (c.params[cpos] == CSolidObject::GetDeletingRefID())) {
+					Command removeCmd(CMD_REMOVE, 0);
+					curTag = c.tag;
+					removeCmd.params.push_back(curTag);
+					ExecuteRemove(removeCmd);
+					break;
 				}
-				break;
 			}
-			case CMD_WAIT:{
-				DrawWaitIcon(*ci);
-				break;
-			}
-			case CMD_SELFD:{
-				lineDrawer.DrawIconAtLastPos(ci->id);
-				break;
-			}
-			default:{
-				DrawDefaultCommand(*ci);
-				break;
-			}
-		}
-	}
-
-	lineDrawer.FinishPath();
-}
-
-
-void CCommandAI::DrawDefaultCommand(const Command& c) const
-{
-	// TODO add Lua callin perhaps, for more elaborate needs?
-	const CCommandColors::DrawData* dd = cmdColors.GetCustomCmdData(c.id);
-	if (dd == NULL) {
-		return;
-	}
-	const int paramsCount = c.params.size();
-
-	if (paramsCount == 1) {
-		const CUnit* unit = uh->GetUnit(c.params[0]);
-
-		if ((unit != NULL) && isTrackable(unit)) {
-			const float3 endPos = helper->GetUnitErrorPos(unit, owner->allyteam);
-			lineDrawer.DrawLineAndIcon(dd->cmdIconID, endPos, dd->color);
-		}
-
-		return;
-	}
-
-	if (paramsCount < 3) {
-		return;
-	}
-
-	const float3 endPos(c.params[0], c.params[1] + 3.0f, c.params[2]);
-
-	if (!dd->showArea || (paramsCount < 4)) {
-		lineDrawer.DrawLineAndIcon(dd->cmdIconID, endPos, dd->color);
-	}
-	else {
-		const float radius = c.params[3];
-		lineDrawer.DrawLineAndIcon(dd->cmdIconID, endPos, dd->color);
-		lineDrawer.Break(endPos, dd->color);
-		glSurfaceCircle(endPos, radius, 20);
-		lineDrawer.RestartWithColor(dd->color);
+		} while(curTag != lastTag);
 	}
 }
 
 
-void CCommandAI::FinishCommand(void)
+
+void CCommandAI::FinishCommand()
 {
 	assert(!commandQue.empty());
 	const Command cmd = commandQue.front();
-	const int cmdID  = cmd.id;  // save
-	const int cmdTag = cmd.tag; // save
+	const int cmdID  = cmd.GetID();
+	const int cmdTag = cmd.tag;
 	const bool dontrepeat = (cmd.options & DONT_REPEAT) ||
 	                        (cmd.options & INTERNAL_ORDER);
 	if (repeatOrders
@@ -1445,7 +1370,7 @@ void CCommandAI::FinishCommand(void)
 	inCommand = false;
 	targetDied = false;
 	unimportantMove = false;
-	orderTarget = 0;
+	SetOrderTarget(NULL);
 	eoh->CommandFinished(*owner, cmd);
 	eventHandler.UnitCmdDone(owner, cmdID, cmdTag);
 
@@ -1482,7 +1407,7 @@ void CCommandAI::StockpileChanged(CWeapon* weapon)
 	UpdateStockpileIcon();
 }
 
-void CCommandAI::UpdateStockpileIcon(void)
+void CCommandAI::UpdateStockpileIcon()
 {
 	vector<CommandDescription>::iterator pci;
 	for(pci=possibleCommands.begin();pci!=possibleCommands.end();++pci){
@@ -1499,37 +1424,40 @@ void CCommandAI::UpdateStockpileIcon(void)
 
 void CCommandAI::WeaponFired(CWeapon* weapon)
 {
-	if(weapon->weaponDef->manualfire && !weapon->weaponDef->dropped && !commandQue.empty()
-		&& (commandQue.front().id==CMD_ATTACK || commandQue.front().id==CMD_DGUN) && inCommand)
-	{
-		owner->AttackUnit(0,true);
+	if (!inCommand) { return; }
+	if (commandQue.empty()) { return; }
+	if (!weapon->weaponDef->manualfire) { return; }
+
+	if (commandQue.front().GetID() == CMD_ATTACK || commandQue.front().GetID() == CMD_MANUALFIRE) {
+		owner->AttackUnit(0, true);
 		eoh->WeaponFired(*owner, *(weapon->weaponDef));
 		FinishCommand();
 	}
 }
 
-void CCommandAI::BuggerOff(float3 pos, float radius)
+void CCommandAI::BuggerOff(const float3& pos, float radius)
 {
 }
 
 void CCommandAI::LoadSave(CLoadSaveInterface* file, bool loading)
 {
-	int queSize=commandQue.size();
+	int queSize = commandQue.size();
 	file->lsInt(queSize);
-	for(int a=0;a<queSize;++a){
-		if(loading){
+	for (int a = 0; a < queSize; ++a) {
+		if (loading) {
 			commandQue.push_back(Command());
 		}
-		Command& c=commandQue[a];
-		file->lsInt(c.id);
+		Command& c = commandQue[a];
+		int cmd_id = c.GetID();
+		file->lsInt(cmd_id);
 		file->lsUChar(c.options);
 		file->lsInt(c.timeOut);
 
-		int paramSize=c.params.size();
+		int paramSize = c.params.size();
 		file->lsInt(paramSize);
-		for(int b=0;b<paramSize;++b){
-			if(loading){
-				c.params.push_back(0);
+		for (int b = 0; b < paramSize; ++b) {
+			if (loading) {
+				c.params.push_back(0.0f);
 			}
 			file->lsFloat(c.params[b]);
 		}
@@ -1537,30 +1465,23 @@ void CCommandAI::LoadSave(CLoadSaveInterface* file, bool loading)
 }
 
 
-/**
- * @brief gets the command that keeps the unit close to the path
- * @return a Fight Command with 6 arguments, the first three being where to return to (the current position of the
- *	unit), and the second being the location of the origional command.
- **/
 void CCommandAI::PushOrUpdateReturnFight(const float3& cmdPos1, const float3& cmdPos2)
 {
 	float3 pos = ClosestPointOnLine(cmdPos1, cmdPos2, owner->pos);
 	Command& c(commandQue.front());
-	assert(c.id == CMD_FIGHT && c.params.size() >= 3);
+	assert(c.GetID() == CMD_FIGHT && c.params.size() >= 3);
 	if (c.params.size() >= 6) {
 		c.params[0] = pos.x;
 		c.params[1] = pos.y;
 		c.params[2] = pos.z;
 	} else {
-		Command c2;
-		c2.id = CMD_FIGHT;
+		Command c2(CMD_FIGHT, c.options|INTERNAL_ORDER);
 		c2.params.push_back(pos.x);
 		c2.params.push_back(pos.y);
 		c2.params.push_back(pos.z);
 		c2.params.push_back(c.params[0]);
 		c2.params.push_back(c.params[1]);
 		c2.params.push_back(c.params[2]);
-		c2.options = c.options|INTERNAL_ORDER;
 		commandQue.push_front(c2);
 	}
 }
@@ -1568,23 +1489,40 @@ void CCommandAI::PushOrUpdateReturnFight(const float3& cmdPos1, const float3& cm
 bool CCommandAI::HasMoreMoveCommands()
 {
 	if (!commandQue.empty()) {
-		for (CCommandQueue::iterator i = (commandQue.begin()++); i != commandQue.end(); i++) {
-			const int id = i->id;
+		for (CCommandQueue::iterator i = (commandQue.begin()++); i != commandQue.end(); ++i) {
+			const int& id = i->GetID();
 
 			// build commands are no different from reclaim or repair commands
 			// in that they can require a unit to move, so return true when we
 			// have one
-			if (id == CMD_FIGHT || id == CMD_AREA_ATTACK || id == CMD_ATTACK || id == CMD_CAPTURE
-			 || id == CMD_DGUN || id == CMD_GUARD || id == CMD_LOAD_UNITS || id == CMD_MOVE
-			 || id == CMD_PATROL || id == CMD_RECLAIM || id == CMD_REPAIR || id == CMD_RESTORE
-			 || id == CMD_RESURRECT || id == CMD_UNLOAD_UNIT || id == CMD_UNLOAD_UNITS || id < 0)
-			{
-				return true;
-			}
-			else if (id == CMD_DEATHWAIT || id == CMD_GATHERWAIT || id == CMD_SELFD
-				    || id == CMD_SQUADWAIT || id == CMD_STOP || id == CMD_TIMEWAIT || id == CMD_WAIT)
-			{
-				return false;
+			switch(id) {
+				case CMD_AREA_ATTACK:
+				case CMD_ATTACK:
+				case CMD_CAPTURE:
+				case CMD_FIGHT:
+				case CMD_GUARD:
+				case CMD_LOAD_UNITS:
+				case CMD_MANUALFIRE:
+				case CMD_MOVE:
+				case CMD_PATROL:
+				case CMD_RECLAIM:
+				case CMD_REPAIR:
+				case CMD_RESTORE:
+				case CMD_RESURRECT:
+				case CMD_UNLOAD_UNIT:
+				case CMD_UNLOAD_UNITS:
+					return true;
+				case CMD_DEATHWAIT:
+				case CMD_GATHERWAIT:
+				case CMD_SELFD:
+				case CMD_SQUADWAIT:
+				case CMD_STOP:
+				case CMD_TIMEWAIT:
+				case CMD_WAIT:
+					return false;
+				default:
+					if (id < 0)
+						return true;
 			}
 		}
 	}
@@ -1606,20 +1544,21 @@ bool CCommandAI::SkipParalyzeTarget(const CUnit* target)
 	if ((target->losStatus[owner->allyteam] & LOS_INLOS) && target->stunned) {
 		if ((commandQue.size() > 2) ||
 				((commandQue.size() == 2) &&
-				 (commandQue.back().id != CMD_SET_WANTED_MAX_SPEED))) {
+				 (commandQue.back().GetID() != CMD_SET_WANTED_MAX_SPEED))) {
 			return true;
 		}
 	}
 	return false;
 }
 
-bool CCommandAI::CanChangeFireState(){
-	return owner->unitDef->canFireControl &&
-		(!owner->unitDef->weapons.empty() || owner->unitDef->type=="Factory" ||
-				owner->unitDef->canKamikaze);
+bool CCommandAI::CanChangeFireState() {
+	const UnitDef* ud = owner->unitDef;
+	const bool b = (!ud->weapons.empty() || ud->canKamikaze || ud->IsFactoryUnit());
+
+	return (ud->canFireControl && b);
 }
 
-/// remove attack commands targeted at our new ally
+
 void CCommandAI::StopAttackingAllyTeam(int ally)
 {
 	std::vector<int> todel;
@@ -1628,7 +1567,7 @@ void CCommandAI::StopAttackingAllyTeam(int ally)
 	for (CCommandQueue::iterator it = commandQue.begin(); it != commandQue.end(); ++it) {
 		const Command& c = *it;
 
-		if ((c.id == CMD_FIGHT || c.id == CMD_ATTACK) && c.params.size() == 1) {
+		if ((c.GetID() == CMD_FIGHT || c.GetID() == CMD_ATTACK) && c.params.size() == 1) {
 			const CUnit* target = uh->GetUnit(c.params[0]);
 
 			if (target && target->allyteam == ally) {
