@@ -1,37 +1,33 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "MobileCAI.h"
 #include "TransportCAI.h"
 #include "ExternalAI/EngineOutHandler.h"
-#include "LineDrawer.h"
-#include "Sim/Units/Groups/Group.h"
 #include "Game/GameHelper.h"
+#include "Game/GlobalUnsynced.h"
 #include "Game/SelectedUnits.h"
-#include "Game/UI/CommandColors.h"
 #include "Map/Ground.h"
 #include "Sim/Misc/AirBaseHandler.h"
 #include "Sim/Misc/LosHandler.h"
 #include "Sim/Misc/TeamHandler.h"
 #include "Sim/MoveTypes/MoveType.h"
-#include "Sim/MoveTypes/TAAirMoveType.h"
-#include "Sim/MoveTypes/AirMoveType.h"
+#include "Sim/MoveTypes/HoverAirMoveType.h"
+#include "Sim/MoveTypes/StrafeAirMoveType.h"
 #include "Sim/Units/UnitDef.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitHandler.h"
+#include "Sim/Units/Groups/Group.h"
 #include "Sim/Units/UnitTypes/TransportUnit.h"
 #include "Sim/Weapons/Weapon.h"
 #include "Sim/Weapons/WeaponDefHandler.h"
-#include "Sim/Weapons/DGunWeapon.h"
-#include "LogOutput.h"
-#include "myMath.h"
+#include "System/Log/ILog.h"
+#include "System/myMath.h"
+#include "System/Util.h"
 #include <assert.h>
-#include "Util.h"
-#include "GlobalUnsynced.h"
-
 
+#define BUGGER_OFF_TTL 200
 #define MAX_CLOSE_IN_RETRY_TICKS 30
 
 
@@ -39,6 +35,7 @@ CR_BIND_DERIVED(CMobileCAI ,CCommandAI , );
 
 CR_REG_METADATA(CMobileCAI, (
 				CR_MEMBER(goalPos),
+				CR_MEMBER(lastBuggerGoalPos),
 				CR_MEMBER(lastUserGoal),
 
 				CR_MEMBER(lastIdleCheck),
@@ -66,13 +63,14 @@ CR_REG_METADATA(CMobileCAI, (
 CMobileCAI::CMobileCAI():
 	CCommandAI(),
 	goalPos(-1,-1,-1),
+	lastBuggerGoalPos(-1,0,-1),
 	lastUserGoal(0,0,0),
 	lastIdleCheck(0),
 	tempOrder(false),
 	lastPC(-1),
 //	patrolTime(0),
 	maxWantedSpeed(0),
-	lastBuggerOffTime(-200),
+	lastBuggerOffTime(-BUGGER_OFF_TTL),
 	buggerOffPos(0,0,0),
 	buggerOffRadius(0),
 	commandPos1(ZeroVector),
@@ -87,13 +85,14 @@ CMobileCAI::CMobileCAI():
 CMobileCAI::CMobileCAI(CUnit* owner):
 	CCommandAI(owner),
 	goalPos(-1,-1,-1),
+	lastBuggerGoalPos(-1,0,-1),
 	lastUserGoal(owner->pos),
 	lastIdleCheck(0),
 	tempOrder(false),
 	lastPC(-1),
 //	patrolTime(0),
 	maxWantedSpeed(0),
-	lastBuggerOffTime(-200),
+	lastBuggerOffTime(-BUGGER_OFF_TTL),
 	buggerOffPos(0,0,0),
 	buggerOffRadius(0),
 	commandPos1(ZeroVector),
@@ -219,7 +218,7 @@ void CMobileCAI::GiveCommandReal(const Command &c, bool fromSynced)
 	if (!AllowedCommand(c, fromSynced))
 		return;
 
-	if (owner->unitDef->canfly && c.id == CMD_AUTOREPAIRLEVEL) {
+	if (owner->unitDef->canfly && c.GetID() == CMD_AUTOREPAIRLEVEL) {
 		if (c.params.empty()) {
 			return;
 		}
@@ -248,7 +247,7 @@ void CMobileCAI::GiveCommandReal(const Command &c, bool fromSynced)
 		return;
 	}
 
-	if (owner->unitDef->canfly && c.id == CMD_IDLEMODE) {
+	if (owner->unitDef->canfly && c.GetID() == CMD_IDLEMODE) {
 		if (c.params.empty()) {
 			return;
 		}
@@ -273,7 +272,7 @@ void CMobileCAI::GiveCommandReal(const Command &c, bool fromSynced)
 		return;
 	}
 
-	if (!(c.options & SHIFT_KEY) && nonQueingCommands.find(c.id) == nonQueingCommands.end()) {
+	if (!(c.options & SHIFT_KEY) && nonQueingCommands.find(c.GetID()) == nonQueingCommands.end()) {
 		tempOrder = false;
 		StopSlowGuard();
 	}
@@ -285,80 +284,83 @@ void CMobileCAI::GiveCommandReal(const Command &c, bool fromSynced)
 /// returns true if the unit has to land
 bool CMobileCAI::RefuelIfNeeded()
 {
-	if (!owner->moveType->reservedPad) {
-		// we don't have a pad yet
-		if (owner->currentFuel <= 0) {
-			// we're out of fuel
-			StopMove();
+	if (owner->moveType->reservedPad != NULL) {
+		// we already have a pad
+		return false;
+	}
 
-			owner->userAttackGround = false;
-			owner->userTarget = 0;
-			inCommand = false;
+	if (owner->currentFuel <= 0.0f) {
+		// we're completely out of fuel
+		StopMove();
 
-			CAirBaseHandler::LandingPad* lp =
-				airBaseHandler->FindAirBase(owner, owner->unitDef->minAirBasePower);
+		owner->userAttackGround = false;
+		owner->SetUserTarget(NULL);
+		inCommand = false;
 
-			if (lp) {
-				// found a pad
-				owner->moveType->ReservePad(lp);
+		CAirBaseHandler::LandingPad* lp =
+			airBaseHandler->FindAirBase(owner, owner->unitDef->minAirBasePower);
+
+		if (lp != NULL) {
+			// found a pad
+			owner->moveType->ReservePad(lp);
+		} else {
+			// no pads available, search for a landing
+			// spot near any that are currently occupied
+			const float3& landingPos = airBaseHandler->FindClosestAirBasePos(owner, owner->unitDef->minAirBasePower);
+
+			if (landingPos != ZeroVector) {
+				// NOTE: owner->userAttackGround is wrongly reset to
+				// true in CUnit::AttackGround() via ExecuteAttack()
+				// so don't call it
+				SetGoal(landingPos, owner->pos);
 			} else {
-				// no pads available, search for a landing
-				// spot near any that are currently occupied
-				float3 landingPos =
-					airBaseHandler->FindClosestAirBasePos(owner, owner->unitDef->minAirBasePower);
-
-				if (landingPos != ZeroVector) {
-					// NOTE: owner->userAttackGround is wrongly reset to
-					// true in CUnit::AttackGround() via ExecuteAttack()
-					// so don't call it
-					SetGoal(landingPos, owner->pos);
-				} else {
-					owner->moveType->StopMoving();
-				}
-			}
-			return true;
-		} else if (owner->currentFuel < (owner->moveType->repairBelowHealth * owner->unitDef->maxFuel) && true
-			/* (commandQue.empty() || commandQue.front().id == CMD_PATROL || commandQue.front().id == CMD_FIGHT) */) {
-			// current fuel level is below our bingo threshold
-			// note: force the refuel attempt (irrespective of
-			// what our currently active command is)
-
-			CAirBaseHandler::LandingPad* lp =
-				airBaseHandler->FindAirBase(owner, owner->unitDef->minAirBasePower);
-
-			if (lp) {
-				StopMove();
-				owner->userAttackGround = false;
-				owner->userTarget = 0;
-				inCommand = false;
-				owner->moveType->ReservePad(lp);
-				return true;
+				owner->moveType->StopMoving();
 			}
 		}
+		return true;
+	} else if (owner->moveType->WantsRefuel() && true
+		/* (commandQue.empty() || commandQue.front().id == CMD_PATROL || commandQue.front().id == CMD_FIGHT) */) {
+		// current fuel level is below our bingo threshold
+		// note: force the refuel attempt (irrespective of
+		// what our currently active command is)
+
+		CAirBaseHandler::LandingPad* lp =
+			airBaseHandler->FindAirBase(owner, owner->unitDef->minAirBasePower);
+
+		if (lp != NULL) {
+			StopMove();
+			owner->userAttackGround = false;
+			owner->SetUserTarget(NULL);
+			inCommand = false;
+			owner->moveType->ReservePad(lp);
+			return true;
+		}
 	}
+
 	return false;
 }
 
 /// returns true if the unit has to land
 bool CMobileCAI::LandRepairIfNeeded()
 {
-	if (!owner->moveType->reservedPad
-		&& owner->health < owner->maxHealth * owner->moveType->repairBelowHealth) {
+	if (owner->moveType->reservedPad == NULL && owner->moveType->WantsRepair()) {
 		// we're damaged, just seek a pad for repairs
 		CAirBaseHandler::LandingPad* lp =
 			airBaseHandler->FindAirBase(owner, owner->unitDef->minAirBasePower);
 
-		if (lp) {
+		if (lp != NULL) {
 			owner->moveType->ReservePad(lp);
 			return true;
 		}
 
-		float3 newGoal = airBaseHandler->FindClosestAirBasePos(owner, owner->unitDef->minAirBasePower);
+		const float3& newGoal = airBaseHandler->FindClosestAirBasePos(owner, owner->unitDef->minAirBasePower);
+
 		if (newGoal != ZeroVector) {
 			SetGoal(newGoal, owner->pos);
 			return true;
 		}
 	}
+
 	return false;
 }
 
@@ -385,7 +387,7 @@ void CMobileCAI::SlowUpdate()
 		IdleCheck();
 
 		//the attack order could terminate directly and thus cause a loop
-		if(commandQue.empty() || commandQue.front().id == CMD_ATTACK) {
+		if(commandQue.empty() || commandQue.front().GetID() == CMD_ATTACK) {
 			return;
 		}
 	}
@@ -394,9 +396,9 @@ void CMobileCAI::SlowUpdate()
 	// to the current command  (and ignore them when it's their turn)
 	if (commandQue.size() >= 2 && !slowGuard) {
 		CCommandQueue::iterator it = commandQue.begin();
-		it++;
+		++it;
 		const Command& c = *it;
-		if ((c.id == CMD_SET_WANTED_MAX_SPEED) && (c.params.size() >= 1)) {
+		if ((c.GetID()== CMD_SET_WANTED_MAX_SPEED) && (c.params.size() >= 1)) {
 			const float defMaxSpeed = owner->maxSpeed;
 			const float newMaxSpeed = std::min(c.params[0], defMaxSpeed);
 			if (newMaxSpeed > 0)
@@ -413,7 +415,7 @@ void CMobileCAI::SlowUpdate()
 void CMobileCAI::Execute()
 {
 	Command& c = commandQue.front();
-	switch (c.id) {
+	switch (c.GetID()) {
 		case CMD_SET_WANTED_MAX_SPEED: { ExecuteSetWantedMaxSpeed(c);	return; }
 		case CMD_MOVE:                 { ExecuteMove(c);				return; }
 		case CMD_PATROL:               { ExecutePatrol(c);				return; }
@@ -433,7 +435,7 @@ void CMobileCAI::Execute()
 void CMobileCAI::ExecuteSetWantedMaxSpeed(Command &c)
 {
 	if (repeatOrders && (commandQue.size() >= 2) &&
-			(commandQue.back().id != CMD_SET_WANTED_MAX_SPEED)) {
+			(commandQue.back().GetID() != CMD_SET_WANTED_MAX_SPEED)) {
 		commandQue.push_back(commandQue.front());
 	}
 	FinishCommand();
@@ -469,10 +471,8 @@ void CMobileCAI::ExecuteLoadUnits(Command &c) {
 
 	if (!inCommand) {
 		inCommand = true;
-		Command newCommand;
-		newCommand.id = CMD_LOAD_UNITS;
+		Command newCommand(CMD_LOAD_UNITS, INTERNAL_ORDER | SHIFT_KEY);
 		newCommand.params.push_back(owner->id);
-		newCommand.options = INTERNAL_ORDER | SHIFT_KEY;
 		tran->commandAI->GiveCommandReal(newCommand);
 	}
 	if (owner->transporter) {
@@ -503,21 +503,21 @@ void CMobileCAI::ExecutePatrol(Command &c)
 {
 	assert(owner->unitDef->canPatrol);
 	if (c.params.size() < 3) {
-		logOutput.Print("Error: got patrol cmd with less than 3 params on %s in MobileCAI",
-			owner->unitDef->humanName.c_str());
+		LOG_L(L_ERROR,
+				"Received a Patrol command with less than 3 params on %s in MobileCAI",
+				owner->unitDef->humanName.c_str());
 		return;
 	}
-	Command temp;
-	temp.id = CMD_FIGHT;
+	Command temp(CMD_FIGHT, c.options | INTERNAL_ORDER);
 	temp.params.push_back(c.params[0]);
 	temp.params.push_back(c.params[1]);
 	temp.params.push_back(c.params[2]);
-	temp.options = c.options | INTERNAL_ORDER;
+
 	commandQue.push_back(c);
 	commandQue.pop_front();
 	commandQue.push_front(temp);
-	Command tmpC;
-	tmpC.id = CMD_PATROL;
+
+	Command tmpC(CMD_PATROL);
 	eoh->CommandFinished(*owner, tmpC);
 	ExecuteFight(temp);
 }
@@ -548,8 +548,9 @@ void CMobileCAI::ExecuteFight(Command &c)
 		tempOrder = false;
 	}
 	if (c.params.size() < 3) {
-		logOutput.Print("Error: got fight cmd with less than 3 params on %s in MobileCAI",
-			owner->unitDef->humanName.c_str());
+		LOG_L(L_ERROR,
+				"Received a Fight command with less than 3 params on %s in MobileCAI",
+				owner->unitDef->humanName.c_str());
 		return;
 	}
 	if(c.params.size() >= 6){
@@ -580,14 +581,12 @@ void CMobileCAI::ExecuteFight(Command &c)
 		SetGoal(pos, owner->pos);
 	}
 
-	if (owner->unitDef->canAttack && owner->fireState >= 2 && !owner->weapons.empty()) {
+	if (owner->unitDef->canAttack && owner->fireState >= FIRESTATE_FIREATWILL && !owner->weapons.empty()) {
 		const float3 curPosOnLine = ClosestPointOnLine(commandPos1, commandPos2, owner->pos);
 		const float searchRadius = owner->maxRange + 100 * owner->moveState * owner->moveState;
 		CUnit* enemy = helper->GetClosestValidTarget(curPosOnLine, searchRadius, owner->allyteam, this);
 		if (enemy != NULL) {
-			Command c2;
-			c2.id=CMD_FIGHT;
-			c2.options=c.options|INTERNAL_ORDER;
+			Command c2(CMD_FIGHT, c.options|INTERNAL_ORDER);
 			c2.params.push_back(enemy->id);
 			PushOrUpdateReturnFight();
 			commandQue.push_front(c2);
@@ -612,7 +611,7 @@ bool CMobileCAI::IsValidTarget(const CUnit* enemy) const {
 		&& !(owner->unitDef->noChaseCategory & enemy->category)
 		&& !enemy->neutral
 		// on "Hold pos", a target can not be valid if there exists no line of fire to it.
-		&& (owner->moveState || owner->weapons.empty() ||
+		&& (owner->moveState > MOVESTATE_HOLDPOS || owner->weapons.empty() ||
 				owner->weapons.front()->TryTargetRotate(const_cast<CUnit*>(enemy), false));
 }
 
@@ -624,81 +623,63 @@ void CMobileCAI::ExecuteGuard(Command &c)
 	assert(owner->unitDef->canGuard);
 	assert(!c.params.empty());
 
-	CUnit* guarded = uh->GetUnit(c.params[0]);
+	const CUnit* guardee = uh->GetUnit(c.params[0]);
 
-	if (guarded != NULL && UpdateTargetLostTimer(guarded->id)) {
-		if (owner->unitDef->canAttack && guarded->lastAttack + 40 < gs->frameNum
-				&& IsValidTarget(guarded->lastAttacker))
-		{
-			StopSlowGuard();
-			Command nc;
-			nc.id = CMD_ATTACK;
-			nc.params.push_back(guarded->lastAttacker->id);
-			nc.options = c.options;
-			commandQue.push_front(nc);
-			SlowUpdate();
-			return;
-		} else {
-			//float3 dif = guarded->speed * guarded->frontdir;
-			float3 dif = guarded->pos - owner->pos;
-			dif.Normalize();
-			float3 goal = guarded->pos - dif * (guarded->radius + owner->radius + 64);
-			if((goalPos - goal).SqLength2D() > 1600
-					|| (goalPos - owner->pos).SqLength2D()
-						< (owner->maxSpeed*30 + 1 + SQUARE_SIZE*2)
-						 * (owner->maxSpeed*30 + 1 + SQUARE_SIZE*2)){
-				SetGoal(goal, owner->pos);
-			}
-			if((goal - owner->pos).SqLength2D() < 6400) {
-				StartSlowGuard(guarded->maxSpeed);
-				if((goal-owner->pos).SqLength2D() < 1800){
-					StopMove();
-					NonMoving();
-				}
-			} else {
-				StopSlowGuard();
-			}
-		}
+	if (guardee == NULL) { FinishCommand(); return; }
+	if (UpdateTargetLostTimer(guardee->id) == 0) { FinishCommand(); return; }
+	if (guardee->outOfMapTime > (GAME_SPEED * 5)) { FinishCommand(); return; }
+
+	const bool pushAttackCommand =
+		owner->unitDef->canAttack &&
+		(guardee->lastAttack + 40 < gs->frameNum) &&
+		IsValidTarget(guardee->lastAttacker);
+
+	if (pushAttackCommand) {
+		Command nc(CMD_ATTACK, c.options);
+		nc.params.push_back(guardee->lastAttacker->id);
+		commandQue.push_front(nc);
+
+		StopSlowGuard();
+		SlowUpdate();
 	} else {
-		FinishCommand();
+		const float3 dif = (guardee->pos - owner->pos).SafeNormalize();
+		const float3 goal = guardee->pos - dif * (guardee->radius + owner->radius + 64.0f);
+		const bool resetGoal =
+			((goalPos - goal).SqLength2D() > 1600.0f) ||
+			(goalPos - owner->pos).SqLength2D() < Square(owner->maxSpeed * GAME_SPEED + 1 + SQUARE_SIZE * 2);
+
+		if (resetGoal) {
+			SetGoal(goal, owner->pos);
+		}
+
+		if ((goal - owner->pos).SqLength2D() < 6400.0f) {
+			StartSlowGuard(guardee->maxSpeed);
+
+			if ((goal - owner->pos).SqLength2D() < 1800.0f) {
+				StopMove();
+				NonMoving();
+			}
+		} else {
+			StopSlowGuard();
+		}
 	}
-	return;
 }
 
-/**
-* @brief Executes the stop command c
-*/
+
 void CMobileCAI::ExecuteStop(Command &c)
 {
 	StopMove();
 	return CCommandAI::ExecuteStop(c);
 }
 
-/**
-* @brief Executes the DGun command c
-*/
-void CMobileCAI::ExecuteDGun(Command &c)
-{
-	if (uh->limitDgun && owner->unitDef->isCommander
-			&& owner->pos.SqDistance(teamHandler->Team(owner->team)->startPos) > Square(uh->dgunRadius)) {
-		StopMove();
-		return FinishCommand();
-	}
-	ExecuteAttack(c);
-}
 
 
-
-
-/**
-* @brief Causes this CMobileCAI to execute the attack order c
-*/
 void CMobileCAI::ExecuteAttack(Command &c)
 {
 	assert(owner->unitDef->canAttack);
 
 	// limit how far away we fly
-	if (tempOrder && (owner->moveState < 2) && orderTarget
+	if (tempOrder && (owner->moveState < MOVESTATE_ROAM) && orderTarget
 			&& LinePointDist(ClosestPointOnLine(commandPos1, commandPos2, owner->pos),
 					commandPos2, orderTarget->pos)
 			> (500 * owner->moveState + owner->maxRange)) {
@@ -722,8 +703,7 @@ void CMobileCAI::ExecuteAttack(Command &c)
 
 				SetGoal(fix - diff * targetUnit->radius, owner->pos);
 
-				orderTarget = targetUnit;
-				AddDeathDependence(orderTarget);
+				SetOrderTarget(targetUnit);
 				inCommand = true;
 			} else {
 				// unit may not fire on itself, cancel order
@@ -741,7 +721,7 @@ void CMobileCAI::ExecuteAttack(Command &c)
 	}
 	else if ((c.params.size() == 3) && (owner->commandShotCount > 0) && (commandQue.size() > 1)) {
 		// the trailing CMD_SET_WANTED_MAX_SPEED in a command pair does not count
-		if ((commandQue.size() > 2) || (commandQue.back().id != CMD_SET_WANTED_MAX_SPEED)) {
+		if ((commandQue.size() > 2) || (commandQue.back().GetID() != CMD_SET_WANTED_MAX_SPEED)) {
 			StopMove();
 			FinishCommand();
 			return;
@@ -760,27 +740,35 @@ void CMobileCAI::ExecuteAttack(Command &c)
 
 	// user clicked on enemy unit (note that we handle aircrafts slightly differently)
 	if (orderTarget) {
-		//bool b1 = owner->AttackUnit(orderTarget, c.id == CMD_DGUN);
 		bool b2 = false;
 		bool b3 = false;
 		bool b4 = false;
 		float edgeFactor = 0.f; // percent offset to target center
 		float3 diff = owner->pos - orderTarget->midPos;
 
-		if (owner->weapons.size() > 0) {
+		if (!owner->weapons.empty()) {
 			if (!(c.options & ALT_KEY) && SkipParalyzeTarget(orderTarget)) {
 				StopMove();
 				FinishCommand();
 				return;
 			}
+
 			CWeapon* w = owner->weapons.front();
+
+			if (c.GetID() == CMD_MANUALFIRE) {
+				assert(owner->unitDef->canManualFire);
+				for (vector<CWeapon*>::iterator it = owner->weapons.begin(); it != owner->weapons.end(); ++it) {
+					if ((*it)->weaponDef->manualfire) {
+						w = *it; break;
+					}
+				}
+			}
+
 			// if we have at least one weapon then check if we
 			// can hit target with our first (meanest) one
-			b2 = w->TryTargetRotate(orderTarget, c.id == CMD_DGUN);
-			b3 = Square(w->range - (w->relWeaponPos).Length())
-					> (orderTarget->pos.SqDistance(owner->pos));
-			b4 = w->TryTargetHeading(GetHeadingFromVector(-diff.x, -diff.z),
-					orderTarget->pos, orderTarget != NULL, orderTarget);
+			b2 = w->TryTargetRotate(orderTarget, c.GetID() == CMD_MANUALFIRE);
+			b3 = Square(w->range - (w->relWeaponPos).Length()) > (orderTarget->pos.SqDistance(owner->pos));
+			b4 = w->TryTargetHeading(GetHeadingFromVector(-diff.x, -diff.z), orderTarget->pos, orderTarget != NULL, orderTarget);
 			edgeFactor = fabs(w->targetBorder);
 		}
 
@@ -791,7 +779,7 @@ void CMobileCAI::ExecuteAttack(Command &c)
 		// also make sure that we're not locked in close-in/in-range state loop
 		// due to rotates invoked by in-range or out-of-range states
 		if (b2) {
-			if (!(tempOrder && owner->moveState == 0)
+			if (!(tempOrder && owner->moveState == MOVESTATE_HOLDPOS)
 				&& (diffLength2d * 1.4f > owner->maxRange
 					- orderTarget->speed.SqLength()
 							/ owner->unitDef->maxAcc)
@@ -808,22 +796,22 @@ void CMobileCAI::ExecuteAttack(Command &c)
 								owner->maxRange * 0.9f), true);
 				}
 			}
-			owner->AttackUnit(orderTarget, c.id == CMD_DGUN);
+			owner->AttackUnit(orderTarget, c.GetID() == CMD_MANUALFIRE);
 		}
 
 		// if we're on hold pos in a temporary order, then none of the close-in
 		// code below should run, and the attack command is cancelled.
-		else if (tempOrder && owner->moveState == 0) {
+		else if (tempOrder && owner->moveState == MOVESTATE_HOLDPOS) {
 			StopMove();
 			FinishCommand();
 			return;
 		}
 
-		// if ((our movetype has type TAAirMoveType and length of 2D vector from us to target
+		// if ((our movetype has type HoverAirMoveType and length of 2D vector from us to target
 		// less than 90% of our maximum range) OR squared length of 2D vector from us to target
 		// less than 1024) then we are close enough
 		else if(diffLength2d < (owner->maxRange * 0.9f)){
-			if (dynamic_cast<CTAAirMoveType*>(owner->moveType)
+			if (dynamic_cast<CHoverAirMoveType*>(owner->moveType)
 					|| (diff.SqLength2D() < 1024))
 			{
 				StopMove();
@@ -876,35 +864,35 @@ void CMobileCAI::ExecuteAttack(Command &c)
 		const float3 pos(c.params[0], c.params[1], c.params[2]);
 		const float3 diff = owner->pos - pos;
 
-		if (owner->weapons.size() > 0) {
+		if (!owner->weapons.empty()) {
 			// if we have at least one weapon then check if
 			// we can hit position with our first (assumed
 			// to be meanest) one
 			CWeapon* w = owner->weapons.front();
 
-			// XXX hack - dgun overrides any checks
-			if (c.id == CMD_DGUN) {
+			// XXX HACK - special weapon overrides any checks
+			if (c.GetID() == CMD_MANUALFIRE) {
+				assert(owner->unitDef->canManualFire);
 				float rr = owner->maxRange * owner->maxRange;
 
-				for (vector<CWeapon*>::iterator it = owner->weapons.begin();
-						it != owner->weapons.end(); ++it) {
-
-					if (dynamic_cast<CDGunWeapon*>(*it))
-						rr = (*it)->range * (*it)->range;
+				for (vector<CWeapon*>::iterator it = owner->weapons.begin(); it != owner->weapons.end(); ++it) {
+					if ((*it)->weaponDef->manualfire) {
+						rr = (*it)->range * (*it)->range; break;
+					}
 				}
 
 				if (diff.SqLength() < rr) {
 					StopMove();
-					owner->AttackGround(pos, c.id == CMD_DGUN);
+					owner->AttackGround(pos, c.GetID() == CMD_MANUALFIRE);
 					owner->moveType->KeepPointingTo(pos, owner->maxRange * 0.9f, true);
 				}
 			} else {
-				const bool inAngle = w->TryTargetRotate(pos, c.id == CMD_DGUN);
+				const bool inAngle = w->TryTargetRotate(pos, c.GetID() == CMD_MANUALFIRE);
 				const bool inRange = diff.SqLength2D() < Square(w->range - (w->relWeaponPos).Length2D());
 
 				if (inAngle || inRange) {
 					StopMove();
-					owner->AttackGround(pos, c.id == CMD_DGUN);
+					owner->AttackGround(pos, c.GetID() == CMD_MANUALFIRE);
 					owner->moveType->KeepPointingTo(pos, owner->maxRange * 0.9f, true);
 				}
 			}
@@ -966,108 +954,47 @@ void CMobileCAI::StopMove()
 	goalPos = owner->pos;
 }
 
-void CMobileCAI::DrawCommands(void)
+
+
+void CMobileCAI::BuggerOff(const float3& pos, float radius)
 {
-	lineDrawer.StartPath(owner->drawMidPos, cmdColors.start);
-
-	if (owner->selfDCountdown != 0) {
-		lineDrawer.DrawIconAtLastPos(CMD_SELFD);
+	if(radius < 0.0f) {
+		lastBuggerOffTime = gs->frameNum - BUGGER_OFF_TTL;
+		return;
 	}
-
-	CCommandQueue::iterator ci;
-	for (ci = commandQue.begin(); ci != commandQue.end(); ++ci) {
-		switch (ci->id) {
-			case CMD_MOVE: {
-				const float3 endPos(ci->params[0], ci->params[1], ci->params[2]);
-				lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.move);
-				break;
-			}
-			case CMD_PATROL: {
-				const float3 endPos(ci->params[0], ci->params[1], ci->params[2]);
-				lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.patrol);
-				break;
-			}
-			case CMD_FIGHT: {
-				if (ci->params.size() != 1) {
-					const float3 endPos(ci->params[0], ci->params[1], ci->params[2]);
-					lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.fight);
-					break;
-				}
-			}
-			case CMD_ATTACK:
-			case CMD_DGUN: {
-				if (ci->params.size() == 1) {
-					const CUnit* unit = uh->GetUnit(ci->params[0]);
-
-					if ((unit != NULL) && isTrackable(unit)) {
-						const float3 endPos = helper->GetUnitErrorPos(unit, owner->allyteam);
-						lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.attack);
-					}
-				}
-				else if (ci->params.size() >= 3) {
-					const float3 endPos(ci->params[0], ci->params[1], ci->params[2]);
-					lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.attack);
-				}
-				break;
-			}
-			case CMD_GUARD: {
-				const CUnit* unit = uh->GetUnit(ci->params[0]);
-
-				if ((unit != NULL) && isTrackable(unit)) {
-					const float3 endPos =
-						helper->GetUnitErrorPos(unit, owner->allyteam);
-					lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.guard);
-				}
-				break;
-			}
-			case CMD_LOAD_ONTO: {
-				const CUnit* unit = uh->GetUnitUnsafe(ci->params[0]);
-				lineDrawer.DrawLineAndIcon(ci->id, unit->pos, cmdColors.load);
-				break;
-			}
-			case CMD_WAIT: {
-				DrawWaitIcon(*ci);
-				break;
-			}
-			case CMD_SELFD: {
-				lineDrawer.DrawIconAtLastPos(ci->id);
-				break;
-			}
-			default: {
-				DrawDefaultCommand(*ci);
-				break;
-			}
-		}
-	}
-	lineDrawer.FinishPath();
-}
-
-
-void CMobileCAI::BuggerOff(float3 pos, float radius)
-{
 	lastBuggerOffTime = gs->frameNum;
 	buggerOffPos = pos;
 	buggerOffRadius = radius + owner->radius;
 }
 
-void CMobileCAI::NonMoving(void)
+void CMobileCAI::NonMoving()
 {
 	if (owner->usingScriptMoveType) {
 		return;
 	}
-	if(lastBuggerOffTime>gs->frameNum-200){
-		float3 dif=owner->pos-buggerOffPos;
-		dif.y=0;
+	if (lastBuggerOffTime > gs->frameNum - BUGGER_OFF_TTL) {
+		float3 dif = owner->pos-buggerOffPos;
+		dif.y = 0.0f;
 		float length=dif.Length();
-		if(!length) {
-			length=0.1f;
+		if (!length) {
+			length = 0.1f;
 			dif = float3(0.1f, 0.0f, 0.0f);
 		}
 		if (length < buggerOffRadius) {
 			float3 goalPos = buggerOffPos + dif * ((buggerOffRadius + 128) / length);
-			Command c;
-			c.id = CMD_MOVE;
-			c.options = 0;//INTERNAL_ORDER;
+			bool randomize = (goalPos.x == lastBuggerGoalPos.x) && (goalPos.z == lastBuggerGoalPos.z);
+			lastBuggerGoalPos.x = goalPos.x;
+			lastBuggerGoalPos.z = goalPos.z;
+			if (randomize) {
+				lastBuggerGoalPos.y += 32.0f; // gradually increase the amplitude of the random factor
+				goalPos.x += (2.0f * lastBuggerGoalPos.y) * gs->randFloat() - lastBuggerGoalPos.y;
+				goalPos.z += (2.0f * lastBuggerGoalPos.y) * gs->randFloat() - lastBuggerGoalPos.y;
+			}
+			else
+				lastBuggerGoalPos.y = 0.0f;
+
+			Command c(CMD_MOVE);
+			//c.options = INTERNAL_ORDER;
 			c.params.push_back(goalPos.x);
 			c.params.push_back(goalPos.y);
 			c.params.push_back(goalPos.z);
@@ -1078,7 +1005,7 @@ void CMobileCAI::NonMoving(void)
 	}
 }
 
-void CMobileCAI::FinishCommand(void)
+void CMobileCAI::FinishCommand()
 {
 	if (!(commandQue.front().options & INTERNAL_ORDER)) {
 		lastUserGoal = owner->pos;
@@ -1087,16 +1014,15 @@ void CMobileCAI::FinishCommand(void)
 	CCommandAI::FinishCommand();
 }
 
-void CMobileCAI::IdleCheck(void)
+void CMobileCAI::IdleCheck()
 {
-	if(owner->unitDef->canAttack && owner->fireState
+	if (owner->unitDef->canAttack && owner->fireState > FIRESTATE_HOLDFIRE
 			&& !owner->weapons.empty() && owner->haveTarget) {
 		if(!owner->userTarget) {
 			owner->haveTarget = false;
 		} else if(owner->pos.SqDistance2D(owner->userTarget->pos) <
 				Square(owner->maxRange + 200*owner->moveState*owner->moveState)) {
-			Command c;
-			c.id = CMD_ATTACK;
+			Command c(CMD_ATTACK);
 			c.options=INTERNAL_ORDER;
 			c.params.push_back(owner->userTarget->id);
 			c.timeOut = gs->frameNum + 140;
@@ -1107,15 +1033,14 @@ void CMobileCAI::IdleCheck(void)
 			return;
 		}
 	}
-	if(owner->unitDef->canAttack && owner->fireState
+	if (owner->unitDef->canAttack && owner->fireState > FIRESTATE_HOLDFIRE
 				&& !owner->weapons.empty() && !owner->haveTarget) {
 		if(owner->lastAttacker && owner->lastAttack + 200 > gs->frameNum
 				&& !(owner->unitDef->noChaseCategory & owner->lastAttacker->category)){
 			float3 apos=owner->lastAttacker->pos;
 			float dist=apos.SqDistance2D(owner->pos);
 			if(dist<Square(owner->maxRange+200*owner->moveState*owner->moveState)){
-				Command c;
-				c.id=CMD_ATTACK;
+				Command c(CMD_ATTACK);
 				c.options=INTERNAL_ORDER;
 				c.params.push_back(owner->lastAttacker->id);
 				c.timeOut=gs->frameNum+140;
@@ -1128,13 +1053,12 @@ void CMobileCAI::IdleCheck(void)
 		}
 	}
 	if (owner->unitDef->canAttack && (gs->frameNum >= lastIdleCheck + 10)
-			&& owner->fireState >= 2 && !owner->weapons.empty() && !owner->haveTarget)
+			&& owner->fireState >= FIRESTATE_FIREATWILL && !owner->weapons.empty() && !owner->haveTarget)
 	{
 		const float searchRadius = owner->maxRange + 150 * owner->moveState * owner->moveState;
 		CUnit* enemy = helper->GetClosestValidTarget(owner->pos, searchRadius, owner->allyteam, this);
 		if (enemy != NULL) {
-			Command c;
-			c.id=CMD_ATTACK;
+			Command c(CMD_ATTACK);
 			c.options=INTERNAL_ORDER;
 			c.params.push_back(enemy->id);
 			c.timeOut=gs->frameNum+140;
@@ -1150,12 +1074,10 @@ void CMobileCAI::IdleCheck(void)
 	}
 	lastIdleCheck = gs->frameNum;
 	if (((owner->pos - lastUserGoal).SqLength2D() > 10000.0f) &&
-	    !owner->haveTarget && !dynamic_cast<CTAAirMoveType*>(owner->moveType)) {
+	    !owner->haveTarget && !dynamic_cast<CHoverAirMoveType*>(owner->moveType)) {
 		//note that this is not internal order so that we dont keep generating
 		//new orders if we cant get to that pos
-		Command c;
-		c.id=CMD_MOVE;
-		c.options=0;
+		Command c(CMD_MOVE);
 		c.params.push_back(lastUserGoal.x);
 		c.params.push_back(lastUserGoal.y);
 		c.params.push_back(lastUserGoal.z);
@@ -1182,7 +1104,7 @@ void CMobileCAI::StartSlowGuard(float speed){
 			if (!commandQue.empty()) {
 				Command currCommand = commandQue.front();
 				if (commandQue.size() <= 1
-						|| commandQue[1].id != CMD_SET_WANTED_MAX_SPEED
+						|| commandQue[1].GetID() != CMD_SET_WANTED_MAX_SPEED
 						|| commandQue[1].params[0] > speed){
 					if (speed > 0)
 						owner->moveType->SetMaxSpeed(speed);
