@@ -6,7 +6,6 @@
 #include "Game/GameSetup.h"
 #include "Lua/LuaParser.h"
 #include "Map/MapInfo.h"
-#include "Map/ReadMap.h"
 #include "Sim/Misc/CategoryHandler.h"
 #include "Sim/Misc/CollisionVolume.h"
 #include "Sim/Misc/DamageArrayHandler.h"
@@ -17,15 +16,15 @@
 #include "Sim/Units/CommandAI/Command.h"
 #include "Rendering/IconHandler.h"
 #include "Rendering/Models/IModelParser.h"
+#include "System/EventHandler.h"
 #include "System/Exceptions.h"
-#include "System/LogOutput.h"
+#include "System/Log/ILog.h"
 #include "System/myMath.h"
 #include "System/Util.h"
 
-
 /******************************************************************************/
 
-UnitDef::UnitDefWeapon::UnitDefWeapon()
+UnitDefWeapon::UnitDefWeapon()
 : name("NOWEAPON")
 , def(NULL)
 , slavedTo(0)
@@ -37,7 +36,7 @@ UnitDef::UnitDefWeapon::UnitDefWeapon()
 {
 }
 
-UnitDef::UnitDefWeapon::UnitDefWeapon(
+UnitDefWeapon::UnitDefWeapon(
 	std::string name, const WeaponDef* def, int slavedTo, float3 mainDir, float maxAngleDif,
 	unsigned int badTargetCat, unsigned int onlyTargetCat, float fuelUse)
 : name(name)
@@ -56,12 +55,10 @@ UnitDef::UnitDefWeapon::UnitDefWeapon(
 
 UnitDef::UnitDef()
 : id(-1)
+, cobID(-1)
 , collisionVolume(NULL)
 , decoyDef(NULL)
-, aihint(0)
-, cobID(-1)
 , techLevel(0)
-, gaia("")
 , metalUpkeep(0.0f)
 , energyUpkeep(0.0f)
 , metalMake(0.0f)
@@ -87,13 +84,13 @@ UnitDef::UnitDef()
 , rSpeed(0.0f)
 , turnRate(0.0f)
 , turnInPlace(false)
-, turnInPlaceDistance(0.0f)
 , turnInPlaceSpeedLimit(0.0f)
 , upright(false)
 , collide(false)
+, losHeight(0.0f)
+, radarHeight(0.0f)
 , losRadius(0.0f)
 , airLosRadius(0.0f)
-, losHeight(0.0f)
 , radarRadius(0.0f)
 , sonarRadius(0.0f)
 , jammerRadius(0.0f)
@@ -116,7 +113,6 @@ UnitDef::UnitDef()
 , strafeToAttack(false)
 , minCollisionSpeed(0.0f)
 , slideTolerance(0.0f)
-, maxSlope(0.0f)
 , maxHeightDif(0.0f)
 , minWaterDepth(0.0f)
 , waterline(0.0f)
@@ -162,8 +158,8 @@ UnitDef::UnitDef()
 , canBeAssisted(false)
 , canRepeat(false)
 , canFireControl(false)
-, fireState(0)
-, moveState(0)
+, fireState(FIRESTATE_HOLDFIRE)
+, moveState(MOVESTATE_HOLDPOS)
 , wingDrag(0.0f)
 , wingAngle(0.0f)
 , drag(0.0f)
@@ -176,7 +172,6 @@ UnitDef::UnitDef()
 , wantedHeight(0.0f)
 , verticalSpeed(0.0f)
 , useSmoothMesh(false)
-, canCrash(false)
 , hoverAttack(false)
 , airStrafe(false)
 , dlHoverFactor(0.0f)
@@ -190,7 +185,6 @@ UnitDef::UnitDef()
 , movedata(NULL)
 , xsize(0)
 , zsize(0)
-, buildangle(0)
 , loadingRadius(0.0f)
 , unloadSpread(0.0f)
 , transportCapacity(0)
@@ -219,7 +213,7 @@ UnitDef::UnitDef()
 , kamikazeDist(0.0f)
 , kamikazeUseLOS(false)
 , targfac(false)
-, canDGun(false)
+, canManualFire(false)
 , needGeo(false)
 , isFeature(false)
 , hideDamage(false)
@@ -243,11 +237,10 @@ UnitDef::UnitDef()
 , flareTime(0)
 , flareSalvoSize(0)
 , flareSalvoDelay(0)
-, smoothAnim(false)
 , canLoopbackAttack(false)
 , levelGround(false)
 , useBuildingGroundDecal(false)
-, buildingDecalType(0)
+, buildingDecalType(-1)
 , buildingDecalSizeX(0)
 , buildingDecalSizeY(0)
 , buildingDecalDecaySpeed(0.0f)
@@ -257,7 +250,6 @@ UnitDef::UnitDef()
 , maxFuel(0.0f)
 , refuelTime(0.0f)
 , minAirBasePower(0.0f)
-, pieceTrailCEGRange(-1)
 , maxThisUnit(0)
 , realMetalCost(0.0f)
 , realEnergyCost(0.0f)
@@ -275,23 +267,19 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 , decoyDef(NULL)
 , techLevel(-1)
 , buildPic(NULL)
-, buildangle(0)
+, movedata(NULL)
+, trackType(0)
+, buildingDecalType(-1)
+, realMetalCost(0.0f)
+, realEnergyCost(0.0f)
+, realMetalUpkeep(0.0f)
+, realEnergyUpkeep(0.0f)
+, realBuildTime(0.0f)
 {
 	humanName = udTable.GetString("name", "");
-	if (humanName.empty()) {
-		const string errmsg = "missing 'name' parameter for the " + unitName + " unitdef";
-		throw content_error(errmsg);
-	}
-	filename  = udTable.GetString("filename", "");
-	if (filename.empty()) {
-		const string errmsg = "missing 'filename' parameter for the" + unitName + " unitdef";
-		throw content_error(errmsg);
-	}
 	tooltip = udTable.GetString("description", name);
 	buildPicName = udTable.GetString("buildPic", "");
-
 	decoyName = udTable.GetString("decoyFor", "");
-	gaia = udTable.GetString("gaia", "");
 
 	isCommander = udTable.GetBool("commander", false);
 
@@ -308,29 +296,19 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	makesMetal   = udTable.GetFloat("makesMetal", 0.0f);
 	energyMake   = udTable.GetFloat("energyMake", 0.0f);
 
-	health       = udTable.GetFloat("maxDamage",  0.0f);
-	autoHeal     = udTable.GetFloat("autoHeal",      0.0f) * (16.0f / GAME_SPEED);
-	idleAutoHeal = udTable.GetFloat("idleAutoHeal", 10.0f) * (16.0f / GAME_SPEED);
+	health       = std::max(0.1f, udTable.GetFloat("maxDamage",  0.0f)); //avoid some nasty divide by 0
+	autoHeal     = udTable.GetFloat("autoHeal",      0.0f) * (UNIT_SLOWUPDATE_RATE / float(GAME_SPEED));
+	idleAutoHeal = udTable.GetFloat("idleAutoHeal", 10.0f) * (UNIT_SLOWUPDATE_RATE / float(GAME_SPEED));
 	idleTime     = udTable.GetInt("idleTime", 600);
 
-	buildangle = udTable.GetInt("buildAngle", 0);
+	// iff a mass value is not defined, default to metalCost
+	// (do not allow it to be zero or negative in either case)
+	metalCost = std::max(1.0f, udTable.GetFloat("buildCostMetal", 0.0f));
+	mass = Clamp(udTable.GetFloat("mass", metalCost), CSolidObject::MINIMUM_MASS, CSolidObject::MAXIMUM_MASS);
 
-	losHeight = 20;
-	metalCost = udTable.GetFloat("buildCostMetal", 0.0f);
-	if (metalCost < 1.0f) {
-		metalCost = 1.0f; //avoid some nasty divide by 0 etc
-	}
-	mass = udTable.GetFloat("mass", 0.0f);
-	if (mass <= 0.0f) {
-		mass = metalCost;
-	}
 	energyCost = udTable.GetFloat("buildCostEnergy", 0.0f);
-	buildTime = udTable.GetFloat("buildTime", 0.0f);
-	if (buildTime < 1.0f) {
-		buildTime = 1.0f; //avoid some nasty divide by 0 etc
-	}
+	buildTime = std::max(0.1f, udTable.GetFloat("buildTime", 0.0f)); //avoid some nasty divide by 0
 
-	aihint = id; // FIXME? (as noted in SelectedUnits.cpp, aihint is ignored)
 	cobID = udTable.GetInt("cobID", -1);
 
 	losRadius = udTable.GetFloat("sightDistance", 0.0f) * modInfo.losMul / (SQUARE_SIZE * (1 << modInfo.losMipLevel));
@@ -354,6 +332,21 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	canRepeat   = udTable.GetBool("canRepeat",   true);
 
 	builder = udTable.GetBool("builder", false);
+	buildRange3D = udTable.GetBool("buildRange3D", false);
+	// 128.0f is the ancient default
+	buildDistance = udTable.GetFloat("buildDistance", 128.0f);
+	// 38.0f was evaluated by bobthedinosaur and FLOZi to be the bare minimum
+	// to not overlap for a 1x1 constructor building a 1x1 structure
+	buildDistance = std::max(38.0f, buildDistance);
+	buildSpeed = udTable.GetFloat("workerTime", 0.0f);
+	builder = builder && (buildSpeed > 0.0f);
+
+	repairSpeed    = udTable.GetFloat("repairSpeed",    buildSpeed);
+	maxRepairSpeed = udTable.GetFloat("maxRepairSpeed", 1e20f);
+	reclaimSpeed   = udTable.GetFloat("reclaimSpeed",   buildSpeed);
+	resurrectSpeed = udTable.GetFloat("resurrectSpeed", buildSpeed);
+	captureSpeed   = udTable.GetFloat("captureSpeed",   buildSpeed);
+	terraformSpeed = udTable.GetFloat("terraformSpeed", buildSpeed);
 
 	canRestore = udTable.GetBool("canRestore", builder);
 	canRepair  = udTable.GetBool("canRepair",  builder);
@@ -369,9 +362,12 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	collide = udTable.GetBool("collide", true);
 	onoffable = udTable.GetBool("onoffable", false);
 
-	maxSlope = Clamp(udTable.GetFloat("maxSlope", 0.0f), 0.0f, 89.0f);
-	maxHeightDif = 40 * tan(maxSlope * (PI / 180));
-	maxSlope = cos(maxSlope * (PI / 180));
+	const float maxSlopeDeg = Clamp(udTable.GetFloat("maxSlope", 0.0f), 0.0f, 89.0f);
+	const float maxSlopeRad = maxSlopeDeg * (PI / 180.0f);
+
+	// FIXME: kill the magic constant
+	maxHeightDif = 40.0f * math::tanf(maxSlopeRad);
+
 	minWaterDepth = udTable.GetFloat("minWaterDepth", -10e6f);
 	maxWaterDepth = udTable.GetFloat("maxWaterDepth", +10e6f);
 	waterline = udTable.GetFloat("waterline", 0.0f);
@@ -392,31 +388,14 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 
 	turnRate    = udTable.GetFloat("turnRate", 0.0f);
 	turnInPlace = udTable.GetBool("turnInPlace", true);
-	turnInPlaceDistance = udTable.GetFloat("turnInPlaceDistance", 350.f);
 	turnInPlaceSpeedLimit = ((turnRate / SPRING_CIRCLE_DIVS) * ((PI + PI) * SQUARE_SIZE)) * (speed / GAME_SPEED);
 	turnInPlaceSpeedLimit = udTable.GetFloat("turnInPlaceSpeedLimit", std::min(speed, turnInPlaceSpeedLimit));
 
-	const bool noAutoFire  = udTable.GetBool("noAutoFire",  false);
-	canFireControl = udTable.GetBool("canFireControl", !noAutoFire);
-	fireState = udTable.GetInt("fireState", canFireControl ? -1 : 2);
-	fireState = std::min(fireState,2);
-	moveState = udTable.GetInt("moveState", (canmove && speed>0.0f)  ? -1 : 1);
-	moveState = std::min(moveState,2);
-
-	buildRange3D = udTable.GetBool("buildRange3D", false);
-	// 128.0f is the ancient default
-	buildDistance = udTable.GetFloat("buildDistance", 128.0f);
-	// 38.0f was evaluated by bobthedinosaur and FLOZi to be the bare minimum
-	// to not overlap for a 1x1 constructor building a 1x1 structure
-	buildDistance = std::max(38.0f, buildDistance);
-	buildSpeed = udTable.GetFloat("workerTime", 0.0f);
-
-	repairSpeed    = udTable.GetFloat("repairSpeed",    buildSpeed);
-	maxRepairSpeed = udTable.GetFloat("maxRepairSpeed", 1e20f);
-	reclaimSpeed   = udTable.GetFloat("reclaimSpeed",   buildSpeed);
-	resurrectSpeed = udTable.GetFloat("resurrectSpeed", buildSpeed);
-	captureSpeed   = udTable.GetFloat("captureSpeed",   buildSpeed);
-	terraformSpeed = udTable.GetFloat("terraformSpeed", buildSpeed);
+	canFireControl = !udTable.GetBool("noAutoFire", false);
+	fireState = udTable.GetInt("fireState", canFireControl? FIRESTATE_NONE: FIRESTATE_FIREATWILL);
+	fireState = std::min(fireState, int(FIRESTATE_FIREATWILL));
+	moveState = udTable.GetInt("moveState", (canmove && speed > 0.0f)? MOVESTATE_NONE: MOVESTATE_MANEUVER);
+	moveState = std::min(moveState, int(MOVESTATE_ROAM));
 
 	flankingBonusMode = udTable.GetInt("flankingBonusMode", modInfo.flankingBonusModeDefault);
 	flankingBonusMax  = udTable.GetFloat("flankingBonusMax", 1.9f);
@@ -427,6 +406,9 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	armoredMultiple = udTable.GetFloat("damageModifier", 1.0f);
 	armorType = damageArrayHandler->GetTypeFromName(name);
 
+	losHeight = udTable.GetFloat("losEmitHeight", 20.0f);
+	radarHeight = udTable.GetFloat("radarEmitHeight", 20.0f);
+
 	radarRadius    = udTable.GetInt("radarDistance",    0);
 	sonarRadius    = udTable.GetInt("sonarDistance",    0);
 	jammerRadius   = udTable.GetInt("radarDistanceJam", 0);
@@ -465,13 +447,8 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	nanoColor = udTable.GetFloat3("nanoColor", float3(0.2f,0.7f,0.2f));
 
 	canhover = udTable.GetBool("canHover", false);
-
 	floater = udTable.GetBool("floater", udTable.KeyExists("WaterLine"));
 
-	if (builder && !buildSpeed) { // core anti is flagged as builder for some reason
-		builder = false;
-	}
-
 	airStrafe      = udTable.GetBool("airStrafe", true);
 	hoverAttack    = udTable.GetBool("hoverAttack", false);
 	wantedHeight   = udTable.GetFloat("cruiseAlt", 0.0f);
@@ -490,7 +467,7 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	minTransportMass  = udTable.GetFloat("minTransportMass", 0.0f);
 	holdSteady        = udTable.GetBool("holdSteady",        false);
 	releaseHeld       = udTable.GetBool("releaseHeld",       false);
-	cantBeTransported = udTable.GetBool("cantBeTransported", false);
+	cantBeTransported = udTable.GetBool("cantBeTransported", !WantsMoveData());
 	transportByEnemy  = udTable.GetBool("transportByEnemy",  true);
 	fallSpeed         = udTable.GetFloat("fallSpeed",    0.2);
 	unitFallSpeed     = udTable.GetFloat("unitFallSpeed",  0);
@@ -515,7 +492,7 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 
 	maxBank = udTable.GetFloat("maxBank", 0.8f);         // max roll
 	maxPitch = udTable.GetFloat("maxPitch", 0.45f);      // max pitch this plane tries to keep
-	turnRadius = udTable.GetFloat("turnRadius", 500.0f); // hint to the ai about how large turn radius this plane needs
+	turnRadius = udTable.GetFloat("turnRadius", 500.0f); // hint to CStrafeAirMoveType about required turn-radius
 	verticalSpeed = udTable.GetFloat("verticalSpeed", 3.0f); // speed of takeoff and landing, at least for gunships
 
 	maxAileron  = udTable.GetFloat("maxAileron",  0.015f); // turn speed around roll axis
@@ -527,10 +504,8 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	minAirBasePower = udTable.GetFloat("minAirBasePower", 0.0f);
 	maxThisUnit = udTable.GetInt("unitRestricted", MAX_UNITS);
 
-	const string lname = StringToLower(name);
-
-	if (gameSetup->restrictedUnits.find(lname) != gameSetup->restrictedUnits.end()) {
-		maxThisUnit = std::min(maxThisUnit, gameSetup->restrictedUnits.find(lname)->second);
+	if (gameSetup->restrictedUnits.find(name) != gameSetup->restrictedUnits.end()) {
+		maxThisUnit = std::min(maxThisUnit, gameSetup->restrictedUnits.find(name)->second);
 	}
 
 	categoryString = udTable.GetString("category", "");
@@ -538,8 +513,7 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	category = CCategoryHandler::Instance()->GetCategories(udTable.GetString("category", ""));
 	noChaseCategory = CCategoryHandler::Instance()->GetCategories(udTable.GetString("noChaseCategory", ""));
 
-	const string iconName = udTable.GetString("iconType", "default");
-	iconType = iconHandler->GetIcon(iconName);
+	iconType = icon::iconHandler->GetIcon(udTable.GetString("iconType", "default"));
 
 	shieldWeaponDef    = NULL;
 	stockpileWeaponDef = NULL;
@@ -550,90 +524,86 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	LuaTable weaponsTable = udTable.SubTable("weapons");
 	ParseWeaponsTable(weaponsTable);
 
-	canDGun = udTable.GetBool("canDGun", false);
+	canManualFire = udTable.GetBool("canDGun", false); // NOTE: deprecated, remove after 0.83.*
+	canManualFire = udTable.GetBool("canManualFire", canManualFire);
+	needGeo = false;
 
-	extractRange = 0.0f;
+	extractRange = mapInfo->map.extractorRadius * int(extractsMetal > 0.0f);
 	extractSquare = udTable.GetBool("extractSquare", false);
 
-	if (extractsMetal) {
-		extractRange = mapInfo->map.extractorRadius;
-		type = "MetalExtractor";
-	}
-	else if (transportCapacity) {
-		type = "Transport";
-	}
-	else if (builder) {
-		if ((speed > 0.0f) || canfly || udTable.GetString("yardMap", "").empty()) {
-			// hubs and nano-towers need to be builders (for now)
-			type = "Builder";
-		} else {
-			type = "Factory";
-		}
-	}
-	else if (canfly && !hoverAttack) {
-		if (!weapons.empty() && (weapons[0].def != 0) &&
-		   (weapons[0].def->type == "AircraftBomb" || weapons[0].def->type == "TorpedoLauncher")) {
-			type = "Bomber";
+	// aircraft have MoveTypes but no MoveData;
+	// static structures have no use for either
+	// (but get StaticMoveType instances)
+	if (WantsMoveData()) {
+		const std::string& moveClass = StringToLower(udTable.GetString("movementClass", ""));
+		const std::string errMsg = "WARNING: Couldn't find a MoveClass named " + moveClass + " (used in UnitDef: " + unitName + ")";
 
-			if (turnRadius == 500) { // only reset it if user hasnt set it explicitly
-				turnRadius *= 2;   // hint to the ai about how large turn radius this plane needs
+		if ((movedata = moveinfo->GetMoveDataFromName(moveClass)) == NULL) {
+			throw content_error(errMsg); //! invalidate unitDef (this gets catched in ParseUnitDef!)
+		}
+
+		movedata->unitDefRefCount += 1;
+
+		if (LOG_IS_ENABLED(L_WARNING)) {
+			const char* typeStr = NULL;
+			const char* wantedTypeStr = NULL;
+			if (canhover) {
+				if (movedata->moveType != MoveData::Hover_Move) {
+					typeStr       = "canhover";
+					wantedTypeStr = "hover";
+				}
+			} else if (floater) {
+				if (movedata->moveType != MoveData::Ship_Move) {
+					typeStr       = "floater";
+					wantedTypeStr = "ship";
+				}
+			} else {
+				if (movedata->moveType != MoveData::Ground_Move) {
+					typeStr       = "!(canhover || floater)";
+					wantedTypeStr = "ground";
+				}
 			}
-		} else {
-			type = "Fighter";
-		}
-		maxAcc = udTable.GetFloat("maxAcc", 0.065f); // engine power
-	}
-	else if (canmove) {
-		type = "GroundUnit";
-	}
-	else {
-		type = "Building";
-	}
-
-	movedata = NULL;
-	if (canmove && !canfly && (type != "Factory")) {
-		string moveclass = StringToLower(udTable.GetString("movementClass", ""));
-		movedata = moveinfo->GetMoveDataFromName(moveclass);
-
-		if (!movedata) {
-			const string errmsg = "WARNING: Couldn't find a MoveClass named " + moveclass + " (used in UnitDef: " + unitName + ")";
-			throw content_error(errmsg); //! invalidate unitDef (this gets catched in ParseUnitDef!)
-		}
-
-		if ((movedata->moveType == MoveData::Hover_Move) ||
-		    (movedata->moveType == MoveData::Ship_Move)) {
-			upright = true;
-		}
-		if (canhover) {
-			if (movedata->moveType != MoveData::Hover_Move) {
-				logOutput.Print("Inconsistent movedata %i for %s (moveclass %s): canhover, but not a hovercraft movetype",
-				     movedata->pathType, name.c_str(), moveclass.c_str());
-			}
-		} else if (floater) {
-			if (movedata->moveType != MoveData::Ship_Move) {
-				logOutput.Print("Inconsistent movedata %i for %s (moveclass %s): floater, but not a ship movetype",
-				     movedata->pathType, name.c_str(), moveclass.c_str());
-			}
-		} else {
-			if (movedata->moveType != MoveData::Ground_Move) {
-				logOutput.Print("Inconsistent movedata %i for %s (moveclass %s): neither canhover nor floater, but not a ground movetype",
-				     movedata->pathType, name.c_str(), moveclass.c_str());
+			if ((typeStr != NULL) && (wantedTypeStr != NULL)) {
+				LOG_L(L_WARNING,
+						"[%s] inconsistent path-type %i for \"%s\" "
+						"(move-class \"%s\"): %s, but not a %s-based movetype",
+						__FUNCTION__, movedata->pathType, name.c_str(),
+						moveClass.c_str(), typeStr, wantedTypeStr);
 			}
 		}
 	}
 
-	if ((maxAcc != 0) && (speed != 0)) {
-		//meant to set the drag such that the maxspeed becomes what it should be
-		drag = 1.0f / (speed/GAME_SPEED * 1.1f / maxAcc)
-		          - (wingAngle * wingAngle * wingDrag);
-		drag = Clamp(drag, 0.0f, 1.0f);
+	if (movedata == NULL) {
+		upright = upright || !canfly;
 	} else {
-		//shouldn't be needed since drag is only used in CAirMoveType anyway,
-		//and aircraft without acceleration or speed aren't common :)
-		//initializing it anyway just for safety
-		drag = 0.005f;
+		upright = upright ||
+			(movedata->moveType == MoveData::Hover_Move) ||
+			(movedata->moveType == MoveData::Ship_Move);
 	}
 
+	if (IsAirUnit()) {
+		if (IsFighterUnit() || IsBomberUnit()) {
+			// double turn-radius for bombers if not set explicitly
+			if (IsBomberUnit() && turnRadius == 500.0f) {
+				turnRadius *= 2.0f;
+			}
+
+			maxAcc = udTable.GetFloat("maxAcc", 0.065f); // engine power
+		}
+
+		if ((maxAcc != 0) && (speed != 0)) {
+			//meant to set the drag such that the maxspeed becomes what it should be
+			drag = (1.0f / (speed / GAME_SPEED * 1.1f / maxAcc)) - (wingAngle * wingAngle * wingDrag);
+			drag = Clamp(drag, 0.0f, 1.0f);
+		} else {
+			//shouldn't be needed since drag is only used in CStrafeAirMoveType anyway,
+			//and aircraft without acceleration or speed aren't common :)
+			//initializing it anyway just for safety
+			drag = 0.005f;
+		}
+	}
+
+
 	objectName = udTable.GetString("objectName", "");
 	if (objectName.find(".") == std::string::npos) {
 		objectName += ".3do"; // NOTE: get rid of this?
@@ -652,20 +622,23 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 
 	// Prevent a division by zero in experience calculations.
 	if (power < 1.0e-3f) {
-		logOutput.Print("Unit %s is really cheap? %f", humanName.c_str(), power);
-		logOutput.Print("This can cause a division by zero in experience calculations.");
+		LOG_L(L_WARNING, "Unit %s is really cheap? %f", humanName.c_str(), power);
+		LOG_L(L_WARNING, "This can cause a division by zero in experience calculations.");
 		power = 1.0e-3f;
 	}
 
 	activateWhenBuilt = udTable.GetBool("activateWhenBuilt", false);
 
-	// TA has only half our res so multiply size with 2
-	xsize = udTable.GetInt("footprintX", 1) * 2;
-	zsize = udTable.GetInt("footprintZ", 1) * 2;
+	// footprint sizes are assumed to be expressed in TA-engine units;
+	// Spring's heightmap resolution is double the footprint (yardmap)
+	// resolution, so we scale the values (which are not allowed to be
+	// 0)
+	// NOTE that this is done for the FeatureDef and MoveData footprints
+	// as well
+	xsize = std::max(1 * 2, (udTable.GetInt("footprintX", 1) * 2));
+	zsize = std::max(1 * 2, (udTable.GetInt("footprintZ", 1) * 2));
 
-	needGeo = false;
-
-	if (speed <= 0.0f) {
+	if (IsImmobileUnit()) {
 		CreateYardMap(udTable.GetString("yardMap", ""));
 	}
 
@@ -691,9 +664,7 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	flareSalvoSize  = udTable.GetInt("flareSalvoSize",  4);
 	flareSalvoDelay = udTable.GetInt("flareSalvoDelay", 0) * GAME_SPEED;
 
-	smoothAnim = udTable.GetBool("smoothAnim", false);
 	canLoopbackAttack = udTable.GetBool("canLoopbackAttack", false);
-	canCrash = udTable.GetBool("canCrash", true);
 	levelGround = udTable.GetBool("levelGround", true);
 	strafeToAttack = udTable.GetBool("strafeToAttack", false);
 
@@ -736,23 +707,19 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 		}
 	}
 
-	LuaTable sfxTable = udTable.SubTable("SFXTypes");
-	LuaTable expTable = sfxTable.SubTable("explosionGenerators");
+	const LuaTable& sfxTable = udTable.SubTable("SFXTypes");
+	const LuaTable& modelCEGTable = sfxTable.SubTable(     "explosionGenerators");
+	const LuaTable& pieceCEGTable = sfxTable.SubTable("pieceExplosionGenerators");
 
-	for (int expNum = 1; expNum <= 1024; expNum++) {
-		std::string expsfx = expTable.GetString(expNum, "");
+	std::vector<int> modelCEGKeys; modelCEGTable.GetKeys(modelCEGKeys);
+	std::vector<int> pieceCEGKeys; pieceCEGTable.GetKeys(pieceCEGKeys);
 
-		if (expsfx == "") {
-			break;
-		} else {
-			sfxExplGenNames.push_back(expsfx);
-		}
+	for (unsigned int n = 0; n < modelCEGKeys.size(); n++) {
+		modelCEGTags.push_back(modelCEGTable.GetString(modelCEGKeys[n], ""));
+	}
+	for (unsigned int n = 0; n < pieceCEGKeys.size(); n++) {
+		pieceCEGTags.push_back(pieceCEGTable.GetString(pieceCEGKeys[n], ""));
 	}
-
-	// we use range in a modulo operation, so it needs to be >= 1
-	pieceTrailCEGTag = udTable.GetString("pieceTrailCEGTag", "");
-	pieceTrailCEGRange = udTable.GetInt("pieceTrailCEGRange", 1);
-	pieceTrailCEGRange = std::max(pieceTrailCEGRange, 1);
 
 	// custom parameters table
 	udTable.SubTable("customParams").GetMap(customParams);
@@ -761,7 +728,7 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 
 UnitDef::~UnitDef()
 {
-	for (int u = 0; u < 4; u++) {
+	for (int u = 0; u < NUM_FACINGS; u++) {
 		yardmaps[u].clear();
 	}
 
@@ -774,24 +741,29 @@ UnitDef::~UnitDef()
 	delete collisionVolume;
 	collisionVolume = NULL;
 
-	for (std::vector<CExplosionGenerator*>::iterator it = sfxExplGens.begin(); it != sfxExplGens.end(); ++it) {
-		delete *it;
+	for (std::vector<IExplosionGenerator*>::iterator it = sfxExplGens.begin(); it != sfxExplGens.end(); ++it) {
+		explGenHandler->UnloadGenerator(*it);
+		*it = NULL;
 	}
 }
 
 
 S3DModel* UnitDef::LoadModel() const
 {
-	// not exactly kosher, but...
-	UnitDef* udef = const_cast<UnitDef*>(this);
-
-	if (udef->modelDef.model == NULL) {
-		udef->modelDef.model = modelParser->Load3DModel(udef->modelDef.modelPath, udef->modelCenterOffset);
-		udef->modelDef.modelTextures["tex1"] = udef->modelDef.model->tex1;
-		udef->modelDef.modelTextures["tex2"] = udef->modelDef.model->tex2;
+	if (this->modelDef.model == NULL) {
+		this->modelDef.model = modelParser->Load3DModel(this->modelDef.modelPath, this->modelCenterOffset);
+		this->modelDef.modelTextures["tex1"] = this->modelDef.model->tex1;
+		this->modelDef.modelTextures["tex2"] = this->modelDef.model->tex2;
+	} else {
+		eventHandler.LoadedModelRequested();
 	}
 
-	return (udef->modelDef.model);
+	return (this->modelDef.model);
+}
+
+float UnitDef::GetModelRadius() const
+{
+	return (LoadModel()->radius);
 }
 
 
@@ -820,34 +792,33 @@ void UnitDef::ParseWeaponsTable(const LuaTable& weaponsTable)
 
 		while (weapons.size() < w) {
 			if (!noWeaponDef) {
-				logOutput.Print("Error: Spring requires a NOWEAPON weapon type "
-				                "to be present as a placeholder for missing weapons");
+				LOG_L(L_ERROR, "Spring requires a NOWEAPON weapon type "
+						"to be present as a placeholder for missing weapons");
 				break;
 			} else {
-				weapons.push_back(UnitDef::UnitDefWeapon());
+				weapons.push_back(UnitDefWeapon());
 				weapons.back().def = noWeaponDef;
 			}
 		}
 
-		const string badTarget = wTable.GetString("badTargetCategory", "");
-		unsigned int btc = CCategoryHandler::Instance()->GetCategories(badTarget);
+		const string& badTarget = wTable.GetString("badTargetCategory", "");
+		const string& onlyTarget = wTable.GetString("onlyTargetCategory", "");
 
-		const string onlyTarget = wTable.GetString("onlyTargetCategory", "");
-		unsigned int otc;
-		if (onlyTarget.empty()) {
-			otc = 0xffffffff;
-		} else {
+		unsigned int btc = CCategoryHandler::Instance()->GetCategories(badTarget);
+		unsigned int otc = 0xffffffff;
+
+		if (!onlyTarget.empty()) {
 			otc = CCategoryHandler::Instance()->GetCategories(onlyTarget);
 		}
 
 		const unsigned int slaveTo = wTable.GetInt("slaveTo", 0);
 
-		float3 mainDir = wTable.GetFloat3("mainDir", float3(1.0f, 0.0f, 0.0f)).SafeNormalize();
+		const float3 mainDir = wTable.GetFloat3("mainDir", float3(1.0f, 0.0f, 0.0f)).SafeNormalize();
 		const float angleDif = cos(wTable.GetFloat("maxAngleDif", 360.0f) * (PI / 360.0f));
 
 		const float fuelUse = wTable.GetFloat("fuelUsage", 0.0f);
 
-		UnitDef::UnitDefWeapon weapon(name, wd, slaveTo, mainDir, angleDif, btc, otc, fuelUse);
+		UnitDefWeapon weapon(name, wd, slaveTo, mainDir, angleDif, btc, otc, fuelUse);
 		weapons.push_back(weapon);
 
 		maxWeaponRange = std::max(maxWeaponRange, wd->range);
@@ -864,9 +835,7 @@ void UnitDef::ParseWeaponsTable(const LuaTable& weaponsTable)
 
 		if (wd->stockpile) {
 			// interceptors have priority
-			if (wd->interceptor        ||
-			    !stockpileWeaponDef ||
-			    !stockpileWeaponDef->interceptor) {
+			if (wd->interceptor || !stockpileWeaponDef || !stockpileWeaponDef->interceptor) {
 				stockpileWeaponDef = wd;
 			}
 		}
@@ -874,74 +843,71 @@ void UnitDef::ParseWeaponsTable(const LuaTable& weaponsTable)
 }
 
 
-void UnitDef::CreateYardMap(std::string yardmapStr)
+
+void UnitDef::CreateYardMap(std::string yardMapStr)
 {
-	StringToLowerInPlace(yardmapStr);
-
-	const int mw = xsize;
-	const int mh = zsize;
-	const int maxIdx = mw * mh;
-
-	// create the yardmaps for each build-facing
-	for (int u = 0; u < 4; u++) {
-		yardmaps[u].resize(maxIdx);
+	if (yardMapStr.empty()) {
+		// if a unit is immobile but does *not* have a yardmap
+		// defined, assume it is not supposed to be a building
+		// (so do not assign a default per facing)
+		return;
 	}
 
-	// Spring resolution's is double that of TA's (so 4 times as much area)
-	unsigned char* originalMap = new unsigned char[maxIdx / 4];
+	StringToLowerInPlace(yardMapStr);
 
-	memset(originalMap, 255, maxIdx / 4);
+	// create the yardmaps for each build-facing
+	// (xsize and zsize are in heightmap units at
+	// this point)
+	for (int u = 0; u < NUM_FACINGS; u++) {
+		yardmaps[u].resize(xsize * zsize);
+	}
 
-	if (!yardmapStr.empty()) {
-		std::string::iterator si = yardmapStr.begin();
+	const unsigned int hxsize = xsize >> 1;
+	const unsigned int hzsize = zsize >> 1;
 
-		for (int y = 0; y < mh / 2; y++) {
-			for (int x = 0; x < mw / 2; x++) {
+	std::vector<unsigned char> yardMap(hxsize * hzsize, 255);
 
-				if (*si == 'g') needGeo = true;
-				else if (*si == 'c') originalMap[x + y * mw / 2] = 1; // blocking (not walkable, not buildable)
-				else if (*si == 'y') originalMap[x + y * mw / 2] = 0; // non-blocking (walkable, buildable)
-			//	else if (*si == 'o') originalMap[x + y * mw / 2] = 2; // walkable, not buildable?
+	unsigned int x = 0;
+	unsigned int z = 0;
 
-				// advance one non-space character (matching the column
-				// <x> we have just advanced) in this row, and skip any
-				// spaces
-				do {
-					si++;
-				} while (si != yardmapStr.end() && *si == ' ');
+	for (unsigned int n = 0; n < yardMapStr.size(); n++) {
+		const unsigned char c = yardMapStr[n];
 
-				if (si == yardmapStr.end()) {
-					// no more chars for remaining colums in this row
-					break;
-				}
-			}
+		if (c == ' ') { continue; }
+		if (c == 'g') { needGeo = true; continue; }
+		if (c == 'c') { yardMap[x + z * hxsize] = 1; }   // blocking (not walkable, not buildable)
+		if (c == 'y') { yardMap[x + z * hxsize] = 0; }   // non-blocking (walkable, buildable)
+	//	if (c == 'o') { yardMap[x + z * hxsize] = 2; }   // walkable, not buildable?
+	//	if (c == 'w') { yardMap[x + z * hxsize] = 3; }   // not walkable, buildable?
 
-			if (si == yardmapStr.end()) {
-				// no more chars for any remaining rows
- 				break;
-			}
- 		}
- 	}
+		x += 1;
+		z += ((x == hxsize)? 1: 0);
+		x %= hxsize;
 
-	for (int y = 0; y < mh; y++) {
-		for (int x = 0; x < mw; x++) {
-			int orgIdx = x / 2 + y / 2 * mw / 2;
-			char orgMapChar = originalMap[orgIdx];
-
-			yardmaps[0][         (x + y * mw)                ] = orgMapChar;
-			yardmaps[1][maxIdx - (mh * (x + 1) - (y + 1) + 1)] = orgMapChar;
-			yardmaps[2][maxIdx - (x + y * mw + 1)            ] = orgMapChar;
-			yardmaps[3][          mh * (x + 1) - (y + 1)     ] = orgMapChar;
+		if (z >= hzsize) {
+			break;
 		}
 	}
 
-	delete[] originalMap;
+	for (unsigned int z = 0; z < zsize; z++) {
+		for (unsigned int x = 0; x < xsize; x++) {
+			const unsigned int yardMapIdx = (x >> 1) + ((z >> 1) * hxsize);
+			const unsigned char yardMapChar = yardMap[yardMapIdx];
+
+			yardmaps[FACING_SOUTH][                  (x + z * xsize)                ] = yardMapChar;
+			yardmaps[FACING_EAST ][(xsize * zsize) - (zsize * (x + 1) - (z + 1) + 1)] = yardMapChar;
+			yardmaps[FACING_NORTH][(xsize * zsize) - (x + z * xsize + 1)            ] = yardMapChar;
+			yardmaps[FACING_WEST ][                   zsize * (x + 1) - (z + 1)     ] = yardMapChar;
+		}
+	}
 }
 
 
+
 void UnitDef::SetNoCost(bool noCost)
 {
 	if (noCost) {
+		// initialized from UnitDefHandler::PushNewUnitDef
 		realMetalCost    = metalCost;
 		realEnergyCost   = energyCost;
 		realMetalUpkeep  = metalUpkeep;
@@ -963,63 +929,44 @@ void UnitDef::SetNoCost(bool noCost)
 }
 
 
-bool UnitDef::IsTerrainHeightOK(const float height) const {
-	return GetAllowedTerrainHeight(height) == height;
-}
 
-float UnitDef::GetAllowedTerrainHeight(float height) const {
-	float maxwd = maxWaterDepth;
-	float minwd = minWaterDepth;
-	if(movedata) {
-		if(movedata->moveType == MoveData::Ship_Move)
-			minwd = movedata->depth;
-		else
-			maxwd = movedata->depth;
-	}
-	height = std::min(height, -minwd);
-	if(!floater && !canhover)
-		height = std::max(-maxwd, height);
-	return height;
-}
+bool UnitDef::IsAllowedTerrainHeight(float rawHeight, float* clampedHeight) const {
+	float maxDepth = this->maxWaterDepth;
+	float minDepth = this->minWaterDepth;
 
-/******************************************************************************/
-
-BuildInfo::BuildInfo(const std::string& name, const float3& p, int facing)
-{
-	def = unitDefHandler->GetUnitDefByName(name);
-	pos = p;
-	buildFacing = abs(facing) % 4;
-}
-
-
-void BuildInfo::FillCmd(Command& c) const
-{
-	c.id = -def->id;
-	c.params.resize(4);
-	c.params[0] = pos.x;
-	c.params[1] = pos.y;
-	c.params[2] = pos.z;
-	c.params[3] = (float) buildFacing;
-}
-
-
-bool BuildInfo::Parse(const Command& c)
-{
-	if (c.params.size() >= 3) {
-		pos = float3(c.params[0],c.params[1],c.params[2]);
-
-		if(c.id < 0) {
-			def = unitDefHandler->GetUnitDefByID(-c.id);
-			buildFacing = 0;
-
-			if (c.params.size() == 4)
-				buildFacing = int(abs(c.params[3])) % 4;
-
-			return true;
+	if (movedata != NULL) {
+		// we are a mobile ground-unit
+		if (movedata->moveType == MoveData::Ship_Move) {
+			minDepth = movedata->depth;
+		} else {
+			maxDepth = movedata->depth;
 		}
 	}
-	return false;
+
+	if (floater || canhover) {
+		// if we are a surface unit, <maxDepth> is irrelevant
+		// (eg. hovercraft may be dropped anywhere over water)
+		maxDepth = +10e6f;
+	}
+
+	if (clampedHeight != NULL) {
+		*clampedHeight = rawHeight;
+		*clampedHeight = std::max(*clampedHeight, -maxDepth);
+		*clampedHeight = std::min(*clampedHeight, -minDepth);
+	}
+
+	// <rawHeight> must lie in the range [-maxDepth, -minDepth]
+	return (rawHeight >= -maxDepth && rawHeight <= -minDepth);
 }
 
 
+
+bool UnitDef::HasBomberWeapon() const {
+	if (weapons.empty()) { return false; }
+	if (weapons[0].def == NULL) { return false; }
+	return
+		weapons[0].def->type == "AircraftBomb" ||
+		weapons[0].def->type == "TorpedoLauncher";
+}
+
 /******************************************************************************/
