@@ -3,11 +3,8 @@
 #ifndef UNITHANDLER_H
 #define UNITHANDLER_H
 
-#include <set>
 #include <vector>
 #include <list>
-#include <stack>
-#include <string>
 
 #include "UnitDef.h"
 #include "UnitSet.h"
@@ -17,20 +14,24 @@ class CUnit;
 class CBuilderCAI;
 class CFeature;
 class CLoadSaveInterface;
+struct BuildInfo;
 
 class CUnitHandler
 {
 	CR_DECLARE(CUnitHandler)
 
 public:
+	CUnitHandler();
+	~CUnitHandler();
+
 	void Update();
 	void DeleteUnit(CUnit* unit);
 	void DeleteUnitNow(CUnit* unit);
-	int AddUnit(CUnit* unit);
-	CUnitHandler(bool serializing=false);
-	void Serialize(creg::ISerializer& s);
+	bool AddUnit(CUnit* unit);
+	void Serialize(creg::ISerializer& s) {}
 	void PostLoad();
-	virtual ~CUnitHandler();
+
+	unsigned int MaxUnits() const { return maxUnits; }
 
 	///< test if a unit can be built at specified position
 	///<   return values for the following is
@@ -40,34 +41,24 @@ public:
 	int TestUnitBuildSquare(
 		const BuildInfo&,
 		CFeature*&,
-		int,
+		int allyteam,
+		bool synced,
 		std::vector<float3>* canbuildpos = NULL,
 		std::vector<float3>* featurepos = NULL,
 		std::vector<float3>* nobuildpos = NULL,
 		const std::vector<Command>* commands = NULL
 	);
 	///< test a single mapsquare for build possibility
-	int TestBuildSquare(const float3& pos, const UnitDef *unitdef,CFeature *&feature, int allyteam);
+	int TestBuildSquare(const float3& pos, const UnitDef *unitdef,CFeature *&feature, int allyteam, bool synced);
 
 	/// Returns true if a unit of type unitID can be built, false otherwise
-	bool CanBuildUnit(const UnitDef* unitdef, int team);
+	bool CanBuildUnit(const UnitDef* unitdef, int team) const;
 
 	void AddBuilderCAI(CBuilderCAI*);
 	void RemoveBuilderCAI(CBuilderCAI*);
-	float GetBuildHeight(float3 pos, const UnitDef* unitdef);
+	float GetBuildHeight(const float3& pos, const UnitDef* unitdef, bool synced = true);
 
-	void LoadSaveUnits(CLoadSaveInterface* file, bool loading);
-	Command GetBuildCommand(float3 pos, float3 dir);
-
-	int MaxUnitsPerTeam() const
-	{
-		return unitsPerTeam;
-	};
-
-	size_t MaxUnits() const
-	{
-		return units.size();
-	};
+	Command GetBuildCommand(const float3& pos, const float3& dir);
 
 
 	// note: negative ID's are implicitly converted
@@ -78,29 +69,19 @@ public:
 	std::vector< std::vector<CUnitSet> > unitsByDefs; ///< units sorted by team and unitDef
 
 	std::list<CUnit*> activeUnits;                    ///< used to get all active units
-	std::vector<int> freeIDs;
 	std::vector<CUnit*> units;                        ///< used to get units from IDs (0 if not created)
-
-	std::vector<CUnit*> toBeRemoved;                  ///< units that will be removed at start of next update
-
-	std::list<CUnit*>::iterator slowUpdateIterator;
-
 	std::list<CBuilderCAI*> builderCAIs;
 
-	float waterDamage;
-
-	float maxUnitRadius; ///< largest radius seen so far
-
-	int lastDamageWarning;
-	int lastCmdDamageWarning;
-
-	bool limitDgun;
-	float dgunRadius;
-
+	float maxUnitRadius;                              ///< largest radius of any unit added so far
 	bool morphUnitToFeature;
 
 private:
-	int unitsPerTeam;
+	std::list<unsigned int> freeUnitIDs;
+	std::vector<CUnit*> unitsToBeRemoved;            ///< units that will be removed at start of next update
+	std::list<CUnit*>::iterator slowUpdateIterator;
+
+	///< global unit-limit (derived from the per-team limit)
+	unsigned int maxUnits;
 };
 
 extern CUnitHandler* uh;
