@@ -1,36 +1,36 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 #include <assert.h>
 #include <algorithm>
 #include "Builder.h"
 #include "Building.h"
 #include "Game/GameHelper.h"
-#include "LogOutput.h"
+#include "Game/GlobalUnsynced.h"
 #include "Lua/LuaRules.h"
 #include "Map/Ground.h"
 #include "Map/MapDamage.h"
 #include "Map/ReadMap.h"
-#include "myMath.h"
-#include "Rendering/GlobalRendering.h"
+#include "System/myMath.h"
 #include "Sim/Features/Feature.h"
 #include "Sim/Features/FeatureHandler.h"
 #include "Sim/Misc/GroundBlockingObjectMap.h"
 #include "Sim/Misc/ModInfo.h"
 #include "Sim/Misc/TeamHandler.h"
+#include "Sim/MoveTypes/MoveType.h"
 #include "Sim/Projectiles/ProjectileHandler.h"
-#include "Sim/Projectiles/Unsynced/GfxProjectile.h"
-#include "Sim/Units/COB/CobInstance.h"
+#include "Sim/Units/Scripts/CobInstance.h"
 #include "Sim/Units/CommandAI/BuilderCAI.h"
 #include "Sim/Units/CommandAI/CommandAI.h"
 #include "Sim/Units/UnitDefHandler.h"
 #include "Sim/Units/UnitHandler.h"
 #include "Sim/Units/UnitLoader.h"
-#include "System/GlobalUnsynced.h"
 #include "System/EventHandler.h"
-#include "System/Sound/IEffectChannel.h"
+#include "System/Log/ILog.h"
+#include "System/Sound/SoundChannels.h"
 #include "System/mmgr.h"
 
+#define PLAY_SOUNDS 1
+
 using std::min;
 using std::max;
 
@@ -50,6 +50,7 @@ CR_REG_METADATA(CBuilder, (
 				CR_MEMBER(curBuild),
 				CR_MEMBER(curCapture),
 				CR_MEMBER(curReclaim),
+				CR_MEMBER(reclaimingUnit),
 				CR_MEMBER(helpTerraform),
 				CR_MEMBER(terraforming),
 				CR_MEMBER(myTerraformLeft),
@@ -81,6 +82,7 @@ CBuilder::CBuilder():
 	curBuild(0),
 	curCapture(0),
 	curReclaim(0),
+	reclaimingUnit(false),
 	helpTerraform(0),
 	terraforming(false),
 	terraformHelp(0),
@@ -111,7 +113,7 @@ void CBuilder::PostLoad()
 }
 
 
-void CBuilder::UnitInit(const UnitDef* def, int team, const float3& position)
+void CBuilder::PreInit(const UnitDef* def, int team, int facing, const float3& position, bool build)
 {
 	range3D = def->buildRange3D;
 	buildDistance = def->buildDistance;
@@ -125,7 +127,7 @@ void CBuilder::UnitInit(const UnitDef* def, int team, const float3& position)
 	captureSpeed   = scale * def->captureSpeed;
 	terraformSpeed = scale * def->terraformSpeed;
 
-	CUnit::UnitInit(def, team, position);
+	CUnit::PreInit(def, team, facing, position, build);
 }
 
 
@@ -133,7 +135,7 @@ void CBuilder::Update()
 {
 	if (!beingBuilt && !stunned) {
 		if (terraforming && inBuildStance) {
-			const float* heightmap = readmap->GetHeightmap();
+			const float* heightmap = readmap->GetCornerHeightMapSynced();
 			assert(!mapDamage->disabled); // The map should not be deformed in the first place.
 			float terraformScale = 0.1;
 
@@ -155,14 +157,14 @@ void CBuilder::Update()
 						curBuild->AddBuildPower(0.0f, this);
 						for (int z = tz1; z <= tz2; z++) {
 							for (int x = tx1; x <= tx2; x++) {
-								int idx = z * (gs->mapx + 1) + x;
+								int idx = z * gs->mapxp1 + x;
 								float ch = heightmap[idx];
 
 								readmap->AddHeight(idx, (curBuild->pos.y - ch) * terraformScale);
 							}
 						}
 
-						if(curBuild->terraformLeft<=0){
+						if (curBuild->terraformLeft<=0) {
 							terraforming=false;
 							mapDamage->RecalcArea(tx1,tx2,tz1,tz2);
 							curBuild->groundLevelled = true;
@@ -186,9 +188,9 @@ void CBuilder::Update()
 
 					for (int z = tz1; z <= tz2; z++) {
 						for (int x = tx1; x <= tx2; x++) {
-							int idx = z * (gs->mapx + 1) + x;
+							int idx = z * gs->mapxp1 + x;
 							float ch = heightmap[idx];
-							float oh = readmap->orgheightmap[idx];
+							float oh = readmap->GetOriginalHeightMapSynced()[idx];
 
 							readmap->AddHeight(idx, (oh - ch) * terraformScale);
 						}
@@ -204,26 +206,26 @@ void CBuilder::Update()
 
 			ScriptDecloak(true);
 
-			CreateNanoParticle(terraformCenter,terraformRadius*0.5f,false);
+			CreateNanoParticle(terraformCenter, terraformRadius * 0.5f, false);
 
 			for (int z = tz1; z <= tz2; z++) {
 				// smooth the borders x
 				for (int x = 1; x <= 3; x++) {
 					if (tx1 - 3 >= 0) {
-						const float ch3 = heightmap[z * (gs->mapx + 1) + tx1    ];
-						const float ch  = heightmap[z * (gs->mapx + 1) + tx1 - x];
-						const float ch2 = heightmap[z * (gs->mapx + 1) + tx1 - 3];
+						const float ch3 = heightmap[z * gs->mapxp1 + tx1    ];
+						const float ch  = heightmap[z * gs->mapxp1 + tx1 - x];
+						const float ch2 = heightmap[z * gs->mapxp1 + tx1 - 3];
 						const float amount = ((ch3 * (3 - x) + ch2 * x) / 3 - ch) * terraformScale;
 
-						readmap->AddHeight(z * (gs->mapx + 1) + tx1 - x, amount);
+						readmap->AddHeight(z * gs->mapxp1 + tx1 - x, amount);
 					}
 					if (tx2 + 3 < gs->mapx) {
-						const float ch3 = heightmap[z * (gs->mapx + 1) + tx2    ];
-						const float ch  = heightmap[z * (gs->mapx + 1) + tx2 + x];
-						const float ch2 = heightmap[z * (gs->mapx + 1) + tx2 + 3];
+						const float ch3 = heightmap[z * gs->mapxp1 + tx2    ];
+						const float ch  = heightmap[z * gs->mapxp1 + tx2 + x];
+						const float ch2 = heightmap[z * gs->mapxp1 + tx2 + 3];
 						const float amount = ((ch3 * (3 - x) + ch2 * x) / 3 - ch) * terraformScale;
 
-						readmap->AddHeight(z * (gs->mapx + 1) + tx2 + x, amount);
+						readmap->AddHeight(z * gs->mapxp1 + tx2 + x, amount);
 					}
 				}
 			}
@@ -231,20 +233,20 @@ void CBuilder::Update()
 				// smooth the borders z
 				for (int x = tx1; x <= tx2; x++) {
 					if (tz1 - 3 >= 0) {
-						const float ch3 = heightmap[(tz1    ) * (gs->mapx + 1) + x];
-						const float ch  = heightmap[(tz1 - z) * (gs->mapx + 1) + x];
-						const float ch2 = heightmap[(tz1 - 3) * (gs->mapx + 1) + x];
+						const float ch3 = heightmap[(tz1    ) * gs->mapxp1 + x];
+						const float ch  = heightmap[(tz1 - z) * gs->mapxp1 + x];
+						const float ch2 = heightmap[(tz1 - 3) * gs->mapxp1 + x];
 						const float adjust = ((ch3 * (3 - z) + ch2 * z) / 3 - ch) * terraformScale;
 
-						readmap->AddHeight((tz1 - z) * (gs->mapx + 1) + x, adjust);
+						readmap->AddHeight((tz1 - z) * gs->mapxp1 + x, adjust);
 					}
 					if (tz2 + 3 < gs->mapy) {
-						const float ch3 = heightmap[(tz2    ) * (gs->mapx + 1) + x];
-						const float ch  = heightmap[(tz2 + z) * (gs->mapx + 1) + x];
-						const float ch2 = heightmap[(tz2 + 3) * (gs->mapx + 1) + x];
+						const float ch3 = heightmap[(tz2    ) * gs->mapxp1 + x];
+						const float ch  = heightmap[(tz2 + z) * gs->mapxp1 + x];
+						const float ch2 = heightmap[(tz2 + 3) * gs->mapxp1 + x];
 						const float adjust = ((ch3 * (3 - z) + ch2 * z) / 3 - ch) * terraformScale;
 
-						readmap->AddHeight((tz2 + z) * (gs->mapx + 1) + x, adjust);
+						readmap->AddHeight((tz2 + z) * gs->mapxp1 + x, adjust);
 					}
 				}
 			}
@@ -254,7 +256,7 @@ void CBuilder::Update()
 				ScriptDecloak(true);
 
 				helpTerraform->terraformHelp += terraformSpeed;
-				CreateNanoParticle(helpTerraform->terraformCenter,helpTerraform->terraformRadius*0.5f,false);
+				CreateNanoParticle(helpTerraform->terraformCenter, helpTerraform->terraformRadius * 0.5f, false);
 			} else {
 				DeleteDeathDependence(helpTerraform, DEPENDENCE_TERRAFORM);
 				helpTerraform=0;
@@ -277,29 +279,31 @@ void CBuilder::Update()
 						adjBuildSpeed = min(unitDef->maxRepairSpeed / 2.0f - curBuild->repairAmount, repairSpeed); // repair
 					}
 
-					if(adjBuildSpeed > 0 && !commandAI->commandQue.empty()
-							&& commandAI->commandQue.front().id == CMD_WAIT) {
+					if (adjBuildSpeed > 0 && !commandAI->commandQue.empty()
+							&& commandAI->commandQue.front().GetID() == CMD_WAIT) {
 						curBuild->AddBuildPower(0, this);
 					} else if (adjBuildSpeed > 0 && curBuild->AddBuildPower(adjBuildSpeed, this)) {
 						CreateNanoParticle(curBuild->midPos, curBuild->radius * 0.5f, false);
 					} else {
-						if(!curBuild->beingBuilt && curBuild->health >= curBuild->maxHealth) {
+						if (!curBuild->beingBuilt && curBuild->health >= curBuild->maxHealth) {
 							StopBuild();
+							inBuildStance = false;
 						}
 					}
 				}
 			}
 		}
-		else if(curReclaim && f3SqDist(curReclaim->pos, pos)<Square(buildDistance+curReclaim->radius) && inBuildStance){
+		else if (curReclaim && f3SqDist(curReclaim->pos, pos)<Square(buildDistance+curReclaim->radius) && inBuildStance) {
 			ScriptDecloak(true);
 
 			if (curReclaim->AddBuildPower(-reclaimSpeed, this)) {
-				CreateNanoParticle(curReclaim->midPos, curReclaim->radius * 0.7f, true);
+				CreateNanoParticle(curReclaim->midPos, curReclaim->radius * 0.7f, true, (reclaimingUnit && curReclaim->team != team));
 			}
 		}
-		else if(curResurrect && f3SqDist(curResurrect->pos, pos)<Square(buildDistance+curResurrect->radius) && inBuildStance){
-			const UnitDef* ud=unitDefHandler->GetUnitDefByName(curResurrect->createdFromUnit);
-			if(ud){
+		else if (curResurrect && f3SqDist(curResurrect->pos, pos)<Square(buildDistance+curResurrect->radius) && inBuildStance) {
+			const UnitDef* ud = curResurrect->udef;
+
+			if (ud) {
 				if ((modInfo.reclaimMethod != 1) && (curResurrect->reclaimLeft < 1)) {
 					// This corpse has been reclaimed a little, need to restore the resources
 					// before we can let the player resurrect it.
@@ -309,18 +313,19 @@ void CBuilder::Update()
 					// Corpse has been restored, begin resurrection
 					if (UseEnergy(ud->energyCost * resurrectSpeed / ud->buildTime * modInfo.resurrectEnergyCostFactor)) {
 						curResurrect->resurrectProgress+=resurrectSpeed/ud->buildTime;
-						CreateNanoParticle(curResurrect->midPos,curResurrect->radius*0.7f,gs->randInt()&1);
+						CreateNanoParticle(curResurrect->midPos, curResurrect->radius * 0.7f, (gs->randInt() & 1));
 					}
-					if(curResurrect->resurrectProgress>1){		//resurrect finished
+					if (curResurrect->resurrectProgress > 1) {
+						// resurrect finished
 						curResurrect->UnBlock();
-						CUnit* u = unitLoader.LoadUnit(curResurrect->createdFromUnit, curResurrect->pos,
+						CUnit* u = unitLoader->LoadUnit(ud, curResurrect->pos,
 													team, false, curResurrect->buildFacing, this);
+
 						if (!this->unitDef->canBeAssisted) {
 							u->soloBuilder = this;
-							u->AddDeathDependence(this);
+							u->AddDeathDependence(this, DEPENDENCE_BUILDER);
 						}
-						u->health*=0.05f;
-						u->lineage = this->lineage;
+						u->health *= 0.05f;
 
 						CBuilderCAI *cai = (CBuilderCAI *)commandAI;
 						for (CUnitSet::iterator it = cai->resurrecters.begin(); it != cai->resurrecters.end(); ++it) {
@@ -328,7 +333,7 @@ void CBuilder::Update()
 							if (bld->commandAI->commandQue.empty())
 								continue;
 							const Command& c = bld->commandAI->commandQue.front();
-							if (c.id != CMD_RESURRECT || c.params.size() != 1)
+							if (c.GetID() != CMD_RESURRECT || c.params.size() != 1)
 								continue;
 							const int cmdFeatureId = (int)c.params[0];
 							if (cmdFeatureId - uh->MaxUnits() == curResurrect->id && teamHandler->Ally(allyteam, bld->allyteam))
@@ -344,8 +349,8 @@ void CBuilder::Update()
 				StopBuild(true);
 			}
 		}
-		else if(curCapture && f3SqDist(curCapture->pos, pos)<Square(buildDistance+curCapture->radius) && inBuildStance){
-			if(curCapture->team!=team){
+		else if (curCapture && f3SqDist(curCapture->pos, pos)<Square(buildDistance+curCapture->radius) && inBuildStance) {
+			if (curCapture->team!=team) {
 
 				float captureProgressTemp = curCapture->captureProgress + 1.0f/(150+curCapture->buildTime/captureSpeed*(curCapture->health+curCapture->maxHealth)/curCapture->maxHealth*0.4f);
 				if (captureProgressTemp >= 1.0f) {
@@ -359,19 +364,14 @@ void CBuilder::Update()
 					teamHandler->Team(team)->energyPull += energyUseScaled;
 				} else {
 					curCapture->captureProgress = captureProgressTemp;
-					CreateNanoParticle(curCapture->midPos,curCapture->radius*0.7f,false);
-					if(curCapture->captureProgress >= 1.0f){
+					CreateNanoParticle(curCapture->midPos, curCapture->radius * 0.7f, false, true);
+					if (curCapture->captureProgress >= 1.0f) {
 						if (!curCapture->ChangeTeam(team, CUnit::ChangeCaptured)) {
 							// capture failed
 							if (team == gu->myTeam) {
-								logOutput.Print("%s: Capture failed, unit type limit reached", unitDef->humanName.c_str());
-								logOutput.SetLastMsgPos(pos);
+								LOG_L(L_WARNING, "%s: Capture failed, unit type limit reached", unitDef->humanName.c_str());
+								eventHandler.LastMessagePosition(pos);
 							}
-						} else {
-							// capture succesful
-							int oldLineage = curCapture->lineage;
-							curCapture->lineage = this->lineage;
-							teamHandler->Team(oldLineage)->LeftLineage(curCapture);
 						}
 						curCapture->captureProgress=0.0f;
 						StopBuild(true);
@@ -389,7 +389,7 @@ void CBuilder::Update()
 
 void CBuilder::SlowUpdate(void)
 {
-	if(terraforming){
+	if (terraforming) {
 		mapDamage->RecalcArea(tx1,tx2,tz1,tz2);
 	}
 	CUnit::SlowUpdate();
@@ -398,16 +398,16 @@ void CBuilder::SlowUpdate(void)
 
 void CBuilder::SetRepairTarget(CUnit* target)
 {
-	if(target==curBuild)
+	if (target==curBuild)
 		return;
 
 	StopBuild(false);
 	TempHoldFire();
 
 	curBuild=target;
-	AddDeathDependence(curBuild);
+	AddDeathDependence(curBuild, DEPENDENCE_BUILD);
 
-	if(!target->groundLevelled) {
+	if (!target->groundLevelled) {
 		//resume levelling the ground
 		tx1 = (int)max((float)0,(target->pos.x - (target->unitDef->xsize*0.5f*SQUARE_SIZE))/SQUARE_SIZE);
 		tx2 = min(gs->mapx,tx1+target->unitDef->xsize);
@@ -425,23 +425,25 @@ void CBuilder::SetRepairTarget(CUnit* target)
 
 void CBuilder::SetReclaimTarget(CSolidObject* target)
 {
-	if(dynamic_cast<CFeature*>(target) && !((CFeature*) target)->def->reclaimable){
+	if (dynamic_cast<CFeature*>(target) && !((CFeature*) target)->def->reclaimable) {
 		return;
 	}
 
-	if(dynamic_cast<CUnit*>(target) && !((CUnit*) target)->unitDef->reclaimable){
+	CUnit* recUnit = dynamic_cast<CUnit*>(target);
+	if (recUnit && !recUnit->unitDef->reclaimable) {
 		return;
 	}
 
-	if(curReclaim==target || this == target){
+	if (curReclaim==target || this == target) {
 		return;
 	}
 
 	StopBuild(false);
 	TempHoldFire();
 
+	reclaimingUnit = !!recUnit;
 	curReclaim=target;
-	AddDeathDependence(curReclaim);
+	AddDeathDependence(curReclaim, DEPENDENCE_RECLAIM);
 
 	SetBuildStanceToward(target->pos);
 }
@@ -449,14 +451,14 @@ void CBuilder::SetReclaimTarget(CSolidObject* target)
 
 void CBuilder::SetResurrectTarget(CFeature* target)
 {
-	if(curResurrect==target || target->createdFromUnit=="")
+	if (curResurrect == target || target->udef == NULL)
 		return;
 
 	StopBuild(false);
 	TempHoldFire();
 
-	curResurrect=target;
-	AddDeathDependence(curResurrect);
+	curResurrect = target;
+	AddDeathDependence(curResurrect, DEPENDENCE_RESURRECT);
 
 	SetBuildStanceToward(target->pos);
 }
@@ -471,7 +473,7 @@ void CBuilder::SetCaptureTarget(CUnit* target)
 	TempHoldFire();
 
 	curCapture = target;
-	AddDeathDependence(curCapture);
+	AddDeathDependence(curCapture, DEPENDENCE_CAPTURE);
 
 	SetBuildStanceToward(target->pos);
 }
@@ -493,15 +495,16 @@ void CBuilder::StartRestore(float3 centerPos, float radius)
 	tz2 = (int)min((float)gs->mapy,(centerPos.z+radius)/SQUARE_SIZE);
 
 	float tcost = 0.0f;
-	const float* heightmap = readmap->GetHeightmap();
+	const float* curHeightMap = readmap->GetCornerHeightMapSynced();
+	const float* orgHeightMap = readmap->GetOriginalHeightMapSynced();
 
 	for (int z = tz1; z <= tz2; z++) {
 		for (int x = tx1; x <= tx2; x++) {
-			float delta = readmap->orgheightmap[z * (gs->mapx + 1) + x] - heightmap[z * (gs->mapx + 1) + x];
+			float delta = orgHeightMap[z * gs->mapxp1 + x] - curHeightMap[z * gs->mapxp1 + x];
 			tcost += fabs(delta);
 		}
 	}
-	myTerraformLeft=tcost;
+	myTerraformLeft = tcost;
 
 	SetBuildStanceToward(centerPos);
 }
@@ -509,15 +512,15 @@ void CBuilder::StartRestore(float3 centerPos, float radius)
 
 void CBuilder::StopBuild(bool callScript)
 {
-	if(curBuild)
+	if (curBuild)
 		DeleteDeathDependence(curBuild, DEPENDENCE_BUILD);
-	if(curReclaim)
+	if (curReclaim)
 		DeleteDeathDependence(curReclaim, DEPENDENCE_RECLAIM);
-	if(helpTerraform)
+	if (helpTerraform)
 		DeleteDeathDependence(helpTerraform, DEPENDENCE_TERRAFORM);
-	if(curResurrect)
+	if (curResurrect)
 		DeleteDeathDependence(curResurrect, DEPENDENCE_RESURRECT);
-	if(curCapture)
+	if (curCapture)
 		DeleteDeathDependence(curCapture, DEPENDENCE_CAPTURE);
 	curBuild=0;
 	curReclaim=0;
@@ -525,22 +528,21 @@ void CBuilder::StopBuild(bool callScript)
 	curResurrect=0;
 	curCapture=0;
 	terraforming=false;
-	if(callScript)
+	if (callScript)
 		script->StopBuilding();
 	ReleaseTempHoldFire();
-//	logOutput.Print("stop build");
 }
 
 
-bool CBuilder::StartBuild(BuildInfo& buildInfo, CFeature*& feature)
+bool CBuilder::StartBuild(BuildInfo& buildInfo, CFeature*& feature, bool& waitstance)
 {
 	StopBuild(false);
 
-	buildInfo.pos = helper->Pos2BuildPos(buildInfo);
+	buildInfo.pos = helper->Pos2BuildPos(buildInfo, true);
 
 	// Pass -1 as allyteam to behave like we have maphack.
 	// This is needed to prevent building on top of cloaked stuff.
-	const int canBuild = uh->TestUnitBuildSquare(buildInfo, feature, -1);
+	const int canBuild = uh->TestUnitBuildSquare(buildInfo, feature, -1, true);
 
 	if (canBuild < 2) {
 		// the ground is blocked at the position we want
@@ -563,7 +565,7 @@ bool CBuilder::StartBuild(BuildInfo& buildInfo, CFeature*& feature)
 
 		if (u != NULL && u->unitDef == buildInfo.def && unitDef->canAssist) {
 			curBuild = u;
-			AddDeathDependence(u);
+			AddDeathDependence(u, DEPENDENCE_BUILD);
 			SetBuildStanceToward(buildInfo.pos);
 			return true;
 		}
@@ -577,11 +579,16 @@ bool CBuilder::StartBuild(BuildInfo& buildInfo, CFeature*& feature)
 	const UnitDef* unitDef = buildInfo.def;
 	SetBuildStanceToward(buildInfo.pos);
 
-	CUnit* b = unitLoader.LoadUnit(buildInfo.def, buildInfo.pos, team,
+	if (!inBuildStance) {
+		waitstance = true;
+		return false;
+	}
+
+	CUnit* b = unitLoader->LoadUnit(buildInfo.def, buildInfo.pos, team,
 	                               true, buildInfo.buildFacing, this);
 
 	// floating structures don't terraform the seabed
-	const float groundheight = ground->GetHeight2(b->pos.x, b->pos.z);
+	const float groundheight = ground->GetHeightReal(b->pos.x, b->pos.z);
 	const bool onWater = (unitDef->floater && groundheight <= 0.0f);
 
 	if (mapDamage->disabled || !unitDef->levelGround || onWater ||
@@ -607,10 +614,9 @@ bool CBuilder::StartBuild(BuildInfo& buildInfo, CFeature*& feature)
 
 	if (!this->unitDef->canBeAssisted) {
 		b->soloBuilder = this;
-		b->AddDeathDependence(this);
+		b->AddDeathDependence(this, DEPENDENCE_BUILDER);
 	}
-	b->lineage = this->lineage;
-	AddDeathDependence(b);
+	AddDeathDependence(b, DEPENDENCE_BUILD);
 	curBuild=b;
 
 	/* The ground isn't going to be terraformed.
@@ -632,17 +638,18 @@ float CBuilder::CalculateBuildTerraformCost(BuildInfo& buildInfo)
 	float3& buildPos=buildInfo.pos;
 
 	float tcost = 0.0f;
-	const float* heightmap = readmap->GetHeightmap();
+	const float* curHeightMap = readmap->GetCornerHeightMapSynced();
+	const float* orgHeightMap = readmap->GetOriginalHeightMapSynced();
 
 	for (int z = tz1; z <= tz2; z++) {
 		for (int x = tx1; x <= tx2; x++) {
-			int idx = z * (gs->mapx + 1) + x;
-			float delta = buildPos.y - heightmap[idx];
+			const int idx = z * gs->mapxp1 + x;
+			float delta = buildPos.y - curHeightMap[idx];
 			float cost;
 			if (delta > 0) {
-				cost = max(3.0f, heightmap[idx]-readmap->orgheightmap[idx] + delta * 0.5f);
+				cost = max(3.0f, curHeightMap[idx] - orgHeightMap[idx] + delta * 0.5f);
 			} else {
-				cost = max(3.0f, readmap->orgheightmap[idx] - heightmap[idx] - delta * 0.5f);
+				cost = max(3.0f, orgHeightMap[idx] - curHeightMap[idx] - delta * 0.5f);
 			}
 			tcost += fabs(delta) * cost;
 		}
@@ -654,23 +661,23 @@ float CBuilder::CalculateBuildTerraformCost(BuildInfo& buildInfo)
 
 void CBuilder::DependentDied(CObject *o)
 {
-	if(o==curBuild){
+	if (o==curBuild) {
 		curBuild=0;
 		StopBuild();
 	}
-	if(o==curReclaim){
+	if (o==curReclaim) {
 		curReclaim=0;
 		StopBuild();
 	}
-	if(o==helpTerraform){
+	if (o==helpTerraform) {
 		helpTerraform=0;
 		StopBuild();
 	}
-	if(o==curResurrect){
+	if (o==curResurrect) {
 		curResurrect=0;
 		StopBuild();
 	}
-	if(o==curCapture){
+	if (o==curCapture) {
 		curCapture=0;
 		StopBuild();
 	}
@@ -692,30 +699,32 @@ void CBuilder::SetBuildStanceToward(float3 pos)
 		script->StartBuilding(ClampRad(h - heading * TAANG2RAD), p - pitch);
 	}
 
-	int soundIdx = unitDef->sounds.build.getRandomIdx();
+	#if (PLAY_SOUNDS == 1)
+	const int soundIdx = unitDef->sounds.build.getRandomIdx();
 	if (soundIdx >= 0) {
 		Channels::UnitReply.PlaySample(
 			unitDef->sounds.build.getID(soundIdx), pos,
 			unitDef->sounds.build.getVolume(soundIdx));
 	}
+	#endif
 }
 
 
 void CBuilder::HelpTerraform(CBuilder* unit)
 {
-	if(helpTerraform==unit)
+	if (helpTerraform==unit)
 		return;
 
 	StopBuild(false);
 
 	helpTerraform=unit;
-	AddDeathDependence(helpTerraform);
+	AddDeathDependence(helpTerraform, DEPENDENCE_TERRAFORM);
 
 	SetBuildStanceToward(unit->terraformCenter);
 }
 
 
-void CBuilder::CreateNanoParticle(float3 goal, float radius, bool inverse)
+void CBuilder::CreateNanoParticle(const float3& goal, float radius, bool inverse, bool highPriority)
 {
 	const int piece = script->QueryNanoPiece();
 
@@ -724,55 +733,12 @@ void CBuilder::CreateNanoParticle(float3 goal, float radius, bool inverse)
 		return;
 #endif
 
-	if (ph->currentNanoParticles < ph->maxNanoParticles && unitDef->showNanoSpray) {
-		float3 relWeaponFirePos = script->GetPiecePos(piece);
-		float3 weaponPos = pos + frontdir * relWeaponFirePos.z + updir * relWeaponFirePos.y + rightdir * relWeaponFirePos.x;
+	const float3 relWeaponFirePos = script->GetPiecePos(piece);
+	const float3 weaponPos = pos
+		+ (frontdir * relWeaponFirePos.z)
+		+ (updir    * relWeaponFirePos.y)
+		+ (rightdir * relWeaponFirePos.x);
 
-		float3 dif = goal - weaponPos;
-		const float l = fastmath::apxsqrt2(dif.SqLength());
-
-		dif /= l;
-		float3 error = gu->usRandVector() * (radius / l);
-		float3 color = unitDef->nanoColor;
-
-		if (globalRendering->teamNanospray) {
-			unsigned char* tcol = teamHandler->Team(team)->color;
-			color = float3(tcol[0] * (1.f / 255.f), tcol[1] * (1.f / 255.f), tcol[2] * (1.f / 255.f));
-		}
-
-		if (inverse) {
-			new CGfxProjectile(weaponPos + (dif + error) * l, -(dif + error) * 3, int(l / 3), color);
-		} else {
-			new CGfxProjectile(weaponPos, (dif + error) * 3, int(l / 3), color);
-		}
-	}
-}
-
-
-void CBuilder::DeleteDeathDependence(CObject* o, DependenceType dep) {
-	switch(dep) {
-		case DEPENDENCE_ATTACKER:
-		case DEPENDENCE_BUILDER:
-		case DEPENDENCE_TARGET:
-		case DEPENDENCE_TRANSPORTEE:
-		case DEPENDENCE_TRANSPORTER:
-			if(o == curBuild || o == curCapture || o == curReclaim || o == curResurrect || o == helpTerraform) return;
-			break;
-		case DEPENDENCE_BUILD:
-			if(o == curCapture || o == curReclaim || o == helpTerraform) return;
-			break;
-		case DEPENDENCE_CAPTURE:
-			if(o == curBuild || o == curReclaim || o == helpTerraform) return;
-			break;
-		case DEPENDENCE_RECLAIM:
-			if(o == curBuild || o == curCapture || o == curResurrect || o == helpTerraform) return;
-			break;
-		case DEPENDENCE_RESURRECT:
-			if(o == curReclaim) return;
-			break;
-		case DEPENDENCE_TERRAFORM:
-			if(o == curBuild || o == curCapture || o == curReclaim) return;
-			break;
-	}
-	CUnit::DeleteDeathDependence(o, dep);
+	// unsynced
+	ph->AddNanoParticle(weaponPos, goal, unitDef, team, radius, inverse, highPriority);
 }
