@@ -5,7 +5,7 @@
 
 #include "Building.h"
 
-class CExtractorBuilding: public CBuilding {
+class CExtractorBuilding : public CBuilding {
 public:
 	CR_DECLARE(CExtractorBuilding);
 	CR_DECLARE_SUB(MetalSquareOfControl);
@@ -24,7 +24,7 @@ public:
 	float GetExtractionRange() const { return extractionRange; }
 	float GetExtractionDepth() const { return extractionDepth; }
 
-	virtual void FinishedBuilding();
+	void FinishedBuilding(bool postInit);
 
 protected:
 	struct MetalSquareOfControl {
