@@ -1,23 +1,24 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 #include "BeamLaser.h"
 #include "Game/GameHelper.h"
+#include "Game/TraceRay.h"
 #include "Sim/Misc/TeamHandler.h"
 #include "Map/Ground.h"
-#include "Matrix44f.h"
+#include "System/Matrix44f.h"
 #include "Sim/Features/FeatureHandler.h"
 #include "Sim/Misc/InterceptHandler.h"
-#include "Sim/MoveTypes/AirMoveType.h"
+#include "Sim/Misc/TeamHandler.h"
+#include "Sim/MoveTypes/StrafeAirMoveType.h"
 #include "Sim/Projectiles/WeaponProjectiles/BeamLaserProjectile.h"
 #include "Sim/Projectiles/WeaponProjectiles/LargeBeamLaserProjectile.h"
 #include "Sim/Projectiles/WeaponProjectiles/LaserProjectile.h"
-#include "Sim/Units/COB/UnitScript.h"
+#include "Sim/Units/Scripts/UnitScript.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitTypes/Building.h"
 #include "PlasmaRepulser.h"
 #include "WeaponDefHandler.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 CR_BIND_DERIVED(CBeamLaser, CWeapon, (NULL));
 
@@ -56,7 +57,7 @@ void CBeamLaser::Update(void)
 
 		if (!onlyForward) {
 			wantedDir = targetPos - weaponPos;
-			wantedDir.Normalize();
+			wantedDir.SafeNormalize();
 		}
 
 		if (!weaponDef->beamburst) {
@@ -119,26 +120,23 @@ bool CBeamLaser::TryTarget(const float3& pos, bool userTarget, CUnit* unit)
 	dir /= length;
 
 	if (!onlyForward) {
-		// skip ground col testing for aircraft
-		float g = ground->LineGroundCol(weaponMuzzlePos, pos);
-		if (g > 0 && g < length * 0.9f)
+		if (!HaveFreeLineOfFire(weaponMuzzlePos, dir, length, unit)) {
 			return false;
+		}
 	}
 
 	const float spread =
 		(accuracy + sprayAngle) *
 		(1.0f - owner->limExperience * weaponDef->ownerExpAccWeight);
 
-	if (avoidFeature && helper->LineFeatureCol(weaponMuzzlePos, dir, length)) {
+	if (avoidFeature && TraceRay::LineFeatureCol(weaponMuzzlePos, dir, length)) {
 		return false;
 	}
-	if (avoidFriendly) {
-		if (helper->TestAllyCone(weaponMuzzlePos, dir, length, spread, owner->allyteam, owner))
-			return false;
+	if (avoidFriendly && TraceRay::TestAllyCone(weaponMuzzlePos, dir, length, spread, owner->allyteam, owner)) {
+		return false;
 	}
-	if (avoidNeutral) {
-		if (helper->TestNeutralCone(weaponMuzzlePos, dir, length, spread, owner))
-			return false;
+	if (avoidNeutral && TraceRay::TestNeutralCone(weaponMuzzlePos, dir, length, spread, owner)) {
+		return false;
 	}
 
 	return true;
@@ -166,24 +164,24 @@ void CBeamLaser::Init(void)
 void CBeamLaser::FireImpl(void)
 {
 	float3 dir;
-	if (onlyForward && dynamic_cast<CAirMoveType*>(owner->moveType)) {
-		// the taairmovetype can't align itself properly, change back when that is fixed
+	if (onlyForward && dynamic_cast<CStrafeAirMoveType*>(owner->moveType)) {
+		// HoverAirMoveType cannot align itself properly, change back when that is fixed
 		dir = owner->frontdir;
 	} else {
 		if (salvoLeft == salvoSize - 1) {
 			dir = targetPos - weaponMuzzlePos;
-			dir.Normalize();
+			dir.SafeNormalize();
 			oldDir = dir;
 		} else if (weaponDef->beamburst) {
 			dir = targetPos - weaponMuzzlePos;
-			dir.Normalize();
+			dir.SafeNormalize();
 		} else {
 			dir = oldDir;
 		}
 	}
 
 	dir += ((salvoError) * (1.0f - owner->limExperience * weaponDef->ownerExpAccWeight));
-	dir.Normalize();
+	dir.SafeNormalize();
 
 	FireInternal(dir, false);
 }
@@ -199,7 +197,7 @@ void CBeamLaser::FireInternal(float3 dir, bool sweepFire)
 		rangeMod = 1.3f;
 	}
 
-	if (owner->directControl) {
+	if (owner->fpsControlPlayer != NULL) {
 		rangeMod = 0.95f;
 	}
 
@@ -212,25 +210,25 @@ void CBeamLaser::FireInternal(float3 dir, bool sweepFire)
 	dir +=
 		((gs->randVector() * sprayAngle *
 		(1.0f - owner->limExperience * weaponDef->ownerExpAccWeight)));
-	dir.Normalize();
+	dir.SafeNormalize();
 
 	bool tryAgain = true;
 
 	// increase range if targets are searched for in a cylinder
 	if (cylinderTargetting > 0.01f) {
-		// const float3 up(0, owner->radius*cylinderTargetting, 0);
-		// const float uplen = up.dot(dir);
-		const float uplen = owner->radius * cylinderTargetting * dir.y;
-		maxLength = math::sqrt(maxLength * maxLength + uplen * uplen);
+		const float verticalDist = owner->radius * cylinderTargetting * dir.y;
+		const float maxLengthModSq = maxLength * maxLength + verticalDist * verticalDist;
+
+		maxLength = math::sqrt(maxLengthModSq);
 	}
 
+
 	// increase range if targetting edge of hitsphere
 	if (targetType == Target_Unit && targetUnit && targetBorder != 0) {
-		maxLength += targetUnit->radius * targetBorder;
+		maxLength += (targetUnit->radius * targetBorder);
 	}
 
 
-
 	// unit at the end of the beam
 	CUnit* hitUnit = NULL;
 	CFeature* hitFeature = NULL;
@@ -239,23 +237,9 @@ void CBeamLaser::FireInternal(float3 dir, bool sweepFire)
 	for (int tries = 0; tries < 5 && tryAgain; ++tries) {
 		tryAgain = false;
 
-		const CUnit* hitUnitPtr = hitUnit;
-		const CFeature* hitFeaturePtr = hitFeature;
-		float length = helper->TraceRay(
-			curPos,
-			dir,
-			maxLength - curLength,
-			weaponDef->damages[0],
-			owner,
-			hitUnitPtr,
-			collisionFlags,
-			&hitFeaturePtr
-		);
+		float length = TraceRay::TraceRay(curPos, dir, maxLength - curLength, collisionFlags, owner, hitUnit, hitFeature);
 
-		hitUnit = const_cast<CUnit*>(hitUnitPtr);
-		hitFeature = const_cast<CFeature*>(hitFeaturePtr);
-
-		if (hitUnit != NULL && hitUnit->allyteam == owner->allyteam && sweepFire) {
+		if (hitUnit && teamHandler->AlliedTeams(hitUnit->team, owner->team) && sweepFire) {
 			// never damage friendlies with sweepfire
 			lastFireFrame = 0;
 			return;
@@ -289,7 +273,9 @@ void CBeamLaser::FireInternal(float3 dir, bool sweepFire)
 	if (hitUnit) {
 		if (hitUnit->unitDef->usePieceCollisionVolumes) {
 			// getting the actual piece here is probably overdoing it
-			hitUnit->SetLastAttackedPiece(hitUnit->localmodel->pieces[0], gs->frameNum);
+			// TODO change this if we really need propper flanking bonus support
+			// for beam-lasers
+			hitUnit->SetLastAttackedPiece(hitUnit->localmodel->GetRoot(), gs->frameNum);
 		}
 
 		if (targetBorder > 0) {
@@ -317,26 +303,28 @@ void CBeamLaser::FireInternal(float3 dir, bool sweepFire)
 				weaponDef->dynDamageInverted
 			);
 		}
-		
-		helper->Explosion(
+
+		DamageArray damageArray = weaponDef->dynDamageExp > 0?
+			dynDamages * (hitIntensity * damageMul):
+			weaponDef->damages * (hitIntensity * damageMul);
+		CGameHelper::ExplosionParams params = {
 			hitPos,
-			weaponDef->dynDamageExp > 0?
-				dynDamages * (hitIntensity * damageMul):
-				weaponDef->damages * (hitIntensity * damageMul),
+			dir,
+			damageArray,
+			weaponDef,
+			owner,
+			hitUnit,
+			hitFeature,
 			areaOfEffect,
 			weaponDef->edgeEffectiveness,
 			weaponDef->explosionSpeed,
-			owner,
-			true,
-			1.0f,
-			weaponDef->noExplode || weaponDef->noSelfDamage, /*false*/
-			weaponDef->impactOnly,                           /*false*/
-			weaponDef->explosionGenerator,
-			hitUnit,
-			dir,
-			weaponDef->id,
-			hitFeature
-		);
+			1.0f,                                             // gfxMod
+			weaponDef->impactOnly,
+			weaponDef->noExplode || weaponDef->noSelfDamage,  // ignoreOwner
+			true                                              // damageGround
+		};
+
+		helper->Explosion(params);
 	}
 
 	if (targetUnit) {
