@@ -1,8 +1,11 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "Game/GameHelper.h"
+#include "System/mmgr.h"
+
 #include "LightningCannon.h"
+#include "WeaponDefHandler.h"
+#include "Game/GameHelper.h"
+#include "Game/TraceRay.h"
 #include "Map/Ground.h"
 #include "PlasmaRepulser.h"
 #include "Rendering/Models/3DModel.h"
@@ -10,8 +13,7 @@
 #include "Sim/Misc/InterceptHandler.h"
 #include "Sim/Projectiles/WeaponProjectiles/LightningProjectile.h"
 #include "Sim/Units/Unit.h"
-#include "WeaponDefHandler.h"
-#include "mmgr.h"
+
 
 CR_BIND_DERIVED(CLightningCannon, CWeapon, (NULL));
 
@@ -64,17 +66,17 @@ bool CLightningCannon::TryTarget(const float3& pos, bool userTarget, CUnit* unit
 
 	dir /= length;
 
-	float g = ground->LineGroundCol(weaponMuzzlePos, pos);
-	if (g > 0 && g < length * 0.9f)
+	if (!HaveFreeLineOfFire(weaponMuzzlePos, dir, length, unit)) {
 		return false;
+	}
 
-	if (avoidFeature && helper->LineFeatureCol(weaponMuzzlePos, dir, length)) {
+	if (avoidFeature && TraceRay::LineFeatureCol(weaponMuzzlePos, dir, length)) {
 		return false;
 	}
-	if (avoidFriendly && helper->TestAllyCone(weaponMuzzlePos, dir, length, (accuracy + sprayAngle), owner->allyteam, owner)) {
+	if (avoidFriendly && TraceRay::TestAllyCone(weaponMuzzlePos, dir, length, (accuracy + sprayAngle), owner->allyteam, owner)) {
 		return false;
 	}
-	if (avoidNeutral && helper->TestNeutralCone(weaponMuzzlePos, dir, length, (accuracy + sprayAngle), owner)) {
+	if (avoidNeutral && TraceRay::TestNeutralCone(weaponMuzzlePos, dir, length, (accuracy + sprayAngle), owner)) {
 		return false;
 	}
 
@@ -97,16 +99,7 @@ void CLightningCannon::FireImpl()
 
 	CUnit* u = NULL;
 	CFeature* f = NULL;
-	float r = 0.0f;
-
-	{
-		const CUnit* cu = NULL;
-		const CFeature* cf = NULL;
-		r = helper->TraceRay(weaponMuzzlePos, dir, range, 0, (const CUnit*)owner, cu, collisionFlags, &cf);
-		u = const_cast<CUnit*>(cu);
-		f = const_cast<CFeature*>(cf);
-	}
-
+	float r = TraceRay::TraceRay(weaponMuzzlePos, dir, range, collisionFlags, owner, u, f);
 
 	float3 newDir;
 	CPlasmaRepulser* shieldHit = NULL;
@@ -121,14 +114,14 @@ void CLightningCannon::FireImpl()
 
 	if (u) {
 		if (u->unitDef->usePieceCollisionVolumes) {
-			u->SetLastAttackedPiece(u->localmodel->pieces[0], gs->frameNum);
+			u->SetLastAttackedPiece(u->localmodel->GetRoot(), gs->frameNum);
 		}
 	}
 
 	// Dynamic Damage
-	DamageArray dynDamages;
+	DamageArray damageArray;
 	if (weaponDef->dynDamageExp > 0) {
-		dynDamages = weaponDefHandler->DynamicDamages(
+		damageArray = weaponDefHandler->DynamicDamages(
 			weaponDef->damages,
 			weaponMuzzlePos,
 			targetPos,
@@ -139,27 +132,28 @@ void CLightningCannon::FireImpl()
 			weaponDef->dynDamageMin,
 			weaponDef->dynDamageInverted
 		);
+	} else {
+		damageArray = weaponDef->damages;
 	}
 
-	helper->Explosion(
+	CGameHelper::ExplosionParams params = {
 		weaponMuzzlePos + dir * r,
-		weaponDef->dynDamageExp > 0?
-			dynDamages:
-			weaponDef->damages,
+		dir,
+		damageArray,
+		weaponDef,
+		owner,
+		u,                                                // hitUnit
+		f,                                                // hitFeature
 		areaOfEffect,
 		weaponDef->edgeEffectiveness,
 		weaponDef->explosionSpeed,
-		owner,
-		false,
-		0.5f,
-		weaponDef->noExplode || weaponDef->noSelfDamage, /*true*/
-		weaponDef->impactOnly,                           /*false*/
-		weaponDef->explosionGenerator,
-		u,
-		dir,
-		weaponDef->id,
-		f
-	);
+		0.5f,                                             // gfxMod
+		weaponDef->impactOnly,
+		weaponDef->noExplode || weaponDef->noSelfDamage,  // ignoreOwner
+		false                                             // damageGround
+	};
+
+	helper->Explosion(params);
 
 	new CLightningProjectile(
 		weaponMuzzlePos,
