@@ -1,15 +1,14 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "Game/GameHelper.h"
+#include "Game/TraceRay.h"
 #include "Map/Ground.h"
 #include "MissileLauncher.h"
-#include "Sim/MoveTypes/AirMoveType.h"
+#include "Sim/MoveTypes/StrafeAirMoveType.h"
 #include "Sim/Projectiles/WeaponProjectiles/MissileProjectile.h"
 #include "Sim/Projectiles/WeaponProjectiles/WeaponProjectile.h"
 #include "Sim/Units/Unit.h"
 #include "WeaponDefHandler.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 CR_BIND_DERIVED(CMissileLauncher, CWeapon, (NULL));
 
@@ -70,7 +69,7 @@ void CMissileLauncher::FireImpl()
 	dir.Normalize();
 
 	float3 startSpeed = dir * weaponDef->startvelocity;
-	if (onlyForward && dynamic_cast<CAirMoveType*>(owner->moveType))
+	if (onlyForward && dynamic_cast<CStrafeAirMoveType*>(owner->moveType))
 		startSpeed += owner->speed;
 
 	new CMissileProjectile(weaponMuzzlePos, startSpeed, owner, areaOfEffect,
@@ -110,17 +109,19 @@ bool CMissileLauncher::TryTarget(const float3& pos, bool userTarget, CUnit* unit
 
 		flatdir /= flatlength;
 
-		float linear = dir.y + weaponDef->trajectoryHeight;
-		float quadratic = -weaponDef->trajectoryHeight / flatlength;
-		float gc = ground->TrajectoryGroundCol(weaponMuzzlePos, flatdir, flatlength - 30, linear, quadratic);
+		const float linear = dir.y + weaponDef->trajectoryHeight;
+		const float quadratic = -weaponDef->trajectoryHeight / flatlength;
+		const float gc = ((collisionFlags & Collision::NOGROUND) == 0)?
+			ground->TrajectoryGroundCol(weaponMuzzlePos, flatdir, flatlength - 30, linear, quadratic):
+			-1.0f;
 
-		if (gc > 0)
+		if (gc > 0.0f)
 			return false;
 
-		if (avoidFriendly && helper->TestTrajectoryAllyCone(weaponMuzzlePos, flatdir, flatlength - 30, linear, quadratic, 0, 8, owner->allyteam, owner)) {
+		if (avoidFriendly && TraceRay::TestTrajectoryAllyCone(weaponMuzzlePos, flatdir, flatlength - 30, linear, quadratic, 0, 8, owner->allyteam, owner)) {
 			return false;
 		}
-		if (avoidNeutral && helper->TestTrajectoryNeutralCone(weaponMuzzlePos, flatdir, flatlength - 30, linear, quadratic, 0, 8, owner)) {
+		if (avoidNeutral && TraceRay::TestTrajectoryNeutralCone(weaponMuzzlePos, flatdir, flatlength - 30, linear, quadratic, 0, 8, owner)) {
 			return false;
 		}
 	} else {
@@ -131,10 +132,9 @@ bool CMissileLauncher::TryTarget(const float3& pos, bool userTarget, CUnit* unit
 		dir /= length;
 
 		if (!onlyForward) {
-			// skip ground col testing for aircraft
-			float g = ground->LineGroundCol(weaponMuzzlePos, pos);
-			if (g > 0 && g < length * 0.9f)
+			if (!HaveFreeLineOfFire(weaponMuzzlePos, dir, length, unit)) {
 				return false;
+			}
 		} else {
 			float3 goaldir = pos - owner->pos;
 			goaldir.Normalize();
@@ -142,10 +142,10 @@ bool CMissileLauncher::TryTarget(const float3& pos, bool userTarget, CUnit* unit
 				return false;
 		}
 
-		if (avoidFriendly && helper->TestAllyCone(weaponMuzzlePos, dir, length, (accuracy + sprayAngle), owner->allyteam, owner)) {
+		if (avoidFriendly && TraceRay::TestAllyCone(weaponMuzzlePos, dir, length, (accuracy + sprayAngle), owner->allyteam, owner)) {
 			return false;
 		}
-		if (avoidNeutral && helper->TestNeutralCone(weaponMuzzlePos, dir, length, (accuracy + sprayAngle), owner)) {
+		if (avoidNeutral && TraceRay::TestNeutralCone(weaponMuzzlePos, dir, length, (accuracy + sprayAngle), owner)) {
 			return false;
 		}
 	}
