@@ -1,8 +1,7 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "creg/STL_List.h"
-#include "creg/STL_Set.h"
+#include "System/creg/STL_List.h"
+#include "System/creg/STL_Set.h"
 #include "PlasmaRepulser.h"
 #include "Lua/LuaRules.h"
 #include "Sim/Misc/GlobalSynced.h"
@@ -12,7 +11,7 @@
 #include "Sim/Projectiles/Unsynced/RepulseGfx.h"
 #include "Sim/Projectiles/Unsynced/ShieldPartProjectile.h"
 #include "Sim/Projectiles/WeaponProjectiles/WeaponProjectile.h"
-#include "Sim/Units/COB/UnitScript.h"
+#include "Sim/Units/Scripts/UnitScript.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Weapons/WeaponDefHandler.h"
 #include "Sim/Weapons/Weapon.h"
@@ -126,18 +125,11 @@ void CPlasmaRepulser::Update(void)
 
 		if (drawMe || wasDrawn) {
 			const float colorMix = std::min(1.0f, curPower / std::max(1.0f, weaponDef->shieldPower));
-			const float3 color = (weaponDef->shieldGoodColor * colorMix) +
-													 (weaponDef->shieldBadColor * (1.0f - colorMix));
+			const float3 color = (weaponDef->shieldGoodColor * colorMix)
+					+ (weaponDef->shieldBadColor * (1.0f - colorMix));
 			std::list<CShieldPartProjectile*>::iterator si;
 			for (si = visibleShieldParts.begin(); si != visibleShieldParts.end(); ++si) {
-				CShieldPartProjectile* part = *si;
-				part->centerPos = weaponPos;
-				part->color = color;
-				if (isEnabled) {
-					part->baseAlpha = drawAlpha;
-				} else {
-					part->baseAlpha = 0.0f;
-				}
+				(*si)->Actualize(weaponPos, color, isEnabled ? drawAlpha : 0.0f);
 			}
 		}
 		wasDrawn = drawMe;
@@ -259,14 +251,17 @@ void CPlasmaRepulser::NewProjectile(CWeaponProjectile* p)
 	}
 
 	float3 dir;
-	if (p->targetPos!=ZeroVector) {
-		dir = p->targetPos-p->pos; // assume that it will travel roughly in the direction of the targetpos if it have one
+
+	if (p->targetPos != ZeroVector) {
+		dir = p->targetPos - p->pos; // assume that it will travel roughly in the direction of the targetpos if it have one
 	} else {
 		dir = p->speed;            // otherwise assume speed will hold constant
 	}
-	dir.y = 0;
-	dir.Normalize();
-	float3 dif = owner->pos-p->pos;
+
+	dir.y = 0.0f;
+	dir.SafeNormalize();
+
+	const float3 dif = owner->pos - p->pos;
 
 	if (weaponDef->exteriorShield && (dif.SqLength() < sqRadius)) {
 		return;
@@ -282,7 +277,7 @@ void CPlasmaRepulser::NewProjectile(CWeaponProjectile* p)
 	// but this still doesn't solve anything for e.g. teleporting shields.
 	if (closeDist < Square(radius * 1.5f)) {
 		incoming.push_back(p);
-		AddDeathDependence(p);
+		AddDeathDependence(p, DEPENDENCE_REPULSED);
 	}
 }
 
