@@ -1,14 +1,13 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "Game/GameHelper.h"
+#include "Game/TraceRay.h"
 #include "Map/Ground.h"
 #include "Sim/Misc/InterceptHandler.h"
 #include "Sim/Projectiles/WeaponProjectiles/StarburstProjectile.h"
 #include "Sim/Units/Unit.h"
 #include "StarburstLauncher.h"
 #include "WeaponDefHandler.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 CR_BIND_DERIVED(CStarburstLauncher, CWeapon, (NULL));
 
@@ -41,17 +40,14 @@ void CStarburstLauncher::Update(void)
 
 void CStarburstLauncher::FireImpl()
 {
-	float3 speed(0,weaponDef->startvelocity,0);
-	float maxrange;
+	float3 speed(0.0f, weaponDef->startvelocity, 0.0f);
+
 	if (weaponDef->fixedLauncher) {
 		speed = weaponDir * weaponDef->startvelocity;
-		maxrange = (float)MAX_WORLD_SIZE;
-	} else if (weaponDef->flighttime > 0) {
-		maxrange = (float)MAX_WORLD_SIZE;
-	} else {
-		maxrange = (float)range;
 	}
 
+	const float maxRange = (weaponDef->flighttime > 0 || weaponDef->fixedLauncher)?
+		MAX_PROJECTILE_RANGE: range;
 	const float3 aimError =
 		(gs->randVector() * sprayAngle + salvoError) *
 		(1.0f - owner->limExperience * weaponDef->ownerExpAccWeight);
@@ -59,7 +55,7 @@ void CStarburstLauncher::FireImpl()
 	CStarburstProjectile* p =
 		new CStarburstProjectile(weaponMuzzlePos + float3(0, 2, 0), speed, owner,
 		targetPos, areaOfEffect, projectileSpeed, tracking, (int) uptime, targetUnit,
-		weaponDef, interceptTarget, maxrange, aimError);
+		weaponDef, interceptTarget, maxRange, aimError);
 
 	if (weaponDef->targetable)
 		interceptHandler.AddInterceptTarget(p, targetPos);
@@ -78,11 +74,11 @@ bool CStarburstLauncher::TryTarget(const float3& pos, bool userTarget, CUnit* un
 			return false;
 	}
 
-	if (avoidFriendly && helper->TestAllyCone(weaponMuzzlePos,
+	if (avoidFriendly && TraceRay::TestAllyCone(weaponMuzzlePos,
 		(weaponDef->fixedLauncher? weaponDir: UpVector), 100, 0, owner->allyteam, owner)) {
 		return false;
 	}
-	if (avoidNeutral && helper->TestNeutralCone(weaponMuzzlePos,
+	if (avoidNeutral && TraceRay::TestNeutralCone(weaponMuzzlePos,
 		(weaponDef->fixedLauncher? weaponDir: UpVector), 100, 0, owner)) {
 		return false;
 	}
