@@ -1,14 +1,14 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "creg/STL_List.h"
-#include "float3.h"
-#include "Game/Camera.h"
+#include "System/mmgr.h"
+#include "System/creg/STL_List.h"
+#include "WeaponDefHandler.h"
+#include "Weapon.h"
 #include "Game/GameHelper.h"
 #include "Game/Player.h"
-#include "LogOutput.h"
+#include "Game/TraceRay.h"
+#include "Lua/LuaRules.h"
 #include "Map/Ground.h"
-#include "myMath.h"
 #include "Sim/Misc/CollisionHandler.h"
 #include "Sim/Misc/CollisionVolume.h"
 #include "Sim/Misc/GeometricObjects.h"
@@ -16,17 +16,16 @@
 #include "Sim/Misc/LosHandler.h"
 #include "Sim/Misc/ModInfo.h"
 #include "Sim/Misc/TeamHandler.h"
-#include "Sim/MoveTypes/TAAirMoveType.h"
+#include "Sim/MoveTypes/AAirMoveType.h"
 #include "Sim/Projectiles/WeaponProjectiles/WeaponProjectile.h"
-#include "Sim/Units/COB/CobInstance.h"
+#include "Sim/Units/Scripts/CobInstance.h"
 #include "Sim/Units/CommandAI/CommandAI.h"
 #include "Sim/Units/Unit.h"
-#include "Sync/SyncTracer.h"
-#include "Sound/IEffectChannel.h"
-#include "EventHandler.h"
-#include "WeaponDefHandler.h"
-#include "Weapon.h"
-#include "mmgr.h"
+#include "System/EventHandler.h"
+#include "System/float3.h"
+#include "System/myMath.h"
+#include "System/Sync/SyncTracer.h"
+#include "System/Sound/SoundChannels.h"
 
 CR_BIND_DERIVED(CWeapon, CObject, (NULL));
 
@@ -165,7 +164,7 @@ CWeapon::CWeapon(CUnit* owner):
 	lastErrorVectorUpdate(0),
 	slavedTo(0),
 	mainDir(0,0,1),
-	maxMainDirAngleDif(-1),
+	maxMainDirAngleDif(-1.0f),
 	avoidFriendly(true),
 	avoidFeature(true),
 	avoidNeutral(true),
@@ -288,7 +287,7 @@ void CWeapon::Update()
 			lastRequest = gs->frameNum;
 
 			const float heading = GetHeadingFromVectorF(wantedDir.x, wantedDir.z);
-			const float pitch = asin(wantedDir.dot(owner->updir));
+			const float pitch = math::asin(Clamp(wantedDir.dot(owner->updir), -1.0f, 1.0f));
 			// for COB, this sets anglegood to return value of aim script when it finished,
 			// for Lua, there exists a callout to set the anglegood member.
 			// FIXME: convert CSolidObject::heading to radians too.
@@ -317,8 +316,8 @@ void CWeapon::Update()
 	}
 
 	if ((salvoLeft == 0)
-	    && (!owner->directControl || owner->directControl->mouse1
-	                              || owner->directControl->mouse2)
+	    && (owner->fpsControlPlayer == NULL || owner->fpsControlPlayer->fpsController.mouse1
+	                                        || owner->fpsControlPlayer->fpsController.mouse2)
 	    && (targetType != Target_None)
 	    && angleGood
 	    && subClassReady
@@ -415,8 +414,6 @@ void CWeapon::Update()
 			weaponDir = owner->frontdir * weaponDir.z + owner->updir * weaponDir.y + owner->rightdir * weaponDir.x;
 			weaponDir.SafeNormalize();
 
-	//		logOutput.Print("RelPosFire %f %f %f",relWeaponPos.x,relWeaponPos.y,relWeaponPos.z);
-
 			if (owner->unitDef->decloakOnFire && (owner->scriptCloak <= 2)) {
 				if (owner->isCloaked) {
 					owner->isCloaked = false;
@@ -468,7 +465,7 @@ bool CWeapon::AttackGround(float3 pos, bool userTarget)
 		owner->updir    * relWeaponMuzzlePos.y +
 		owner->rightdir * relWeaponMuzzlePos.x;
 
-	if (weaponMuzzlePos.y < ground->GetHeight2(weaponMuzzlePos.x, weaponMuzzlePos.z)) {
+	if (weaponMuzzlePos.y < ground->GetHeightReal(weaponMuzzlePos.x, weaponMuzzlePos.z)) {
 		// hope that we are underground because we are a popup weapon and will come above ground later
 		weaponMuzzlePos = owner->pos + UpVector * 10;
 	}
@@ -476,8 +473,8 @@ bool CWeapon::AttackGround(float3 pos, bool userTarget)
 	if (!TryTarget(pos,userTarget, 0))
 		return false;
 	if (targetUnit) {
-		DeleteDeathDependence(targetUnit);
-		targetUnit=0;
+		DeleteDeathDependence(targetUnit, DEPENDENCE_TARGETUNIT);
+		targetUnit = NULL;
 	}
 
 	haveUserTarget = userTarget;
@@ -505,7 +502,7 @@ bool CWeapon::AttackUnit(CUnit* unit, bool userTarget)
 		owner->updir    * relWeaponMuzzlePos.y +
 		owner->rightdir * relWeaponMuzzlePos.x;
 
-	if (weaponMuzzlePos.y < ground->GetHeight2(weaponMuzzlePos.x, weaponMuzzlePos.z)) {
+	if (weaponMuzzlePos.y < ground->GetHeightReal(weaponMuzzlePos.x, weaponMuzzlePos.z)) {
 		// hope that we are underground because we are a popup weapon and will come above ground later
 		weaponMuzzlePos = owner->pos + UpVector * 10;
 	}
@@ -533,8 +530,8 @@ bool CWeapon::AttackUnit(CUnit* unit, bool userTarget)
 		return false;
 
 	if (targetUnit) {
-		DeleteDeathDependence(targetUnit);
-		targetUnit = 0;
+		DeleteDeathDependence(targetUnit, DEPENDENCE_TARGETUNIT);
+		targetUnit = NULL;
 	}
 
 	haveUserTarget = userTarget;
@@ -542,7 +539,7 @@ bool CWeapon::AttackUnit(CUnit* unit, bool userTarget)
 	targetUnit = unit;
 	targetPos = tempTargetPos;
 
-	AddDeathDependence(targetUnit);
+	AddDeathDependence(targetUnit, DEPENDENCE_TARGETUNIT);
 	avoidTarget = false;
 	return true;
 }
@@ -551,8 +548,8 @@ bool CWeapon::AttackUnit(CUnit* unit, bool userTarget)
 void CWeapon::HoldFire()
 {
 	if (targetUnit) {
-		DeleteDeathDependence(targetUnit);
-		targetUnit = 0;
+		DeleteDeathDependence(targetUnit, DEPENDENCE_TARGETUNIT);
+		targetUnit = NULL;
 	}
 	targetType = Target_None;
 	haveUserTarget = false;
@@ -560,14 +557,21 @@ void CWeapon::HoldFire()
 
 
 
-inline bool CWeapon::ShouldCheckForNewTarget() const
+inline bool CWeapon::AllowWeaponTargetCheck() const
 {
-	if (weaponDef->noAutoTarget) { return false; }
-	if (owner->fireState < 2)    { return false; }
-	if (haveUserTarget)          { return false; }
+	if (luaRules != NULL) {
+		const int checkAllowed = luaRules->AllowWeaponTargetCheck(owner->id, weaponNum, weaponDef->id);
+
+		if (checkAllowed >= 0) {
+			return checkAllowed;
+		}
+	}
+
+	if (weaponDef->noAutoTarget)                 { return false; }
+	if (owner->fireState < FIRESTATE_FIREATWILL) { return false; }
+	if (haveUserTarget)                          { return false; }
 
 	if (targetType == Target_None) { return true; }
-
 	if (avoidTarget)             { return true; }
 
 	if (targetType == Target_Unit) {
@@ -630,7 +634,7 @@ void CWeapon::SlowUpdate(bool noAutoTargetOverride)
 
 	relWeaponPos = owner->script->GetPiecePos(weaponPiece);
 
-	if (weaponMuzzlePos.y < ground->GetHeight2(weaponMuzzlePos.x, weaponMuzzlePos.z)) {
+	if (weaponMuzzlePos.y < ground->GetHeightReal(weaponMuzzlePos.x, weaponMuzzlePos.z)) {
 		// hope that we are underground because we are a popup weapon and will come above ground later
 		weaponMuzzlePos = owner->pos + UpVector * 10;
 	}
@@ -651,7 +655,7 @@ void CWeapon::SlowUpdate(bool noAutoTargetOverride)
 		if (!haveUserTarget) {
 			// stop firing at neutral targets (unless in FAW mode)
 			// note: HoldFire sets targetUnit to NULL, so recheck
-			if (targetUnit != NULL && targetUnit->neutral && owner->fireState < 3)
+			if (targetUnit != NULL && targetUnit->neutral && owner->fireState <= FIRESTATE_FIREATWILL)
 				HoldFire();
 
 			// stop firing at allied targets
@@ -667,8 +671,8 @@ void CWeapon::SlowUpdate(bool noAutoTargetOverride)
 	if (slavedTo) {
 		// use targets from the thing we are slaved to
 		if (targetUnit) {
-			DeleteDeathDependence(targetUnit);
-			targetUnit = 0;
+			DeleteDeathDependence(targetUnit, DEPENDENCE_TARGETUNIT);
+			targetUnit = NULL;
 		}
 		targetType = Target_None;
 
@@ -682,7 +686,7 @@ void CWeapon::SlowUpdate(bool noAutoTargetOverride)
 				targetUnit = slavedTo->targetUnit;
 				targetPos = tp;
 
-				AddDeathDependence(targetUnit);
+				AddDeathDependence(targetUnit, DEPENDENCE_TARGETUNIT);
 			}
 		} else if (slavedTo->targetType == Target_Pos) {
 			if (TryTarget(slavedTo->targetPos, false, 0)) {
@@ -694,46 +698,55 @@ void CWeapon::SlowUpdate(bool noAutoTargetOverride)
 	}
 
 
-/*		owner->fireState>=2 && !haveUserTarget &&
-	if (!weaponDef->noAutoTarget && !noAutoTargetOverride) {
-		    ((targetType == Target_None) ||
-		     ((targetType == Target_Unit) &&
-		      ((targetUnit->category & badTargetCategory) ||
-		       (targetUnit->neutral && (owner->fireState < 3)))) ||
-		     (gs->frameNum > lastTargetRetry + 65))) {
-*/
-
-	if (!noAutoTargetOverride && ShouldCheckForNewTarget()) {
+	if (!noAutoTargetOverride && AllowWeaponTargetCheck()) {
 		lastTargetRetry = gs->frameNum;
-		std::map<float, CUnit*> targets;
-		helper->GenerateTargets(this, targetUnit, targets);
 
-		for (std::map<float,CUnit*>::iterator ti=targets.begin();ti!=targets.end();++ti) {
-			if (ti->second->neutral && (owner->fireState < 3)) {
-				continue;
-			}
-			if (targetUnit && (ti->second->category & badTargetCategory)) {
+		std::multimap<float, CUnit*> targets;
+		std::multimap<float, CUnit*>::const_iterator nextTargetIt;
+		std::multimap<float, CUnit*>::const_iterator lastTargetIt;
+
+		helper->GenerateWeaponTargets(this, targetUnit, targets);
+
+		if (!targets.empty())
+			lastTargetIt = --targets.end();
+
+		for (nextTargetIt = targets.begin(); nextTargetIt != targets.end(); ++nextTargetIt) {
+			CUnit* nextTargetUnit = nextTargetIt->second;
+
+			if (nextTargetUnit->neutral && (owner->fireState <= FIRESTATE_FIREATWILL)) {
 				continue;
 			}
 
-			float3 tp =
-				ti->second->midPos +
-				errorVector * (weaponDef->targetMoveError * GAME_SPEED * ti->second->speed.Length() * (1.0f - owner->limExperience));
-			const float appHeight = ground->GetApproximateHeight(tp.x, tp.z) + 2.0f;
-
-			if (tp.y < appHeight) {
-				tp.y = appHeight;
-			}
-
-			if (TryTarget(tp, false, ti->second)) {
-				if (targetUnit) {
-					DeleteDeathDependence(targetUnit);
+			// when only one target is available, <nextTarget> can equal <targetUnit>
+			// and we want to attack whether it is in our bad target category or not
+			// (if only bad targets are available and this is the last, just pick it)
+			if (nextTargetUnit != targetUnit && (nextTargetUnit->category & badTargetCategory)) {
+				if (nextTargetIt != lastTargetIt) {
+					continue;
 				}
-				targetType = Target_Unit;
-				targetUnit = ti->second;
-				targetPos = tp;
+			}
 
-				AddDeathDependence(targetUnit);
+			const float weaponLead = weaponDef->targetMoveError * GAME_SPEED * nextTargetUnit->speed.Length();
+			const float weaponError = weaponLead * (1.0f - owner->limExperience);
+
+			float3 nextTargetPos = nextTargetUnit->midPos + (errorVector * weaponError);
+
+			const float appHeight = ground->GetApproximateHeight(nextTargetPos.x, nextTargetPos.z) + 2.0f;
+
+			if (nextTargetPos.y < appHeight) {
+				nextTargetPos.y = appHeight;
+			}
+
+			if (TryTarget(nextTargetPos, false, nextTargetUnit)) {
+				if (targetUnit) {
+					DeleteDeathDependence(targetUnit, DEPENDENCE_TARGETUNIT);
+				}
+
+				targetType = Target_Unit;
+				targetUnit = nextTargetUnit;
+				targetPos = nextTargetPos;
+
+				AddDeathDependence(targetUnit, DEPENDENCE_TARGETUNIT);
 				break;
 			}
 		}
@@ -746,7 +759,7 @@ void CWeapon::SlowUpdate(bool noAutoTargetOverride)
 		}
 	} else {
 		// if we can't target anything, try switching aim point
-		if (useWeaponPosForAim && (useWeaponPosForAim == 1)) {
+		if (useWeaponPosForAim == 1) {
 			useWeaponPosForAim = 0;
 		} else {
 			useWeaponPosForAim = 1;
@@ -757,7 +770,7 @@ void CWeapon::SlowUpdate(bool noAutoTargetOverride)
 void CWeapon::DependentDied(CObject *o)
 {
 	if (o == targetUnit) {
-		targetUnit = 0;
+		targetUnit = NULL;
 		if (targetType == Target_Unit) {
 			targetType = Target_None;
 			haveUserTarget = false;
@@ -767,93 +780,133 @@ void CWeapon::DependentDied(CObject *o)
 		incoming.remove((CWeaponProjectile*) o);
 	}
 
-	if (o == interceptTarget)
-		interceptTarget = 0;
+	if (o == interceptTarget) {
+		interceptTarget = NULL;
+	}
 }
 
-bool CWeapon::TryTarget(const float3& pos, bool userTarget, CUnit* unit)
+bool CWeapon::HaveFreeLineOfFire(const float3& pos, const float3& dir, float length, const CUnit* target) const {
+	CUnit* unit = NULL;
+	CFeature* feature = NULL;
+
+	const float g = TraceRay::TraceRay(pos, dir, length, ~Collision::NOGROUND, owner, unit, feature);
+
+	// true iff ground does not block the ray of length <length> from <pos> along <dir>
+	return ((g <= 0.0f || g >= (length * 0.9f)) || (unit == target));
+}
+
+bool CWeapon::AdjustTargetVectorLength(
+	CUnit* targetUnit,
+	float3& targetPos,
+	float3& targetVec,
+	float3& targetDir)
+const {
+	bool retCode = false;
+	const float tbScale = math::fabsf(targetBorder);
+
+	CollisionVolume* cvOld = targetUnit->collisionVolume;
+	CollisionVolume  cvNew = CollisionVolume(targetUnit->collisionVolume);
+	CollisionQuery   cq;
+
+	// test for "collision" with a temporarily volume
+	// (scaled uniformly by the absolute target-border
+	// factor)
+	cvNew.RescaleAxes(tbScale, tbScale, tbScale);
+	cvNew.SetTestType(CollisionVolume::COLVOL_HITTEST_DISC);
+
+	targetUnit->collisionVolume = &cvNew;
+
+	if (CCollisionHandler::DetectHit(targetUnit, weaponMuzzlePos, ZeroVector, NULL)) {
+		// our weapon muzzle is inside the target unit's volume; this
+		// means we do not need to make any adjustments to targetVec
+		targetVec = ZeroVector;
+	} else {
+		targetDir.SafeNormalize();
+
+		// otherwise, perform a raytrace to find the proper length correction
+		// factor for non-spherical coldet volumes based on the ray's ingress
+		// (for positive TB values) or egress (for negative TB values) position;
+		// this either increases or decreases the length of <targetVec> but does
+		// not change its direction
+		cvNew.SetTestType(CollisionVolume::COLVOL_HITTEST_CONT);
+
+		// make the ray-segment long enough so it can reach the far side of the
+		// scaled collision volume (helps to ensure a ray-intersection is found)
+		//
+		// note: ray-intersection is NOT guaranteed if the volume itself has a
+		// non-zero offset, since here we are "shooting" at the target UNIT's
+		// midpoint
+		const float3 targetOffset = targetDir * (cvNew.GetBoundingRadius() * 2.0f);
+		const float3 targetRayPos = targetPos + targetOffset;
+
+		if (CCollisionHandler::DetectHit(targetUnit, weaponMuzzlePos, targetRayPos, &cq)) {
+			if (targetBorder > 0.0f) { targetVec -= (targetDir * ((targetPos - cq.p0).Length())); }
+			if (targetBorder < 0.0f) { targetVec += (targetDir * ((cq.p1 - targetPos).Length())); }
+		}
+
+		retCode = true;
+	}
+
+	targetUnit->collisionVolume = cvOld;
+
+	// true indicates we took the else-branch and targetDir is now normalized
+	return retCode;
+}
+
+bool CWeapon::TryTarget(const float3& tgtPos, bool /*userTarget*/, CUnit* targetUnit)
 {
-	if (unit && !(onlyTargetCategory & unit->category)) {
+	if (targetUnit && !(onlyTargetCategory & targetUnit->category)) {
 		return false;
 	}
 
-	if (unit && ((unit->isDead  && (modInfo.fireAtKilled   == 0)) ||
-	            (unit->crashing && (modInfo.fireAtCrashing == 0)))) {
+	if (targetUnit && ((targetUnit->isDead   && (modInfo.fireAtKilled   == 0)) ||
+	                   (targetUnit->crashing && (modInfo.fireAtCrashing == 0)))) {
 		return false;
 	}
 	if (weaponDef->stockpile && !numStockpiled) {
 		return false;
 	}
 
+	float3 targetPos = tgtPos;
+	float3 targetVec = targetPos - weaponMuzzlePos;
+	float3 targetDir = targetVec;
 
-	float3 dif = pos - weaponMuzzlePos;
 	float heightDiff = 0.0f; // negative when target below owner
-	const float absTB = streflop::fabsf(targetBorder);
+	float weaponRange = 0.0f; // range modified by heightDiff and cylinderTargetting
+	bool targetDirNormalized = false;
 
-	if (targetBorder != 0.0f && unit) {
-		float3 difDir(dif);
-		difDir.Normalize();
-
-		{
-			CollisionVolume* cvOld = unit->collisionVolume;
-			CollisionVolume  cvNew = CollisionVolume(unit->collisionVolume);
-			CollisionQuery   cq;
-
-			cvNew.RescaleAxes(absTB, absTB, absTB);
-			cvNew.SetTestType(CollisionVolume::COLVOL_HITTEST_DISC);
-
-			unit->collisionVolume = &cvNew;
-
-			if (CCollisionHandler::DetectHit(unit, weaponMuzzlePos, ZeroVector, NULL)) {
-				// weapon inside target unit's volume, no
-				// real need to calculate penetration depth
-				dif = ZeroVector;
-			} else {
-				// raytrace to find the proper correction
-				// factor for non-spherical volumes based
-				// on ingress position
-				cvNew.SetTestType(CollisionVolume::COLVOL_HITTEST_CONT);
-
-				// intersection is not guaranteed if the
-				// volume has an offset, since here we're
-				// shooting at the target's midpoint
-				if (CCollisionHandler::DetectHit(unit, weaponMuzzlePos, pos + (difDir * cvNew.GetBoundingRadius() * 2.0f), &cq)) {
-					if (targetBorder > 0.0f) { dif -= (difDir * ((pos - cq.p0).Length())); }
-					if (targetBorder < 0.0f) { dif += (difDir * ((cq.p1 - pos).Length())); }
-				}
-			}
-
-			unit->collisionVolume = cvOld;
-		}
-
-		heightDiff = (weaponPos.y + dif.y) - owner->pos.y;
-	} else {
-		heightDiff = pos.y - owner->pos.y;
+	if (targetBorder != 0.0f && targetUnit != NULL) {
+		// adjust the length of <targetVec> based on the targetBorder factor
+		targetDirNormalized = AdjustTargetVectorLength(targetUnit, targetPos, targetVec, targetDir);
+		targetPos.y = weaponPos.y + targetVec.y;
 	}
 
+	heightDiff = targetPos.y - owner->pos.y;
 
-	float r;
-	if (!unit || cylinderTargetting < 0.01f) {
-		r = GetRange2D(heightDiff * heightMod);
+	if (targetUnit == NULL || cylinderTargetting < 0.01f) {
+		// check range in a sphere (with extra radius <heightDiff * heightMod>)
+		weaponRange = GetRange2D(heightDiff * heightMod);
 	} else {
-		if (cylinderTargetting * range > fabs(heightDiff) * heightMod) {
-			r = GetRange2D(0);
-		} else {
-			r = 0;
+		// check range in a cylinder (with height <cylinderTargetting * range>)
+		if ((cylinderTargetting * range) > (math::fabsf(heightDiff) * heightMod)) {
+			weaponRange = GetRange2D(0.0f);
 		}
 	}
 
-	if (dif.SqLength2D() >= r * r)
+	if (targetVec.SqLength2D() >= (weaponRange * weaponRange))
 		return false;
 
 	if (maxMainDirAngleDif > -0.999f) {
-		dif.Normalize();
-		float3 modMainDir=owner->frontdir*mainDir.z+owner->rightdir*mainDir.x+owner->updir*mainDir.y;
+		const float3 targetNormDir = targetDirNormalized? targetDir: targetDir.SafeNormalize();
+		const float3 modMainDir =
+			owner->frontdir * mainDir.z +
+			owner->rightdir * mainDir.x +
+			owner->updir    * mainDir.y;
 
-//		geometricObjects->AddLine(weaponPos,weaponPos+modMainDir*50,3,0,16);
-		if(modMainDir.dot(dif)<maxMainDirAngleDif)
+		if (modMainDir.dot(targetNormDir) < maxMainDirAngleDif)
 			return false;
 	}
+
 	return true;
 }
 
