@@ -5,13 +5,13 @@
 
 #include <map>
 
-#include "float3.h"
+#include "System/float3.h"
 #include "Sim/Misc/DamageArray.h"
 #include "Sim/Misc/GuiSoundSet.h"
 
 struct AtlasedTexture;
 class CColorMap;
-class CExplosionGenerator;
+class IExplosionGenerator;
 struct S3DModel;
 
 struct WeaponDef
@@ -20,128 +20,7 @@ private:
 	CR_DECLARE_STRUCT(WeaponDef);
 
 public:
-	WeaponDef()
-		: range(0.0f)
-		, heightmod(0.0f)
-		, accuracy(0.0f)
-		, sprayAngle(0.0f)
-		, movingAccuracy(0.0f)
-		, ownerExpAccWeight(0.0f)
-		, targetMoveError(0.0f)
-		, leadLimit(0.0f)
-		, leadBonus(0.0f)
-		, predictBoost(0.0f)
-		, areaOfEffect(0.0f)
-		, noSelfDamage(false)
-		, fireStarter(0.0f)
-		, edgeEffectiveness(0.0f)
-		, size(0.0f)
-		, sizeGrowth(0.0f)
-		, collisionSize(0.0f)
-		, salvosize(0)
-		, salvodelay(0.0f)
-		, reload(0.0f)
-		, beamtime(0.0f)
-		, beamburst(false)
-		, waterBounce(false)
-		, groundBounce(false)
-		, bounceRebound(0.0f)
-		, bounceSlip(0.0f)
-		, numBounce(0)
-		, maxAngle(0.0f)
-		, restTime(0.0f)
-		, uptime(0.0f)
-		, flighttime(0)
-		, metalcost(0.0f)
-		, energycost(0.0f)
-		, supplycost(0.0f)
-		, projectilespershot(0)
-		, id(0)
-		, tdfId(0)
-		, turret(false)
-		, onlyForward(false)
-		, fixedLauncher(false)
-		, waterweapon(false)
-		, fireSubmersed(false)
-		, submissile(false)
-		, tracks(false)
-		, dropped(false)
-		, paralyzer(false)
-		, impactOnly(false)
-		, noAutoTarget(false)
-		, manualfire(false)
-		, interceptor(0)
-		, targetable(0)
-		, stockpile(false)
-		, coverageRange(0.0f)
-		, stockpileTime(0.0f)
-		, intensity(0.0f)
-		, thickness(0.0f)
-		, laserflaresize(0.0f)
-		, corethickness(0.0f)
-		, duration(0.0f)
-		, lodDistance(0)
-		, falloffRate(0.0f)
-		, graphicsType(0)
-		, soundTrigger(false)
-		, selfExplode(false)
-		, gravityAffected(false)
-		, highTrajectory(0)
-		, myGravity(0.0f)
-		, noExplode(false)
-		, startvelocity(0.0f)
-		, weaponacceleration(0.0f)
-		, turnrate(0.0f)
-		, maxvelocity(0.0f)
-		, projectilespeed(0.0f)
-		, explosionSpeed(0.0f)
-		, onlyTargetCategory(0)
-		, wobble(0.0f)
-		, dance(0.0f)
-		, trajectoryHeight(0.0f)
-		, largeBeamLaser(false)
-		, isShield(false)
-		, shieldRepulser(false)
-		, smartShield(false)
-		, exteriorShield(false)
-		, visibleShield(false)
-		, visibleShieldRepulse(false)
-		, visibleShieldHitFrames(0)
-		, shieldEnergyUse(0.0f)
-		, shieldRadius(0.0f)
-		, shieldForce(0.0f)
-		, shieldMaxSpeed(0.0f)
-		, shieldPower(0.0f)
-		, shieldPowerRegen(0.0f)
-		, shieldPowerRegenEnergy(0.0f)
-		, shieldStartingPower(0.0f)
-		, shieldRechargeDelay(0.0f)
-		, shieldGoodColor(ZeroVector)
-		, shieldBadColor(ZeroVector)
-		, shieldAlpha(0.0f)
-		, shieldInterceptType(0)
-		, interceptedByShieldType(0)
-		, avoidFriendly(false)
-		, avoidFeature(false)
-		, avoidNeutral(false)
-		, targetBorder(0.0f)
-		, cylinderTargetting(0.0f)
-		, minIntensity(0.0f)
-		, heightBoostFactor(0.0f)
-		, proximityPriority(0.0f)
-		, collisionFlags(0)
-		, explosionGenerator(NULL)
-		, bounceExplosionGenerator(NULL)
-		, sweepFire(false)
-		, canAttackGround(false)
-		, cameraShake(0.0f)
-		, dynDamageExp(0.0f)
-		, dynDamageMin(0.0f)
-		, dynDamageRange(0.0f)
-		, dynDamageInverted(false)
-	{}
-
-	WeaponDef(DamageArray damages)
+	WeaponDef(const DamageArray& damages = DamageArray())
 		: range(0.0f)
 		, heightmod(0.0f)
 		, accuracy(0.0f)
@@ -176,7 +55,6 @@ public:
 		, flighttime(0)
 		, metalcost(0.0f)
 		, energycost(0.0f)
-		, supplycost(0.0f)
 		, projectilespershot(0)
 		, id(0)
 		, tdfId(0)
@@ -187,7 +65,6 @@ public:
 		, fireSubmersed(false)
 		, submissile(false)
 		, tracks(false)
-		, dropped(false)
 		, paralyzer(false)
 		, impactOnly(false)
 		, noAutoTarget(false)
@@ -214,7 +91,6 @@ public:
 		, startvelocity(0.0f)
 		, weaponacceleration(0.0f)
 		, turnrate(0.0f)
-		, maxvelocity(0.0f)
 		, projectilespeed(0.0f)
 		, explosionSpeed(0.0f)
 		, onlyTargetCategory(0)
@@ -222,7 +98,7 @@ public:
 		, dance(0.0f)
 		, trajectoryHeight(0.0f)
 		, largeBeamLaser(false)
-		, isShield(false)
+		, isShield(false) //FIXME REMOVE! (this information is/should be saved in the weapontype)
 		, shieldRepulser(false)
 		, smartShield(false)
 		, exteriorShield(false)
@@ -266,11 +142,11 @@ public:
 	~WeaponDef();
 
 	S3DModel* LoadModel();
+	S3DModel* LoadModel() const;
 
 	std::string name;
 	std::string type;
 	std::string description;
-	std::string filename;
 	std::string cegTag;        ///< tag of CEG that projectiles fired by this weapon should use
 
 	GuiSoundSet firesound;
@@ -316,7 +192,6 @@ public:
 
 	float metalcost;
 	float energycost;
-	float supplycost;
 
 	int projectilespershot;
 
@@ -330,12 +205,11 @@ public:
 	bool fireSubmersed;
 	bool submissile;            ///< Lets a torpedo travel above water like it does below water
 	bool tracks;
-	bool dropped;
 	bool paralyzer;             ///< weapon will only paralyze not do real damage
 	bool impactOnly;            ///< The weapon damages by impacting, not by exploding
 
 	bool noAutoTarget;          ///< cant target stuff (for antinuke,dgun)
-	bool manualfire;            ///< use dgun button
+	bool manualfire;            ///< if true, slave us to the ManualFire button
 	int interceptor;            ///< anti nuke
 	int targetable;             ///< nuke (can be shot by interceptor)
 	bool stockpile;
@@ -362,7 +236,6 @@ public:
 	float startvelocity;
 	float weaponacceleration;
 	float turnrate;
-	float maxvelocity;
 
 	float projectilespeed;
 	float explosionSpeed;
@@ -380,6 +253,7 @@ public:
 			, color2(ZeroVector)
 			, model(NULL)
 			, colorMap(NULL)
+			, explosionScar(true)
 			, smokeTrail(false)
 			, beamweapon(false)
 			, hardStop(false)
@@ -410,6 +284,8 @@ public:
 		std::string bounceExpGenTag;
 		CColorMap* colorMap;
 
+		/// TODO: make the scar-type configurable
+		bool explosionScar;
 		bool smokeTrail;
 		bool beamweapon;
 		/// whether the shot should fade out or stop and contract at max range
@@ -489,8 +365,8 @@ public:
 
 	unsigned int collisionFlags;
 
-	CExplosionGenerator* explosionGenerator;        // can be NULL for default explosions
-	CExplosionGenerator* bounceExplosionGenerator;  // called when a projectile bounces
+	IExplosionGenerator* explosionGenerator;        // can be NULL for default explosions
+	IExplosionGenerator* bounceExplosionGenerator;  // called when a projectile bounces
 
 	bool sweepFire;
 	bool canAttackGround;
