@@ -1,26 +1,27 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include <algorithm>
 #include <cctype>
 #include <iostream>
 #include <stdexcept>
 #include "WeaponDefHandler.h"
-#include "Sim/Misc/GlobalConstants.h"
+
 #include "Game/Game.h"
+#include "Game/TraceRay.h"
 #include "Lua/LuaParser.h"
 #include "Rendering/Textures/ColorMap.h"
-#include "Rendering/Textures/TAPalette.h"
-#include "Sim/Misc/DamageArrayHandler.h"
 #include "Sim/Misc/CategoryHandler.h"
-#include "Sim/Projectiles/Projectile.h"
-#include "System/FileSystem/FileHandler.h"
-#include "System/LogOutput.h"
-#include "Sound/ISound.h"
+#include "Sim/Misc/DamageArrayHandler.h"
+#include "Sim/Misc/GlobalConstants.h"
+#include "Sim/Units/Scripts/CobInstance.h"
+#include "System/Log/ILog.h"
 #include "System/Util.h"
 #include "System/Exceptions.h"
+#include "System/myMath.h"
+#include "System/FileSystem/FileHandler.h"
+#include "System/Sound/ISound.h"
 
 using std::min;
 using std::max;
@@ -33,8 +34,6 @@ CWeaponDefHandler* weaponDefHandler = NULL;
 
 CWeaponDefHandler::CWeaponDefHandler()
 {
-	PrintLoadMsg("Loading weapon definitions");
-
 	const LuaTable rootTable = game->defsParser->GetRoot().SubTable("WeaponDefs");
 	if (!rootTable.IsValid()) {
 		throw content_error("Error loading WeaponDefs");
@@ -72,7 +71,6 @@ void CWeaponDefHandler::ParseWeapon(const LuaTable& wdTable, WeaponDef& wd)
 
 	wd.tdfId = wdTable.GetInt("id", 0);
 
-	wd.filename    = wdTable.GetString("filename", "unknown");
 	wd.description = wdTable.GetString("name",     "Weapon");
 	wd.cegTag      = wdTable.GetString("cegTag",   "");
 
@@ -80,17 +78,17 @@ void CWeaponDefHandler::ParseWeapon(const LuaTable& wdTable, WeaponDef& wd)
 	wd.avoidFeature  = wdTable.GetBool("avoidFeature",  true);
 	wd.avoidNeutral  = wdTable.GetBool("avoidNeutral",  false);
 
+	//FIXME may be smarter to merge the collideXYZ tags with avoidXYZ and removing the collisionFlags tag (and move the code into CWeapon)?
 	wd.collisionFlags = 0;
-	const bool collideFriendly = wdTable.GetBool("collideFriendly", true);
-	const bool collideFeature  = wdTable.GetBool("collideFeature",  true);
-	const bool collideNeutral  = wdTable.GetBool("collideNeutral",  true);
-	if (!collideFriendly) { wd.collisionFlags |= COLLISION_NOFRIENDLY; }
-	if (!collideFeature)  { wd.collisionFlags |= COLLISION_NOFEATURE;  }
-	if (!collideNeutral)  { wd.collisionFlags |= COLLISION_NONEUTRAL;  }
+
+	if (!wdTable.GetBool("collideEnemy",    true)) { wd.collisionFlags |= Collision::NOENEMIES;    }
+	if (!wdTable.GetBool("collideFriendly", true)) { wd.collisionFlags |= Collision::NOFRIENDLIES; }
+	if (!wdTable.GetBool("collideFeature",  true)) { wd.collisionFlags |= Collision::NOFEATURES;   }
+	if (!wdTable.GetBool("collideNeutral",  true)) { wd.collisionFlags |= Collision::NONEUTRALS;   }
+	if (!wdTable.GetBool("collideGround",   true)) { wd.collisionFlags |= Collision::NOGROUND;     }
 
 	wd.minIntensity = wdTable.GetFloat("minIntensity", 0.0f);
 
-	wd.dropped  = wdTable.GetBool("dropped", false); //FIXME: unused in the engine?
 	manualBombSettings = wdTable.GetBool("manualBombSettings", false);
 	wd.turret   = wdTable.GetBool("turret",      false);
 	wd.highTrajectory = wdTable.GetInt("highTrajectory", 2);
@@ -104,7 +102,6 @@ void CWeaponDefHandler::ParseWeapon(const LuaTable& wdTable, WeaponDef& wd)
 	wd.fixedLauncher = wdTable.GetBool("fixedLauncher",   false);
 	wd.noExplode     = wdTable.GetBool("noExplode",       false);
 	wd.isShield      = wdTable.GetBool("isShield",        false);
-	wd.maxvelocity   = wdTable.GetFloat("weaponVelocity", 0.0f);
 	wd.beamtime      = wdTable.GetFloat("beamTime",       1.0f);
 	wd.beamburst     = wdTable.GetBool("beamburst",       false);
 
@@ -123,6 +120,7 @@ void CWeaponDefHandler::ParseWeapon(const LuaTable& wdTable, WeaponDef& wd)
 	wd.lodDistance    = wdTable.GetInt("lodDistance",      1000);
 
 	wd.visuals.modelName     = wdTable.GetString("model",       "");
+	wd.visuals.explosionScar = wdTable.GetBool("explosionScar", true);
 	wd.visuals.smokeTrail    = wdTable.GetBool("smokeTrail",    false);
 	wd.visuals.alwaysVisible = wdTable.GetBool("alwaysVisible", false);
 	wd.visuals.sizeDecay     = wdTable.GetFloat("sizeDecay",    0.0f);
@@ -131,22 +129,12 @@ void CWeaponDefHandler::ParseWeapon(const LuaTable& wdTable, WeaponDef& wd)
 	wd.visuals.noGap         = wdTable.GetBool("noGap",         true);
 	wd.visuals.stages        = wdTable.GetInt("stages",         5);
 
-	wd.gravityAffected = wdTable.GetBool("gravityAffected", wd.dropped);
+	wd.gravityAffected = wdTable.GetBool("gravityAffected", false);
 
 	wd.type = wdTable.GetString("weaponType", "Cannon");
 
-//	logOutput.Print("%s as %s",weaponname.c_str(),wd.type.c_str());
-
-	const bool melee = (wd.type == "Melee");
-	wd.targetBorder = wdTable.GetFloat("targetBorder", melee ? 1.0f : 0.0f);
-	if (wd.targetBorder > 1.0f) {
-		logOutput.Print("warning: targetBorder truncated to 1 (was %f)", wd.targetBorder);
-		wd.targetBorder = 1.0f;
-	} else if (wd.targetBorder < -1.0f) {
-		logOutput.Print("warning: targetBorder truncated to -1 (was %f)", wd.targetBorder);
-		wd.targetBorder = -1.0f;
-	}
-	wd.cylinderTargetting = wdTable.GetFloat("cylinderTargetting", melee ? 1.0f : 0.0f);
+	wd.targetBorder = Clamp(wdTable.GetFloat("targetBorder", (wd.type == "Melee")? 1.0f : 0.0f), -1.0f, 1.0f);
+	wd.cylinderTargetting = Clamp(wdTable.GetFloat("cylinderTargetting", (wd.type == "Melee")? 1.0f : 0.0f), 0.0f, 128.0f);
 
 	wd.range = wdTable.GetFloat("range", 10.0f);
 	const float accuracy       = wdTable.GetFloat("accuracy",   0.0f);
@@ -200,14 +188,14 @@ void CWeaponDefHandler::ParseWeapon(const LuaTable& wdTable, WeaponDef& wd)
 		wd.edgeEffectiveness = 0.999f;
 	}
 
-	wd.projectilespeed = wdTable.GetFloat("weaponVelocity", 0.0f) / GAME_SPEED;
+	wd.projectilespeed = std::max(0.01f, wdTable.GetFloat("weaponVelocity", 0.0f) / GAME_SPEED);
 	wd.startvelocity = max(0.01f, wdTable.GetFloat("startVelocity", 0.0f) / GAME_SPEED);
 	wd.weaponacceleration = wdTable.GetFloat("weaponAcceleration", 0.0f) / GAME_SPEED / GAME_SPEED;
 	wd.reload = wdTable.GetFloat("reloadTime", 1.0f);
 	wd.salvodelay = wdTable.GetFloat("burstRate", 0.1f);
 	wd.salvosize = wdTable.GetInt("burst", 1);
 	wd.projectilespershot = wdTable.GetInt("projectiles", 1);
-	wd.maxAngle = wdTable.GetFloat("tolerance", 3000.0f) * 180.0f / 0x7fff;
+	wd.maxAngle = wdTable.GetFloat("tolerance", 3000.0f) * 180.0f / COBSCALEHALF;
 	wd.restTime = 0.0f;
 	wd.metalcost = wdTable.GetFloat("metalPerShot", 0.0f);
 	wd.energycost = wdTable.GetFloat("energyPerShot", 0.0f);
@@ -312,7 +300,7 @@ void CWeaponDefHandler::ParseWeapon(const LuaTable& wdTable, WeaponDef& wd)
 	}
 	wd.interceptedByShieldType = wdTable.GetInt("interceptedByShieldType", defInterceptType);
 
-	wd.wobble = wdTable.GetFloat("wobble", 0.0f) * PI / 0x7fff / 30.0f;
+	wd.wobble = wdTable.GetFloat("wobble", 0.0f) * TAANG2RAD / 30.0f;
 	wd.dance = wdTable.GetFloat("dance", 0.0f) / GAME_SPEED;
 	wd.trajectoryHeight = wdTable.GetFloat("trajectoryHeight", 0.0f);
 
@@ -322,7 +310,7 @@ void CWeaponDefHandler::ParseWeapon(const LuaTable& wdTable, WeaponDef& wd)
 	if (wdTable.GetBool("toAirWeapon", false)) {
 		// fix if we sometime call aircrafts otherwise
 		wd.onlyTargetCategory = CCategoryHandler::Instance()->GetCategories("VTOL");
-		//logOutput.Print("air only weapon %s %i",weaponname.c_str(),wd.onlyTargetCategory);
+		//LOG("air only weapon %s %i", weaponname.c_str(), wd.onlyTargetCategory);
 	}
 
 	wd.largeBeamLaser = wdTable.GetBool("largeBeamLaser", false);
@@ -340,8 +328,6 @@ void CWeaponDefHandler::ParseWeapon(const LuaTable& wdTable, WeaponDef& wd)
 		wd.heightmod = wdTable.GetFloat("heightMod", 0.2f);
 	}
 
-	wd.supplycost = 0.0f;
-
 	wd.onlyForward = !wd.turret && (wd.type != "StarburstLauncher");
 
 	color  = wdTable.GetInt("color",  0);
@@ -354,7 +340,7 @@ void CWeaponDefHandler::ParseWeapon(const LuaTable& wdTable, WeaponDef& wd)
 	wd.uptime = wdTable.GetFloat("weaponTimer", 0.0f);
 	wd.flighttime = wdTable.GetFloat("flightTime", 0) * 32;
 
-	wd.turnrate = wdTable.GetFloat("turnRate", 0.0f) * PI / 0x7fff / 30.0f;
+	wd.turnrate = wdTable.GetFloat("turnRate", 0.0f) * TAANG2RAD / 30.0f;
 
 	if ((wd.type == "AircraftBomb") && !manualBombSettings) {
 		if (wd.reload < 0.5f) {
@@ -610,10 +596,10 @@ DamageArray CWeaponDefHandler::DynamicDamages(DamageArray damages, float3 startP
 			if (damageMin > 0)
 				dynDamages[i] = max(damages[i] * ddmod, dynDamages[i]);
 
-			// div by 0
+			// to prevent div by 0
 			dynDamages[i] = max(0.0001f, dynDamages[i]);
-//			logOutput.Print("D%i: %f (%f) - mod %f", i, dynDamages[i], damages[i], ddmod);
-//			logOutput.Print("F%i: %f - (1 - (1/%f * %f) ^ %f) * %f", i, damages[i], range, travDist, exp, damages[i]);
+//			LOG_L(L_DEBUG, "D%i: %f (%f) - mod %f", i, dynDamages[i], damages[i], ddmod);
+//			LOG_L(L_DEBUG, "F%i: %f - (1 - (1/%f * %f) ^ %f) * %f", i, damages[i], range, travDist, exp, damages[i]);
 		}
 	}
 	else {
@@ -625,8 +611,8 @@ DamageArray CWeaponDefHandler::DynamicDamages(DamageArray damages, float3 startP
 
 			// div by 0
 			dynDamages[i] = max(0.0001f, dynDamages[i]);
-//			logOutput.Print("D%i: %f (%f) - mod %f", i, dynDamages[i], damages[i], ddmod);
-//			logOutput.Print("F%i: (1 - (1/%f * %f) ^ %f) * %f", i, range, travDist, exp, damages[i]);
+//			LOG_L(L_DEBUG, "D%i: %f (%f) - mod %f", i, dynDamages[i], damages[i], ddmod);
+//			LOG_L(L_DEBUG, "F%i: (1 - (1/%f * %f) ^ %f) * %f", i, range, travDist, exp, damages[i]);
 		}
 	}
 	return dynDamages;
