@@ -9,13 +9,9 @@
 #include "Sim/Misc/CommonDefHandler.h"
 #include "Sim/Misc/GuiSoundSet.h"
 #include "WeaponDef.h"
-#include "float3.h"
+#include "System/float3.h"
 
 class LuaTable;
-class CExplosionGenerator;
-
-class CExplosionGeneratorHandler;
-
 
 class CWeaponDefHandler : CommonDefHandler
 {
@@ -43,12 +39,6 @@ private:
 };
 
 
-//not very sweet, but still better than replacing "const WeaponDef" _everywhere_
-inline S3DModel* LoadModel(const WeaponDef* wdef)
-{
-	return const_cast<WeaponDef*>(wdef)->LoadModel();
-}
-
 
 extern CWeaponDefHandler* weaponDefHandler;
 
