@@ -1,16 +1,15 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "BaseNetProtocol.h"
+#include "System/BaseNetProtocol.h"
 
 #include <boost/cstdint.hpp>
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "Game/PlayerStatistics.h"
 #include "Sim/Misc/TeamStatistics.h"
-#include "Net/RawPacket.h"
-#include "Net/PackPacket.h"
-#include "Net/ProtocolDef.h"
+#include "System/Net/RawPacket.h"
+#include "System/Net/PackPacket.h"
+#include "System/Net/ProtocolDef.h"
 #if defined(_MSC_VER)
 #include "System.h" // for uint16_t (and possibly other types)
 #endif
@@ -117,20 +116,29 @@ PacketType CBaseNetProtocol::SendPause(uchar myPlayerNum, uchar bPaused)
 
 
 
-PacketType CBaseNetProtocol::SendAICommand(uchar myPlayerNum, short unitID, int id, uchar options, const std::vector<float>& params)
+PacketType CBaseNetProtocol::SendAICommand(uchar myPlayerNum, unsigned char aiID, short unitID, int id, int aiCommandId, uchar options, const std::vector<float>& params)
 {
-	unsigned size = 11 + params.size() * sizeof(float);
-	PackPacket* packet = new PackPacket(size, NETMSG_AICOMMAND);
-	*packet << static_cast<unsigned short>(size) << myPlayerNum << unitID << id << options << params;
+	int cmdTypeId = NETMSG_AICOMMAND;
+	unsigned size = 12 + (params.size() * sizeof(float));
+	if (aiCommandId != -1) {
+		cmdTypeId = NETMSG_AICOMMAND_TRACKED;
+		size += 4;
+	}
+	PackPacket* packet = new PackPacket(size, cmdTypeId);
+	*packet << static_cast<unsigned short>(size) << myPlayerNum << aiID << unitID << id << options;
+	if (cmdTypeId == NETMSG_AICOMMAND_TRACKED) {
+		*packet << aiCommandId;
+	}
+	*packet << params;
 	return PacketType(packet);
 }
 
-PacketType CBaseNetProtocol::SendAIShare(uchar myPlayerNum, uchar sourceTeam, uchar destTeam, float metal, float energy, const std::vector<short>& unitIDs)
+PacketType CBaseNetProtocol::SendAIShare(uchar myPlayerNum, unsigned char aiID, uchar sourceTeam, uchar destTeam, float metal, float energy, const std::vector<short>& unitIDs)
 {
-	boost::uint16_t totalNumBytes = (1 + sizeof(boost::uint16_t)) + (3 + (2 * sizeof(float)) + (unitIDs.size() * sizeof(short)));
+	boost::uint16_t totalNumBytes = 1 + sizeof(boost::uint16_t) + 1 + 1 + 1 + 1 + (2 * sizeof(float)) + (unitIDs.size() * sizeof(short));
 
 	PackPacket* packet = new PackPacket(totalNumBytes, NETMSG_AISHARE);
-	*packet << totalNumBytes << myPlayerNum << sourceTeam << destTeam << metal << energy << unitIDs;
+	*packet << totalNumBytes << myPlayerNum << aiID << sourceTeam << destTeam << metal << energy << unitIDs;
 	return PacketType(packet);
 }
 
@@ -185,11 +193,11 @@ PacketType CBaseNetProtocol::SendDirectControlUpdate(uchar myPlayerNum, uchar st
 }
 
 
-PacketType CBaseNetProtocol::SendAttemptConnect(const std::string& name, const std::string& passwd, const std::string& version, bool reconnect)
+PacketType CBaseNetProtocol::SendAttemptConnect(const std::string& name, const std::string& passwd, const std::string& version, int netloss, bool reconnect)
 {
-	boost::uint16_t size = 9 + name.size() + passwd.size() + version.size();
+	boost::uint16_t size = 10 + name.size() + passwd.size() + version.size();
 	PackPacket* packet = new PackPacket(size , NETMSG_ATTEMPTCONNECT);
-	*packet << size << NETWORK_VERSION << name << passwd << version << uchar(reconnect);
+	*packet << size << NETWORK_VERSION << name << passwd << version << uchar(reconnect) << uchar(netloss);
 	return PacketType(packet);
 }
 
@@ -221,9 +229,12 @@ PacketType CBaseNetProtocol::SendPlayerStat(uchar myPlayerNum, const PlayerStati
 	return PacketType(packet);
 }
 
-PacketType CBaseNetProtocol::SendGameOver()
+PacketType CBaseNetProtocol::SendGameOver(uchar myPlayerNum, const std::vector<uchar>& winningAllyTeams)
 {
-	return PacketType(new PackPacket(1, NETMSG_GAMEOVER));
+	const unsigned size = (3 * sizeof(uchar)) + (winningAllyTeams.size() * sizeof(uchar));
+	PackPacket* packet = new PackPacket(size, NETMSG_GAMEOVER);
+	*packet << static_cast<uchar>(size) << myPlayerNum << winningAllyTeams;
+	return PacketType(packet);
 }
 
 
@@ -345,12 +356,12 @@ PacketType CBaseNetProtocol::SendTeamDied(uchar myPlayerNum, uchar whichTeam)
 }
 
 PacketType CBaseNetProtocol::SendAICreated(const uchar myPlayerNum,
-                                           const uint  whichSkirmishAI,
+                                           const uchar whichSkirmishAI,
                                            const uchar team,
                                            const std::string& name)
 {
 	// do not hand optimize this math; the compiler will do that
-	const uint size = 1 + 1 + 1 + 4 + 1 + (name.size() + 1);
+	const uint size = 1 + 1 + 1 + 1 + 1 + (name.size() + 1);
 	PackPacket* packet = new PackPacket(size, NETMSG_AI_CREATED);
 	*packet
 		<< static_cast<uchar>(size)
@@ -362,11 +373,11 @@ PacketType CBaseNetProtocol::SendAICreated(const uchar myPlayerNum,
 }
 
 PacketType CBaseNetProtocol::SendAIStateChanged(const uchar myPlayerNum,
-                                                const uint  whichSkirmishAI,
+                                                const uchar whichSkirmishAI,
                                                 const uchar newState)
 {
 	// do not hand optimize this math; the compiler will do that
-	PackPacket* packet = new PackPacket(1 + 1 + 4 + 1, NETMSG_AI_STATE_CHANGED);
+	PackPacket* packet = new PackPacket(1 + 1 + 1 + 1, NETMSG_AI_STATE_CHANGED);
 	*packet << myPlayerNum << whichSkirmishAI << newState;
 	return PacketType(packet);
 }
@@ -388,6 +399,14 @@ PacketType CBaseNetProtocol::SendCreateNewPlayer( uchar playerNum, bool spectato
 
 }
 
+PacketType CBaseNetProtocol::SendCurrentFrameProgress(int frameNum)
+{
+	PackPacket* packet = new PackPacket(5, NETMSG_GAME_FRAME_PROGRESS);
+	*packet << frameNum;
+	return PacketType(packet);
+}
+
+
 
 #ifdef SYNCDEBUG
 PacketType CBaseNetProtocol::SendSdCheckrequest(int frameNum)
@@ -437,7 +456,7 @@ PacketType CBaseNetProtocol::SendSdBlockresponse(uchar myPlayerNum, std::vector<
 
 CBaseNetProtocol::CBaseNetProtocol()
 {
-	netcode::ProtocolDef* proto = netcode::ProtocolDef::instance();
+	netcode::ProtocolDef* proto = netcode::ProtocolDef::GetInstance();
 	// proto->AddType() length parameter:
 	//   > 0:  if its fixed length
 	//   < 0:  means the next x bytes represent the length
@@ -457,6 +476,7 @@ CBaseNetProtocol::CBaseNetProtocol()
 	proto->AddType(NETMSG_PAUSE, 3);
 
 	proto->AddType(NETMSG_AICOMMAND, -2);
+	proto->AddType(NETMSG_AICOMMAND_TRACKED, -2);
 	proto->AddType(NETMSG_AICOMMANDS, -2);
 	proto->AddType(NETMSG_AISHARE, -2);
 
@@ -471,7 +491,7 @@ CBaseNetProtocol::CBaseNetProtocol()
 	proto->AddType(NETMSG_SETSHARE, 11);
 	proto->AddType(NETMSG_SENDPLAYERSTAT, 1);
 	proto->AddType(NETMSG_PLAYERSTAT, 2 + sizeof(PlayerStatistics));
-	proto->AddType(NETMSG_GAMEOVER, 1);
+	proto->AddType(NETMSG_GAMEOVER, -1);
 	proto->AddType(NETMSG_MAPDRAW, -1);
 	proto->AddType(NETMSG_SYNCRESPONSE, 9);
 	proto->AddType(NETMSG_SYSTEMMSG, -2);
@@ -489,7 +509,8 @@ CBaseNetProtocol::CBaseNetProtocol()
 	proto->AddType(NETMSG_CREATE_NEWPLAYER, -2);
 
 	proto->AddType(NETMSG_AI_CREATED, -1);
-	proto->AddType(NETMSG_AI_STATE_CHANGED, 7);
+	proto->AddType(NETMSG_AI_STATE_CHANGED, 4);
+	proto->AddType(NETMSG_GAME_FRAME_PROGRESS,5);
 
 #ifdef SYNCDEBUG
 	proto->AddType(NETMSG_SD_CHKREQUEST, 5);
