@@ -1,8 +1,16 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "EventHandler.h"
+#include "System/EventHandler.h"
+
+#include "Game/UI/LuaUI.h"  // FIXME -- should be moved
+#include "Lua/LuaCallInCheck.h"
 #include "Lua/LuaOpenGL.h"  // FIXME -- should be moved
+#include "Lua/LuaRules.h"
+#include "Lua/LuaGaia.h"
+
+#include "System/Config/ConfigHandler.h"
+#include "System/Platform/Threading.h"
+#include "System/GlobalConfig.h"
 
 using std::string;
 using std::vector;
@@ -36,10 +44,12 @@ CEventHandler::CEventHandler()
 	SETUP_EVENT(GamePreload,   MANAGED_BIT);
 	SETUP_EVENT(GameStart,     MANAGED_BIT);
 	SETUP_EVENT(GameOver,      MANAGED_BIT);
+	SETUP_EVENT(GamePaused,    MANAGED_BIT);
 	SETUP_EVENT(GameFrame,     MANAGED_BIT);
 	SETUP_EVENT(TeamDied,      MANAGED_BIT);
 	SETUP_EVENT(TeamChanged,   MANAGED_BIT);
 	SETUP_EVENT(PlayerChanged, MANAGED_BIT);
+	SETUP_EVENT(PlayerAdded,   MANAGED_BIT);
 	SETUP_EVENT(PlayerRemoved, MANAGED_BIT);
 
 	SETUP_EVENT(UnitCreated,     MANAGED_BIT);
@@ -71,7 +81,10 @@ CEventHandler::CEventHandler()
 	SETUP_EVENT(UnitCloaked,    MANAGED_BIT);
 	SETUP_EVENT(UnitDecloaked,  MANAGED_BIT);
 
-	SETUP_EVENT(UnitMoveFailed, MANAGED_BIT);
+	SETUP_EVENT(UnitUnitCollision,    MANAGED_BIT);
+	SETUP_EVENT(UnitFeatureCollision, MANAGED_BIT);
+	SETUP_EVENT(UnitMoved,            MANAGED_BIT);
+	SETUP_EVENT(UnitMoveFailed,       MANAGED_BIT);
 
 	SETUP_EVENT(FeatureCreated,   MANAGED_BIT);
 	SETUP_EVENT(FeatureDestroyed, MANAGED_BIT);
@@ -87,6 +100,7 @@ CEventHandler::CEventHandler()
 	// unsynced call-ins
 	SETUP_EVENT(Save,           MANAGED_BIT | UNSYNCED_BIT);
 
+	SETUP_EVENT(UnsyncedHeightMapUpdate, MANAGED_BIT | UNSYNCED_BIT);
 	SETUP_EVENT(Update,         MANAGED_BIT | UNSYNCED_BIT);
 
 	SETUP_EVENT(KeyPress,       MANAGED_BIT | UNSYNCED_BIT);
@@ -102,11 +116,14 @@ CEventHandler::CEventHandler()
 	SETUP_EVENT(DefaultCommand, MANAGED_BIT | UNSYNCED_BIT);
 	SETUP_EVENT(CommandNotify,  MANAGED_BIT | UNSYNCED_BIT);
 	SETUP_EVENT(AddConsoleLine, MANAGED_BIT | UNSYNCED_BIT);
+	SETUP_EVENT(LastMessagePosition, MANAGED_BIT | UNSYNCED_BIT);
 	SETUP_EVENT(GroupChanged,   MANAGED_BIT | UNSYNCED_BIT);
 	SETUP_EVENT(GameSetup,      MANAGED_BIT | UNSYNCED_BIT);
 	SETUP_EVENT(WorldTooltip,   MANAGED_BIT | UNSYNCED_BIT);
 	SETUP_EVENT(MapDrawCmd,     MANAGED_BIT | UNSYNCED_BIT);
 
+	SETUP_EVENT(SunChanged,     MANAGED_BIT | UNSYNCED_BIT); //FIXME make synced? (whole dynSun code needs to then)
+
 	SETUP_EVENT(ViewResize,     MANAGED_BIT | UNSYNCED_BIT);
 
 	SETUP_EVENT(DrawGenesis,         MANAGED_BIT | UNSYNCED_BIT);
@@ -131,12 +148,18 @@ CEventHandler::CEventHandler()
 	SETUP_EVENT(RenderProjectileCreated,   MANAGED_BIT | UNSYNCED_BIT);
 	SETUP_EVENT(RenderProjectileDestroyed, MANAGED_BIT | UNSYNCED_BIT);
 
+	SETUP_EVENT(GameProgress,  MANAGED_BIT | UNSYNCED_BIT);
+
 	// unmanaged call-ins
+	SetupEvent("Shutdown", NULL, 0);
 	SetupEvent("RecvLuaMsg", NULL, 0);
 
-	SetupEvent("AICallIn", NULL, UNSYNCED_BIT);
 	SetupEvent("DrawUnit", NULL, UNSYNCED_BIT);
 	SetupEvent("DrawFeature", NULL, UNSYNCED_BIT);
+	SetupEvent("RecvSkirmishAIMessage", NULL, UNSYNCED_BIT);
+
+	// LuaUI
+	SetupEvent("ConfigureLayout", NULL, UNSYNCED_BIT | CONTROL_BIT);
 
 	// LuaRules
 	SetupEvent("CommandFallback",        NULL, CONTROL_BIT);
@@ -154,6 +177,8 @@ CEventHandler::CEventHandler()
 	SetupEvent("MoveCtrlNotify",         NULL, CONTROL_BIT);
 	SetupEvent("UnitPreDamaged",         NULL, CONTROL_BIT);
 	SetupEvent("ShieldPreDamaged",       NULL, CONTROL_BIT);
+	SetupEvent("AllowWeaponTargetCheck", NULL, CONTROL_BIT);
+	SetupEvent("AllowWeaponTarget",      NULL, CONTROL_BIT);
 }
 
 
@@ -333,12 +358,22 @@ void CEventHandler::GameStart()
 }
 
 
-void CEventHandler::GameOver()
+void CEventHandler::GameOver(const std::vector<unsigned char>& winningAllyTeams)
 {
 	const int count = listGameOver.size();
 	for (int i = 0; i < count; i++) {
 		CEventClient* ec = listGameOver[i];
-		ec->GameOver();
+		ec->GameOver(winningAllyTeams);
+	}
+}
+
+
+void CEventHandler::GamePaused(int playerID, bool paused)
+{
+	const int count = listGamePaused.size();
+	for (int i = 0; i < count; i++) {
+		CEventClient* ec = listGamePaused[i];
+		ec->GamePaused(playerID, paused);
 	}
 }
 
@@ -383,6 +418,16 @@ void CEventHandler::PlayerChanged(int playerID)
 }
 
 
+void CEventHandler::PlayerAdded(int playerID)
+{
+	const int count = listPlayerAdded.size();
+	for (int i = 0; i < count; i++) {
+		CEventClient* ec = listPlayerAdded[i];
+		ec->PlayerAdded(playerID);
+	}
+}
+
+
 void CEventHandler::PlayerRemoved(int playerID, int reason)
 {
 	const int count = listPlayerRemoved.size();
@@ -396,7 +441,7 @@ void CEventHandler::PlayerRemoved(int playerID, int reason)
 /******************************************************************************/
 /******************************************************************************/
 
-void CEventHandler::Load(CArchiveBase* archive)
+void CEventHandler::Load(IArchive* archive)
 {
 	const int count = listLoad.size();
 
@@ -410,16 +455,30 @@ void CEventHandler::Load(CArchiveBase* archive)
 }
 
 
+#ifdef USE_GML
+	#define GML_DRAW_CALLIN_SELECTOR() if(!globalConfig->enableDrawCallIns) return
+#else
+	#define GML_DRAW_CALLIN_SELECTOR()
+#endif
+
+#define GML_CALLIN_MUTEXES() \
+	GML_THRMUTEX_LOCK(feat, GML_DRAW); \
+	GML_THRMUTEX_LOCK(unit, GML_DRAW)/*; \
+	GML_THRMUTEX_LOCK(proj, GML_DRAW)*/
+
+
+#define EVENTHANDLER_CHECK(name, ...) \
+	const int count = list ## name.size(); \
+	if (count <= 0) \
+		return __VA_ARGS__; \
+	GML_CALLIN_MUTEXES()
+
+
 void CEventHandler::Update()
 {
-	const int count = listUpdate.size();
+	GML_DRAW_CALLIN_SELECTOR();
 
-	if (count <= 0)
-		return;
-
-	GML_RECMUTEX_LOCK(unit); // Update
-	GML_RECMUTEX_LOCK(feat); // Update
-	GML_RECMUTEX_LOCK(lua); // Update
+	EVENTHANDLER_CHECK(Update);
 
 	for (int i = 0; i < count; i++) {
 		CEventClient* ec = listUpdate[i];
@@ -428,9 +487,77 @@ void CEventHandler::Update()
 }
 
 
+
+void CEventHandler::UpdateUnits(void) { eventBatchHandler->UpdateUnits(); }
+void CEventHandler::UpdateDrawUnits() { eventBatchHandler->UpdateDrawUnits(); }
+void CEventHandler::DeleteSyncedUnits() {
+	eventBatchHandler->DeleteSyncedUnits();
+	GML_STDMUTEX_LOCK(luaui); // DeleteSyncedUnits
+	if (luaUI) luaUI->ExecuteUnitEventBatch();
+}
+
+void CEventHandler::UpdateFeatures(void) { eventBatchHandler->UpdateFeatures(); }
+void CEventHandler::UpdateDrawFeatures() { eventBatchHandler->UpdateDrawFeatures(); }
+void CEventHandler::DeleteSyncedFeatures() {
+	eventBatchHandler->DeleteSyncedFeatures();
+	GML_STDMUTEX_LOCK(luaui); // DeleteSyncedFeatures
+	if (luaUI) luaUI->ExecuteFeatEventBatch();
+}
+
+void CEventHandler::UpdateProjectiles() { eventBatchHandler->UpdateProjectiles(); }
+void CEventHandler::UpdateDrawProjectiles() { eventBatchHandler->UpdateDrawProjectiles(); }
+
+inline void ExecuteAllCallsFromSynced() {
+#if (LUA_MT_OPT & LUA_MUTEX)
+	bool exec;
+	do { // these calls can be chained, need to purge them all
+		exec = false;
+		if (luaRules && luaRules->ExecuteCallsFromSynced())
+			exec = true;
+		if (luaGaia && luaGaia->ExecuteCallsFromSynced())
+			exec = true;
+
+		GML_STDMUTEX_LOCK(luaui); // ExecuteAllCallsFromSynced
+		if (luaUI && luaUI->ExecuteCallsFromSynced())
+			exec = true;
+	} while (exec);
+#endif
+}
+
+void CEventHandler::DeleteSyncedProjectiles() {
+	ExecuteAllCallsFromSynced();
+	eventBatchHandler->DeleteSyncedProjectiles();
+
+	GML_STDMUTEX_LOCK(luaui); // DeleteSyncedProjectiles
+	if (luaUI) luaUI->ExecuteProjEventBatch();
+}
+
+void CEventHandler::UpdateObjects() {
+	eventBatchHandler->UpdateObjects();
+}
+void CEventHandler::DeleteSyncedObjects() {
+	ExecuteAllCallsFromSynced();
+
+	GML_STDMUTEX_LOCK(luaui); // DeleteSyncedObjects
+	if (luaUI) luaUI->ExecuteObjEventBatch();
+}
+
+
+
+void CEventHandler::SunChanged(const float3& sunDir)
+{
+	EVENTHANDLER_CHECK(SunChanged);
+
+	for (int i = 0; i < count; i++) {
+		CEventClient* ec = listSunChanged[i];
+		ec->SunChanged(sunDir);
+	}
+}
+
 void CEventHandler::ViewResize()
 {
-	const int count = listViewResize.size();
+	EVENTHANDLER_CHECK(ViewResize);
+
 	for (int i = 0; i < count; i++) {
 		CEventClient* ec = listViewResize[i];
 		ec->ViewResize();
@@ -438,24 +565,29 @@ void CEventHandler::ViewResize()
 }
 
 
+void CEventHandler::GameProgress(int gameFrame)
+{
+	const int count = listGameProgress.size();
+	for (int i = 0; i < count; i++) {
+		CEventClient* ec = listGameProgress[i];
+		ec->GameProgress(gameFrame);
+	}
+}
+
+
 #define DRAW_CALLIN(name)                         \
-  void CEventHandler:: Draw ## name ()        \
+  void CEventHandler:: Draw ## name ()            \
   {                                               \
-    const int count = listDraw ## name.size();    \
-    if (count <= 0) {                             \
-      return;                                     \
-    }                                             \
-                                                  \
-    GML_RECMUTEX_LOCK(unit); /* DRAW_CALLIN */    \
-    GML_RECMUTEX_LOCK(feat); /* DRAW_CALLIN */    \
-    GML_RECMUTEX_LOCK(lua); /* DRAW_CALLIN */     \
+    GML_DRAW_CALLIN_SELECTOR();                   \
+		                                              \
+    EVENTHANDLER_CHECK(Draw ## name);             \
                                                   \
     LuaOpenGL::EnableDraw ## name ();             \
     listDraw ## name [0]->Draw ## name ();        \
                                                   \
     for (int i = 1; i < count; i++) {             \
       LuaOpenGL::ResetDraw ## name ();            \
-      CEventClient* ec = listDraw ## name [i];      \
+      CEventClient* ec = listDraw ## name [i];    \
       ec-> Draw ## name ();                       \
     }                                             \
                                                   \
@@ -478,8 +610,9 @@ DRAW_CALLIN(InMiniMap)
 
 bool CEventHandler::CommandNotify(const Command& cmd)
 {
+	EVENTHANDLER_CHECK(CommandNotify, false);
+
 	// reverse order, user has the override
-	const int count = listCommandNotify.size();
 	for (int i = (count - 1); i >= 0; i--) {
 		CEventClient* ec = listCommandNotify[i];
 		if (ec->CommandNotify(cmd)) {
@@ -492,8 +625,9 @@ bool CEventHandler::CommandNotify(const Command& cmd)
 
 bool CEventHandler::KeyPress(unsigned short key, bool isRepeat)
 {
+	EVENTHANDLER_CHECK(KeyPress, false);
+
 	// reverse order, user has the override
-	const int count = listKeyPress.size();
 	for (int i = (count - 1); i >= 0; i--) {
 		CEventClient* ec = listKeyPress[i];
 		if (ec->KeyPress(key, isRepeat)) {
@@ -506,8 +640,9 @@ bool CEventHandler::KeyPress(unsigned short key, bool isRepeat)
 
 bool CEventHandler::KeyRelease(unsigned short key)
 {
+	EVENTHANDLER_CHECK(KeyRelease, false);
+
 	// reverse order, user has the override
-	const int count = listKeyRelease.size();
 	for (int i = (count - 1); i >= 0; i--) {
 		CEventClient* ec = listKeyRelease[i];
 		if (ec->KeyRelease(key)) {
@@ -520,8 +655,9 @@ bool CEventHandler::KeyRelease(unsigned short key)
 
 bool CEventHandler::MousePress(int x, int y, int button)
 {
+	EVENTHANDLER_CHECK(MousePress, false);
+
 	// reverse order, user has the override
-	const int count = listMousePress.size();
 	for (int i = (count - 1); i >= 0; i--) {
 		CEventClient* ec = listMousePress[i];
 		if (ec->MousePress(x, y, button)) {
@@ -542,6 +678,8 @@ int CEventHandler::MouseRelease(int x, int y, int button)
 	}
 	else
 	{
+		GML_CALLIN_MUTEXES();
+
 		const int retval = mouseOwner->MouseRelease(x, y, button);
 		mouseOwner = NULL;
 		return retval;
@@ -554,14 +692,18 @@ bool CEventHandler::MouseMove(int x, int y, int dx, int dy, int button)
 	if (mouseOwner == NULL) {
 		return false;
 	}
+
+	GML_CALLIN_MUTEXES();
+
 	return mouseOwner->MouseMove(x, y, dx, dy, button);
 }
 
 
 bool CEventHandler::MouseWheel(bool up, float value)
 {
+	EVENTHANDLER_CHECK(MouseWheel, false);
+
 	// reverse order, user has the override
-	const int count = listMouseWheel.size();
 	for (int i = (count - 1); i >= 0; i--) {
 		CEventClient* ec = listMouseWheel[i];
 		if (ec->MouseWheel(up, value)) {
@@ -573,8 +715,9 @@ bool CEventHandler::MouseWheel(bool up, float value)
 
 bool CEventHandler::JoystickEvent(const std::string& event, int val1, int val2)
 {
+	EVENTHANDLER_CHECK(JoystickEvent, false);
+
 	// reverse order, user has the override
-	const int count = listMouseWheel.size();
 	for (int i = (count - 1); i >= 0; i--) {
 		CEventClient* ec = listMouseWheel[i];
 		if (ec->JoystickEvent(event, val1, val2)) {
@@ -586,8 +729,9 @@ bool CEventHandler::JoystickEvent(const std::string& event, int val1, int val2)
 
 bool CEventHandler::IsAbove(int x, int y)
 {
+	EVENTHANDLER_CHECK(IsAbove, false);
+
 	// reverse order, user has the override
-	const int count = listIsAbove.size();
 	for (int i = (count - 1); i >= 0; i--) {
 		CEventClient* ec = listIsAbove[i];
 		if (ec->IsAbove(x, y)) {
@@ -599,8 +743,9 @@ bool CEventHandler::IsAbove(int x, int y)
 
 string CEventHandler::GetTooltip(int x, int y)
 {
+	EVENTHANDLER_CHECK(GetTooltip, "");
+
 	// reverse order, user has the override
-	const int count = listGetTooltip.size();
 	for (int i = (count - 1); i >= 0; i--) {
 		CEventClient* ec = listGetTooltip[i];
 		const string tt = ec->GetTooltip(x, y);
@@ -612,20 +757,34 @@ string CEventHandler::GetTooltip(int x, int y)
 }
 
 
-bool CEventHandler::AddConsoleLine(const string& msg, const CLogSubsystem& subsystem)
+bool CEventHandler::AddConsoleLine(const std::string& msg, const std::string& section, int level)
 {
-	const int count = listAddConsoleLine.size();
+	EVENTHANDLER_CHECK(AddConsoleLine, false);
+
 	for (int i = 0; i < count; i++) {
 		CEventClient* ec = listAddConsoleLine[i];
-		ec->AddConsoleLine(msg, subsystem);
+		ec->AddConsoleLine(msg, section, level);
 	}
 	return true;
 }
 
 
+void CEventHandler::LastMessagePosition(const float3& pos)
+{
+	EVENTHANDLER_CHECK(LastMessagePosition);
+	//GML_STDMUTEX_LOCK(log); // LastMessagePosition FIXME would this be required?
+
+	for (int i = 0; i < count; i++) {
+		CEventClient* ec = listLastMessagePosition[i];
+		ec->LastMessagePosition(pos);
+	}
+}
+
+
 bool CEventHandler::GroupChanged(int groupID)
 {
-	const int count = listGroupChanged.size();
+	EVENTHANDLER_CHECK(GroupChanged, false);
+
 	for (int i = 0; i < count; i++) {
 		CEventClient* ec = listGroupChanged[i];
 		ec->GroupChanged(groupID);
@@ -638,8 +797,9 @@ bool CEventHandler::GroupChanged(int groupID)
 bool CEventHandler::GameSetup(const string& state, bool& ready,
                                   const map<int, string>& playerStates)
 {
+	EVENTHANDLER_CHECK(GameSetup, false);
+
 	// reverse order, user has the override
-	const int count = listGameSetup.size();
 	for (int i = (count - 1); i >= 0; i--) {
 		CEventClient* ec = listGameSetup[i];
 		if (ec->GameSetup(state, ready, playerStates)) {
@@ -654,8 +814,9 @@ string CEventHandler::WorldTooltip(const CUnit* unit,
                                    const CFeature* feature,
                                    const float3* groundPos)
 {
+	EVENTHANDLER_CHECK(WorldTooltip, "");
+
 	// reverse order, user has the override
-	const int count = listWorldTooltip.size();
 	for (int i = (count - 1); i >= 0; i--) {
 		CEventClient* ec = listWorldTooltip[i];
 		const string tt = ec->WorldTooltip(unit, feature, groundPos);
@@ -671,8 +832,9 @@ bool CEventHandler::MapDrawCmd(int playerID, int type,
                                const float3* pos0, const float3* pos1,
                                    const string* label)
 {
+	EVENTHANDLER_CHECK(MapDrawCmd, false);
+
 	// reverse order, user has the override
-	const int count = listMapDrawCmd.size();
 	for (int i = (count - 1); i >= 0; i--) {
 		CEventClient* ec = listMapDrawCmd[i];
 		if (ec->MapDrawCmd(playerID, type, pos0, pos1, label)) {
@@ -685,3 +847,4 @@ bool CEventHandler::MapDrawCmd(int playerID, int type,
 
 /******************************************************************************/
 /******************************************************************************/
+
