@@ -3,12 +3,14 @@
 #ifndef PACK_PACKET_H
 #define PACK_PACKET_H
 
+#include "RawPacket.h"
+#include "System/SafeVector.h"
+
 #include <string>
 #include <vector>
 #include <assert.h>
 #include <cstring>
 
-#include "RawPacket.h"
 
 namespace netcode
 {
@@ -18,33 +20,50 @@ class PackPacket : public RawPacket
 public:
 	PackPacket(const unsigned length);
 	PackPacket(const unsigned length, unsigned char msgID);
-	
+
 	template <typename T>
 	PackPacket& operator<<(const T& t) {
 		unsigned size = sizeof(T);
-		assert(size + pos <= length);
-		*(T*)(data+pos) = t;
+		assert((size + pos) <= length);
+		*(T*)(data + pos) = t;
 		pos += size;
 		return *this;
-	};
+	}
+
 	PackPacket& operator<<(const std::string& text);
+
 	template <typename element>
 	PackPacket& operator<<(const std::vector<element>& vec) {
 		const size_t size = vec.size()* sizeof(element);
-		assert(size + pos <= length);
+		assert((size + pos) <= length);
 		if (size > 0) {
 			std::memcpy((data+pos), (void*)(&vec[0]), size);
 			pos += size;
 		}
 		return *this;
-	};
-	
-	unsigned char* GetWritingPos() {return data+pos;};
-	
+	}
+
+#ifdef USE_SAFE_VECTOR
+	template <typename element>
+	PackPacket& operator<<(const safe_vector<element>& vec) {
+		const size_t size = vec.size()* sizeof(element);
+		assert((size + pos) <= length);
+		if (size > 0) {
+			std::memcpy((data+pos), (void*)(&vec[0]), size);
+			pos += size;
+		}
+		return *this;
+	}
+#endif
+
+	unsigned char* GetWritingPos() {
+		return data + pos;
+	}
+
 private:
 	unsigned pos;
 };
 
-};
+} // namespace netcode
 
-#endif
+#endif // PACK_PACKET_H
