@@ -5,5 +5,8 @@
 IAudioChannel::IAudioChannel()
 	: volume(1.0f)
 	, enabled(true)
+	, emmitsPerFrame(1000)
+	, emmitsThisFrame(0)
+	, maxConcurrentSources(1024)
 {
 }
