@@ -6,8 +6,9 @@
 #include <string>
 #include <boost/cstdint.hpp>
 
-class BaseCmd;
+class CmdLineParams;
 class CGameController;
+class COffscreenGLContext;
 union SDL_Event;
 
 /**
@@ -22,88 +23,37 @@ public:
 	~SpringApp();
 
 	int Run(int argc, char *argv[]);                //!< Run game loop
+	static void Shutdown();                         //!< Shuts down application
 
 protected:
 	bool Initialize();                              //!< Initialize app
 	void ParseCmdLine();                            //!< Parse command line
 	void Startup();                                 //!< Parses startup data (script etc.) and starts SelectMenu or PreGame
 	bool InitWindow(const char* title);             //!< Initializes window
-	void InitOpenGL();                              //!< Initializes OpenGL
-	void UpdateOldConfigs();                        //!< Forces an update to new config defaults
-	void LoadFonts();                               //!< Initialize glFonts (font & smallFont)
-	bool SetSDLVideoMode();                         //!< Sets SDL video mode
-	void SetProcessAffinity(int) const;
-	void Shutdown();                                //!< Shuts down application
+	static void InitOpenGL();                       //!< Initializes OpenGL
+	static void UpdateOldConfigs();                 //!< Forces an update to new config defaults
+	static void LoadFonts();                        //!< Initialize glFonts (font & smallFont)
+	static bool SetSDLVideoMode();                  //!< Sets SDL video mode
+	static void SetProcessAffinity(int);
 	int Update();                                   //!< Run simulation and draw
+	bool UpdateSim(CGameController *ac);
+
+	static bool GetDisplayGeometry();
+	static void SetupViewportGeometry();
+	static void RestoreWindowPosition();
+	static void SaveWindowPosition();
 
 #if defined(USE_GML) && GML_ENABLE_SIM
 	int Sim();                                      //!< Simulation  loop
 	static void Simcb(void *c) {((SpringApp *)c)->Sim();}
-	volatile int gmlKeepRunning;
 #endif
 
-	void UpdateSDLKeys();                           //!< Update SDL key array
-	bool GetDisplayGeometry();
-	void SetupViewportGeometry();
-
-	void RestoreWindowPosition();
-	void SaveWindowPosition();
-
 	/**
 	 * @brief command line
 	 *
 	 * Pointer to instance of commandline parser
 	 */
-	BaseCmd *cmdline;
-
-	/**
-	 * @brief screen width
-	 *
-	 * Game screen width
-	 */
-	int screenWidth;
-
-	/**
-	 * @brief screen height
-	 *
-	 * Game screen height
-	 */
-	int screenHeight;
-
-	/**
-	 * @brief window position - X
-	 *
-	 * Game window position from the left of the screen (if not full-screen)
-	 */
-	int windowPosX;
-
-	/**
-	 * @brief window position - Y
-	 *
-	 * Game window position from the top of the screen (if not full-screen)
-	 */
-	int windowPosY;
-
-	/**
-	 * @brief window state
-	 *
-	 * Game window state (0=normal,1=maximized,2=minimized)
-	 */
-	int windowState;
-
-	/**
-	 * @brief FSAA
-	 *
-	 * Level of full-screen anti-aliasing
-	 */
-	bool FSAA;
-
-	/**
-	 * @brief depthBufferBits
-	 *
-	 * number of Depthbuffer bits
-	 */
-	bool depthBufferBits;
+	CmdLineParams* cmdline;
 
 	/**
 	 * @brief last required draw
@@ -112,6 +62,8 @@ protected:
 	 */
 	int lastRequiredDraw;
 
+	static COffscreenGLContext* ogc;
+
 private:
 	bool MainEventHandler(const SDL_Event& ev);
 };
@@ -124,26 +76,4 @@ private:
  */
 extern CGameController* activeController;
 
-/**
- * @brief global quit
- *
- * Global boolean indicating whether the user
- * wants to quit
- */
-extern bool globalQuit;
-
-/**
- * @brief keys
- *
- * Array of possible keys, and which are being pressed
- */
-extern boost::uint8_t *keys;
-
-/**
- * @brief currentUnicode
- *
- * Unicode character for the current KeyPressed or KeyReleased
- */
-extern boost::uint16_t currentUnicode;
-
 #endif
