@@ -1,22 +1,23 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "myMath.h"
-#include "Util.h"
-#include "LogOutput.h"
-#include "Platform/errorhandler.h"
-#include "Sim/Units/COB/CobInstance.h" // for TAANG2RAD (ugh)
+#include "System/myMath.h"
+#include "System/OpenMP_cond.h"
+#include "System/Sync/FPUCheck.h"
+#include "System/Util.h"
+#include "System/Log/ILog.h"
+#include "System/Platform/errorhandler.h"
+#include "Sim/Units/Scripts/CobInstance.h" // for TAANG2RAD (ugh)
 
 float2 CMyMath::headingToVectorTable[NUM_HEADINGS];
 
 void CMyMath::Init()
 {
 	const unsigned int sseBits = proc::GetProcSSEBits();
-		logOutput.Print("[CMyMath::Init] CPU SSE mask: %u, flags:\n", sseBits);
-		logOutput.Print("\tSSE 1.0:  %d,  SSE 2.0:  %d\n", (sseBits >> 5) & 1, (sseBits >> 4) & 1);
-		logOutput.Print("\tSSE 3.0:  %d, SSSE 3.0:  %d\n", (sseBits >> 3) & 1, (sseBits >> 2) & 1);
-		logOutput.Print("\tSSE 4.1:  %d,  SSE 4.2:  %d\n", (sseBits >> 1) & 1, (sseBits >> 0) & 1);
-		logOutput.Print("\tSSE 4.0A: %d,  SSE 5.0A: %d\n", (sseBits >> 8) & 1, (sseBits >> 7) & 1);
+		LOG("[CMyMath::Init] CPU SSE mask: %u, flags:", sseBits);
+		LOG("\tSSE 1.0:  %d,  SSE 2.0:  %d", (sseBits >> 5) & 1, (sseBits >> 4) & 1);
+		LOG("\tSSE 3.0:  %d, SSSE 3.0:  %d", (sseBits >> 3) & 1, (sseBits >> 2) & 1);
+		LOG("\tSSE 4.1:  %d,  SSE 4.2:  %d", (sseBits >> 1) & 1, (sseBits >> 0) & 1);
+		LOG("\tSSE 4.0A: %d,  SSE 5.0A: %d", (sseBits >> 8) & 1, (sseBits >> 7) & 1);
 
 #ifdef STREFLOP_H
 	// SSE 1.0 is mandatory in synced context
@@ -24,19 +25,19 @@ void CMyMath::Init()
 		#ifdef STREFLOP_SSE
 		handleerror(0, "CPU is missing SSE instruction support", "Sync Error", 0);
 		#elif STREFLOP_X87
-		logOutput.Print("\tWARNING: streflop floating-point math is not SSE-enabled\n");
-		logOutput.Print("\tWARNING: this may cause desyncs during multi-player games\n");
-		logOutput.Print("\tWARNING: this CPU is not SSE-capable, can only use X87 mode\n");
+		LOG_L(L_WARNING, "\tStreflop floating-point math is not SSE-enabled");
+		LOG_L(L_WARNING, "\tThis may cause desyncs during multi-player games");
+		LOG_L(L_WARNING, "\tThis CPU is not SSE-capable; it can only use X87 mode");
 		#else
 		handleerror(0, "streflop FP-math mode must be either SSE or X87", "Sync Error", 0);
 		#endif
 	} else {
 		#ifdef STREFLOP_SSE
-		logOutput.Print("\tusing streflop SSE FP-math mode, CPU supports SSE instructions\n");
+		LOG("\tusing streflop SSE FP-math mode, CPU supports SSE instructions");
 		#elif STREFLOP_X87
-		logOutput.Print("\tNOTE: streflop floating-point math is set to X87 mode\n");
-		logOutput.Print("\tNOTE: this may cause desyncs during multi-player games\n");
-		logOutput.Print("\tNOTE: this CPU is SSE-capable, consider recompiling\n");
+		LOG_L(L_WARNING, "\tStreflop floating-point math is set to X87 mode");
+		LOG_L(L_WARNING, "\tThis may cause desyncs during multi-player games");
+		LOG_L(L_WARNING, "\tThis CPU is SSE-capable; consider recompiling");
 		#else
 		handleerror(0, "streflop FP-math mode must be either SSE or X87", "Sync Error", 0);
 		#endif
@@ -44,12 +45,32 @@ void CMyMath::Init()
 
 	// Set single precision floating point math.
 	streflop_init<streflop::Simple>();
+#if defined(__SUPPORT_SNAN__) && !defined(USE_GML)
+	streflop::feraiseexcept(streflop::FPU_Exceptions(FE_INVALID | FE_DIVBYZERO | FE_OVERFLOW));
+#endif
+
+	// Initialize FPU in all OpenMP threads, too
+	// Note: Tested on Linux it seems it's not needed to do this.
+	//       Either OMP threads copy the FPU state of the mainthread
+	//       or the FPU state per-process on Linux.
+	//       Still it hurts nobody to call these functions ;-)
+#ifdef _OPENMP
+	#pragma omp parallel
+	{
+		//good_fpu_control_registers("OMP-Init");
+		streflop_init<streflop::Simple>();
+	#if defined(__SUPPORT_SNAN__) && !defined(USE_GML)
+		streflop::feraiseexcept(streflop::FPU_Exceptions(FE_INVALID | FE_DIVBYZERO | FE_OVERFLOW));
+	#endif
+	}
+#endif
+
 #else
 	// probably should check if SSE was enabled during
 	// compilation and issue a warning about illegal
 	// instructions if so (or just die with an error)
-	logOutput.Print("WARNING: floating-point math is not controlled by streflop\n");
-	logOutput.Print("WARNING: this makes keeping multi-player sync 99% impossible\n");
+	LOG_L(L_WARNING, "Floating-point math is not controlled by streflop");
+	LOG_L(L_WARNING, "This makes keeping multi-player sync 99% impossible");
 #endif
 
 
@@ -136,50 +157,50 @@ float3 ClosestPointOnLine(const float3& l1, const float3& l2, const float3& p)
  * credits:
  * http://ompf.org/ray/ray_box.html
  */
-std::pair<float,float> GetMapBoundaryIntersectionPoints(const float3& start, const float3& dir)
+std::pair<float, float> GetMapBoundaryIntersectionPoints(const float3& start, const float3& dir)
 {
-	float rcpdirx = 1/dir.x;
-	float rcpdirz = 1/dir.z;
+	const float rcpdirx = (dir.x != 0.0f)? (1.0f / dir.x): 10000.0f;
+	const float rcpdirz = (dir.z != 0.0f)? (1.0f / dir.z): 10000.0f;
 	float l1, l2, far, near;
 
 	//! x component
-	l1	= (0               - start.x) * rcpdirx;
-	l2	= (float3::maxxpos - start.x) * rcpdirx;
-	near	= std::min(l1,l2);
-	far	= std::max(l1,l2);
+	l1 = (           0.0f - start.x) * rcpdirx;
+	l2 = (float3::maxxpos - start.x) * rcpdirx;
+	near = std::min(l1, l2);
+	far = std::max(l1, l2);
 
 	//! z component
-	l1	= (0               - start.z) * rcpdirz;
-	l2	= (float3::maxzpos - start.z) * rcpdirz;
-	near	= std::max(std::min(l1,l2), near);
-	far	= std::min(std::max(l1,l2), far);
+	l1 = (           0.0f - start.z) * rcpdirz;
+	l2 = (float3::maxzpos - start.z) * rcpdirz;
+	near = std::max(std::min(l1, l2), near);
+	far = std::min(std::max(l1, l2), far);
 
-
-	if (far < 0.f || far < near) {
+	if (far < 0.0f || far < near) {
 		//! outside of boundary
-		return std::pair<float,float>(-1.f,-1.f);
+		near = -1.0f;
+		far = -1.0f;
 	}
-	return std::pair<float,float>(near,far);
+	return std::pair<float, float>(near, far);
 }
 
 
 bool ClampLineInMap(float3& start, float3& end)
 {
-	float3 dir(end - start);
-	std::pair<float,float> interp = GetMapBoundaryIntersectionPoints(start, dir);
+	const float3 dir = end - start;
+	const std::pair<float, float>& interp = GetMapBoundaryIntersectionPoints(start, dir);
 	const float& near = interp.first;
 	const float& far  = interp.second;
 
-	if (far < 0.f) {
+	if (far < 0.0f) {
 		//! outside of map!
-		start = float3(-1,-1,-1);
-		end   = float3(-1,-1,-1);
+		start = float3(-1.0f, -1.0f, -1.0f);
+		end   = float3(-1.0f, -1.0f, -1.0f);
 		return true;
 	}
 
-	if (far < 1.f || near > 0.f) {
-		end   = start + dir * std::min(far, 1.f);
-		start = start + dir * std::max(near, 0.f);
+	if (far < 1.0f || near > 0.0f) {
+		end   = start + dir * std::min(far, 1.0f);
+		start = start + dir * std::max(near, 0.0f);
 
 		//! precision of near,far are limited, so better clamp it afterwards
 		end.CheckInBounds();
@@ -192,19 +213,19 @@ bool ClampLineInMap(float3& start, float3& end)
 
 bool ClampRayInMap(const float3& start, float3& end)
 {
-	const float3 dir(end - start);
-	std::pair<float,float> interp = GetMapBoundaryIntersectionPoints(start, dir);
+	const float3 dir = end - start;
+	std::pair<float, float> interp = GetMapBoundaryIntersectionPoints(start, dir);
 	const float& near = interp.first;
 	const float& far  = interp.second;
 
-	if (far < 0.f) {
+	if (far < 0.0f) {
 		//! outside of map!
 		end = start;
 		return true;
 	}
 
-	if (far < 1.f || near > 0.f) {
-		end = start + dir * std::min(far, 1.f);
+	if (far < 1.0f || near > 0.0f) {
+		end = start + dir * std::min(far, 1.0f);
 
 		//! precision of near,far are limited, so better clamp it afterwards
 		end.CheckInBounds();
