@@ -4,13 +4,13 @@
 #define MYMATH_H
 
 #include "Sim/Misc/GlobalConstants.h"
-#include "Vec2.h"
-#include "float3.h"
+#include "System/Vec2.h"
+#include "System/float3.h"
 
 #ifdef __GNUC__
 	#define _const __attribute__((const))
 #else
-	#define _const 
+	#define _const
 #endif
 
 static const float INVPI  = 1.0f / PI;
@@ -35,7 +35,8 @@ enum FacingMap {
 	FACING_NORTH = 2,
 	FACING_SOUTH = 0,
 	FACING_EAST  = 1,
-	FACING_WEST  = 3
+	FACING_WEST  = 3,
+	NUM_FACINGS  = 4,
 };
 
 class CMyMath {
@@ -94,7 +95,6 @@ float3 smoothstep(const float edge0, const float edge1, float3 vec);
 
 float Square(const float x) _const;
 int Round(const float f) _const;
-float Clamp(const float& v, const float& min, const float& max);
 template<class T> T Clamp(const T& v, const T& min, const T& max);
 
 
@@ -120,6 +120,11 @@ void ClampRad(float* f);
 bool RadsAreEqual(const float f1, const float f2) _const;
 
 
+/**
+ */
+float GetRadFromXY(const float dx, const float dy) _const;
+
+
 #include "myMath.inl"
 
 #undef _const
