@@ -11,11 +11,13 @@
 #ifndef TSC_H
 #define TSC_H
 
+#define DETACH_SYNCED 1
+
 #include <list>
 #include <vector>
 
-#include "creg/creg_cond.h"
-#include "gml.h"
+#include "System/creg/creg_cond.h"
+#include "gmlcnf.h"
 #include <set>
 
 /////////////////////////////////////////////////////////
@@ -74,12 +76,7 @@ public:
 
 	ListIT erase_delete_set_synced(ListIT& it) {
 		delObj.push_back(*it);
-#ifdef _MSC_VER
-		return cont.erase(it);
-#else
-		cont.erase(it++);
-		return it;
-#endif
+		return set_erase(cont, it);
 	}
 
 	bool can_delete_synced() const {
@@ -132,12 +129,7 @@ public:
 
 	ListIT erase_delete_set(ListIT& it) {
 		delete *it;
-#ifdef _MSC_VER
-		return cont.erase(it);
-#else
-		cont.erase(it++);
-		return it;
-#endif
+		return set_erase(cont, it);
 	}
 
 	bool can_delete() const {
@@ -386,12 +378,12 @@ public:
 	}
 
 	void clear() {
-		for(SimIT it = cont.begin(); it != cont.end(); ++it) {
+		for(SimIT it = cont.begin(); it != cont.end(); ++it)
+			delete *it;
+		for (VecIT it = delRender.begin(); it != delRender.end(); ++it)
+			delete *it;
+		for (SetIT it = postDelRender.begin(); it != postDelRender.end(); ++it)
 			delete *it;
-		}
-		cont.clear();
-		delete_delayed();
-		delete_erased_synced();
 	}
 
 	void PostLoad() {
@@ -423,12 +415,7 @@ public:
 
 	SimIT erase_delete_set_synced(SimIT& it) {
 		delRender.push_back(*it);
-#ifdef _MSC_VER
-		return cont.erase(it);
-#else
-		cont.erase(it++);
-		return it;
-#endif
+		return set_erase(cont, it);
 	}
 
 	bool can_delete_synced() {
@@ -492,12 +479,7 @@ public:
 
 	SimIT erase_delete_set(SimIT& it) {
 		delRender.push_back(*it);
-#ifdef _MSC_VER
-		return cont.erase(it);
-#else
-		cont.erase(it++);
-		return it;
-#endif
+		return set_erase(cont, it);
 	}
 
 	bool can_delete() {
@@ -525,11 +507,7 @@ public:
 		for (SetIT it = postDelRender.begin(); it != postDelRender.end();) {
 			T s = *it;
 			if(contRender.erase(s) || addRender.erase(s)) {
-#ifdef _MSC_VER
-				it = postDelRender.erase(it);
-#else
-				postDelRender.erase(it++);
-#endif
+				it = set_erase(postDelRender, it);
 				delete s;
 			}
 			else
@@ -770,11 +748,7 @@ public:
 			T s = *it;
 			size_t d;
 			if((d = contRender.erase(s)) || addRender.erase(s)) {
-#ifdef _MSC_VER
-				it = postDelRender.erase(it);
-#else
-				postDelRender.erase(it++);
-#endif
+				it = set_erase(postDelRender, it);
 				if(d)
 					D::Remove(s);
 				D::Delete(s);
@@ -866,7 +840,7 @@ private:
 
 
 
-template <class C, class R, class T>
+template <class C, class R, class T, class D>
 class ThreadListSim {
 private:
 	typedef typename C::iterator SimIT;
@@ -889,6 +863,12 @@ public:
 		delete_erased_synced();
 	}
 
+	void detach_all() {
+		for(SimIT it = cont.begin(); it != cont.end(); ++it) {
+			D::Detach(*it);
+		}
+	}
+
 	void PostLoad() {
 		for (SimIT it = cont.begin(); it != cont.end(); ++it) {
 		}
@@ -922,6 +902,16 @@ public:
 		del.clear();
 	}
 
+	void detach_erased_synced() {
+		for (VecIT it = del.begin(); it != del.end(); ++it) {
+			D::Detach(*it);
+#if !defined(USE_GML) || !GML_ENABLE_SIM
+			delete *it;
+#endif
+		}
+		del.clear();
+	}
+
 	void resize(const size_t& s) {
 		cont.resize(s);
 	}
@@ -949,6 +939,15 @@ public:
 		return cont.erase(it);
 	}
 
+	SimIT erase_detach(SimIT& it) {
+#if !defined(USE_GML) || !GML_ENABLE_SIM
+		delete *it;
+#else
+		D::Detach(*it);
+#endif
+		return cont.erase(it);
+	}
+
 public:
 	typedef SimIT iterator;
 
