@@ -1,5 +1,5 @@
 // GML - OpenGL Multithreading Library
-// for Spring http://spring.clan-sy.com
+// for Spring http://springrts.com
 // Author: Mattias "zerver" Radeskog
 // (C) Ware Zerver Tech. http://zerver.net
 // Ware Zerver Tech. licenses this library
@@ -9,9 +9,11 @@
 
 // GML works by "patching" all OpenGL calls. It is injected via a #include "gml.h" statement located in myGL.h.
 // All files that need GL should therefore include myGL.h. INCLUDING gl.h, glu.h, glext.h ... IS FORBIDDEN.
-// When a client thread (gmlThreadNumber > 0) executes a GL call, it is redirected into a queue.
+// When a client thread (gmlThreadNumber > 2) executes a GL call, it is redirected into a queue.
 // The server thread (gmlThreadNumber = 0) will then consume GL calls from the queues of each thread.
 // When the server thread makes a GL call, it calls directly into OpenGL of course.
+// The game load thread (gmlThreadNumber = 1) can also make GL calls.
+// The sim thread (gmlThreadNumber = 2) is allowed to make GL calls only if gmlShareLists is enabled.
 
 // Since a single server thread makes all GL calls, there is no point in multithreading code that contains
 // lots of GL calls but almost no CPU intensive calculations. Also, there is no point in multithreading
@@ -32,14 +34,30 @@
 // Please note: Some functions may require more advanced coding to implement
 // If a function is not yet supported by GML, a compile error pointing to 'GML_FUNCTION_NOT_IMPLEMENTED' will occur
 
-#include "StdAfx.h"
 
 #ifdef USE_GML
+
+#include <GL/glew.h>
 #include "gmlcls.h"
-#include "LogOutput.h"
+#include "gmlque.h"
+
+#include "Game/UI/KeyBindings.h"
+#include "Lua/LuaConfig.h"
+#include "System/Log/ILog.h"
+#include "System/Platform/Threading.h"
+
 
 const char *gmlProfMutex = "lua";
-unsigned drawCallInTime = 0;
+unsigned gmlLockTime = 0;
+
+int gmlProcNumLoop = 25;
+int gmlProcInterval = 8;
+bool gmlShareLists = true;
+int gmlMaxServerThreadNum = GML_SIM_THREAD_NUM;
+int gmlNoGLThreadNum = GML_NO_THREAD_NUM;
+volatile bool gmlMultiThreadSim = true;
+volatile bool gmlStartSim = false;
+volatile bool gmlKeepRunning = false;
 
 #define EXEC_RUN (BYTE *)NULL
 #define EXEC_SYNC (BYTE *)-1
@@ -67,6 +85,7 @@ int gmlItemsConsumed=0;
 int gmlNextTickUpdate=0;
 unsigned gmlCurrentTicks;
 
+std::set<Threading::NativeThreadId> threadnums;
 
 // gmlCPUCount returns the number of CPU cores
 // it was taken from the latest version of boost
@@ -142,8 +161,9 @@ void gmlInit() {
 		gmlGetFloatvCache[gmlFloatParams[i]]=fi;
 	}
 	for(int i=0; i<sizeof(gmlStringParams)/sizeof(GLenum); ++i) {
-		std::string si=(char *)glGetString(gmlStringParams[i]);
-		gmlGetStringCache[gmlStringParams[i]]=si;
+		const char* pstring = (const char*) glGetString(gmlStringParams[i]);
+		const std::string si = (pstring != NULL)? pstring: "[NULL]";
+		gmlGetStringCache[gmlStringParams[i]] = si;
 	}
 	gmlInited=TRUE;
 }
@@ -168,8 +188,6 @@ EXTERN inline GLhandleARB glCreateShaderObjectARB_GEOMETRY_EXT() {
 }
 gmlQueue gmlQueues[GML_MAX_NUM_THREADS];
 
-boost::thread *gmlThreads[GML_MAX_NUM_THREADS];
-
 gmlSingleItemServer<GLhandleARB, GLhandleARB (*)(void)> gmlShaderServer_VERTEX(&glCreateShader_VERTEX, 2, 0);
 gmlSingleItemServer<GLhandleARB, GLhandleARB (*)(void)> gmlShaderServer_FRAGMENT(&glCreateShader_FRAGMENT, 2, 0);
 gmlSingleItemServer<GLhandleARB, GLhandleARB (*)(void)> gmlShaderServer_GEOMETRY_EXT(&glCreateShader_GEOMETRY_EXT, 2, 0);
@@ -211,8 +229,6 @@ gmlMultiItemServer<GLuint, GLsizei, void (GML_GLAPIENTRY *)(GLsizei, GLuint *)>
 boost::mutex caimutex;
 boost::mutex decalmutex;
 boost::mutex treemutex;
-boost::mutex modelmutex;
-boost::mutex texmutex;
 boost::mutex mapmutex;
 boost::mutex inmapmutex;
 boost::mutex tempmutex;
@@ -220,23 +236,36 @@ boost::mutex posmutex;
 boost::mutex runitmutex;
 boost::mutex netmutex;
 boost::mutex histmutex;
-boost::mutex logmutex;
 boost::mutex timemutex;
 boost::mutex watermutex;
 boost::mutex dquemutex;
 boost::mutex scarmutex;
 boost::mutex trackmutex;
-boost::mutex projmutex;
 boost::mutex rprojmutex;
 boost::mutex rflashmutex;
 boost::mutex rpiecemutex;
 boost::mutex rfeatmutex;
 boost::mutex drawmutex;
+boost::mutex scallmutex;
+boost::mutex ulbatchmutex;
+boost::mutex flbatchmutex;
+boost::mutex olbatchmutex;
+boost::mutex plbatchmutex;
+boost::mutex glbatchmutex;
+boost::mutex mlbatchmutex;
+boost::mutex llbatchmutex;
+boost::mutex cmdmutex;
+boost::mutex luauimutex;
+boost::mutex xcallmutex;
+boost::mutex blockmutex;
+boost::mutex tnummutex;
+boost::mutex ntexmutex;
+boost::mutex lodmutex;
 
 #include <boost/thread/recursive_mutex.hpp>
 boost::recursive_mutex unitmutex;
 boost::recursive_mutex selmutex;
-boost::recursive_mutex &luamutex=selmutex;
+//boost::recursive_mutex luamutex;
 boost::recursive_mutex quadmutex;
 boost::recursive_mutex featmutex;
 boost::recursive_mutex grassmutex;
@@ -246,10 +275,50 @@ boost::recursive_mutex &qnummutex=quadmutex;
 boost::recursive_mutex &groupmutex=selmutex;
 boost::recursive_mutex &grpselmutex=selmutex;
 boost::recursive_mutex laycmdmutex;
+//boost::recursive_mutex luadrawmutex;
+boost::recursive_mutex projmutex;
+boost::recursive_mutex objmutex;
+boost::recursive_mutex modelmutex;
 
 gmlMutex simmutex;
+
+#if GML_DEBUG_MUTEX
+boost::mutex lmmutex;
+std::map<std::string, int> lockmaps[GML_MAX_NUM_THREADS];
+std::map<boost::recursive_mutex *, int> lockmmaps[GML_MAX_NUM_THREADS];
 #endif
 
+void PrintMTStartupMessage(int showMTInfo) {
+	if (showMTInfo != MT_LUA_NONE) {
+		LOG("\n************** SPRING MULTITHREADING VERSION IMPORTANT NOTICE **************");
+		LOG("Engine or game settings have forced Spring MT to use compatibility mode %d", showMTInfo);
+		if (showMTInfo == MT_LUA_SINGLE) {
+			CKeyBindings::HotkeyList lslist = keyBindings->GetHotkeys("luaui selector");
+			std::string lskey = lslist.empty() ? "" : " (press " + lslist.front() + ")";
+			LOG("If your game uses lua based rendering, it may run very slow with Spring MT");
+			LOG("A high LUA-SYNC-CPU(MT) value in the upper right corner could indicate a problem");
+			LOG("Consider changing the engine setting 'MultiThreadLua' to 2 to improve performance,");
+			LOG("or try to disable LuaShaders and all rendering widgets%s\n", lskey.c_str());
+		} else if (showMTInfo == MT_LUA_SINGLE_BATCH) {
+			LOG("If your game uses lua gadget based rendering, it may run very slow with Spring MT");
+			LOG("A high LUA-SYNC-CPU(MT) value in the upper right corner could indicate a problem\n");
+		} else if (showMTInfo == MT_LUA_DUAL_EXPORT) {
+			LOG("If your game uses lua gadgets that export data, it may run very slow with Spring MT");
+			LOG("A high LUA-EXP-SIZE(MT) value in the upper right corner could indicate a problem\n");
+		}
+	}
+}
+
+#endif
+
+bool ThreadRegistered() {
+	boost::mutex::scoped_lock tnumlock(tnummutex);
+	Threading::NativeThreadId thid = Threading::GetCurrentThreadId();
+	if (threadnums.find(thid) != threadnums.end())
+		return true;
+	threadnums.insert(thid);
+	return false;
+}
 // GMLqueue implementation
 gmlQueue::gmlQueue():
 ReadPos(0),WritePos(0),WriteSize(0),Read(0),Write(0),Locked1(FALSE),Locked2(FALSE),Reloc(FALSE),Sync(EXEC_RUN),WasSynced(FALSE),
@@ -710,7 +779,7 @@ const char *gmlNOPDummy=(gmlFunctionNames[GML_NOP]="gmlNOP");
 gmlItemSequenceServer<GLuint, GLsizei,GLuint (GML_GLAPIENTRY *)(GLsizei)> gmlListServer(&glGenLists, &gmlDeleteLists, 100, 25, 20, 5);
 
 #if GML_CALL_DEBUG
-lua_State *gmlCurrentLuaState = NULL;
+lua_State *gmlCurrentLuaStates[GML_MAX_NUM_THREADS] = { NULL };
 #endif
 
 // queue handler - exequtes one GL command from queue (pointed to by p)
@@ -994,6 +1063,7 @@ inline void QueueHandler(BYTE *&p, BYTE *&ptr) {
 		GML_MAKEHANDLER3V(Uniform3fv)
 		GML_MAKEHANDLER3V(Uniform4fv)
 		GML_MAKEHANDLER4R(MapBufferRange)
+		GML_MAKEHANDLER1(PrimitiveRestartIndexNV)
 	}
 }
 
@@ -1020,12 +1090,12 @@ void gmlQueue::ExecuteDebug() {
 
 	while(p<e) {
 		if(*(int *)p!=GML_NOP)
-			logOutput.Print("GML error: Sim thread called %s",gmlFunctionNames[*(int *)p]);
+			LOG_SL("GML", L_ERROR, "Sim thread called %s", gmlFunctionNames[*(int*)p]);
 		QueueHandler(p,ptr);
 //		++procs;
 	}
 //	if(procs>1 || (procs==1 && *(int *)Read!=GML_NOP))
-//		logOutput.Print("GML error: %d OpenGL calls detected in SimFrame()",procs);
+//		LOG_SL("GML", L_ERROR, "%d OpenGL calls detected in SimFrame()", procs);
 }
 
 #include "gmlsrv.h"
