@@ -1,5 +1,5 @@
 // GML - OpenGL Multithreading Library
-// for Spring http://spring.clan-sy.com
+// for Spring http://springrts.com
 // Author: Mattias "zerver" Radeskog
 // (C) Ware Zerver Tech. http://zerver.net
 // Ware Zerver Tech. licenses this library
@@ -14,22 +14,42 @@
 #error USE_GML_SIM requires USE_GML
 #endif
 
-#define GML_MUTEX_PROFILER 0 // enables profiler
+#define GML_MUTEX_PROFILER 0 // enables mutex profiler
 
 #ifdef USE_GML
+
+extern bool gmlShareLists; // use glShareLists to allow opengl calls in sim thread, unfortunately this may reduce the FPS a bit
+extern int gmlMaxServerThreadNum;
+extern int gmlNoGLThreadNum;
+extern volatile bool gmlMultiThreadSim;
+extern volatile bool gmlStartSim;
+extern volatile bool gmlKeepRunning;
+
 #define GML_MUTEX_PROFILE 0 // detailed profiling of specific mutex
 extern const char *gmlProfMutex;
+#define GML_PROC_PROFILER 0 // enables gmlprocessor profiler
 
 #include <set>
 #include <map>
 
+#include <GL/glew.h>
 #include "gmlcls.h"
+#include "gmlque.h"
 
 extern gmlQueue gmlQueues[GML_MAX_NUM_THREADS];
 
 #include "gmlfun.h"
 
-extern boost::thread *gmlThreads[GML_MAX_NUM_THREADS];
+#if GML_PROC_PROFILER
+	extern int gmlProcNumLoop;
+	extern int gmlProcInterval;
+	#define GML_PROFILER(name) \
+	name && (globalRendering->drawFrame & gmlProcInterval);\
+	SCOPED_TIMER(!name ? "NoProc" : ((name && (globalRendering->drawFrame & gmlProcInterval)) ? " " GML_QUOTE(name) "MTProc" : " " GML_QUOTE(name) "Proc"));\
+	for(int i = 0; i < (name ? gmlProcNumLoop : 1); ++i)
+#else
+	#define GML_PROFILER(name) name;
+#endif
 
 extern gmlSingleItemServer<GLhandleARB, GLhandleARB (*)(void)> gmlShaderServer_VERTEX;
 extern gmlSingleItemServer<GLhandleARB, GLhandleARB (*)(void)> gmlShaderServer_FRAGMENT;
@@ -85,15 +105,15 @@ extern void gmlInit();
 	}\
 	GML_ITEMSERVER_CHECK_RET(threadnum,ret);
 
-EXTERN inline GLhandleARB gmlCreateProgram() {
+EXTERN inline GLhandleARB GML_GLAPIENTRY gmlCreateProgram() {
 	GML_IF_NONCLIENT_THREAD_RET(GLhandleARB,glCreateProgram);
 	return gmlProgramServer.GetItems();
 }
-EXTERN inline GLhandleARB gmlCreateProgramObjectARB() {
+EXTERN inline GLhandleARB GML_GLAPIENTRY gmlCreateProgramObjectARB() {
 	GML_IF_NONCLIENT_THREAD_RET(GLhandleARB,glCreateProgramObjectARB);
 	return gmlProgramObjectARBServer.GetItems();
 }
-EXTERN inline GLhandleARB gmlCreateShader(GLenum type) {
+EXTERN inline GLhandleARB GML_GLAPIENTRY gmlCreateShader(GLenum type) {
 	GML_IF_NONCLIENT_THREAD_RET(GLhandleARB,glCreateShader,type);
 	if(type==GL_VERTEX_SHADER)
 		return gmlShaderServer_VERTEX.GetItems();
@@ -103,7 +123,7 @@ EXTERN inline GLhandleARB gmlCreateShader(GLenum type) {
 		return gmlShaderServer_GEOMETRY_EXT.GetItems();
 	return 0;
 }
-EXTERN inline GLhandleARB gmlCreateShaderObjectARB(GLenum type) {
+EXTERN inline GLhandleARB GML_GLAPIENTRY gmlCreateShaderObjectARB(GLenum type) {
 	GML_IF_NONCLIENT_THREAD_RET(GLhandleARB,glCreateShaderObjectARB,type);
 	if(type==GL_VERTEX_SHADER_ARB)
 		return gmlShaderObjectARBServer_VERTEX.GetItems();
@@ -113,45 +133,45 @@ EXTERN inline GLhandleARB gmlCreateShaderObjectARB(GLenum type) {
 		return gmlShaderObjectARBServer_GEOMETRY_EXT.GetItems();
 	return 0;
 }
-EXTERN inline GLUquadric *gmluNewQuadric() {
+EXTERN inline GLUquadric *GML_GLAPIENTRY gmluNewQuadric() {
 	GML_IF_NONCLIENT_THREAD_RET(GLUquadric *,gluNewQuadric);
 	return gmlQuadricServer.GetItems();
 }
 
-EXTERN inline void gmlGenTextures(GLsizei n, GLuint *items) {
+EXTERN inline void GML_GLAPIENTRY gmlGenTextures(GLsizei n, GLuint *items) {
 	GML_IF_NONCLIENT_THREAD(glGenTextures,n,items);
 	gmlTextureServer.GetItems(n, items);
 }
-EXTERN inline void gmlGenBuffersARB(GLsizei n, GLuint *items) {
+EXTERN inline void GML_GLAPIENTRY gmlGenBuffersARB(GLsizei n, GLuint *items) {
 	GML_IF_NONCLIENT_THREAD(glGenBuffersARB,n,items);
 	gmlBufferARBServer.GetItems(n, items);
 }
-EXTERN inline void gmlGenFencesNV(GLsizei n, GLuint *items) {
+EXTERN inline void GML_GLAPIENTRY gmlGenFencesNV(GLsizei n, GLuint *items) {
 	GML_IF_NONCLIENT_THREAD(glGenFencesNV,n,items);
 	gmlFencesNVServer.GetItems(n, items);
 }
-EXTERN inline void gmlGenProgramsARB(GLsizei n, GLuint *items) {
+EXTERN inline void GML_GLAPIENTRY gmlGenProgramsARB(GLsizei n, GLuint *items) {
 	GML_IF_NONCLIENT_THREAD(glGenProgramsARB,n,items);
 	gmlProgramsARBServer.GetItems(n, items);
 }
-EXTERN inline void gmlGenRenderbuffersEXT(GLsizei n, GLuint *items) {
+EXTERN inline void GML_GLAPIENTRY gmlGenRenderbuffersEXT(GLsizei n, GLuint *items) {
 	GML_IF_NONCLIENT_THREAD(glGenRenderbuffersEXT,n,items);
 	gmlRenderbuffersEXTServer.GetItems(n, items);
 }
-EXTERN inline void gmlGenFramebuffersEXT(GLsizei n, GLuint *items) {
+EXTERN inline void GML_GLAPIENTRY gmlGenFramebuffersEXT(GLsizei n, GLuint *items) {
 	GML_IF_NONCLIENT_THREAD(glGenFramebuffersEXT,n,items);
 	gmlFramebuffersEXTServer.GetItems(n, items);
 }
-EXTERN inline void gmlGenQueries(GLsizei n, GLuint *items) {
+EXTERN inline void GML_GLAPIENTRY gmlGenQueries(GLsizei n, GLuint *items) {
 	GML_IF_NONCLIENT_THREAD(glGenQueries,n,items);
 	gmlQueryServer.GetItems(n, items);
 }
-EXTERN inline void gmlGenBuffers(GLsizei n, GLuint *items) {
+EXTERN inline void GML_GLAPIENTRY gmlGenBuffers(GLsizei n, GLuint *items) {
 	GML_IF_NONCLIENT_THREAD(glGenBuffers,n,items);
 	gmlBufferServer.GetItems(n, items);
 }
 
-EXTERN inline GLuint gmlGenLists(GLsizei items) {
+EXTERN inline GLuint GML_GLAPIENTRY gmlGenLists(GLsizei items) {
 	GML_IF_NONCLIENT_THREAD_RET(GLuint,glGenLists,items);
 	return gmlListServer.GetItems(items);
 }
@@ -162,91 +182,10 @@ EXTERN inline GLuint gmlGenLists(GLsizei items) {
 #define GML_VECTOR gmlVector
 #define GML_CLASSVECTOR gmlClassVector
 
+#include "gmlmut.h"
+
 #if GML_ENABLE_SIM
-#include <boost/thread/mutex.hpp>
-extern boost::mutex caimutex;
-extern boost::mutex decalmutex;
-extern boost::mutex treemutex;
-extern boost::mutex modelmutex;
-extern boost::mutex texmutex;
-extern boost::mutex mapmutex;
-extern boost::mutex inmapmutex;
-extern boost::mutex tempmutex;
-extern boost::mutex posmutex;
-extern boost::mutex runitmutex;
-extern boost::mutex netmutex;
-extern boost::mutex histmutex;
-extern boost::mutex logmutex;
-extern boost::mutex timemutex;
-extern boost::mutex watermutex;
-extern boost::mutex dquemutex;
-extern boost::mutex scarmutex;
-extern boost::mutex trackmutex;
-extern boost::mutex projmutex;
-extern boost::mutex rprojmutex;
-extern boost::mutex rflashmutex;
-extern boost::mutex rpiecemutex;
-extern boost::mutex rfeatmutex;
-extern boost::mutex drawmutex;
 
-#include <boost/thread/recursive_mutex.hpp>
-extern boost::recursive_mutex unitmutex;
-extern boost::recursive_mutex quadmutex;
-extern boost::recursive_mutex selmutex;
-extern boost::recursive_mutex &luamutex;
-extern boost::recursive_mutex featmutex;
-extern boost::recursive_mutex grassmutex;
-extern boost::recursive_mutex &guimutex;
-extern boost::recursive_mutex filemutex;
-extern boost::recursive_mutex &qnummutex;
-extern boost::recursive_mutex &groupmutex;
-extern boost::recursive_mutex &grpselmutex;
-extern boost::recursive_mutex laycmdmutex;
-
-extern gmlMutex simmutex;
-
-#if GML_MUTEX_PROFILER
-#	include "System/TimeProfiler.h"
-#	if GML_MUTEX_PROFILE
-#		ifdef _DEBUG
-#			define GML_MTXCMP(a,b) !strcmp(a,b) // comparison of static strings using addresses may not work in debug mode
-#		else
-#			define GML_MTXCMP(a,b) (a==b)
-#		endif
-#		define GML_LINEMUTEX_LOCK(name, type, line)\
-			char st##name[sizeof(ScopedTimer)];\
-			int stc##name=GML_MTXCMP(GML_QUOTE(name),gmlProfMutex);\
-			if(stc##name)\
-				new (st##name) ScopedTimer(GML_QUOTE(name ## line ## Mutex));\
-			boost::type::scoped_lock name##lock(name##mutex);\
-			if(stc##name)\
-				((ScopedTimer *)st##name)->~ScopedTimer()
-#		define GML_PROFMUTEX_LOCK(name, type, line) GML_LINEMUTEX_LOCK(name, type, line)
-#		define GML_STDMUTEX_LOCK(name) GML_PROFMUTEX_LOCK(name, mutex, __LINE__)
-#		define GML_RECMUTEX_LOCK(name) GML_PROFMUTEX_LOCK(name, recursive_mutex, __LINE__)
-#	else
-#		define GML_PROFMUTEX_LOCK(name, type)\
-			char st##name[sizeof(ScopedTimer)];\
-			new (st##name) ScopedTimer(GML_QUOTE(name##Mutex));\
-			boost::type::scoped_lock name##lock(name##mutex);\
-			((ScopedTimer *)st##name)->~ScopedTimer()
-#		define GML_STDMUTEX_LOCK(name) GML_PROFMUTEX_LOCK(name, mutex)
-#		define GML_RECMUTEX_LOCK(name) GML_PROFMUTEX_LOCK(name, recursive_mutex)
-#	endif
-#else
-#	define GML_STDMUTEX_LOCK(name) boost::mutex::scoped_lock name##lock(name##mutex)
-#	define GML_RECMUTEX_LOCK(name) boost::recursive_mutex::scoped_lock name##lock(name##mutex)
-#endif
-class gmlMutexLock {
-	gmlMutex &mtx;
-public:
-	gmlMutexLock(gmlMutex &m) : mtx(m) { mtx.Lock(); }
-	virtual ~gmlMutexLock() { mtx.Unlock(); }
-};
-#define GML_MSTMUTEX_LOCK(name) gmlMutexLock name##mutexlock(name##mutex)
-#define GML_MSTMUTEX_DOLOCK(name) name##mutex.Lock()
-#define GML_MSTMUTEX_DOUNLOCK(name) name##mutex.Unlock()
-#define GML_STDMUTEX_LOCK_NOPROF(name) boost::mutex::scoped_lock name##lock(name##mutex)
 extern int gmlNextTickUpdate;
 extern unsigned gmlCurrentTicks;
 
@@ -265,37 +204,35 @@ inline unsigned gmlGetTicks() {
 
 #if GML_CALL_DEBUG
 #define GML_EXPGEN_CHECK() \
-	extern volatile int gmlMultiThreadSim, gmlStartSim;\
-	if(gmlThreadNumber!=gmlThreadCount && gmlMultiThreadSim && gmlStartSim) {\
-		logOutput.Print("GML error: Draw thread created ExpGenSpawnable (%s)", GML_CURRENT_LUA());\
-		if(gmlCurrentLuaState) luaL_error(gmlCurrentLuaState,"Invalid call");\
+	if(gmlThreadNumber != GML_SIM_THREAD_NUM && gmlMultiThreadSim && gmlStartSim) {\
+		lua_State *currentLuaState = gmlCurrentLuaStates[gmlThreadNumber];\
+		LOG_SL("GML", L_ERROR, "Draw thread created ExpGenSpawnable (%s)", GML_CURRENT_LUA(currentLuaState));\
+		if(currentLuaState) luaL_error(currentLuaState,"Invalid call");\
 	}
 #define GML_CALL_DEBUGGER() gmlCallDebugger gmlCDBG(L);
-#define GML_DRAW_CALLIN_TIME() (gmlCallDebugger::getDrawCallInTime())
+#define GML_LOCK_TIME() (gmlCallDebugger::getLockTime())
 #else
 #define GML_EXPGEN_CHECK()
 #define GML_CALL_DEBUGGER()
-#define GML_DRAW_CALLIN_TIME() 0
+#define GML_LOCK_TIME() 0
 #endif
 
+#if GML_ENABLE_SIM
 #define GML_GET_TICKS(var) var=gmlGetTicks()
 #define GML_UPDATE_TICKS() gmlUpdateTicks()
+#else
+#define GML_GET_TICKS(var)
+#define GML_UPDATE_TICKS()
+#endif
 
 #else
 
-#define GML_STDMUTEX_LOCK(name)
-#define GML_RECMUTEX_LOCK(name)
-#define GML_STDMUTEX_LOCK_NOPROF(name)
-#define GML_MSTMUTEX_LOCK(name)
-#define GML_MSTMUTEX_DOLOCK(name)
-#define GML_MSTMUTEX_DOUNLOCK(name)
-
 #define GML_GET_TICKS(var)
 #define GML_UPDATE_TICKS()
 
 #define GML_EXPGEN_CHECK()
 #define GML_CALL_DEBUGGER()
-#define GML_DRAW_CALLIN_TIME() 0
+#define GML_LOCK_TIME() 0
 
 #endif
 
@@ -306,6 +243,8 @@ inline unsigned gmlGetTicks() {
 
 #define GML_STDMUTEX_LOCK(name)
 #define GML_RECMUTEX_LOCK(name)
+#define GML_THRMUTEX_LOCK(name,thr)
+#define GML_OBJMUTEX_LOCK(name,thr,...)
 #define GML_STDMUTEX_LOCK_NOPROF(name)
 #define GML_MSTMUTEX_LOCK(name)
 #define GML_MSTMUTEX_DOLOCK(name)
@@ -316,20 +255,8 @@ inline unsigned gmlGetTicks() {
 
 #define GML_EXPGEN_CHECK()
 #define GML_CALL_DEBUGGER()
-#define GML_DRAW_CALLIN_TIME() 0
+#define GML_LOCK_TIME() 0
 
 #endif // USE_GML
 
-#ifdef    HEADLESS
-#define glGenerateMipmapEXT_NONGML NULL
-#define glUseProgram_NONGML NULL
-#define glProgramParameteriEXT_NONGML NULL
-#define glBlendEquation_NONGML NULL
-#else  // HEADLESS
-#define glGenerateMipmapEXT_NONGML GLEW_GET_FUN(__glewGenerateMipmapEXT)
-#define glUseProgram_NONGML GLEW_GET_FUN(__glewUseProgram)
-#define glProgramParameteriEXT_NONGML GLEW_GET_FUN(__glewProgramParameteriEXT)
-#define glBlendEquation_NONGML GLEW_GET_FUN(__glewBlendEquation)
-#endif // HEADLESS
-
 #endif
